@@ -4,17 +4,17 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"io"
 	"net/http"
 	neturl "net/url"
-	"slices"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/gorilla/websocket"
 
+	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/logger"
 	"github.com/canonical/lxd/shared/tcp"
@@ -27,8 +27,15 @@ type ProtocolLXD struct {
 	ctxConnected       context.Context
 	ctxConnectedCancel context.CancelFunc
 
-	// eventListenersLock is used to synchronize access to the event listeners.
-	eventListenerManager *eventListenerManager
+	// eventConns contains event listener connections associated to a project name (or empty for all projects).
+	eventConns map[string]*websocket.Conn
+
+	// eventConnsLock controls write access to the eventConns.
+	eventConnsLock sync.Mutex
+
+	// eventListeners is a slice of event listeners associated to a project name (or empty for all projects).
+	eventListeners     map[string][]*EventListener
+	eventListenersLock sync.Mutex
 
 	http            *http.Client
 	httpCertificate string
@@ -82,7 +89,7 @@ func (r *ProtocolLXD) GetConnectionInfo() (*ConnectionInfo, error) {
 			}
 
 			url := "https://" + addr
-			if !slices.Contains(urls, url) {
+			if !shared.ValueInSlice(url, urls) {
 				urls = append(urls, url)
 			}
 		}
@@ -130,7 +137,7 @@ func (r *ProtocolLXD) isSameServer(server Server) bool {
 // GetHTTPClient returns the http client used for the connection. This can be used to set custom http options.
 func (r *ProtocolLXD) GetHTTPClient() (*http.Client, error) {
 	if r.http == nil {
-		return nil, errors.New("HTTP client isn't set, bad connection")
+		return nil, fmt.Errorf("HTTP client isn't set, bad connection")
 	}
 
 	return r.http, nil
@@ -352,7 +359,7 @@ func (r *ProtocolLXD) queryStruct(method string, path string, data any, ETag str
 	}
 
 	// Log the data
-	logger.Debug("Got response struct from LXD")
+	logger.Debugf("Got response struct from LXD")
 	logger.Debug(logger.Pretty(target))
 
 	return etag, nil
@@ -399,7 +406,7 @@ func (r *ProtocolLXD) queryOperation(method string, path string, data any, ETag
 	}
 
 	// Log the data
-	logger.Debug("Got operation from LXD")
+	logger.Debugf("Got operation from LXD")
 	logger.Debug(logger.Pretty(op.Operation))
 
 	return &op, etag, nil
