@@ -1,14 +1,10 @@
 package lxd
 
 import (
-	"net/http"
-
 	"github.com/canonical/lxd/shared/api"
 )
 
 // GetCluster returns information about a cluster.
-//
-// If this client is not trusted, the password must be supplied.
 func (r *ProtocolLXD) GetCluster() (*api.Cluster, string, error) {
 	err := r.CheckExtension("clustering")
 	if err != nil {
@@ -16,8 +12,7 @@ func (r *ProtocolLXD) GetCluster() (*api.Cluster, string, error) {
 	}
 
 	cluster := &api.Cluster{}
-	u := api.NewURL().Path("cluster")
-	etag, err := r.queryStruct(http.MethodGet, u.String(), nil, "", &cluster)
+	etag, err := r.queryStruct("GET", "/cluster", nil, "", &cluster)
 	if err != nil {
 		return nil, "", err
 	}
@@ -32,7 +27,7 @@ func (r *ProtocolLXD) UpdateCluster(cluster api.ClusterPut, ETag string) (Operat
 		return nil, err
 	}
 
-	if cluster.ServerAddress != "" || cluster.ClusterPassword != "" || cluster.ClusterToken != "" || len(cluster.MemberConfig) > 0 {
+	if cluster.ServerAddress != "" || cluster.ClusterToken != "" || len(cluster.MemberConfig) > 0 {
 		err := r.CheckExtension("clustering_join")
 		if err != nil {
 			return nil, err
@@ -46,8 +41,7 @@ func (r *ProtocolLXD) UpdateCluster(cluster api.ClusterPut, ETag string) (Operat
 		}
 	}
 
-	u := api.NewURL().Path("cluster")
-	op, _, err := r.queryOperation(http.MethodPut, u.String(), cluster, "", true)
+	op, _, err := r.queryOperation("PUT", "/cluster", cluster, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -63,12 +57,12 @@ func (r *ProtocolLXD) DeleteClusterMember(name string, force bool) error {
 		return err
 	}
 
-	u := api.NewURL().Path("cluster", "members", name)
+	params := ""
 	if force {
-		u = u.WithQuery("force", "1")
+		params += "?force=1"
 	}
 
-	_, _, err = r.query(http.MethodDelete, u.String(), nil, "")
+	_, _, err = r.query("DELETE", "/cluster/members/"+name+params, nil, "")
 	if err != nil {
 		return err
 	}
@@ -85,14 +79,14 @@ func (r *ProtocolLXD) GetClusterMemberNames() ([]string, error) {
 
 	// Fetch the raw URL values.
 	urls := []string{}
-	u := api.NewURL().Path("cluster", "members")
-	_, err = r.queryStruct(http.MethodGet, u.String(), nil, "", &urls)
+	baseURL := "/cluster/members"
+	_, err = r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
 
 	// Parse it.
-	return urlsToResourceNames(u.String(), urls...)
+	return urlsToResourceNames(baseURL, urls...)
 }
 
 // GetClusterMembers returns the current members of the cluster.
@@ -103,8 +97,7 @@ func (r *ProtocolLXD) GetClusterMembers() ([]api.ClusterMember, error) {
 	}
 
 	members := []api.ClusterMember{}
-	u := api.NewURL().Path("cluster", "members").WithQuery("recursion", "1")
-	_, err = r.queryStruct(http.MethodGet, u.String(), nil, "", &members)
+	_, err = r.queryStruct("GET", "/cluster/members?recursion=1", nil, "", &members)
 	if err != nil {
 		return nil, err
 	}
@@ -120,8 +113,7 @@ func (r *ProtocolLXD) GetClusterMember(name string) (*api.ClusterMember, string,
 	}
 
 	member := api.ClusterMember{}
-	u := api.NewURL().Path("cluster", "members", name)
-	etag, err := r.queryStruct(http.MethodGet, u.String(), nil, "", &member)
+	etag, err := r.queryStruct("GET", "/cluster/members/"+name, nil, "", &member)
 	if err != nil {
 		return nil, "", err
 	}
@@ -144,8 +136,7 @@ func (r *ProtocolLXD) UpdateClusterMember(name string, member api.ClusterMemberP
 	}
 
 	// Send the request
-	u := api.NewURL().Path("cluster", "members", name)
-	_, _, err = r.query(http.MethodPut, u.String(), member, ETag)
+	_, _, err = r.query("PUT", "/cluster/members/"+name, member, ETag)
 	if err != nil {
 		return err
 	}
@@ -160,8 +151,7 @@ func (r *ProtocolLXD) RenameClusterMember(name string, member api.ClusterMemberP
 		return err
 	}
 
-	u := api.NewURL().Path("cluster", "members", name)
-	_, _, err = r.query(http.MethodPost, u.String(), member, "")
+	_, _, err = r.query("POST", "/cluster/members/"+name, member, "")
 	if err != nil {
 		return err
 	}
@@ -176,8 +166,7 @@ func (r *ProtocolLXD) CreateClusterMember(member api.ClusterMembersPost) (Operat
 		return nil, err
 	}
 
-	u := api.NewURL().Path("cluster", "members")
-	op, _, err := r.queryOperation(http.MethodPost, u.String(), member, "", true)
+	op, _, err := r.queryOperation("POST", "/cluster/members", member, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -192,8 +181,7 @@ func (r *ProtocolLXD) UpdateClusterCertificate(certs api.ClusterCertificatePut,
 		return err
 	}
 
-	u := api.NewURL().Path("cluster", "certificate")
-	_, _, err = r.query(http.MethodPut, u.String(), certs, ETag)
+	_, _, err = r.query("PUT", "/cluster/certificate", certs, ETag)
 	if err != nil {
 		return err
 	}
@@ -210,7 +198,7 @@ func (r *ProtocolLXD) GetClusterMemberState(name string) (*api.ClusterMemberStat
 
 	state := api.ClusterMemberState{}
 	u := api.NewURL().Path("cluster", "members", name, "state")
-	etag, err := r.queryStruct(http.MethodGet, u.String(), nil, "", &state)
+	etag, err := r.queryStruct("GET", u.String(), nil, "", &state)
 	if err != nil {
 		return nil, "", err
 	}
@@ -225,15 +213,7 @@ func (r *ProtocolLXD) UpdateClusterMemberState(name string, state api.ClusterMem
 		return nil, err
 	}
 
-	if state.Action != "" {
-		err = r.CheckExtension("clustering_restore_skip_mode")
-		if err != nil {
-			return nil, err
-		}
-	}
-
-	u := api.NewURL().Path("cluster", "members", name, "state")
-	op, _, err := r.queryOperation(http.MethodPost, u.String(), state, "", true)
+	op, _, err := r.queryOperation("POST", "/cluster/members/"+name+"/state", state, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -249,8 +229,8 @@ func (r *ProtocolLXD) GetClusterGroups() ([]api.ClusterGroup, error) {
 	}
 
 	groups := []api.ClusterGroup{}
-	u := api.NewURL().Path("cluster", "groups").WithQuery("recursion", "1")
-	_, err = r.queryStruct(http.MethodGet, u.String(), nil, "", &groups)
+
+	_, err = r.queryStruct("GET", "/cluster/groups?recursion=1", nil, "", &groups)
 	if err != nil {
 		return nil, err
 	}
@@ -266,8 +246,8 @@ func (r *ProtocolLXD) GetClusterGroupNames() ([]string, error) {
 	}
 
 	urls := []string{}
-	u := api.NewURL().Path("cluster", "groups")
-	_, err = r.queryStruct(http.MethodGet, u.String(), nil, "", &urls)
+
+	_, err = r.queryStruct("GET", "/cluster/groups", nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -283,8 +263,7 @@ func (r *ProtocolLXD) RenameClusterGroup(name string, group api.ClusterGroupPost
 		return err
 	}
 
-	u := api.NewURL().Path("cluster", "groups", name)
-	_, _, err = r.query(http.MethodPost, u.String(), group, "")
+	_, _, err = r.query("POST", "/cluster/groups/"+name, group, "")
 	if err != nil {
 		return err
 	}
@@ -299,8 +278,7 @@ func (r *ProtocolLXD) CreateClusterGroup(group api.ClusterGroupsPost) error {
 		return err
 	}
 
-	u := api.NewURL().Path("cluster", "groups")
-	_, _, err = r.query(http.MethodPost, u.String(), group, "")
+	_, _, err = r.query("POST", "/cluster/groups", group, "")
 	if err != nil {
 		return err
 	}
@@ -315,8 +293,7 @@ func (r *ProtocolLXD) DeleteClusterGroup(name string) error {
 		return err
 	}
 
-	u := api.NewURL().Path("cluster", "groups", name)
-	_, _, err = r.query(http.MethodDelete, u.String(), nil, "")
+	_, _, err = r.query("DELETE", "/cluster/groups/"+name, nil, "")
 	if err != nil {
 		return err
 	}
@@ -332,8 +309,7 @@ func (r *ProtocolLXD) UpdateClusterGroup(name string, group api.ClusterGroupPut,
 	}
 
 	// Send the request
-	u := api.NewURL().Path("cluster", "groups", name)
-	_, _, err = r.query(http.MethodPut, u.String(), group, ETag)
+	_, _, err = r.query("PUT", "/cluster/groups/"+name, group, ETag)
 	if err != nil {
 		return err
 	}
@@ -349,8 +325,7 @@ func (r *ProtocolLXD) GetClusterGroup(name string) (*api.ClusterGroup, string, e
 	}
 
 	group := api.ClusterGroup{}
-	u := api.NewURL().Path("cluster", "groups", name)
-	etag, err := r.queryStruct(http.MethodGet, u.String(), nil, "", &group)
+	etag, err := r.queryStruct("GET", "/cluster/groups/"+name, nil, "", &group)
 	if err != nil {
 		return nil, "", err
 	}
