@@ -2,12 +2,10 @@ package lxd
 
 import (
 	"encoding/json"
-	"errors"
 	"fmt"
 	"io"
 	"net/http"
 	"net/url"
-	"slices"
 	"strconv"
 	"strings"
 
@@ -34,7 +32,7 @@ func (r *ProtocolLXD) GetContainerNames() ([]string, error) {
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := "/containers"
-	_, err := r.queryStruct(http.MethodGet, "/containers", nil, "", &urls)
+	_, err := r.queryStruct("GET", "/containers", nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -50,7 +48,7 @@ func (r *ProtocolLXD) GetContainers() ([]api.Container, error) {
 	containers := []api.Container{}
 
 	// Fetch the raw value
-	_, err := r.queryStruct(http.MethodGet, "/containers?recursion=1", nil, "", &containers)
+	_, err := r.queryStruct("GET", "/containers?recursion=1", nil, "", &containers)
 	if err != nil {
 		return nil, err
 	}
@@ -70,7 +68,7 @@ func (r *ProtocolLXD) GetContainersFull() ([]api.ContainerFull, error) {
 	}
 
 	// Fetch the raw value
-	_, err = r.queryStruct(http.MethodGet, "/containers?recursion=2", nil, "", &containers)
+	_, err = r.queryStruct("GET", "/containers?recursion=2", nil, "", &containers)
 	if err != nil {
 		return nil, err
 	}
@@ -85,7 +83,7 @@ func (r *ProtocolLXD) GetContainer(name string) (*api.Container, string, error)
 	container := api.Container{}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct(http.MethodGet, "/containers/"+url.PathEscape(name), nil, "", &container)
+	etag, err := r.queryStruct("GET", "/containers/"+url.PathEscape(name), nil, "", &container)
 	if err != nil {
 		return nil, "", err
 	}
@@ -105,7 +103,7 @@ func (r *ProtocolLXD) CreateContainerFromBackup(args ContainerBackupArgs) (Opera
 
 	if args.PoolName == "" {
 		// Send the request
-		op, _, err := r.queryOperation(http.MethodPost, "/containers", args.BackupFile, "", true)
+		op, _, err := r.queryOperation("POST", "/containers", args.BackupFile, "", true)
 		if err != nil {
 			return nil, err
 		}
@@ -124,7 +122,7 @@ func (r *ProtocolLXD) CreateContainerFromBackup(args ContainerBackupArgs) (Opera
 		return nil, err
 	}
 
-	req, err := http.NewRequest(http.MethodPost, reqURL, args.BackupFile)
+	req, err := http.NewRequest("POST", reqURL, args.BackupFile)
 	if err != nil {
 		return nil, err
 	}
@@ -174,7 +172,7 @@ func (r *ProtocolLXD) CreateContainer(container api.ContainersPost) (Operation,
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, "/containers", container, "", true)
+	op, _, err := r.queryOperation("POST", "/containers", container, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -184,7 +182,7 @@ func (r *ProtocolLXD) CreateContainer(container api.ContainersPost) (Operation,
 
 func (r *ProtocolLXD) tryCreateContainer(req api.ContainersPost, urls []string) (RemoteOperation, error) {
 	if len(urls) == 0 {
-		return nil, errors.New("The source server isn't listening on the network")
+		return nil, fmt.Errorf("The source server isn't listening on the network")
 	}
 
 	rop := remoteOperation{
@@ -323,35 +321,35 @@ func (r *ProtocolLXD) CopyContainer(source InstanceServer, container api.Contain
 		// Quick checks.
 		if args.ContainerOnly {
 			if !r.HasExtension("container_only_migration") {
-				return nil, errors.New("The target server is missing the required \"container_only_migration\" API extension")
+				return nil, fmt.Errorf("The target server is missing the required \"container_only_migration\" API extension")
 			}
 
 			if !source.HasExtension("container_only_migration") {
-				return nil, errors.New("The source server is missing the required \"container_only_migration\" API extension")
+				return nil, fmt.Errorf("The source server is missing the required \"container_only_migration\" API extension")
 			}
 		}
 
-		if slices.Contains([]string{"push", "relay"}, args.Mode) {
+		if shared.ValueInSlice(args.Mode, []string{"push", "relay"}) {
 			if !r.HasExtension("container_push") {
-				return nil, errors.New("The target server is missing the required \"container_push\" API extension")
+				return nil, fmt.Errorf("The target server is missing the required \"container_push\" API extension")
 			}
 
 			if !source.HasExtension("container_push") {
-				return nil, errors.New("The source server is missing the required \"container_push\" API extension")
+				return nil, fmt.Errorf("The source server is missing the required \"container_push\" API extension")
 			}
 		}
 
 		if args.Mode == "push" && !source.HasExtension("container_push_target") {
-			return nil, errors.New("The source server is missing the required \"container_push_target\" API extension")
+			return nil, fmt.Errorf("The source server is missing the required \"container_push_target\" API extension")
 		}
 
 		if args.Refresh {
 			if !r.HasExtension("container_incremental_copy") {
-				return nil, errors.New("The target server is missing the required \"container_incremental_copy\" API extension")
+				return nil, fmt.Errorf("The target server is missing the required \"container_incremental_copy\" API extension")
 			}
 
 			if !source.HasExtension("container_incremental_copy") {
-				return nil, errors.New("The source server is missing the required \"container_incremental_copy\" API extension")
+				return nil, fmt.Errorf("The source server is missing the required \"container_incremental_copy\" API extension")
 			}
 		}
 
@@ -540,7 +538,7 @@ func (r *ProtocolLXD) CopyContainer(source InstanceServer, container api.Contain
 // Deprecated: Use UpdateInstance instead.
 func (r *ProtocolLXD) UpdateContainer(name string, container api.ContainerPut, ETag string) (Operation, error) {
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPut, "/containers/"+url.PathEscape(name), container, ETag, true)
+	op, _, err := r.queryOperation("PUT", "/containers/"+url.PathEscape(name), container, ETag, true)
 	if err != nil {
 		return nil, err
 	}
@@ -554,11 +552,11 @@ func (r *ProtocolLXD) UpdateContainer(name string, container api.ContainerPut, E
 func (r *ProtocolLXD) RenameContainer(name string, container api.ContainerPost) (Operation, error) {
 	// Quick check.
 	if container.Migration {
-		return nil, errors.New("Can't ask for a migration through RenameContainer")
+		return nil, fmt.Errorf("Can't ask for a migration through RenameContainer")
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, "/containers/"+url.PathEscape(name), container, "", true)
+	op, _, err := r.queryOperation("POST", "/containers/"+url.PathEscape(name), container, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -568,7 +566,7 @@ func (r *ProtocolLXD) RenameContainer(name string, container api.ContainerPost)
 
 func (r *ProtocolLXD) tryMigrateContainer(source InstanceServer, name string, req api.ContainerPost, urls []string) (RemoteOperation, error) {
 	if len(urls) == 0 {
-		return nil, errors.New("The target server isn't listening on the network")
+		return nil, fmt.Errorf("The target server isn't listening on the network")
 	}
 
 	rop := remoteOperation{
@@ -634,11 +632,11 @@ func (r *ProtocolLXD) MigrateContainer(name string, container api.ContainerPost)
 
 	// Quick check.
 	if !container.Migration {
-		return nil, errors.New("Can't ask for a rename through MigrateContainer")
+		return nil, fmt.Errorf("Can't ask for a rename through MigrateContainer")
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, "/containers/"+url.PathEscape(name), container, "", true)
+	op, _, err := r.queryOperation("POST", "/containers/"+url.PathEscape(name), container, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -651,7 +649,7 @@ func (r *ProtocolLXD) MigrateContainer(name string, container api.ContainerPost)
 // Deprecated: Use DeleteInstance instead.
 func (r *ProtocolLXD) DeleteContainer(name string) (Operation, error) {
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodDelete, "/containers/"+url.PathEscape(name), nil, "", true)
+	op, _, err := r.queryOperation("DELETE", "/containers/"+url.PathEscape(name), nil, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -678,7 +676,7 @@ func (r *ProtocolLXD) ExecContainer(containerName string, exec api.ContainerExec
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, "/containers/"+url.PathEscape(containerName)+"/exec", exec, "", true)
+	op, _, err := r.queryOperation("POST", "/containers/"+url.PathEscape(containerName)+"/exec", exec, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -832,7 +830,7 @@ func (r *ProtocolLXD) GetContainerFile(containerName string, path string) (io.Re
 		return nil, nil, err
 	}
 
-	req, err := http.NewRequest(http.MethodGet, requestURL, nil)
+	req, err := http.NewRequest("GET", requestURL, nil)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -922,7 +920,7 @@ func (r *ProtocolLXD) CreateContainerFile(containerName string, path string, arg
 		return err
 	}
 
-	req, err := http.NewRequest(http.MethodPost, requestURL, args.Content)
+	req, err := http.NewRequest("POST", requestURL, args.Content)
 	if err != nil {
 		return err
 	}
@@ -973,7 +971,7 @@ func (r *ProtocolLXD) DeleteContainerFile(containerName string, path string) err
 	}
 
 	// Send the request
-	_, _, err = r.query(http.MethodDelete, "/containers/"+url.PathEscape(containerName)+"/files?path="+url.QueryEscape(path), nil, "")
+	_, _, err = r.query("DELETE", "/containers/"+url.PathEscape(containerName)+"/files?path="+url.QueryEscape(path), nil, "")
 	if err != nil {
 		return err
 	}
@@ -988,7 +986,7 @@ func (r *ProtocolLXD) GetContainerSnapshotNames(containerName string) ([]string,
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := "/containers/" + url.PathEscape(containerName) + "/snapshots"
-	_, err := r.queryStruct(http.MethodGet, baseURL, nil, "", &urls)
+	_, err := r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -1004,7 +1002,7 @@ func (r *ProtocolLXD) GetContainerSnapshots(containerName string) ([]api.Contain
 	snapshots := []api.ContainerSnapshot{}
 
 	// Fetch the raw value
-	_, err := r.queryStruct(http.MethodGet, "/containers/"+url.PathEscape(containerName)+"/snapshots?recursion=1", nil, "", &snapshots)
+	_, err := r.queryStruct("GET", "/containers/"+url.PathEscape(containerName)+"/snapshots?recursion=1", nil, "", &snapshots)
 	if err != nil {
 		return nil, err
 	}
@@ -1019,7 +1017,7 @@ func (r *ProtocolLXD) GetContainerSnapshot(containerName string, name string) (*
 	snapshot := api.ContainerSnapshot{}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct(http.MethodGet, "/containers/"+url.PathEscape(containerName)+"/snapshots/"+url.PathEscape(name), nil, "", &snapshot)
+	etag, err := r.queryStruct("GET", "/containers/"+url.PathEscape(containerName)+"/snapshots/"+url.PathEscape(name), nil, "", &snapshot)
 	if err != nil {
 		return nil, "", err
 	}
@@ -1040,7 +1038,7 @@ func (r *ProtocolLXD) CreateContainerSnapshot(containerName string, snapshot api
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, "/containers/"+url.PathEscape(containerName)+"/snapshots", snapshot, "", true)
+	op, _, err := r.queryOperation("POST", "/containers/"+url.PathEscape(containerName)+"/snapshots", snapshot, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1075,7 +1073,7 @@ func (r *ProtocolLXD) CopyContainerSnapshot(source InstanceServer, containerName
 			return nil, err
 		}
 
-		req.Stateful = snapshot.Stateful
+		req.ContainerPut.Stateful = snapshot.Stateful
 		req.Source.Live = false // Snapshots are never running and so we don't need live migration.
 	}
 
@@ -1084,18 +1082,18 @@ func (r *ProtocolLXD) CopyContainerSnapshot(source InstanceServer, containerName
 	// Process the copy arguments
 	if args != nil {
 		// Quick checks.
-		if slices.Contains([]string{"push", "relay"}, args.Mode) {
+		if shared.ValueInSlice(args.Mode, []string{"push", "relay"}) {
 			if !r.HasExtension("container_push") {
-				return nil, errors.New("The target server is missing the required \"container_push\" API extension")
+				return nil, fmt.Errorf("The target server is missing the required \"container_push\" API extension")
 			}
 
 			if !source.HasExtension("container_push") {
-				return nil, errors.New("The source server is missing the required \"container_push\" API extension")
+				return nil, fmt.Errorf("The source server is missing the required \"container_push\" API extension")
 			}
 		}
 
 		if args.Mode == "push" && !source.HasExtension("container_push_target") {
-			return nil, errors.New("The source server is missing the required \"container_push_target\" API extension")
+			return nil, fmt.Errorf("The source server is missing the required \"container_push_target\" API extension")
 		}
 
 		// Allow overriding the target name
@@ -1283,11 +1281,11 @@ func (r *ProtocolLXD) CopyContainerSnapshot(source InstanceServer, containerName
 func (r *ProtocolLXD) RenameContainerSnapshot(containerName string, name string, container api.ContainerSnapshotPost) (Operation, error) {
 	// Quick check.
 	if container.Migration {
-		return nil, errors.New("Can't ask for a migration through RenameContainerSnapshot")
+		return nil, fmt.Errorf("Can't ask for a migration through RenameContainerSnapshot")
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, "/containers/"+url.PathEscape(containerName)+"/snapshots/"+url.PathEscape(name), container, "", true)
+	op, _, err := r.queryOperation("POST", "/containers/"+url.PathEscape(containerName)+"/snapshots/"+url.PathEscape(name), container, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1299,7 +1297,7 @@ func (r *ProtocolLXD) RenameContainerSnapshot(containerName string, name string,
 // It runs the migration asynchronously and returns a RemoteOperation to track the migration status and any errors.
 func (r *ProtocolLXD) tryMigrateContainerSnapshot(source InstanceServer, containerName string, name string, req api.ContainerSnapshotPost, urls []string) (RemoteOperation, error) {
 	if len(urls) == 0 {
-		return nil, errors.New("The target server isn't listening on the network")
+		return nil, fmt.Errorf("The target server isn't listening on the network")
 	}
 
 	rop := remoteOperation{
@@ -1358,11 +1356,11 @@ func (r *ProtocolLXD) tryMigrateContainerSnapshot(source InstanceServer, contain
 func (r *ProtocolLXD) MigrateContainerSnapshot(containerName string, name string, container api.ContainerSnapshotPost) (Operation, error) {
 	// Quick check.
 	if !container.Migration {
-		return nil, errors.New("Can't ask for a rename through MigrateContainerSnapshot")
+		return nil, fmt.Errorf("Can't ask for a rename through MigrateContainerSnapshot")
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, "/containers/"+url.PathEscape(containerName)+"/snapshots/"+url.PathEscape(name), container, "", true)
+	op, _, err := r.queryOperation("POST", "/containers/"+url.PathEscape(containerName)+"/snapshots/"+url.PathEscape(name), container, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1375,7 +1373,7 @@ func (r *ProtocolLXD) MigrateContainerSnapshot(containerName string, name string
 // Deprecated: Use DeleteInstanceSnapshot instead.
 func (r *ProtocolLXD) DeleteContainerSnapshot(containerName string, name string) (Operation, error) {
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodDelete, "/containers/"+url.PathEscape(containerName)+"/snapshots/"+url.PathEscape(name), nil, "", true)
+	op, _, err := r.queryOperation("DELETE", "/containers/"+url.PathEscape(containerName)+"/snapshots/"+url.PathEscape(name), nil, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1393,7 +1391,7 @@ func (r *ProtocolLXD) UpdateContainerSnapshot(containerName string, name string,
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPut, "/containers/"+url.PathEscape(containerName)+"/snapshots/"+url.PathEscape(name), container, ETag, true)
+	op, _, err := r.queryOperation("PUT", "/containers/"+url.PathEscape(containerName)+"/snapshots/"+url.PathEscape(name), container, ETag, true)
 	if err != nil {
 		return nil, err
 	}
@@ -1408,7 +1406,7 @@ func (r *ProtocolLXD) GetContainerState(name string) (*api.ContainerState, strin
 	state := api.ContainerState{}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct(http.MethodGet, "/containers/"+url.PathEscape(name)+"/state", nil, "", &state)
+	etag, err := r.queryStruct("GET", "/containers/"+url.PathEscape(name)+"/state", nil, "", &state)
 	if err != nil {
 		return nil, "", err
 	}
@@ -1421,7 +1419,7 @@ func (r *ProtocolLXD) GetContainerState(name string) (*api.ContainerState, strin
 // Deprecated: Use UpdateInstanceState instead.
 func (r *ProtocolLXD) UpdateContainerState(name string, state api.ContainerStatePut, ETag string) (Operation, error) {
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPut, "/containers/"+url.PathEscape(name)+"/state", state, ETag, true)
+	op, _, err := r.queryOperation("PUT", "/containers/"+url.PathEscape(name)+"/state", state, ETag, true)
 	if err != nil {
 		return nil, err
 	}
@@ -1436,7 +1434,7 @@ func (r *ProtocolLXD) GetContainerLogfiles(name string) ([]string, error) {
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := "/containers/" + url.PathEscape(name) + "/logs"
-	_, err := r.queryStruct(http.MethodGet, baseURL, nil, "", &urls)
+	_, err := r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -1459,7 +1457,7 @@ func (r *ProtocolLXD) GetContainerLogfile(name string, filename string) (io.Read
 		return nil, err
 	}
 
-	req, err := http.NewRequest(http.MethodGet, url, nil)
+	req, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -1486,7 +1484,7 @@ func (r *ProtocolLXD) GetContainerLogfile(name string, filename string) (io.Read
 // Deprecated: Use DeleteInstanceLogfile instead.
 func (r *ProtocolLXD) DeleteContainerLogfile(name string, filename string) error {
 	// Send the request
-	_, _, err := r.query(http.MethodDelete, "/containers/"+url.PathEscape(name)+"/logs/"+url.PathEscape(filename), nil, "")
+	_, _, err := r.query("DELETE", "/containers/"+url.PathEscape(name)+"/logs/"+url.PathEscape(filename), nil, "")
 	if err != nil {
 		return err
 	}
@@ -1506,7 +1504,7 @@ func (r *ProtocolLXD) GetContainerMetadata(name string) (*api.ImageMetadata, str
 	metadata := api.ImageMetadata{}
 
 	url := "/containers/" + url.PathEscape(name) + "/metadata"
-	etag, err := r.queryStruct(http.MethodGet, url, nil, "", &metadata)
+	etag, err := r.queryStruct("GET", url, nil, "", &metadata)
 	if err != nil {
 		return nil, "", err
 	}
@@ -1524,7 +1522,7 @@ func (r *ProtocolLXD) SetContainerMetadata(name string, metadata api.ImageMetada
 	}
 
 	url := "/containers/" + url.PathEscape(name) + "/metadata"
-	_, _, err = r.query(http.MethodPut, url, metadata, ETag)
+	_, _, err = r.query("PUT", url, metadata, ETag)
 	if err != nil {
 		return err
 	}
@@ -1544,7 +1542,7 @@ func (r *ProtocolLXD) GetContainerTemplateFiles(containerName string) ([]string,
 	templates := []string{}
 
 	url := "/containers/" + url.PathEscape(containerName) + "/metadata/templates"
-	_, err = r.queryStruct(http.MethodGet, url, nil, "", &templates)
+	_, err = r.queryStruct("GET", url, nil, "", &templates)
 	if err != nil {
 		return nil, err
 	}
@@ -1568,7 +1566,7 @@ func (r *ProtocolLXD) GetContainerTemplateFile(containerName string, templateNam
 		return nil, err
 	}
 
-	req, err := http.NewRequest(http.MethodGet, url, nil)
+	req, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -1606,7 +1604,7 @@ func (r *ProtocolLXD) CreateContainerTemplateFile(containerName string, template
 		return err
 	}
 
-	req, err := http.NewRequest(http.MethodPost, url, content)
+	req, err := http.NewRequest("POST", url, content)
 	if err != nil {
 		return err
 	}
@@ -1641,7 +1639,7 @@ func (r *ProtocolLXD) DeleteContainerTemplateFile(name string, templateName stri
 		return err
 	}
 
-	_, _, err = r.query(http.MethodDelete, "/containers/"+url.PathEscape(name)+"/metadata/templates?path="+url.QueryEscape(templateName), nil, "")
+	_, _, err = r.query("DELETE", "/containers/"+url.PathEscape(name)+"/metadata/templates?path="+url.QueryEscape(templateName), nil, "")
 	return err
 }
 
@@ -1655,7 +1653,7 @@ func (r *ProtocolLXD) ConsoleContainer(containerName string, console api.Contain
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, "/containers/"+url.PathEscape(containerName)+"/console", console, "", true)
+	op, _, err := r.queryOperation("POST", "/containers/"+url.PathEscape(containerName)+"/console", console, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1663,11 +1661,11 @@ func (r *ProtocolLXD) ConsoleContainer(containerName string, console api.Contain
 	opAPI := op.Get()
 
 	if args == nil || args.Terminal == nil {
-		return nil, errors.New("A terminal must be set")
+		return nil, fmt.Errorf("A terminal must be set")
 	}
 
 	if args.Control == nil {
-		return nil, errors.New("A control channel must be set")
+		return nil, fmt.Errorf("A control channel must be set")
 	}
 
 	// Parse the fds
@@ -1686,7 +1684,7 @@ func (r *ProtocolLXD) ConsoleContainer(containerName string, console api.Contain
 	var controlConn *websocket.Conn
 	// Call the control handler with a connection to the control socket
 	if fds[api.SecretNameControl] == "" {
-		return nil, errors.New("Did not receive a file descriptor for the control channel")
+		return nil, fmt.Errorf("Did not receive a file descriptor for the control channel")
 	}
 
 	controlConn, err = r.GetOperationWebsocket(opAPI.ID, fds[api.SecretNameControl])
@@ -1740,7 +1738,7 @@ func (r *ProtocolLXD) GetContainerConsoleLog(containerName string, args *Contain
 		return nil, err
 	}
 
-	req, err := http.NewRequest(http.MethodGet, url, nil)
+	req, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -1772,7 +1770,7 @@ func (r *ProtocolLXD) DeleteContainerConsoleLog(containerName string, args *Cont
 	}
 
 	// Send the request
-	_, _, err = r.query(http.MethodDelete, "/containers/"+url.PathEscape(containerName)+"/console", nil, "")
+	_, _, err = r.query("DELETE", "/containers/"+url.PathEscape(containerName)+"/console", nil, "")
 	if err != nil {
 		return err
 	}
@@ -1792,7 +1790,7 @@ func (r *ProtocolLXD) GetContainerBackupNames(containerName string) ([]string, e
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := "/containers/" + url.PathEscape(containerName) + "/backups"
-	_, err = r.queryStruct(http.MethodGet, baseURL, nil, "", &urls)
+	_, err = r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -1813,7 +1811,7 @@ func (r *ProtocolLXD) GetContainerBackups(containerName string) ([]api.Container
 	// Fetch the raw value
 	backups := []api.ContainerBackup{}
 
-	_, err = r.queryStruct(http.MethodGet, "/containers/"+url.PathEscape(containerName)+"/backups?recursion=1", nil, "", &backups)
+	_, err = r.queryStruct("GET", "/containers/"+url.PathEscape(containerName)+"/backups?recursion=1", nil, "", &backups)
 	if err != nil {
 		return nil, err
 	}
@@ -1832,7 +1830,7 @@ func (r *ProtocolLXD) GetContainerBackup(containerName string, name string) (*ap
 
 	// Fetch the raw value
 	backup := api.ContainerBackup{}
-	etag, err := r.queryStruct(http.MethodGet, "/containers/"+url.PathEscape(containerName)+"/backups/"+url.PathEscape(name), nil, "", &backup)
+	etag, err := r.queryStruct("GET", "/containers/"+url.PathEscape(containerName)+"/backups/"+url.PathEscape(name), nil, "", &backup)
 	if err != nil {
 		return nil, "", err
 	}
@@ -1850,7 +1848,7 @@ func (r *ProtocolLXD) CreateContainerBackup(containerName string, backup api.Con
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, "/containers/"+url.PathEscape(containerName)+"/backups", backup, "", true)
+	op, _, err := r.queryOperation("POST", "/containers/"+url.PathEscape(containerName)+"/backups", backup, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1868,7 +1866,7 @@ func (r *ProtocolLXD) RenameContainerBackup(containerName string, name string, b
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, "/containers/"+url.PathEscape(containerName)+"/backups/"+url.PathEscape(name), backup, "", true)
+	op, _, err := r.queryOperation("POST", "/containers/"+url.PathEscape(containerName)+"/backups/"+url.PathEscape(name), backup, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1886,7 +1884,7 @@ func (r *ProtocolLXD) DeleteContainerBackup(containerName string, name string) (
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodDelete, "/containers/"+url.PathEscape(containerName)+"/backups/"+url.PathEscape(name), nil, "", true)
+	op, _, err := r.queryOperation("DELETE", "/containers/"+url.PathEscape(containerName)+"/backups/"+url.PathEscape(name), nil, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1910,7 +1908,7 @@ func (r *ProtocolLXD) GetContainerBackupFile(containerName string, name string,
 	}
 
 	// Prepare the download request
-	request, err := http.NewRequest(http.MethodGet, uri, nil)
+	request, err := http.NewRequest("GET", uri, nil)
 	if err != nil {
 		return nil, err
 	}
