@@ -4,14 +4,12 @@ import (
 	"bufio"
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"io"
 	"net"
 	"net/http"
 	"net/url"
 	"path/filepath"
-	"slices"
 	"strconv"
 	"strings"
 
@@ -42,7 +40,7 @@ func (r *ProtocolLXD) instanceTypeToPath(instanceType api.InstanceType) (string,
 			return "/containers", v, nil
 		}
 
-		return "", v, errors.New("Requested instance type not supported by server")
+		return "", v, fmt.Errorf("Requested instance type not supported by server")
 	}
 
 	// If a specific instance type has been requested, add the instance-type filter parameter
@@ -64,7 +62,7 @@ func (r *ProtocolLXD) GetInstanceNames(instanceType api.InstanceType) ([]string,
 
 	// Fetch the raw URL values.
 	urls := []string{}
-	_, err = r.queryStruct(http.MethodGet, baseURL+"?"+v.Encode(), nil, "", &urls)
+	_, err = r.queryStruct("GET", baseURL+"?"+v.Encode(), nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -86,7 +84,7 @@ func (r *ProtocolLXD) GetInstanceNamesAllProjects(instanceType api.InstanceType)
 	v.Set("all-projects", "true")
 
 	// Fetch the raw URL values.
-	_, err = r.queryStruct(http.MethodGet, path+"?"+v.Encode(), nil, "", &instances)
+	_, err = r.queryStruct("GET", path+"?"+v.Encode(), nil, "", &instances)
 	if err != nil {
 		return nil, err
 	}
@@ -111,7 +109,7 @@ func (r *ProtocolLXD) GetInstances(instanceType api.InstanceType) ([]api.Instanc
 	v.Set("recursion", "1")
 
 	// Fetch the raw value
-	_, err = r.queryStruct(http.MethodGet, path+"?"+v.Encode(), nil, "", &instances)
+	_, err = r.queryStruct("GET", path+"?"+v.Encode(), nil, "", &instances)
 	if err != nil {
 		return nil, err
 	}
@@ -137,7 +135,7 @@ func (r *ProtocolLXD) GetInstancesWithFilter(instanceType api.InstanceType, filt
 	v.Set("filter", parseFilters(filters))
 
 	// Fetch the raw value
-	_, err = r.queryStruct(http.MethodGet, path+"?"+v.Encode(), nil, "", &instances)
+	_, err = r.queryStruct("GET", path+"?"+v.Encode(), nil, "", &instances)
 	if err != nil {
 		return nil, err
 	}
@@ -163,7 +161,7 @@ func (r *ProtocolLXD) GetInstancesAllProjects(instanceType api.InstanceType) ([]
 	}
 
 	// Fetch the raw value
-	_, err = r.queryStruct(http.MethodGet, path+"?"+v.Encode(), nil, "", &instances)
+	_, err = r.queryStruct("GET", path+"?"+v.Encode(), nil, "", &instances)
 	if err != nil {
 		return nil, err
 	}
@@ -195,7 +193,7 @@ func (r *ProtocolLXD) GetInstancesAllProjectsWithFilter(instanceType api.Instanc
 	}
 
 	// Fetch the raw value
-	_, err = r.queryStruct(http.MethodGet, path+"?"+v.Encode(), nil, "", &instances)
+	_, err = r.queryStruct("GET", path+"?"+v.Encode(), nil, "", &instances)
 	if err != nil {
 		return nil, err
 	}
@@ -211,7 +209,7 @@ func (r *ProtocolLXD) UpdateInstances(state api.InstancesPut, ETag string) (Oper
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPut, path+"?"+v.Encode(), state, ETag, true)
+	op, _, err := r.queryOperation("PUT", path+"?"+v.Encode(), state, ETag, true)
 	if err != nil {
 		return nil, err
 	}
@@ -227,7 +225,7 @@ func (r *ProtocolLXD) rebuildInstance(instanceName string, instance api.Instance
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, path+"/"+url.PathEscape(instanceName)+"/rebuild", instance, "", true)
+	op, _, err := r.queryOperation("POST", path+"/"+url.PathEscape(instanceName)+"/rebuild", instance, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -239,7 +237,7 @@ func (r *ProtocolLXD) rebuildInstance(instanceName string, instance api.Instance
 // It runs the rebuild process asynchronously and returns a RemoteOperation to monitor the progress and any errors.
 func (r *ProtocolLXD) tryRebuildInstance(instanceName string, req api.InstanceRebuildPost, urls []string, op Operation) (RemoteOperation, error) {
 	if len(urls) == 0 {
-		return nil, errors.New("The source server isn't listening on the network")
+		return nil, fmt.Errorf("The source server isn't listening on the network")
 	}
 
 	rop := remoteOperation{
@@ -362,7 +360,7 @@ func (r *ProtocolLXD) GetInstancesFull(instanceType api.InstanceType) ([]api.Ins
 	}
 
 	// Fetch the raw value
-	_, err = r.queryStruct(http.MethodGet, path+"?"+v.Encode(), nil, "", &instances)
+	_, err = r.queryStruct("GET", path+"?"+v.Encode(), nil, "", &instances)
 	if err != nil {
 		return nil, err
 	}
@@ -393,7 +391,7 @@ func (r *ProtocolLXD) GetInstancesFullWithFilter(instanceType api.InstanceType,
 	}
 
 	// Fetch the raw value
-	_, err = r.queryStruct(http.MethodGet, path+"?"+v.Encode(), nil, "", &instances)
+	_, err = r.queryStruct("GET", path+"?"+v.Encode(), nil, "", &instances)
 	if err != nil {
 		return nil, err
 	}
@@ -424,7 +422,7 @@ func (r *ProtocolLXD) GetInstancesFullAllProjects(instanceType api.InstanceType)
 	}
 
 	// Fetch the raw value
-	_, err = r.queryStruct(http.MethodGet, path+"?"+v.Encode(), nil, "", &instances)
+	_, err = r.queryStruct("GET", path+"?"+v.Encode(), nil, "", &instances)
 	if err != nil {
 		return nil, err
 	}
@@ -461,7 +459,7 @@ func (r *ProtocolLXD) GetInstancesFullAllProjectsWithFilter(instanceType api.Ins
 	}
 
 	// Fetch the raw value
-	_, err = r.queryStruct(http.MethodGet, path+"?"+v.Encode(), nil, "", &instances)
+	_, err = r.queryStruct("GET", path+"?"+v.Encode(), nil, "", &instances)
 	if err != nil {
 		return nil, err
 	}
@@ -479,7 +477,7 @@ func (r *ProtocolLXD) GetInstance(name string) (*api.Instance, string, error) {
 	}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct(http.MethodGet, path+"/"+url.PathEscape(name), nil, "", &instance)
+	etag, err := r.queryStruct("GET", path+"/"+url.PathEscape(name), nil, "", &instance)
 	if err != nil {
 		return nil, "", err
 	}
@@ -502,7 +500,7 @@ func (r *ProtocolLXD) GetInstanceUEFIVars(name string) (*api.InstanceUEFIVars, s
 	}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct(http.MethodGet, path+"/"+url.PathEscape(name)+"/uefi-vars", nil, "", &instanceUEFI)
+	etag, err := r.queryStruct("GET", path+"/"+url.PathEscape(name)+"/uefi-vars", nil, "", &instanceUEFI)
 	if err != nil {
 		return nil, "", err
 	}
@@ -523,7 +521,7 @@ func (r *ProtocolLXD) UpdateInstanceUEFIVars(name string, instanceUEFI api.Insta
 	}
 
 	// Send the request
-	_, _, err = r.query(http.MethodPut, path+"/"+url.PathEscape(name)+"/uefi-vars", instanceUEFI, ETag)
+	_, _, err = r.query("PUT", path+"/"+url.PathEscape(name)+"/uefi-vars", instanceUEFI, ETag)
 	if err != nil {
 		return err
 	}
@@ -571,7 +569,7 @@ func (r *ProtocolLXD) GetInstanceFull(name string) (*api.InstanceFull, string, e
 	}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct(http.MethodGet, path+"/"+url.PathEscape(name)+"?recursion=1", nil, "", &instance)
+	etag, err := r.queryStruct("GET", path+"/"+url.PathEscape(name)+"?recursion=1", nil, "", &instance)
 	if err != nil {
 		return nil, "", err
 	}
@@ -594,7 +592,7 @@ func (r *ProtocolLXD) CreateInstanceFromBackup(args InstanceBackupArgs) (Operati
 
 	if args.PoolName == "" && args.Name == "" && len(args.Devices) == 0 {
 		// Send the request
-		op, _, err := r.queryOperation(http.MethodPost, path, args.BackupFile, "", true)
+		op, _, err := r.queryOperation("POST", path, args.BackupFile, "", true)
 		if err != nil {
 			return nil, err
 		}
@@ -630,7 +628,7 @@ func (r *ProtocolLXD) CreateInstanceFromBackup(args InstanceBackupArgs) (Operati
 		return nil, err
 	}
 
-	req, err := http.NewRequest(http.MethodPost, reqURL, args.BackupFile)
+	req, err := http.NewRequest("POST", reqURL, args.BackupFile)
 	if err != nil {
 		return nil, err
 	}
@@ -707,7 +705,7 @@ func (r *ProtocolLXD) CreateInstance(instance api.InstancesPost) (Operation, err
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, path, instance, "", true)
+	op, _, err := r.queryOperation("POST", path, instance, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -719,7 +717,7 @@ func (r *ProtocolLXD) CreateInstance(instance api.InstancesPost) (Operation, err
 // It runs the instance creation asynchronously and returns a RemoteOperation to monitor the progress and any errors.
 func (r *ProtocolLXD) tryCreateInstance(req api.InstancesPost, urls []string, op Operation) (RemoteOperation, error) {
 	if len(urls) == 0 {
-		return nil, errors.New("The source server isn't listening on the network")
+		return nil, fmt.Errorf("The source server isn't listening on the network")
 	}
 
 	rop := remoteOperation{
@@ -853,41 +851,41 @@ func (r *ProtocolLXD) CopyInstance(source InstanceServer, instance api.Instance,
 		// Quick checks.
 		if args.InstanceOnly {
 			if !r.HasExtension("container_only_migration") {
-				return nil, errors.New("The target server is missing the required \"container_only_migration\" API extension")
+				return nil, fmt.Errorf("The target server is missing the required \"container_only_migration\" API extension")
 			}
 
 			if !source.HasExtension("container_only_migration") {
-				return nil, errors.New("The source server is missing the required \"container_only_migration\" API extension")
+				return nil, fmt.Errorf("The source server is missing the required \"container_only_migration\" API extension")
 			}
 		}
 
-		if slices.Contains([]string{"push", "relay"}, args.Mode) {
+		if shared.ValueInSlice(args.Mode, []string{"push", "relay"}) {
 			if !r.HasExtension("container_push") {
-				return nil, errors.New("The target server is missing the required \"container_push\" API extension")
+				return nil, fmt.Errorf("The target server is missing the required \"container_push\" API extension")
 			}
 
 			if !source.HasExtension("container_push") {
-				return nil, errors.New("The source server is missing the required \"container_push\" API extension")
+				return nil, fmt.Errorf("The source server is missing the required \"container_push\" API extension")
 			}
 		}
 
 		if args.Mode == "push" && !source.HasExtension("container_push_target") {
-			return nil, errors.New("The source server is missing the required \"container_push_target\" API extension")
+			return nil, fmt.Errorf("The source server is missing the required \"container_push_target\" API extension")
 		}
 
 		if args.Refresh {
 			if !r.HasExtension("container_incremental_copy") {
-				return nil, errors.New("The target server is missing the required \"container_incremental_copy\" API extension")
+				return nil, fmt.Errorf("The target server is missing the required \"container_incremental_copy\" API extension")
 			}
 
 			if !source.HasExtension("container_incremental_copy") {
-				return nil, errors.New("The source server is missing the required \"container_incremental_copy\" API extension")
+				return nil, fmt.Errorf("The source server is missing the required \"container_incremental_copy\" API extension")
 			}
 		}
 
 		if args.AllowInconsistent {
 			if !r.HasExtension("instance_allow_inconsistent_copy") {
-				return nil, errors.New("The target server is missing the required \"instance_allow_inconsistent_copy\" API extension")
+				return nil, fmt.Errorf("The target server is missing the required \"instance_allow_inconsistent_copy\" API extension")
 			}
 		}
 
@@ -903,8 +901,6 @@ func (r *ProtocolLXD) CopyInstance(source InstanceServer, instance api.Instance,
 		req.Source.ContainerOnly = args.InstanceOnly //nolint:staticcheck,unused
 		req.Source.Refresh = args.Refresh
 		req.Source.AllowInconsistent = args.AllowInconsistent
-
-		req.Source.OverrideSnapshotProfiles = args.OverrideSnapshotProfiles
 	}
 
 	if req.Source.Live {
@@ -1094,7 +1090,7 @@ func (r *ProtocolLXD) UpdateInstance(name string, instance api.InstancePut, ETag
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPut, path+"/"+url.PathEscape(name), instance, ETag, true)
+	op, _, err := r.queryOperation("PUT", path+"/"+url.PathEscape(name), instance, ETag, true)
 	if err != nil {
 		return nil, err
 	}
@@ -1111,11 +1107,11 @@ func (r *ProtocolLXD) RenameInstance(name string, instance api.InstancePost) (Op
 
 	// Quick check.
 	if instance.Migration {
-		return nil, errors.New("Can't ask for a migration through RenameInstance")
+		return nil, fmt.Errorf("Can't ask for a migration through RenameInstance")
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, path+"/"+url.PathEscape(name), instance, "", true)
+	op, _, err := r.queryOperation("POST", path+"/"+url.PathEscape(name), instance, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1127,7 +1123,7 @@ func (r *ProtocolLXD) RenameInstance(name string, instance api.InstancePost) (Op
 // The function runs the migration operation asynchronously and returns a RemoteOperation to track the progress and handle any errors.
 func (r *ProtocolLXD) tryMigrateInstance(source InstanceServer, name string, req api.InstancePost, urls []string, op Operation) (RemoteOperation, error) {
 	if len(urls) == 0 {
-		return nil, errors.New("The target server isn't listening on the network")
+		return nil, fmt.Errorf("The target server isn't listening on the network")
 	}
 
 	rop := remoteOperation{
@@ -1246,11 +1242,11 @@ func (r *ProtocolLXD) MigrateInstance(name string, instance api.InstancePost) (O
 
 	// Quick check.
 	if !instance.Migration {
-		return nil, errors.New("Can't ask for a rename through MigrateInstance")
+		return nil, fmt.Errorf("Can't ask for a rename through MigrateInstance")
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, path+"/"+url.PathEscape(name), instance, "", true)
+	op, _, err := r.queryOperation("POST", path+"/"+url.PathEscape(name), instance, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1266,7 +1262,7 @@ func (r *ProtocolLXD) DeleteInstance(name string) (Operation, error) {
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodDelete, path+"/"+url.PathEscape(name), nil, "", true)
+	op, _, err := r.queryOperation("DELETE", path+"/"+url.PathEscape(name), nil, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1309,7 +1305,7 @@ func (r *ProtocolLXD) ExecInstance(instanceName string, exec api.InstanceExecPos
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, uri, exec, "", true)
+	op, _, err := r.queryOperation("POST", uri, exec, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1564,7 +1560,7 @@ func (r *ProtocolLXD) GetInstanceFile(instanceName string, filePath string) (io.
 		return nil, nil, err
 	}
 
-	req, err := http.NewRequest(http.MethodGet, requestURL, nil)
+	req, err := http.NewRequest("GET", requestURL, nil)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -1663,7 +1659,7 @@ func (r *ProtocolLXD) CreateInstanceFile(instanceName string, filePath string, a
 		return err
 	}
 
-	req, err := http.NewRequest(http.MethodPost, requestURL, args.Content)
+	req, err := http.NewRequest("POST", requestURL, args.Content)
 	if err != nil {
 		return err
 	}
@@ -1707,8 +1703,6 @@ func (r *ProtocolLXD) CreateInstanceFile(instanceName string, filePath string, a
 		req.Header.Set("X-LXD-modify-perm", strings.Join(modifyPerm, ","))
 	}
 
-	req.Header.Set("Content-Type", "application/octet-stream")
-
 	// Send the request
 	resp, err := r.DoHTTP(req)
 	if err != nil {
@@ -1751,7 +1745,7 @@ func (r *ProtocolLXD) DeleteInstanceFile(instanceName string, filePath string) e
 	}
 
 	// Send the request
-	_, _, err = r.query(http.MethodDelete, requestURL, nil, "")
+	_, _, err = r.query("DELETE", requestURL, nil, "")
 	if err != nil {
 		return err
 	}
@@ -1821,7 +1815,7 @@ func (r *ProtocolLXD) rawSFTPConn(apiURL *url.URL) (net.Conn, error) {
 	}
 
 	if resp.Header.Get("Upgrade") != "sftp" {
-		return nil, errors.New("Missing or unexpected Upgrade header in response")
+		return nil, fmt.Errorf("Missing or unexpected Upgrade header in response")
 	}
 
 	return conn, err
@@ -1870,7 +1864,7 @@ func (r *ProtocolLXD) GetInstanceSnapshotNames(instanceName string) ([]string, e
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := path + "/" + url.PathEscape(instanceName) + "/snapshots"
-	_, err = r.queryStruct(http.MethodGet, baseURL, nil, "", &urls)
+	_, err = r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -1889,7 +1883,7 @@ func (r *ProtocolLXD) GetInstanceSnapshots(instanceName string) ([]api.InstanceS
 	snapshots := []api.InstanceSnapshot{}
 
 	// Fetch the raw value
-	_, err = r.queryStruct(http.MethodGet, path+"/"+url.PathEscape(instanceName)+"/snapshots?recursion=1", nil, "", &snapshots)
+	_, err = r.queryStruct("GET", path+"/"+url.PathEscape(instanceName)+"/snapshots?recursion=1", nil, "", &snapshots)
 	if err != nil {
 		return nil, err
 	}
@@ -1907,7 +1901,7 @@ func (r *ProtocolLXD) GetInstanceSnapshot(instanceName string, name string) (*ap
 	snapshot := api.InstanceSnapshot{}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct(http.MethodGet, path+"/"+url.PathEscape(instanceName)+"/snapshots/"+url.PathEscape(name), nil, "", &snapshot)
+	etag, err := r.queryStruct("GET", path+"/"+url.PathEscape(instanceName)+"/snapshots/"+url.PathEscape(name), nil, "", &snapshot)
 	if err != nil {
 		return nil, "", err
 	}
@@ -1931,7 +1925,7 @@ func (r *ProtocolLXD) CreateInstanceSnapshot(instanceName string, snapshot api.I
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, path+"/"+url.PathEscape(instanceName)+"/snapshots", snapshot, "", true)
+	op, _, err := r.queryOperation("POST", path+"/"+url.PathEscape(instanceName)+"/snapshots", snapshot, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -1964,7 +1958,7 @@ func (r *ProtocolLXD) CopyInstanceSnapshot(source InstanceServer, instanceName s
 			return nil, err
 		}
 
-		req.Stateful = snapshot.Stateful
+		req.InstancePut.Stateful = snapshot.Stateful
 		req.Source.Live = false // Snapshots are never running and so we don't need live migration.
 	}
 
@@ -1973,19 +1967,19 @@ func (r *ProtocolLXD) CopyInstanceSnapshot(source InstanceServer, instanceName s
 	// Process the copy arguments
 	if args != nil {
 		// Quick checks.
-		if slices.Contains([]string{"push", "relay"}, args.Mode) {
+		if shared.ValueInSlice(args.Mode, []string{"push", "relay"}) {
 			err := r.CheckExtension("container_push")
 			if err != nil {
 				return nil, err
 			}
 
 			if !source.HasExtension("container_push") {
-				return nil, errors.New("The source server is missing the required \"container_push\" API extension")
+				return nil, fmt.Errorf("The source server is missing the required \"container_push\" API extension")
 			}
 		}
 
 		if args.Mode == "push" && !source.HasExtension("container_push_target") {
-			return nil, errors.New("The source server is missing the required \"container_push_target\" API extension")
+			return nil, fmt.Errorf("The source server is missing the required \"container_push_target\" API extension")
 		}
 
 		// Allow overriding the target name
@@ -2192,11 +2186,11 @@ func (r *ProtocolLXD) RenameInstanceSnapshot(instanceName string, name string, i
 
 	// Quick check.
 	if instance.Migration {
-		return nil, errors.New("Can't ask for a migration through RenameInstanceSnapshot")
+		return nil, fmt.Errorf("Can't ask for a migration through RenameInstanceSnapshot")
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, path+"/"+url.PathEscape(instanceName)+"/snapshots/"+url.PathEscape(name), instance, "", true)
+	op, _, err := r.queryOperation("POST", path+"/"+url.PathEscape(instanceName)+"/snapshots/"+url.PathEscape(name), instance, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -2206,7 +2200,7 @@ func (r *ProtocolLXD) RenameInstanceSnapshot(instanceName string, name string, i
 
 func (r *ProtocolLXD) tryMigrateInstanceSnapshot(source InstanceServer, instanceName string, name string, req api.InstanceSnapshotPost, urls []string) (RemoteOperation, error) {
 	if len(urls) == 0 {
-		return nil, errors.New("The target server isn't listening on the network")
+		return nil, fmt.Errorf("The target server isn't listening on the network")
 	}
 
 	rop := remoteOperation{
@@ -2268,11 +2262,11 @@ func (r *ProtocolLXD) MigrateInstanceSnapshot(instanceName string, name string,
 
 	// Quick check.
 	if !instance.Migration {
-		return nil, errors.New("Can't ask for a rename through MigrateInstanceSnapshot")
+		return nil, fmt.Errorf("Can't ask for a rename through MigrateInstanceSnapshot")
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, path+"/"+url.PathEscape(instanceName)+"/snapshots/"+url.PathEscape(name), instance, "", true)
+	op, _, err := r.queryOperation("POST", path+"/"+url.PathEscape(instanceName)+"/snapshots/"+url.PathEscape(name), instance, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -2288,7 +2282,7 @@ func (r *ProtocolLXD) DeleteInstanceSnapshot(instanceName string, name string) (
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodDelete, path+"/"+url.PathEscape(instanceName)+"/snapshots/"+url.PathEscape(name), nil, "", true)
+	op, _, err := r.queryOperation("DELETE", path+"/"+url.PathEscape(instanceName)+"/snapshots/"+url.PathEscape(name), nil, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -2309,7 +2303,7 @@ func (r *ProtocolLXD) UpdateInstanceSnapshot(instanceName string, name string, i
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPut, path+"/"+url.PathEscape(instanceName)+"/snapshots/"+url.PathEscape(name), instance, ETag, true)
+	op, _, err := r.queryOperation("PUT", path+"/"+url.PathEscape(instanceName)+"/snapshots/"+url.PathEscape(name), instance, ETag, true)
 	if err != nil {
 		return nil, err
 	}
@@ -2335,7 +2329,7 @@ func (r *ProtocolLXD) GetInstanceState(name string) (*api.InstanceState, string,
 	state := api.InstanceState{}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct(http.MethodGet, uri, nil, "", &state)
+	etag, err := r.queryStruct("GET", uri, nil, "", &state)
 	if err != nil {
 		return nil, "", err
 	}
@@ -2351,7 +2345,7 @@ func (r *ProtocolLXD) UpdateInstanceState(name string, state api.InstanceStatePu
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPut, path+"/"+url.PathEscape(name)+"/state", state, ETag, true)
+	op, _, err := r.queryOperation("PUT", path+"/"+url.PathEscape(name)+"/state", state, ETag, true)
 	if err != nil {
 		return nil, err
 	}
@@ -2369,7 +2363,7 @@ func (r *ProtocolLXD) GetInstanceLogfiles(name string) ([]string, error) {
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := path + "/" + url.PathEscape(name) + "/logs"
-	_, err = r.queryStruct(http.MethodGet, baseURL, nil, "", &urls)
+	_, err = r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -2395,7 +2389,7 @@ func (r *ProtocolLXD) GetInstanceLogfile(name string, filename string) (io.ReadC
 		return nil, err
 	}
 
-	req, err := http.NewRequest(http.MethodGet, url, nil)
+	req, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -2425,7 +2419,7 @@ func (r *ProtocolLXD) DeleteInstanceLogfile(name string, filename string) error
 	}
 
 	// Send the request
-	_, _, err = r.query(http.MethodDelete, path+"/"+url.PathEscape(name)+"/logs/"+url.PathEscape(filename), nil, "")
+	_, _, err = r.query("DELETE", path+"/"+url.PathEscape(name)+"/logs/"+url.PathEscape(filename), nil, "")
 	if err != nil {
 		return err
 	}
@@ -2455,7 +2449,7 @@ func (r *ProtocolLXD) getInstanceExecOutputLogFile(name string, filename string)
 		return nil, err
 	}
 
-	req, err := http.NewRequest(http.MethodGet, url, nil)
+	req, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -2490,7 +2484,7 @@ func (r *ProtocolLXD) deleteInstanceExecOutputLogFile(instanceName string, filen
 	}
 
 	// Send the request
-	_, _, err = r.query(http.MethodDelete, path+"/"+url.PathEscape(instanceName)+"/logs/exec-output/"+url.PathEscape(filename), nil, "")
+	_, _, err = r.query("DELETE", path+"/"+url.PathEscape(instanceName)+"/logs/exec-output/"+url.PathEscape(filename), nil, "")
 	if err != nil {
 		return err
 	}
@@ -2513,7 +2507,7 @@ func (r *ProtocolLXD) GetInstanceMetadata(name string) (*api.ImageMetadata, stri
 	metadata := api.ImageMetadata{}
 
 	url := path + "/" + url.PathEscape(name) + "/metadata"
-	etag, err := r.queryStruct(http.MethodGet, url, nil, "", &metadata)
+	etag, err := r.queryStruct("GET", url, nil, "", &metadata)
 	if err != nil {
 		return nil, "", err
 	}
@@ -2534,7 +2528,7 @@ func (r *ProtocolLXD) UpdateInstanceMetadata(name string, metadata api.ImageMeta
 	}
 
 	url := path + "/" + url.PathEscape(name) + "/metadata"
-	_, _, err = r.query(http.MethodPut, url, metadata, ETag)
+	_, _, err = r.query("PUT", url, metadata, ETag)
 	if err != nil {
 		return err
 	}
@@ -2557,7 +2551,7 @@ func (r *ProtocolLXD) GetInstanceTemplateFiles(instanceName string) ([]string, e
 	templates := []string{}
 
 	url := path + "/" + url.PathEscape(instanceName) + "/metadata/templates"
-	_, err = r.queryStruct(http.MethodGet, url, nil, "", &templates)
+	_, err = r.queryStruct("GET", url, nil, "", &templates)
 	if err != nil {
 		return nil, err
 	}
@@ -2584,7 +2578,7 @@ func (r *ProtocolLXD) GetInstanceTemplateFile(instanceName string, templateName
 		return nil, err
 	}
 
-	req, err := http.NewRequest(http.MethodGet, url, nil)
+	req, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -2625,7 +2619,7 @@ func (r *ProtocolLXD) CreateInstanceTemplateFile(instanceName string, templateNa
 		return err
 	}
 
-	req, err := http.NewRequest(http.MethodPost, url, content)
+	req, err := http.NewRequest("POST", url, content)
 	if err != nil {
 		return err
 	}
@@ -2656,7 +2650,7 @@ func (r *ProtocolLXD) DeleteInstanceTemplateFile(name string, templateName strin
 		return err
 	}
 
-	_, _, err = r.query(http.MethodDelete, path+"/"+url.PathEscape(name)+"/metadata/templates?path="+url.QueryEscape(templateName), nil, "")
+	_, _, err = r.query("DELETE", path+"/"+url.PathEscape(name)+"/metadata/templates?path="+url.QueryEscape(templateName), nil, "")
 	return err
 }
 
@@ -2685,7 +2679,7 @@ func (r *ProtocolLXD) ConsoleInstance(instanceName string, console api.InstanceC
 
 	// Send the request
 	useEventListener := r.CheckExtension("operation_wait") != nil
-	op, _, err := r.queryOperation(http.MethodPost, path+"/"+url.PathEscape(instanceName)+"/console", console, "", useEventListener)
+	op, _, err := r.queryOperation("POST", path+"/"+url.PathEscape(instanceName)+"/console", console, "", useEventListener)
 	if err != nil {
 		return nil, err
 	}
@@ -2693,11 +2687,11 @@ func (r *ProtocolLXD) ConsoleInstance(instanceName string, console api.InstanceC
 	opAPI := op.Get()
 
 	if args == nil || args.Terminal == nil {
-		return nil, errors.New("A terminal must be set")
+		return nil, fmt.Errorf("A terminal must be set")
 	}
 
 	if args.Control == nil {
-		return nil, errors.New("A control channel must be set")
+		return nil, fmt.Errorf("A control channel must be set")
 	}
 
 	// Parse the fds
@@ -2718,20 +2712,17 @@ func (r *ProtocolLXD) ConsoleInstance(instanceName string, console api.InstanceC
 		}
 	}
 
+	var controlConn *websocket.Conn
 	// Call the control handler with a connection to the control socket
 	if fds[api.SecretNameControl] == "" {
-		return nil, errors.New("Did not receive a file descriptor for the control channel")
+		return nil, fmt.Errorf("Did not receive a file descriptor for the control channel")
 	}
 
-	controlConn, err := r.GetOperationWebsocket(opAPI.ID, fds[api.SecretNameControl])
+	controlConn, err = r.GetOperationWebsocket(opAPI.ID, fds[api.SecretNameControl])
 	if err != nil {
 		return nil, err
 	}
 
-	go func() {
-		_, _, _ = controlConn.ReadMessage() // Consume pings from server.
-	}()
-
 	go args.Control(controlConn)
 
 	// Connect to the websocket
@@ -2787,7 +2778,7 @@ func (r *ProtocolLXD) ConsoleInstanceDynamic(instanceName string, console api.In
 	}
 
 	// Send the request.
-	op, _, err := r.queryOperation(http.MethodPost, path+"/"+url.PathEscape(instanceName)+"/console", console, "", true)
+	op, _, err := r.queryOperation("POST", path+"/"+url.PathEscape(instanceName)+"/console", console, "", true)
 	if err != nil {
 		return nil, nil, err
 	}
@@ -2795,11 +2786,11 @@ func (r *ProtocolLXD) ConsoleInstanceDynamic(instanceName string, console api.In
 	opAPI := op.Get()
 
 	if args == nil {
-		return nil, nil, errors.New("No arguments provided")
+		return nil, nil, fmt.Errorf("No arguments provided")
 	}
 
 	if args.Control == nil {
-		return nil, nil, errors.New("A control channel must be set")
+		return nil, nil, fmt.Errorf("A control channel must be set")
 	}
 
 	// Parse the fds.
@@ -2822,7 +2813,7 @@ func (r *ProtocolLXD) ConsoleInstanceDynamic(instanceName string, console api.In
 
 	// Call the control handler with a connection to the control socket.
 	if fds[api.SecretNameControl] == "" {
-		return nil, nil, errors.New("Did not receive a file descriptor for the control channel")
+		return nil, nil, fmt.Errorf("Did not receive a file descriptor for the control channel")
 	}
 
 	controlConn, err := r.GetOperationWebsocket(opAPI.ID, fds[api.SecretNameControl])
@@ -2830,10 +2821,6 @@ func (r *ProtocolLXD) ConsoleInstanceDynamic(instanceName string, console api.In
 		return nil, nil, err
 	}
 
-	go func() {
-		_, _, _ = controlConn.ReadMessage() // Consume pings from server.
-	}()
-
 	go args.Control(controlConn)
 
 	// Handle main disconnect.
@@ -2885,7 +2872,7 @@ func (r *ProtocolLXD) GetInstanceConsoleLog(instanceName string, args *InstanceC
 		return nil, err
 	}
 
-	req, err := http.NewRequest(http.MethodGet, url, nil)
+	req, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -2920,7 +2907,7 @@ func (r *ProtocolLXD) DeleteInstanceConsoleLog(instanceName string, args *Instan
 	}
 
 	// Send the request
-	_, _, err = r.query(http.MethodDelete, path+"/"+url.PathEscape(instanceName)+"/console", nil, "")
+	_, _, err = r.query("DELETE", path+"/"+url.PathEscape(instanceName)+"/console", nil, "")
 	if err != nil {
 		return err
 	}
@@ -2943,7 +2930,7 @@ func (r *ProtocolLXD) GetInstanceBackupNames(instanceName string) ([]string, err
 	// Fetch the raw URL values.
 	urls := []string{}
 	baseURL := path + "/" + url.PathEscape(instanceName) + "/backups"
-	_, err = r.queryStruct(http.MethodGet, baseURL, nil, "", &urls)
+	_, err = r.queryStruct("GET", baseURL, nil, "", &urls)
 	if err != nil {
 		return nil, err
 	}
@@ -2967,7 +2954,7 @@ func (r *ProtocolLXD) GetInstanceBackups(instanceName string) ([]api.InstanceBac
 	// Fetch the raw value
 	backups := []api.InstanceBackup{}
 
-	_, err = r.queryStruct(http.MethodGet, path+"/"+url.PathEscape(instanceName)+"/backups?recursion=1", nil, "", &backups)
+	_, err = r.queryStruct("GET", path+"/"+url.PathEscape(instanceName)+"/backups?recursion=1", nil, "", &backups)
 	if err != nil {
 		return nil, err
 	}
@@ -2989,7 +2976,7 @@ func (r *ProtocolLXD) GetInstanceBackup(instanceName string, name string) (*api.
 
 	// Fetch the raw value
 	backup := api.InstanceBackup{}
-	etag, err := r.queryStruct(http.MethodGet, path+"/"+url.PathEscape(instanceName)+"/backups/"+url.PathEscape(name), nil, "", &backup)
+	etag, err := r.queryStruct("GET", path+"/"+url.PathEscape(instanceName)+"/backups/"+url.PathEscape(name), nil, "", &backup)
 	if err != nil {
 		return nil, "", err
 	}
@@ -3010,7 +2997,7 @@ func (r *ProtocolLXD) CreateInstanceBackup(instanceName string, backup api.Insta
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, path+"/"+url.PathEscape(instanceName)+"/backups", backup, "", true)
+	op, _, err := r.queryOperation("POST", path+"/"+url.PathEscape(instanceName)+"/backups", backup, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -3031,7 +3018,7 @@ func (r *ProtocolLXD) RenameInstanceBackup(instanceName string, name string, bac
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodPost, path+"/"+url.PathEscape(instanceName)+"/backups/"+url.PathEscape(name), backup, "", true)
+	op, _, err := r.queryOperation("POST", path+"/"+url.PathEscape(instanceName)+"/backups/"+url.PathEscape(name), backup, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -3052,7 +3039,7 @@ func (r *ProtocolLXD) DeleteInstanceBackup(instanceName string, name string) (Op
 	}
 
 	// Send the request
-	op, _, err := r.queryOperation(http.MethodDelete, path+"/"+url.PathEscape(instanceName)+"/backups/"+url.PathEscape(name), nil, "", true)
+	op, _, err := r.queryOperation("DELETE", path+"/"+url.PathEscape(instanceName)+"/backups/"+url.PathEscape(name), nil, "", true)
 	if err != nil {
 		return nil, err
 	}
@@ -3079,7 +3066,7 @@ func (r *ProtocolLXD) GetInstanceBackupFile(instanceName string, name string, re
 	}
 
 	// Prepare the download request
-	request, err := http.NewRequest(http.MethodGet, uri, nil)
+	request, err := http.NewRequest("GET", uri, nil)
 	if err != nil {
 		return nil, err
 	}
@@ -3139,7 +3126,7 @@ func (r *ProtocolLXD) proxyMigration(targetOp *operation, targetSecrets map[stri
 	}
 
 	if targetSecrets[api.SecretNameControl] == "" {
-		return errors.New("Migration target didn't setup the required \"control\" socket")
+		return fmt.Errorf("Migration target didn't setup the required \"control\" socket")
 	}
 
 	// Struct used to hold everything together
