@@ -4,7 +4,8 @@ import (
 	"fmt"
 	"io"
 	"net/http"
-	"slices"
+
+	"github.com/gorilla/websocket"
 
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
@@ -17,7 +18,7 @@ func (r *ProtocolLXD) GetServer() (*api.Server, string, error) {
 	server := api.Server{}
 
 	// Fetch the raw value
-	etag, err := r.queryStruct(http.MethodGet, "", nil, "", &server)
+	etag, err := r.queryStruct("GET", "", nil, "", &server)
 	if err != nil {
 		return nil, "", err
 	}
@@ -45,7 +46,7 @@ func (r *ProtocolLXD) GetServer() (*api.Server, string, error) {
 // UpdateServer updates the server status to match the provided Server struct.
 func (r *ProtocolLXD) UpdateServer(server api.ServerPut, ETag string) error {
 	// Send the request
-	_, _, err := r.query(http.MethodPut, "", server, ETag)
+	_, _, err := r.query("PUT", "", server, ETag)
 	if err != nil {
 		return err
 	}
@@ -63,7 +64,7 @@ func (r *ProtocolLXD) HasExtension(extension string) bool {
 		return true
 	}
 
-	return slices.Contains(r.server.APIExtensions, extension)
+	return shared.ValueInSlice(extension, r.server.APIExtensions)
 }
 
 // CheckExtension checks if the server has the specified extension.
@@ -90,7 +91,7 @@ func (r *ProtocolLXD) GetServerResources() (*api.Resources, error) {
 	resources := api.Resources{}
 
 	// Fetch the raw value
-	_, err = r.queryStruct(http.MethodGet, "/resources", nil, "", &resources)
+	_, err = r.queryStruct("GET", "/resources", nil, "", &resources)
 	if err != nil {
 		return nil, err
 	}
@@ -113,7 +114,8 @@ func (r *ProtocolLXD) UseProject(name string) InstanceServer {
 		requireAuthenticated: r.requireAuthenticated,
 		clusterTarget:        r.clusterTarget,
 		project:              name,
-		eventListenerManager: r.eventListenerManager,
+		eventConns:           make(map[string]*websocket.Conn),  // New project specific listener conns.
+		eventListeners:       make(map[string][]*EventListener), // New project specific listeners.
 		oidcClient:           r.oidcClient,
 	}
 }
@@ -134,7 +136,8 @@ func (r *ProtocolLXD) UseTarget(name string) InstanceServer {
 		httpUserAgent:        r.httpUserAgent,
 		requireAuthenticated: r.requireAuthenticated,
 		project:              r.project,
-		eventListenerManager: r.eventListenerManager,
+		eventConns:           make(map[string]*websocket.Conn),  // New target specific listener conns.
+		eventListeners:       make(map[string][]*EventListener), // New target specific listeners.
 		oidcClient:           r.oidcClient,
 		clusterTarget:        name,
 	}
@@ -159,7 +162,7 @@ func (r *ProtocolLXD) GetMetrics() (string, error) {
 		return "", err
 	}
 
-	req, err := http.NewRequest(http.MethodGet, requestURL, nil)
+	req, err := http.NewRequest("GET", requestURL, nil)
 	if err != nil {
 		return "", err
 	}
