@@ -3,7 +3,6 @@ package lxd
 import (
 	"context"
 	"crypto/tls"
-	"errors"
 	"fmt"
 	"net"
 	"net/http"
@@ -19,31 +18,15 @@ import (
 
 // tlsHTTPClient creates an HTTP client with a specified Transport Layer Security (TLS) configuration.
 // It takes in parameters for client certificates, keys, Certificate Authority, server certificates,
-// a boolean for skipping verification, a boolean for including only legacy curves in ClientHello, a
-// proxy function, and a transport wrapper function.
+// a boolean for skipping verification, a proxy function, and a transport wrapper function.
 // It returns the HTTP client with the provided configurations and handles any errors that might occur during the setup process.
-func tlsHTTPClient(client *http.Client, tlsClientCert string, tlsClientKey string, tlsCA string, tlsServerCert string, insecureSkipVerify bool, legacyCurvesOnly bool, proxy func(req *http.Request) (*url.URL, error), transportWrapper func(t *http.Transport) HTTPTransporter) (*http.Client, error) {
+func tlsHTTPClient(client *http.Client, tlsClientCert string, tlsClientKey string, tlsCA string, tlsServerCert string, insecureSkipVerify bool, proxy func(req *http.Request) (*url.URL, error), transportWrapper func(t *http.Transport) HTTPTransporter) (*http.Client, error) {
 	// Get the TLS configuration
 	tlsConfig, err := shared.GetTLSConfigMem(tlsClientCert, tlsClientKey, tlsCA, tlsServerCert, insecureSkipVerify)
 	if err != nil {
 		return nil, err
 	}
 
-	// If legacyCurvesOnly is true, don't include the post-quantum curve
-	// (`X25519MLKEM768` or `X25519Kyber768Draft00`) in the list of supported
-	// curves. Those extra curves causes the ClientHello message to be too
-	// large to fit in a single packet, causing some broken middleboxes to reset
-	// the connection because they failed to reassemble the TCP packets.
-	if legacyCurvesOnly {
-		// Matches the default list of curves in Go 1.23 with `GODEBUG=tlskyber=0,tlsmlkem=0`
-		tlsConfig.CurvePreferences = []tls.CurveID{
-			tls.X25519,
-			tls.CurveP256,
-			tls.CurveP384,
-			tls.CurveP521,
-		}
-	}
-
 	// Define the http transport
 	transport := &http.Transport{
 		TLSClientConfig:       tlsConfig,
@@ -135,7 +118,7 @@ func tlsHTTPClient(client *http.Client, tlsClientCert string, tlsClientKey strin
 // Any errors encountered during the setup process are also handled by the function.
 func unixHTTPClient(args *ConnectionArgs, path string, transportWrapper func(t *http.Transport) HTTPTransporter) (*http.Client, error) {
 	// Setup a Unix socket dialer
-	unixDial := func(_ context.Context, _ string, _ string) (net.Conn, error) {
+	unixDial := func(_ context.Context, network, addr string) (net.Conn, error) {
 		raddr, err := net.ResolveUnixAddr("unix", path)
 		if err != nil {
 			return nil, err
@@ -295,7 +278,7 @@ func openBrowser(url string) error {
 	case "darwin":
 		err = exec.Command("open", url).Start()
 	default:
-		err = errors.New("unsupported platform")
+		err = fmt.Errorf("unsupported platform")
 	}
 
 	if err != nil {
