@@ -5,7 +5,6 @@ import subprocess
 import yaml
 from git import Repo
 import filecmp
-from redirects import redirects
 
 # Custom configuration for the Sphinx documentation builder.
 # All configuration specific to your project should be done in this file.
@@ -34,7 +33,6 @@ author = 'LXD contributors'
 # To not display any title, set this option to an empty string.
 html_title = ''
 
-# Set global version variable used in objects.inv to numeric version defined in flex.go
 with open("../shared/version/flex.go") as fd:
     version = fd.readlines()[3].split()[-1].strip("\"")
 
@@ -60,11 +58,11 @@ copyright = '2014-%s %s' % (datetime.date.today().year, author)
 # don't know yet)
 # NOTE: If no ogp_* variable is defined (e.g. if you remove this section) the
 # sphinxext.opengraph extension will be disabled.
-ogp_site_url = 'https://documentation.ubuntu.com/lxd/stable-5.21/'
+ogp_site_url = 'https://documentation.ubuntu.com/lxd/en/latest/'
 # The documentation website name (usually the same as the product name)
 ogp_site_name = 'LXD documentation'
 # The URL of an image or logo that is used in the preview
-ogp_image = 'https://documentation.ubuntu.com/lxd/stable-5.21/_static/tag.png'
+ogp_image = 'https://documentation.ubuntu.com/lxd/en/latest/_static/tag.png'
 
 # Update with the local path to the favicon for your product
 # (default is the circle of friends)
@@ -134,22 +132,6 @@ html_context = {
 # slug (for example, "lxd") here.
 slug = "lxd"
 
-#######################
-# Sitemap configuration: https://sphinx-sitemap.readthedocs.io/
-#######################
-
-# Base URL of RTD hosted project
-
-html_baseurl = 'https://documentation.ubuntu.com/lxd/'
-
-# Configures URL scheme for sphinx-sitemap to generate correct URLs
-# based on the version if built in RTD
-if 'READTHEDOCS_VERSION' in os.environ:
-    rtd_version = os.environ["READTHEDOCS_VERSION"]
-    sitemap_url_scheme = f'{rtd_version}/{{link}}'
-else:
-    sitemap_url_scheme = '{link}'
-
 ############################################################
 ### Redirects
 ############################################################
@@ -160,8 +142,12 @@ else:
 # (see https://docs.readthedocs.io/en/stable/guides/redirects.html).
 # NOTE: If this variable is not defined, set to None, or the dictionary is empty,
 # the sphinx_reredirects extension will be disabled.
-# redirects = {}
-# minae: Redirects is imported from redirects.py
+redirects = {
+    'howto/instances_snapshots/index': '../instances_backup/',
+    'reference/network_external/index': '../networks/',
+    'explanation/containers_and_vms/index': '../instances/',
+    'explanation/clustering/index': '../clusters/'
+}
 
 ############################################################
 ### Link checker
@@ -173,29 +159,21 @@ linkcheck_ignore = [
     'https://127.0.0.1:8443/1.0',
     'https://web.libera.chat/#lxd',
     'http://localhost:8000',
-    'http://localhost:8080',
-    'http://localhost:8080/admin',
-    r'/lxd/stable-5.21/api/.*',
+    r'/lxd/en/latest/api/.*',
     r'/api/.*',
     # Those links may fail from time to time
     'https://www.dell.com/',
     'https://www.dell.com/en-us/shop/powerflex/sf/powerflex',
     'https://www.gnu.org/licenses/agpl-3.0.en.html',
-    r"https://ceph\.io(/.*)?",
-    # 403 from GH runners
-    'https://www.schlachter.tech/solutions/pongo2-template-engine/',
-    # Cloudflare protection on SourceForge domains might block linkcheck
-    r"https://.*\.sourceforge\.net/.*",
-]
+    ]
 
 # Pages on which to ignore anchors
 # (This list will be appended to linkcheck_anchors_ignore_for_url)
 
 custom_linkcheck_anchors_ignore_for_url = [
     r'https://snapcraft\.io/docs/.*',
-    'https://docs.docker.com/network/packet-filtering-firewalls/',
-    'https://maas.io/docs/how-to-manage-machines'
-]
+    'https://docs.docker.com/network/packet-filtering-firewalls/'
+    ]
 
 linkcheck_exclude_documents = [r'.*/manpages/.*']
 
@@ -234,7 +212,6 @@ custom_extensions = [
     'sphinx_remove_toctrees',
     'canonical.filtered-toc',
     'sphinxcontrib.cairosvgconverter',
-    'sphinx_sitemap',
 ]
 
 # Add custom required Python modules that must be added to the
@@ -248,8 +225,7 @@ custom_required_modules = [
     'gitpython',
     'pyyaml',
     'sphinx-remove-toctrees',
-    'sphinxcontrib-svg2pdfconverter[CairoSVG]',
-    'sphinx-sitemap',
+    'sphinxcontrib-svg2pdfconverter[CairoSVG]'
 ]
 
 # Add files or directories that should be excluded from processing.
@@ -260,14 +236,10 @@ custom_excludes = [
 ]
 
 # Add CSS files (located in .sphinx/_static/)
-custom_html_css_files = [
-    'cookie-banner.css'
-]
+custom_html_css_files = []
 
 # Add JavaScript files (located in .sphinx/_static/)
-custom_html_js_files = [
-    'js/bundle.js',
-]
+custom_html_js_files = []
 
 ## The following settings override the default configuration.
 
@@ -307,7 +279,7 @@ if os.path.exists('./related_topics.yaml'):
 if ('LOCAL_SPHINX_BUILD' in os.environ) and (os.environ['LOCAL_SPHINX_BUILD'] == 'True'):
     swagger_url_scheme = '/api/#{{path}}'
 else:
-    swagger_url_scheme = '/lxd/latest/api/#{{path}}'
+    swagger_url_scheme = '/lxd/en/latest/api/#{{path}}'
 
 myst_url_schemes = {
     'http': None,
@@ -430,8 +402,9 @@ if ('TOPICAL' in os.environ) and (os.environ['TOPICAL'] == 'True'):
     custom_tags.append('topical')
     toc_filter_exclude = ['diataxis']
 else:
-    custom_excludes.extend(['security.md','external_resources.md','reference/network_external.md'])
+    custom_excludes.extend(['security.md','external_resources.md','reference/network_external.md','migration.md'])
     redirects['security/index'] = '../explanation/security/'
+    redirects['migration/index'] = '../howto/import_machines_to_instances/'
     custom_tags.append('diataxis')
     toc_filter_exclude = ['topical']
 
