@@ -4,12 +4,12 @@ import (
 	"errors"
 	"fmt"
 	"os"
-	"slices"
 	"strings"
 
 	"github.com/spf13/cobra"
 
 	"github.com/canonical/lxd/lxc/config"
+	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	cli "github.com/canonical/lxd/shared/cmd"
 	"github.com/canonical/lxd/shared/i18n"
@@ -137,19 +137,18 @@ func (c *cmdAction) Command(action string) *cobra.Command {
 
 	cmd.Flags().BoolVar(&c.flagAll, "all", false, i18n.G("Run against all instances"))
 
-	switch action {
-	case "stop":
+	if action == "stop" {
 		cmd.Flags().BoolVar(&c.flagStateful, "stateful", false, i18n.G("Store the instance state"))
-	case "start":
+	} else if action == "start" {
 		cmd.Flags().BoolVar(&c.flagStateless, "stateless", false, i18n.G("Ignore the instance state"))
 	}
 
-	if slices.Contains([]string{"start", "restart", "stop"}, action) {
+	if shared.ValueInSlice(action, []string{"start", "restart", "stop"}) {
 		cmd.Flags().StringVar(&c.flagConsole, "console", "", i18n.G("Immediately attach to the console")+"``")
 		cmd.Flags().Lookup("console").NoOptDefVal = "console"
 	}
 
-	if slices.Contains([]string{"restart", "stop"}, action) {
+	if shared.ValueInSlice(action, []string{"restart", "stop"}) {
 		cmd.Flags().BoolVarP(&c.flagForce, "force", "f", false, i18n.G("Force the instance to stop"))
 		cmd.Flags().IntVar(&c.flagTimeout, "timeout", -1, i18n.G("Time to wait for the instance to shutdown cleanly")+"``")
 	}
@@ -177,7 +176,10 @@ func (c *cmdAction) doActionAll(action string, resource remoteResource) error {
 	}
 
 	// Only store state if asked to.
-	state := action == "stop" && c.flagStateful
+	state := false
+	if action == "stop" && c.flagStateful {
+		state = true
+	}
 
 	req := api.InstancesPut{
 		State: &api.InstanceStatePut{
@@ -406,7 +408,7 @@ func (c *cmdAction) run(cmd *cobra.Command, args []string) error {
 
 		success = false
 		msg := fmt.Sprintf(i18n.G("error: %v"), result.err)
-		for line := range strings.SplitSeq(msg, "\n") {
+		for _, line := range strings.Split(msg, "\n") {
 			fmt.Fprintf(os.Stderr, "%s: %s\n", result.name, line)
 		}
 	}
