@@ -5,10 +5,7 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"maps"
-	"net/http"
 	"os"
-	"slices"
 	"sort"
 	"strings"
 
@@ -131,7 +128,7 @@ func (c *cmdClusterList) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, ":", false, instanceServerRemoteCompletionFilters(*c.global.conf)...)
+			return c.global.cmpRemotes(toComplete, false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -222,7 +219,7 @@ func (c *cmdClusterShow) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -279,7 +276,7 @@ func (c *cmdClusterInfo) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -338,7 +335,7 @@ func (c *cmdClusterGet) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -374,7 +371,7 @@ func (c *cmdClusterGet) run(cmd *cobra.Command, args []string) error {
 
 	if c.flagIsProperty {
 		w := member.Writable()
-		res, err := getFieldByJSONTag(&w, args[1])
+		res, err := getFieldByJsonTag(&w, args[1])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the cluster member %q: %v"), args[1], resource.name, err)
 		}
@@ -411,7 +408,7 @@ func (c *cmdClusterSet) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -455,7 +452,7 @@ func (c *cmdClusterSet) run(cmd *cobra.Command, args []string) error {
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJSONTag(&writable, k)
+				err := unsetFieldByJsonTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -467,7 +464,9 @@ func (c *cmdClusterSet) run(cmd *cobra.Command, args []string) error {
 			}
 		}
 	} else {
-		maps.Copy(writable.Config, keys)
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
 	return resource.server.UpdateClusterMember(resource.name, writable, "")
@@ -493,7 +492,7 @@ func (c *cmdClusterUnset) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -537,7 +536,7 @@ func (c *cmdClusterRename) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -597,7 +596,7 @@ func (c *cmdClusterRemove) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -626,7 +625,7 @@ Are you really sure you want to force removing %s? (yes/no): `), name)
 	input, _ := reader.ReadString('\n')
 	input = strings.TrimSuffix(input, "\n")
 
-	if !slices.Contains([]string{i18n.G("yes")}, strings.ToLower(input)) {
+	if !shared.ValueInSlice(strings.ToLower(input), []string{i18n.G("yes")}) {
 		return errors.New(i18n.G("User aborted delete operation"))
 	}
 
@@ -694,7 +693,7 @@ func (c *cmdClusterEnable) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
+			return c.global.cmpRemotes(toComplete, false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -783,7 +782,7 @@ func (c *cmdClusterEdit) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -903,7 +902,7 @@ func (c *cmdClusterAdd) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -987,7 +986,7 @@ func (c *cmdClusterListTokens) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
+			return c.global.cmpRemotes(toComplete, false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -1096,7 +1095,7 @@ func (c *cmdClusterRevokeToken) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -1185,7 +1184,7 @@ func (c *cmdClusterUpdateCertificate) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1302,21 +1301,14 @@ func (c *cmdClusterEvacuate) command() *cobra.Command {
 	cmd.Aliases = []string{"evac"}
 	cmd.Use = usage("evacuate", i18n.G("[<remote>:]<member>"))
 	cmd.Short = i18n.G("Evacuate cluster member")
-	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
-		`Evacuate cluster member
-
-Evacuation actions:
- - stop: stop all instances on the member
- - migrate: migrate all instances on the member to other members
- - live-migrate: live migrate all instances on the member to other members
-`))
+	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(`Evacuate cluster member`))
 
 	cmd.Flags().BoolVar(&c.action.flagForce, "force", false, i18n.G(`Force evacuation without user confirmation`)+"``")
-	cmd.Flags().StringVar(&c.action.flagAction, "action", "", i18n.G(`Force a particular instance evacuation action. One of stop, migrate or live-migrate`)+"``")
+	cmd.Flags().StringVar(&c.action.flagAction, "action", "", i18n.G(`Force a particular evacuation action`)+"``")
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -1342,11 +1334,10 @@ func (c *cmdClusterRestore) command() *cobra.Command {
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(`Restore cluster member`))
 
 	cmd.Flags().BoolVar(&c.action.flagForce, "force", false, i18n.G(`Force restoration without user confirmation`)+"``")
-	cmd.Flags().StringVar(&c.action.flagAction, "action", "", i18n.G(`Force a particular instance restore action. Use "skip" to restore only the cluster member status without starting local instances or migrating back evacuated instances`)+"``")
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("cluster_member", toComplete)
+			return c.global.cmpClusterMembers(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -1399,12 +1390,6 @@ func (c *cmdClusterEvacuateAction) run(cmd *cobra.Command, args []string) error
 
 	op, err := resource.server.UpdateClusterMemberState(resource.name, state)
 	if err != nil {
-		var statusErr api.StatusError
-
-		if errors.As(err, &statusErr) && statusErr.Status() == http.StatusServiceUnavailable {
-			return errors.New("Offline cluster members cannot be evacuated")
-		}
-
 		return fmt.Errorf("Failed to update cluster member state: %w", err)
 	}
 
