@@ -82,51 +82,10 @@ func (c *cmdConfig) command() *cobra.Command {
 
 	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
 	cmd.Args = cobra.NoArgs
-	cmd.Run = func(cmd *cobra.Command, _ []string) { _ = cmd.Usage() }
+	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
 	return cmd
 }
 
-// configSetValidArgsFunc returns completions for server and instance configuration keys (and remotes if no remote is
-// specified). If the command is targeting an instance, only configuration keys valid for that instance type are offered.
-// This function can also be used for the `config get` command, since it is valid to get any key (even if unset). If the
-// command name is "set", configuration key completions are suffixed with an `=`.
-func (c *cmdConfig) configSetValidArgsFunc(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-	var suffix string
-	if cmd.Name() == "set" {
-		suffix = "="
-	}
-
-	if len(args) == 0 {
-		remote, partial, err := c.global.conf.ParseRemote(toComplete)
-		if err != nil {
-			return handleCompletionError(err)
-		}
-
-		// Default remote: Return all config keys, remotes, and instances.
-		if partial == toComplete {
-			serverConfigKeys, _ := c.global.cmpServerAllKeys(remote, suffix, toComplete)
-			instancesAndRemotes, _ := c.global.cmpTopLevelResource("instance", toComplete)
-			return append(serverConfigKeys, instancesAndRemotes...), cobra.ShellCompDirectiveNoFileComp | cobra.ShellCompDirectiveNoSpace
-		}
-
-		// Non-default remote: Return instances and remotes
-		return c.global.cmpTopLevelResource("instance", toComplete)
-	}
-
-	remote, instanceName, err := c.global.conf.ParseRemote(args[0])
-	if err != nil {
-		return handleCompletionError(err)
-	}
-
-	if instanceName == "" {
-		return c.global.cmpServerAllKeys(remote, suffix, toComplete)
-	}
-
-	instanceType := c.global.getInstanceType(remote, instanceName)
-
-	return c.global.cmpInstanceKeysByType(instanceType, suffix, toComplete)
-}
-
 // Edit.
 type cmdConfigEdit struct {
 	global *cmdGlobal
@@ -147,9 +106,9 @@ func (c *cmdConfigEdit) command() *cobra.Command {
 	cmd.Flags().StringVar(&c.config.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("instance", toComplete)
+			return c.global.cmpInstances(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -440,13 +399,19 @@ func (c *cmdConfigGet) command() *cobra.Command {
 	cmd.RunE = c.run
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-		// Only one key is allowed.
-		if len(args) > 1 {
-			return nil, cobra.ShellCompDirectiveNoFileComp
+		if len(args) == 0 {
+			if strings.Contains(toComplete, ".") {
+				return c.global.cmpServerAllKeys(toComplete)
+			}
+
+			return c.global.cmpInstances(toComplete)
 		}
 
-		// The caller can get any key, even if unset. So call completions for 'config set'.
-		return c.config.configSetValidArgsFunc(cmd, args, toComplete)
+		if len(args) == 1 {
+			return c.global.cmpInstanceKeys(args[0])
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
 
 	return cmd
@@ -489,7 +454,7 @@ func (c *cmdConfigGet) run(cmd *cobra.Command, args []string) error {
 			}
 
 			if c.flagIsProperty {
-				res, err := getFieldByJSONTag(inst, args[len(args)-1])
+				res, err := getFieldByJsonTag(inst, args[len(args)-1])
 				if err != nil {
 					return fmt.Errorf(i18n.G("The property %q does not exist on the instance snapshot %s/%s: %v"), args[len(args)-1], fields[0], fields[1], err)
 				}
@@ -513,7 +478,7 @@ func (c *cmdConfigGet) run(cmd *cobra.Command, args []string) error {
 
 		if c.flagIsProperty {
 			w := resp.Writable()
-			res, err := getFieldByJSONTag(&w, args[len(args)-1])
+			res, err := getFieldByJsonTag(&w, args[len(args)-1])
 			if err != nil {
 				return fmt.Errorf(i18n.G("The property %q does not exist on the instance %q: %v"), args[len(args)-1], resource.name, err)
 			}
@@ -547,12 +512,11 @@ func (c *cmdConfigGet) run(cmd *cobra.Command, args []string) error {
 		}
 
 		value := resp.Config[args[len(args)-1]]
-		switch value {
-		case nil:
+		if value == nil {
 			value = ""
-		case true:
+		} else if value == true { //nolint:revive
 			value = "true"
-		case false:
+		} else if value == false { //nolint:revive
 			value = "false"
 		}
 
@@ -585,16 +549,28 @@ For backward compatibility, a single configuration key may still be set with:
     Will set a CPU limit of "2" for the instance.
 
 lxc config set core.https_address=[::]:8443
-    Will have LXD listen on IPv4 and IPv6 port 8443.
-
-lxc config set core.trust_password=blah
-    Will set the server's trust password to blah.`))
+    Will have LXD listen on IPv4 and IPv6 port 8443.`))
 
 	cmd.Flags().StringVar(&c.config.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as an instance property"))
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = c.config.configSetValidArgsFunc
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			if strings.Contains(toComplete, ".") {
+				return c.global.cmpServerAllKeys(toComplete)
+			}
+
+			return c.global.cmpInstances(toComplete)
+		}
+
+		if len(args) == 1 {
+			return c.global.cmpInstanceKeys(args[0])
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
 	return cmd
 }
 
@@ -675,7 +651,7 @@ func (c *cmdConfigSet) run(cmd *cobra.Command, args []string) error {
 			if c.flagIsProperty {
 				if cmd.Name() == "unset" {
 					for k := range keys {
-						err := unsetFieldByJSONTag(&writable, k)
+						err := unsetFieldByJsonTag(&writable, k)
 						if err != nil {
 							return fmt.Errorf(i18n.G("Error unsetting properties: %v"), err)
 						}
@@ -707,7 +683,7 @@ func (c *cmdConfigSet) run(cmd *cobra.Command, args []string) error {
 		if c.flagIsProperty {
 			if cmd.Name() == "unset" {
 				for k := range keys {
-					err := unsetFieldByJSONTag(&writable, k)
+					err := unsetFieldByJsonTag(&writable, k)
 					if err != nil {
 						return fmt.Errorf(i18n.G("Error unsetting properties: %v"), err)
 					}
@@ -800,12 +776,12 @@ func (c *cmdConfigShow) command() *cobra.Command {
 	cmd.Flags().StringVar(&c.config.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) != 0 {
 			return nil, cobra.ShellCompDirectiveNoFileComp
 		}
 
-		return c.global.cmpTopLevelResource("instance", toComplete)
+		return c.global.cmpInstances(toComplete)
 	}
 
 	return cmd
@@ -932,40 +908,22 @@ func (c *cmdConfigUnset) command() *cobra.Command {
 	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as an instance property"))
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-		if len(args) > 1 {
-			return nil, cobra.ShellCompDirectiveNoFileComp
-		}
-
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			remote, partial, err := c.global.conf.ParseRemote(toComplete)
-			if err != nil {
-				return handleCompletionError(err)
+			if strings.Contains(toComplete, ".") {
+				// Only complete config keys which are currently set.
+				return c.global.cmpServerSetKeys(toComplete)
 			}
 
-			// Default remote: Return set config keys in the default remote, instance remotes, and instances in the default remote.
-			if partial == toComplete {
-				serverConfigKeys, _ := c.global.cmpServerSetKeys(remote, toComplete)
-				instancesAndRemotes, _ := c.global.cmpTopLevelResource("instance", toComplete)
-				return append(serverConfigKeys, instancesAndRemotes...), cobra.ShellCompDirectiveNoFileComp | cobra.ShellCompDirectiveNoSpace
-			}
-
-			// Non-default remote: Return instances and remotes
-			return c.global.cmpTopLevelResource("instance", toComplete)
+			return c.global.cmpInstances(toComplete)
 		}
 
-		remote, instanceName, err := c.global.conf.ParseRemote(args[0])
-		if err != nil {
-			return handleCompletionError(err)
+		if len(args) == 1 {
+			// Only complete config keys which are currently set.
+			return c.global.cmpInstanceSetKeys(args[0])
 		}
 
-		// If we have a remote but no instance, return set config keys in the specified remote.
-		if instanceName == "" {
-			return c.global.cmpServerSetKeys(remote, toComplete)
-		}
-
-		// Return set config keys for the instance.
-		return c.global.cmpInstanceSetKeys(remote, instanceName, toComplete)
+		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
 
 	return cmd
@@ -1021,7 +979,7 @@ func (c *cmdConfigUefi) command() *cobra.Command {
 
 	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
 	cmd.Args = cobra.NoArgs
-	cmd.Run = func(cmd *cobra.Command, _ []string) { _ = cmd.Usage() }
+	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
 	return cmd
 }
 
