@@ -1,14 +1,15 @@
 package main
 
 import (
+	"errors"
 	"fmt"
 	"strings"
 
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
 )
 
 type cmdConfigDevice struct {
@@ -17,7 +18,7 @@ type cmdConfigDevice struct {
 	profile *cmdProfile
 }
 
-func (c *cmdConfigDevice) Command() *cobra.Command {
+func (c *cmdConfigDevice) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = usage("device")
 	cmd.Short = i18n.G("Manage devices")
@@ -26,37 +27,37 @@ func (c *cmdConfigDevice) Command() *cobra.Command {
 
 	// Add
 	configDeviceAddCmd := cmdConfigDeviceAdd{global: c.global, config: c.config, profile: c.profile, configDevice: c}
-	cmd.AddCommand(configDeviceAddCmd.Command())
+	cmd.AddCommand(configDeviceAddCmd.command())
 
 	// Get
 	configDeviceGetCmd := cmdConfigDeviceGet{global: c.global, config: c.config, profile: c.profile, configDevice: c}
-	cmd.AddCommand(configDeviceGetCmd.Command())
+	cmd.AddCommand(configDeviceGetCmd.command())
 
 	// List
 	configDeviceListCmd := cmdConfigDeviceList{global: c.global, config: c.config, profile: c.profile, configDevice: c}
-	cmd.AddCommand(configDeviceListCmd.Command())
+	cmd.AddCommand(configDeviceListCmd.command())
 
 	// Override
 	if c.config != nil {
 		configDeviceOverrideCmd := cmdConfigDeviceOverride{global: c.global, config: c.config, profile: c.profile, configDevice: c}
-		cmd.AddCommand(configDeviceOverrideCmd.Command())
+		cmd.AddCommand(configDeviceOverrideCmd.command())
 	}
 
 	// Remove
 	configDeviceRemoveCmd := cmdConfigDeviceRemove{global: c.global, config: c.config, profile: c.profile, configDevice: c}
-	cmd.AddCommand(configDeviceRemoveCmd.Command())
+	cmd.AddCommand(configDeviceRemoveCmd.command())
 
 	// Set
 	configDeviceSetCmd := cmdConfigDeviceSet{global: c.global, config: c.config, profile: c.profile, configDevice: c}
-	cmd.AddCommand(configDeviceSetCmd.Command())
+	cmd.AddCommand(configDeviceSetCmd.command())
 
 	// Show
 	configDeviceShowCmd := cmdConfigDeviceShow{global: c.global, config: c.config, profile: c.profile, configDevice: c}
-	cmd.AddCommand(configDeviceShowCmd.Command())
+	cmd.AddCommand(configDeviceShowCmd.command())
 
 	// Unset
 	configDeviceUnsetCmd := cmdConfigDeviceUnset{global: c.global, config: c.config, profile: c.profile, configDevice: c, configDeviceSet: &configDeviceSetCmd}
-	cmd.AddCommand(configDeviceUnsetCmd.Command())
+	cmd.AddCommand(configDeviceUnsetCmd.command())
 
 	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
 	cmd.Args = cobra.NoArgs
@@ -72,7 +73,7 @@ type cmdConfigDeviceAdd struct {
 	profile      *cmdProfile
 }
 
-func (c *cmdConfigDeviceAdd) Command() *cobra.Command {
+func (c *cmdConfigDeviceAdd) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Short = i18n.G("Add instance devices")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
@@ -95,12 +96,33 @@ lxc profile device add [<remote>:]profile1 <device-name> disk pool=some-pool sou
     Will mount the some-volume volume on some-pool onto /opt in the instance.`))
 	}
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
+
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			if c.config != nil {
+				return c.global.cmpInstances(toComplete)
+			} else if c.profile != nil {
+				return c.global.cmpProfiles(toComplete, true)
+			}
+		}
+
+		// The second positional argument is used for the device name, so we provide device completions for the third positional argument.
+		if len(args) == 2 {
+			return c.global.cmpInstanceAllDevices(toComplete)
+		}
+
+		if len(args) == 3 {
+			return c.global.cmpInstanceAllDeviceOptions(args[0], args[2])
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
 
 	return cmd
 }
 
-func (c *cmdConfigDeviceAdd) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdConfigDeviceAdd) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 3, -1)
 	if exit {
@@ -116,7 +138,7 @@ func (c *cmdConfigDeviceAdd) Run(cmd *cobra.Command, args []string) error {
 	resource := resources[0]
 
 	if resource.name == "" {
-		return fmt.Errorf(i18n.G("Missing name"))
+		return errors.New(i18n.G("Missing name"))
 	}
 
 	// Add the device
@@ -142,9 +164,13 @@ func (c *cmdConfigDeviceAdd) Run(cmd *cobra.Command, args []string) error {
 			return err
 		}
 
+		if profile.Devices == nil {
+			profile.Devices = make(map[string]map[string]string)
+		}
+
 		_, ok := profile.Devices[devname]
 		if ok {
-			return fmt.Errorf(i18n.G("The device already exists"))
+			return errors.New(i18n.G("The device already exists"))
 		}
 
 		profile.Devices[devname] = device
@@ -161,7 +187,7 @@ func (c *cmdConfigDeviceAdd) Run(cmd *cobra.Command, args []string) error {
 
 		_, ok := inst.Devices[devname]
 		if ok {
-			return fmt.Errorf(i18n.G("The device already exists"))
+			return errors.New(i18n.G("The device already exists"))
 		}
 
 		inst.Devices[devname] = device
@@ -192,7 +218,7 @@ type cmdConfigDeviceGet struct {
 	profile      *cmdProfile
 }
 
-func (c *cmdConfigDeviceGet) Command() *cobra.Command {
+func (c *cmdConfigDeviceGet) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	if c.config != nil {
 		cmd.Use = usage("get", i18n.G("[<remote>:]<instance> <device> <key>"))
@@ -204,12 +230,32 @@ func (c *cmdConfigDeviceGet) Command() *cobra.Command {
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Get values for device configuration keys`))
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
+
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			if c.config != nil {
+				return c.global.cmpInstances(toComplete)
+			} else if c.profile != nil {
+				return c.global.cmpProfiles(toComplete, true)
+			}
+		}
+
+		if len(args) == 1 {
+			if c.config != nil {
+				return c.global.cmpInstanceDeviceNames(args[0])
+			} else if c.profile != nil {
+				return c.global.cmpProfileDeviceNames(args[0])
+			}
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
 
 	return cmd
 }
 
-func (c *cmdConfigDeviceGet) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdConfigDeviceGet) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 3, 3)
 	if exit {
@@ -225,7 +271,7 @@ func (c *cmdConfigDeviceGet) Run(cmd *cobra.Command, args []string) error {
 	resource := resources[0]
 
 	if resource.name == "" {
-		return fmt.Errorf(i18n.G("Missing name"))
+		return errors.New(i18n.G("Missing name"))
 	}
 
 	// Get the config key
@@ -240,7 +286,7 @@ func (c *cmdConfigDeviceGet) Run(cmd *cobra.Command, args []string) error {
 
 		dev, ok := profile.Devices[devname]
 		if !ok {
-			return fmt.Errorf(i18n.G("Device doesn't exist"))
+			return errors.New(i18n.G("Device doesn't exist"))
 		}
 
 		fmt.Println(dev[key])
@@ -254,10 +300,10 @@ func (c *cmdConfigDeviceGet) Run(cmd *cobra.Command, args []string) error {
 		if !ok {
 			_, ok = inst.ExpandedDevices[devname]
 			if !ok {
-				return fmt.Errorf(i18n.G("Device doesn't exist"))
+				return errors.New(i18n.G("Device doesn't exist"))
 			}
 
-			return fmt.Errorf(i18n.G("Device from profile(s) cannot be retrieved for individual instance"))
+			return errors.New(i18n.G("Device from profile(s) cannot be retrieved for individual instance"))
 		}
 
 		fmt.Println(dev[key])
@@ -274,7 +320,7 @@ type cmdConfigDeviceList struct {
 	profile      *cmdProfile
 }
 
-func (c *cmdConfigDeviceList) Command() *cobra.Command {
+func (c *cmdConfigDeviceList) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Aliases = []string{"ls"}
 	cmd.Short = i18n.G("List instance devices")
@@ -286,12 +332,24 @@ func (c *cmdConfigDeviceList) Command() *cobra.Command {
 		cmd.Use = usage("list", i18n.G("[<remote>:]<profile>"))
 	}
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
+
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			if c.config != nil {
+				return c.global.cmpInstances(toComplete)
+			} else if c.profile != nil {
+				return c.global.cmpProfiles(toComplete, true)
+			}
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
 
 	return cmd
 }
 
-func (c *cmdConfigDeviceList) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdConfigDeviceList) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
@@ -307,7 +365,7 @@ func (c *cmdConfigDeviceList) Run(cmd *cobra.Command, args []string) error {
 	resource := resources[0]
 
 	if resource.name == "" {
-		return fmt.Errorf(i18n.G("Missing name"))
+		return errors.New(i18n.G("Missing name"))
 	}
 
 	// List the devices
@@ -345,19 +403,27 @@ type cmdConfigDeviceOverride struct {
 	profile      *cmdProfile
 }
 
-func (c *cmdConfigDeviceOverride) Command() *cobra.Command {
+func (c *cmdConfigDeviceOverride) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = usage("override", i18n.G("[<remote>:]<instance> <device> [key=value...]"))
 	cmd.Short = i18n.G("Copy profile inherited devices and override configuration keys")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Copy profile inherited devices and override configuration keys`))
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
+
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			return c.global.cmpInstances(toComplete)
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
 
 	return cmd
 }
 
-func (c *cmdConfigDeviceOverride) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdConfigDeviceOverride) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, -1)
 	if exit {
@@ -373,7 +439,7 @@ func (c *cmdConfigDeviceOverride) Run(cmd *cobra.Command, args []string) error {
 	resource := resources[0]
 
 	if resource.name == "" {
-		return fmt.Errorf(i18n.G("Missing name"))
+		return errors.New(i18n.G("Missing name"))
 	}
 
 	// Override the device
@@ -385,12 +451,12 @@ func (c *cmdConfigDeviceOverride) Run(cmd *cobra.Command, args []string) error {
 	devname := args[1]
 	_, ok := inst.Devices[devname]
 	if ok {
-		return fmt.Errorf(i18n.G("The device already exists"))
+		return errors.New(i18n.G("The device already exists"))
 	}
 
 	device, ok := inst.ExpandedDevices[devname]
 	if !ok {
-		return fmt.Errorf(i18n.G("The profile device doesn't exist"))
+		return errors.New(i18n.G("The profile device doesn't exist"))
 	}
 
 	if len(args) > 2 {
@@ -433,7 +499,7 @@ type cmdConfigDeviceRemove struct {
 	profile      *cmdProfile
 }
 
-func (c *cmdConfigDeviceRemove) Command() *cobra.Command {
+func (c *cmdConfigDeviceRemove) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	if c.config != nil {
 		cmd.Use = usage("remove", i18n.G("[<remote>:]<instance> <name>..."))
@@ -446,12 +512,30 @@ func (c *cmdConfigDeviceRemove) Command() *cobra.Command {
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Remove instance devices`))
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
+
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			if c.config != nil {
+				return c.global.cmpInstances(toComplete)
+			} else if c.profile != nil {
+				return c.global.cmpProfiles(toComplete, true)
+			}
+		}
+
+		if c.config != nil {
+			return c.global.cmpInstanceDeviceNames(args[0])
+		} else if c.profile != nil {
+			return c.global.cmpProfileDeviceNames(args[0])
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
 
 	return cmd
 }
 
-func (c *cmdConfigDeviceRemove) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdConfigDeviceRemove) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 2, -1)
 	if exit {
@@ -467,7 +551,7 @@ func (c *cmdConfigDeviceRemove) Run(cmd *cobra.Command, args []string) error {
 	resource := resources[0]
 
 	if resource.name == "" {
-		return fmt.Errorf(i18n.G("Missing name"))
+		return errors.New(i18n.G("Missing name"))
 	}
 
 	// Remove the device
@@ -480,7 +564,7 @@ func (c *cmdConfigDeviceRemove) Run(cmd *cobra.Command, args []string) error {
 		for _, devname := range args[1:] {
 			_, ok := profile.Devices[devname]
 			if !ok {
-				return fmt.Errorf(i18n.G("Device doesn't exist"))
+				return errors.New(i18n.G("Device doesn't exist"))
 			}
 
 			delete(profile.Devices, devname)
@@ -501,10 +585,10 @@ func (c *cmdConfigDeviceRemove) Run(cmd *cobra.Command, args []string) error {
 			if !ok {
 				_, ok := inst.ExpandedDevices[devname]
 				if !ok {
-					return fmt.Errorf(i18n.G("Device doesn't exist"))
+					return errors.New(i18n.G("Device doesn't exist"))
 				}
 
-				return fmt.Errorf(i18n.G("Device from profile(s) cannot be removed from individual instance. Override device or modify profile instead"))
+				return errors.New(i18n.G("Device from profile(s) cannot be removed from individual instance. Override device or modify profile instead"))
 			}
 
 			delete(inst.Devices, devname)
@@ -536,7 +620,7 @@ type cmdConfigDeviceSet struct {
 	profile      *cmdProfile
 }
 
-func (c *cmdConfigDeviceSet) Command() *cobra.Command {
+func (c *cmdConfigDeviceSet) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Short = i18n.G("Set device configuration keys")
 	if c.config != nil {
@@ -555,12 +639,32 @@ For backward compatibility, a single configuration key may still be set with:
     lxc profile device set [<remote>:]<profile> <device> <key> <value>`))
 	}
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
+
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			if c.config != nil {
+				return c.global.cmpInstances(toComplete)
+			} else if c.profile != nil {
+				return c.global.cmpProfiles(toComplete, true)
+			}
+		}
+
+		if len(args) == 1 {
+			if c.config != nil {
+				return c.global.cmpInstanceDeviceNames(args[0])
+			} else if c.profile != nil {
+				return c.global.cmpProfileDeviceNames(args[0])
+			}
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
 
 	return cmd
 }
 
-func (c *cmdConfigDeviceSet) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdConfigDeviceSet) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 3, -1)
 	if exit {
@@ -576,7 +680,7 @@ func (c *cmdConfigDeviceSet) Run(cmd *cobra.Command, args []string) error {
 	resource := resources[0]
 
 	if resource.name == "" {
-		return fmt.Errorf(i18n.G("Missing name"))
+		return errors.New(i18n.G("Missing name"))
 	}
 
 	// Set the device config key
@@ -595,7 +699,7 @@ func (c *cmdConfigDeviceSet) Run(cmd *cobra.Command, args []string) error {
 
 		dev, ok := profile.Devices[devname]
 		if !ok {
-			return fmt.Errorf(i18n.G("Device doesn't exist"))
+			return errors.New(i18n.G("Device doesn't exist"))
 		}
 
 		for k, v := range keys {
@@ -618,10 +722,10 @@ func (c *cmdConfigDeviceSet) Run(cmd *cobra.Command, args []string) error {
 		if !ok {
 			_, ok = inst.ExpandedDevices[devname]
 			if !ok {
-				return fmt.Errorf(i18n.G("Device doesn't exist"))
+				return errors.New(i18n.G("Device doesn't exist"))
 			}
 
-			return fmt.Errorf(i18n.G("Device from profile(s) cannot be modified for individual instance. Override device or modify profile instead"))
+			return errors.New(i18n.G("Device from profile(s) cannot be modified for individual instance. Override device or modify profile instead"))
 		}
 
 		for k, v := range keys {
@@ -652,7 +756,7 @@ type cmdConfigDeviceShow struct {
 	profile      *cmdProfile
 }
 
-func (c *cmdConfigDeviceShow) Command() *cobra.Command {
+func (c *cmdConfigDeviceShow) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	if c.config != nil {
 		cmd.Use = usage("show", i18n.G("[<remote>:]<instance>"))
@@ -664,12 +768,24 @@ func (c *cmdConfigDeviceShow) Command() *cobra.Command {
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Show full device configuration`))
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
+
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			if c.config != nil {
+				return c.global.cmpInstances(toComplete)
+			} else if c.profile != nil {
+				return c.global.cmpProfiles(toComplete, true)
+			}
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
 
 	return cmd
 }
 
-func (c *cmdConfigDeviceShow) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdConfigDeviceShow) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
@@ -685,7 +801,7 @@ func (c *cmdConfigDeviceShow) Run(cmd *cobra.Command, args []string) error {
 	resource := resources[0]
 
 	if resource.name == "" {
-		return fmt.Errorf(i18n.G("Missing name"))
+		return errors.New(i18n.G("Missing name"))
 	}
 
 	// Show the devices
@@ -725,7 +841,7 @@ type cmdConfigDeviceUnset struct {
 	profile         *cmdProfile
 }
 
-func (c *cmdConfigDeviceUnset) Command() *cobra.Command {
+func (c *cmdConfigDeviceUnset) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	if c.config != nil {
 		cmd.Use = usage("unset", i18n.G("[<remote>:]<instance> <device> <key>"))
@@ -737,12 +853,32 @@ func (c *cmdConfigDeviceUnset) Command() *cobra.Command {
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(
 		`Unset device configuration keys`))
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
+
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			if c.config != nil {
+				return c.global.cmpInstances(toComplete)
+			} else if c.profile != nil {
+				return c.global.cmpProfiles(toComplete, true)
+			}
+		}
+
+		if len(args) == 1 {
+			if c.config != nil {
+				return c.global.cmpInstanceDeviceNames(args[0])
+			} else if c.profile != nil {
+				return c.global.cmpProfileDeviceNames(args[0])
+			}
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
 
 	return cmd
 }
 
-func (c *cmdConfigDeviceUnset) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdConfigDeviceUnset) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 3, 3)
 	if exit {
@@ -750,5 +886,5 @@ func (c *cmdConfigDeviceUnset) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	args = append(args, "")
-	return c.configDeviceSet.Run(cmd, args)
+	return c.configDeviceSet.run(cmd, args)
 }
