@@ -1,6 +1,7 @@
 package main
 
 import (
+	"errors"
 	"fmt"
 	"strings"
 
@@ -33,7 +34,7 @@ type cmdCopy struct {
 	flagAllowInconsistent bool
 }
 
-func (c *cmdCopy) Command() *cobra.Command {
+func (c *cmdCopy) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = usage("copy", i18n.G("[<remote>:]<source>[/<snapshot>] [[<remote>:]<destination>]"))
 	cmd.Aliases = []string{"cp"}
@@ -49,7 +50,7 @@ Transfer modes (--mode):
 The pull transfer mode is the default as it is compatible with all LXD versions.
 `))
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
 	cmd.Flags().StringArrayVarP(&c.flagConfig, "config", "c", nil, i18n.G("Config key/value to apply to the new instance")+"``")
 	cmd.Flags().StringArrayVarP(&c.flagDevice, "device", "d", nil, i18n.G("New key/value to apply to a specific device")+"``")
 	cmd.Flags().StringArrayVarP(&c.flagProfile, "profile", "p", nil, i18n.G("Profile to apply to the new instance")+"``")
@@ -64,6 +65,18 @@ The pull transfer mode is the default as it is compatible with all LXD versions.
 	cmd.Flags().BoolVar(&c.flagRefresh, "refresh", false, i18n.G("Perform an incremental copy"))
 	cmd.Flags().BoolVar(&c.flagAllowInconsistent, "allow-inconsistent", false, i18n.G("Ignore copy errors for volatile files"))
 
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			return c.global.cmpInstances(toComplete)
+		}
+
+		if len(args) == 1 {
+			return c.global.cmpRemotes(toComplete, false)
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
 	return cmd
 }
 
@@ -82,12 +95,12 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 
 	// Make sure we have an instance or snapshot name
 	if sourceName == "" {
-		return fmt.Errorf(i18n.G("You must specify a source instance name"))
+		return errors.New(i18n.G("You must specify a source instance name"))
 	}
 
 	// Don't allow refreshing without profiles.
 	if c.flagRefresh && c.flagNoProfiles {
-		return fmt.Errorf(i18n.G("--no-profiles cannot be used with --refresh"))
+		return errors.New(i18n.G("--no-profiles cannot be used with --refresh"))
 	}
 
 	// If the instance is being copied to a different remote and no destination name is
@@ -99,7 +112,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 
 	// Ensure that a destination name is provided.
 	if destName == "" {
-		return fmt.Errorf(i18n.G("You must specify a destination instance name"))
+		return errors.New(i18n.G("You must specify a destination instance name"))
 	}
 
 	// Connect to the source host
@@ -128,7 +141,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 
 	// Confirm that --target is only used with a cluster
 	if c.flagTarget != "" && !dest.IsClustered() {
-		return fmt.Errorf(i18n.G("To use --target, the destination remote must be a cluster"))
+		return errors.New(i18n.G("To use --target, the destination remote must be a cluster"))
 	}
 
 	// Parse the config overrides
@@ -153,7 +166,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 
 	if shared.IsSnapshot(sourceName) {
 		if instanceOnly {
-			return fmt.Errorf(i18n.G("--instance-only can't be passed when the source is a snapshot"))
+			return errors.New(i18n.G("--instance-only can't be passed when the source is a snapshot"))
 		}
 
 		// Prepare the instance creation request
@@ -164,7 +177,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 		}
 
 		if c.flagRefresh {
-			return fmt.Errorf(i18n.G("--refresh can only be used with instances"))
+			return errors.New(i18n.G("--refresh can only be used with instances"))
 		}
 
 		// Copy of a snapshot into a new instance
@@ -450,7 +463,7 @@ func (c *cmdCopy) copyInstance(conf *config.Config, sourceResource string, destR
 	return nil
 }
 
-func (c *cmdCopy) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdCopy) run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
 	// Quick checks.
