@@ -2,7 +2,7 @@ package main
 
 import (
 	"bytes"
-	"fmt"
+	"errors"
 	"io"
 	"os"
 	"strconv"
@@ -34,7 +34,7 @@ type cmdExec struct {
 	interactive bool
 }
 
-func (c *cmdExec) Command() *cobra.Command {
+func (c *cmdExec) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = usage("exec", i18n.G("[<remote>:]<instance> [flags] [--] <command line>"))
 	cmd.Short = i18n.G("Execute commands in instances")
@@ -50,7 +50,7 @@ executable, passing the shell commands as arguments, for example:
 
 Mode defaults to non-interactive, interactive mode is selected if both stdin AND stdout are terminals (stderr is ignored).`))
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
 	cmd.Flags().StringArrayVar(&c.flagEnvironment, "env", nil, i18n.G("Environment variable to set (e.g. HOME=/home/foo)")+"``")
 	cmd.Flags().StringVar(&c.flagMode, "mode", "auto", i18n.G("Override the terminal mode (auto, interactive or non-interactive)")+"``")
 	cmd.Flags().BoolVarP(&c.flagForceInteractive, "force-interactive", "t", false, i18n.G("Force pseudo-terminal allocation"))
@@ -60,6 +60,14 @@ Mode defaults to non-interactive, interactive mode is selected if both stdin AND
 	cmd.Flags().Uint32Var(&c.flagGroup, "group", 0, i18n.G("Group ID to run the command as (default 0)")+"``")
 	cmd.Flags().StringVar(&c.flagCwd, "cwd", "", i18n.G("Directory to run the command in (default /root)")+"``")
 
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			return c.global.cmpInstancesAction(toComplete, "exec", false)
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
 	return cmd
 }
 
@@ -80,7 +88,7 @@ func (c *cmdExec) sendTermSize(control *websocket.Conn) error {
 	return control.WriteJSON(msg)
 }
 
-func (c *cmdExec) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdExec) run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
 	// Quick checks.
@@ -90,11 +98,11 @@ func (c *cmdExec) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	if c.flagForceInteractive && c.flagForceNonInteractive {
-		return fmt.Errorf(i18n.G("You can't pass -t and -T at the same time"))
+		return errors.New(i18n.G("You can't pass -t and -T at the same time"))
 	}
 
 	if c.flagMode != "auto" && (c.flagForceInteractive || c.flagForceNonInteractive) {
-		return fmt.Errorf(i18n.G("You can't pass -t or -T at the same time as --mode"))
+		return errors.New(i18n.G("You can't pass -t or -T at the same time as --mode"))
 	}
 
 	// Connect to the daemon
@@ -116,13 +124,10 @@ func (c *cmdExec) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	for _, arg := range c.flagEnvironment {
-		pieces := strings.SplitN(arg, "=", 2)
-		value := ""
-		if len(pieces) > 1 {
-			value = pieces[1]
+		variable, value, found := strings.Cut(arg, "=")
+		if found {
+			env[variable] = value
 		}
-
-		env[pieces[0]] = value
 	}
 
 	// Configure the terminal
