@@ -23,7 +23,6 @@ type cmdExport struct {
 	flagInstanceOnly         bool
 	flagOptimizedStorage     bool
 	flagCompressionAlgorithm string
-	flagExportVersion        string
 }
 
 func (c *cmdExport) command() *cobra.Command {
@@ -42,16 +41,6 @@ func (c *cmdExport) command() *cobra.Command {
 	cmd.Flags().BoolVar(&c.flagOptimizedStorage, "optimized-storage", false,
 		i18n.G("Use storage driver optimized format (can only be restored on a similar pool)"))
 	cmd.Flags().StringVar(&c.flagCompressionAlgorithm, "compression", "", i18n.G("Compression algorithm to use (none for uncompressed)")+"``")
-	cmd.Flags().StringVar(&c.flagExportVersion, "export-version", "",
-		i18n.G("Use a different metadata format version than the latest one supported by the server")+"``")
-
-	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]cobra.Completion, cobra.ShellCompDirective) {
-		if len(args) > 0 {
-			return nil, cobra.ShellCompDirectiveNoFileComp
-		}
-
-		return c.global.cmpTopLevelResource("instance", toComplete)
-	}
 
 	return cmd
 }
@@ -87,11 +76,6 @@ func (c *cmdExport) run(cmd *cobra.Command, args []string) error {
 		CompressionAlgorithm: c.flagCompressionAlgorithm,
 	}
 
-	req.Version, err = getExportVersion(d, c.flagExportVersion)
-	if err != nil {
-		return err
-	}
-
 	op, err := d.CreateInstanceBackup(name, req)
 	if err != nil {
 		return fmt.Errorf("Create instance backup: %w", err)
@@ -163,23 +147,22 @@ func (c *cmdExport) run(cmd *cobra.Command, args []string) error {
 		}
 	}()
 
-	// Prepare the download request.
-	// Assign the renderer to a new variable to not interfer with the old one.
-	exportProgress := cli.ProgressRenderer{
+	// Prepare the download request
+	progress = cli.ProgressRenderer{
 		Format: i18n.G("Exporting the backup: %s"),
 		Quiet:  c.global.flagQuiet,
 	}
 
 	backupFileRequest := lxd.BackupFileRequest{
 		BackupFile:      io.WriteSeeker(target),
-		ProgressHandler: exportProgress.UpdateProgress,
+		ProgressHandler: progress.UpdateProgress,
 	}
 
 	// Export tarball
 	_, err = d.GetInstanceBackupFile(name, backupName, &backupFileRequest)
 	if err != nil {
 		_ = os.Remove(targetName)
-		exportProgress.Done("")
+		progress.Done("")
 		return fmt.Errorf("Fetch instance backup file: %w", err)
 	}
 
@@ -206,6 +189,6 @@ func (c *cmdExport) run(cmd *cobra.Command, args []string) error {
 		return fmt.Errorf("Failed to close export file: %w", err)
 	}
 
-	exportProgress.Done(i18n.G("Backup exported successfully!"))
+	progress.Done(i18n.G("Backup exported successfully!"))
 	return nil
 }
