@@ -13,7 +13,6 @@ import (
 	"os/signal"
 	"path"
 	"path/filepath"
-	"slices"
 	"strconv"
 	"strings"
 
@@ -168,7 +167,7 @@ func (c *cmdFileCreate) run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	if !slices.Contains([]string{"file", "symlink", "directory"}, c.flagType) {
+	if !shared.ValueInSlice(c.flagType, []string{"file", "symlink", "directory"}) {
 		return fmt.Errorf(i18n.G("Invalid type %q"), c.flagType)
 	}
 
@@ -210,18 +209,23 @@ func (c *cmdFileCreate) run(cmd *cobra.Command, args []string) error {
 	}
 
 	// Determine the target uid
-	uid := max(c.file.flagUID, 0)
+	uid := 0
+	if c.file.flagUID > 0 {
+		uid = c.file.flagUID
+	}
 
 	// Determine the target gid
-	gid := max(c.file.flagGID, 0)
+	gid := 0
+	if c.file.flagGID > 0 {
+		gid = c.file.flagGID
+	}
 
 	var mode os.FileMode
 
 	// Determine the target mode
-	switch c.flagType {
-	case "directory":
+	if c.flagType == "directory" {
 		mode = os.FileMode(DirMode)
-	case "file":
+	} else if c.flagType == "file" {
 		mode = os.FileMode(FileMode)
 	}
 
@@ -250,12 +254,11 @@ func (c *cmdFileCreate) run(cmd *cobra.Command, args []string) error {
 	var readCloser io.ReadCloser
 	var contentLength int64
 
-	switch c.flagType {
-	case "symlink":
+	if c.flagType == "symlink" {
 		content = strings.NewReader(symlinkTargetPath)
 		readCloser = io.NopCloser(content)
 		contentLength = int64(len(symlinkTargetPath))
-	case "file":
+	} else if c.flagType == "file" {
 		// Just creating an empty file.
 		content = strings.NewReader("")
 		readCloser = io.NopCloser(content)
@@ -320,6 +323,14 @@ func (c *cmdFileDelete) command() *cobra.Command {
 
 	cmd.RunE = c.run
 
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			return c.global.cmpInstances(toComplete)
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		return c.global.cmpFiles(toComplete, false)
 	}
@@ -375,7 +386,7 @@ func (c *cmdFileEdit) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("instance", toComplete)
+			return c.global.cmpInstances(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -407,7 +418,7 @@ func (c *cmdFileEdit) run(cmd *cobra.Command, args []string) error {
 	}
 
 	// Create temp file
-	f, err := os.CreateTemp("", "lxd_file_edit_*"+filepath.Ext(args[0]))
+	f, err := os.CreateTemp("", "lxd_file_edit_")
 	if err != nil {
 		return fmt.Errorf(i18n.G("Unable to create a temporary file: %v"), err)
 	}
@@ -581,7 +592,7 @@ func (c *cmdFilePull) run(cmd *cobra.Command, args []string) error {
 			}
 
 			// Follow the symlink
-			if targetPath != "-" && !c.file.flagRecursive {
+			if !(targetPath == "-" || c.file.flagRecursive) {
 				err = os.Symlink(strings.TrimSpace(string(linkTarget)), targetPath)
 				if err != nil {
 					return err
@@ -608,7 +619,7 @@ func (c *cmdFilePull) run(cmd *cobra.Command, args []string) error {
 
 				i++
 				if i > 255 {
-					return errors.New("Too many links")
+					return fmt.Errorf("Too many links")
 				}
 
 				// Update link target for next iteration.
@@ -813,12 +824,18 @@ func (c *cmdFilePush) run(cmd *cobra.Command, args []string) error {
 	modifyExistingUID := c.file.flagUID != -1
 
 	// Determine the target uid
-	uid := max(c.file.flagUID, 0)
+	uid := 0
+	if c.file.flagUID >= 0 {
+		uid = c.file.flagUID
+	}
 
 	modifyExistingGID := c.file.flagGID != -1
 
 	// Determine the target gid
-	gid := max(c.file.flagGID, 0)
+	gid := 0
+	if c.file.flagGID >= 0 {
+		gid = c.file.flagGID
+	}
 
 	if (len(sourcefilenames) > 1) && !targetIsDir {
 		return errors.New(i18n.G("Missing target directory"))
@@ -960,8 +977,7 @@ func (c *cmdFile) recursivePullFile(d lxd.InstanceServer, inst string, p string,
 	target := filepath.Join(targetDir, filepath.Base(p))
 	logger.Infof("Pulling %s from %s (%s)", target, p, resp.Type)
 
-	switch resp.Type {
-	case "directory":
+	if resp.Type == "directory" {
 		err := os.Mkdir(target, os.FileMode(resp.Mode))
 		if err != nil {
 			return err
@@ -975,7 +991,7 @@ func (c *cmdFile) recursivePullFile(d lxd.InstanceServer, inst string, p string,
 				return err
 			}
 		}
-	case "file":
+	} else if resp.Type == "file" {
 		f, err := os.Create(target)
 		if err != nil {
 			return err
@@ -1017,7 +1033,7 @@ func (c *cmdFile) recursivePullFile(d lxd.InstanceServer, inst string, p string,
 		}
 
 		progress.Done("")
-	case "symlink":
+	} else if resp.Type == "symlink" {
 		linkTarget, err := io.ReadAll(buf)
 		if err != nil {
 			return err
@@ -1027,8 +1043,7 @@ func (c *cmdFile) recursivePullFile(d lxd.InstanceServer, inst string, p string,
 		if err != nil {
 			return err
 		}
-
-	default:
+	} else {
 		return fmt.Errorf(i18n.G("Unknown file type '%s'"), resp.Type)
 	}
 
