@@ -1,8 +1,8 @@
 package main
 
 import (
-	"fmt"
 	"os"
+	"strconv"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -22,7 +22,7 @@ type cmdImport struct {
 	flagDevice  []string
 }
 
-func (c *cmdImport) Command() *cobra.Command {
+func (c *cmdImport) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = usage("import", i18n.G("[<remote>:] <backup file> [<instance name>]"))
 	cmd.Short = i18n.G("Import instance backups")
@@ -32,14 +32,14 @@ func (c *cmdImport) Command() *cobra.Command {
 		`lxc import backup0.tar.gz
     Create a new instance using backup0.tar.gz as the source.`))
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
 	cmd.Flags().StringVarP(&c.flagStorage, "storage", "s", "", i18n.G("Storage pool name")+"``")
 	cmd.Flags().StringArrayVarP(&c.flagDevice, "device", "d", nil, i18n.G("New key/value to apply to a specific device")+"``")
 
 	return cmd
 }
 
-func (c *cmdImport) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdImport) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 3)
 	if exit {
@@ -108,7 +108,7 @@ func (c *cmdImport) Run(cmd *cobra.Command, args []string) error {
 			Tracker: &ioprogress.ProgressTracker{
 				Length: fstat.Size(),
 				Handler: func(percent int64, speed int64) {
-					progress.UpdateProgress(ioprogress.ProgressData{Text: fmt.Sprintf("%d%% (%s/s)", percent, units.GetByteSizeString(speed, 2))})
+					progress.UpdateProgress(ioprogress.ProgressData{Text: strconv.FormatInt(percent, 10) + "% (" + units.GetByteSizeString(speed, 2) + "/s)"})
 				},
 			},
 		},
