@@ -1,6 +1,7 @@
 package main
 
 import (
+	"errors"
 	"fmt"
 	"io"
 	"sort"
@@ -10,7 +11,6 @@ import (
 	"gopkg.in/yaml.v2"
 
 	"github.com/canonical/lxd/client"
-	"github.com/canonical/lxd/lxc/config"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	cli "github.com/canonical/lxd/shared/cmd"
@@ -26,7 +26,7 @@ type cmdInfo struct {
 	flagTarget    string
 }
 
-func (c *cmdInfo) Command() *cobra.Command {
+func (c *cmdInfo) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = usage("info", i18n.G("[<remote>:][<instance>]"))
 	cmd.Short = i18n.G("Show instance or server information")
@@ -39,15 +39,23 @@ func (c *cmdInfo) Command() *cobra.Command {
 lxc info [<remote>:] [--resources]
     For LXD server information.`))
 
-	cmd.RunE = c.Run
-	cmd.Flags().BoolVar(&c.flagShowLog, "show-log", false, i18n.G("Show the instance's last 100 log lines?"))
+	cmd.RunE = c.run
+	cmd.Flags().BoolVar(&c.flagShowLog, "show-log", false, i18n.G("Show the instance's last 100 log lines"))
 	cmd.Flags().BoolVar(&c.flagResources, "resources", false, i18n.G("Show the resources available to the server"))
 	cmd.Flags().StringVar(&c.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			return c.global.cmpInstances(toComplete)
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
 	return cmd
 }
 
-func (c *cmdInfo) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdInfo) run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
 	// Quick checks.
@@ -79,7 +87,7 @@ func (c *cmdInfo) Run(cmd *cobra.Command, args []string) error {
 		return c.remoteInfo(d)
 	}
 
-	return c.instanceInfo(d, conf.Remotes[remote], cName, c.flagShowLog)
+	return c.instanceInfo(d, cName, c.flagShowLog)
 }
 
 func (c *cmdInfo) renderGPU(gpu api.ResourcesGPUCard, prefix string, initial bool) {
@@ -106,7 +114,7 @@ func (c *cmdInfo) renderGPU(gpu api.ResourcesGPUCard, prefix string, initial boo
 	}
 
 	if gpu.DRM != nil {
-		fmt.Printf(prefix + i18n.G("DRM:") + "\n")
+		fmt.Print(prefix + i18n.G("DRM:") + "\n")
 		fmt.Printf(prefix+"  "+i18n.G("ID: %d")+"\n", gpu.DRM.ID)
 
 		if gpu.DRM.CardName != "" {
@@ -123,7 +131,7 @@ func (c *cmdInfo) renderGPU(gpu api.ResourcesGPUCard, prefix string, initial boo
 	}
 
 	if gpu.Nvidia != nil {
-		fmt.Printf(prefix + i18n.G("NVIDIA information:") + "\n")
+		fmt.Print(prefix + i18n.G("NVIDIA information:") + "\n")
 		fmt.Printf(prefix+"  "+i18n.G("Architecture: %v")+"\n", gpu.Nvidia.Architecture)
 		fmt.Printf(prefix+"  "+i18n.G("Brand: %v")+"\n", gpu.Nvidia.Brand)
 		fmt.Printf(prefix+"  "+i18n.G("Model: %v")+"\n", gpu.Nvidia.Model)
@@ -133,7 +141,7 @@ func (c *cmdInfo) renderGPU(gpu api.ResourcesGPUCard, prefix string, initial boo
 	}
 
 	if gpu.SRIOV != nil {
-		fmt.Printf(prefix + i18n.G("SR-IOV information:") + "\n")
+		fmt.Print(prefix + i18n.G("SR-IOV information:") + "\n")
 		fmt.Printf(prefix+"  "+i18n.G("Current number of VFs: %d")+"\n", gpu.SRIOV.CurrentVFs)
 		fmt.Printf(prefix+"  "+i18n.G("Maximum number of VFs: %d")+"\n", gpu.SRIOV.MaximumVFs)
 		if len(gpu.SRIOV.VFs) > 0 {
@@ -146,7 +154,7 @@ func (c *cmdInfo) renderGPU(gpu api.ResourcesGPUCard, prefix string, initial boo
 	}
 
 	if gpu.Mdev != nil {
-		fmt.Printf(prefix + i18n.G("Mdev profiles:") + "\n")
+		fmt.Print(prefix + i18n.G("Mdev profiles:") + "\n")
 
 		keys := make([]string, 0, len(gpu.Mdev))
 		for k := range gpu.Mdev {
@@ -192,7 +200,7 @@ func (c *cmdInfo) renderNIC(nic api.ResourcesNetworkCard, prefix string, initial
 	}
 
 	if len(nic.Ports) > 0 {
-		fmt.Printf(prefix + i18n.G("Ports:") + "\n")
+		fmt.Print(prefix + i18n.G("Ports:") + "\n")
 		for _, port := range nic.Ports {
 			fmt.Printf(prefix+"  "+i18n.G("- Port %d (%s)")+"\n", port.Port, port.Protocol)
 			fmt.Printf(prefix+"    "+i18n.G("ID: %s")+"\n", port.ID)
@@ -224,7 +232,7 @@ func (c *cmdInfo) renderNIC(nic api.ResourcesNetworkCard, prefix string, initial
 			}
 
 			if port.Infiniband != nil {
-				fmt.Printf(prefix + "    " + i18n.G("Infiniband:") + "\n")
+				fmt.Print(prefix + "    " + i18n.G("Infiniband:") + "\n")
 
 				if port.Infiniband.IsSMName != "" {
 					fmt.Printf(prefix+"      "+i18n.G("IsSM: %s (%s)")+"\n", port.Infiniband.IsSMName, port.Infiniband.IsSMDevice)
@@ -242,7 +250,7 @@ func (c *cmdInfo) renderNIC(nic api.ResourcesNetworkCard, prefix string, initial
 	}
 
 	if nic.SRIOV != nil {
-		fmt.Printf(prefix + i18n.G("SR-IOV information:") + "\n")
+		fmt.Print(prefix + i18n.G("SR-IOV information:") + "\n")
 		fmt.Printf(prefix+"  "+i18n.G("Current number of VFs: %d")+"\n", nic.SRIOV.CurrentVFs)
 		fmt.Printf(prefix+"  "+i18n.G("Maximum number of VFs: %d")+"\n", nic.SRIOV.MaximumVFs)
 		if len(nic.SRIOV.VFs) > 0 {
@@ -284,7 +292,7 @@ func (c *cmdInfo) renderDisk(disk api.ResourcesStorageDisk, prefix string, initi
 	fmt.Printf(prefix+i18n.G("Removable: %v")+"\n", disk.Removable)
 
 	if len(disk.Partitions) != 0 {
-		fmt.Printf(prefix + i18n.G("Partitions:") + "\n")
+		fmt.Print(prefix + i18n.G("Partitions:") + "\n")
 		for _, partition := range disk.Partitions {
 			fmt.Printf(prefix+"  "+i18n.G("- Partition %d")+"\n", partition.Partition)
 			fmt.Printf(prefix+"    "+i18n.G("ID: %s")+"\n", partition.ID)
@@ -306,17 +314,17 @@ func (c *cmdInfo) renderCPU(cpu api.ResourcesCPUSocket, prefix string) {
 	}
 
 	if cpu.Cache != nil {
-		fmt.Printf(prefix + i18n.G("Caches:") + "\n")
+		fmt.Print(prefix + i18n.G("Caches:") + "\n")
 		for _, cache := range cpu.Cache {
 			fmt.Printf(prefix+"  "+i18n.G("- Level %d (type: %s): %s")+"\n", cache.Level, cache.Type, units.GetByteSizeStringIEC(int64(cache.Size), 0))
 		}
 	}
 
-	fmt.Printf(prefix + i18n.G("Cores:") + "\n")
+	fmt.Print(prefix + i18n.G("Cores:") + "\n")
 	for _, core := range cpu.Cores {
 		fmt.Printf(prefix+"  - "+i18n.G("Core %d")+"\n", core.Core)
 		fmt.Printf(prefix+"    "+i18n.G("Frequency: %vMhz")+"\n", core.Frequency)
-		fmt.Printf(prefix + "    " + i18n.G("Threads:") + "\n")
+		fmt.Print(prefix + "    " + i18n.G("Threads:") + "\n")
 		for _, thread := range core.Threads {
 			fmt.Printf(prefix+"      - "+i18n.G("%d (id: %d, online: %v, NUMA node: %v)")+"\n", thread.Thread, thread.ID, thread.Online, thread.NUMANode)
 		}
@@ -335,7 +343,7 @@ func (c *cmdInfo) remoteInfo(d lxd.InstanceServer) error {
 	// Targeting
 	if c.flagTarget != "" {
 		if !d.IsClustered() {
-			return fmt.Errorf(i18n.G("To use --target, the destination remote must be a cluster"))
+			return errors.New(i18n.G("To use --target, the destination remote must be a cluster"))
 		}
 
 		d = d.UseTarget(c.flagTarget)
@@ -343,7 +351,7 @@ func (c *cmdInfo) remoteInfo(d lxd.InstanceServer) error {
 
 	if c.flagResources {
 		if !d.HasExtension("resources_v2") {
-			return fmt.Errorf(i18n.G("The server doesn't implement the newer v2 resources API"))
+			return errors.New(i18n.G("The server doesn't implement the newer v2 resources API"))
 		}
 
 		resources, err := d.GetServerResources()
@@ -364,20 +372,20 @@ func (c *cmdInfo) remoteInfo(d lxd.InstanceServer) error {
 		}
 
 		// Memory
-		fmt.Printf("\n" + i18n.G("Memory:") + "\n")
+		fmt.Print("\n" + i18n.G("Memory:") + "\n")
 		if resources.Memory.HugepagesTotal > 0 {
-			fmt.Printf("  " + i18n.G("Hugepages:"+"\n"))
+			fmt.Print("  " + i18n.G("Hugepages:"+"\n"))
 			fmt.Printf("    "+i18n.G("Free: %v")+"\n", units.GetByteSizeStringIEC(int64(resources.Memory.HugepagesTotal-resources.Memory.HugepagesUsed), 2))
 			fmt.Printf("    "+i18n.G("Used: %v")+"\n", units.GetByteSizeStringIEC(int64(resources.Memory.HugepagesUsed), 2))
 			fmt.Printf("    "+i18n.G("Total: %v")+"\n", units.GetByteSizeStringIEC(int64(resources.Memory.HugepagesTotal), 2))
 		}
 
 		if len(resources.Memory.Nodes) > 1 {
-			fmt.Printf("  " + i18n.G("NUMA nodes:"+"\n"))
+			fmt.Print("  " + i18n.G("NUMA nodes:"+"\n"))
 			for _, node := range resources.Memory.Nodes {
 				fmt.Printf("    "+i18n.G("Node %d:"+"\n"), node.NUMANode)
 				if node.HugepagesTotal > 0 {
-					fmt.Printf("      " + i18n.G("Hugepages:"+"\n"))
+					fmt.Print("      " + i18n.G("Hugepages:"+"\n"))
 					fmt.Printf("        "+i18n.G("Free: %v")+"\n", units.GetByteSizeStringIEC(int64(node.HugepagesTotal-node.HugepagesUsed), 2))
 					fmt.Printf("        "+i18n.G("Used: %v")+"\n", units.GetByteSizeStringIEC(int64(node.HugepagesUsed), 2))
 					fmt.Printf("        "+i18n.G("Total: %v")+"\n", units.GetByteSizeStringIEC(int64(node.HugepagesTotal), 2))
@@ -395,10 +403,10 @@ func (c *cmdInfo) remoteInfo(d lxd.InstanceServer) error {
 
 		// GPUs
 		if len(resources.GPU.Cards) == 1 {
-			fmt.Printf("\n" + i18n.G("GPU:") + "\n")
+			fmt.Print("\n" + i18n.G("GPU:") + "\n")
 			c.renderGPU(resources.GPU.Cards[0], "  ", true)
 		} else if len(resources.GPU.Cards) > 1 {
-			fmt.Printf("\n" + i18n.G("GPUs:") + "\n")
+			fmt.Print("\n" + i18n.G("GPUs:") + "\n")
 			for id, gpu := range resources.GPU.Cards {
 				fmt.Printf("  "+i18n.G("Card %d:")+"\n", id)
 				c.renderGPU(gpu, "    ", true)
@@ -407,10 +415,10 @@ func (c *cmdInfo) remoteInfo(d lxd.InstanceServer) error {
 
 		// Network interfaces
 		if len(resources.Network.Cards) == 1 {
-			fmt.Printf("\n" + i18n.G("NIC:") + "\n")
+			fmt.Print("\n" + i18n.G("NIC:") + "\n")
 			c.renderNIC(resources.Network.Cards[0], "  ", true)
 		} else if len(resources.Network.Cards) > 1 {
-			fmt.Printf("\n" + i18n.G("NICs:") + "\n")
+			fmt.Print("\n" + i18n.G("NICs:") + "\n")
 			for id, nic := range resources.Network.Cards {
 				fmt.Printf("  "+i18n.G("Card %d:")+"\n", id)
 				c.renderNIC(nic, "    ", true)
@@ -419,10 +427,10 @@ func (c *cmdInfo) remoteInfo(d lxd.InstanceServer) error {
 
 		// Storage
 		if len(resources.Storage.Disks) == 1 {
-			fmt.Printf("\n" + i18n.G("Disk:") + "\n")
+			fmt.Print("\n" + i18n.G("Disk:") + "\n")
 			c.renderDisk(resources.Storage.Disks[0], "  ", true)
 		} else if len(resources.Storage.Disks) > 1 {
-			fmt.Printf("\n" + i18n.G("Disks:") + "\n")
+			fmt.Print("\n" + i18n.G("Disks:") + "\n")
 			for id, nic := range resources.Storage.Disks {
 				fmt.Printf("  "+i18n.G("Disk %d:")+"\n", id)
 				c.renderDisk(nic, "    ", true)
@@ -447,10 +455,10 @@ func (c *cmdInfo) remoteInfo(d lxd.InstanceServer) error {
 	return nil
 }
 
-func (c *cmdInfo) instanceInfo(d lxd.InstanceServer, remote config.Remote, name string, showLog bool) error {
+func (c *cmdInfo) instanceInfo(d lxd.InstanceServer, name string, showLog bool) error {
 	// Quick checks.
 	if c.flagTarget != "" {
-		return fmt.Errorf(i18n.G("--target cannot be used with instances"))
+		return errors.New(i18n.G("--target cannot be used with instances"))
 	}
 
 	// Get the full instance data.
@@ -499,18 +507,30 @@ func (c *cmdInfo) instanceInfo(d lxd.InstanceServer, remote config.Remote, name
 		fmt.Printf("  "+i18n.G("Processes: %d")+"\n", inst.State.Processes)
 
 		// Disk usage
-		diskInfo := ""
+		diskUsage := ""
+		diskTotal := ""
 		if inst.State.Disk != nil {
 			for entry, disk := range inst.State.Disk {
-				if disk.Usage != 0 {
-					diskInfo += fmt.Sprintf("    %s: %s\n", entry, units.GetByteSizeStringIEC(disk.Usage, 2))
+				// Only show usage when supported.
+				if disk.Usage != -1 {
+					diskUsage += fmt.Sprintf("    %s: %s\n", entry, units.GetByteSizeStringIEC(disk.Usage, 2))
+				}
+			}
+
+			for entry, disk := range inst.State.Disk {
+				// Only show total for disks that are bounded within the pool.
+				if disk.Total != -1 {
+					diskTotal += fmt.Sprintf("    %s: %s\n", entry, units.GetByteSizeStringIEC(disk.Usage, 2))
 				}
 			}
 		}
 
-		if diskInfo != "" {
-			fmt.Printf("  %s\n", i18n.G("Disk usage:"))
-			fmt.Print(diskInfo)
+		if diskUsage != "" {
+			fmt.Printf("  %s\n%s", i18n.G("Disk usage:"), diskUsage)
+		}
+
+		if diskTotal != "" {
+			fmt.Printf("  %s\n%s", i18n.G("Disk total:"), diskTotal)
 		}
 
 		// CPU usage
