@@ -1,6 +1,7 @@
 package main
 
 import (
+	"errors"
 	"fmt"
 	"io"
 	"net/url"
@@ -36,17 +37,24 @@ type cmdInit struct {
 	flagVM         bool
 }
 
-func (c *cmdInit) Command() *cobra.Command {
+func (c *cmdInit) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = usage("init", i18n.G("[<remote>:]<image> [<remote>:][<name>]"))
 	cmd.Short = i18n.G("Create instances from images")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(`Create instances from images`))
-	cmd.Example = cli.FormatSection("", i18n.G(`lxc init ubuntu:22.04 u1
+	cmd.Example = cli.FormatSection("", i18n.G(`lxc init ubuntu:24.04 u1
+    Create a container (but do not start it)
 
-lxc init ubuntu:22.04 u1 < config.yaml
-    Create the instance with configuration from config.yaml`))
+lxc init ubuntu:24.04 u1 < config.yaml
+    Create a container with configuration from config.yaml
 
-	cmd.RunE = c.Run
+lxc init ubuntu:24.04 v1 --vm -c limits.cpu=4 -c limits.memory=4GiB
+    Create a virtual machine with 4 vCPUs and 4GiB of RAM
+
+lxc init ubuntu:24.04 v1 --vm -c limits.cpu=2 -c limits.memory=8GiB -d root,size=32GiB
+    Create a virtual machine with 2 vCPUs, 8GiB of RAM and a root disk of 32GiB`))
+
+	cmd.RunE = c.run
 	cmd.Flags().StringArrayVarP(&c.flagConfig, "config", "c", nil, i18n.G("Config key/value to apply to the new instance")+"``")
 	cmd.Flags().StringArrayVarP(&c.flagProfile, "profile", "p", nil, i18n.G("Profile to apply to the new instance")+"``")
 	cmd.Flags().StringArrayVarP(&c.flagDevice, "device", "d", nil, i18n.G("New key/value to apply to a specific device")+"``")
@@ -59,10 +67,22 @@ lxc init ubuntu:22.04 u1 < config.yaml
 	cmd.Flags().BoolVar(&c.flagEmpty, "empty", false, i18n.G("Create an empty instance"))
 	cmd.Flags().BoolVar(&c.flagVM, "vm", false, i18n.G("Create a virtual machine"))
 
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) != 0 {
+			return nil, cobra.ShellCompDirectiveNoFileComp
+		}
+
+		return c.global.cmpImages(toComplete)
+	}
+
+	_ = cmd.RegisterFlagCompletionFunc("profile", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		return c.global.cmpProfiles(toComplete, true)
+	})
+
 	return cmd
 }
 
-func (c *cmdInit) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdInit) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 0, 2)
 	if exit {
@@ -74,11 +94,11 @@ func (c *cmdInit) Run(cmd *cobra.Command, args []string) error {
 		return nil
 	}
 
-	_, _, err = c.create(c.global.conf, args)
+	_, _, err = c.create(c.global.conf, args, false)
 	return err
 }
 
-func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer, string, error) {
+func (c *cmdInit) create(conf *config.Config, args []string, launch bool) (lxd.InstanceServer, string, error) {
 	var name string
 	var image string
 	var remote string
@@ -89,7 +109,7 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 	var configMap map[string]string
 	var profiles []string
 
-	// If stdin isn't a terminal, read text from it
+	// If stdin isn't a terminal, read text from it.
 	if !termios.IsTerminal(getStdinFd()) {
 		contents, err := io.ReadAll(os.Stdin)
 		if err != nil {
@@ -123,7 +143,7 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 
 	if c.flagEmpty {
 		if len(args) > 1 {
-			return nil, "", fmt.Errorf(i18n.G("--empty cannot be combined with an image name"))
+			return nil, "", errors.New(i18n.G("--empty cannot be combined with an image name"))
 		}
 
 		if len(args) == 0 {
@@ -132,7 +152,7 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 				return nil, "", err
 			}
 		} else if len(args) == 1 {
-			// Switch image / instance names
+			// Switch image / instance names.
 			name = image
 			remote = iremote
 			image = ""
@@ -145,10 +165,6 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 		return nil, "", err
 	}
 
-	if c.flagTarget != "" {
-		d = d.UseTarget(c.flagTarget)
-	}
-
 	// Overwrite profiles.
 	if c.flagProfile != nil {
 		profiles = c.flagProfile
@@ -157,10 +173,18 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 	}
 
 	if !c.global.flagQuiet {
-		if name == "" {
-			fmt.Printf(i18n.G("Creating the instance") + "\n")
+		if d.HasExtension("instance_create_start") && launch {
+			if name == "" {
+				fmt.Print(i18n.G("Launching the instance") + "\n")
+			} else {
+				fmt.Printf(i18n.G("Launching %s")+"\n", name)
+			}
 		} else {
-			fmt.Printf(i18n.G("Creating %s")+"\n", name)
+			if name == "" {
+				fmt.Print(i18n.G("Creating the instance") + "\n")
+			} else {
+				fmt.Printf(i18n.G("Creating %s")+"\n", name)
+			}
 		}
 	}
 
@@ -240,11 +264,17 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 		instanceDBType = api.InstanceTypeVM
 	}
 
-	// Setup instance creation request
+	// Set the target if provided.
+	if c.flagTarget != "" {
+		d = d.UseTarget(c.flagTarget)
+	}
+
+	// Setup instance creation request.
 	req := api.InstancesPost{
 		Name:         name,
 		InstanceType: c.flagType,
 		Type:         instanceDBType,
+		Start:        launch,
 	}
 
 	req.Config = configMap
@@ -299,14 +329,15 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 
 	var opInfo api.Operation
 	if !c.flagEmpty {
-		// Get the image server and image info
+		// Get the image server and image info.
 		iremote, image = guessImage(conf, d, remote, iremote, image)
 
-		// Deal with the default image
+		// Deal with the default image.
 		if image == "" {
 			image = "default"
 		}
 
+		// Fetch image info from the given remote.
 		imgRemote, imgInfo, err := getImgInfo(d, conf, iremote, remote, image, &req.Source)
 		if err != nil {
 			return nil, "", err
@@ -314,19 +345,19 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 
 		if conf.Remotes[iremote].Protocol != "simplestreams" {
 			if imgInfo.Type != "virtual-machine" && c.flagVM {
-				return nil, "", fmt.Errorf(i18n.G("Asked for a VM but image is of type container"))
+				return nil, "", errors.New(i18n.G("Asked for a VM but image is of type container"))
 			}
 
 			req.Type = api.InstanceType(imgInfo.Type)
 		}
 
-		// Create the instance
+		// Create the instance.
 		op, err := d.CreateInstanceFromImage(imgRemote, *imgInfo, req)
 		if err != nil {
 			return nil, "", err
 		}
 
-		// Watch the background operation
+		// Watch the background operation.
 		progress := cli.ProgressRenderer{
 			Format: i18n.G("Retrieving image: %s"),
 			Quiet:  c.global.flagQuiet,
@@ -346,7 +377,7 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 
 		progress.Done("")
 
-		// Extract the instance name
+		// Extract the instance name.
 		info, err := op.GetTarget()
 		if err != nil {
 			return nil, "", err
@@ -354,7 +385,7 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 
 		opInfo = *info
 	} else {
-		req.Source.Type = "none"
+		req.Source.Type = api.SourceTypeNone
 
 		op, err := d.CreateInstance(req)
 		if err != nil {
@@ -371,10 +402,10 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 
 	instances, ok := opInfo.Resources["instances"]
 	if !ok || len(instances) == 0 {
-		// Try using the older "containers" field
+		// Try using the older "containers" field.
 		instances, ok = opInfo.Resources["containers"]
 		if !ok || len(instances) == 0 {
-			return nil, "", fmt.Errorf(i18n.G("Didn't get any affected image, instance or snapshot from server"))
+			return nil, "", errors.New(i18n.G("Didn't get any affected image, instance or snapshot from server"))
 		}
 	}
 
@@ -388,7 +419,7 @@ func (c *cmdInit) create(conf *config.Config, args []string) (lxd.InstanceServer
 		fmt.Printf(i18n.G("Instance name is: %s")+"\n", name)
 	}
 
-	// Validate the network setup
+	// Validate the network setup.
 	c.checkNetwork(d, name)
 
 	return d, name, nil
@@ -406,7 +437,7 @@ func (c *cmdInit) checkNetwork(d lxd.InstanceServer, name string) {
 		}
 	}
 
-	fmt.Fprintf(os.Stderr, "\n"+i18n.G("The instance you are starting doesn't have any network attached to it.")+"\n")
-	fmt.Fprintf(os.Stderr, "  "+i18n.G("To create a new network, use: lxc network create")+"\n")
-	fmt.Fprintf(os.Stderr, "  "+i18n.G("To attach a network to an instance, use: lxc network attach")+"\n\n")
+	fmt.Fprint(os.Stderr, "\n"+i18n.G("The instance you are starting doesn't have any network attached to it.")+"\n")
+	fmt.Fprint(os.Stderr, "  "+i18n.G("To create a new network, use: lxc network create")+"\n")
+	fmt.Fprint(os.Stderr, "  "+i18n.G("To attach a network to an instance, use: lxc network attach")+"\n\n")
 }
