@@ -192,7 +192,7 @@ func TestColumns(t *testing.T) {
 			l = rand.Intn(20) + 20
 		}
 
-		for range l {
+		for i := 0; i < l; i++ {
 			buffer.WriteByte(alphanum[rand.Intn(len(alphanum))])
 		}
 	}
@@ -250,7 +250,7 @@ func TestColumns(t *testing.T) {
 		buffer.WriteRune(',')
 	}
 
-	for range 1000 {
+	for i := 0; i < 1000; i++ {
 		go func() {
 			var buffer bytes.Buffer
 
@@ -260,7 +260,7 @@ func TestColumns(t *testing.T) {
 			}
 
 			num := l
-			for range l {
+			for j := 0; j < l; j++ {
 				switch rand.Intn(5) {
 				case 0:
 					if buffer.Len() > 0 {
