@@ -2,6 +2,7 @@ package main
 
 import (
 	"bufio"
+	"errors"
 	"fmt"
 	"os"
 	"os/user"
@@ -86,10 +87,9 @@ All of LXD's features can be driven through the various commands below.
 For help with any of those, simply call them with --help.`))
 	app.SilenceUsage = true
 	app.SilenceErrors = true
-	app.CompletionOptions = cobra.CompletionOptions{DisableDefaultCmd: true}
 
 	// Global flags
-	globalCmd := cmdGlobal{cmd: app, asker: cli.NewAsker(bufio.NewReader(os.Stdin))}
+	globalCmd := cmdGlobal{cmd: app, asker: cli.NewAsker(bufio.NewReader(os.Stdin), nil)}
 	app.PersistentFlags().BoolVar(&globalCmd.flagVersion, "version", false, i18n.G("Print version number"))
 	app.PersistentFlags().BoolVarP(&globalCmd.flagHelp, "help", "h", false, i18n.G("Print help"))
 	app.PersistentFlags().BoolVar(&globalCmd.flagForceLocal, "force-local", false, i18n.G("Force using the local unix socket"))
@@ -99,6 +99,15 @@ For help with any of those, simply call them with --help.`))
 	app.PersistentFlags().BoolVarP(&globalCmd.flagQuiet, "quiet", "q", false, i18n.G("Don't show progress information"))
 	app.PersistentFlags().BoolVar(&globalCmd.flagSubCmds, "sub-commands", false, i18n.G("Use with help or --help to view sub-commands"))
 
+	_ = app.RegisterFlagCompletionFunc("project", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		projects, directives := globalCmd.cmpProjects(toComplete)
+		if projects != nil {
+			return projects, directives
+		}
+
+		return nil, cobra.ShellCompDirectiveError
+	})
+
 	// Wrappers
 	app.PersistentPreRunE = globalCmd.PreRun
 	app.PersistentPostRunE = globalCmd.PostRun
@@ -107,152 +116,152 @@ For help with any of those, simply call them with --help.`))
 	app.SetVersionTemplate("{{.Version}}\n")
 	app.Version = version.Version
 	if version.IsLTSVersion {
-		app.Version = fmt.Sprintf("%s LTS", version.Version)
+		app.Version = version.Version + " LTS"
 	}
 
 	// alias sub-command
 	aliasCmd := cmdAlias{global: &globalCmd}
-	app.AddCommand(aliasCmd.Command())
+	app.AddCommand(aliasCmd.command())
 
 	// cluster sub-command
 	clusterCmd := cmdCluster{global: &globalCmd}
-	app.AddCommand(clusterCmd.Command())
+	app.AddCommand(clusterCmd.command())
 
 	// config sub-command
 	configCmd := cmdConfig{global: &globalCmd}
-	app.AddCommand(configCmd.Command())
+	app.AddCommand(configCmd.command())
 
 	// console sub-command
 	consoleCmd := cmdConsole{global: &globalCmd}
-	app.AddCommand(consoleCmd.Command())
+	app.AddCommand(consoleCmd.command())
 
 	// copy sub-command
 	copyCmd := cmdCopy{global: &globalCmd}
-	app.AddCommand(copyCmd.Command())
+	app.AddCommand(copyCmd.command())
 
 	// delete sub-command
 	deleteCmd := cmdDelete{global: &globalCmd}
-	app.AddCommand(deleteCmd.Command())
+	app.AddCommand(deleteCmd.command())
 
 	// exec sub-command
 	execCmd := cmdExec{global: &globalCmd}
-	app.AddCommand(execCmd.Command())
+	app.AddCommand(execCmd.command())
 
 	// export sub-command
 	exportCmd := cmdExport{global: &globalCmd}
-	app.AddCommand(exportCmd.Command())
+	app.AddCommand(exportCmd.command())
 
 	// file sub-command
 	fileCmd := cmdFile{global: &globalCmd}
-	app.AddCommand(fileCmd.Command())
+	app.AddCommand(fileCmd.command())
 
 	// import sub-command
 	importCmd := cmdImport{global: &globalCmd}
-	app.AddCommand(importCmd.Command())
+	app.AddCommand(importCmd.command())
 
 	// info sub-command
 	infoCmd := cmdInfo{global: &globalCmd}
-	app.AddCommand(infoCmd.Command())
+	app.AddCommand(infoCmd.command())
 
 	// image sub-command
 	imageCmd := cmdImage{global: &globalCmd}
-	app.AddCommand(imageCmd.Command())
+	app.AddCommand(imageCmd.command())
 
 	// init sub-command
 	initCmd := cmdInit{global: &globalCmd}
-	app.AddCommand(initCmd.Command())
+	app.AddCommand(initCmd.command())
 
 	// launch sub-command
 	launchCmd := cmdLaunch{global: &globalCmd, init: &initCmd}
-	app.AddCommand(launchCmd.Command())
+	app.AddCommand(launchCmd.command())
 
 	// list sub-command
 	listCmd := cmdList{global: &globalCmd}
-	app.AddCommand(listCmd.Command())
+	app.AddCommand(listCmd.command())
 
 	// manpage sub-command
 	manpageCmd := cmdManpage{global: &globalCmd}
-	app.AddCommand(manpageCmd.Command())
+	app.AddCommand(manpageCmd.command())
 
 	// monitor sub-command
 	monitorCmd := cmdMonitor{global: &globalCmd}
-	app.AddCommand(monitorCmd.Command())
+	app.AddCommand(monitorCmd.command())
 
 	// move sub-command
 	moveCmd := cmdMove{global: &globalCmd}
-	app.AddCommand(moveCmd.Command())
+	app.AddCommand(moveCmd.command())
 
 	// network sub-command
 	networkCmd := cmdNetwork{global: &globalCmd}
-	app.AddCommand(networkCmd.Command())
+	app.AddCommand(networkCmd.command())
 
 	// operation sub-command
 	operationCmd := cmdOperation{global: &globalCmd}
-	app.AddCommand(operationCmd.Command())
+	app.AddCommand(operationCmd.command())
 
 	// pause sub-command
 	pauseCmd := cmdPause{global: &globalCmd}
-	app.AddCommand(pauseCmd.Command())
+	app.AddCommand(pauseCmd.command())
 
 	// publish sub-command
 	publishCmd := cmdPublish{global: &globalCmd}
-	app.AddCommand(publishCmd.Command())
+	app.AddCommand(publishCmd.command())
 
 	// profile sub-command
 	profileCmd := cmdProfile{global: &globalCmd}
-	app.AddCommand(profileCmd.Command())
+	app.AddCommand(profileCmd.command())
 
 	// project sub-command
 	projectCmd := cmdProject{global: &globalCmd}
-	app.AddCommand(projectCmd.Command())
+	app.AddCommand(projectCmd.command())
 
 	// query sub-command
 	queryCmd := cmdQuery{global: &globalCmd}
-	app.AddCommand(queryCmd.Command())
+	app.AddCommand(queryCmd.command())
 
 	// rebuild sub-command
 	rebuildCmd := cmdRebuild{global: &globalCmd}
-	app.AddCommand(rebuildCmd.Command())
+	app.AddCommand(rebuildCmd.command())
 
 	// rename sub-command
 	renameCmd := cmdRename{global: &globalCmd}
-	app.AddCommand(renameCmd.Command())
+	app.AddCommand(renameCmd.command())
 
 	// restart sub-command
 	restartCmd := cmdRestart{global: &globalCmd}
-	app.AddCommand(restartCmd.Command())
+	app.AddCommand(restartCmd.command())
 
 	// remote sub-command
 	remoteCmd := cmdRemote{global: &globalCmd}
-	app.AddCommand(remoteCmd.Command())
+	app.AddCommand(remoteCmd.command())
 
 	// restore sub-command
 	restoreCmd := cmdRestore{global: &globalCmd}
-	app.AddCommand(restoreCmd.Command())
+	app.AddCommand(restoreCmd.command())
 
 	// snapshot sub-command
 	snapshotCmd := cmdSnapshot{global: &globalCmd}
-	app.AddCommand(snapshotCmd.Command())
+	app.AddCommand(snapshotCmd.command())
 
 	// storage sub-command
 	storageCmd := cmdStorage{global: &globalCmd}
-	app.AddCommand(storageCmd.Command())
+	app.AddCommand(storageCmd.command())
 
 	// start sub-command
 	startCmd := cmdStart{global: &globalCmd}
-	app.AddCommand(startCmd.Command())
+	app.AddCommand(startCmd.command())
 
 	// stop sub-command
 	stopCmd := cmdStop{global: &globalCmd}
-	app.AddCommand(stopCmd.Command())
+	app.AddCommand(stopCmd.command())
 
 	// version sub-command
 	versionCmd := cmdVersion{global: &globalCmd}
-	app.AddCommand(versionCmd.Command())
+	app.AddCommand(versionCmd.command())
 
 	// warning sub-command
 	warningCmd := cmdWarning{global: &globalCmd}
-	app.AddCommand(warningCmd.Command())
+	app.AddCommand(warningCmd.command())
 
 	authCmd := cmdAuth{global: &globalCmd}
 	app.AddCommand(authCmd.command())
@@ -277,9 +286,14 @@ For help with any of those, simply call them with --help.`))
 		if globalCmd.flagHelpAll {
 			// Show all commands
 			for _, cmd := range app.Commands() {
+				if cmd.Name() == "completion" {
+					continue
+				}
+
 				cmd.Hidden = false
 			}
 		}
+
 		if globalCmd.flagSubCmds {
 			app.SetUsageTemplate(usageTemplateSubCmds())
 		}
@@ -290,11 +304,12 @@ For help with any of those, simply call them with --help.`))
 	if err != nil {
 		// Handle non-Linux systems
 		if err == config.ErrNotLinux {
-			fmt.Fprintf(os.Stderr, i18n.G(`This client hasn't been configured to use a remote LXD server yet.
+			msg := i18n.G(`This client hasn't been configured to use a remote LXD server yet.
 As your platform can't run native Linux instances, you must connect to a remote LXD server.
 
 If you already added a remote server, make it the default with "lxc remote switch NAME".
-To easily setup a local LXD server in a virtual machine, consider using: https://multipass.run`)+"\n")
+To easily setup a local LXD server in a virtual machine, consider using: https://multipass.run`)
+			fmt.Fprintln(os.Stderr, msg)
 			os.Exit(1)
 		}
 
@@ -358,55 +373,60 @@ func (c *cmdGlobal) PreRun(cmd *cobra.Command, args []string) error {
 
 	// Setup password helper
 	c.conf.PromptPassword = func(filename string) (string, error) {
-		return cli.AskPasswordOnce(fmt.Sprintf(i18n.G("Password for %s: "), filename)), nil
+		return c.asker.AskPasswordOnce(fmt.Sprintf(i18n.G("Password for %s: "), filename)), nil
 	}
 
 	// If the user is running a command that may attempt to connect to the local daemon
 	// and this is the first time the client has been run by the user, then check to see
 	// if LXD has been properly configured.  Don't display the message if the var path
 	// does not exist (LXD not installed), as the user may be targeting a remote daemon.
-	if !c.flagForceLocal && shared.PathExists(shared.VarPath("")) && !shared.PathExists(c.confPath) {
+	if !c.flagForceLocal && !shared.PathExists(c.confPath) {
 		// Create the config dir so that we don't get in here again for this user.
 		err = os.MkdirAll(c.conf.ConfigDir, 0750)
 		if err != nil {
 			return err
 		}
 
-		// Attempt to connect to the local server
-		runInit := true
-		d, err := lxd.ConnectLXDUnix("", nil)
-		if err == nil {
-			// Check if server is initialized.
-			info, _, err := d.GetServer()
-			if err == nil && info.Environment.Storage != "" {
-				runInit = false
-			}
-
-			// Detect usable project.
-			names, err := d.GetProjectNames()
+		// Handle local servers.
+		if shared.PathExists(shared.VarPath("")) {
+			// Attempt to connect to the local server
+			runInit := true
+			d, err := lxd.ConnectLXDUnix("", nil)
 			if err == nil {
-				if len(names) == 1 && names[0] != "default" {
-					remote := c.conf.Remotes["local"]
-					remote.Project = names[0]
-					c.conf.Remotes["local"] = remote
+				// Check if server is initialized.
+				info, _, err := d.GetServer()
+				if err == nil && info.Environment.Storage != "" {
+					runInit = false
+				}
+
+				// Detect usable project.
+				names, err := d.GetProjectNames()
+				if err == nil {
+					if len(names) == 1 && names[0] != "default" {
+						remote := c.conf.Remotes["local"]
+						remote.Project = names[0]
+						c.conf.Remotes["local"] = remote
+					}
 				}
 			}
-		}
 
-		flush := false
-		if runInit {
-			fmt.Fprintf(os.Stderr, i18n.G("If this is your first time running LXD on this machine, you should also run: lxd init")+"\n")
-			flush = true
-		}
+			flush := false
+			if runInit {
+				msg := i18n.G("If this is your first time running LXD on this machine, you should also run: lxd init")
+				fmt.Fprintln(os.Stderr, msg)
+				flush = true
+			}
 
-		if !shared.ValueInSlice(cmd.Name(), []string{"init", "launch"}) {
-			fmt.Fprintf(os.Stderr, i18n.G(`To start your first container, try: lxc launch ubuntu:22.04
-Or for a virtual machine: lxc launch ubuntu:22.04 --vm`)+"\n")
-			flush = true
-		}
+			if !shared.ValueInSlice(cmd.Name(), []string{"init", "launch"}) {
+				msg := i18n.G(`To start your first container, try: lxc launch ubuntu:24.04
+Or for a virtual machine: lxc launch ubuntu:24.04 --vm`)
+				fmt.Fprintln(os.Stderr, msg)
+				flush = true
+			}
 
-		if flush {
-			fmt.Fprintf(os.Stderr, "\n")
+			if flush {
+				fmt.Fprintf(os.Stderr, "\n")
+			}
 		}
 
 		// And save the initial configuration
@@ -495,7 +515,8 @@ func (c *cmdGlobal) CheckArgs(cmd *cobra.Command, args []string, minArgs int, ma
 			return true, nil
 		}
 
-		return true, fmt.Errorf(i18n.G("Invalid number of arguments"))
+		msg := i18n.G("Invalid number of arguments")
+		return true, errors.New(msg)
 	}
 
 	return false, nil
