@@ -4,9 +4,9 @@ import (
 	"github.com/spf13/cobra"
 	"github.com/spf13/cobra/doc"
 
-	"github.com/lxc/lxd/shared"
-	cli "github.com/lxc/lxd/shared/cmd"
-	"github.com/lxc/lxd/shared/i18n"
+	"github.com/canonical/lxd/shared"
+	cli "github.com/canonical/lxd/shared/cmd"
+	"github.com/canonical/lxd/shared/i18n"
 )
 
 type cmdManpage struct {
@@ -15,7 +15,7 @@ type cmdManpage struct {
 	flagFormat string
 }
 
-func (c *cmdManpage) Command() *cobra.Command {
+func (c *cmdManpage) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = usage("manpage", i18n.G("<target>"))
 	cmd.Short = i18n.G("Generate manpages for all commands")
@@ -24,18 +24,27 @@ func (c *cmdManpage) Command() *cobra.Command {
 	cmd.Hidden = true
 	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "man", i18n.G("Format (man|md|rest|yaml)")+"``")
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
 
 	return cmd
 }
 
-func (c *cmdManpage) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdManpage) run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	exit, err := c.global.CheckArgs(cmd, args, 1, 1)
 	if exit {
 		return err
 	}
 
+	// If asked to do all commands, mark them all visible.
+	for _, c := range c.global.cmd.Commands() {
+		if c.Name() == "completion" {
+			continue
+		}
+
+		c.Hidden = false
+	}
+
 	// Generate the documentation.
 	switch c.flagFormat {
 	case "man":
