@@ -2,6 +2,7 @@ package main
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"os"
 
@@ -26,7 +27,7 @@ type cmdMonitor struct {
 	flagFormat      string
 }
 
-func (c *cmdMonitor) Command() *cobra.Command {
+func (c *cmdMonitor) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = usage("monitor", i18n.G("[<remote>:]"))
 	cmd.Short = i18n.G("Monitor a local or remote LXD server")
@@ -44,7 +45,7 @@ lxc monitor --pretty --type=logging --loglevel=info
 lxc monitor --type=lifecycle
     Only show lifecycle events.`))
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
 	cmd.Flags().BoolVar(&c.flagPretty, "pretty", false, i18n.G("Pretty rendering (short for --format=pretty)"))
 	cmd.Flags().BoolVar(&c.flagAllProjects, "all-projects", false, i18n.G("Show events from all projects"))
 	cmd.Flags().StringArrayVar(&c.flagType, "type", nil, i18n.G("Event type to listen for")+"``")
@@ -54,7 +55,7 @@ lxc monitor --type=lifecycle
 	return cmd
 }
 
-func (c *cmdMonitor) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdMonitor) run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
 	var err error
@@ -76,7 +77,7 @@ func (c *cmdMonitor) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	if c.flagFormat != "pretty" && c.flagLogLevel != "" {
-		return fmt.Errorf(i18n.G("Log level filtering can only be used with pretty formatting"))
+		return errors.New(i18n.G("Log level filtering can only be used with pretty formatting"))
 	}
 
 	// Connect to the event source.
@@ -217,9 +218,9 @@ func (c *cmdMonitor) unpackCtx(ctx []any) logrus.Fields {
 	var key string
 	for _, entry := range ctx {
 		if key == "" {
-			key = fmt.Sprintf("%v", entry)
+			key = fmt.Sprint(entry)
 		} else {
-			out[key] = fmt.Sprintf("%v", entry)
+			out[key] = fmt.Sprint(entry)
 			key = ""
 		}
 	}
