@@ -1,6 +1,7 @@
 package main
 
 import (
+	"errors"
 	"fmt"
 	"strings"
 
@@ -29,7 +30,7 @@ type cmdMove struct {
 	flagAllowInconsistent bool
 }
 
-func (c *cmdMove) Command() *cobra.Command {
+func (c *cmdMove) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = usage("move", i18n.G("[<remote>:]<instance>[/<snapshot>] [<remote>:][<instance>[/<snapshot>]]"))
 	cmd.Aliases = []string{"mv"}
@@ -54,23 +55,40 @@ lxc move <old name> <new name> [--instance-only]
 lxc move <instance>/<old snapshot name> <instance>/<new snapshot name>
     Rename a snapshot.`))
 
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
 	cmd.Flags().StringArrayVarP(&c.flagConfig, "config", "c", nil, i18n.G("Config key/value to apply to the target instance")+"``")
 	cmd.Flags().StringArrayVarP(&c.flagDevice, "device", "d", nil, i18n.G("New key/value to apply to a specific device")+"``")
 	cmd.Flags().StringArrayVarP(&c.flagProfile, "profile", "p", nil, i18n.G("Profile to apply to the target instance")+"``")
 	cmd.Flags().BoolVar(&c.flagNoProfiles, "no-profiles", false, i18n.G("Unset all profiles on the target instance"))
 	cmd.Flags().BoolVar(&c.flagInstanceOnly, "instance-only", false, i18n.G("Move the instance without its snapshots"))
 	cmd.Flags().StringVar(&c.flagMode, "mode", moveDefaultMode, i18n.G("Transfer mode. One of pull, push or relay.")+"``")
-	cmd.Flags().BoolVar(&c.flagStateless, "stateless", false, i18n.G("Copy a stateful instance stateless"))
+	cmd.Flags().BoolVar(&c.flagStateless, "stateless", false, i18n.G("Copy a stateful instance as stateless"))
 	cmd.Flags().StringVarP(&c.flagStorage, "storage", "s", "", i18n.G("Storage pool name")+"``")
 	cmd.Flags().StringVar(&c.flagTarget, "target", "", i18n.G("Cluster member name")+"``")
 	cmd.Flags().StringVar(&c.flagTargetProject, "target-project", "", i18n.G("Copy to a project different from the source")+"``")
 	cmd.Flags().BoolVar(&c.flagAllowInconsistent, "allow-inconsistent", false, i18n.G("Ignore copy errors for volatile files"))
 
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		if len(args) == 0 {
+			instances, directives := c.global.cmpInstances(toComplete)
+			return instances, directives | cobra.ShellCompDirectiveNoSpace
+		}
+
+		if len(args) == 1 {
+			return c.global.cmpRemotes(toComplete, false)
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
+	_ = cmd.RegisterFlagCompletionFunc("mode", func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+		return []string{"pull", "push", "relay"}, cobra.ShellCompDirectiveNoFileComp
+	})
+
 	return cmd
 }
 
-func (c *cmdMove) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdMove) run(cmd *cobra.Command, args []string) error {
 	conf := c.global.conf
 
 	// Quick checks.
@@ -114,7 +132,7 @@ func (c *cmdMove) Run(cmd *cobra.Command, args []string) error {
 	// simply won't work).
 	if sourceRemote == destRemote && c.flagTarget == "" && c.flagStorage == "" && c.flagTargetProject == "" {
 		if c.flagConfig != nil || c.flagDevice != nil || c.flagProfile != nil || c.flagNoProfiles {
-			return fmt.Errorf(i18n.G("Can't override configuration or profiles in local rename"))
+			return errors.New(i18n.G("Can't override configuration or profiles in local rename"))
 		}
 
 		source, err := conf.GetInstanceServer(sourceRemote)
@@ -128,11 +146,11 @@ func (c *cmdMove) Run(cmd *cobra.Command, args []string) error {
 			dstParent, dstSnap, dstIsSnap := api.GetParentAndSnapshotName(destName)
 
 			if srcParent != dstParent {
-				return fmt.Errorf(i18n.G("Invalid new snapshot name, parent must be the same as source"))
+				return errors.New(i18n.G("Invalid new snapshot name, parent must be the same as source"))
 			}
 
 			if !dstIsSnap {
-				return fmt.Errorf(i18n.G("Invalid new snapshot name"))
+				return errors.New(i18n.G("Invalid new snapshot name"))
 			}
 
 			op, err := source.RenameInstanceSnapshot(srcParent, srcSnap, api.InstanceSnapshotPost{Name: dstSnap})
@@ -165,19 +183,19 @@ func (c *cmdMove) Run(cmd *cobra.Command, args []string) error {
 		// cluster member to another, let's use the dedicated API.
 		if sourceRemote == destRemote {
 			if c.flagInstanceOnly {
-				return fmt.Errorf(i18n.G("The --instance-only flag can't be used with --target"))
+				return errors.New(i18n.G("The --instance-only flag can't be used with --target"))
 			}
 
 			if c.flagStorage != "" {
-				return fmt.Errorf(i18n.G("The --storage flag can't be used with --target"))
+				return errors.New(i18n.G("The --storage flag can't be used with --target"))
 			}
 
 			if c.flagTargetProject != "" {
-				return fmt.Errorf(i18n.G("The --target-project flag can't be used with --target"))
+				return errors.New(i18n.G("The --target-project flag can't be used with --target"))
 			}
 
 			if c.flagMode != moveDefaultMode {
-				return fmt.Errorf(i18n.G("The --mode flag can't be used with --target"))
+				return errors.New(i18n.G("The --mode flag can't be used with --target"))
 			}
 
 			return moveClusterInstance(conf, sourceResource, destResource, c.flagTarget, c.global.flagQuiet, stateful)
@@ -189,7 +207,7 @@ func (c *cmdMove) Run(cmd *cobra.Command, args []string) error {
 		}
 
 		if !dest.IsClustered() {
-			return fmt.Errorf(i18n.G("The destination LXD server is not clustered"))
+			return errors.New(i18n.G("The destination LXD server is not clustered"))
 		}
 	}
 
@@ -203,7 +221,7 @@ func (c *cmdMove) Run(cmd *cobra.Command, args []string) error {
 
 		if source.HasExtension("instance_move_config") {
 			if c.flagMode != moveDefaultMode {
-				return fmt.Errorf(i18n.G("The --mode flag can't be used with --storage or --target-project"))
+				return errors.New(i18n.G("The --mode flag can't be used with --storage or --target-project"))
 			}
 
 			// Evaluate provided profiles. If no profiles were provided and flag noProfiles is false,
@@ -224,7 +242,7 @@ func (c *cmdMove) Run(cmd *cobra.Command, args []string) error {
 			}
 
 			if c.flagMode != moveDefaultMode {
-				return fmt.Errorf(i18n.G("The --mode flag can't be used with --storage or --target-project"))
+				return errors.New(i18n.G("The --mode flag can't be used with --storage or --target-project"))
 			}
 
 			return moveInstance(conf, sourceResource, destResource, c.flagStorage, c.flagTargetProject, []string{}, []string{}, nil, c.flagInstanceOnly, stateful)
@@ -253,7 +271,7 @@ func (c *cmdMove) Run(cmd *cobra.Command, args []string) error {
 	del := cmdDelete{global: c.global}
 	del.flagForce = true
 	del.flagForceProtected = true
-	err = del.Run(cmd, args[:1])
+	err = del.run(cmd, args[:1])
 	if err != nil {
 		return fmt.Errorf("Failed to delete original instance after copying it: %w", err)
 	}
@@ -277,7 +295,7 @@ func moveClusterInstance(conf *config.Config, sourceResource string, destResourc
 
 	// Make sure we have an instance or snapshot name.
 	if sourceName == "" {
-		return fmt.Errorf(i18n.G("You must specify a source instance name"))
+		return errors.New(i18n.G("You must specify a source instance name"))
 	}
 
 	// The destination name is optional.
@@ -293,7 +311,7 @@ func moveClusterInstance(conf *config.Config, sourceResource string, destResourc
 
 	// Check that it's a cluster
 	if !source.IsClustered() {
-		return fmt.Errorf(i18n.G("The source LXD server is not clustered"))
+		return errors.New(i18n.G("The source LXD server is not clustered"))
 	}
 
 	// The migrate API will do the right thing when passed a target.
@@ -353,7 +371,7 @@ func moveInstance(conf *config.Config, sourceResource string, destResource strin
 
 	// Make sure we have an instance or snapshot name.
 	if sourceName == "" {
-		return fmt.Errorf(i18n.G("You must specify a source instance name"))
+		return errors.New(i18n.G("You must specify a source instance name"))
 	}
 
 	// The destination name is optional.
