@@ -4,11 +4,9 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"maps"
 	"os"
 	"reflect"
 	"sort"
-	"strconv"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -86,8 +84,7 @@ type cmdNetworkACLList struct {
 	global     *cmdGlobal
 	networkACL *cmdNetworkACL
 
-	flagFormat      string
-	flagAllProjects bool
+	flagFormat string
 }
 
 func (c *cmdNetworkACLList) command() *cobra.Command {
@@ -99,11 +96,10 @@ func (c *cmdNetworkACLList) command() *cobra.Command {
 
 	cmd.RunE = c.run
 	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "table", i18n.G("Format (csv|json|table|yaml|compact)")+"``")
-	cmd.Flags().BoolVar(&c.flagAllProjects, "all-projects", false, i18n.G("Display network ACLs from all projects"))
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
+			return c.global.cmpRemotes(toComplete, false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -137,32 +133,20 @@ func (c *cmdNetworkACLList) run(cmd *cobra.Command, args []string) error {
 		return errors.New(i18n.G("Filtering isn't supported yet"))
 	}
 
-	var acls []api.NetworkACL
-	if c.flagAllProjects {
-		acls, err = resource.server.GetNetworkACLsAllProjects()
-		if err != nil {
-			return err
-		}
-	} else {
-		acls, err = resource.server.GetNetworkACLs()
-		if err != nil {
-			return err
-		}
+	acls, err := resource.server.GetNetworkACLs()
+	if err != nil {
+		return err
 	}
 
 	data := [][]string{}
 	for _, acl := range acls {
-		strUsedBy := strconv.Itoa(len(acl.UsedBy))
+		strUsedBy := fmt.Sprint(len(acl.UsedBy))
 		details := []string{
 			acl.Name,
 			acl.Description,
 			strUsedBy,
 		}
 
-		if c.flagAllProjects {
-			details = append([]string{acl.Project}, details...)
-		}
-
 		data = append(data, details)
 	}
 
@@ -174,10 +158,6 @@ func (c *cmdNetworkACLList) run(cmd *cobra.Command, args []string) error {
 		i18n.G("USED BY"),
 	}
 
-	if c.flagAllProjects {
-		header = append([]string{i18n.G("PROJECT")}, header...)
-	}
-
 	return cli.RenderTable(c.flagFormat, header, data, acls)
 }
 
@@ -196,7 +176,7 @@ func (c *cmdNetworkACLShow) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_acl", toComplete)
+			return c.global.cmpNetworkACLs(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -257,7 +237,7 @@ func (c *cmdNetworkACLShowLog) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_acl", toComplete)
+			return c.global.cmpNetworkACLs(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -315,7 +295,7 @@ func (c *cmdNetworkACLGet) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_acl", toComplete)
+			return c.global.cmpNetworkACLs(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -354,7 +334,7 @@ func (c *cmdNetworkACLGet) run(cmd *cobra.Command, args []string) error {
 
 	if c.flagIsProperty {
 		w := resp.Writable()
-		res, err := getFieldByJSONTag(&w, args[1])
+		res, err := getFieldByJsonTag(&w, args[1])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the network ACL %q: %v"), args[1], resource.name, err)
 		}
@@ -391,7 +371,7 @@ lxc network acl create a1 < config.yaml
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_acl", toComplete)
+			return c.global.cmpNetworkACLs(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -489,7 +469,7 @@ For backward compatibility, a single configuration key may still be set with:
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_acl", toComplete)
+			return c.global.cmpNetworkACLs(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -533,7 +513,7 @@ func (c *cmdNetworkACLSet) run(cmd *cobra.Command, args []string) error {
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJSONTag(&writable, k)
+				err := unsetFieldByJsonTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -545,7 +525,9 @@ func (c *cmdNetworkACLSet) run(cmd *cobra.Command, args []string) error {
 			}
 		}
 	} else {
-		maps.Copy(writable.Config, keys)
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
 	return resource.server.UpdateNetworkACL(resource.name, writable, etag)
@@ -571,7 +553,7 @@ func (c *cmdNetworkACLUnset) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_acl", toComplete)
+			return c.global.cmpNetworkACLs(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -613,7 +595,7 @@ func (c *cmdNetworkACLEdit) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_acl", toComplete)
+			return c.global.cmpNetworkACLs(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -751,7 +733,7 @@ func (c *cmdNetworkACLRename) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_acl", toComplete)
+			return c.global.cmpNetworkACLs(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -808,7 +790,7 @@ func (c *cmdNetworkACLDelete) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_acl", toComplete)
+			return c.global.cmpNetworkACLs(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -880,7 +862,7 @@ func (c *cmdNetworkACLRule) commandAdd() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_acl", toComplete)
+			return c.global.cmpNetworkACLs(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -903,7 +885,7 @@ func networkACLRuleJSONStructFieldMap() map[string]int {
 	ruleType := reflect.TypeOf(api.NetworkACLRule{})
 	allowedKeys := make(map[string]int, ruleType.NumField())
 
-	for i := range ruleType.NumField() {
+	for i := 0; i < ruleType.NumField(); i++ {
 		field := ruleType.Field(i)
 		if field.PkgPath != "" {
 			continue // Skip unexported fields. It is empty for upper case (exported) field names.
@@ -997,12 +979,11 @@ func (c *cmdNetworkACLRule) runAdd(cmd *cobra.Command, args []string) error {
 	}
 
 	// Add rule to the requested direction (if direction valid).
-	switch args[1] {
-	case "ingress":
+	if args[1] == "ingress" {
 		netACL.Ingress = append(netACL.Ingress, *rule)
-	case "egress":
+	} else if args[1] == "egress" {
 		netACL.Egress = append(netACL.Egress, *rule)
-	default:
+	} else {
 		return errors.New(i18n.G("The direction argument must be one of: ingress, egress"))
 	}
 
@@ -1020,7 +1001,7 @@ func (c *cmdNetworkACLRule) commandRemove() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_acl", toComplete)
+			return c.global.cmpNetworkACLs(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1126,22 +1107,21 @@ func (c *cmdNetworkACLRule) runRemove(cmd *cobra.Command, args []string) error {
 	}
 
 	// Remove matching rule(s) from the requested direction (if direction valid).
-	switch args[1] {
-	case "ingress":
+	if args[1] == "ingress" {
 		rules, err := removeFromRules(netACL.Ingress, filters)
 		if err != nil {
 			return err
 		}
 
 		netACL.Ingress = rules
-	case "egress":
+	} else if args[1] == "egress" {
 		rules, err := removeFromRules(netACL.Egress, filters)
 		if err != nil {
 			return err
 		}
 
 		netACL.Egress = rules
-	default:
+	} else {
 		return errors.New(i18n.G("The direction argument must be one of: ingress, egress"))
 	}
 
