@@ -4,18 +4,15 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"maps"
 	"net"
 	"net/url"
 	"os"
 	"sort"
-	"strconv"
 	"strings"
 
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	"github.com/canonical/lxd/client"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	cli "github.com/canonical/lxd/shared/cmd"
@@ -97,7 +94,7 @@ func (c *cmdNetworkForwardList) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -143,7 +140,7 @@ func (c *cmdNetworkForwardList) run(cmd *cobra.Command, args []string) error {
 			forward.ListenAddress,
 			forward.Description,
 			forward.Config["target_address"],
-			strconv.Itoa(len(forward.Ports)),
+			fmt.Sprint(len(forward.Ports)),
 		}
 
 		if clustered {
@@ -186,7 +183,7 @@ func (c *cmdNetworkForwardShow) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -269,7 +266,7 @@ lxc network forward create n1 127.0.0.1 < config.yaml
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -296,7 +293,7 @@ func (c *cmdNetworkForwardCreate) run(cmd *cobra.Command, args []string) error {
 	}
 
 	transporter, wrapper := newLocationHeaderTransportWrapper()
-	client, err := c.global.conf.GetInstanceServerWithConnectionArgs(remoteName, &lxd.ConnectionArgs{TransportWrapper: wrapper})
+	client, err := c.global.conf.GetInstanceServerWithTransportWrapper(remoteName, wrapper)
 	if err != nil {
 		return err
 	}
@@ -343,7 +340,7 @@ func (c *cmdNetworkForwardCreate) run(cmd *cobra.Command, args []string) error {
 
 	if listenAddress == "" {
 		if c.flagAllocate == "" {
-			return errors.New("Must provide a listen address or --allocate=ipv{4,6}")
+			return fmt.Errorf("Must provide a listen address or --allocate=ipv{4,6}")
 		}
 
 		if c.flagAllocate != "ipv4" && c.flagAllocate != "ipv6" {
@@ -419,7 +416,7 @@ func (c *cmdNetworkForwardGet) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -468,7 +465,7 @@ func (c *cmdNetworkForwardGet) run(cmd *cobra.Command, args []string) error {
 
 	if c.flagIsProperty {
 		w := forward.Writable()
-		res, err := getFieldByJSONTag(&w, args[2])
+		res, err := getFieldByJsonTag(&w, args[2])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the network forward %q: %v"), args[1], resource.name, err)
 		}
@@ -509,7 +506,7 @@ For backward compatibility, a single configuration key may still be set with:
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -572,7 +569,7 @@ func (c *cmdNetworkForwardSet) run(cmd *cobra.Command, args []string) error {
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJSONTag(&writable, k)
+				err := unsetFieldByJsonTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -584,7 +581,9 @@ func (c *cmdNetworkForwardSet) run(cmd *cobra.Command, args []string) error {
 			}
 		}
 	} else {
-		maps.Copy(writable.Config, keys)
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
 	writable.Normalise()
@@ -612,7 +611,7 @@ func (c *cmdNetworkForwardUnset) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -659,7 +658,7 @@ func (c *cmdNetworkForwardEdit) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -813,7 +812,7 @@ func (c *cmdNetworkForwardDelete) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -902,7 +901,7 @@ func (c *cmdNetworkForwardPort) commandAdd() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -988,7 +987,7 @@ func (c *cmdNetworkForwardPort) commandRemove() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
