@@ -4,18 +4,15 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"maps"
 	"net"
 	"net/url"
 	"os"
 	"sort"
-	"strconv"
 	"strings"
 
 	"github.com/spf13/cobra"
 	"gopkg.in/yaml.v2"
 
-	"github.com/canonical/lxd/client"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	cli "github.com/canonical/lxd/shared/cmd"
@@ -101,7 +98,7 @@ func (c *cmdNetworkLoadBalancerList) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -146,7 +143,7 @@ func (c *cmdNetworkLoadBalancerList) run(cmd *cobra.Command, args []string) erro
 		details := []string{
 			loadBalancer.ListenAddress,
 			loadBalancer.Description,
-			strconv.Itoa(len(loadBalancer.Ports)),
+			fmt.Sprint(len(loadBalancer.Ports)),
 		}
 
 		if clustered {
@@ -188,7 +185,7 @@ func (c *cmdNetworkLoadBalancerShow) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -290,7 +287,7 @@ func (c *cmdNetworkLoadBalancerCreate) run(cmd *cobra.Command, args []string) er
 	}
 
 	transporter, wrapper := newLocationHeaderTransportWrapper()
-	client, err := c.global.conf.GetInstanceServerWithConnectionArgs(remoteName, &lxd.ConnectionArgs{TransportWrapper: wrapper})
+	client, err := c.global.conf.GetInstanceServerWithTransportWrapper(remoteName, wrapper)
 	if err != nil {
 		return err
 	}
@@ -339,7 +336,7 @@ func (c *cmdNetworkLoadBalancerCreate) run(cmd *cobra.Command, args []string) er
 
 	if listenAddress == "" {
 		if c.flagAllocate == "" {
-			return errors.New("Must provide a listen address or --allocate=ipv{4,6}")
+			return fmt.Errorf("Must provide a listen address or --allocate=ipv{4,6}")
 		}
 
 		if c.flagAllocate != "ipv4" && c.flagAllocate != "ipv6" {
@@ -447,7 +444,7 @@ func (c *cmdNetworkLoadBalancerGet) run(cmd *cobra.Command, args []string) error
 
 	if c.flagIsProperty {
 		w := loadBalancer.Writable()
-		res, err := getFieldByJSONTag(&w, args[2])
+		res, err := getFieldByJsonTag(&w, args[2])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the load balancer %q: %v"), args[2], resource.name, err)
 		}
@@ -488,7 +485,7 @@ For backward compatibility, a single configuration key may still be set with:
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -551,7 +548,7 @@ func (c *cmdNetworkLoadBalancerSet) run(cmd *cobra.Command, args []string) error
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJSONTag(&writable, k)
+				err := unsetFieldByJsonTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -563,7 +560,9 @@ func (c *cmdNetworkLoadBalancerSet) run(cmd *cobra.Command, args []string) error
 			}
 		}
 	} else {
-		maps.Copy(writable.Config, keys)
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
 	writable.Normalise()
@@ -621,7 +620,7 @@ func (c *cmdNetworkLoadBalancerEdit) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -776,7 +775,7 @@ func (c *cmdNetworkLoadBalancerDelete) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -864,7 +863,7 @@ func (c *cmdNetworkLoadBalancerBackend) commandAdd() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -940,7 +939,7 @@ func (c *cmdNetworkLoadBalancerBackend) commandRemove() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1053,7 +1052,7 @@ func (c *cmdNetworkLoadBalancerPort) commandAdd() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1127,7 +1126,7 @@ func (c *cmdNetworkLoadBalancerPort) commandRemove() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
