@@ -4,7 +4,6 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"maps"
 	"os"
 	"sort"
 	"strings"
@@ -87,7 +86,7 @@ func (c *cmdNetworkPeerList) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -170,7 +169,7 @@ func (c *cmdNetworkPeerShow) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -239,7 +238,7 @@ func (c *cmdNetworkPeerCreate) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -334,12 +333,11 @@ func (c *cmdNetworkPeerCreate) run(cmd *cobra.Command, args []string) error {
 			return fmt.Errorf(i18n.G("Failed getting peer's status: %w"), err)
 		}
 
-		switch createdPeer.Status {
-		case api.NetworkStatusCreated:
+		if createdPeer.Status == api.NetworkStatusCreated {
 			fmt.Printf(i18n.G("Network peer %s created")+"\n", peer.Name)
-		case api.NetworkStatusPending:
+		} else if createdPeer.Status == api.NetworkStatusPending {
 			fmt.Printf(i18n.G("Network peer %s pending (please complete mutual peering on peer network)")+"\n", peer.Name)
-		default:
+		} else {
 			fmt.Printf(i18n.G("Network peer %s is in unexpected state %q")+"\n", peer.Name, createdPeer.Status)
 		}
 	}
@@ -366,7 +364,7 @@ func (c *cmdNetworkPeerGet) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -415,7 +413,7 @@ func (c *cmdNetworkPeerGet) run(cmd *cobra.Command, args []string) error {
 
 	if c.flagIsProperty {
 		w := peer.Writable()
-		res, err := getFieldByJSONTag(&w, args[2])
+		res, err := getFieldByJsonTag(&w, args[2])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the network peer %q: %v"), args[2], resource.name, err)
 		}
@@ -455,7 +453,7 @@ For backward compatibility, a single configuration key may still be set with:
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -513,7 +511,7 @@ func (c *cmdNetworkPeerSet) run(cmd *cobra.Command, args []string) error {
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJSONTag(&writable, k)
+				err := unsetFieldByJsonTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -525,7 +523,9 @@ func (c *cmdNetworkPeerSet) run(cmd *cobra.Command, args []string) error {
 			}
 		}
 	} else {
-		maps.Copy(writable.Config, keys)
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
 	return client.UpdateNetworkPeer(resource.name, peer.Name, writable, etag)
@@ -551,7 +551,7 @@ func (c *cmdNetworkPeerUnset) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -596,7 +596,7 @@ func (c *cmdNetworkPeerEdit) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -733,7 +733,7 @@ func (c *cmdNetworkPeerDelete) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network", toComplete)
+			return c.global.cmpNetworks(toComplete)
 		}
 
 		if len(args) == 1 {
