@@ -4,11 +4,8 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"maps"
 	"os"
-	"slices"
 	"sort"
-	"strconv"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -86,7 +83,7 @@ func (c *cmdNetworkZoneList) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = usage("list", i18n.G("[<remote>:]"))
 	cmd.Aliases = []string{"ls"}
-	cmd.Short = i18n.G("List available network zones")
+	cmd.Short = i18n.G("List available network zoneS")
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G("List available network zone"))
 
 	cmd.RunE = c.run
@@ -95,7 +92,7 @@ func (c *cmdNetworkZoneList) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
+			return c.global.cmpRemotes(toComplete, false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -144,7 +141,7 @@ func (c *cmdNetworkZoneList) run(cmd *cobra.Command, args []string) error {
 
 	data := [][]string{}
 	for _, zone := range zones {
-		strUsedBy := strconv.Itoa(len(zone.UsedBy))
+		strUsedBy := fmt.Sprint(len(zone.UsedBy))
 		details := []string{
 			zone.Name,
 			zone.Description,
@@ -188,7 +185,7 @@ func (c *cmdNetworkZoneShow) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -253,7 +250,7 @@ func (c *cmdNetworkZoneGet) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -292,7 +289,7 @@ func (c *cmdNetworkZoneGet) run(cmd *cobra.Command, args []string) error {
 
 	if c.flagIsProperty {
 		w := resp.Writable()
-		res, err := getFieldByJSONTag(&w, args[1])
+		res, err := getFieldByJsonTag(&w, args[1])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the network zone %q: %v"), args[1], resource.name, err)
 		}
@@ -329,7 +326,7 @@ lxc network zone create z1 < config.yaml
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -425,7 +422,7 @@ For backward compatibility, a single configuration key may still be set with:
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -469,7 +466,7 @@ func (c *cmdNetworkZoneSet) run(cmd *cobra.Command, args []string) error {
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJSONTag(&writable, k)
+				err := unsetFieldByJsonTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -481,7 +478,9 @@ func (c *cmdNetworkZoneSet) run(cmd *cobra.Command, args []string) error {
 			}
 		}
 	} else {
-		maps.Copy(writable.Config, keys)
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
 	return resource.server.UpdateNetworkZone(resource.name, writable, etag)
@@ -507,7 +506,7 @@ func (c *cmdNetworkZoneUnset) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -549,7 +548,7 @@ func (c *cmdNetworkZoneEdit) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -675,7 +674,7 @@ func (c *cmdNetworkZoneDelete) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -790,7 +789,7 @@ func (c *cmdNetworkZoneRecordList) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -866,7 +865,7 @@ func (c *cmdNetworkZoneRecordShow) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -932,7 +931,7 @@ func (c *cmdNetworkZoneRecordGet) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -974,7 +973,7 @@ func (c *cmdNetworkZoneRecordGet) run(cmd *cobra.Command, args []string) error {
 
 	if c.flagIsProperty {
 		w := resp.Writable()
-		res, err := getFieldByJSONTag(&w, args[2])
+		res, err := getFieldByJsonTag(&w, args[2])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the network zone record %q: %v"), args[2], resource.name, err)
 		}
@@ -1011,7 +1010,7 @@ lxc network zone record create z1 r1 < config.yaml
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1108,7 +1107,7 @@ func (c *cmdNetworkZoneRecordSet) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1155,7 +1154,7 @@ func (c *cmdNetworkZoneRecordSet) run(cmd *cobra.Command, args []string) error {
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJSONTag(&writable, k)
+				err := unsetFieldByJsonTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -1167,7 +1166,9 @@ func (c *cmdNetworkZoneRecordSet) run(cmd *cobra.Command, args []string) error {
 			}
 		}
 	} else {
-		maps.Copy(writable.Config, keys)
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
 	return resource.server.UpdateNetworkZoneRecord(resource.name, args[1], writable, etag)
@@ -1193,7 +1194,7 @@ func (c *cmdNetworkZoneRecordUnset) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1239,7 +1240,7 @@ func (c *cmdNetworkZoneRecordEdit) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1368,7 +1369,7 @@ func (c *cmdNetworkZoneRecordDelete) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1445,7 +1446,7 @@ func (c *cmdNetworkZoneRecordEntry) commandAdd() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1502,7 +1503,7 @@ func (c *cmdNetworkZoneRecordEntry) commandRemove() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("network_zone", toComplete)
+			return c.global.cmpNetworkZones(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -1546,7 +1547,7 @@ func (c *cmdNetworkZoneRecordEntry) runRemove(cmd *cobra.Command, args []string)
 		}
 
 		found = true
-		netRecord.Entries = slices.Delete(netRecord.Entries, i, i+1)
+		netRecord.Entries = append(netRecord.Entries[:i], netRecord.Entries[i+1:]...)
 	}
 
 	if !found {
