@@ -4,12 +4,9 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"maps"
 	"net/http"
 	"os"
-	"slices"
 	"sort"
-	"strconv"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -96,7 +93,7 @@ func (c *cmdProfile) command() *cobra.Command {
 
 	// Workaround for subcommand usage errors. See: https://github.com/spf13/cobra/issues/706
 	cmd.Args = cobra.NoArgs
-	cmd.Run = func(cmd *cobra.Command, _ []string) { _ = cmd.Usage() }
+	cmd.Run = func(cmd *cobra.Command, args []string) { _ = cmd.Usage() }
 	return cmd
 }
 
@@ -115,21 +112,16 @@ func (c *cmdProfileAdd) command() *cobra.Command {
 
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-		if len(args) > 1 {
-			return nil, cobra.ShellCompDirectiveNoFileComp
-		}
-
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("instance", toComplete)
+			return c.global.cmpInstances(toComplete)
 		}
 
-		remote, _, err := c.global.conf.ParseRemote(args[0])
-		if err != nil {
-			return handleCompletionError(err)
+		if len(args) == 1 {
+			return c.global.cmpProfiles(args[0], false)
 		}
 
-		return c.global.cmpTopLevelResourceInRemote(remote, "profile", toComplete)
+		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
 
 	return cmd
@@ -204,17 +196,12 @@ lxc profile assign foo ''
 
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("instance", toComplete)
+			return c.global.cmpInstances(toComplete)
 		}
 
-		remote, _, err := c.global.conf.ParseRemote(args[0])
-		if err != nil {
-			return handleCompletionError(err)
-		}
-
-		return c.global.cmpTopLevelResourceInRemote(remote, "profile", toComplete)
+		return c.global.cmpProfiles(args[0], false)
 	}
 
 	return cmd
@@ -293,13 +280,13 @@ func (c *cmdProfileCopy) command() *cobra.Command {
 
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("profile", toComplete)
+			return c.global.cmpProfiles(toComplete, true)
 		}
 
 		if len(args) == 1 {
-			return c.global.cmpTopLevelResource("profile", toComplete)
+			return c.global.cmpProfiles(toComplete, true)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -377,9 +364,9 @@ lxc profile create p1 < config.yaml
 
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
+			return c.global.cmpRemotes(toComplete, false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -455,9 +442,9 @@ func (c *cmdProfileDelete) command() *cobra.Command {
 
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("profile", toComplete)
+			return c.global.cmpProfiles(toComplete, true)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -516,9 +503,9 @@ func (c *cmdProfileEdit) command() *cobra.Command {
 
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("profile", toComplete)
+			return c.global.cmpProfiles(toComplete, true)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -651,9 +638,9 @@ func (c *cmdProfileGet) command() *cobra.Command {
 
 	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Get the key as a profile property"))
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("profile", toComplete)
+			return c.global.cmpProfiles(toComplete, true)
 		}
 
 		if len(args) == 1 {
@@ -693,7 +680,7 @@ func (c *cmdProfileGet) run(cmd *cobra.Command, args []string) error {
 
 	if c.flagIsProperty {
 		w := profile.Writable()
-		res, err := getFieldByJSONTag(&w, args[1])
+		res, err := getFieldByJsonTag(&w, args[1])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the profile %q: %v"), args[1], resource.name, err)
 		}
@@ -740,9 +727,9 @@ u - Used By`))
 	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "table", i18n.G("Format (csv|json|table|yaml|compact)")+"``")
 	cmd.Flags().BoolVar(&c.flagAllProjects, "all-projects", false, i18n.G("Display profiles from all projects"))
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
+			return c.global.cmpRemotes(toComplete, false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -805,7 +792,7 @@ func (c *cmdProfileList) projectNameColumnData(profile api.Profile) string {
 }
 
 func (c *cmdProfileList) usedByColumnData(profile api.Profile) string {
-	return strconv.Itoa(len(profile.UsedBy))
+	return fmt.Sprint(len(profile.UsedBy))
 }
 
 func (c *cmdProfileList) run(cmd *cobra.Command, args []string) error {
@@ -882,21 +869,16 @@ func (c *cmdProfileRemove) command() *cobra.Command {
 
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-		if len(args) > 1 {
-			return nil, cobra.ShellCompDirectiveNoFileComp
-		}
-
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("instance", toComplete)
+			return c.global.cmpInstances(toComplete)
 		}
 
-		remote, _, err := c.global.conf.ParseRemote(args[0])
-		if err != nil {
-			return handleCompletionError(err)
+		if len(args) == 1 {
+			return c.global.cmpProfiles(args[0], false)
 		}
 
-		return c.global.cmpTopLevelResourceInRemote(remote, "profile", toComplete)
+		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
 
 	return cmd
@@ -927,7 +909,7 @@ func (c *cmdProfileRemove) run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	if !slices.Contains(inst.Profiles, args[1]) {
+	if !shared.ValueInSlice(args[1], inst.Profiles) {
 		return fmt.Errorf(i18n.G("Profile %s isn't currently applied to %s"), args[1], resource.name)
 	}
 
@@ -975,9 +957,9 @@ func (c *cmdProfileRename) command() *cobra.Command {
 
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("profile", toComplete)
+			return c.global.cmpProfiles(toComplete, true)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -1039,16 +1021,16 @@ For backward compatibility, a single configuration key may still be set with:
 	cmd.RunE = c.run
 	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Set the key as a profile property"))
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
-		if len(args) > 1 {
-			return nil, cobra.ShellCompDirectiveNoFileComp
-		}
-
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("profile", toComplete)
+			return c.global.cmpProfiles(toComplete, true)
 		}
 
-		return c.global.cmpInstanceKeysByType(api.InstanceTypeAny, "=", toComplete)
+		if len(args) == 1 {
+			return c.global.cmpInstanceAllKeys(args[0])
+		}
+
+		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
 
 	return cmd
@@ -1089,7 +1071,7 @@ func (c *cmdProfileSet) run(cmd *cobra.Command, args []string) error {
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJSONTag(&writable, k)
+				err := unsetFieldByJsonTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -1101,7 +1083,9 @@ func (c *cmdProfileSet) run(cmd *cobra.Command, args []string) error {
 			}
 		}
 	} else {
-		maps.Copy(writable.Config, keys)
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
 	return resource.server.UpdateProfile(resource.name, writable, etag)
@@ -1122,9 +1106,9 @@ func (c *cmdProfileShow) command() *cobra.Command {
 
 	cmd.RunE = c.run
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("profile", toComplete)
+			return c.global.cmpProfiles(toComplete, true)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -1187,9 +1171,9 @@ func (c *cmdProfileUnset) command() *cobra.Command {
 	cmd.RunE = c.run
 	cmd.Flags().BoolVarP(&c.flagIsProperty, "property", "p", false, i18n.G("Unset the key as a profile property"))
 
-	cmd.ValidArgsFunction = func(_ *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("profile", toComplete)
+			return c.global.cmpProfiles(toComplete, true)
 		}
 
 		if len(args) == 1 {
