@@ -4,11 +4,9 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"maps"
 	"os"
 	"slices"
 	"sort"
-	"strconv"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -111,7 +109,7 @@ lxc project create p1 < config.yaml
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
+			return c.global.cmpRemotes(toComplete, false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -203,7 +201,7 @@ func (c *cmdProjectDelete) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("project", toComplete)
+			return c.global.cmpProjects(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -277,7 +275,7 @@ func (c *cmdProjectEdit) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("project", toComplete)
+			return c.global.cmpProjects(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -411,7 +409,7 @@ func (c *cmdProjectGet) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("project", toComplete)
+			return c.global.cmpProjects(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -451,7 +449,7 @@ func (c *cmdProjectGet) run(cmd *cobra.Command, args []string) error {
 
 	if c.flagIsProperty {
 		w := project.Writable()
-		res, err := getFieldByJSONTag(&w, args[1])
+		res, err := getFieldByJsonTag(&w, args[1])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the project %q: %v"), args[1], resource.name, err)
 		}
@@ -485,7 +483,7 @@ func (c *cmdProjectList) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
+			return c.global.cmpRemotes(toComplete, false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -565,7 +563,7 @@ func (c *cmdProjectList) run(cmd *cobra.Command, args []string) error {
 			name = name + " (" + i18n.G("current") + ")"
 		}
 
-		strUsedBy := strconv.Itoa(len(project.UsedBy))
+		strUsedBy := fmt.Sprint(len(project.UsedBy))
 		data = append(data, []string{name, images, profiles, storageVolumes, storageBuckets, networks, networkZones, project.Description, strUsedBy})
 	}
 
@@ -604,7 +602,7 @@ func (c *cmdProjectRename) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("project", toComplete)
+			return c.global.cmpProjects(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -673,7 +671,7 @@ For backward compatibility, a single configuration key may still be set with:
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("project", toComplete)
+			return c.global.cmpProjects(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -717,7 +715,7 @@ func (c *cmdProjectSet) run(cmd *cobra.Command, args []string) error {
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJSONTag(&writable, k)
+				err := unsetFieldByJsonTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -729,7 +727,9 @@ func (c *cmdProjectSet) run(cmd *cobra.Command, args []string) error {
 			}
 		}
 	} else {
-		maps.Copy(writable.Config, keys)
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
 	return resource.server.UpdateProject(resource.name, writable, etag)
@@ -756,7 +756,7 @@ func (c *cmdProjectUnset) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("project", toComplete)
+			return c.global.cmpProjects(toComplete)
 		}
 
 		if len(args) == 1 {
@@ -799,7 +799,7 @@ func (c *cmdProjectShow) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("project", toComplete)
+			return c.global.cmpProjects(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -860,7 +860,7 @@ func (c *cmdProjectSwitch) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("project", toComplete)
+			return c.global.cmpProjects(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -928,7 +928,7 @@ func (c *cmdProjectInfo) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpTopLevelResource("project", toComplete)
+			return c.global.cmpProjects(toComplete)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -973,7 +973,7 @@ func (c *cmdProjectInfo) run(cmd *cobra.Command, args []string) error {
 			if slices.Contains(byteLimits, shortKey) {
 				limit = units.GetByteSizeStringIEC(v.Limit, 2)
 			} else {
-				limit = strconv.FormatInt(v.Limit, 10)
+				limit = fmt.Sprint(v.Limit)
 			}
 		}
 
@@ -981,7 +981,7 @@ func (c *cmdProjectInfo) run(cmd *cobra.Command, args []string) error {
 		if slices.Contains(byteLimits, shortKey) {
 			usage = units.GetByteSizeStringIEC(v.Usage, 2)
 		} else {
-			usage = strconv.FormatInt(v.Usage, 10)
+			usage = fmt.Sprint(v.Usage)
 		}
 
 		columnName := strings.ToUpper(k)
