@@ -48,7 +48,7 @@ func (c *cmdPublish) command() *cobra.Command {
 		}
 
 		if len(args) == 1 {
-			return c.global.cmpRemotes(toComplete, ":", true, instanceServerRemoteCompletionFilters(*c.global.conf)...)
+			return c.global.cmpRemotes(toComplete, false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
