@@ -1,7 +1,6 @@
 package main
 
 import (
-	"context"
 	"crypto/x509"
 	"encoding/pem"
 	"errors"
@@ -10,9 +9,7 @@ import (
 	"net/http"
 	"net/url"
 	"os"
-	"slices"
 	"sort"
-	"strconv"
 	"strings"
 
 	"github.com/spf13/cobra"
@@ -131,7 +128,7 @@ func (c *cmdRemoteAdd) findProject(d lxd.InstanceServer, project string) (string
 			}
 
 			// Deal with multiple projects.
-			if slices.Contains(names, "default") {
+			if shared.ValueInSlice("default", names) {
 				// If we have access to the default project, use it.
 				return "", nil
 			}
@@ -224,7 +221,7 @@ func (c *cmdRemoteAdd) addRemoteFromToken(addr string, server string, token stri
 
 	_, err = conf.GetInstanceServer(server)
 	if err != nil {
-		certificate, err = shared.GetRemoteCertificate(context.Background(), addr, c.global.conf.UserAgent)
+		certificate, err = shared.GetRemoteCertificate(addr, c.global.conf.UserAgent)
 		if err != nil {
 			return api.StatusErrorf(http.StatusServiceUnavailable, "%s: %w", i18n.G("Unavailable remote server"), err)
 		}
@@ -258,7 +255,6 @@ func (c *cmdRemoteAdd) addRemoteFromToken(addr string, server string, token stri
 		}
 	}
 
-	// Implicitly runs GetServer which updates the servers extensions.
 	d, err := conf.GetInstanceServer(server)
 	if err != nil {
 		return api.StatusErrorf(http.StatusServiceUnavailable, "%s: %w", i18n.G("Unavailable remote server"), err)
@@ -435,7 +431,7 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 		rHost = host
 		rPort = port
 	} else {
-		rPort = strconv.Itoa(shared.HTTPSDefaultPort)
+		rPort = fmt.Sprint(shared.HTTPSDefaultPort)
 	}
 
 	if rScheme == "unix" {
@@ -511,7 +507,7 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 	var certificate *x509.Certificate
 	if err != nil {
 		// Failed to connect using the system CA, so retrieve the remote certificate
-		certificate, err = shared.GetRemoteCertificate(context.Background(), addr, c.global.conf.UserAgent)
+		certificate, err = shared.GetRemoteCertificate(addr, c.global.conf.UserAgent)
 		if err != nil {
 			return err
 		}
@@ -532,10 +528,9 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 					return err
 				}
 
-				// Check length before accessing line to prevent runtime panic.
 				// Continue with adding the remote if digest matches, or the user
 				// confirmed a fingerprint.
-				if string(line) == digest || (len(line) > 0 && strings.ToLower(string(line[0])) == i18n.G("y")) {
+				if string(line) == digest || strings.ToLower(string(line[0])) == i18n.G("y") {
 					break
 				}
 
@@ -547,7 +542,7 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 				}
 
 				// Error out if the user didn't confirm the fingerprint.
-				if len(line) == 0 || strings.ToLower(string(line[0])) == i18n.G("n") {
+				if len(line) < 1 || strings.ToLower(string(line[0])) == i18n.G("n") {
 					return errors.New(i18n.G("Server certificate NACKed by user"))
 				}
 
@@ -604,13 +599,13 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 
 	// If not specified, the preferred order of authentication is 1) OIDC 2) TLS.
 	if c.flagAuthType == "" {
-		if !srv.Public && slices.Contains(srv.AuthMethods, api.AuthenticationMethodOIDC) {
+		if !srv.Public && shared.ValueInSlice(api.AuthenticationMethodOIDC, srv.AuthMethods) {
 			c.flagAuthType = api.AuthenticationMethodOIDC
 		} else {
 			c.flagAuthType = api.AuthenticationMethodTLS
 		}
 
-		if slices.Contains([]string{api.AuthenticationMethodOIDC}, c.flagAuthType) {
+		if shared.ValueInSlice(c.flagAuthType, []string{api.AuthenticationMethodOIDC}) {
 			// Update the remote configuration
 			remote := conf.Remotes[server]
 			remote.AuthType = c.flagAuthType
@@ -636,7 +631,7 @@ func (c *cmdRemoteAdd) run(cmd *cobra.Command, args []string) error {
 		}
 	}
 
-	if !srv.Public && !slices.Contains(srv.AuthMethods, c.flagAuthType) {
+	if !srv.Public && !shared.ValueInSlice(c.flagAuthType, srv.AuthMethods) {
 		return fmt.Errorf(i18n.G("Authentication type '%s' not supported by server"), c.flagAuthType)
 	}
 
@@ -882,7 +877,7 @@ func (c *cmdRemoteRename) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, "", false, filterStaticRemotes)
+			return c.global.cmpRemoteNames(true)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -963,7 +958,7 @@ func (c *cmdRemoteRemove) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, "", false, filterStaticRemotes, filterGlobalRemotes, filterDefaultRemote(*c.global.conf))
+			return c.global.cmpRemoteNames(false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -1027,8 +1022,7 @@ func (c *cmdRemoteSwitch) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			// It's valid to switch to a public remote, but filter them from completions to prevent leading new users down a bad path.
-			return c.global.cmpRemotes(toComplete, "", false, filterDefaultRemote(*c.global.conf), filterPublicRemotes)
+			return c.global.cmpRemoteNames(false)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
@@ -1076,7 +1070,7 @@ func (c *cmdRemoteSetURL) command() *cobra.Command {
 
 	cmd.ValidArgsFunction = func(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 		if len(args) == 0 {
-			return c.global.cmpRemotes(toComplete, "", false, filterStaticRemotes)
+			return c.global.cmpRemoteNames(true)
 		}
 
 		return nil, cobra.ShellCompDirectiveNoFileComp
