@@ -4,7 +4,6 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"maps"
 	"os"
 	"sort"
 	"strings"
@@ -428,7 +427,7 @@ func (c *cmdStorageBucketGet) run(cmd *cobra.Command, args []string) error {
 
 	if c.flagIsProperty {
 		w := resp.Writable()
-		res, err := getFieldByJSONTag(&w, args[2])
+		res, err := getFieldByJsonTag(&w, args[2])
 		if err != nil {
 			return fmt.Errorf(i18n.G("The property %q does not exist on the storage bucket %q: %v"), args[2], resource.name, err)
 		}
@@ -448,9 +447,7 @@ func (c *cmdStorageBucketGet) run(cmd *cobra.Command, args []string) error {
 type cmdStorageBucketList struct {
 	global        *cmdGlobal
 	storageBucket *cmdStorageBucket
-
-	flagFormat      string
-	flagAllProjects bool
+	flagFormat    string
 }
 
 func (c *cmdStorageBucketList) command() *cobra.Command {
@@ -461,7 +458,6 @@ func (c *cmdStorageBucketList) command() *cobra.Command {
 
 	cmd.Long = cli.FormatSection(i18n.G("Description"), i18n.G(`List storage buckets`))
 	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", "table", i18n.G("Format (csv|json|table|yaml|compact)")+"``")
-	cmd.Flags().BoolVar(&c.flagAllProjects, "all-projects", false, i18n.G("Display storage pool buckets from all projects"))
 
 	cmd.RunE = c.run
 
@@ -489,17 +485,9 @@ func (c *cmdStorageBucketList) run(cmd *cobra.Command, args []string) error {
 
 	client := resource.server
 
-	var buckets []api.StorageBucket
-	if c.flagAllProjects {
-		buckets, err = client.GetStoragePoolBucketsAllProjects(resource.name)
-		if err != nil {
-			return err
-		}
-	} else {
-		buckets, err = client.GetStoragePoolBuckets(resource.name)
-		if err != nil {
-			return err
-		}
+	buckets, err := client.GetStoragePoolBuckets(resource.name)
+	if err != nil {
+		return err
 	}
 
 	clustered := resource.server.IsClustered()
@@ -515,10 +503,6 @@ func (c *cmdStorageBucketList) run(cmd *cobra.Command, args []string) error {
 			details = append(details, bucket.Location)
 		}
 
-		if c.flagAllProjects {
-			details = append([]string{bucket.Project}, details...)
-		}
-
 		data = append(data, details)
 	}
 
@@ -533,10 +517,6 @@ func (c *cmdStorageBucketList) run(cmd *cobra.Command, args []string) error {
 		header = append(header, i18n.G("LOCATION"))
 	}
 
-	if c.flagAllProjects {
-		header = append([]string{i18n.G("PROJECT")}, header...)
-	}
-
 	return cli.RenderTable(c.flagFormat, header, data, buckets)
 }
 
@@ -612,7 +592,7 @@ func (c *cmdStorageBucketSet) run(cmd *cobra.Command, args []string) error {
 	if c.flagIsProperty {
 		if cmd.Name() == "unset" {
 			for k := range keys {
-				err := unsetFieldByJSONTag(&writable, k)
+				err := unsetFieldByJsonTag(&writable, k)
 				if err != nil {
 					return fmt.Errorf(i18n.G("Error unsetting property: %v"), err)
 				}
@@ -624,7 +604,9 @@ func (c *cmdStorageBucketSet) run(cmd *cobra.Command, args []string) error {
 			}
 		}
 	} else {
-		maps.Copy(writable.Config, keys)
+		for k, v := range keys {
+			writable.Config[k] = v
+		}
 	}
 
 	err = client.UpdateStoragePoolBucket(resource.name, args[1], writable, etag)
