@@ -3,6 +3,7 @@ package main
 import (
 	"fmt"
 	"io"
+	"net/http"
 	"os"
 	"reflect"
 	"sort"
@@ -140,7 +141,7 @@ func parseDeviceOverrides(deviceOverrideArgs []string) (map[string]map[string]st
 
 // IsAliasesSubset returns true if the first array is completely contained in the second array.
 func IsAliasesSubset(a1 []api.ImageAlias, a2 []api.ImageAlias) bool {
-	set := make(map[string]interface{})
+	set := make(map[string]any)
 	for _, alias := range a2 {
 		set[alias.Name] = nil
 	}
@@ -324,9 +325,9 @@ func structHasField(typ reflect.Type, field string) bool {
 }
 
 // getServerSupportedFilters returns two lists: one with filters supported by server and second one with not supported.
-func getServerSupportedFilters(filters []string, i interface{}) ([]string, []string) {
-	supportedFilters := []string{}
-	unsupportedFilters := []string{}
+func getServerSupportedFilters(filters []string, i any) (supportedFilters []string, unsupportedFilters []string) {
+	supportedFilters = []string{}
+	unsupportedFilters = []string{}
 
 	for _, filter := range filters {
 		membs := strings.SplitN(filter, "=", 2)
@@ -346,7 +347,7 @@ func getServerSupportedFilters(filters []string, i interface{}) ([]string, []str
 }
 
 // guessImage checks that the image name (provided by the user) is correct given an instance remote and image remote.
-func guessImage(conf *config.Config, d lxd.InstanceServer, instRemote string, imgRemote string, imageRef string) (string, string) {
+func guessImage(conf *config.Config, d lxd.InstanceServer, instRemote string, imgRemote string, imageRef string) (imageRemote string, image string) {
 	if instRemote != imgRemote {
 		return imgRemote, imageRef
 	}
@@ -410,9 +411,44 @@ func getImgInfo(d lxd.InstanceServer, conf *config.Config, imgRemote string, ins
 		// Get the image info
 		imgInfo, _, err = imgRemoteServer.GetImage(imageRef)
 		if err != nil {
-			return nil, nil, err
+			return nil, nil, fmt.Errorf(i18n.G("Failed to find image %q on remote %q"), imageRef, imgRemote)
 		}
 	}
 
 	return imgRemoteServer, imgInfo, nil
 }
+
+// newLocationHeaderTransportWrapper returns a new transport wrapper that can be used to inspect the `Location` header
+// upon the response of a resource creation request to LXD.
+func newLocationHeaderTransportWrapper() (*locationHeaderTransport, func(transport *http.Transport) lxd.HTTPTransporter) {
+	transporter := &locationHeaderTransport{}
+	return transporter, func(transport *http.Transport) lxd.HTTPTransporter {
+		transporter.transport = transport
+		return transporter
+	}
+}
+
+// locationHeaderTransport implements lxd.HTTPTransporter by wrapping a http.Transport.
+type locationHeaderTransport struct {
+	transport *http.Transport
+	location  string
+}
+
+// RoundTrip implements http.RoundTripper for locationHeaderTransport. It extracts the `Location` header from the HTTP
+// response for later use. This is useful when the resource is not known in advance (e.g. for auto-allocated IP addresses
+// of network forwards and load-balancers).
+func (c *locationHeaderTransport) RoundTrip(r *http.Request) (*http.Response, error) {
+	resp, err := c.transport.RoundTrip(r)
+	if err != nil {
+		return nil, err
+	}
+
+	c.location = resp.Header.Get("Location")
+
+	return resp, err
+}
+
+// Transport returns the underlying transport of cloudInstanceServerTransport (to implement lxd.HTTPTransporter).
+func (c *locationHeaderTransport) Transport() *http.Transport {
+	return c.transport
+}
