@@ -1,13 +1,17 @@
 package main
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"net/http"
 	"os"
+	"path/filepath"
 	"strings"
+	"time"
 
 	"github.com/canonical/lxd/lxd/events"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
 	"github.com/canonical/lxd/lxd/response"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
@@ -23,12 +27,12 @@ var eventsCmd = APIEndpoint{
 }
 
 type eventsServe struct {
-	req *http.Request
-	d   *Daemon
+	d *Daemon
 }
 
-func (r *eventsServe) Render(w http.ResponseWriter) error {
-	return eventsSocket(r.d, r.req, w)
+// Render starts event socket.
+func (r *eventsServe) Render(w http.ResponseWriter, request *http.Request) error {
+	return eventsSocket(r.d, request, w)
 }
 
 func (r *eventsServe) String() string {
@@ -86,7 +90,7 @@ func eventsSocket(d *Daemon, r *http.Request, w http.ResponseWriter) error {
 }
 
 func eventsGet(d *Daemon, r *http.Request) response.Response {
-	return &eventsServe{req: r, d: d}
+	return &eventsServe{d: d}
 }
 
 func eventsPost(d *Daemon, r *http.Request) response.Response {
@@ -115,9 +119,10 @@ func eventsProcess(event api.Event) {
 	}
 
 	type deviceEvent struct {
-		Action string            `json:"action"`
-		Config map[string]string `json:"config"`
-		Name   string            `json:"name"`
+		Action string                    `json:"action"`
+		Config map[string]string         `json:"config"`
+		Name   string                    `json:"name"`
+		Mount  instancetype.VMAgentMount `json:"mount"`
 	}
 
 	e := deviceEvent{}
@@ -142,14 +147,47 @@ func eventsProcess(event api.Event) {
 	}
 
 	// Attempt to perform the mount.
-	mntSource := fmt.Sprintf("lxd_%s", e.Name)
+	mntSource := "lxd_" + e.Name
+	if e.Mount.Source != "" {
+		mntSource = e.Mount.Source
+	}
 
-	_ = os.MkdirAll(e.Config["path"], 0755)
-	_, err = shared.RunCommand("mount", "-t", "virtiofs", mntSource, e.Config["path"])
-	if err != nil {
-		logger.Infof("Failed to mount hotplug %q (Type: %q) to %q", mntSource, "virtiofs", e.Config["path"])
+	l := logger.AddContext(logger.Ctx{"type": "virtiofs", "source": mntSource, "path": e.Config["path"]})
+
+	// Reject path containing "..".
+	if strings.Contains(e.Config["path"], "..") {
+		l.Error("Invalid path containing '..'")
 		return
 	}
 
-	logger.Infof("Mounted hotplug %q (Type: %q) to %q", mntSource, "virtiofs", e.Config["path"])
+	// If the path is not absolute, the mount will be created relative to the current directory.
+	// (since the mount command executed below originates from the `lxd-agent` binary that is in the `/run/lxd_agent` directory).
+	// This is not ideal and not consistent with the way mounts are handled with containers. For consistency make the path absolute.
+	e.Config["path"], err = filepath.Abs(e.Config["path"])
+	if err != nil || !strings.HasPrefix(e.Config["path"], "/") {
+		l.Error("Failed to make path absolute")
+		return
+	}
+
+	_ = os.MkdirAll(e.Config["path"], 0755)
+
+	// Parse mount options, if provided.
+	var args []string
+	if len(e.Mount.Options) > 0 {
+		args = append(args, "-o", strings.Join(e.Mount.Options, ","))
+	}
+
+	args = append(args, "-t", "virtiofs", mntSource, e.Config["path"])
+
+	for i := 0; i < 5; i++ {
+		_, err = shared.RunCommandContext(context.Background(), "mount", args...)
+		if err == nil {
+			l.Info("Mounted hotplug")
+			return
+		}
+
+		time.Sleep(500 * time.Millisecond)
+	}
+
+	l.Info("Failed to mount hotplug", logger.Ctx{"err": err})
 }
