@@ -4,9 +4,9 @@ import (
 	"context"
 	"encoding/json"
 	"errors"
+	"fmt"
 	"io"
 	"io/fs"
-	"maps"
 	"net/http"
 	"os"
 	"os/exec"
@@ -53,13 +53,15 @@ func execPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if !post.WaitForWS {
-		return response.BadRequest(errors.New("Websockets are required for VM exec"))
+		return response.BadRequest(fmt.Errorf("Websockets are required for VM exec"))
 	}
 
 	env := map[string]string{}
 
 	if post.Environment != nil {
-		maps.Copy(env, post.Environment)
+		for k, v := range post.Environment {
+			env[k] = v
+		}
 	}
 
 	// Set default value for PATH
@@ -134,7 +136,7 @@ func execPost(d *Daemon, r *http.Request) response.Response {
 
 	resources := map[string][]api.URL{}
 
-	op, err := operations.OperationCreate(r.Context(), nil, "", operations.OperationClassWebsocket, operationtype.CommandExec, resources, ws.Metadata(), ws.Do, nil, ws.Connect)
+	op, err := operations.OperationCreate(nil, "", operations.OperationClassWebsocket, operationtype.CommandExec, resources, ws.Metadata(), ws.Do, nil, ws.Connect, r)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -184,7 +186,7 @@ func (s *execWs) Metadata() any {
 func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.ResponseWriter) error {
 	secret := r.FormValue("secret")
 	if secret == "" {
-		return errors.New("missing secret")
+		return fmt.Errorf("missing secret")
 	}
 
 	for fd, fdSecret := range s.fds {
@@ -212,10 +214,10 @@ func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.Respo
 			}
 
 			if !found {
-				return errors.New("Unknown websocket number")
+				return fmt.Errorf("Unknown websocket number")
 			}
 
-			return errors.New("Websocket number already connected")
+			return fmt.Errorf("Websocket number already connected")
 		}
 	}
 
@@ -243,7 +245,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 	select {
 	case <-s.requiredConnectedCtx.Done():
 	case <-time.After(time.Second * 5):
-		return errors.New("Timed out waiting for websockets to connect")
+		return fmt.Errorf("Timed out waiting for websockets to connect")
 	}
 
 	var err error
@@ -272,7 +274,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 	} else {
 		ttys = make([]*os.File, 3)
 		ptys = make([]*os.File, 3)
-		for i := range ttys {
+		for i := 0; i < len(ttys); i++ {
 			ptys[i], ttys[i], err = os.Pipe()
 			if err != nil {
 				return err
@@ -475,7 +477,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 		}()
 	} else {
 		wgEOF.Add(len(ttys) - 1)
-		for i := range ttys {
+		for i := 0; i < len(ttys); i++ {
 			go func(i int) {
 				l.Debug("Exec mirror websocket started", logger.Ctx{"number": i})
 				defer l.Debug("Exec mirror websocket finished", logger.Ctx{"number": i})
