@@ -3,7 +3,7 @@ package main
 import (
 	"bufio"
 	"bytes"
-	"fmt"
+	"math"
 	"net"
 	"net/http"
 	"os"
@@ -97,13 +97,24 @@ func cpuState() api.InstanceStateCPU {
 func memoryState() api.InstanceStateMemory {
 	memory := api.InstanceStateMemory{}
 
-	stats, err := getMemoryMetrics(nil)
+	stats, err := getMemoryMetrics()
 	if err != nil {
 		return memory
 	}
 
-	memory.Usage = int64(stats.MemTotalBytes) - int64(stats.MemFreeBytes)
-	memory.Total = int64(stats.MemTotalBytes)
+	// Bound checking before converting from uint64 to int64
+	if stats.MemTotalBytes > math.MaxInt64 {
+		memory.Total = math.MaxInt64
+	} else {
+		memory.Total = int64(stats.MemTotalBytes)
+	}
+
+	usage := stats.MemTotalBytes - stats.MemFreeBytes
+	if usage > math.MaxInt64 {
+		memory.Usage = math.MaxInt64
+	} else {
+		memory.Usage = int64(usage)
+	}
 
 	// Memory peak in bytes
 	value, err := os.ReadFile("/sys/fs/cgroup/memory/memory.max_usage_in_bytes")
@@ -150,25 +161,25 @@ func networkState() map[string]api.InstanceStateNetwork {
 		}
 
 		// Counters
-		value, err := os.ReadFile(fmt.Sprintf("/sys/class/net/%s/statistics/tx_bytes", iface.Name))
+		value, err := os.ReadFile("/sys/class/net/" + iface.Name + "/statistics/tx_bytes")
 		valueInt, err1 := strconv.ParseInt(strings.TrimSpace(string(value)), 10, 64)
 		if err == nil && err1 == nil {
 			network.Counters.BytesSent = valueInt
 		}
 
-		value, err = os.ReadFile(fmt.Sprintf("/sys/class/net/%s/statistics/rx_bytes", iface.Name))
+		value, err = os.ReadFile("/sys/class/net/" + iface.Name + "/statistics/rx_bytes")
 		valueInt, err1 = strconv.ParseInt(strings.TrimSpace(string(value)), 10, 64)
 		if err == nil && err1 == nil {
 			network.Counters.BytesReceived = valueInt
 		}
 
-		value, err = os.ReadFile(fmt.Sprintf("/sys/class/net/%s/statistics/tx_packets", iface.Name))
+		value, err = os.ReadFile("/sys/class/net/" + iface.Name + "/statistics/tx_packets")
 		valueInt, err1 = strconv.ParseInt(strings.TrimSpace(string(value)), 10, 64)
 		if err == nil && err1 == nil {
 			network.Counters.PacketsSent = valueInt
 		}
 
-		value, err = os.ReadFile(fmt.Sprintf("/sys/class/net/%s/statistics/rx_packets", iface.Name))
+		value, err = os.ReadFile("/sys/class/net/" + iface.Name + "/statistics/rx_packets")
 		valueInt, err1 = strconv.ParseInt(strings.TrimSpace(string(value)), 10, 64)
 		if err == nil && err1 == nil {
 			network.Counters.PacketsReceived = valueInt
@@ -178,7 +189,10 @@ func networkState() map[string]api.InstanceStateNetwork {
 		addrs, _ := iface.Addrs()
 
 		for _, addr := range addrs {
-			addressFields := strings.Split(addr.String(), "/")
+			addressFields := strings.SplitN(addr.String(), "/", 2)
+			if len(addressFields) != 2 {
+				continue
+			}
 
 			networkAddress := api.InstanceStateNetworkAddress{
 				Address: addressFields[0],
@@ -224,7 +238,8 @@ func processesState() int64 {
 
 	// Go through the pid list, adding new pids at the end so we go through them all
 	for i := 0; i < len(pids); i++ {
-		fname := fmt.Sprintf("/proc/%d/task/%d/children", pids[i], pids[i])
+		pid := strconv.FormatInt(pids[i], 10)
+		fname := "/proc/" + pid + "/task/" + pid + "/children"
 		fcont, err := os.ReadFile(fname)
 		if err != nil {
 			// the process terminated during execution of this loop
