@@ -39,7 +39,7 @@ func main() {
 
 	// netcat sub-command
 	netcatCmd := cmdNetcat{global: &globalCmd}
-	app.AddCommand(netcatCmd.command())
+	app.AddCommand(netcatCmd.Command())
 
 	// Run the main command and handle errors
 	err := app.Execute()
