@@ -163,7 +163,7 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 
 	// Ensure trust token is not used along trust certificate and/or its corresponding key.
 	if c.flagToken != "" && (c.flagCertPath != "" || c.flagKeyPath != "") {
-		return nil, "", errors.New("Authentication token is mutually exclusive with certificate path and key")
+		return nil, "", fmt.Errorf("Authentication token is mutually exclusive with certificate path and key")
 	}
 
 	if c.flagNonInteractive || c.flagServer != "" {
@@ -210,11 +210,11 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 	}
 
 	args := lxd.ConnectionArgs{
-		UserAgent:          "LXD-MIGRATE " + version.Version,
+		UserAgent:          fmt.Sprintf("LXD-MIGRATE %s", version.Version),
 		InsecureSkipVerify: true,
 	}
 
-	certificate, err := shared.GetRemoteCertificate(context.Background(), serverURL, args.UserAgent)
+	certificate, err := shared.GetRemoteCertificate(serverURL, args.UserAgent)
 	if err != nil {
 		return nil, "", fmt.Errorf("Failed to get remote certificate: %w", err)
 	}
@@ -309,8 +309,7 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 			authMethod = availableAuthMethods[authMethodInt-1]
 		}
 
-		switch authMethod {
-		case authMethodTLSCertificate:
+		if authMethod == authMethodTLSCertificate {
 			certPath, err = c.global.asker.AskString("Please provide the certificate path: ", "", func(path string) error {
 				if !shared.PathExists(path) {
 					return errors.New("File does not exist")
@@ -332,8 +331,7 @@ func (c *cmdMigrate) askServer() (lxd.InstanceServer, string, error) {
 			if err != nil {
 				return nil, "", err
 			}
-
-		case authMethodTLSCertificateToken:
+		} else if authMethod == authMethodTLSCertificateToken {
 			token, err = c.global.asker.AskString("Please provide the certificate token: ", "", func(token string) error {
 				_, err := shared.CertificateTokenDecode(token)
 				if err != nil {
@@ -539,10 +537,9 @@ func (c *cmdMigrate) runInteractive(config *cmdMigrateData, server lxd.InstanceS
 			return err
 		}
 
-		switch instanceType {
-		case 1:
+		if instanceType == 1 {
 			config.InstanceArgs.Type = api.InstanceTypeContainer
-		case 2:
+		} else if instanceType == 2 {
 			config.InstanceArgs.Type = api.InstanceTypeVM
 		}
 	}
@@ -869,7 +866,7 @@ func (c *cmdMigrate) run(cmd *cobra.Command, args []string) error {
 
 	if config.InstanceArgs.Type == api.InstanceTypeContainer {
 		// Create the rootfs directory
-		fullPath = path + "/rootfs"
+		fullPath = fmt.Sprintf("%s/rootfs", path)
 
 		err = os.Mkdir(fullPath, 0755)
 		if err != nil {
@@ -986,9 +983,9 @@ func (c *cmdMigrate) askProfiles(server lxd.InstanceServer, config *cmdMigrateDa
 			return nil
 		}
 
-		profiles := strings.SplitSeq(s, " ")
+		profiles := strings.Split(s, " ")
 
-		for profile := range profiles {
+		for _, profile := range profiles {
 			if !slices.Contains(profileNames, profile) {
 				return fmt.Errorf("Unknown profile %q", profile)
 			}
@@ -1013,7 +1010,7 @@ func (c *cmdMigrate) askConfig(config *cmdMigrateData) error {
 			return nil
 		}
 
-		for entry := range strings.SplitSeq(s, " ") {
+		for _, entry := range strings.Split(s, " ") {
 			if !strings.Contains(entry, "=") {
 				return fmt.Errorf("Bad key=value configuration: %v", entry)
 			}
@@ -1025,7 +1022,7 @@ func (c *cmdMigrate) askConfig(config *cmdMigrateData) error {
 		return err
 	}
 
-	for entry := range strings.SplitSeq(configs, " ") {
+	for _, entry := range strings.Split(configs, " ") {
 		key, value, _ := strings.Cut(entry, "=")
 		config.InstanceArgs.Config[key] = value
 	}
