@@ -7,13 +7,11 @@ import (
 	"encoding/pem"
 	"errors"
 	"fmt"
-	"net"
 	"net/url"
 	"os"
 	"os/exec"
 	"path/filepath"
 	"reflect"
-	"strconv"
 	"strings"
 
 	"golang.org/x/sys/unix"
@@ -153,7 +151,7 @@ func transferRootDiskForConversion(ctx context.Context, op lxd.Operation, rootfs
 
 func (c *cmdMigrate) connectLocal(path string) (lxd.InstanceServer, error) {
 	args := lxd.ConnectionArgs{}
-	args.UserAgent = "LXD-MIGRATE " + version.Version
+	args.UserAgent = fmt.Sprintf("LXD-MIGRATE %s", version.Version)
 
 	return lxd.ConnectLXDUnix(path, &args)
 }
@@ -206,13 +204,13 @@ func (c *cmdMigrate) connectTarget(url string, certPath string, keyPath string,
 	}
 
 	// Attempt to connect using the system CA
-	args.UserAgent = "LXD-MIGRATE " + version.Version
+	args.UserAgent = fmt.Sprintf("LXD-MIGRATE %s", version.Version)
 	instanceServer, err := lxd.ConnectLXD(url, &args)
 
 	var certificate *x509.Certificate
 	if err != nil {
 		// Failed to connect using the system CA, so retrieve the remote certificate
-		certificate, err = shared.GetRemoteCertificate(context.Background(), url, args.UserAgent)
+		certificate, err = shared.GetRemoteCertificate(url, args.UserAgent)
 		if err != nil {
 			return nil, "", err
 		}
@@ -244,8 +242,12 @@ func (c *cmdMigrate) connectTarget(url string, certPath string, keyPath string,
 
 	if authType == api.AuthenticationMethodTLS {
 		if token != "" {
-			req := api.CertificatesPost{
-				Password: token,
+			req := api.CertificatesPost{}
+
+			if instanceServer.HasExtension("explicit_trust_token") {
+				req.TrustToken = token
+			} else {
+				req.Password = token //nolint:staticcheck
 			}
 
 			err = instanceServer.CreateCertificate(req)
@@ -256,7 +258,7 @@ func (c *cmdMigrate) connectTarget(url string, certPath string, keyPath string,
 			// In non-interactive mode stop at this point, as we know that the server
 			// does not trust us, but we should not make any further interaction with the caller.
 			if certPath != "" || keyPath != "" {
-				return nil, "", errors.New("Provided certificate is not trusted by the server")
+				return nil, "", fmt.Errorf("Provided certificate is not trusted by the server")
 			}
 
 			return nil, "", errors.New("Failed to authenticate with the server: Please, either provide a trust token or an already trusted certificate")
@@ -271,7 +273,6 @@ func (c *cmdMigrate) connectTarget(url string, certPath string, keyPath string,
 			}
 		} else {
 			fmt.Println("It is recommended to have this certificate be manually added to LXD through `lxc config trust add` on the target server.\nAlternatively you could use a pre-defined trust password to add it remotely (use of a trust password can be a security issue).")
-
 			fmt.Println("")
 
 			useTrustPassword, err := c.global.asker.AskBool("Would you like to use a trust password? [default=no]: ", "no")
@@ -365,7 +366,7 @@ func parseURL(URL string) (string, error) {
 
 	// Create a URL with scheme and hostname since it wasn't provided
 	if u.Scheme == "" && u.Host == "" && u.Path != "" {
-		u, err = url.Parse("https://" + u.Path)
+		u, err = url.Parse(fmt.Sprintf("https://%s", u.Path))
 		if err != nil {
 			return "", err
 		}
@@ -373,7 +374,7 @@ func parseURL(URL string) (string, error) {
 
 	// If no port was provided, use default port
 	if u.Port() == "" {
-		u.Host = net.JoinHostPort(u.Hostname(), strconv.Itoa(shared.HTTPSDefaultPort))
+		u.Host = fmt.Sprintf("%s:%d", u.Hostname(), shared.HTTPSDefaultPort)
 	}
 
 	return u.String(), nil
