@@ -16,7 +16,7 @@ type cmdGlobal struct {
 func main() {
 	// daemon command (main)
 	daemonCmd := cmdDaemon{}
-	app := daemonCmd.command()
+	app := daemonCmd.Command()
 	app.Use = "lxd-user"
 	app.Short = "LXD user project daemon"
 	app.Long = `Description:
