@@ -3,20 +3,16 @@ package main
 import (
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
-	"maps"
 	"math"
 	"net"
 	"net/http"
 	"os"
 	"slices"
-	"strconv"
 
 	"github.com/canonical/lxd/client"
 	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/auth/oidc"
-	backupConfig "github.com/canonical/lxd/lxd/backup/config"
 	"github.com/canonical/lxd/lxd/cluster"
 	clusterConfig "github.com/canonical/lxd/lxd/cluster/config"
 	"github.com/canonical/lxd/lxd/config"
@@ -234,7 +230,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 	// Get the authentication methods.
 	authMethods := []string{api.AuthenticationMethodTLS}
 
-	oidcIssuer, oidcClientID, _, _, _, _ := s.GlobalConfig.OIDCServer()
+	oidcIssuer, oidcClientID, _, _, _ := s.GlobalConfig.OIDCServer()
 	if oidcIssuer != "" && oidcClientID != "" {
 		authMethods = append(authMethods, api.AuthenticationMethodOIDC)
 	}
@@ -260,8 +256,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// If a target was specified, forward the request to the relevant node.
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
@@ -314,36 +309,34 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 	}
 
 	env := api.ServerEnvironment{
-		Addresses:                  addresses,
-		Architectures:              architectures,
-		BackupMetadataVersionRange: []uint32{api.BackupMetadataVersion1, backupConfig.MaxMetadataVersion},
-		Certificate:                certificate,
-		CertificateFingerprint:     certificateFingerprint,
-		Kernel:                     s.OS.Uname.Sysname,
-		KernelArchitecture:         s.OS.Uname.Machine,
-		KernelVersion:              s.OS.Uname.Release,
-		OSName:                     s.OS.ReleaseInfo["NAME"],
-		OSVersion:                  s.OS.ReleaseInfo["VERSION_ID"],
-		Project:                    projectName,
-		Server:                     "lxd",
-		ServerPid:                  os.Getpid(),
-		ServerVersion:              version.Version,
-		ServerLTS:                  version.IsLTSVersion,
-		ServerClustered:            s.ServerClustered,
-		ServerEventMode:            string(cluster.ServerEventMode()),
-		ServerName:                 serverName,
-		Firewall:                   s.Firewall.String(),
+		Addresses:              addresses,
+		Architectures:          architectures,
+		Certificate:            certificate,
+		CertificateFingerprint: certificateFingerprint,
+		Kernel:                 s.OS.Uname.Sysname,
+		KernelArchitecture:     s.OS.Uname.Machine,
+		KernelVersion:          s.OS.Uname.Release,
+		OSName:                 s.OS.ReleaseInfo["NAME"],
+		OSVersion:              s.OS.ReleaseInfo["VERSION_ID"],
+		Project:                projectName,
+		Server:                 "lxd",
+		ServerPid:              os.Getpid(),
+		ServerVersion:          version.Version,
+		ServerLTS:              version.IsLTSVersion,
+		ServerClustered:        s.ServerClustered,
+		ServerEventMode:        string(cluster.ServerEventMode()),
+		ServerName:             serverName,
+		Firewall:               s.Firewall.String(),
 	}
 
 	env.KernelFeatures = map[string]string{
-		"bpf_token":                 strconv.FormatBool(s.OS.BPFToken),
-		"netnsid_getifaddrs":        strconv.FormatBool(s.OS.NetnsGetifaddrs),
-		"uevent_injection":          strconv.FormatBool(s.OS.UeventInjection),
-		"unpriv_binfmt":             strconv.FormatBool(s.OS.UnprivBinfmt),
-		"unpriv_fscaps":             strconv.FormatBool(s.OS.VFS3Fscaps),
-		"seccomp_listener":          strconv.FormatBool(s.OS.SeccompListener),
-		"seccomp_listener_continue": strconv.FormatBool(s.OS.SeccompListenerContinue),
-		"idmapped_mounts":           strconv.FormatBool(s.OS.IdmappedMounts),
+		"netnsid_getifaddrs":        fmt.Sprint(s.OS.NetnsGetifaddrs),
+		"uevent_injection":          fmt.Sprint(s.OS.UeventInjection),
+		"unpriv_binfmt":             fmt.Sprint(s.OS.UnprivBinfmt),
+		"unpriv_fscaps":             fmt.Sprint(s.OS.VFS3Fscaps),
+		"seccomp_listener":          fmt.Sprint(s.OS.SeccompListener),
+		"seccomp_listener_continue": fmt.Sprint(s.OS.SeccompListenerContinue),
+		"idmapped_mounts":           fmt.Sprint(s.OS.IdmappedMounts),
 	}
 
 	drivers := instanceDrivers.DriverStatuses()
@@ -379,7 +372,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 
 		// Add supported instance types.
 		instType := driver.Info.Type.String()
-		if !slices.Contains(env.InstanceTypes, instType) {
+		if !shared.ValueInSlice(instType, env.InstanceTypes) {
 			env.InstanceTypes = append(env.InstanceTypes, instType)
 		}
 	}
@@ -387,7 +380,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 	if s.OS.LXCFeatures != nil {
 		env.LXCFeatures = map[string]string{}
 		for k, v := range s.OS.LXCFeatures {
-			env.LXCFeatures[k] = strconv.FormatBool(v)
+			env.LXCFeatures[k] = fmt.Sprint(v)
 		}
 	}
 
@@ -411,7 +404,7 @@ func api10Get(d *Daemon, r *http.Request) response.Response {
 
 	fullSrv := &api.Server{ServerUntrusted: srv}
 	fullSrv.Environment = env
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	fullSrv.AuthUserName = requestor.Username
 	fullSrv.AuthUserMethod = requestor.Protocol
 
@@ -474,8 +467,7 @@ func api10Put(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	// If a target was specified, forward the request to the relevant node.
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
@@ -574,8 +566,7 @@ func api10Patch(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	// If a target was specified, forward the request to the relevant node.
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
@@ -632,7 +623,9 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 		}
 
 		// Keep old config around in case something goes wrong. In that case the config will be reverted.
-		maps.Copy(oldNodeConfig, newNodeConfig.Dump())
+		for k, v := range newNodeConfig.Dump() {
+			oldNodeConfig[k] = v
+		}
 
 		// We currently don't allow changing the cluster.https_address once it's set.
 		if s.ServerClustered {
@@ -650,7 +643,7 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 			}
 
 			if curConfig["cluster.https_address"] != newClusterHTTPSAddress {
-				return errors.New("Changing cluster.https_address is currently not supported")
+				return fmt.Errorf("Changing cluster.https_address is currently not supported")
 			}
 		}
 
@@ -735,7 +728,7 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 	var newClusterConfig *clusterConfig.Config
 	var oldClusterConfig map[string]any
 
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.Background(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 		newClusterConfig, err = clusterConfig.Load(ctx, tx)
 		if err != nil {
@@ -746,9 +739,9 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 		oldClusterConfig = newClusterConfig.Dump()
 
 		if patch {
-			clusterChanged, err = newClusterConfig.Patch(tx, req.Config)
+			clusterChanged, err = newClusterConfig.Patch(req.Config)
 		} else {
-			clusterChanged, err = newClusterConfig.Replace(tx, req.Config)
+			clusterChanged, err = newClusterConfig.Replace(req.Config)
 		}
 
 		return err
@@ -778,7 +771,7 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 				return fmt.Errorf("Failed to load cluster config: %w", err)
 			}
 
-			_, err = newClusterConfig.Replace(tx, req.Config)
+			_, err = newClusterConfig.Replace(req.Config)
 			if err != nil {
 				return fmt.Errorf("Failed updating cluster config: %w", err)
 			}
@@ -831,12 +824,12 @@ func doAPI10Update(d *Daemon, r *http.Request, req api.ServerPut, patch bool) re
 
 	revert.Success()
 
-	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.ConfigUpdated.Event(request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.ConfigUpdated.Event(request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
 
-func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]string, newNodeConfig *node.Config, newClusterConfig *clusterConfig.Config) error {
+func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]string, nodeConfig *node.Config, clusterConfig *clusterConfig.Config) error {
 	s := d.State()
 
 	maasChanged := false
@@ -857,7 +850,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 		case "core.proxy_https":
 			fallthrough
 		case "core.proxy_ignore_hosts":
-			daemonConfigSetProxy(d, newClusterConfig)
+			daemonConfigSetProxy(d, clusterConfig)
 		case "maas.api.url":
 			fallthrough
 		case "maas.api.key":
@@ -869,7 +862,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 			}
 
 		case "cluster.offline_threshold":
-			d.gateway.HeartbeatOfflineThreshold = newClusterConfig.OfflineThreshold()
+			d.gateway.HeartbeatOfflineThreshold = clusterConfig.OfflineThreshold()
 			d.taskClusterHeartbeat.Reset()
 		case "images.auto_update_interval":
 			fallthrough
@@ -900,7 +893,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 			acmeCAURLChanged = true
 		case "acme.domain":
 			acmeDomainChanged = true
-		case "oidc.issuer", "oidc.client.id", "oidc.client.secret", "oidc.scopes", "oidc.audience", "oidc.groups.claim":
+		case "oidc.issuer", "oidc.client.id", "oidc.scopes", "oidc.audience", "oidc.groups.claim":
 			oidcChanged = true
 		}
 	}
@@ -932,7 +925,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 			return err
 		}
 
-		s.Endpoints.NetworkUpdateTrustedProxy(newClusterConfig.HTTPSTrustedProxy())
+		s.Endpoints.NetworkUpdateTrustedProxy(clusterConfig.HTTPSTrustedProxy())
 	}
 
 	value, ok = nodeChanged["cluster.https_address"]
@@ -942,7 +935,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 			return err
 		}
 
-		s.Endpoints.NetworkUpdateTrustedProxy(newClusterConfig.HTTPSTrustedProxy())
+		s.Endpoints.NetworkUpdateTrustedProxy(clusterConfig.HTTPSTrustedProxy())
 	}
 
 	value, ok = nodeChanged["core.debug_address"]
@@ -986,8 +979,8 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 	}
 
 	if maasChanged {
-		url, key := newClusterConfig.MAASController()
-		machine := newNodeConfig.MAASMachine()
+		url, key := clusterConfig.MAASController()
+		machine := nodeConfig.MAASMachine()
 		err := d.setupMAASController(url, key, machine)
 		if err != nil {
 			return err
@@ -995,22 +988,22 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 	}
 
 	if bgpChanged {
-		address := newNodeConfig.BGPAddress()
-		asn := newClusterConfig.BGPASN()
-		routerid := newNodeConfig.BGPRouterID()
+		address := nodeConfig.BGPAddress()
+		asn := clusterConfig.BGPASN()
+		routerid := nodeConfig.BGPRouterID()
 
 		if asn > math.MaxUint32 {
-			return errors.New("Cannot convert BGP ASN to uint32: Upper bound exceeded")
+			return fmt.Errorf("Cannot convert BGP ASN to uint32: Upper bound exceeded")
 		}
 
-		err := s.BGP.Configure(address, uint32(asn), net.ParseIP(routerid))
+		err := s.BGP.Reconfigure(address, uint32(asn), net.ParseIP(routerid))
 		if err != nil {
 			return fmt.Errorf("Failed reconfiguring BGP: %w", err)
 		}
 	}
 
 	if dnsChanged {
-		address := newNodeConfig.DNSAddress()
+		address := nodeConfig.DNSAddress()
 
 		err := s.DNS.Reconfigure(address)
 		if err != nil {
@@ -1019,7 +1012,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 	}
 
 	if lokiChanged {
-		lokiURL, lokiUsername, lokiPassword, lokiCACert, lokiInstance, lokiLoglevel, lokiLabels, lokiTypes := newClusterConfig.LokiServer()
+		lokiURL, lokiUsername, lokiPassword, lokiCACert, lokiInstance, lokiLoglevel, lokiLabels, lokiTypes := clusterConfig.LokiServer()
 
 		if lokiURL == "" || lokiLoglevel == "" || len(lokiTypes) == 0 {
 			d.internalListener.RemoveHandler("loki")
@@ -1048,7 +1041,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 	}
 
 	if oidcChanged {
-		oidcIssuer, oidcClientID, oidcClientSecret, oidcScopes, oidcAudience, oidcGroupsClaim := newClusterConfig.OIDCServer()
+		oidcIssuer, oidcClientID, oidcScopes, oidcAudience, oidcGroupsClaim := clusterConfig.OIDCServer()
 
 		if oidcIssuer == "" || oidcClientID == "" {
 			d.oidcVerifier = nil
@@ -1059,7 +1052,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 				return util.HTTPClient("", d.proxy)
 			}
 
-			d.oidcVerifier, err = oidc.NewVerifier(oidcIssuer, oidcClientID, oidcClientSecret, oidcScopes, oidcAudience, s.ServerCert, d.identityCache, httpClientFunc, &oidc.Opts{GroupsClaim: oidcGroupsClaim})
+			d.oidcVerifier, err = oidc.NewVerifier(oidcIssuer, oidcClientID, oidcScopes, oidcAudience, s.ServerCert, d.identityCache, httpClientFunc, &oidc.Opts{GroupsClaim: oidcGroupsClaim})
 			if err != nil {
 				return fmt.Errorf("Failed creating verifier: %w", err)
 			}
@@ -1067,7 +1060,7 @@ func doAPI10UpdateTriggers(d *Daemon, nodeChanged, clusterChanged map[string]str
 	}
 
 	if syslogSocketChanged {
-		err := d.setupSyslogSocket(newNodeConfig.SyslogSocket())
+		err := d.setupSyslogSocket(nodeConfig.SyslogSocket())
 		if err != nil {
 			return err
 		}
