@@ -44,12 +44,12 @@ func TestCluster_Get(t *testing.T) {
 	daemon, cleanup := newTestDaemon(t)
 	defer cleanup()
 
-	client, err := lxd.ConnectLXDUnix(daemon.os.GetUnixSocket(), nil)
+	client, err := lxd.ConnectLXDUnix(daemon.UnixSocket(), nil)
 	require.NoError(t, err)
 
 	cluster, _, err := client.GetCluster()
 	require.NoError(t, err)
-	assert.Empty(t, cluster.ServerName)
+	assert.Equal(t, "", cluster.ServerName)
 	assert.False(t, cluster.Enabled)
 }
 
@@ -97,7 +97,6 @@ func (f *clusterFixture) EnableNetworking(daemon *Daemon, password string) {
 	require.NoError(f.t, err)
 	serverPut := server.Writable()
 	serverPut.Config["core.https_address"] = address
-	serverPut.Config["core.trust_password"] = password
 
 	require.NoError(f.t, client.UpdateServer(serverPut, ""))
 }
@@ -116,7 +115,6 @@ func (f *clusterFixture) EnableNetworkingWithClusterAddress(daemon *Daemon, pass
 	require.NoError(f.t, err)
 	serverPut := server.Writable()
 	serverPut.Config["core.https_address"] = address
-	serverPut.Config["core.trust_password"] = password
 	serverPut.Config["cluster.https_address"] = address
 
 	require.NoError(f.t, client.UpdateServer(serverPut, ""))
@@ -132,7 +130,7 @@ func (f *clusterFixture) ClientUnix(daemon *Daemon) lxd.InstanceServer {
 	client, ok := f.clients[daemon]
 	if !ok {
 		var err error
-		client, err = lxd.ConnectLXDUnix(daemon.os.GetUnixSocket(), nil)
+		client, err = lxd.ConnectLXDUnix(daemon.UnixSocket(), nil)
 		require.NoError(f.t, err)
 	}
 
