@@ -4,7 +4,6 @@ import (
 	"context"
 	"database/sql"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
@@ -12,10 +11,8 @@ import (
 	"path/filepath"
 	"runtime"
 	runtimeDebug "runtime/debug"
-	"slices"
 	"strconv"
 	"strings"
-	"time"
 
 	"github.com/gorilla/mux"
 	"golang.org/x/sys/unix"
@@ -53,7 +50,6 @@ var apiInternal = []APIEndpoint{
 	internalClusterRebalanceCmd,
 	internalClusterHealCmd,
 	internalContainerOnStartCmd,
-	internalContainerOnStartHostCmd,
 	internalContainerOnStopCmd,
 	internalContainerOnStopNSCmd,
 	internalGarbageCollectorCmd,
@@ -66,7 +62,6 @@ var apiInternal = []APIEndpoint{
 	internalWarningCreateCmd,
 	internalIdentityCacheRefreshCmd,
 	internalPruneTokenCmd,
-	internalOperationWaitCmd,
 }
 
 var internalShutdownCmd = APIEndpoint{
@@ -87,12 +82,6 @@ var internalContainerOnStartCmd = APIEndpoint{
 	Get: APIEndpointAction{Handler: internalContainerOnStart, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
-var internalContainerOnStartHostCmd = APIEndpoint{
-	Path: "containers/{instanceRef}/onstarthost",
-
-	Get: APIEndpointAction{Handler: internalContainerOnStartHost, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
-}
-
 var internalContainerOnStopNSCmd = APIEndpoint{
 	Path: "containers/{instanceRef}/onstopns",
 
@@ -153,11 +142,6 @@ var internalPruneTokenCmd = APIEndpoint{
 	Post: APIEndpointAction{Handler: removeTokenHandler, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
 }
 
-var internalOperationWaitCmd = APIEndpoint{
-	Path: "testing/operation-wait",
-	Post: APIEndpointAction{Handler: operationWaitHandler, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanEdit)},
-}
-
 var internalIdentityCacheRefreshCmd = APIEndpoint{
 	Path: "identity-cache-refresh",
 
@@ -194,7 +178,7 @@ func internalCreateWarning(d *Daemon, r *http.Request) response.Response {
 		}
 	}
 
-	err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		return tx.UpsertWarning(ctx, req.Location, req.Project, req.EntityType, req.EntityID, warningtype.Type(req.TypeCode), req.Message)
 	})
 	if err != nil {
@@ -223,7 +207,7 @@ func internalOptimizeImage(d *Daemon, r *http.Request) response.Response {
 	return response.EmptySyncResponse
 }
 
-func internalRefreshImage(d *Daemon, _ *http.Request) response.Response {
+func internalRefreshImage(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	err := autoUpdateImages(s.ShutdownCtx, s)
@@ -238,63 +222,14 @@ func internalWaitReady(d *Daemon, r *http.Request) response.Response {
 	// Check that we're not shutting down.
 	isClosing := d.State().ShutdownCtx.Err() != nil
 	if isClosing {
-		return response.Unavailable(errors.New("LXD daemon is shutting down"))
+		return response.Unavailable(fmt.Errorf("LXD daemon is shutting down"))
 	}
 
-	timeoutSeconds := request.QueryParam(r, "timeout")
-	waitNetwork := shared.IsTrue(request.QueryParam(r, "network"))
-	waitStorage := shared.IsTrue(request.QueryParam(r, "storage"))
-
-	notReadyErr := errors.New("LXD daemon not ready yet")
-	networkNotReadyErr := errors.New("Networks not ready yet")
-	storageNotReadyErr := errors.New("Storage pools not ready yet")
-
-	if timeoutSeconds != "" {
-		timeoutSecondsUint, err := strconv.ParseUint(timeoutSeconds, 10, 32)
-		if err != nil {
-			return response.SmartError(fmt.Errorf("Invalid timeout: %w", err))
-		}
-
-		// If timeout is 0 then block indefinitely but if >0 use that as max wait seconds.
-		ctx := r.Context()
-		if timeoutSecondsUint > 0 {
-			var cancel context.CancelFunc
-			ctx, cancel = context.WithTimeout(ctx, time.Duration(timeoutSecondsUint)*time.Second)
-			defer cancel()
-		}
-
-		// If network is true, then block until all networks are marked as ready.
-		if waitNetwork {
-			select {
-			case <-d.waitNetworkReady.Done(): // Block until all networks are ready.
-			case <-ctx.Done(): // Don't leave this go routine around if client disconnects or timeout reached.
-				return response.Unavailable(networkNotReadyErr)
-			}
-		}
-
-		// If storage is true, then block until all storage pools are marked as ready.
-		if waitStorage {
-			select {
-			case <-d.waitStorageReady.Done(): // Block until all storage pools are ready.
-			case <-ctx.Done(): // Don't leave this go routine around if client disconnects or timeout reached.
-				return response.Unavailable(storageNotReadyErr)
-			}
-		}
-
-		select {
-		case <-d.waitReady.Done(): // Block until LXD is ready and then return EmptySyncResponse.
-		case <-ctx.Done(): // Don't leave this go routine around if client disconnects or timeout reached.
-			return response.Unavailable(notReadyErr)
-		}
-	} else if d.waitReady.Err() == nil {
-		return response.Unavailable(notReadyErr)
-	} else if waitNetwork && d.waitNetworkReady.Err() == nil {
-		return response.Unavailable(networkNotReadyErr)
-	} else if waitStorage && d.waitStorageReady.Err() == nil {
-		return response.Unavailable(storageNotReadyErr)
+	if d.waitReady.Err() == nil {
+		return response.Unavailable(fmt.Errorf("LXD daemon not ready yet"))
 	}
 
-	return response.EmptySyncResponse // LXD is ready.
+	return response.EmptySyncResponse
 }
 
 func internalShutdown(d *Daemon, r *http.Request) response.Response {
@@ -314,10 +249,7 @@ func internalShutdown(d *Daemon, r *http.Request) response.Response {
 	return response.ManualResponse(func(w http.ResponseWriter) error {
 		defer forceCtxCancel()
 
-		select {
-		case <-d.setupChan: // Wait for daemon to start.
-		case <-r.Context().Done(): // Don't leave this go routine around if client disconnects.
-		}
+		<-d.setupChan // Wait for daemon to start.
 
 		// Run shutdown sequence synchronously.
 		stopErr := d.Stop(forceCtx, unix.SIGPWR)
@@ -329,14 +261,15 @@ func internalShutdown(d *Daemon, r *http.Request) response.Response {
 		// Send the response before the LXD daemon process ends.
 		f, ok := w.(http.Flusher)
 		if !ok {
-			return errors.New("http.ResponseWriter is not type http.Flusher")
+			return fmt.Errorf("http.ResponseWriter is not type http.Flusher")
 		}
 
 		f.Flush()
 
+		// Send result of d.Stop() to cmdDaemon so that process stops with correct exit code from Stop().
 		go func() {
-			<-r.Context().Done()    // Wait until request is finished.
-			d.shutdownDoneCh <- nil // Send nil error to cmdDaemon to ensure LXD isn't restarted by systemd.
+			<-r.Context().Done() // Wait until request is finished.
+			d.shutdownDoneCh <- stopErr
 		}()
 
 		return nil
@@ -385,7 +318,7 @@ func internalContainerHookLoadFromReference(s *state.State, r *http.Request) (in
 	}
 
 	if inst.Type() != instancetype.Container {
-		return nil, errors.New("Instance is not container type")
+		return nil, fmt.Errorf("Instance is not container type")
 	}
 
 	return inst, nil
@@ -400,42 +333,9 @@ func internalContainerOnStart(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	l := logger.AddContext(logger.Ctx{"project": inst.Project().Name, "instance": inst.Name()})
-
 	err = inst.OnHook(instance.HookStart, nil)
 	if err != nil {
-		l.Error("The start hook failed", logger.Ctx{"err": err})
-		return response.SmartError(err)
-	}
-
-	return response.EmptySyncResponse
-}
-
-func internalContainerOnStartHost(d *Daemon, r *http.Request) response.Response {
-	s := d.State()
-
-	inst, err := internalContainerHookLoadFromReference(s, r)
-	if err != nil {
-		logger.Error("The start-host hook failed to load", logger.Ctx{"err": err})
-		return response.SmartError(err)
-	}
-
-	l := logger.AddContext(logger.Ctx{"project": inst.Project().Name, "instance": inst.Name()})
-
-	lxcPID := request.QueryParam(r, "lxc_pid")
-	if lxcPID == "" {
-		err := errors.New("No lxc_pid GET parameter was provided")
-		l.Error("The start-host hook failed", logger.Ctx{"err": err})
-		return response.BadRequest(err)
-	}
-
-	args := map[string]string{
-		"LXC_PID": lxcPID,
-	}
-
-	err = inst.OnHook(instance.HookStartHost, args)
-	if err != nil {
-		l.Error("The start-host hook failed", logger.Ctx{"err": err})
+		logger.Error("The start hook failed", logger.Ctx{"instance": inst.Name(), "err": err})
 		return response.SmartError(err)
 	}
 
@@ -451,8 +351,6 @@ func internalContainerOnStopNS(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	l := logger.AddContext(logger.Ctx{"project": inst.Project().Name, "instance": inst.Name()})
-
 	target := request.QueryParam(r, "target")
 	if target == "" {
 		target = "unknown"
@@ -467,7 +365,7 @@ func internalContainerOnStopNS(d *Daemon, r *http.Request) response.Response {
 
 	err = inst.OnHook(instance.HookStopNS, args)
 	if err != nil {
-		l.Error("The stopns hook failed", logger.Ctx{"err": err})
+		logger.Error("The stopns hook failed", logger.Ctx{"instance": inst.Name(), "err": err})
 		return response.SmartError(err)
 	}
 
@@ -483,8 +381,6 @@ func internalContainerOnStop(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	l := logger.AddContext(logger.Ctx{"project": inst.Project().Name, "instance": inst.Name()})
-
 	target := request.QueryParam(r, "target")
 	if target == "" {
 		target = "unknown"
@@ -496,7 +392,7 @@ func internalContainerOnStop(d *Daemon, r *http.Request) response.Response {
 
 	err = inst.OnHook(instance.HookStop, args)
 	if err != nil {
-		l.Error("The stop hook failed", logger.Ctx{"err": err})
+		logger.Error("The stop hook failed", logger.Ctx{"instance": inst.Name(), "err": err})
 		return response.SmartError(err)
 	}
 
@@ -513,7 +409,7 @@ type internalSQLQuery struct {
 }
 
 type internalSQLBatch struct {
-	Results []internalSQLResult `json:"Results" yaml:"results"`
+	Results []internalSQLResult
 }
 
 type internalSQLResult struct {
@@ -529,8 +425,8 @@ func internalSQLGet(d *Daemon, r *http.Request) response.Response {
 
 	database := r.FormValue("database")
 
-	if !slices.Contains([]string{"local", "global"}, database) {
-		return response.BadRequest(errors.New("Invalid database"))
+	if !shared.ValueInSlice(database, []string{"local", "global"}) {
+		return response.BadRequest(fmt.Errorf("Invalid database"))
 	}
 
 	schemaFormValue := r.FormValue("schema")
@@ -572,12 +468,12 @@ func internalSQLPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	if !slices.Contains([]string{"local", "global"}, req.Database) {
-		return response.BadRequest(errors.New("Invalid database"))
+	if !shared.ValueInSlice(req.Database, []string{"local", "global"}) {
+		return response.BadRequest(fmt.Errorf("Invalid database"))
 	}
 
 	if req.Query == "" {
-		return response.BadRequest(errors.New("No query provided"))
+		return response.BadRequest(fmt.Errorf("No query provided"))
 	}
 
 	var db *sql.DB
@@ -594,7 +490,7 @@ func internalSQLPost(d *Daemon, r *http.Request) response.Response {
 		return response.SyncResponse(true, batch)
 	}
 
-	for query := range strings.SplitSeq(req.Query, ";") {
+	for _, query := range strings.Split(req.Query, ";") {
 		query = strings.TrimLeft(query, " ")
 
 		if query == "" {
@@ -694,9 +590,9 @@ func internalSQLExec(tx *sql.Tx, query string, result *internalSQLResult) error
 // internalImportFromBackup creates instance, storage pool and volume DB records from an instance's backup file.
 // It expects the instance volume to be mounted so that the backup.yaml file is readable.
 // Also accepts an optional map of device overrides.
-func internalImportFromBackup(ctx context.Context, s *state.State, projectName string, instName string, allowNameOverride bool, deviceOverrides map[string]map[string]string) error {
+func internalImportFromBackup(s *state.State, projectName string, instName string, allowNameOverride bool, deviceOverrides map[string]map[string]string) error {
 	if instName == "" {
-		return errors.New("The name of the instance is required")
+		return fmt.Errorf("The name of the instance is required")
 	}
 
 	storagePoolsPath := shared.VarPath("storage-pools")
@@ -767,33 +663,28 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 		return err
 	}
 
-	if backupConf.Instance == nil {
-		return errors.New("Instance definition in backup config is missing")
+	if backupConf.Container == nil {
+		return fmt.Errorf("Instance definition in backup config is missing")
 	}
 
 	if allowNameOverride && instName != "" {
-		backupConf.Instance.Name = instName
+		backupConf.Container.Name = instName
 	}
 
-	if instName != backupConf.Instance.Name {
-		return fmt.Errorf("Requested instance name %q doesn't match instance name %q in backup config", instName, backupConf.Instance.Name)
+	if instName != backupConf.Container.Name {
+		return fmt.Errorf("Instance name requested %q doesn't match instance name in backup config %q", instName, backupConf.Container.Name)
 	}
 
-	if len(backupConf.Pools) == 0 {
+	if backupConf.Pool == nil {
 		// We don't know what kind of storage type the pool is.
-		return errors.New("No storage pool struct in the backup file found. The storage pool needs to be recovered manually")
-	}
-
-	rootVolPool, err := backupConf.RootVolumePool()
-	if err != nil {
-		return fmt.Errorf("Failed getting the root volume's pool: %w", err)
+		return fmt.Errorf("No storage pool struct in the backup file found. The storage pool needs to be recovered manually")
 	}
 
 	// Try to retrieve the storage pool the instance supposedly lives on.
 	pool, err := storagePools.LoadByName(s, instancePoolName)
 	if response.IsNotFoundError(err) {
 		// Create the storage pool db entry if it doesn't exist.
-		_, err = storagePoolDBCreate(ctx, s, instancePoolName, "", rootVolPool.Driver, rootVolPool.Config)
+		_, err = storagePoolDBCreate(s, instancePoolName, "", backupConf.Pool.Driver, backupConf.Pool.Config)
 		if err != nil {
 			return fmt.Errorf("Create storage pool database entry: %w", err)
 		}
@@ -806,12 +697,12 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 		return fmt.Errorf("Find storage pool database entry: %w", err)
 	}
 
-	if rootVolPool.Name != instancePoolName {
-		return fmt.Errorf(`The storage pool %q the instance was detected on does not match the storage pool %q specified in the backup file`, instancePoolName, rootVolPool.Name)
+	if backupConf.Pool.Name != instancePoolName {
+		return fmt.Errorf(`The storage pool %q the instance was detected on does not match the storage pool %q specified in the backup file`, instancePoolName, backupConf.Pool.Name)
 	}
 
-	if rootVolPool.Driver != pool.Driver().Info().Name {
-		return fmt.Errorf(`The storage pool's %q driver %q conflicts with the driver %q recorded in the instance's backup file`, instancePoolName, pool.Driver().Info().Name, rootVolPool.Driver)
+	if backupConf.Pool.Driver != pool.Driver().Info().Name {
+		return fmt.Errorf(`The storage pool's %q driver %q conflicts with the driver %q recorded in the instance's backup file`, instancePoolName, pool.Driver().Info().Name, backupConf.Pool.Driver)
 	}
 
 	// Check snapshots are consistent.
@@ -822,8 +713,8 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 
 	// Check if a storage volume entry for the instance already exists.
 	var dbVolume *db.StorageVolume
-	err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
-		dbVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, instanceDBVolType, backupConf.Instance.Name, true)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, instanceDBVolType, backupConf.Container.Name, true)
 		if err != nil && !response.IsNotFoundError(err) {
 			return err
 		}
@@ -835,12 +726,12 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 	}
 
 	if dbVolume != nil {
-		return fmt.Errorf(`Storage volume for instance %q already exists in the database`, backupConf.Instance.Name)
+		return fmt.Errorf(`Storage volume for instance %q already exists in the database`, backupConf.Container.Name)
 	}
 
-	err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Check if an entry for the instance already exists in the db.
-		_, err := tx.GetInstanceID(ctx, projectName, backupConf.Instance.Name)
+		_, err := tx.GetInstanceID(ctx, projectName, backupConf.Container.Name)
 
 		return err
 	})
@@ -849,30 +740,25 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 	}
 
 	if err == nil {
-		return fmt.Errorf(`Entry for instance %q already exists in the database`, backupConf.Instance.Name)
+		return fmt.Errorf(`Entry for instance %q already exists in the database`, backupConf.Container.Name)
 	}
 
-	if len(backupConf.Volumes) == 0 {
-		return errors.New(`No storage volume struct in the backup file found. The storage volume needs to be recovered manually`)
+	if backupConf.Volume == nil {
+		return fmt.Errorf(`No storage volume struct in the backup file found. The storage volume needs to be recovered manually`)
 	}
 
 	if dbVolume != nil {
-		rootVol, err := backupConf.RootVolume()
-		if err != nil {
-			return fmt.Errorf("Failed getting the root volume: %w", err)
+		if dbVolume.Name != backupConf.Volume.Name {
+			return fmt.Errorf(`The name %q of the storage volume is not identical to the instance's name "%s"`, dbVolume.Name, backupConf.Container.Name)
 		}
 
-		if dbVolume.Name != rootVol.Name {
-			return fmt.Errorf(`The storage volume name %q does not match the instance's name %q`, dbVolume.Name, backupConf.Instance.Name)
+		if dbVolume.Type != backupConf.Volume.Type {
+			return fmt.Errorf(`The type %q of the storage volume is not identical to the instance's type %q`, dbVolume.Type, backupConf.Volume.Type)
 		}
 
-		if dbVolume.Type != rootVol.Type {
-			return fmt.Errorf(`The storage volume type %q does not match the instance's type %q`, dbVolume.Type, rootVol.Type)
-		}
-
-		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			// Remove the storage volume db entry for the instance since force was specified.
-			return tx.RemoveStoragePoolVolume(ctx, projectName, backupConf.Instance.Name, instanceDBVolType, pool.ID())
+			return tx.RemoveStoragePoolVolume(ctx, projectName, backupConf.Container.Name, instanceDBVolType, pool.ID())
 		})
 		if err != nil {
 			return err
@@ -881,8 +767,8 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 
 	var profiles []api.Profile
 
-	err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
-		profiles, err = tx.GetProfiles(ctx, projectName, backupConf.Instance.Profiles)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		profiles, err = tx.GetProfiles(ctx, projectName, backupConf.Container.Profiles)
 
 		return err
 	})
@@ -891,32 +777,32 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 	}
 
 	// Initialise the devices maps.
-	if backupConf.Instance.Devices == nil {
-		backupConf.Instance.Devices = make(map[string]map[string]string, 0)
+	if backupConf.Container.Devices == nil {
+		backupConf.Container.Devices = make(map[string]map[string]string, 0)
 	}
 
-	if backupConf.Instance.ExpandedDevices == nil {
-		backupConf.Instance.ExpandedDevices = make(map[string]map[string]string, 0)
+	if backupConf.Container.ExpandedDevices == nil {
+		backupConf.Container.ExpandedDevices = make(map[string]map[string]string, 0)
 	}
 
 	// Apply device overrides.
 	// Do this before calling internalImportRootDevicePopulate so that device overrides are taken into account.
-	resultingDevices, err := shared.ApplyDeviceOverrides(backupConf.Instance.Devices, backupConf.Instance.ExpandedDevices, deviceOverrides)
+	resultingDevices, err := shared.ApplyDeviceOverrides(backupConf.Container.Devices, backupConf.Container.ExpandedDevices, deviceOverrides)
 	if err != nil {
 		return err
 	}
 
-	backupConf.Instance.Devices = resultingDevices
+	backupConf.Container.Devices = resultingDevices
 
 	// Add root device if needed.
 	// And ensure root device is associated with same pool as instance has been imported to.
-	internalImportRootDevicePopulate(instancePoolName, backupConf.Instance.Devices, backupConf.Instance.ExpandedDevices, profiles)
+	internalImportRootDevicePopulate(instancePoolName, backupConf.Container.Devices, backupConf.Container.ExpandedDevices, profiles)
 
 	revert := revert.New()
 	defer revert.Fail()
 
-	if backupConf.Instance == nil {
-		return errors.New("No instance config in backup config")
+	if backupConf.Container == nil {
+		return fmt.Errorf("No instance config in backup config")
 	}
 
 	instDBArgs, err := backup.ConfigToInstanceDBArgs(s, backupConf, projectName, true)
@@ -932,8 +818,11 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 	revert.Add(cleanup)
 	defer instOp.Done(err)
 
-	instancePath := storagePools.InstancePath(instanceType, projectName, backupConf.Instance.Name, false)
-	isPrivileged := backupConf.Instance.Config["security.privileged"] == ""
+	instancePath := storagePools.InstancePath(instanceType, projectName, backupConf.Container.Name, false)
+	isPrivileged := false
+	if backupConf.Container.Config["security.privileged"] == "" {
+		isPrivileged = true
+	}
 
 	err = storagePools.CreateContainerMountpoint(instanceMountPoint, instancePath, isPrivileged)
 	if err != nil {
@@ -941,11 +830,11 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 	}
 
 	for _, snap := range existingSnapshots {
-		snapInstName := backupConf.Instance.Name + shared.SnapshotDelimiter + snap.Name
+		snapInstName := backupConf.Container.Name + shared.SnapshotDelimiter + snap.Name
 
-		snapErr := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+		snapErr := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			// Check if an entry for the snapshot already exists in the db.
-			_, err := tx.GetInstanceSnapshotID(ctx, projectName, backupConf.Instance.Name, snap.Name)
+			_, err := tx.GetInstanceSnapshotID(ctx, projectName, backupConf.Container.Name, snap.Name)
 
 			return err
 		})
@@ -959,7 +848,7 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 
 		// Check if a storage volume entry for the snapshot already exists.
 		var dbVolume *db.StorageVolume
-		err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			dbVolume, err = tx.GetStoragePoolVolume(ctx, pool.ID(), projectName, instanceDBVolType, snapInstName, true)
 			if err != nil && !response.IsNotFoundError(err) {
 				return err
@@ -983,7 +872,7 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 			return err
 		}
 
-		err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			profiles, err = tx.GetProfiles(ctx, projectName, snap.Profiles)
 
 			return err
@@ -1028,7 +917,7 @@ func internalImportFromBackup(ctx context.Context, s *state.State, projectName s
 		// Recreate missing mountpoints and symlinks.
 		volStorageName := project.Instance(projectName, snapInstName)
 		snapshotMountPoint := storageDrivers.GetVolumeMountPath(instancePoolName, instanceVolType, volStorageName)
-		snapshotPath := storagePools.InstancePath(instanceType, projectName, backupConf.Instance.Name, true)
+		snapshotPath := storagePools.InstancePath(instanceType, projectName, backupConf.Container.Name, true)
 		snapshotTargetPath := storageDrivers.GetVolumeSnapshotDir(instancePoolName, instanceVolType, volStorageName)
 
 		err = storagePools.CreateSnapshotMountpoint(snapshotMountPoint, snapshotTargetPath, snapshotPath)
@@ -1122,7 +1011,7 @@ func internalImportRootDevicePopulate(instancePoolName string, localDevices map[
 		// If there is already a device called "root" in the instance's config, but it does not qualify as
 		// a root disk, then try to find a free name for the new root disk device.
 		rootDevName := "root"
-		for i := range 100 {
+		for i := 0; i < 100; i++ {
 			if localDevices[rootDevName] == nil {
 				break
 			}
@@ -1135,8 +1024,8 @@ func internalImportRootDevicePopulate(instancePoolName string, localDevices map[
 	}
 }
 
-func internalGC(_ *Daemon, _ *http.Request) response.Response {
-	logger.Info("Started forced garbage collection run")
+func internalGC(d *Daemon, r *http.Request) response.Response {
+	logger.Infof("Started forced garbage collection run")
 	runtime.GC()
 	runtimeDebug.FreeOSMemory()
 
@@ -1147,24 +1036,24 @@ func internalGC(_ *Daemon, _ *http.Request) response.Response {
 	logger.Infof("Requested from system: %s", units.GetByteSizeStringIEC(int64(m.Sys), 2))
 	logger.Infof("Releasable to OS: %s", units.GetByteSizeStringIEC(int64(m.HeapIdle-m.HeapReleased), 2))
 
-	logger.Info("Completed forced garbage collection run")
+	logger.Infof("Completed forced garbage collection run")
 
 	return response.EmptySyncResponse
 }
 
-func internalRAFTSnapshot(_ *Daemon, _ *http.Request) response.Response {
+func internalRAFTSnapshot(d *Daemon, r *http.Request) response.Response {
 	logger.Warn("Forced RAFT snapshot not supported")
 
-	return response.InternalError(errors.New("Not supported"))
+	return response.InternalError(fmt.Errorf("Not supported"))
 }
 
-func internalBGPState(d *Daemon, _ *http.Request) response.Response {
+func internalBGPState(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	return response.SyncResponse(true, s.BGP.Debug())
 }
 
-func internalIdentityCacheRefresh(d *Daemon, _ *http.Request) response.Response {
+func internalIdentityCacheRefresh(d *Daemon, r *http.Request) response.Response {
 	logger.Debug("Received identity cache update notification - refreshing cache")
 	d.State().UpdateIdentityCache()
 	return response.EmptySyncResponse
