@@ -6,7 +6,6 @@ import (
 	"errors"
 	"fmt"
 	"net/http"
-	"slices"
 
 	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/backup"
@@ -73,7 +72,7 @@ type internalRecoverImportPost struct {
 }
 
 // internalRecoverScan provides the discovery and import functionality for both recovery validate and import steps.
-func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.StoragePoolsPost, validateOnly bool) response.Response {
+func internalRecoverScan(s *state.State, userPools []api.StoragePoolsPost, validateOnly bool) response.Response {
 	var err error
 	var projects map[string]*api.Project
 	var projectProfiles map[string][]*api.Profile
@@ -81,7 +80,7 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 
 	// Retrieve all project, profile and network info in a single transaction so we can use it for all
 	// imported instances and volumes, and avoid repeatedly querying the same information.
-	err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Load list of projects for validation.
 		ps, err := dbCluster.GetProjects(ctx, tx.Tx())
 		if err != nil {
@@ -151,7 +150,7 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 	addDependencyError := func(err error) {
 		errStr := err.Error()
 
-		if !slices.Contains(res.DependencyErrors, errStr) {
+		if !shared.ValueInSlice(errStr, res.DependencyErrors) {
 			res.DependencyErrors = append(res.DependencyErrors, errStr)
 		}
 	}
@@ -173,7 +172,7 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 			// If the pool DB record doesn't exist, and we are clustered, then don't proceed
 			// any further as we do not support pool DB record recovery when clustered.
 			if s.ServerClustered {
-				return response.BadRequest(errors.New("Storage pool recovery not supported when clustered"))
+				return response.BadRequest(fmt.Errorf("Storage pool recovery not supported when clustered"))
 			}
 
 			// If pool doesn't exist in DB, initialise a temporary pool with the supplied info.
@@ -259,12 +258,12 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 			networkProjectName = project.NetworkProjectFromRecord(projectInfo)
 
 			for _, poolVol := range poolVols {
-				if poolVol.Instance == nil {
+				if poolVol.Container == nil {
 					continue // Skip dependency checks for non-instance volumes.
 				}
 
 				// Check that the instance's profile dependencies are met.
-				for _, poolInstProfileName := range poolVol.Instance.Profiles {
+				for _, poolInstProfileName := range poolVol.Container.Profiles {
 					foundProfile := false
 					for _, profile := range projectProfiles[profileProjectname] {
 						if profile.Name == poolInstProfileName {
@@ -278,7 +277,7 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 				}
 
 				// Check that the instance's NIC network dependencies are met.
-				for _, devConfig := range poolVol.Instance.ExpandedDevices {
+				for _, devConfig := range poolVol.Container.ExpandedDevices {
 					if devConfig["type"] != "nic" {
 						continue
 					}
@@ -313,9 +312,9 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 					var displaySnapshotCount int
 
 					// Build display fields for scan results.
-					if poolVol.Instance != nil {
-						displayType = poolVol.Instance.Type
-						displayName = poolVol.Instance.Name
+					if poolVol.Container != nil {
+						displayType = poolVol.Container.Type
+						displayName = poolVol.Container.Name
 						displaySnapshotCount = len(poolVol.Snapshots)
 					} else if poolVol.Bucket != nil {
 						displayType = "bucket"
@@ -323,14 +322,8 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 						displaySnapshotCount = 0
 					} else {
 						displayType = "volume"
-
-						customVol, err := poolVol.CustomVolume()
-						if err != nil {
-							return response.SmartError(fmt.Errorf("Failed getting the custom volume: %w", err))
-						}
-
-						displayName = customVol.Name
-						displaySnapshotCount = len(customVol.Snapshots)
+						displayName = poolVol.Volume.Name
+						displaySnapshotCount = len(poolVol.VolumeSnapshots)
 					}
 
 					res.UnknownVolumes = append(res.UnknownVolumes, internalRecoverValidateVolume{
@@ -360,20 +353,11 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 				poolVols = append(poolVols, value...)
 			}
 
-			var rootVolPool *api.StoragePool
-
 			// Search unknown volumes looking for an instance volume that can be used to
 			// restore the pool DB config from. This is preferable over using the user
 			// supplied config as it will include any additional settings not supplied.
 			for _, poolVol := range poolVols {
-				rootVolPool, err = poolVol.RootVolumePool()
-				if err != nil {
-					// We are looking for a volume with a pool configuration.
-					// Skip this volume if it doesn't contain what we are looking for.
-					continue
-				}
-
-				if rootVolPool.Config != nil {
+				if poolVol.Pool != nil && poolVol.Pool.Config != nil {
 					instPoolVol = poolVol
 					break // Stop search once we've found an instance with pool config.
 				}
@@ -381,8 +365,8 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 
 			if instPoolVol != nil {
 				// Create storage pool DB record from config in the instance.
-				logger.Info("Creating storage pool DB record from instance config", logger.Ctx{"name": rootVolPool.Name, "description": rootVolPool.Description, "driver": rootVolPool.Driver, "config": rootVolPool.Config})
-				poolID, err = dbStoragePoolCreateAndUpdateCache(ctx, s, rootVolPool.Name, rootVolPool.Description, rootVolPool.Driver, rootVolPool.Config)
+				logger.Info("Creating storage pool DB record from instance config", logger.Ctx{"name": instPoolVol.Pool.Name, "description": instPoolVol.Pool.Description, "driver": instPoolVol.Pool.Driver, "config": instPoolVol.Pool.Config})
+				poolID, err = dbStoragePoolCreateAndUpdateCache(s, instPoolVol.Pool.Name, instPoolVol.Pool.Description, instPoolVol.Pool.Driver, instPoolVol.Pool.Config)
 				if err != nil {
 					return response.SmartError(fmt.Errorf("Failed creating storage pool %q database entry: %w", pool.Name(), err))
 				}
@@ -392,19 +376,19 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 				poolDriverName := pool.Driver().Info().Name
 				poolDriverConfig := pool.Driver().Config()
 				logger.Info("Creating storage pool DB record from user config", logger.Ctx{"name": pool.Name(), "driver": poolDriverName, "config": poolDriverConfig})
-				poolID, err = dbStoragePoolCreateAndUpdateCache(ctx, s, pool.Name(), "", poolDriverName, poolDriverConfig)
+				poolID, err = dbStoragePoolCreateAndUpdateCache(s, pool.Name(), "", poolDriverName, poolDriverConfig)
 				if err != nil {
 					return response.SmartError(fmt.Errorf("Failed creating storage pool %q database entry: %w", pool.Name(), err))
 				}
 			}
 
 			revert.Add(func() {
-				_ = dbStoragePoolDeleteAndUpdateCache(context.Background(), s, pool.Name())
+				_ = dbStoragePoolDeleteAndUpdateCache(s, pool.Name())
 			})
 
 			// Set storage pool node to storagePoolCreated.
 			// Must come before storage pool is loaded from the database.
-			err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+			err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 				return tx.StoragePoolNodeCreated(poolID)
 			})
 			if err != nil {
@@ -438,21 +422,16 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 			// Recover unknown custom volumes (do this first before recovering instances so that any
 			// instances that reference unknown custom volume disk devices can be created).
 			for _, poolVol := range poolVols {
-				if poolVol.Instance != nil || poolVol.Bucket != nil {
+				if poolVol.Container != nil || poolVol.Bucket != nil {
 					continue // Skip instance volumes and buckets.
-				} else if poolVol.Instance == nil && len(poolVol.Volumes) == 0 {
-					return response.SmartError(errors.New("Volume is neither instance nor custom volume"))
-				}
-
-				rootVol, err := poolVol.RootVolume()
-				if err != nil {
-					response.SmartError(fmt.Errorf("Failed getting the root volume: %w", err))
+				} else if poolVol.Container == nil && poolVol.Volume == nil {
+					return response.SmartError(fmt.Errorf("Volume is neither instance nor custom volume"))
 				}
 
 				// Import custom volume and any snapshots.
 				cleanup, err := pool.ImportCustomVolume(customStorageProjectName, poolVol, nil)
 				if err != nil {
-					return response.SmartError(fmt.Errorf("Failed importing custom volume %q in project %q: %w", rootVol.Name, projectName, err))
+					return response.SmartError(fmt.Errorf("Failed importing custom volume %q in project %q: %w", poolVol.Volume.Name, projectName, err))
 				}
 
 				revert.Add(cleanup)
@@ -460,13 +439,13 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 
 			// Recover unknown instance volumes.
 			for _, poolVol := range poolVols {
-				if poolVol.Instance == nil && (len(poolVol.Volumes) != 0 || poolVol.Bucket != nil) {
+				if poolVol.Container == nil && (poolVol.Volume != nil || poolVol.Bucket != nil) {
 					continue // Skip custom volumes, invalid volumes and buckets.
 				}
 
 				// Recover instance volumes and any snapshots.
-				profiles := make([]api.Profile, 0, len(poolVol.Instance.Profiles))
-				for _, profileName := range poolVol.Instance.Profiles {
+				profiles := make([]api.Profile, 0, len(poolVol.Container.Profiles))
+				for _, profileName := range poolVol.Container.Profiles {
 					for i := range projectProfiles[profileProjectName] {
 						if projectProfiles[profileProjectName][i].Name == profileName {
 							profiles = append(profiles, *projectProfiles[profileProjectName][i])
@@ -476,7 +455,7 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 
 				inst, cleanup, err := internalRecoverImportInstance(s, pool, projectName, poolVol, profiles)
 				if err != nil {
-					return response.SmartError(fmt.Errorf("Failed creating instance %q record in project %q: %w", poolVol.Instance.Name, projectName, err))
+					return response.SmartError(fmt.Errorf("Failed creating instance %q record in project %q: %w", poolVol.Container.Name, projectName, err))
 				}
 
 				revert.Add(cleanup)
@@ -494,7 +473,7 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 
 					cleanup, err := internalRecoverImportInstanceSnapshot(s, pool, projectName, poolVol, poolInstSnap, profiles)
 					if err != nil {
-						return response.SmartError(fmt.Errorf("Failed creating instance %q snapshot %q record in project %q: %w", poolVol.Instance.Name, poolInstSnap.Name, projectName, err))
+						return response.SmartError(fmt.Errorf("Failed creating instance %q snapshot %q record in project %q: %w", poolVol.Container.Name, poolInstSnap.Name, projectName, err))
 					}
 
 					revert.Add(cleanup)
@@ -503,7 +482,7 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 				// Recreate instance mount path and symlinks (must come after snapshot recovery).
 				cleanup, err = pool.ImportInstance(inst, poolVol, nil)
 				if err != nil {
-					return response.SmartError(fmt.Errorf("Failed importing instance %q in project %q: %w", poolVol.Instance.Name, projectName, err))
+					return response.SmartError(fmt.Errorf("Failed importing instance %q in project %q: %w", poolVol.Container.Name, projectName, err))
 				}
 
 				revert.Add(cleanup)
@@ -514,7 +493,7 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 				if err == nil {
 					err = pool.SetInstanceQuota(inst, rootConfig["size"], rootConfig["size.state"], nil)
 					if err != nil {
-						return response.SmartError(fmt.Errorf("Failed reinitializing root disk quota %q for instance %q in project %q: %w", rootConfig["size"], poolVol.Instance.Name, projectName, err))
+						return response.SmartError(fmt.Errorf("Failed reinitializing root disk quota %q for instance %q in project %q: %w", rootConfig["size"], poolVol.Container.Name, projectName, err))
 					}
 				}
 			}
@@ -544,20 +523,20 @@ func internalRecoverScan(ctx context.Context, s *state.State, userPools []api.St
 // internalRecoverImportInstance recreates the database records for an instance and returns the new instance.
 // Returns a revert fail function that can be used to undo this function if a subsequent step fails.
 func internalRecoverImportInstance(s *state.State, pool storagePools.Pool, projectName string, poolVol *backupConfig.Config, profiles []api.Profile) (instance.Instance, revert.Hook, error) {
-	if poolVol.Instance == nil {
-		return nil, nil, errors.New("Pool volume is not an instance volume")
+	if poolVol.Container == nil {
+		return nil, nil, fmt.Errorf("Pool volume is not an instance volume")
 	}
 
 	// Add root device if needed.
-	if poolVol.Instance.Devices == nil {
-		poolVol.Instance.Devices = make(map[string]map[string]string, 0)
+	if poolVol.Container.Devices == nil {
+		poolVol.Container.Devices = make(map[string]map[string]string, 0)
 	}
 
-	if poolVol.Instance.ExpandedDevices == nil {
-		poolVol.Instance.ExpandedDevices = make(map[string]map[string]string, 0)
+	if poolVol.Container.ExpandedDevices == nil {
+		poolVol.Container.ExpandedDevices = make(map[string]map[string]string, 0)
 	}
 
-	internalImportRootDevicePopulate(pool.Name(), poolVol.Instance.Devices, poolVol.Instance.ExpandedDevices, profiles)
+	internalImportRootDevicePopulate(pool.Name(), poolVol.Container.Devices, poolVol.Container.ExpandedDevices, profiles)
 
 	dbInst, err := backup.ConfigToInstanceDBArgs(s, poolVol, projectName, true)
 	if err != nil {
@@ -565,7 +544,7 @@ func internalRecoverImportInstance(s *state.State, pool storagePools.Pool, proje
 	}
 
 	if dbInst.Type < 0 {
-		return nil, nil, errors.New("Invalid instance type")
+		return nil, nil, fmt.Errorf("Invalid instance type")
 	}
 
 	inst, instOp, cleanup, err := instance.CreateInternal(s, *dbInst, false)
@@ -580,8 +559,8 @@ func internalRecoverImportInstance(s *state.State, pool storagePools.Pool, proje
 
 // internalRecoverImportInstance recreates the database records for an instance snapshot.
 func internalRecoverImportInstanceSnapshot(s *state.State, pool storagePools.Pool, projectName string, poolVol *backupConfig.Config, snap *api.InstanceSnapshot, profiles []api.Profile) (revert.Hook, error) {
-	if poolVol.Instance == nil || snap == nil {
-		return nil, errors.New("Pool volume is not an instance volume")
+	if poolVol.Container == nil || snap == nil {
+		return nil, fmt.Errorf("Pool volume is not an instance volume")
 	}
 
 	// Add root device if needed.
@@ -600,7 +579,7 @@ func internalRecoverImportInstanceSnapshot(s *state.State, pool storagePools.Poo
 		return nil, err
 	}
 
-	instanceType, err := instancetype.New(poolVol.Instance.Type)
+	instanceType, err := instancetype.New(poolVol.Container.Type)
 	if err != nil {
 		return nil, err
 	}
@@ -627,7 +606,7 @@ func internalRecoverImportInstanceSnapshot(s *state.State, pool storagePools.Poo
 		Devices:      deviceConfig.NewDevices(snap.Devices),
 		Ephemeral:    snap.Ephemeral,
 		LastUsedDate: snap.LastUsedAt,
-		Name:         poolVol.Instance.Name + shared.SnapshotDelimiter + snap.Name,
+		Name:         poolVol.Container.Name + shared.SnapshotDelimiter + snap.Name,
 		Profiles:     profiles,
 		Stateful:     snap.Stateful,
 	}, false)
@@ -649,7 +628,7 @@ func internalRecoverValidate(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	return internalRecoverScan(r.Context(), d.State(), req.Pools, true)
+	return internalRecoverScan(d.State(), req.Pools, true)
 }
 
 // internalRecoverImport performs the pool volume recovery.
@@ -661,5 +640,5 @@ func internalRecoverImport(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	return internalRecoverScan(r.Context(), d.State(), req.Pools, false)
+	return internalRecoverScan(d.State(), req.Pools, false)
 }
