@@ -10,14 +10,13 @@ import (
 	"net"
 	"net/http"
 	"net/url"
-	"slices"
 	"strings"
 
 	"github.com/gorilla/mux"
 
 	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/db"
-	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/db/operationtype"
 	"github.com/canonical/lxd/lxd/lifecycle"
 	"github.com/canonical/lxd/lxd/network"
@@ -158,13 +157,13 @@ func projectsGet(d *Daemon, r *http.Request) response.Response {
 
 	var apiProjects []*api.Project
 	var projectURLs []string
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		allProjects, err := dbCluster.GetProjects(ctx, tx.Tx())
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		allProjects, err := cluster.GetProjects(ctx, tx.Tx())
 		if err != nil {
 			return err
 		}
 
-		projects := make([]dbCluster.Project, 0, len(allProjects))
+		projects := make([]cluster.Project, 0, len(allProjects))
 		for _, project := range allProjects {
 			if userHasPermission(entity.ProjectURL(project.Name)) {
 				projects = append(projects, project)
@@ -204,7 +203,7 @@ func projectsGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	for _, apiProject := range apiProjects {
-		apiProject.UsedBy = projecthelpers.FilterUsedBy(r.Context(), s.Authorizer, apiProject.UsedBy)
+		apiProject.UsedBy = projecthelpers.FilterUsedBy(s.Authorizer, r, apiProject.UsedBy)
 	}
 
 	if len(withEntitlements) > 0 {
@@ -225,7 +224,7 @@ func projectsGet(d *Daemon, r *http.Request) response.Response {
 
 // projectUsedBy returns a list of URLs for all instances, images, profiles,
 // storage volumes, networks, and acls that use this project.
-func projectUsedBy(ctx context.Context, tx *db.ClusterTx, project *dbCluster.Project) ([]string, error) {
+func projectUsedBy(ctx context.Context, tx *db.ClusterTx, project *cluster.Project) ([]string, error) {
 	reportedEntityTypes := []entity.Type{
 		entity.TypeInstance,
 		entity.TypeProfile,
@@ -235,7 +234,7 @@ func projectUsedBy(ctx context.Context, tx *db.ClusterTx, project *dbCluster.Pro
 		entity.TypeNetworkACL,
 	}
 
-	entityURLs, err := dbCluster.GetEntityURLs(ctx, tx.Tx(), project.Name, reportedEntityTypes...)
+	entityURLs, err := cluster.GetEntityURLs(ctx, tx.Tx(), project.Name, reportedEntityTypes...)
 	if err != nil {
 		return nil, fmt.Errorf("Failed to get project used-by URLs: %w", err)
 	}
@@ -295,7 +294,7 @@ func projectsPost(d *Daemon, r *http.Request) response.Response {
 		project.Config = map[string]string{}
 	}
 
-	for featureName, featureInfo := range dbCluster.ProjectFeatures {
+	for featureName, featureInfo := range cluster.ProjectFeatures {
 		_, ok := project.Config[featureName]
 		if !ok && featureInfo.DefaultEnabled {
 			project.Config[featureName] = "true"
@@ -320,25 +319,25 @@ func projectsPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	var id int64
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		id, err = dbCluster.CreateProject(ctx, tx.Tx(), dbCluster.Project{Description: project.Description, Name: project.Name})
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		id, err = cluster.CreateProject(ctx, tx.Tx(), cluster.Project{Description: project.Description, Name: project.Name})
 		if err != nil {
 			return fmt.Errorf("Failed adding database record: %w", err)
 		}
 
-		err = dbCluster.CreateProjectConfig(ctx, tx.Tx(), id, project.Config)
+		err = cluster.CreateProjectConfig(ctx, tx.Tx(), id, project.Config)
 		if err != nil {
 			return fmt.Errorf("Unable to create project config for project %q: %w", project.Name, err)
 		}
 
 		if shared.IsTrue(project.Config["features.profiles"]) {
-			err = projectCreateDefaultProfile(ctx, tx, project.Name, project.StoragePool, project.Network)
+			err = projectCreateDefaultProfile(tx, project.Name, project.StoragePool, project.Network)
 			if err != nil {
 				return err
 			}
 
 			if project.Config["features.images"] == "false" {
-				err = dbCluster.InitProjectWithoutImages(ctx, tx.Tx(), project.Name)
+				err = cluster.InitProjectWithoutImages(ctx, tx.Tx(), project.Name)
 				if err != nil {
 					return err
 				}
@@ -351,7 +350,7 @@ func projectsPost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Failed creating project %q: %w", project.Name, err))
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	lc := lifecycle.ProjectCreated.Event(project.Name, requestor, nil)
 	s.Events.SendLifecycle(project.Name, lc)
 
@@ -359,26 +358,26 @@ func projectsPost(d *Daemon, r *http.Request) response.Response {
 }
 
 // Create the default profile of a project.
-func projectCreateDefaultProfile(ctx context.Context, tx *db.ClusterTx, project string, storagePool string, network string) error {
+func projectCreateDefaultProfile(tx *db.ClusterTx, project string, storagePool string, network string) error {
 	// Create a default profile
-	profile := dbCluster.Profile{}
+	profile := cluster.Profile{}
 	profile.Project = project
 	profile.Name = api.ProjectDefaultName
 	profile.Description = "Default LXD profile for project " + project
 
-	profileID, err := dbCluster.CreateProfile(ctx, tx.Tx(), profile)
+	profileID, err := cluster.CreateProfile(context.TODO(), tx.Tx(), profile)
 	if err != nil {
 		return fmt.Errorf("Add default profile to database: %w", err)
 	}
 
-	devices := map[string]dbCluster.Device{}
+	devices := map[string]cluster.Device{}
 	if storagePool != "" {
 		rootDev := map[string]string{}
 		rootDev["path"] = "/"
 		rootDev["pool"] = storagePool
-		device := dbCluster.Device{
+		device := cluster.Device{
 			Name:   "root",
-			Type:   dbCluster.TypeDisk,
+			Type:   cluster.TypeDisk,
 			Config: rootDev,
 		}
 
@@ -388,9 +387,9 @@ func projectCreateDefaultProfile(ctx context.Context, tx *db.ClusterTx, project
 	if network != "" {
 		networkDev := map[string]string{}
 		networkDev["network"] = network
-		device := dbCluster.Device{
+		device := cluster.Device{
 			Name:   "eth0",
-			Type:   dbCluster.TypeNIC,
+			Type:   cluster.TypeNIC,
 			Config: networkDev,
 		}
 
@@ -398,7 +397,7 @@ func projectCreateDefaultProfile(ctx context.Context, tx *db.ClusterTx, project
 	}
 
 	if len(devices) > 0 {
-		err = dbCluster.CreateProfileDevices(context.TODO(), tx.Tx(), profileID, devices)
+		err = cluster.CreateProfileDevices(context.TODO(), tx.Tx(), profileID, devices)
 		if err != nil {
 			return fmt.Errorf("Add root device to default profile of new project: %w", err)
 		}
@@ -456,8 +455,8 @@ func projectGet(d *Daemon, r *http.Request) response.Response {
 
 	// Get the database entry
 	var project *api.Project
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		dbProject, err := dbCluster.GetProject(ctx, tx.Tx(), name)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbProject, err := cluster.GetProject(ctx, tx.Tx(), name)
 		if err != nil {
 			return err
 		}
@@ -486,7 +485,7 @@ func projectGet(d *Daemon, r *http.Request) response.Response {
 		project.Config,
 	}
 
-	project.UsedBy = projecthelpers.FilterUsedBy(r.Context(), s.Authorizer, project.UsedBy)
+	project.UsedBy = projecthelpers.FilterUsedBy(s.Authorizer, r, project.UsedBy)
 
 	return response.SyncResponseETag(true, project, etag)
 }
@@ -530,8 +529,8 @@ func projectPut(d *Daemon, r *http.Request) response.Response {
 
 	// Get the current data
 	var project *api.Project
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		dbProject, err := dbCluster.GetProject(ctx, tx.Tx(), name)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbProject, err := cluster.GetProject(ctx, tx.Tx(), name)
 		if err != nil {
 			return err
 		}
@@ -571,10 +570,10 @@ func projectPut(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	s.Events.SendLifecycle(project.Name, lifecycle.ProjectUpdated.Event(project.Name, requestor, nil))
 
-	return projectChange(r.Context(), s, project, req)
+	return projectChange(s, project, req)
 }
 
 // swagger:operation PATCH /1.0/projects/{name} projects project_patch
@@ -616,8 +615,8 @@ func projectPatch(d *Daemon, r *http.Request) response.Response {
 
 	// Get the current data
 	var project *api.Project
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		dbProject, err := dbCluster.GetProject(ctx, tx.Tx(), name)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		dbProject, err := cluster.GetProject(ctx, tx.Tx(), name)
 		if err != nil {
 			return err
 		}
@@ -687,14 +686,14 @@ func projectPatch(d *Daemon, r *http.Request) response.Response {
 		}
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	s.Events.SendLifecycle(project.Name, lifecycle.ProjectUpdated.Event(project.Name, requestor, nil))
 
-	return projectChange(r.Context(), s, project, req)
+	return projectChange(s, project, req)
 }
 
 // Common logic between PUT and PATCH.
-func projectChange(ctx context.Context, s *state.State, project *api.Project, req api.ProjectPut) response.Response {
+func projectChange(s *state.State, project *api.Project, req api.ProjectPut) response.Response {
 	// Make a list of config keys that have changed.
 	configChanged := []string{}
 	for key := range project.Config {
@@ -713,7 +712,7 @@ func projectChange(ctx context.Context, s *state.State, project *api.Project, re
 	// Record which features have been changed.
 	var featuresChanged []string
 	for _, configKeyChanged := range configChanged {
-		_, isFeature := dbCluster.ProjectFeatures[configKeyChanged]
+		_, isFeature := cluster.ProjectFeatures[configKeyChanged]
 		if isFeature {
 			featuresChanged = append(featuresChanged, configKeyChanged)
 		}
@@ -722,7 +721,7 @@ func projectChange(ctx context.Context, s *state.State, project *api.Project, re
 	// Quick checks.
 	if len(featuresChanged) > 0 {
 		if project.Name == api.ProjectDefaultName {
-			return response.BadRequest(errors.New("You can't change the features of the default project"))
+			return response.BadRequest(fmt.Errorf("You can't change the features of the default project"))
 		}
 
 		// Consider the project empty if it is only used by the default profile.
@@ -739,7 +738,7 @@ func projectChange(ctx context.Context, s *state.State, project *api.Project, re
 
 				// If feature is currently disabled, and it is being changed in the request, it
 				// must be being enabled. So check if feature can be enabled on non-empty projects.
-				if shared.IsFalse(project.Config[featureChanged]) && !dbCluster.ProjectFeatures[featureChanged].CanEnableNonEmpty {
+				if shared.IsFalse(project.Config[featureChanged]) && !cluster.ProjectFeatures[featureChanged].CanEnableNonEmpty {
 					return response.BadRequest(fmt.Errorf("Project feature %q cannot be enabled on non-empty projects", featureChanged))
 				}
 			}
@@ -759,28 +758,28 @@ func projectChange(ctx context.Context, s *state.State, project *api.Project, re
 			return err
 		}
 
-		err = dbCluster.UpdateProject(ctx, tx.Tx(), project.Name, req)
+		err = cluster.UpdateProject(ctx, tx.Tx(), project.Name, req)
 		if err != nil {
 			return fmt.Errorf("Persist profile changes: %w", err)
 		}
 
-		if slices.Contains(configChanged, "features.profiles") {
+		if shared.ValueInSlice("features.profiles", configChanged) {
 			if shared.IsTrue(req.Config["features.profiles"]) {
-				err = projectCreateDefaultProfile(ctx, tx, project.Name, "", "")
+				err = projectCreateDefaultProfile(tx, project.Name, "", "")
 				if err != nil {
 					return err
 				}
 			} else {
 				// Delete the project-specific default profile.
-				err = dbCluster.DeleteProfile(ctx, tx.Tx(), project.Name, api.ProjectDefaultName)
+				err = cluster.DeleteProfile(ctx, tx.Tx(), project.Name, api.ProjectDefaultName)
 				if err != nil {
 					return fmt.Errorf("Delete project default profile: %w", err)
 				}
 			}
 		}
 
-		if slices.Contains(configChanged, "features.images") && shared.IsFalse(req.Config["features.images"]) && shared.IsTrue(req.Config["features.profiles"]) {
-			err = dbCluster.InitProjectWithoutImages(ctx, tx.Tx(), project.Name)
+		if shared.ValueInSlice("features.images", configChanged) && shared.IsFalse(req.Config["features.images"]) && shared.IsTrue(req.Config["features.profiles"]) {
+			err = cluster.InitProjectWithoutImages(ctx, tx.Tx(), project.Name)
 			if err != nil {
 				return err
 			}
@@ -841,13 +840,13 @@ func projectPost(d *Daemon, r *http.Request) response.Response {
 
 	// Quick checks.
 	if name == api.ProjectDefaultName {
-		return response.Forbidden(errors.New("The 'default' project cannot be renamed"))
+		return response.Forbidden(fmt.Errorf("The 'default' project cannot be renamed"))
 	}
 
 	// Perform the rename.
 	run := func(op *operations.Operation) error {
 		err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			project, err := dbCluster.GetProject(ctx, tx.Tx(), req.Name)
+			project, err := cluster.GetProject(ctx, tx.Tx(), req.Name)
 			if err != nil && !response.IsNotFoundError(err) {
 				return fmt.Errorf("Failed checking if project %q exists: %w", req.Name, err)
 			}
@@ -856,7 +855,7 @@ func projectPost(d *Daemon, r *http.Request) response.Response {
 				return fmt.Errorf("A project named %q already exists", req.Name)
 			}
 
-			project, err = dbCluster.GetProject(ctx, tx.Tx(), name)
+			project, err = cluster.GetProject(ctx, tx.Tx(), name)
 			if err != nil {
 				return fmt.Errorf("Failed loading project %q: %w", name, err)
 			}
@@ -867,7 +866,7 @@ func projectPost(d *Daemon, r *http.Request) response.Response {
 			}
 
 			if !empty {
-				return errors.New("Only empty projects can be renamed")
+				return fmt.Errorf("Only empty projects can be renamed")
 			}
 
 			err = projecthelpers.ValidName(req.Name)
@@ -875,19 +874,19 @@ func projectPost(d *Daemon, r *http.Request) response.Response {
 				return err
 			}
 
-			return dbCluster.RenameProject(ctx, tx.Tx(), name, req.Name)
+			return cluster.RenameProject(ctx, tx.Tx(), name, req.Name)
 		})
 		if err != nil {
 			return err
 		}
 
-		requestor := request.CreateRequestor(r.Context())
+		requestor := request.CreateRequestor(r)
 		s.Events.SendLifecycle(req.Name, lifecycle.ProjectRenamed.Event(req.Name, requestor, logger.Ctx{"old_name": name}))
 
 		return nil
 	}
 
-	op, err := operations.OperationCreate(r.Context(), s, "", operations.OperationClassTask, operationtype.ProjectRename, nil, nil, run, nil, nil)
+	op, err := operations.OperationCreate(s, "", operations.OperationClassTask, operationtype.ProjectRename, nil, nil, run, nil, nil, r)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -923,11 +922,11 @@ func projectDelete(d *Daemon, r *http.Request) response.Response {
 
 	// Quick checks.
 	if name == api.ProjectDefaultName {
-		return response.Forbidden(errors.New("The 'default' project cannot be deleted"))
+		return response.Forbidden(fmt.Errorf("The 'default' project cannot be deleted"))
 	}
 
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		project, err := dbCluster.GetProject(ctx, tx.Tx(), name)
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		project, err := cluster.GetProject(ctx, tx.Tx(), name)
 		if err != nil {
 			return fmt.Errorf("Fetch project %q: %w", name, err)
 		}
@@ -938,17 +937,17 @@ func projectDelete(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if !empty {
-			return errors.New("Only empty projects can be removed")
+			return fmt.Errorf("Only empty projects can be removed")
 		}
 
-		return dbCluster.DeleteProject(ctx, tx.Tx(), name)
+		return cluster.DeleteProject(ctx, tx.Tx(), name)
 	})
 
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	s.Events.SendLifecycle(name, lifecycle.ProjectDeleted.Event(name, requestor, nil))
 
 	return response.EmptySyncResponse
@@ -1018,8 +1017,8 @@ func projectStateGet(d *Daemon, r *http.Request) response.Response {
 }
 
 // Check if a project is empty.
-func projectIsEmpty(ctx context.Context, project *dbCluster.Project, tx *db.ClusterTx) (bool, error) {
-	instances, err := dbCluster.GetInstances(ctx, tx.Tx(), dbCluster.InstanceFilter{Project: &project.Name})
+func projectIsEmpty(ctx context.Context, project *cluster.Project, tx *db.ClusterTx) (bool, error) {
+	instances, err := cluster.GetInstances(ctx, tx.Tx(), cluster.InstanceFilter{Project: &project.Name})
 	if err != nil {
 		return false, err
 	}
@@ -1028,7 +1027,7 @@ func projectIsEmpty(ctx context.Context, project *dbCluster.Project, tx *db.Clus
 		return false, nil
 	}
 
-	images, err := dbCluster.GetImages(ctx, tx.Tx(), dbCluster.ImageFilter{Project: &project.Name})
+	images, err := cluster.GetImages(ctx, tx.Tx(), cluster.ImageFilter{Project: &project.Name})
 	if err != nil {
 		return false, err
 	}
@@ -1037,13 +1036,17 @@ func projectIsEmpty(ctx context.Context, project *dbCluster.Project, tx *db.Clus
 		return false, nil
 	}
 
-	profiles, err := dbCluster.GetProfiles(ctx, tx.Tx(), dbCluster.ProfileFilter{Project: &project.Name})
+	profiles, err := cluster.GetProfiles(ctx, tx.Tx(), cluster.ProfileFilter{Project: &project.Name})
 	if err != nil {
 		return false, err
 	}
 
-	// Consider the project empty if it is only used by the default profile.
-	if len(profiles) > 1 || (len(profiles) == 1 && profiles[0].Name != "default") {
+	if len(profiles) > 0 {
+		// Consider the project empty if it is only used by the default profile.
+		if len(profiles) == 1 && profiles[0].Name == "default" {
+			return true, nil
+		}
+
 		return false, nil
 	}
 
@@ -1245,32 +1248,7 @@ func projectValidateConfig(s *state.State, config map[string]string, defaultNetw
 		// ---
 		//  type: string
 		//  shortdesc: Cluster groups that can be targeted
-		"restricted.cluster.groups": func(value string) error {
-			if value == "" {
-				return nil
-			}
-
-			groupNames := shared.SplitNTrimSpace(value, ",", -1, true)
-			return s.DB.Cluster.Transaction(s.ShutdownCtx, func(ctx context.Context, tx *db.ClusterTx) error {
-				groups, err := dbCluster.GetClusterGroups(ctx, tx.Tx())
-				if err != nil {
-					return fmt.Errorf("Failed to validate restricted cluster group configuration: %w", err)
-				}
-
-			outer:
-				for _, groupName := range groupNames {
-					for _, group := range groups {
-						if groupName == group.Name {
-							continue outer
-						}
-					}
-
-					return api.StatusErrorf(http.StatusNotFound, "Cluster group %q not found", groupName)
-				}
-
-				return nil
-			})
-		},
+		"restricted.cluster.groups": validate.Optional(validate.IsListOf(validate.IsAny)),
 		// lxdmeta:generate(entities=project; group=restricted; key=restricted.cluster.target)
 		// Possible values are `allow` or `block`.
 		// When set to `allow`, this option allows targeting of cluster members (either directly or via a group) when creating or moving instances.
@@ -1569,7 +1547,7 @@ func projectValidateConfig(s *state.State, config map[string]string, defaultNetw
 	// be bypassed by settings from the default project's profiles that are not checked against this project's
 	// restrictions when they are configured.
 	if shared.IsTrue(config["restricted"]) && shared.IsFalse(config["features.profiles"]) {
-		return errors.New("Projects without their own profiles cannot be restricted")
+		return fmt.Errorf("Projects without their own profiles cannot be restricted")
 	}
 
 	return nil
