@@ -6,13 +6,14 @@ import (
 	"fmt"
 	"net/http"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/util"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/lxd/util"
 )
 
-func authenticateAgentCert(d *Daemon, r *http.Request) (bool, instance.Instance, error) {
+func authenticateAgentCert(s *state.State, r *http.Request) (bool, instance.Instance, error) {
 	var vsockID int
 	trusted := false
 
@@ -23,7 +24,7 @@ func authenticateAgentCert(d *Daemon, r *http.Request) (bool, instance.Instance,
 
 	var clusterInst *cluster.Instance
 
-	err = d.db.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
 		clusterInst, err = tx.GetLocalInstanceWithVsockID(ctx, vsockID)
@@ -37,15 +38,16 @@ func authenticateAgentCert(d *Daemon, r *http.Request) (bool, instance.Instance,
 		return false, nil, err
 	}
 
-	inst, err := instance.LoadByProjectAndName(d.State(), clusterInst.Project, clusterInst.Name)
+	inst, err := instance.LoadByProjectAndName(s, clusterInst.Project, clusterInst.Name)
 	if err != nil {
 		return false, nil, err
 	}
 
 	agentCert := inst.(instance.VM).AgentCertificate()
+	trustedCerts := map[string]x509.Certificate{"0": *agentCert}
 
 	for _, cert := range r.TLS.PeerCertificates {
-		trusted, _ = util.CheckTrustState(*cert, map[string]x509.Certificate{"0": *agentCert}, nil, false)
+		trusted, _ = util.CheckMutualTLS(*cert, trustedCerts)
 		if trusted {
 			return true, inst, nil
 		}
