@@ -1,7 +1,6 @@
 package apparmor
 
 import (
-	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -43,7 +42,7 @@ func InstanceProfileName(inst instance) string {
 // InstanceNamespaceName returns the instance's AppArmor namespace.
 func InstanceNamespaceName(inst instance) string {
 	// Unlike in profile names, / isn't an allowed character so replace with a -.
-	path := strings.ReplaceAll(strings.Trim(shared.VarPath(""), "/"), "/", "-")
+	path := strings.Replace(strings.Trim(shared.VarPath(""), "/"), "/", "-", -1)
 	name := project.Instance(inst.Project().Name, inst.Name()) + "_<" + path + ">"
 	return profileName("", name)
 }
@@ -149,7 +148,7 @@ func instanceProfile(sysOS *sys.OS, inst instance) (string, error) {
 	rawContent := ""
 	rawApparmor, ok := inst.ExpandedConfig()["raw.apparmor"]
 	if ok {
-		for line := range strings.SplitSeq(strings.Trim(rawApparmor, "\n"), "\n") {
+		for _, line := range strings.Split(strings.Trim(rawApparmor, "\n"), "\n") {
 			rawContent += "  " + line + "\n"
 		}
 	}
@@ -204,7 +203,7 @@ func instanceProfile(sysOS *sys.OS, inst instance) (string, error) {
 
 		vmInst, ok := inst.(instanceVM)
 		if !ok {
-			return "", errors.New("Instance is not VM type")
+			return "", fmt.Errorf("Instance is not VM type")
 		}
 
 		// Get start time firmware path to allow access to it.
