@@ -1,11 +1,9 @@
 package apparmor
 
 import (
-	"fmt"
 	"strings"
 	"text/template"
 
-	"github.com/canonical/lxd/lxd/sys"
 	"github.com/canonical/lxd/shared"
 )
 
@@ -15,6 +13,9 @@ profile "{{ .name }}" flags=(attach_disconnected,mediate_deleted) {
   #include <abstractions/dbus>
   #include <abstractions/nameservice>
 
+  # Allow processes to send us signals by default
+  signal (receive),
+
   # Capabilities
   capability chown,
   capability net_bind_service,
@@ -77,14 +78,14 @@ profile "{{ .name }}" flags=(attach_disconnected,mediate_deleted) {
 `))
 
 // dnsmasqProfile generates the AppArmor profile template from the given network.
-func dnsmasqProfile(sysOS *sys.OS, n network) (string, error) {
+func dnsmasqProfile(n network) (string, error) {
 	rootPath := ""
 	if shared.InSnap() {
 		rootPath = "/var/lib/snapd/hostfs"
 	}
 
 	// Render the profile.
-	var sb *strings.Builder = &strings.Builder{}
+	var sb = &strings.Builder{}
 	err := dnsmasqProfileTpl.Execute(sb, map[string]any{
 		"name":        DnsmasqProfileName(n),
 		"networkName": n.Name(),
@@ -103,7 +104,7 @@ func dnsmasqProfile(sysOS *sys.OS, n network) (string, error) {
 // DnsmasqProfileName returns the AppArmor profile name.
 func DnsmasqProfileName(n network) string {
 	path := shared.VarPath("")
-	name := fmt.Sprintf("%s_<%s>", n.Name(), path)
+	name := n.Name() + "_<" + path + ">"
 	return profileName("dnsmasq", name)
 }
 
