@@ -4,19 +4,17 @@ import (
 	"archive/tar"
 	"bytes"
 	"context"
-	"errors"
 	"fmt"
 	"io"
 	"os"
 	"os/exec"
-	"strconv"
 	"strings"
 
 	"golang.org/x/sys/unix"
 
 	"github.com/canonical/lxd/lxd/apparmor"
-	"github.com/canonical/lxd/lxd/state"
 	"github.com/canonical/lxd/lxd/subprocess"
+	"github.com/canonical/lxd/lxd/sys"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/ioprogress"
 	"github.com/canonical/lxd/shared/logger"
@@ -34,7 +32,7 @@ func (nwc *nullWriteCloser) Close() error {
 // ExtractWithFds runs extractor process under specifc AppArmor profile.
 // The allowedCmds argument specify commands which are allowed to run by apparmor.
 // The cmd argument is automatically added to allowedCmds slice.
-func ExtractWithFds(s *state.State, cmd string, args []string, allowedCmds []string, stdin io.ReadCloser, output *os.File) error {
+func ExtractWithFds(cmd string, args []string, allowedCmds []string, stdin io.ReadCloser, sysOS *sys.OS, output *os.File) error {
 	outputPath := output.Name()
 
 	allowedCmds = append(allowedCmds, cmd)
@@ -48,13 +46,13 @@ func ExtractWithFds(s *state.State, cmd string, args []string, allowedCmds []str
 		allowedCmdPaths = append(allowedCmdPaths, cmdPath)
 	}
 
-	err := apparmor.ArchiveLoad(s, outputPath, allowedCmdPaths)
+	err := apparmor.ArchiveLoad(sysOS, outputPath, allowedCmdPaths)
 	if err != nil {
 		return fmt.Errorf("Failed to start extract: Failed to load profile: %w", err)
 	}
 
-	defer func() { _ = apparmor.ArchiveDelete(s.OS, outputPath) }()
-	defer func() { _ = apparmor.ArchiveUnload(s.OS, outputPath) }()
+	defer func() { _ = apparmor.ArchiveDelete(sysOS, outputPath) }()
+	defer func() { _ = apparmor.ArchiveUnload(sysOS, outputPath) }()
 
 	var buffer bytes.Buffer
 	p := subprocess.NewProcessWithFds(cmd, args, stdin, output, &nullWriteCloser{&buffer})
@@ -77,7 +75,7 @@ func ExtractWithFds(s *state.State, cmd string, args []string, allowedCmds []str
 // The unpacker arguments are those returned by DetectCompressionFile().
 // The returned cancelFunc should be called when finished with reader to clean up any resources used.
 // This can be done before reading to the end of the tarball if desired.
-func CompressedTarReader(s *state.State, ctx context.Context, r io.ReadSeeker, unpacker []string, outputPath string) (*tar.Reader, context.CancelFunc, error) {
+func CompressedTarReader(ctx context.Context, r io.ReadSeeker, unpacker []string, sysOS *sys.OS, outputPath string) (*tar.Reader, context.CancelFunc, error) {
 	ctx, cancelFunc := context.WithCancel(ctx)
 
 	_, err := r.Seek(0, io.SeekStart)
@@ -93,7 +91,7 @@ func CompressedTarReader(s *state.State, ctx context.Context, r io.ReadSeeker, u
 			return nil, cancelFunc, fmt.Errorf("Failed to start unpack: Failed to find executable: %w", err)
 		}
 
-		err = apparmor.ArchiveLoad(s, outputPath, []string{cmdPath})
+		err = apparmor.ArchiveLoad(sysOS, outputPath, []string{cmdPath})
 		if err != nil {
 			return nil, cancelFunc, fmt.Errorf("Failed to start unpack: Failed to load profile: %w", err)
 		}
@@ -114,8 +112,8 @@ func CompressedTarReader(s *state.State, ctx context.Context, r io.ReadSeeker, u
 			ctxCancelFunc()
 			_ = pipeWriter.Close()
 			_, _ = p.Wait(ctx)
-			_ = apparmor.ArchiveUnload(s.OS, outputPath)
-			_ = apparmor.ArchiveDelete(s.OS, outputPath)
+			_ = apparmor.ArchiveUnload(sysOS, outputPath)
+			_ = apparmor.ArchiveDelete(sysOS, outputPath)
 		}
 
 		tr = tar.NewReader(pipeReader)
@@ -127,7 +125,7 @@ func CompressedTarReader(s *state.State, ctx context.Context, r io.ReadSeeker, u
 }
 
 // Unpack extracts image from archive.
-func Unpack(s *state.State, file string, path string, blockBackend bool, tracker *ioprogress.ProgressTracker) error {
+func Unpack(file string, path string, blockBackend bool, sysOS *sys.OS, tracker *ioprogress.ProgressTracker) error {
 	extractArgs, extension, unpacker, err := shared.DetectCompression(file)
 	if err != nil {
 		return err
@@ -139,7 +137,7 @@ func Unpack(s *state.State, file string, path string, blockBackend bool, tracker
 	var reader io.Reader
 	if strings.HasPrefix(extension, ".tar") {
 		command = "tar"
-		if s.OS.RunningInUserNS {
+		if sysOS.RunningInUserNS {
 			// We can't create char/block devices so avoid extracting them.
 			args = append(args, "--anchored")
 			args = append(args, "--wildcards")
@@ -194,7 +192,7 @@ func Unpack(s *state.State, file string, path string, blockBackend bool, tracker
 		mem, err := shared.DeviceTotalMemory()
 		mem = mem / 1024 / 1024 / 10
 		if err == nil && mem < 256 {
-			memString := strconv.FormatInt(mem, 10)
+			memString := fmt.Sprint(mem)
 			args = append(args, "-da", memString, "-fr", memString, "-p", "1")
 		}
 
@@ -215,10 +213,10 @@ func Unpack(s *state.State, file string, path string, blockBackend bool, tracker
 		readCloser = io.NopCloser(reader)
 	}
 
-	err = ExtractWithFds(s, command, args, allowedCmds, readCloser, outputDir)
+	err = ExtractWithFds(command, args, allowedCmds, readCloser, sysOS, outputDir)
 	if err != nil {
 		// We can't create char/block devices in unpriv containers so ignore related errors.
-		if s.OS.RunningInUserNS && command == "unsquashfs" {
+		if sysOS.RunningInUserNS && command == "unsquashfs" {
 			runError, ok := err.(shared.RunError)
 			if !ok {
 				return err
@@ -231,7 +229,7 @@ func Unpack(s *state.State, file string, path string, blockBackend bool, tracker
 
 			// Confirm that all errors are related to character or block devices.
 			found := false
-			for line := range strings.SplitSeq(stdErr, "\n") {
+			for _, line := range strings.Split(stdErr, "\n") {
 				line = strings.TrimSpace(line)
 				if line == "" {
 					continue
@@ -266,10 +264,10 @@ func Unpack(s *state.State, file string, path string, blockBackend bool, tracker
 		// Check if we're running out of space
 		if int64(fs.Bfree) < 10 {
 			if blockBackend {
-				return errors.New("Unable to unpack image, run out of disk space (consider increasing your pool's volume.size)")
+				return fmt.Errorf("Unable to unpack image, run out of disk space (consider increasing your pool's volume.size)")
 			}
 
-			return errors.New("Unable to unpack image, run out of disk space")
+			return fmt.Errorf("Unable to unpack image, run out of disk space")
 		}
 
 		logger.Warn("Unpack failed", logger.Ctx{"file": file, "allowedCmds": allowedCmds, "extension": extension, "path": path, "err": err})
