@@ -4,11 +4,11 @@ package drivers
 
 import (
 	"context"
-	_ "embed" // embed is used to read the OpenFGA authorization model from openfga_model.openfga.
+	// embed is used to read the OpenFGA authorization model from openfga_model.openfga.
+	_ "embed"
 	"errors"
 	"fmt"
 	"net/http"
-	"slices"
 	"time"
 
 	"github.com/oklog/ulid/v2"
@@ -22,6 +22,7 @@ import (
 	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/identity"
 	"github.com/canonical/lxd/lxd/request"
+	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/entity"
 	"github.com/canonical/lxd/shared/logger"
@@ -55,7 +56,7 @@ var dummyDatastoreULID = ulid.Make().String()
 // load sets up the authorizer.
 func (e *embeddedOpenFGA) load(ctx context.Context, identityCache *identity.Cache, opts Opts) error {
 	if identityCache == nil {
-		return errors.New("Must provide certificate cache")
+		return fmt.Errorf("Must provide certificate cache")
 	}
 
 	e.identityCache = identityCache
@@ -75,7 +76,7 @@ func (e *embeddedOpenFGA) load(ctx context.Context, identityCache *identity.Cach
 	e.tlsAuthorizer = tlsDriver
 
 	if opts.openfgaDatastore == nil {
-		return errors.New("The OpenFGA datastore option must be set")
+		return fmt.Errorf("The OpenFGA datastore option must be set")
 	}
 
 	openfgaServerOptions := []server.OpenFGAServiceV1Option{
@@ -112,29 +113,7 @@ func (e *embeddedOpenFGA) load(ctx context.Context, identityCache *identity.Cach
 	return nil
 }
 
-// CheckPermission checks if the current requestor has the given entitlement on the given entity URL.
-func (e *embeddedOpenFGA) CheckPermission(ctx context.Context, entityURL *api.URL, entitlement auth.Entitlement) error {
-	return e.checkPermission(ctx, entityURL, entitlement, true)
-}
-
-// CheckPermissionWithoutEffectiveProject checks a permission without considering the effective project that is set in
-// the request context.
-func (e *embeddedOpenFGA) CheckPermissionWithoutEffectiveProject(ctx context.Context, entityURL *api.URL, entitlement auth.Entitlement) error {
-	return e.checkPermission(ctx, entityURL, entitlement, false)
-}
-
-// GetPermissionChecker returns an auth.PermissionChecker for the OpenFGA authorization driver.
-func (e *embeddedOpenFGA) GetPermissionChecker(ctx context.Context, entitlement auth.Entitlement, entityType entity.Type) (auth.PermissionChecker, error) {
-	return e.getPermissionChecker(ctx, entitlement, entityType, true)
-}
-
-// GetPermissionCheckerWithoutEffectiveProject returns an auth.PermissionChecker that does not consider the effective
-// project that is set in the request context.
-func (e *embeddedOpenFGA) GetPermissionCheckerWithoutEffectiveProject(ctx context.Context, entitlement auth.Entitlement, entityType entity.Type) (auth.PermissionChecker, error) {
-	return e.getPermissionChecker(ctx, entitlement, entityType, false)
-}
-
-// checkPermission checks whether the user who sent the request has the given entitlement on the given entity using the
+// CheckPermission checks whether the user who sent the request has the given entitlement on the given entity using the
 // embedded OpenFGA server. A http.StatusNotFound error is returned when the entity does not exist, or when the entity
 // exists but the caller does not have permission to view it. A http.StatusForbidden error is returned if the caller has
 // permission to view the entity, but does not have the given entitlement.
@@ -144,12 +123,11 @@ func (e *embeddedOpenFGA) GetPermissionCheckerWithoutEffectiveProject(ctx contex
 // contained within projects that do not have features enabled. For example, if the given entity URL is for a network in
 // project "foo", but project "foo" does not have `features.networks=true`, then we must not use project "foo" in our
 // authorization check because this network does not exist in the database. We will always expect the given entity URL
-// to contain the request project name, but we expect that effective project will be set in the request.Info in the
-// given context. The driver will rewrite the project name with the effective project name for the purpose of the
-// authorization check, but will not automatically allow "punching through" to the effective (default) project. An
-// administrator can allow specific permissions against those entities. This behaviour is turned off when
-// checkEffectiveProject is false.
-func (e *embeddedOpenFGA) checkPermission(ctx context.Context, entityURL *api.URL, entitlement auth.Entitlement, checkEffectiveProject bool) error {
+// to contain the request project name, but we expect that request.CtxEffectiveProjectName will be set in the request
+// context. The driver will rewrite the project name with the effective project name for the purpose of the authorization
+// check, but will not automatically allow "punching through" to the effective (default) project. An administrator can
+// allow specific permissions against those entities.
+func (e *embeddedOpenFGA) CheckPermission(ctx context.Context, entityURL *api.URL, entitlement auth.Entitlement) error {
 	entityType, projectName, location, pathArguments, err := entity.ParseURL(entityURL.URL)
 	if err != nil {
 		return fmt.Errorf("Failed to parse entity URL: %w", err)
@@ -204,20 +182,18 @@ func (e *embeddedOpenFGA) checkPermission(ctx context.Context, entityURL *api.UR
 		}
 
 		for _, lxdGroup := range lxdGroups {
-			if !slices.Contains(groups, lxdGroup) {
+			if !shared.ValueInSlice(lxdGroup, groups) {
 				groups = append(groups, lxdGroup)
 			}
 		}
 	}
 
-	if checkEffectiveProject {
-		// The project in the given URL may be for a project that does not have a feature enabled, in this case the auth check
-		// will fail because the resource doesn't actually exist in that project. To correct this, we use the effective project from
-		// the request context if present.
-		reqInfo := request.GetContextInfo(ctx)
-		if reqInfo.EffectiveProjectName != "" {
-			projectName = reqInfo.EffectiveProjectName
-		}
+	// The project in the given URL may be for a project that does not have a feature enabled, in this case the auth check
+	// will fail because the resource doesn't actually exist in that project. To correct this, we use the effective project from
+	// the request context if present.
+	effectiveProject, _ := request.GetCtxValue[string](ctx, request.CtxEffectiveProjectName)
+	if effectiveProject != "" {
+		projectName = effectiveProject
 	}
 
 	// Construct the URL in a standardised form (adding the project parameter if it was not present).
@@ -271,7 +247,6 @@ func (e *embeddedOpenFGA) checkPermission(ctx context.Context, entityURL *api.UR
 		// If we have a not found error from the underlying OpenFGADatastore we should mask it to make requests consistent.
 		// (all not found errors returned before an access control decision is made are masked to prevent discovery).
 		if api.StatusErrorCheck(err, http.StatusNotFound) {
-			l.Debug("Entity not found", logger.Ctx{"http_code": http.StatusNotFound})
 			return api.NewGenericStatusError(http.StatusNotFound)
 		}
 
@@ -303,7 +278,6 @@ func (e *embeddedOpenFGA) checkPermission(ctx context.Context, entityURL *api.UR
 				// If we have a not found error from the underlying OpenFGADatastore we should mask it to make requests consistent.
 				// (all not found errors returned before an access control decision is made are masked to prevent discovery).
 				if api.StatusErrorCheck(err, http.StatusNotFound) {
-					l.Debug("Entity not found", logger.Ctx{"http_code": http.StatusNotFound})
 					return api.NewGenericStatusError(http.StatusNotFound)
 				}
 
@@ -337,13 +311,13 @@ func (e *embeddedOpenFGA) checkPermission(ctx context.Context, entityURL *api.UR
 	return nil
 }
 
-// getPermissionChecker returns an auth.PermissionChecker using the embedded OpenFGA server.
+// GetPermissionChecker returns an auth.PermissionChecker using the embedded OpenFGA server.
 //
-// Note: As with checkPermission, we need to be careful about the usage of this function for entity types that may not
-// be enabled within a project. If the effective project must be considered, then it must be set on the request.Info
-// before the returned auth.PermissionChecker is called. If checkEffectiveProject is false, the effective project is not
-// considered.
-func (e *embeddedOpenFGA) getPermissionChecker(ctx context.Context, entitlement auth.Entitlement, entityType entity.Type, checkEffectiveProject bool) (auth.PermissionChecker, error) {
+// Note: As with CheckPermission, we need to be careful about the usage of this function for entity types that may not
+// be enabled within a project. For these cases request.CtxEffectiveProjectName must be set in the given context before
+// this function is called. The returned auth.PermissionChecker will expect entity URLs to contain the request URL. These
+// will be re-written to contain the effective project if set, so that they correspond to the list returned by OpenFGA.
+func (e *embeddedOpenFGA) GetPermissionChecker(ctx context.Context, entitlement auth.Entitlement, entityType entity.Type) (auth.PermissionChecker, error) {
 	logCtx := logger.Ctx{"entity_type": entityType, "entitlement": entitlement}
 	ctx, cancel := context.WithTimeout(ctx, 5*time.Second)
 	defer cancel()
@@ -413,7 +387,7 @@ func (e *embeddedOpenFGA) getPermissionChecker(ctx context.Context, entitlement
 		}
 
 		for _, lxdGroup := range lxdGroups {
-			if !slices.Contains(groups, lxdGroup) {
+			if !shared.ValueInSlice(lxdGroup, groups) {
 				groups = append(groups, lxdGroup)
 			}
 		}
@@ -483,14 +457,12 @@ func (e *embeddedOpenFGA) getPermissionChecker(ctx context.Context, entitlement
 			return false
 		}
 
-		if checkEffectiveProject {
-			// The project in the given URL may be for a project that does not have a feature enabled, in this case the auth check
-			// will fail because the resource doesn't actually exist in that project. To correct this, we use the effective project from
-			// the request context if present.
-			reqInfo := request.GetContextInfo(ctx)
-			if reqInfo.EffectiveProjectName != "" {
-				projectName = reqInfo.EffectiveProjectName
-			}
+		// The project in the given URL may be for a project that does not have a feature enabled, in this case the auth check
+		// will fail because the resource doesn't actually exist in that project. To correct this, we use the effective project from
+		// the request context if present.
+		effectiveProject, _ := request.GetCtxValue[string](ctx, request.CtxEffectiveProjectName)
+		if effectiveProject != "" {
+			projectName = effectiveProject
 		}
 
 		standardisedEntityURL, err := entityType.URL(projectName, location, pathArguments...)
@@ -500,7 +472,7 @@ func (e *embeddedOpenFGA) getPermissionChecker(ctx context.Context, entitlement
 		}
 
 		object := fmt.Sprintf("%s:%s", entityType, standardisedEntityURL.String())
-		return slices.Contains(objects, object)
+		return shared.ValueInSlice(object, objects)
 	}, nil
 }
 
