@@ -7,7 +7,6 @@ import (
 	"fmt"
 	"net/http"
 	"net/url"
-	"slices"
 	"strings"
 	"time"
 
@@ -22,6 +21,7 @@ import (
 	"github.com/canonical/lxd/lxd/request"
 	"github.com/canonical/lxd/lxd/response"
 	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/entity"
 )
@@ -379,7 +379,7 @@ func createAuthGroup(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Send a lifecycle event for the group creation
-	lc := lifecycle.AuthGroupCreated.Event(group.Name, request.CreateRequestor(r.Context()), nil)
+	lc := lifecycle.AuthGroupCreated.Event(group.Name, request.CreateRequestor(r), nil)
 	s.Events.SendLifecycle(api.ProjectDefaultName, lc)
 
 	return response.SyncResponseLocation(true, nil, entity.AuthGroupURL(group.Name).String())
@@ -564,7 +564,7 @@ func updateAuthGroup(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Send a lifecycle event for the group update
-	lc := lifecycle.AuthGroupUpdated.Event(groupName, request.CreateRequestor(r.Context()), nil)
+	lc := lifecycle.AuthGroupUpdated.Event(groupName, request.CreateRequestor(r), nil)
 	s.Events.SendLifecycle(api.ProjectDefaultName, lc)
 
 	return response.EmptySyncResponse
@@ -655,7 +655,7 @@ func patchAuthGroup(d *Daemon, r *http.Request) response.Response {
 
 		newPermissions := make([]api.Permission, 0, len(groupPut.Permissions))
 		for _, permission := range groupPut.Permissions {
-			if !slices.Contains(apiGroup.Permissions, permission) {
+			if !shared.ValueInSlice(permission, apiGroup.Permissions) {
 				newPermissions = append(newPermissions, permission)
 			}
 		}
@@ -672,7 +672,7 @@ func patchAuthGroup(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Send a lifecycle event for the group update
-	lc := lifecycle.AuthGroupUpdated.Event(groupName, request.CreateRequestor(r.Context()), nil)
+	lc := lifecycle.AuthGroupUpdated.Event(groupName, request.CreateRequestor(r), nil)
 	s.Events.SendLifecycle(api.ProjectDefaultName, lc)
 
 	return response.EmptySyncResponse
@@ -757,7 +757,7 @@ func renameAuthGroup(d *Daemon, r *http.Request) response.Response {
 	s.UpdateIdentityCache()
 
 	// Send a lifecycle event for the group rename
-	lc := lifecycle.AuthGroupRenamed.Event(groupPost.Name, request.CreateRequestor(r.Context()), map[string]any{"old_name": groupName})
+	lc := lifecycle.AuthGroupRenamed.Event(groupPost.Name, request.CreateRequestor(r), map[string]any{"old_name": groupName})
 	s.Events.SendLifecycle(api.ProjectDefaultName, lc)
 
 	return response.SyncResponseLocation(true, nil, entity.AuthGroupURL(groupPost.Name).String())
@@ -817,7 +817,7 @@ func deleteAuthGroup(d *Daemon, r *http.Request) response.Response {
 	s.UpdateIdentityCache()
 
 	// Send a lifecycle event for the group deletion
-	lc := lifecycle.AuthGroupDeleted.Event(groupName, request.CreateRequestor(r.Context()), nil)
+	lc := lifecycle.AuthGroupDeleted.Event(groupName, request.CreateRequestor(r), nil)
 	s.Events.SendLifecycle(api.ProjectDefaultName, lc)
 
 	return response.EmptySyncResponse
