@@ -9,7 +9,6 @@ import (
 	"encoding/base64"
 	"encoding/json"
 	"encoding/pem"
-	"errors"
 	"fmt"
 	"net"
 	"net/http"
@@ -23,7 +22,6 @@ import (
 	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/certificate"
 	"github.com/canonical/lxd/lxd/cluster"
-	clusterConfig "github.com/canonical/lxd/lxd/cluster/config"
 	"github.com/canonical/lxd/lxd/db"
 	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/db/operationtype"
@@ -36,7 +34,6 @@ import (
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/entity"
-	"github.com/canonical/lxd/shared/logger"
 	"github.com/canonical/lxd/shared/version"
 )
 
@@ -157,7 +154,7 @@ func certificatesGet(d *Daemon, r *http.Request) response.Response {
 		var baseCerts []dbCluster.Certificate
 		urlToCertificate := make(map[*api.URL]auth.EntitlementReporter)
 		var err error
-		err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			baseCerts, err = dbCluster.GetCertificates(ctx, tx.Tx())
 			if err != nil {
 				return err
@@ -210,7 +207,7 @@ func certificatesGet(d *Daemon, r *http.Request) response.Response {
 // clusterMemberJoinTokenValid searches for cluster join token that matches the join token provided.
 // Returns matching operation if found and cancels the operation, otherwise returns nil.
 func clusterMemberJoinTokenValid(s *state.State, r *http.Request, projectName string, joinToken *api.ClusterMemberJoinToken) (*api.Operation, error) {
-	ops, err := operationsGetByType(r.Context(), s, projectName, operationtype.ClusterJoinToken)
+	ops, err := operationsGetByType(s, r, projectName, operationtype.ClusterJoinToken)
 	if err != nil {
 		return nil, fmt.Errorf("Failed getting cluster join token operations: %w", err)
 	}
@@ -253,7 +250,7 @@ func clusterMemberJoinTokenValid(s *state.State, r *http.Request, projectName st
 
 	if foundOp != nil {
 		// Token is single-use, so cancel it now.
-		err = operationCancel(r.Context(), s, projectName, foundOp)
+		err = operationCancel(s, r, projectName, foundOp)
 		if err != nil {
 			return nil, fmt.Errorf("Failed to cancel operation %q: %w", foundOp.ID, err)
 		}
@@ -297,7 +294,7 @@ func clusterMemberJoinTokenValid(s *state.State, r *http.Request, projectName st
 // If an operation is found it is cancelled and the request metadata is returned. If no operation is found then a nil value
 // is returned. An error is only returned if an internal error occurs, or if a token is found but is not valid.
 func certificateTokenValid(s *state.State, r *http.Request, addToken *api.CertificateAddToken) (*api.CertificatesPost, error) {
-	ops, err := operationsGetByType(r.Context(), s, api.ProjectDefaultName, operationtype.CertificateAddToken)
+	ops, err := operationsGetByType(s, r, api.ProjectDefaultName, operationtype.CertificateAddToken)
 	if err != nil {
 		return nil, fmt.Errorf("Failed getting certificate token operations: %w", err)
 	}
@@ -340,7 +337,7 @@ func certificateTokenValid(s *state.State, r *http.Request, addToken *api.Certif
 	}
 
 	// Token is single-use, so cancel it now.
-	err = operationCancel(r.Context(), s, api.ProjectDefaultName, foundOp)
+	err = operationCancel(s, r, api.ProjectDefaultName, foundOp)
 	if err != nil {
 		return nil, fmt.Errorf("Failed to cancel operation %q: %w", foundOp.ID, err)
 	}
@@ -376,7 +373,7 @@ func certificateTokenValid(s *state.State, r *http.Request, addToken *api.Certif
 	// Certificate add tokens must have a request field in the metadata.
 	tokenReqAny, ok := foundOp.Metadata["request"]
 	if !ok {
-		return nil, errors.New(`Missing "request" key in certificate add operation data`)
+		return nil, fmt.Errorf(`Missing "request" key in certificate add operation data`)
 	}
 
 	var tokenReq api.CertificatesPost
@@ -412,7 +409,7 @@ func certificateTokenValid(s *state.State, r *http.Request, addToken *api.Certif
 //  Add a trusted certificate
 //
 //  Adds a certificate to the trust store as an untrusted user.
-//  In this mode, the `password` property must be set to the correct value.
+//  In this mode, the `token` property must be set to the correct value.
 //
 //  The `certificate` field can be omitted in which case the TLS client
 //  certificate in use for the connection will be retrieved and added to the
@@ -448,7 +445,7 @@ func certificateTokenValid(s *state.State, r *http.Request, addToken *api.Certif
 //	Add a trusted certificate
 //
 //	Adds a certificate to the trust store.
-//	In this mode, the `password` property is always ignored.
+//	In this mode, the `token` property is always ignored.
 //
 //	---
 //	consumes:
@@ -481,6 +478,10 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
+	if req.Password != "" { //nolint:staticcheck
+		return response.NotImplemented(fmt.Errorf("Password authentication is no longer supported, please update your client"))
+	}
+
 	// Validate name.
 	if strings.HasPrefix(req.Name, "-") || strings.Contains(req.Name, "/") {
 		return response.BadRequest(fmt.Errorf("Invalid certificate name %q, name must not start with a hyphen or contain forward slashes", req.Name))
@@ -490,36 +491,19 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 
 	// Quick check.
 	if req.Token && req.Certificate != "" {
-		return response.BadRequest(errors.New("Can't use certificate if token is requested"))
+		return response.BadRequest(fmt.Errorf("Can't use certificate if token is requested"))
 	}
 
 	if req.Token {
 		if req.Type != "client" {
-			return response.BadRequest(errors.New("Tokens can only be issued for client certificates"))
+			return response.BadRequest(fmt.Errorf("Tokens can only be issued for client certificates"))
 		}
 
 		if localHTTPSAddress == "" {
-			return response.BadRequest(errors.New("Can't issue token when server isn't listening on network"))
+			return response.BadRequest(fmt.Errorf("Can't issue token when server isn't listening on network"))
 		}
 	}
 
-	// Access check.
-	// Can't us s.GlobalConfig.TrustPassword() here as global config is not yet updated.
-	var secret string
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		config, err := clusterConfig.Load(ctx, tx)
-		if err != nil {
-			return err
-		}
-
-		secret = config.TrustPassword()
-
-		return nil
-	})
-	if err != nil {
-		return response.SmartError(err)
-	}
-
 	// Check if the caller has permission to create certificates.
 	var userCanCreateCertificates bool
 	err = s.Authorizer.CheckPermission(r.Context(), entity.ServerURL(), auth.EntitlementCanCreateIdentities)
@@ -529,14 +513,14 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 		userCanCreateCertificates = true
 	}
 
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo == nil {
-		return response.SmartError(errors.New("Failed to get authentication status: Missing request context info"))
+	trusted, err := request.GetCtxValue[bool](r.Context(), request.CtxTrusted)
+	if err != nil {
+		return response.SmartError(fmt.Errorf("Failed to get authentication status: %w", err))
 	}
 
 	// If caller is already trusted and the trust token is provided, we validate the token and cancel
 	// the corresponding token operation.
-	if reqInfo.Trusted && req.TrustToken != "" {
+	if trusted && req.TrustToken != "" {
 		// Decode the trust token.
 		joinToken, err := shared.CertificateTokenDecode(req.TrustToken)
 		if err != nil {
@@ -550,15 +534,15 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if tokenReq == nil {
-			return response.Forbidden(errors.New("No matching certificate add operation found"))
+			return response.Forbidden(fmt.Errorf("No matching certificate add operation found"))
 		}
 
-		return response.Conflict(errors.New("Client is already trusted"))
+		return response.Conflict(fmt.Errorf("Client is already trusted"))
 	}
 
 	// If caller is already trusted and does not have permission to create certificates, they cannot create more certificates.
-	if reqInfo.Trusted && !userCanCreateCertificates && req.Certificate == "" && !req.Token {
-		return response.BadRequest(errors.New("Client is already trusted"))
+	if trusted && !userCanCreateCertificates && req.Certificate == "" && !req.Token {
+		return response.BadRequest(fmt.Errorf("Client is already trusted"))
 	}
 
 	if !userCanCreateCertificates {
@@ -567,20 +551,13 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 			return response.Forbidden(nil)
 		}
 
-		// A password/token is required for non-admin users.
-		if req.Password == "" && req.TrustToken == "" { //nolint:staticcheck
+		// A token is required for non-admin users.
+		if req.TrustToken == "" {
 			return response.Forbidden(nil)
 		}
 
-		var joinTokenEncoded string
-		if req.Password != "" { //nolint:staticcheck
-			joinTokenEncoded = req.Password //nolint:staticcheck
-		} else if req.TrustToken != "" {
-			joinTokenEncoded = req.TrustToken
-		}
-
-		// Check if cluster member join token supplied as password or token.
-		joinToken, err := shared.JoinTokenDecode(joinTokenEncoded)
+		// Check if cluster member join token supplied as token.
+		joinToken, err := shared.JoinTokenDecode(req.TrustToken)
 		if err == nil {
 			// If so then check there is a matching join operation.
 			joinOp, err := clusterMemberJoinTokenValid(s, r, api.ProjectDefaultName, joinToken)
@@ -589,35 +566,31 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 			}
 
 			if joinOp == nil {
-				return response.Forbidden(errors.New("No matching cluster join operation found"))
+				return response.Forbidden(fmt.Errorf("No matching cluster join operation found"))
 			}
 		} else {
-			// Check if certificate add token supplied as password or token.
-			joinToken, err := shared.CertificateTokenDecode(joinTokenEncoded)
-			if err == nil {
-				// If so then check there is a matching join operation.
-				tokenReq, err := certificateTokenValid(s, r, joinToken)
-				if err != nil {
-					return response.SmartError(fmt.Errorf("Failed during search for certificate add token operation: %w", err))
-				}
+			// Check if certificate add token supplied as token.
+			joinToken, err := shared.CertificateTokenDecode(req.TrustToken)
+			if err != nil {
+				return response.Forbidden(nil)
+			}
 
-				if tokenReq == nil {
-					return response.Forbidden(errors.New("No matching certificate add operation found"))
-				}
+			// If so then check there is a matching join operation.
+			tokenReq, err := certificateTokenValid(s, r, joinToken)
+			if err != nil {
+				return response.SmartError(fmt.Errorf("Failed during search for certificate add token operation: %w", err))
+			}
 
-				// Create a new request from the token data as the user isn't allowed to override anything.
-				req = api.CertificatesPost{
-					Name:       tokenReq.Name,
-					Type:       tokenReq.Type,
-					Restricted: tokenReq.Restricted,
-					Projects:   tokenReq.Projects,
-				}
-			} else {
-				// Otherwise check if password matches trust password.
-				if util.PasswordCheck(secret, req.Password) != nil { //nolint:staticcheck
-					logger.Warn("Bad trust password", logger.Ctx{"url": r.URL.RequestURI(), "ip": r.RemoteAddr})
-					return response.Forbidden(nil)
-				}
+			if tokenReq == nil {
+				return response.Forbidden(fmt.Errorf("No matching certificate add operation found"))
+			}
+
+			// Create a new request from the token data as the user isn't allowed to override anything.
+			req = api.CertificatesPost{
+				Name:       tokenReq.Name,
+				Type:       tokenReq.Type,
+				Restricted: tokenReq.Restricted,
+				Projects:   tokenReq.Projects,
 			}
 		}
 	}
@@ -641,9 +614,27 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 			return response.BadRequest(fmt.Errorf("Invalid certificate material: %w", err))
 		}
 	} else if req.Token {
-		token, err := createCertificateAddToken(s, req.Name, "")
+		// Get all addresses the server is listening on. This is encoded in the certificate token,
+		// so that the client will not have to specify a server address. The client will iterate
+		// through all these addresses until it can connect to one of them.
+		addresses, err := util.ListenAddresses(localHTTPSAddress)
 		if err != nil {
-			return response.SmartError(fmt.Errorf("Failed to create certificate add token: %w", err))
+			return response.InternalError(err)
+		}
+
+		// Generate join secret for new client. This will be stored inside the join token operation and will be
+		// supplied by the joining client (encoded inside the join token) which will allow us to lookup the correct
+		// operation in order to validate the requested joining client name is correct and authorised.
+		joinSecret, err := shared.RandomCryptoString()
+		if err != nil {
+			return response.InternalError(err)
+		}
+
+		// Generate fingerprint of network certificate so joining member can automatically trust the correct
+		// certificate when it is presented during the join process.
+		fingerprint, err := shared.CertFingerprintStr(string(s.Endpoints.NetworkPublicKey()))
+		if err != nil {
+			return response.InternalError(err)
 		}
 
 		if req.Projects == nil {
@@ -651,9 +642,9 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 		}
 
 		meta := map[string]any{
-			"secret":      token.Secret,
-			"fingerprint": token.Fingerprint,
-			"addresses":   token.Addresses,
+			"secret":      joinSecret,
+			"fingerprint": fingerprint,
+			"addresses":   addresses,
 			"request":     req,
 		}
 
@@ -669,7 +660,7 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 			meta["expiresAt"] = expiresAt
 		}
 
-		op, err := operations.OperationCreate(r.Context(), s, api.ProjectDefaultName, operations.OperationClassToken, operationtype.CertificateAddToken, nil, meta, nil, nil, nil)
+		op, err := operations.OperationCreate(s, api.ProjectDefaultName, operations.OperationClassToken, operationtype.CertificateAddToken, nil, meta, nil, nil, nil, r)
 		if err != nil {
 			return response.InternalError(err)
 		}
@@ -681,12 +672,12 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 			// This can happen if the client doesn't send a client certificate or if the server is in
 			// CA mode. We rely on this check to prevent non-CA trusted client certificates from being
 			// added when in CA mode.
-			return response.BadRequest(errors.New("No client certificate provided"))
+			return response.BadRequest(fmt.Errorf("No client certificate provided"))
 		}
 
 		cert = r.TLS.PeerCertificates[len(r.TLS.PeerCertificates)-1]
 	} else {
-		return response.BadRequest(errors.New("Can't use TLS data on non-TLS link"))
+		return response.BadRequest(fmt.Errorf("Can't use TLS data on non-TLS link"))
 	}
 
 	// Check validity.
@@ -715,7 +706,7 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 		}
 	}
 
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Check if we already have the certificate.
 		existingCert, _ := dbCluster.GetCertificateByFingerprintPrefix(ctx, tx.Tx(), fingerprint)
 		if existingCert != nil {
@@ -761,7 +752,7 @@ func certificatesPost(d *Daemon, r *http.Request) response.Response {
 	// Reload the identity cache to add the new certificate.
 	s.UpdateIdentityCache()
 
-	lc := lifecycle.CertificateCreated.Event(fingerprint, request.CreateRequestor(r.Context()), nil)
+	lc := lifecycle.CertificateCreated.Event(fingerprint, request.CreateRequestor(r), nil)
 	s.Events.SendLifecycle(api.ProjectDefaultName, lc)
 
 	return response.SyncResponseLocation(true, nil, lc.Source)
@@ -814,7 +805,7 @@ func certificateGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	var cert *api.Certificate
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		dbCertInfo, err := dbCluster.GetCertificateByFingerprintPrefix(ctx, tx.Tx(), fingerprint)
 		if err != nil {
 			return err
@@ -879,7 +870,7 @@ func certificatePut(d *Daemon, r *http.Request) response.Response {
 
 	// Get current database record.
 	var apiEntry *api.Certificate
-	err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		oldEntry, err := dbCluster.GetCertificateByFingerprintPrefix(ctx, tx.Tx(), fingerprint)
 		if err != nil {
 			return err
@@ -946,7 +937,7 @@ func certificatePatch(d *Daemon, r *http.Request) response.Response {
 
 	// Get current database record.
 	var apiEntry *api.Certificate
-	err = d.State().DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		oldEntry, err := dbCluster.GetCertificateByFingerprintPrefix(ctx, tx.Tx(), fingerprint)
 		if err != nil {
 			return err
@@ -1006,17 +997,17 @@ func doCertificateUpdate(ctx context.Context, d *Daemon, dbInfo api.Certificate,
 	certProjects := req.Projects
 	if !userCanEditCertificate {
 		if r.TLS == nil {
-			response.Forbidden(errors.New("Cannot update certificate information"))
+			response.Forbidden(fmt.Errorf("Cannot update certificate information"))
 		}
 
 		// Ensure the user in not trying to change fields other than the certificate.
 		if dbInfo.Restricted != req.Restricted || dbInfo.Name != req.Name || len(dbInfo.Projects) != len(req.Projects) {
-			return response.Forbidden(errors.New("Only the certificate can be changed"))
+			return response.Forbidden(fmt.Errorf("Only the certificate can be changed"))
 		}
 
-		for i := range dbInfo.Projects {
+		for i := 0; i < len(dbInfo.Projects); i++ {
 			if dbInfo.Projects[i] != req.Projects[i] {
-				return response.Forbidden(errors.New("Only the certificate can be changed"))
+				return response.Forbidden(fmt.Errorf("Only the certificate can be changed"))
 			}
 		}
 
@@ -1054,7 +1045,7 @@ func doCertificateUpdate(ctx context.Context, d *Daemon, dbInfo api.Certificate,
 			}
 
 			if !trusted {
-				return response.Forbidden(errors.New("Certificate cannot be changed"))
+				return response.Forbidden(fmt.Errorf("Certificate cannot be changed"))
 			}
 		}
 	}
@@ -1114,7 +1105,7 @@ func doCertificateUpdate(ctx context.Context, d *Daemon, dbInfo api.Certificate,
 	// Reload the identity cache.
 	s.UpdateIdentityCache()
 
-	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.CertificateUpdated.Event(dbInfo.Fingerprint, request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.CertificateUpdated.Event(dbInfo.Fingerprint, request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
@@ -1146,7 +1137,7 @@ func certificateDelete(d *Daemon, r *http.Request) response.Response {
 	}
 
 	var certInfo *dbCluster.Certificate
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Get current database record.
 		var err error
 		certInfo, err = dbCluster.GetCertificateByFingerprintPrefix(ctx, tx.Tx(), fingerprint)
@@ -1160,11 +1151,6 @@ func certificateDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	// Disallow deletion of server certificates.
-	if certInfo.Type == certificate.TypeServer {
-		return response.BadRequest(errors.New("Cannot delete a server certificate. Remove the cluster member instead"))
-	}
-
 	var userCanEditCertificate bool
 	err = s.Authorizer.CheckPermission(r.Context(), entity.CertificateURL(certInfo.Fingerprint), auth.EntitlementCanDelete)
 	if err != nil && !auth.IsDeniedError(err) {
@@ -1176,7 +1162,7 @@ func certificateDelete(d *Daemon, r *http.Request) response.Response {
 	// Non-admins are able to delete only their own certificate.
 	if !userCanEditCertificate {
 		if r.TLS == nil {
-			response.Forbidden(errors.New("Cannot delete certificate"))
+			response.Forbidden(fmt.Errorf("Cannot delete certificate"))
 		}
 
 		certBlock, _ := pem.Decode([]byte(certInfo.Certificate))
@@ -1201,7 +1187,7 @@ func certificateDelete(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if !trusted {
-			return response.Forbidden(errors.New("Certificate cannot be deleted"))
+			return response.Forbidden(fmt.Errorf("Certificate cannot be deleted"))
 		}
 	}
 
@@ -1230,7 +1216,7 @@ func certificateDelete(d *Daemon, r *http.Request) response.Response {
 	// Reload the cache.
 	s.UpdateIdentityCache()
 
-	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.CertificateDeleted.Event(fingerprint, request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(api.ProjectDefaultName, lifecycle.CertificateDeleted.Event(fingerprint, request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
@@ -1255,7 +1241,7 @@ func certificateValidate(networkCert *shared.CertInfo, cert *x509.Certificate) e
 	if cert.PublicKeyAlgorithm == x509.RSA {
 		pubKey, ok := cert.PublicKey.(*rsa.PublicKey)
 		if !ok {
-			return errors.New("Unable to validate the RSA certificate")
+			return fmt.Errorf("Unable to validate the RSA certificate")
 		}
 
 		// Check that we're dealing with at least 2048bit (Size returns a value in bytes).
