@@ -2,20 +2,13 @@ package config
 
 import (
 	"context"
-	"crypto/rand"
-	"encoding/hex"
-	"errors"
 	"fmt"
-	"io"
-	"maps"
-	"slices"
 	"strconv"
 	"strings"
 	"time"
 
 	"github.com/sirupsen/logrus"
 	"github.com/zitadel/oidc/v3/pkg/oidc"
-	"golang.org/x/crypto/scrypt"
 
 	"github.com/canonical/lxd/lxd/config"
 	"github.com/canonical/lxd/lxd/db"
@@ -26,7 +19,8 @@ import (
 
 // Config holds cluster-wide configuration values.
 type Config struct {
-	m config.Map // Low-level map holding the config values.
+	tx *db.ClusterTx // DB transaction the values in this config are bound to.
+	m  config.Map    // Low-level map holding the config values.
 }
 
 // Load loads a new Config object with the current cluster configuration
@@ -43,7 +37,7 @@ func Load(ctx context.Context, tx *db.ClusterTx) (*Config, error) {
 		return nil, fmt.Errorf("failed to load node config: %w", err)
 	}
 
-	return &Config{m: m}, nil
+	return &Config{tx: tx, m: m}, nil
 }
 
 // BackupsCompressionAlgorithm returns the compression algorithm to use for backups.
@@ -81,11 +75,6 @@ func (c *Config) HTTPSAllowedCredentials() bool {
 	return c.m.GetBool("core.https_allowed_credentials")
 }
 
-// TrustPassword returns the LXD trust password for authenticating clients.
-func (c *Config) TrustPassword() string {
-	return c.m.GetString("core.trust_password")
-}
-
 // TrustCACertificates returns whether client certificates are checked
 // against a CA.
 func (c *Config) TrustCACertificates() bool {
@@ -235,8 +224,8 @@ func (c *Config) RemoteTokenExpiry() string {
 }
 
 // OIDCServer returns all the OpenID Connect settings needed to connect to a server.
-func (c *Config) OIDCServer() (issuer string, clientID string, clientSecret string, scopes []string, audience string, groupsClaim string) {
-	return c.m.GetString("oidc.issuer"), c.m.GetString("oidc.client.id"), c.m.GetString("oidc.client.secret"), strings.Fields(c.m.GetString("oidc.scopes")), c.m.GetString("oidc.audience"), c.m.GetString("oidc.groups.claim")
+func (c *Config) OIDCServer() (issuer string, clientID string, scopes []string, audience string, groupsClaim string) {
+	return c.m.GetString("oidc.issuer"), c.m.GetString("oidc.client.id"), strings.Fields(c.m.GetString("oidc.scopes")), c.m.GetString("oidc.audience"), c.m.GetString("oidc.groups.claim")
 }
 
 // ClusterHealingThreshold returns the configured healing threshold, i.e. the
@@ -268,27 +257,29 @@ func (c *Config) Dump() map[string]any {
 // Replace the current configuration with the given values.
 //
 // Return what has actually changed.
-func (c *Config) Replace(tx *db.ClusterTx, values map[string]any) (map[string]string, error) {
-	return c.update(tx, values)
+func (c *Config) Replace(values map[string]any) (map[string]string, error) {
+	return c.update(values)
 }
 
 // Patch changes only the configuration keys in the given map.
 //
 // Return what has actually changed.
-func (c *Config) Patch(tx *db.ClusterTx, patch map[string]any) (map[string]string, error) {
+func (c *Config) Patch(patch map[string]any) (map[string]string, error) {
 	values := c.Dump() // Use current values as defaults
-	maps.Copy(values, patch)
+	for name, value := range patch {
+		values[name] = value
+	}
 
-	return c.update(tx, values)
+	return c.update(values)
 }
 
-func (c *Config) update(tx *db.ClusterTx, values map[string]any) (map[string]string, error) {
+func (c *Config) update(values map[string]any) (map[string]string, error) {
 	changed, err := c.m.Change(values)
 	if err != nil {
 		return nil, err
 	}
 
-	err = tx.UpdateClusterConfig(changed)
+	err = c.tx.UpdateClusterConfig(changed)
 	if err != nil {
 		return nil, fmt.Errorf("cannot persist configuration changes: %w", err)
 	}
@@ -500,14 +491,6 @@ var ConfigSchema = config.Schema{
 	//  shortdesc: How long to wait before shutdown
 	"core.shutdown_timeout": {Type: config.Int64, Default: "5"},
 
-	// lxdmeta:generate(entities=server; group=core; key=core.trust_password)
-	//
-	// ---
-	//  type: string
-	//  scope: global
-	//  shortdesc: Password to be provided by clients to set up a trust
-	"core.trust_password": {Hidden: true, Setter: passwordSetter},
-
 	// lxdmeta:generate(entities=server; group=core; key=core.trust_ca_certificates)
 	//
 	// ---
@@ -604,7 +587,7 @@ var ConfigSchema = config.Schema{
 	//  type: string
 	//  scope: global
 	//  shortdesc: Password used for Loki authentication
-	"loki.auth.password": {Hidden: true},
+	"loki.auth.password": {},
 
 	// lxdmeta:generate(entities=server; group=loki; key=loki.api.ca_cert)
 	//
@@ -681,14 +664,6 @@ var ConfigSchema = config.Schema{
 	//  shortdesc: OpenID Connect client ID
 	"oidc.client.id": {},
 
-	// lxdmeta:generate(entities=server; group=oidc; key=oidc.client.secret)
-	//
-	// ---
-	//  type: string
-	//  scope: global
-	//  shortdesc: OpenID Connect client secret
-	"oidc.client.secret": {},
-
 	// lxdmeta:generate(entities=server; group=oidc; key=oidc.issuer)
 	//
 	// ---
@@ -720,7 +695,7 @@ var ConfigSchema = config.Schema{
 		Default: strings.Join([]string{oidc.ScopeOpenID, oidc.ScopeEmail, oidc.ScopeOfflineAccess, oidc.ScopeProfile}, " "),
 		Validator: validate.Optional(func(value string) error {
 			scopes := strings.Fields(value)
-			if !slices.Contains(scopes, oidc.ScopeOpenID) || !slices.Contains(scopes, oidc.ScopeEmail) {
+			if !shared.ValueInSlice(oidc.ScopeOpenID, scopes) || !shared.ValueInSlice(oidc.ScopeEmail, scopes) {
 				return fmt.Errorf("oidc.scopes requires the %q and %q OpenID Connect scopes", oidc.ScopeOpenID, oidc.ScopeEmail)
 			}
 
@@ -817,7 +792,7 @@ func offlineThresholdValidator(value string) error {
 	// which is the lower bound granularity of the offline check.
 	threshold, err := strconv.Atoi(value)
 	if err != nil {
-		return errors.New("Offline threshold is not a number")
+		return fmt.Errorf("Offline threshold is not a number")
 	}
 
 	if threshold <= minThreshold {
@@ -830,11 +805,11 @@ func offlineThresholdValidator(value string) error {
 func imageMinimalReplicaValidator(value string) error {
 	count, err := strconv.Atoi(value)
 	if err != nil {
-		return errors.New("Minimal image replica count is not a number")
+		return fmt.Errorf("Minimal image replica count is not a number")
 	}
 
 	if count < 1 && count != -1 {
-		return errors.New("Invalid value for image replica count")
+		return fmt.Errorf("Invalid value for image replica count")
 	}
 
 	return nil
@@ -843,11 +818,11 @@ func imageMinimalReplicaValidator(value string) error {
 func maxVotersValidator(value string) error {
 	n, err := strconv.Atoi(value)
 	if err != nil {
-		return errors.New("Value is not a number")
+		return fmt.Errorf("Value is not a number")
 	}
 
 	if n < 3 || n%2 != 1 {
-		return errors.New("Value must be an odd number equal to or higher than 3")
+		return fmt.Errorf("Value must be an odd number equal to or higher than 3")
 	}
 
 	return nil
@@ -856,36 +831,12 @@ func maxVotersValidator(value string) error {
 func maxStandByValidator(value string) error {
 	n, err := strconv.Atoi(value)
 	if err != nil {
-		return errors.New("Value is not a number")
+		return fmt.Errorf("Value is not a number")
 	}
 
 	if n < 0 || n > 5 {
-		return errors.New("Value must be between 0 and 5")
+		return fmt.Errorf("Value must be between 0 and 5")
 	}
 
 	return nil
 }
-
-func passwordSetter(value string) (string, error) {
-	// Nothing to do on unset
-	if value == "" {
-		return value, nil
-	}
-
-	// Hash the password
-	buf := make([]byte, 32)
-	_, err := io.ReadFull(rand.Reader, buf)
-	if err != nil {
-		return "", err
-	}
-
-	hash, err := scrypt.Key([]byte(value), buf, 1<<14, 8, 1, 64)
-	if err != nil {
-		return "", err
-	}
-
-	buf = append(buf, hash...)
-	value = hex.EncodeToString(buf)
-
-	return value, nil
-}
