@@ -7,8 +7,8 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	clusterConfig "github.com/lxc/lxd/lxd/cluster/config"
-	"github.com/lxc/lxd/lxd/db"
+	clusterConfig "github.com/canonical/lxd/lxd/cluster/config"
+	"github.com/canonical/lxd/lxd/db"
 )
 
 // The server configuration is initially empty.
@@ -62,7 +62,7 @@ func TestConfigLoad_OfflineThresholdValidator(t *testing.T) {
 	require.NoError(t, err)
 
 	_, err = config.Patch(map[string]any{"cluster.offline_threshold": "2"})
-	require.EqualError(t, err, "cannot set 'cluster.offline_threshold' to '2': Value must be greater than '10'")
+	require.EqualError(t, err, `Cannot set "cluster.offline_threshold" to "2": Value must be greater than 10`)
 }
 
 // Max number of voters must be odd.
@@ -74,7 +74,7 @@ func TestConfigLoad_MaxVotersValidator(t *testing.T) {
 	require.NoError(t, err)
 
 	_, err = config.Patch(map[string]any{"cluster.max_voters": "4"})
-	require.EqualError(t, err, "cannot set 'cluster.max_voters' to '4': Value must be an odd number equal to or higher than 3")
+	require.EqualError(t, err, `Cannot set "cluster.max_voters" to "4": Value must be an odd number equal to or higher than 3`)
 }
 
 // If some previously set values are missing from the ones passed to Replace(),
