@@ -24,14 +24,10 @@ import (
 // Connect is a convenience around lxd.ConnectLXD that configures the client
 // with the correct parameters for node-to-node communication.
 //
-// If a request context is passed (as defined by request.IsRequestContext) then the
-// identity info from the context is sent in the request, as well as respective proxy
-// environment variables.
-//
 // If 'notify' switch is true, then the user agent will be set to the special
 // to the UserAgentNotifier value, which can be used in some cases to distinguish
 // between a regular client request and an internal cluster request.
-func Connect(ctx context.Context, address string, networkCert *shared.CertInfo, serverCert *shared.CertInfo, notify bool) (lxd.InstanceServer, error) {
+func Connect(address string, networkCert *shared.CertInfo, serverCert *shared.CertInfo, r *http.Request, notify bool) (lxd.InstanceServer, error) {
 	// Wait for a connection to the events API first for non-notify connections.
 	if !notify {
 		ctx, cancel := context.WithTimeout(context.Background(), time.Duration(10)*time.Second)
@@ -54,25 +50,30 @@ func Connect(ctx context.Context, address string, networkCert *shared.CertInfo,
 		args.UserAgent = clusterRequest.UserAgentNotifier
 	}
 
-	reqInfo := request.GetContextInfo(ctx)
-	if reqInfo != nil {
+	if r != nil {
 		proxy := func(req *http.Request) (*url.URL, error) {
-			if reqInfo.SourceAddress != "" {
-				req.Header.Add(request.HeaderForwardedAddress, reqInfo.SourceAddress)
+			ctx := r.Context()
+
+			val, ok := ctx.Value(request.CtxUsername).(string)
+			if ok {
+				req.Header.Add(request.HeaderForwardedUsername, val)
 			}
 
-			if reqInfo.Username != "" {
-				req.Header.Add(request.HeaderForwardedUsername, reqInfo.Username)
+			val, ok = ctx.Value(request.CtxProtocol).(string)
+			if ok {
+				req.Header.Add(request.HeaderForwardedProtocol, val)
 			}
 
-			if reqInfo.Protocol != "" {
-				req.Header.Add(request.HeaderForwardedProtocol, reqInfo.Protocol)
-			}
+			req.Header.Add(request.HeaderForwardedAddress, r.RemoteAddr)
 
-			if reqInfo.IdentityProviderGroups != nil {
-				b, err := json.Marshal(reqInfo.IdentityProviderGroups)
-				if err == nil {
-					req.Header.Add(request.HeaderForwardedIdentityProviderGroups, string(b))
+			identityProviderGroupsAny := ctx.Value(request.CtxIdentityProviderGroups)
+			if ok {
+				identityProviderGroups, ok := identityProviderGroupsAny.([]string)
+				if ok {
+					b, err := json.Marshal(identityProviderGroups)
+					if err == nil {
+						req.Header.Add(request.HeaderForwardedIdentityProviderGroups, string(b))
+					}
 				}
 			}
 
@@ -89,14 +90,14 @@ func Connect(ctx context.Context, address string, networkCert *shared.CertInfo,
 // ConnectIfInstanceIsRemote figures out the address of the cluster member which is running the instance with the
 // given name in the specified project. If it's not the local member will connect to it and return the connected
 // client (configured with the specified project), otherwise it will just return nil.
-func ConnectIfInstanceIsRemote(ctx context.Context, s *state.State, projectName string, instName string, instanceType instancetype.Type) (lxd.InstanceServer, error) {
+func ConnectIfInstanceIsRemote(s *state.State, projectName string, instName string, r *http.Request, instanceType instancetype.Type) (lxd.InstanceServer, error) {
 	// No need to connect if not clustered.
 	if !s.ServerClustered {
 		return nil, nil
 	}
 
 	var address string // Cluster member address.
-	err := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 		address, err = tx.GetNodeAddressOfInstance(ctx, projectName, instName, instanceType)
 		return err
@@ -109,7 +110,7 @@ func ConnectIfInstanceIsRemote(ctx context.Context, s *state.State, projectName
 		return nil, nil // The instance is running on this local member, no need to connect.
 	}
 
-	client, err := Connect(ctx, address, s.Endpoints.NetworkCert(), s.ServerCert(), false)
+	client, err := Connect(address, s.Endpoints.NetworkCert(), s.ServerCert(), r, false)
 	if err != nil {
 		return nil, err
 	}
@@ -120,7 +121,7 @@ func ConnectIfInstanceIsRemote(ctx context.Context, s *state.State, projectName
 }
 
 // SetupTrust is a convenience around InstanceServer.CreateCertificate that adds the given server certificate to
-// the trusted pool of the cluster at the given address, using the given password. The certificate is added as
+// the trusted pool of the cluster at the given address, using the given token. The certificate is added as
 // type CertificateTypeServer to allow intra-member communication. If a certificate with the same fingerprint
 // already exists with a different name or type, then no error is returned.
 func SetupTrust(serverCert *shared.CertInfo, clusterPut api.ClusterPut) error {
@@ -146,7 +147,6 @@ func SetupTrust(serverCert *shared.CertInfo, clusterPut api.ClusterPut) error {
 		Projects:    cert.Projects,
 		Restricted:  cert.Restricted,
 		Certificate: cert.Certificate,
-		Password:    clusterPut.ClusterPassword,
 		TrustToken:  clusterPut.ClusterToken,
 	}
 
