@@ -5,7 +5,6 @@ import (
 	"context"
 	"crypto/tls"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"io"
 	"net"
@@ -17,8 +16,8 @@ import (
 	"sync"
 	"time"
 
-	"github.com/canonical/go-dqlite/v2"
-	"github.com/canonical/go-dqlite/v2/client"
+	"github.com/canonical/go-dqlite/v3"
+	"github.com/canonical/go-dqlite/v3/client"
 
 	"github.com/canonical/lxd/lxd/db"
 	"github.com/canonical/lxd/lxd/identity"
@@ -340,10 +339,10 @@ func (g *Gateway) HandlerFuncs(heartbeatHandler HeartbeatHandler, identityCache
 // WaitUpgradeNotification waits for a notification from another node that all
 // nodes in the cluster should now have been upgraded and have matching schema
 // and API versions.
-func (g *Gateway) WaitUpgradeNotification(ctx context.Context) {
+func (g *Gateway) WaitUpgradeNotification() {
 	select {
 	case <-g.upgradeCh:
-	case <-ctx.Done():
+	case <-time.After(time.Minute):
 	}
 }
 
@@ -489,7 +488,7 @@ func (g *Gateway) TransferLeadership() error {
 	}
 
 	if id == 0 {
-		return errors.New("No online voter found")
+		return fmt.Errorf("No online voter found")
 	}
 
 	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
@@ -521,39 +520,33 @@ func (g *Gateway) Shutdown() error {
 	logger.Info("Stop database gateway")
 
 	var err error
-	if g.server == nil {
-		return nil
+	if g.server != nil {
+		if g.info.Role == db.RaftVoter {
+			g.Sync()
+		}
+
+		err = g.server.Close()
+		close(g.stopCh)
+
+		// Unset the memory dial, since Shutdown() is also called for
+		// switching between in-memory and network mode.
+		g.lock.Lock()
+		g.memoryDial = nil
+		g.lock.Unlock()
+
+		// Record the raft term and index in the logs on every shutdown. This
+		// allows an administrator to determine the furthest-ahead cluster member
+		// in case recovery is needed.
+		lastEntryInfo, err := dqlite.ReadLastEntryInfo(g.db.Dir())
+		if err != nil {
+			return err
+		}
+
+		// This isn't really a warning, but it's important that this break through
+		// the snap's default log level of 'Warn'.
+		logger.Warn("Dqlite last entry", logger.Ctx{"term": lastEntryInfo.Term, "index": lastEntryInfo.Index})
 	}
 
-	if g.info.Role == db.RaftVoter {
-		g.Sync()
-	}
-
-	err = g.server.Close()
-	if err != nil {
-		logger.Error("Failed stopping dqlite", logger.Ctx{"err": err})
-	}
-
-	close(g.stopCh)
-
-	// Unset the memory dial, since Shutdown() is also called for
-	// switching between in-memory and network mode.
-	g.lock.Lock()
-	g.memoryDial = nil
-	g.lock.Unlock()
-
-	// Record the raft term and index in the logs on every shutdown. This
-	// allows an administrator to determine the furthest-ahead cluster member
-	// in case recovery is needed.
-	lastEntryInfo, err := dqlite.ReadLastEntryInfo(g.db.Dir())
-	if err != nil {
-		return err
-	}
-
-	// This isn't really a warning, but it's important that this break through
-	// the snap's default log level of 'Warn'.
-	logger.Warn("Dqlite last entry", logger.Ctx{"term": lastEntryInfo.Term, "index": lastEntryInfo.Index})
-
 	return err
 }
 
@@ -625,7 +618,7 @@ func (g *Gateway) Reset(networkCert *shared.CertInfo) error {
 }
 
 // ErrNodeIsNotClustered indicates the node is not clustered.
-var ErrNodeIsNotClustered = errors.New("Server is not clustered")
+var ErrNodeIsNotClustered = fmt.Errorf("Server is not clustered")
 
 // LeaderAddress returns the address of the current raft leader.
 func (g *Gateway) LeaderAddress() (string, error) {
@@ -702,7 +695,7 @@ func (g *Gateway) LeaderAddress() (string, error) {
 		// This should never happen because the raft_nodes table should
 		// be never empty for a clustered node, but check it for good
 		// measure.
-		return "", errors.New("No raft node known")
+		return "", fmt.Errorf("No raft node known")
 	}
 
 	transport, cleanup := tlsTransport(config)
@@ -719,7 +712,7 @@ func (g *Gateway) LeaderAddress() (string, error) {
 		return leader, nil
 	}
 
-	return "", errors.New("RAFT cluster is unavailable")
+	return "", fmt.Errorf("RAFT cluster is unavailable")
 }
 
 // NetworkUpdateCert sets a new network certificate for the gateway
@@ -736,7 +729,7 @@ func (g *Gateway) NetworkUpdateCert(cert *shared.CertInfo) {
 // @bootstrap should only be true when turning a non-clustered LXD instance into
 // the first (and leader) node of a new LXD cluster.
 func (g *Gateway) init(bootstrap bool) error {
-	logger.Debug("Initializing database gateway")
+	logger.Debugf("Initializing database gateway")
 	g.stopCh = make(chan struct{})
 
 	info, err := loadInfo(g.db)
@@ -746,7 +739,7 @@ func (g *Gateway) init(bootstrap bool) error {
 
 	dir := g.db.DqliteDir()
 	if shared.PathExists(filepath.Join(dir, "logs.db")) {
-		return errors.New("Unsupported upgrade path, please first upgrade to LXD 4.0")
+		return fmt.Errorf("Unsupported upgrade path, please first upgrade to LXD 4.0")
 	}
 
 	// If the resulting raft instance is not nil, it means that this node
@@ -839,7 +832,7 @@ func (g *Gateway) init(bootstrap bool) error {
 func (g *Gateway) WaitLeadership() error {
 	n := 80
 	sleep := 250 * time.Millisecond
-	for range n {
+	for i := 0; i < n; i++ {
 		g.lock.RLock()
 		isLeader, err := g.isLeader()
 		if err != nil {
@@ -882,7 +875,7 @@ func (g *Gateway) isLeader() (bool, error) {
 }
 
 // ErrNotLeader signals that a node not the leader.
-var ErrNotLeader = errors.New("Not leader")
+var ErrNotLeader = fmt.Errorf("Not leader")
 
 // Return information about the LXD nodes that a currently part of the raft
 // cluster, as configured in the raft log. It returns an error if this node is
@@ -1002,7 +995,7 @@ func attemptGetLeaderAddressFromNodeAddress(ctx context.Context, transport http.
 	}
 
 	url := "https://" + nodeAddress + databaseEndpoint
-	request, err := http.NewRequest(http.MethodGet, url, nil)
+	request, err := http.NewRequest("GET", url, nil)
 	if err != nil {
 		return "", err
 	}
@@ -1115,7 +1108,7 @@ func dqliteNetworkDial(ctx context.Context, name string, addr string, g *Gateway
 				g.upgradeTriggered = true
 			}
 		}
-		return nil, errors.New("Upgrade needed")
+		return nil, fmt.Errorf("Upgrade needed")
 	}
 
 	if response.StatusCode != http.StatusSwitchingProtocols {
@@ -1123,7 +1116,7 @@ func dqliteNetworkDial(ctx context.Context, name string, addr string, g *Gateway
 	}
 
 	if response.Header.Get("Upgrade") != "dqlite" {
-		return nil, errors.New("Missing or unexpected Upgrade header in response")
+		return nil, fmt.Errorf("Missing or unexpected Upgrade header in response")
 	}
 
 	revert.Success()
