@@ -33,21 +33,21 @@ const (
 
 // APIHeartbeatMember contains specific cluster node info.
 type APIHeartbeatMember struct {
-	ID            int64            `json:"ID"`            // ID field value in nodes table.
-	Address       string           `json:"Address"`       // Host and Port of node.
-	Name          string           `json:"Name"`          // Name of cluster member.
-	RaftID        uint64           `json:"RaftID"`        // ID field value in raft_nodes table, zero if non-raft node.
-	RaftRole      int              `json:"RaftRole"`      // Node role in the raft cluster, from the raft_nodes table
-	LastHeartbeat time.Time        `json:"LastHeartbeat"` // Last time we received a successful response from node.
-	Online        bool             `json:"Online"`        // Calculated from offline threshold and LastHeatbeat time.
-	Roles         []db.ClusterRole `json:"Roles"`         // Supplementary non-database roles the member has.
+	ID            int64            // ID field value in nodes table.
+	Address       string           // Host and Port of node.
+	Name          string           // Name of cluster member.
+	RaftID        uint64           // ID field value in raft_nodes table, zero if non-raft node.
+	RaftRole      int              // Node role in the raft cluster, from the raft_nodes table
+	LastHeartbeat time.Time        // Last time we received a successful response from node.
+	Online        bool             // Calculated from offline threshold and LastHeatbeat time.
+	Roles         []db.ClusterRole // Supplementary non-database roles the member has.
 	updated       bool             // Has node been updated during this heartbeat run. Not sent to nodes.
 }
 
 // APIHeartbeatVersion contains max versions for all nodes in cluster.
 type APIHeartbeatVersion struct {
-	Schema        int `json:"Schema"`
-	APIExtensions int `json:"APIExtensions"`
+	Schema        int
+	APIExtensions int
 }
 
 // NewAPIHearbeat returns initialised APIHeartbeat.
@@ -61,14 +61,14 @@ func NewAPIHearbeat(cluster *db.Cluster) *APIHeartbeat {
 type APIHeartbeat struct {
 	sync.Mutex // Used to control access to Members maps.
 	cluster    *db.Cluster
-	Members    map[int64]APIHeartbeatMember `json:"Members"`
-	Version    APIHeartbeatVersion          `json:"Version"`
-	Time       time.Time                    `json:"Time"`
+	Members    map[int64]APIHeartbeatMember
+	Version    APIHeartbeatVersion
+	Time       time.Time
 
 	// Indicates if heartbeat contains a fresh set of node states.
 	// This can be used to indicate to the receiving node that the state is fresh enough to
 	// trigger node refresh activies (such as forkdns).
-	FullStateList bool `json:"FullStateList"`
+	FullStateList bool
 }
 
 // Update updates an existing APIHeartbeat struct with the raft and all node states supplied.
@@ -284,17 +284,17 @@ func (g *Gateway) HeartbeatRestart() bool {
 }
 
 func (g *Gateway) heartbeat(ctx context.Context, mode heartbeatMode) {
+	if g.Cluster == nil || g.server == nil || g.memoryDial != nil {
+		// We're not a raft node or we're not clustered
+		return
+	}
+
 	// Avoid concurent heartbeat loops.
 	// This is possible when both the regular task and the out of band heartbeat round from a dqlite
 	// connection or notification restart both kick in at the same time.
 	g.HeartbeatLock.Lock()
 	defer g.HeartbeatLock.Unlock()
 
-	if g.Cluster == nil || g.server == nil || g.memoryDial != nil {
-		// We're not a raft node or we're not clustered
-		return
-	}
-
 	// Acquire the cancellation lock and populate it so that this heartbeat round can be cancelled if a
 	// notification cancellation request arrives during the round. Also setup a defer so that the cancellation
 	// function is set to nil when this function ends to indicate there is no ongoing heartbeat round.
@@ -457,7 +457,7 @@ func (g *Gateway) heartbeat(ctx context.Context, mode heartbeatMode) {
 	err = query.Retry(ctx, func(ctx context.Context) error {
 		// Durating cluster member fluctuations/upgrades the cluster can become unavailable so check here.
 		if g.Cluster == nil {
-			return errors.New("Cluster unavailable")
+			return fmt.Errorf("Cluster unavailable")
 		}
 
 		return g.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
@@ -536,7 +536,7 @@ func HeartbeatNode(taskCtx context.Context, address string, networkCert *shared.
 		return err
 	}
 
-	request, err := http.NewRequest(http.MethodPut, url, bytes.NewReader(buffer.Bytes()))
+	request, err := http.NewRequest("PUT", url, bytes.NewReader(buffer.Bytes()))
 	if err != nil {
 		return err
 	}
