@@ -3,17 +3,16 @@ package cluster
 import (
 	"context"
 	"os"
-	"path/filepath"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/node"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/node"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 // Load information about the dqlite node associated with this LXD member
 // should have, such as its ID, address and role.
-func loadInfo(database *db.Node, cert *shared.CertInfo) (*db.RaftNode, error) {
+func loadInfo(database *db.Node) (*db.RaftNode, error) {
 	// Figure out if we actually need to act as dqlite node.
 	var info *db.RaftNode
 	err := database.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
@@ -38,7 +37,7 @@ func loadInfo(database *db.Node, cert *shared.CertInfo) (*db.RaftNode, error) {
 	logger.Info("Starting database node", logger.Ctx{"id": info.ID, "local": info.Address, "role": info.Role})
 
 	// Data directory
-	dir := filepath.Join(database.Dir(), "global")
+	dir := database.DqliteDir()
 	if !shared.PathExists(dir) {
 		err := os.Mkdir(dir, 0750)
 		if err != nil {
