@@ -9,13 +9,11 @@ import (
 	"math"
 	"os"
 	"path/filepath"
-	"slices"
-	"strings"
 	"sync"
 	"time"
 
-	"github.com/canonical/go-dqlite/v2/app"
-	"github.com/canonical/go-dqlite/v2/client"
+	"github.com/canonical/go-dqlite/v3/app"
+	"github.com/canonical/go-dqlite/v3/client"
 
 	"github.com/canonical/lxd/lxd/certificate"
 	"github.com/canonical/lxd/lxd/db"
@@ -33,7 +31,7 @@ import (
 // errClusterBusy is returned by dqlite if attempting attempting to join a cluster at the same time as a role-change.
 // This error tells us we can retry and probably join the cluster or fail due to something else.
 // The error code here is SQLITE_BUSY.
-var errClusterBusy = errors.New("a configuration change is already in progress (5)")
+var errClusterBusy = fmt.Errorf("a configuration change is already in progress (5)")
 
 // Bootstrap turns a non-clustered LXD instance into the first (and leader)
 // node of a new LXD cluster.
@@ -43,7 +41,7 @@ var errClusterBusy = errors.New("a configuration change is already in progress (
 func Bootstrap(state *state.State, gateway *Gateway, serverName string) error {
 	// Check parameters
 	if serverName == "" {
-		return errors.New("Server name must not be empty")
+		return fmt.Errorf("Server name must not be empty")
 	}
 
 	err := membershipCheckNoLeftoverClusterCert(state.OS.VarDir)
@@ -109,65 +107,63 @@ func Bootstrap(state *state.State, gateway *Gateway, serverName string) error {
 	// to be used when validating endpoint connections. This will allow Dqlite to connect to ourselves.
 	state.UpdateIdentityCache()
 
-	err = state.DB.Cluster.RunExclusive(func(t db.Transactor) error {
-		// Shutdown the gateway. This will trash any dqlite connection against
-		// our in-memory dqlite driver and shutdown the associated raft
-		// instance. We also lock regular access to the cluster database since
-		// we don't want any other database code to run while we're
-		// reconfiguring raft.
-		err = gateway.Shutdown()
+	// Shutdown the gateway. This will trash any dqlite connection against
+	// our in-memory dqlite driver and shutdown the associated raft
+	// instance. We also lock regular access to the cluster database since
+	// we don't want any other database code to run while we're
+	// reconfiguring raft.
+	err = state.DB.Cluster.EnterExclusive()
+	if err != nil {
+		return fmt.Errorf("Failed to acquire cluster database lock: %w", err)
+	}
+
+	err = gateway.Shutdown()
+	if err != nil {
+		return fmt.Errorf("Failed to shutdown gRPC SQL gateway: %w", err)
+	}
+
+	// The cluster CA certificate is a symlink against the regular server CA certificate.
+	if shared.PathExists(filepath.Join(state.OS.VarDir, "server.ca")) {
+		err := os.Symlink("server.ca", filepath.Join(state.OS.VarDir, "cluster.ca"))
 		if err != nil {
-			return fmt.Errorf("Failed to shutdown gRPC SQL gateway: %w", err)
+			return fmt.Errorf("Failed to symlink server CA cert to cluster CA cert: %w", err)
 		}
+	}
 
-		// The cluster CA certificate is a symlink against the regular server CA certificate.
-		if shared.PathExists(filepath.Join(state.OS.VarDir, "server.ca")) {
-			err := os.Symlink("server.ca", filepath.Join(state.OS.VarDir, "cluster.ca"))
-			if err != nil {
-				return fmt.Errorf("Failed to symlink server CA cert to cluster CA cert: %w", err)
-			}
-		}
+	// Generate a new cluster certificate.
+	clusterCert, err := util.LoadClusterCert(state.OS.VarDir)
+	if err != nil {
+		return fmt.Errorf("Failed to create cluster cert: %w", err)
+	}
 
-		// Generate a new cluster certificate.
-		clusterCert, err := util.LoadClusterCert(state.OS.VarDir)
+	// If endpoint listeners are active, apply new cluster certificate.
+	if state.Endpoints != nil {
+		gateway.networkCert = clusterCert
+		state.Endpoints.NetworkUpdateCert(clusterCert)
+	}
+
+	// Re-initialize the gateway. This will create a new raft factory an
+	// dqlite driver instance, which will be exposed over gRPC by the
+	// gateway handlers.
+	err = gateway.init(true)
+	if err != nil {
+		return fmt.Errorf("Failed to re-initialize gRPC SQL gateway: %w", err)
+	}
+
+	err = gateway.WaitLeadership()
+	if err != nil {
+		return err
+	}
+
+	// Make sure we can actually connect to the cluster database through
+	// the network endpoint. This also releases the previously acquired
+	// lock and makes the Go SQL pooling system invalidate the old
+	// connection, so new queries will be executed over the new network
+	// connection.
+	err = state.DB.Cluster.ExitExclusive(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		_, err := tx.GetNodes(ctx)
 		if err != nil {
-			return fmt.Errorf("Failed to create cluster cert: %w", err)
-		}
-
-		// If endpoint listeners are active, apply new cluster certificate.
-		if state.Endpoints != nil {
-			gateway.networkCert = clusterCert
-			state.Endpoints.NetworkUpdateCert(clusterCert)
-		}
-
-		// Re-initialize the gateway. This will create a new raft factory an
-		// dqlite driver instance, which will be exposed over gRPC by the
-		// gateway handlers.
-		err = gateway.init(true)
-		if err != nil {
-			return fmt.Errorf("Failed to re-initialize gRPC SQL gateway: %w", err)
-		}
-
-		err = gateway.WaitLeadership()
-		if err != nil {
-			return err
-		}
-
-		// Make sure we can actually connect to the cluster database through
-		// the network endpoint. This also releases the previously acquired
-		// lock and makes the Go SQL pooling system invalidate the old
-		// connection, so new queries will be executed over the new network
-		// connection.
-		err = t(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			_, err = tx.GetNodes(ctx)
-			if err != nil {
-				return fmt.Errorf("Failed getting cluster members: %w", err)
-			}
-
-			return nil
-		})
-		if err != nil {
-			return err
+			return fmt.Errorf("Failed getting cluster members: %w", err)
 		}
 
 		return nil
@@ -237,11 +233,11 @@ func EnsureServerCertificateTrusted(serverName string, serverCert *shared.CertIn
 func Accept(state *state.State, gateway *Gateway, name, address string, schema, api, arch int) ([]db.RaftNode, error) {
 	// Check parameters
 	if name == "" {
-		return nil, errors.New("Member name must not be empty")
+		return nil, fmt.Errorf("Member name must not be empty")
 	}
 
 	if address == "" {
-		return nil, errors.New("Member address must not be empty")
+		return nil, fmt.Errorf("Member address must not be empty")
 	}
 
 	// Insert the new node into the nodes table.
@@ -303,12 +299,12 @@ func Accept(state *state.State, gateway *Gateway, name, address string, schema,
 
 	maxVoters := state.GlobalConfig.MaxVoters()
 	if maxVoters > math.MaxInt {
-		return nil, errors.New("Cannot convert maximum voter cluster members to int: Upper bound exceeded")
+		return nil, fmt.Errorf("Cannot convert maximum voter cluster members to int: Upper bound exceeded")
 	}
 
 	maxStandBy := state.GlobalConfig.MaxStandBy()
 	if maxStandBy > math.MaxInt {
-		return nil, errors.New("Cannot convert maximum standby cluster members to int: Upper bound exceeded")
+		return nil, fmt.Errorf("Cannot convert maximum standby cluster members to int: Upper bound exceeded")
 	}
 
 	if count > 1 && voters < int(maxVoters) {
@@ -332,7 +328,7 @@ func Accept(state *state.State, gateway *Gateway, name, address string, schema,
 func Join(state *state.State, gateway *Gateway, networkCert *shared.CertInfo, serverCert *shared.CertInfo, name string, raftNodes []db.RaftNode) error {
 	// Check parameters
 	if name == "" {
-		return errors.New("Member name must not be empty")
+		return fmt.Errorf("Member name must not be empty")
 	}
 
 	var localClusterAddress string
@@ -396,236 +392,242 @@ func Join(state *state.State, gateway *Gateway, networkCert *shared.CertInfo, se
 		return err
 	}
 
+	reverter := revert.New()
+	defer reverter.Fail()
+
 	// Lock regular access to the cluster database since we don't want any
 	// other database code to run while we're reconfiguring raft.
-	err = state.DB.Cluster.RunExclusive(func(t db.Transactor) error {
-		reverter := revert.New()
-		defer reverter.Fail()
+	err = state.DB.Cluster.EnterExclusive()
+	if err != nil {
+		return fmt.Errorf("Failed to acquire cluster database lock: %w", err)
+	}
+
+	reverter.Add(func() {
+		err := state.DB.Cluster.ExitExclusive(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+			return nil
+		})
+		if err != nil {
+			logger.Error("Failed to unlock global database after cluster join error", logger.Ctx{"err": err})
+		}
+	})
+
+	// Shutdown the gateway and wipe any raft data. This will trash any
+	// gRPC SQL connection against our in-memory dqlite driver and shutdown
+	// the associated raft instance.
+	err = gateway.Shutdown()
+	if err != nil {
+		return fmt.Errorf("Failed to shutdown gRPC SQL gateway: %w", err)
+	}
+
+	err = os.RemoveAll(state.OS.GlobalDatabaseDir())
+	if err != nil {
+		return fmt.Errorf("Failed to remove existing raft data: %w", err)
+	}
+
+	// Re-initialize the gateway. This will create a new raft factory an
+	// dqlite driver instance, which will be exposed over gRPC by the
+	// gateway handlers.
+	oldCert := gateway.networkCert
+	gateway.networkCert = networkCert
+	err = gateway.init(false)
+	if err != nil {
+		return fmt.Errorf("Failed to re-initialize gRPC SQL gateway: %w", err)
+	}
+
+	reverter.Add(func() {
+		err = state.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
+			return tx.ReplaceRaftNodes([]db.RaftNode{})
+		})
+		if err != nil {
+			logger.Error("Failed to clear local raft node records after cluster join error", logger.Ctx{"err": err})
+			return
+		}
 
-		// Shutdown the gateway and wipe any raft data. This will trash any
-		// gRPC SQL connection against our in-memory dqlite driver and shutdown
-		// the associated raft instance.
 		err = gateway.Shutdown()
 		if err != nil {
-			return fmt.Errorf("Failed to shutdown gRPC SQL gateway: %w", err)
+			logger.Error("Failed to shutdown gateway after cluster join error", logger.Ctx{"err": err})
+			return
 		}
 
 		err = os.RemoveAll(state.OS.GlobalDatabaseDir())
 		if err != nil {
-			return fmt.Errorf("Failed to remove existing raft data: %w", err)
+			logger.Error("Failed to remove raft data after cluster join error", logger.Ctx{"err": err})
+			return
 		}
 
-		// Re-initialize the gateway. This will create a new raft factory an
-		// dqlite driver instance, which will be exposed over gRPC by the
-		// gateway handlers.
-		oldCert := gateway.networkCert
-		gateway.networkCert = networkCert
+		gateway.networkCert = oldCert
 		err = gateway.init(false)
 		if err != nil {
-			return fmt.Errorf("Failed to re-initialize gRPC SQL gateway: %w", err)
+			logger.Error("Failed to re-initialize gateway after cluster join error", logger.Ctx{"err": err})
+			return
 		}
 
-		reverter.Add(func() {
-			err = state.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
-				return tx.ReplaceRaftNodes([]db.RaftNode{})
-			})
-			if err != nil {
-				logger.Error("Failed to clear local raft node records after cluster join error", logger.Ctx{"err": err})
-				return
-			}
-
-			err = gateway.Shutdown()
-			if err != nil {
-				logger.Error("Failed to shutdown gateway after cluster join error", logger.Ctx{"err": err})
-				return
-			}
-
-			err = os.RemoveAll(state.OS.GlobalDatabaseDir())
-			if err != nil {
-				logger.Error("Failed to remove raft data after cluster join error", logger.Ctx{"err": err})
-				return
-			}
-
-			gateway.networkCert = oldCert
-			err = gateway.init(false)
-			if err != nil {
-				logger.Error("Failed to re-initialize gateway after cluster join error", logger.Ctx{"err": err})
-				return
-			}
-
-			err = cluster.EnsureSchema(state.DB.Cluster.DB(), localClusterAddress, state.OS.GlobalDatabaseDir())
-			if err != nil {
-				logger.Error("Failed to reload schema after cluster join error", logger.Ctx{"err": err})
-				return
-			}
-		})
-
-		// If we are listed among the database nodes, join the raft cluster.
-		var info db.RaftNode
-		for _, node := range raftNodes {
-			if node.Address == localClusterAddress {
-				info = node
-			}
-		}
-
-		if (db.RaftNode{}) == info {
-			return errors.New("Joining member not found")
-		}
-
-		logger.Info("Joining dqlite raft cluster", logger.Ctx{"id": info.ID, "local": info.Address, "role": info.Role})
-		ctx, cancel := context.WithTimeout(context.Background(), time.Minute)
-		defer cancel()
-		client, err := client.FindLeader(
-			ctx, gateway.NodeStore(),
-			client.WithDialFunc(gateway.raftDial()),
-			client.WithLogFunc(DqliteLog),
-		)
+		_, err = cluster.EnsureSchema(state.DB.Cluster.DB(), localClusterAddress, state.OS.GlobalDatabaseDir())
 		if err != nil {
-			return fmt.Errorf("Failed to connect to cluster leader: %w", err)
+			logger.Error("Failed to reload schema after cluster join error", logger.Ctx{"err": err})
+			return
+		}
+	})
+
+	// If we are listed among the database nodes, join the raft cluster.
+	var info db.RaftNode
+	for _, node := range raftNodes {
+		if node.Address == localClusterAddress {
+			info = node
+		}
+	}
+
+	if (db.RaftNode{}) == info {
+		return fmt.Errorf("Joining member not found")
+	}
+
+	logger.Info("Joining dqlite raft cluster", logger.Ctx{"id": info.ID, "local": info.Address, "role": info.Role})
+	ctx, cancel := context.WithTimeout(context.Background(), time.Minute)
+	defer cancel()
+	client, err := client.FindLeader(
+		ctx, gateway.NodeStore(),
+		client.WithDialFunc(gateway.raftDial()),
+		client.WithLogFunc(DqliteLog),
+	)
+	if err != nil {
+		return fmt.Errorf("Failed to connect to cluster leader: %w", err)
+	}
+
+	defer func() { _ = client.Close() }()
+
+	logger.Info("Adding node to cluster", logger.Ctx{"id": info.ID, "local": info.Address, "role": info.Role})
+	ctx, cancel = context.WithTimeout(context.Background(), time.Minute)
+	defer cancel()
+
+	// Repeatedly try to join in case the cluster is busy with a role-change.
+	joined := false
+	for !joined {
+		select {
+		case <-ctx.Done():
+			return fmt.Errorf("Failed to join cluster: %w", ctx.Err())
+		default:
+			err = client.Add(ctx, info.NodeInfo)
+			if err != nil && err.Error() == errClusterBusy.Error() {
+				// If the cluster is busy with a role change, sleep a second and then keep trying to join.
+				time.Sleep(1 * time.Second)
+				continue
+			}
+
+			if err != nil {
+				return fmt.Errorf("Failed to join cluster: %w", err)
+			}
+
+			joined = true
+		}
+	}
+
+	// Make sure we can actually connect to the cluster database through
+	// the network endpoint. This also releases the previously acquired
+	// lock and makes the Go SQL pooling system invalidate the old
+	// connection, so new queries will be executed over the new gRPC
+	// network connection. Also, update the storage_pools and networks
+	// tables with our local configuration.
+	logger.Info("Migrate local data to cluster database")
+	err = state.DB.Cluster.ExitExclusive(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		node, err := tx.GetPendingNodeByAddress(ctx, localClusterAddress)
+		if err != nil {
+			return fmt.Errorf("Failed to get ID of joining node: %w", err)
 		}
 
-		defer func() { _ = client.Close() }()
+		state.DB.Cluster.NodeID(node.ID)
+		tx.NodeID(node.ID)
 
-		logger.Info("Adding node to cluster", logger.Ctx{"id": info.ID, "local": info.Address, "role": info.Role})
-		ctx, cancel = context.WithTimeout(context.Background(), time.Minute)
-		defer cancel()
+		// Storage pools.
+		ids, err := tx.GetNonPendingStoragePoolsNamesToIDs(ctx)
+		if err != nil {
+			return fmt.Errorf("Failed to get cluster storage pool IDs: %w", err)
+		}
 
-		// Repeatedly try to join in case the cluster is busy with a role-change.
-		joined := false
-		for !joined {
-			select {
-			case <-ctx.Done():
-				return fmt.Errorf("Failed to join cluster: %w", ctx.Err())
-			default:
-				err = client.Add(ctx, info.NodeInfo)
-				if err != nil && err.Error() == errClusterBusy.Error() {
-					// If the cluster is busy with a role change, sleep a second and then keep trying to join.
-					time.Sleep(1 * time.Second)
-					continue
-				}
+		for name, id := range ids {
+			err := tx.UpdateStoragePoolAfterNodeJoin(id, node.ID)
+			if err != nil {
+				return fmt.Errorf("Failed to add joining node's to the pool: %w", err)
+			}
 
+			driver, err := tx.GetStoragePoolDriver(ctx, id)
+			if err != nil {
+				return fmt.Errorf("Failed to get storage pool driver: %w", err)
+			}
+
+			// For all pools we add the config provided by the joining node.
+			config, ok := pools[name]
+			if !ok {
+				return fmt.Errorf("Joining member has no config for pool %s", name)
+			}
+
+			err = tx.CreateStoragePoolConfig(id, node.ID, config)
+			if err != nil {
+				return fmt.Errorf("Failed to add joining node's pool config: %w", err)
+			}
+
+			if shared.ValueInSlice(driver, []string{"ceph", "cephfs"}) {
+				// For ceph pools we have to create volume
+				// entries for the joining node.
+				err := tx.UpdateCephStoragePoolAfterNodeJoin(ctx, id, node.ID)
 				if err != nil {
-					return fmt.Errorf("Failed to join cluster: %w", err)
+					return fmt.Errorf("Failed to create ceph volumes for joining node: %w", err)
 				}
-
-				joined = true
 			}
 		}
 
-		// Make sure we can actually connect to the cluster database through
-		// the network endpoint. This also releases the previously acquired
-		// lock and makes the Go SQL pooling system invalidate the old
-		// connection, so new queries will be executed over the new gRPC
-		// network connection. Also, update the storage_pools and networks
-		// tables with our local configuration.
-		logger.Info("Migrate local data to cluster database")
-		err = t(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-			node, err := tx.GetPendingNodeByAddress(ctx, localClusterAddress)
-			if err != nil {
-				return fmt.Errorf("Failed to get ID of joining node: %w", err)
-			}
+		// Networks.
+		netids, err := tx.GetNonPendingNetworkIDs(ctx)
+		if err != nil {
+			return fmt.Errorf("Failed to get cluster network IDs: %w", err)
+		}
 
-			state.DB.Cluster.NodeID(node.ID)
-			tx.NodeID(node.ID)
-
-			// Storage pools.
-			ids, err := tx.GetNonPendingStoragePoolsNamesToIDs(ctx)
-			if err != nil {
-				return fmt.Errorf("Failed to get cluster storage pool IDs: %w", err)
-			}
-
-			for name, id := range ids {
-				err := tx.UpdateStoragePoolAfterNodeJoin(id, node.ID)
-				if err != nil {
-					return fmt.Errorf("Failed to add joining node's to the pool: %w", err)
-				}
-
-				driver, err := tx.GetStoragePoolDriver(ctx, id)
-				if err != nil {
-					return fmt.Errorf("Failed to get storage pool driver: %w", err)
-				}
-
-				// For all pools we add the config provided by the joining node.
-				config, ok := pools[name]
+		for _, network := range netids {
+			for name, id := range network {
+				config, ok := networks[name]
 				if !ok {
-					return fmt.Errorf("Joining member has no config for pool %s", name)
+					return fmt.Errorf("Joining member has no config for network %s", name)
 				}
 
-				err = tx.CreateStoragePoolConfig(id, node.ID, config)
+				err := tx.NetworkNodeJoin(id, node.ID)
 				if err != nil {
-					return fmt.Errorf("Failed to add joining node's pool config: %w", err)
+					return fmt.Errorf("Failed to add joining node's to the network: %w", err)
 				}
 
-				if slices.Contains([]string{"ceph", "cephfs"}, driver) {
-					// For ceph pools we have to create volume
-					// entries for the joining node.
-					err := tx.UpdateCephStoragePoolAfterNodeJoin(ctx, id, node.ID)
-					if err != nil {
-						return fmt.Errorf("Failed to create ceph volumes for joining node: %w", err)
-					}
-				}
-			}
-
-			// Networks.
-			netids, err := tx.GetNonPendingNetworkIDs(ctx)
-			if err != nil {
-				return fmt.Errorf("Failed to get cluster network IDs: %w", err)
-			}
-
-			for _, network := range netids {
-				for name, id := range network {
-					config, ok := networks[name]
-					if !ok {
-						return fmt.Errorf("Joining member has no config for network %s", name)
-					}
-
-					err := tx.NetworkNodeJoin(id, node.ID)
-					if err != nil {
-						return fmt.Errorf("Failed to add joining node's to the network: %w", err)
-					}
-
-					err = tx.CreateNetworkConfig(id, node.ID, config)
-					if err != nil {
-						return fmt.Errorf("Failed to add joining node's network config: %w", err)
-					}
-				}
-			}
-
-			// Migrate outstanding operations.
-			for _, operation := range operations {
-				op := cluster.Operation{
-					UUID:   operation.UUID,
-					Type:   operation.Type,
-					NodeID: tx.GetNodeID(),
-				}
-
-				_, err := cluster.CreateOrReplaceOperation(ctx, tx.Tx(), op)
+				err = tx.CreateNetworkConfig(id, node.ID, config)
 				if err != nil {
-					return fmt.Errorf("Failed to migrate operation %s: %w", operation.UUID, err)
+					return fmt.Errorf("Failed to add joining node's network config: %w", err)
 				}
 			}
-
-			// Remove the pending flag for ourselves
-			// notifications.
-			err = tx.SetNodePendingFlag(node.ID, false)
-			if err != nil {
-				return fmt.Errorf("Failed to unmark the node as pending: %w", err)
-			}
-
-			// Set last heartbeat time to now, as member is clearly online as it just successfully joined,
-			// that way when we send the notification to all members below it will consider this member online.
-			err = tx.SetNodeHeartbeat(node.Address, time.Now().UTC())
-			if err != nil {
-				return fmt.Errorf("Failed setting last heartbeat time for member: %w", err)
-			}
-
-			return nil
-		})
-		if err != nil {
-			return err
 		}
 
-		reverter.Success()
+		// Migrate outstanding operations.
+		for _, operation := range operations {
+			op := cluster.Operation{
+				UUID:   operation.UUID,
+				Type:   operation.Type,
+				NodeID: tx.GetNodeID(),
+			}
+
+			_, err := cluster.CreateOrReplaceOperation(ctx, tx.Tx(), op)
+			if err != nil {
+				return fmt.Errorf("Failed to migrate operation %s: %w", operation.UUID, err)
+			}
+		}
+
+		// Remove the pending flag for ourselves
+		// notifications.
+		err = tx.SetNodePendingFlag(node.ID, false)
+		if err != nil {
+			return fmt.Errorf("Failed to unmark the node as pending: %w", err)
+		}
+
+		// Set last heartbeat time to now, as member is clearly online as it just successfully joined,
+		// that way when we send the notification to all members below it will consider this member online.
+		err = tx.SetNodeHeartbeat(node.Address, time.Now().UTC())
+		if err != nil {
+			return fmt.Errorf("Failed setting last heartbeat time for member: %w", err)
+		}
+
 		return nil
 	})
 	if err != nil {
@@ -637,6 +639,8 @@ func Join(state *state.State, gateway *Gateway, networkCert *shared.CertInfo, se
 		NotifyHeartbeat(state, gateway)
 	}
 
+	reverter.Success()
+
 	return nil
 }
 
@@ -788,7 +792,7 @@ func Assign(state *state.State, gateway *Gateway, nodes []db.RaftNode) error {
 
 	// Ensure we actually have an address.
 	if address == "" {
-		return errors.New("Cluster member is not exposed on the network")
+		return fmt.Errorf("Cluster member is not exposed on the network")
 	}
 
 	// Figure out our node identity.
@@ -801,10 +805,11 @@ func Assign(state *state.State, gateway *Gateway, nodes []db.RaftNode) error {
 
 	// Ensure that our address was actually included in the given list of raft nodes.
 	if info == nil {
-		return errors.New("This member is not included in the given list of database nodes")
+		return fmt.Errorf("This member is not included in the given list of database nodes")
 	}
 
-	// Replace our local list of raft nodes with the given one (which includes ourselves).
+	// Replace our local list of raft nodes with the given one (which
+	// includes ourselves).
 	err = state.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
 		err = tx.ReplaceRaftNodes(nodes)
 		if err != nil {
@@ -817,133 +822,138 @@ func Assign(state *state.State, gateway *Gateway, nodes []db.RaftNode) error {
 		return err
 	}
 
-	assign := func() error {
-		logger.Info("Changing local dqlite raft role", logger.Ctx{"id": info.ID, "local": info.Address, "role": info.Role})
+	var transactor func(context.Context, func(ctx context.Context, tx *db.ClusterTx) error) error
 
-		ctx, cancel := context.WithTimeout(context.Background(), time.Minute)
-		defer cancel()
+	// If we are already running a dqlite node, it means we have cleanly
+	// joined the cluster before, using the roles support API. In that case
+	// there's no need to restart the gateway and we can just change our
+	// dqlite role.
+	if gateway.IsDqliteNode() {
+		transactor = state.DB.Cluster.Transaction
+		goto assign
+	}
 
-		client, err := client.FindLeader(ctx, gateway.NodeStore(), client.WithDialFunc(gateway.raftDial()))
+	// If we get here it means that we are an upgraded node from cluster
+	// without roles support, or we didn't cleanly join the cluster. Either
+	// way, we don't have a dqlite node running, so we need to restart the
+	// gateway.
+
+	// Lock regular access to the cluster database since we don't want any
+	// other database code to run while we're reconfiguring raft.
+	err = state.DB.Cluster.EnterExclusive()
+	if err != nil {
+		return fmt.Errorf("Failed to acquire cluster database lock: %w", err)
+	}
+
+	transactor = state.DB.Cluster.ExitExclusive
+
+	// Wipe all existing raft data, for good measure (perhaps they were
+	// somehow leftover).
+	err = os.RemoveAll(state.OS.GlobalDatabaseDir())
+	if err != nil {
+		return fmt.Errorf("Failed to remove existing raft data: %w", err)
+	}
+
+	// Re-initialize the gateway. This will create a new raft factory an
+	// dqlite driver instance, which will be exposed over gRPC by the
+	// gateway handlers.
+	err = gateway.init(false)
+	if err != nil {
+		return fmt.Errorf("Failed to re-initialize gRPC SQL gateway: %w", err)
+	}
+
+assign:
+	logger.Info("Changing local dqlite raft role", logger.Ctx{"id": info.ID, "local": info.Address, "role": info.Role})
+
+	ctx, cancel := context.WithTimeout(context.Background(), time.Minute)
+	defer cancel()
+
+	client, err := client.FindLeader(ctx, gateway.NodeStore(), client.WithDialFunc(gateway.raftDial()))
+	if err != nil {
+		return fmt.Errorf("Connect to cluster leader: %w", err)
+	}
+
+	defer func() { _ = client.Close() }()
+
+	// Figure out our current role.
+	role := db.RaftRole(-1)
+	cluster, err := client.Cluster(ctx)
+	if err != nil {
+		return fmt.Errorf("Fetch current cluster configuration: %w", err)
+	}
+
+	for _, server := range cluster {
+		if server.ID == info.ID {
+			role = server.Role
+			break
+		}
+	}
+	if role == -1 {
+		return fmt.Errorf("Node %s does not belong to the current raft configuration", address)
+	}
+
+	// If we're stepping back from voter to spare, let's first transition
+	// to stand-by first and wait for the configuration change to be
+	// notified to us. This prevent us from thinking we're still voters and
+	// potentially disrupt the cluster.
+	if role == db.RaftVoter && info.Role == db.RaftSpare {
+		err = client.Assign(ctx, info.ID, db.RaftStandBy)
 		if err != nil {
-			return fmt.Errorf("Connect to cluster leader: %w", err)
+			return fmt.Errorf("Failed to step back to stand-by: %w", err)
 		}
 
-		defer func() { _ = client.Close() }()
-
-		// Figure out our current role.
-		role := db.RaftRole(-1)
-		cluster, err := client.Cluster(ctx)
+		local, err := gateway.getClient()
 		if err != nil {
-			return fmt.Errorf("Fetch current cluster configuration: %w", err)
+			return fmt.Errorf("Failed to get local dqlite client: %w", err)
 		}
 
-		for _, server := range cluster {
-			if server.ID == info.ID {
-				role = server.Role
-				break
-			}
-		}
-		if role == -1 {
-			return fmt.Errorf("Node %s does not belong to the current raft configuration", address)
-		}
-
-		// If we're stepping back from voter to spare, let's first transition
-		// to stand-by first and wait for the configuration change to be
-		// notified to us. This prevent us from thinking we're still voters and
-		// potentially disrupt the cluster.
-		if role == db.RaftVoter && info.Role == db.RaftSpare {
-			err = client.Assign(ctx, info.ID, db.RaftStandBy)
+		notified := false
+		for i := 0; i < 10; i++ {
+			time.Sleep(500 * time.Millisecond)
+			servers, err := local.Cluster(context.Background())
 			if err != nil {
-				return fmt.Errorf("Failed to step back to stand-by: %w", err)
+				return fmt.Errorf("Failed to get current cluster: %w", err)
 			}
 
-			local, err := gateway.getClient()
-			if err != nil {
-				return fmt.Errorf("Failed to get local dqlite client: %w", err)
-			}
-
-			notified := false
-			for range 10 {
-				time.Sleep(500 * time.Millisecond)
-				servers, err := local.Cluster(context.Background())
-				if err != nil {
-					return fmt.Errorf("Failed to get current cluster: %w", err)
+			for _, server := range servers {
+				if server.ID != info.ID {
+					continue
 				}
 
-				for _, server := range servers {
-					if server.ID != info.ID {
-						continue
-					}
-
-					if server.Role == db.RaftStandBy {
-						notified = true
-						break
-					}
-				}
-				if notified {
+				if server.Role == db.RaftStandBy {
+					notified = true
 					break
 				}
 			}
-			if !notified {
-				return errors.New("Timeout waiting for configuration change notification")
+			if notified {
+				break
 			}
 		}
-
-		// Give the Assign operation a bit more budget in case we're promoting
-		// to voter, since that might require a snapshot transfer.
-		if info.Role == db.RaftVoter {
-			ctx, cancel = context.WithTimeout(context.Background(), 20*time.Second)
-			defer cancel()
+		if !notified {
+			return fmt.Errorf("Timeout waiting for configuration change notification")
 		}
-
-		err = client.Assign(ctx, info.ID, info.Role)
-		if err != nil {
-			return fmt.Errorf("Failed to assign role: %w", err)
-		}
-
-		gateway.info = info
-
-		return nil
 	}
 
-	if gateway.IsDqliteNode() {
-		// If we are already running a dqlite node, it means we have cleanly
-		// joined the cluster before, using the roles support API. In that case
-		// there's no need to restart the gateway and we can just change our
-		// dqlite role.
-		err = assign()
-		if err != nil {
-			return err
-		}
-	} else {
-		// If we get here it means that we are an upgraded node from cluster
-		// without roles support, or we didn't cleanly join the cluster. Either
-		// way, we don't have a dqlite node running, so we need to restart the
-		// gateway.
+	// Give the Assign operation a bit more budget in case we're promoting
+	// to voter, since that might require a snapshot transfer.
+	if info.Role == db.RaftVoter {
+		ctx, cancel = context.WithTimeout(context.Background(), 20*time.Second)
+		defer cancel()
+	}
 
-		// Lock regular access to the cluster database since we don't want any
-		// other database code to run while we're reconfiguring raft.
-		err = state.DB.Cluster.RunExclusive(func(t db.Transactor) error {
-			// Wipe all existing raft data, for good measure (perhaps they were
-			// somehow leftover).
-			err = os.RemoveAll(state.OS.GlobalDatabaseDir())
-			if err != nil {
-				return fmt.Errorf("Failed to remove existing raft data: %w", err)
-			}
+	err = client.Assign(ctx, info.ID, info.Role)
+	if err != nil {
+		return fmt.Errorf("Failed to assign role: %w", err)
+	}
 
-			// Re-initialize the gateway. This will create a new raft factory an
-			// dqlite driver instance, which will be exposed over gRPC by the
-			// gateway handlers.
-			err = gateway.init(false)
-			if err != nil {
-				return fmt.Errorf("Failed to re-initialize gRPC SQL gateway: %w", err)
-			}
+	gateway.info = info
 
-			return assign()
-		})
-		if err != nil {
-			return err
-		}
+	// Unlock regular access to our cluster database.
+	err = transactor(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		return nil
+	})
+	if err != nil {
+		return fmt.Errorf("Cluster database initialization failed: %w", err)
 	}
 
 	// Generate partial heartbeat request containing just a raft node list.
@@ -1051,12 +1061,7 @@ func Handover(state *state.State, gateway *Gateway, address string) (string, []d
 	}
 
 	if nodeID == 0 {
-		raftNodeAddresses := make([]string, 0, len(nodes))
-		for _, node := range nodes {
-			raftNodeAddresses = append(raftNodeAddresses, node.Address)
-		}
-
-		return "", nil, fmt.Errorf("No dqlite node has address %s (%s)", address, strings.Join(raftNodeAddresses, ","))
+		return "", nil, fmt.Errorf("No dqlite node has address %s: %w", address, err)
 	}
 
 	roles, err := newRolesChanges(state, gateway, nodes, nil)
@@ -1099,7 +1104,7 @@ func newRolesChanges(state *state.State, gateway *Gateway, nodes []db.RaftNode,
 	cluster := map[client.NodeInfo]*client.NodeMetadata{}
 
 	for _, node := range nodes {
-		if !slices.Contains(unavailableMembers, node.Address) && HasConnectivity(gateway.networkCert, gateway.state().ServerCert(), node.Address) {
+		if !shared.ValueInSlice(node.Address, unavailableMembers) && HasConnectivity(gateway.networkCert, gateway.state().ServerCert(), node.Address) {
 			cluster[node.NodeInfo] = &client.NodeMetadata{
 				FailureDomain: domains[node.Address],
 			}
@@ -1110,12 +1115,12 @@ func newRolesChanges(state *state.State, gateway *Gateway, nodes []db.RaftNode,
 
 	maxVoters := state.GlobalConfig.MaxVoters()
 	if maxVoters > math.MaxInt {
-		return nil, errors.New("Cannot convert maximum voter nodes to int: Upper bound exceeded")
+		return nil, fmt.Errorf("Cannot convert maximum voter nodes to int: Upper bound exceeded")
 	}
 
 	maxStandBy := state.GlobalConfig.MaxStandBy()
 	if maxStandBy > math.MaxInt {
-		return nil, errors.New("Cannot convert maximum standby nodes to int: Upper bound exceeded")
+		return nil, fmt.Errorf("Cannot convert maximum standby nodes to int: Upper bound exceeded")
 	}
 
 	roles := &app.RolesChanges{
@@ -1203,15 +1208,15 @@ func membershipCheckNodeStateForBootstrapOrJoin(ctx context.Context, tx *db.Node
 	// Ensure that we're not in an inconsistent situation, where no cluster address is set, but still there
 	// are entries in the raft_nodes table.
 	if !hasClusterAddress && hasRaftNodes {
-		return errors.New("Inconsistent state: found leftover entries in raft_nodes")
+		return fmt.Errorf("Inconsistent state: found leftover entries in raft_nodes")
 	}
 
 	if !hasClusterAddress {
-		return errors.New("No cluster.https_address config is set on this member")
+		return fmt.Errorf("No cluster.https_address config is set on this member")
 	}
 
 	if hasRaftNodes {
-		return errors.New("The member is already part of a cluster")
+		return fmt.Errorf("The member is already part of a cluster")
 	}
 
 	return nil
@@ -1226,7 +1231,7 @@ func membershipCheckClusterStateForBootstrapOrJoin(ctx context.Context, tx *db.C
 	}
 
 	if len(members) != 1 {
-		return errors.New("Inconsistent state: Found leftover entries in cluster members")
+		return fmt.Errorf("Inconsistent state: Found leftover entries in cluster members")
 	}
 
 	return nil
@@ -1240,7 +1245,7 @@ func membershipCheckClusterStateForAccept(ctx context.Context, tx *db.ClusterTx,
 	}
 
 	if len(members) == 1 && members[0].Address == "0.0.0.0" {
-		return errors.New("Clustering isn't enabled")
+		return fmt.Errorf("Clustering isn't enabled")
 	}
 
 	for _, member := range members {
@@ -1283,7 +1288,7 @@ func membershipCheckClusterStateForLeave(ctx context.Context, tx *db.ClusterTx,
 	}
 
 	if len(members) == 1 {
-		return errors.New("Member is the only member in the cluster")
+		return fmt.Errorf("Member is the only member in the cluster")
 	}
 
 	return nil
@@ -1295,7 +1300,7 @@ func membershipCheckNoLeftoverClusterCert(dir string) error {
 	// Ensure that there's no leftover cluster certificate.
 	for _, basename := range []string{"cluster.crt", "cluster.key", "cluster.ca"} {
 		if shared.PathExists(filepath.Join(dir, basename)) {
-			return errors.New("Inconsistent state: found leftover cluster certificate")
+			return fmt.Errorf("Inconsistent state: found leftover cluster certificate")
 		}
 	}
 
