@@ -2,11 +2,9 @@ package cluster_test
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"net/http"
 	"net/http/httptest"
-	"slices"
 	"strconv"
 	"testing"
 	"time"
@@ -71,7 +69,7 @@ func TestNewNotifier(t *testing.T) {
 	}
 	require.NoError(t, err)
 	for i := range addresses {
-		assert.True(t, slices.Contains(addresses, f.Address(i+1)))
+		assert.True(t, shared.ValueInSlice(f.Address(i+1), addresses))
 	}
 }
 
@@ -156,7 +154,7 @@ func TestNewNotify_NotifyAliveShuttingDown(t *testing.T) {
 	cleanupF := f.Nodes(cert, 3)
 	defer cleanupF()
 
-	f.Unavailable(1, errors.New("LXD is shutting down"))
+	f.Unavailable(1, fmt.Errorf("LXD is shutting down"))
 
 	// Populate state.LocalConfig after nodes created above.
 	var err error
@@ -203,13 +201,13 @@ func (h *notifyFixtures) Nodes(cert *shared.CertInfo, n int) func() {
 	}
 
 	servers := make([]*httptest.Server, n)
-	for i := range n {
+	for i := 0; i < n; i++ {
 		servers[i] = newRestServer(strconv.Itoa(i), cert)
 	}
 
 	// Insert new entries in the nodes table of the cluster database.
 	err := h.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		for i := range n {
+		for i := 0; i < n; i++ {
 			name := strconv.Itoa(i)
 			address := servers[i].Listener.Addr().String()
 			var err error
