@@ -2,14 +2,13 @@ package cluster
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"math/rand"
 	"net/http"
 	"os"
 	"time"
 
-	"github.com/canonical/go-dqlite/v2/client"
+	"github.com/canonical/go-dqlite/v3/client"
 
 	"github.com/canonical/lxd/client"
 	"github.com/canonical/lxd/lxd/db"
@@ -35,7 +34,7 @@ func NotifyUpgradeCompleted(state *state.State, networkCert *shared.CertInfo, se
 		}
 
 		url := info.Addresses[0] + databaseEndpoint
-		request, err := http.NewRequest(http.MethodPatch, url, nil)
+		request, err := http.NewRequest("PATCH", url, nil)
 		if err != nil {
 			return fmt.Errorf("failed to create database notify upgrade request: %w", err)
 		}
@@ -75,7 +74,7 @@ func MaybeUpdate(state *state.State) error {
 	}
 
 	if state.DB.Cluster == nil {
-		return errors.New("Failed checking cluster update, state not initialised yet")
+		return fmt.Errorf("Failed checking cluster update, state not initialised yet")
 	}
 
 	err = state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
@@ -90,11 +89,11 @@ func MaybeUpdate(state *state.State) error {
 
 	if err != nil {
 		// Just log the error and return.
-		return fmt.Errorf("Failed to check if this member is out-of-date: %w", err)
+		return fmt.Errorf("Failed to check if this node is out-of-date: %w", err)
 	}
 
 	if !shouldUpdate {
-		logger.Debug("Cluster member is up-to-date")
+		logger.Debugf("Cluster node is up-to-date")
 		return nil
 	}
 
