@@ -1,7 +1,7 @@
 package config_test
 
 import (
-	"errors"
+	"fmt"
 	"strings"
 	"testing"
 
@@ -102,13 +102,11 @@ func TestChange(t *testing.T) {
 		"bar": {Setter: upperCase},
 		"egg": {Type: config.Bool},
 		"yuk": {Type: config.Bool, Default: "true"},
-		"xyz": {Hidden: true},
 	}
 
 	values := map[string]string{ // Initial values
 		"foo": "hello",
 		"bar": "x",
-		"xyz": "sekret",
 	}
 
 	cases := []struct {
@@ -137,20 +135,15 @@ func TestChange(t *testing.T) {
 			map[string]string{"yuk": "false"},
 		},
 		{
-			`the special value 'true' is a passthrough for hidden keys`,
-			map[string]any{"xyz": true},
-			map[string]string{"xyz": "sekret"},
+			`multiple values are all mutated`,
+			map[string]any{"foo": "x", "bar": "hey", "egg": "0"},
+			map[string]string{"foo": "x", "bar": "HEY", "egg": ""},
 		},
 		{
 			`the special value nil is converted to empty string`,
 			map[string]any{"foo": nil},
 			map[string]string{"foo": ""},
 		},
-		{
-			`multiple values are all mutated`,
-			map[string]any{"foo": "x", "bar": "hey", "egg": "0"},
-			map[string]string{"foo": "x", "bar": "HEY", "egg": ""},
-		},
 	}
 
 	for _, c := range cases {
@@ -267,19 +260,16 @@ func TestMap_ChangeError(t *testing.T) {
 	}
 }
 
-// A Map dump contains only values that differ from their default. Hidden
-// values are obfuscated.
+// A Map dump contains only values that differ from their default.
 func TestMap_Dump(t *testing.T) {
 	schema := config.Schema{
 		"foo": {},
 		"bar": {Default: "x"},
-		"egg": {Hidden: true},
 	}
 
 	values := map[string]string{
 		"foo": "hello",
 		"bar": "x",
-		"egg": "123",
 	}
 
 	m, err := config.Load(schema, values)
@@ -287,7 +277,6 @@ func TestMap_Dump(t *testing.T) {
 
 	dump := map[string]any{
 		"foo": "hello",
-		"egg": true,
 	}
 
 	assert.Equal(t, dump, m.Dump())
@@ -311,7 +300,7 @@ func TestMap_Getters(t *testing.T) {
 	assert.NoError(t, err)
 
 	assert.Equal(t, "hello", m.GetString("foo"))
-	assert.True(t, m.GetBool("bar"))
+	assert.Equal(t, true, m.GetBool("bar"))
 	assert.Equal(t, int64(123), m.GetInt64("egg"))
 }
 
@@ -334,7 +323,7 @@ func TestMap_GettersPanic(t *testing.T) {
 
 // A Key setter that always fail.
 func failingSetter(string) (string, error) {
-	return "", errors.New("Boom")
+	return "", fmt.Errorf("Boom")
 }
 
 // A Key setter that uppercases the value.
