@@ -1,7 +1,7 @@
 package config
 
 import (
-	"errors"
+	"fmt"
 	"testing"
 
 	"github.com/stretchr/testify/assert"
@@ -33,7 +33,7 @@ var validateCases = []struct {
 // Validator that returns an error if the value is not the empty string.
 func isNotEmptyString(value string) error {
 	if value == "" {
-		return errors.New("empty value not valid")
+		return fmt.Errorf("empty value not valid")
 	}
 
 	return nil
@@ -57,7 +57,7 @@ var validateErrorCases = []struct {
 }{
 	{Key{Type: Int64}, "1.2", "Invalid integer"},
 	{Key{Type: Bool}, "yyy", "Invalid boolean"},
-	{Key{Validator: func(string) error { return errors.New("Some error") }}, "", "Some error"},
+	{Key{Validator: func(string) error { return fmt.Errorf("Some error") }}, "", "Some error"},
 	{Key{Deprecated: "Don't use this"}, "foo", `Deprecated: "Don't use this"`},
 }
 
