@@ -2,7 +2,6 @@ package main
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
@@ -20,11 +19,11 @@ import (
 	"github.com/canonical/lxd/shared/api"
 )
 
-func daemonStorageVolumesUnmount(s *state.State, ctx context.Context) error {
+func daemonStorageVolumesUnmount(s *state.State) error {
 	var storageBackups string
 	var storageImages string
 
-	err := s.DB.Node.Transaction(ctx, func(ctx context.Context, tx *db.NodeTx) error {
+	err := s.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
 		nodeConfig, err := node.ConfigLoad(ctx, tx)
 		if err != nil {
 			return err
@@ -51,7 +50,7 @@ func daemonStorageVolumesUnmount(s *state.State, ctx context.Context) error {
 			return err
 		}
 
-		// Unmount volume.
+		// Mount volume.
 		_, err = pool.UnmountCustomVolume(api.ProjectDefaultName, volumeName, nil)
 		if err != nil {
 			return fmt.Errorf("Failed to unmount storage volume %q: %w", source, err)
@@ -60,22 +59,17 @@ func daemonStorageVolumesUnmount(s *state.State, ctx context.Context) error {
 		return nil
 	}
 
-	select {
-	case <-ctx.Done():
-		return errors.New("Timed out waiting for image and backup volume")
-	default:
-		if storageBackups != "" {
-			err := unmount(storageBackups)
-			if err != nil {
-				return fmt.Errorf("Failed to unmount backups storage: %w", err)
-			}
+	if storageBackups != "" {
+		err := unmount(storageBackups)
+		if err != nil {
+			return fmt.Errorf("Failed to unmount backups storage: %w", err)
 		}
+	}
 
-		if storageImages != "" {
-			err := unmount(storageImages)
-			if err != nil {
-				return fmt.Errorf("Failed to unmount images storage: %w", err)
-			}
+	if storageImages != "" {
+		err := unmount(storageImages)
+		if err != nil {
+			return fmt.Errorf("Failed to unmount images storage: %w", err)
 		}
 	}
 
@@ -85,7 +79,7 @@ func daemonStorageVolumesUnmount(s *state.State, ctx context.Context) error {
 func daemonStorageMount(s *state.State) error {
 	var storageBackups string
 	var storageImages string
-	err := s.DB.Node.Transaction(context.Background(), func(ctx context.Context, tx *db.NodeTx) error {
+	err := s.DB.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
 		nodeConfig, err := node.ConfigLoad(ctx, tx)
 		if err != nil {
 			return err
@@ -140,19 +134,19 @@ func daemonStorageMount(s *state.State) error {
 
 func daemonStorageSplitVolume(volume string) (poolName string, volumeName string, err error) {
 	if strings.Count(volume, "/") != 1 {
-		return "", "", errors.New("Invalid syntax for volume, must be <pool>/<volume>")
+		return "", "", fmt.Errorf("Invalid syntax for volume, must be <pool>/<volume>")
 	}
 
 	poolName, volumeName, _ = strings.Cut(volume, "/")
 
-	err = storageDrivers.ValidPoolName(poolName)
-	if err != nil {
-		return "", "", fmt.Errorf("Invalid pool name %q: %w", poolName, err)
+	// Validate pool name.
+	if strings.Contains(poolName, "\\") || strings.Contains(poolName, "..") {
+		return "", "", fmt.Errorf("Invalid pool name: %q", poolName)
 	}
 
-	err = storageDrivers.ValidVolumeName(volumeName)
-	if err != nil {
-		return "", "", fmt.Errorf("Invalid volume name %q: %w", volumeName, err)
+	// Validate volume name.
+	if strings.Contains(volumeName, "\\") || strings.Contains(volumeName, "..") {
+		return "", "", fmt.Errorf("Invalid volume name: %q", volumeName)
 	}
 
 	return poolName, volumeName, nil
@@ -213,7 +207,7 @@ func daemonStorageValidate(s *state.State, target string) (validatedTarget strin
 	}
 
 	if len(snapshots) != 0 {
-		return "", errors.New("Storage volumes for use by LXD itself cannot have snapshots")
+		return "", fmt.Errorf("Storage volumes for use by LXD itself cannot have snapshots")
 	}
 
 	// Mount volume.
