@@ -136,29 +136,25 @@ SELECT instances_backups.name, instances_backups.instance_id,
 // GetInstanceBackups returns the names of all backups of the instance with the
 // given name.
 func (c *ClusterTx) GetInstanceBackups(ctx context.Context, projectName string, name string) ([]string, error) {
-	var instanceBackupNames []string
+	var result []string
 
 	q := `SELECT instances_backups.name FROM instances_backups
 JOIN instances ON instances_backups.instance_id=instances.id
 JOIN projects ON projects.id=instances.project_id
 WHERE projects.name=? AND instances.name=?`
+	inargs := []any{projectName, name}
+	outfmt := []any{name}
 
-	err := query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
-		var backupName string
-		err := scan(&backupName)
-		if err != nil {
-			return err
-		}
-
-		instanceBackupNames = append(instanceBackupNames, backupName)
-
-		return nil
-	}, projectName, name)
+	dbResults, err := queryScan(ctx, c, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
 
-	return instanceBackupNames, nil
+	for _, r := range dbResults {
+		result = append(result, r[0].(string))
+	}
+
+	return result, nil
 }
 
 // CreateInstanceBackup creates a new backup.
@@ -242,49 +238,46 @@ func (c *ClusterTx) RenameInstanceBackup(ctx context.Context, oldName, newName s
 
 // GetExpiredInstanceBackups returns a list of expired instance backups.
 func (c *ClusterTx) GetExpiredInstanceBackups(ctx context.Context) ([]InstanceBackup, error) {
-	var expiredInstanceBackups []InstanceBackup
+	var result []InstanceBackup
+	var name string
+	var expiryDate string
+	var instanceID int
 
 	q := `SELECT instances_backups.name, instances_backups.expiry_date, instances_backups.instance_id FROM instances_backups`
+	outfmt := []any{name, expiryDate, instanceID}
 
-	err := query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
-		var name string
-		var expiryDate string
-		var instanceID int
+	dbResults, err := queryScan(ctx, c, q, nil, outfmt)
+	if err != nil {
+		return nil, err
+	}
 
-		err := scan(&name, &expiryDate, &instanceID)
-		if err != nil {
-			return err
-		}
+	for _, r := range dbResults {
+		timestamp := r[1]
 
 		var backupExpiry time.Time
-		err = backupExpiry.UnmarshalText([]byte(expiryDate))
+		err = backupExpiry.UnmarshalText([]byte(timestamp.(string)))
 		if err != nil {
-			return err
+			return []InstanceBackup{}, err
 		}
 
 		// Since zero time causes some issues due to timezones, we check the
 		// unix timestamp instead of IsZero().
 		if backupExpiry.Unix() <= 0 {
 			// Backup doesn't expire
-			return nil
+			continue
 		}
 
 		// Backup has expired
 		if time.Now().Unix()-backupExpiry.Unix() >= 0 {
-			expiredInstanceBackups = append(expiredInstanceBackups, InstanceBackup{
-				Name:       name,
-				InstanceID: instanceID,
+			result = append(result, InstanceBackup{
+				Name:       r[0].(string),
+				InstanceID: r[2].(int),
 				ExpiryDate: backupExpiry,
 			})
 		}
-
-		return nil
-	})
-	if err != nil {
-		return nil, err
 	}
 
-	return expiredInstanceBackups, nil
+	return result, nil
 }
 
 // GetExpiredStorageVolumeBackups returns a list of expired storage volume backups.
@@ -369,30 +362,26 @@ func (c *ClusterTx) GetStoragePoolVolumeBackups(ctx context.Context, projectName
 
 // GetStoragePoolVolumeBackupsNames returns the names of all backups of the storage volume with the given name.
 func (c *ClusterTx) GetStoragePoolVolumeBackupsNames(ctx context.Context, projectName string, volumeName string, poolID int64) ([]string, error) {
-	var storagePoolVolumeBackupsNames []string
+	var result []string
 
 	q := `SELECT storage_volumes_backups.name FROM storage_volumes_backups
 JOIN storage_volumes ON storage_volumes_backups.storage_volume_id=storage_volumes.id
 JOIN projects ON projects.id=storage_volumes.project_id
 WHERE projects.name=? AND storage_volumes.name=?
 ORDER BY storage_volumes_backups.id`
+	inargs := []any{projectName, volumeName}
+	outfmt := []any{volumeName}
 
-	err := query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
-		var backupName string
-		err := scan(&backupName)
-		if err != nil {
-			return err
-		}
-
-		storagePoolVolumeBackupsNames = append(storagePoolVolumeBackupsNames, backupName)
-
-		return nil
-	}, projectName, volumeName)
+	dbResults, err := queryScan(ctx, c, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
 
-	return storagePoolVolumeBackupsNames, nil
+	for _, r := range dbResults {
+		result = append(result, r[0].(string))
+	}
+
+	return result, nil
 }
 
 // CreateStoragePoolVolumeBackup creates a new storage volume backup.
