@@ -29,7 +29,7 @@ func (e entityTypeCertificate) urlsByProjectQuery() string {
 }
 
 func (e entityTypeCertificate) urlByIDQuery() string {
-	return e.allURLsQuery() + " AND identities.id = ?"
+	return fmt.Sprintf(`%s AND identities.id = ?`, e.allURLsQuery())
 }
 
 func (e entityTypeCertificate) idFromURLQuery() string {
