@@ -20,7 +20,7 @@ func (e entityTypeClusterMember) urlsByProjectQuery() string {
 }
 
 func (e entityTypeClusterMember) urlByIDQuery() string {
-	return e.allURLsQuery() + " WHERE nodes.id = ?"
+	return fmt.Sprintf(`%s WHERE nodes.id = ?`, e.allURLsQuery())
 }
 
 func (e entityTypeClusterMember) idFromURLQuery() string {
