@@ -20,7 +20,7 @@ func (e entityTypeIdentityProviderGroup) urlsByProjectQuery() string {
 }
 
 func (e entityTypeIdentityProviderGroup) urlByIDQuery() string {
-	return e.allURLsQuery() + " WHERE identity_provider_groups.id = ?"
+	return fmt.Sprintf(`%s WHERE identity_provider_groups.id = ?`, e.allURLsQuery())
 }
 
 func (e entityTypeIdentityProviderGroup) idFromURLQuery() string {
