@@ -19,11 +19,11 @@ JOIN projects ON networks.project_id = projects.id`, e.code())
 }
 
 func (e entityTypeNetwork) urlsByProjectQuery() string {
-	return e.allURLsQuery() + " WHERE projects.name = ?"
+	return fmt.Sprintf(`%s WHERE projects.name = ?`, e.allURLsQuery())
 }
 
 func (e entityTypeNetwork) urlByIDQuery() string {
-	return e.allURLsQuery() + " WHERE networks.id = ?"
+	return fmt.Sprintf(`%s WHERE networks.id = ?`, e.allURLsQuery())
 }
 
 func (e entityTypeNetwork) idFromURLQuery() string {
