@@ -19,11 +19,11 @@ JOIN projects ON networks_zones.project_id = projects.id`, e.code())
 }
 
 func (e entityTypeNetworkZone) urlsByProjectQuery() string {
-	return e.allURLsQuery() + " WHERE projects.name = ?"
+	return fmt.Sprintf(`%s WHERE projects.name = ?`, e.allURLsQuery())
 }
 
 func (e entityTypeNetworkZone) urlByIDQuery() string {
-	return e.allURLsQuery() + " WHERE networks_zones.id = ?"
+	return fmt.Sprintf(`%s WHERE networks_zones.id = ?`, e.allURLsQuery())
 }
 
 func (e entityTypeNetworkZone) idFromURLQuery() string {
