@@ -12,11 +12,16 @@ import (
 	"errors"
 	"fmt"
 	"net/http"
+	"strings"
+	"time"
+
+	"github.com/google/uuid"
 
 	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/certificate"
 	"github.com/canonical/lxd/lxd/db/query"
 	"github.com/canonical/lxd/lxd/identity"
+	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/entity"
 )
@@ -47,6 +52,8 @@ import (
 //go:generate mapper method -i -e identity DeleteOne-by-AuthMethod-and-Identifier
 //go:generate mapper method -i -e identity DeleteMany-by-Name-and-Type
 //go:generate mapper method -i -e identity Update struct=Identity
+//go:generate goimports -w identities.mapper.go
+//go:generate goimports -w identities.interface.mapper.go
 
 // AuthMethod is a database representation of an authentication method.
 //
@@ -117,6 +124,8 @@ const (
 	identityTypeCertificateMetricsRestricted   int64 = 4
 	identityTypeOIDCClient                     int64 = 5
 	identityTypeCertificateMetricsUnrestricted int64 = 6
+	identityTypeCertificateClient              int64 = 7
+	identityTypeCertificateClientPending       int64 = 8
 )
 
 // Scan implements sql.Scanner for IdentityType. This converts the integer value back into the correct API constant or
@@ -149,6 +158,10 @@ func (i *IdentityType) Scan(value any) error {
 		*i = api.IdentityTypeCertificateMetricsUnrestricted
 	case identityTypeOIDCClient:
 		*i = api.IdentityTypeOIDCClient
+	case identityTypeCertificateClient:
+		*i = api.IdentityTypeCertificateClient
+	case identityTypeCertificateClientPending:
+		*i = api.IdentityTypeCertificateClientPending
 	default:
 		return fmt.Errorf("Unknown identity type `%d`", identityTypeInt)
 	}
@@ -171,6 +184,10 @@ func (i IdentityType) Value() (driver.Value, error) {
 		return identityTypeCertificateMetricsUnrestricted, nil
 	case api.IdentityTypeOIDCClient:
 		return identityTypeOIDCClient, nil
+	case api.IdentityTypeCertificateClient:
+		return identityTypeCertificateClient, nil
+	case api.IdentityTypeCertificateClientPending:
+		return identityTypeCertificateClientPending, nil
 	}
 
 	return nil, fmt.Errorf("Invalid identity type %q", i)
@@ -270,10 +287,16 @@ func (i Identity) ToCertificate() (*Certificate, error) {
 	return c, nil
 }
 
-// X509 returns an x509.Certificate from the identity metadata. The AuthMethod of the Identity must be api.AuthenticationMethodTLS.
-func (i Identity) X509() (*x509.Certificate, error) {
+// CertificateMetadata returns the metadata associated with the identity as CertificateMetadata. It fails if the
+// authentication method is not api.AuthentictionMethodTLS or if the type is api.IdentityTypeClientCertificatePending,
+// as they do not have metadata of this type.
+func (i Identity) CertificateMetadata() (*CertificateMetadata, error) {
 	if i.AuthMethod != api.AuthenticationMethodTLS {
-		return nil, fmt.Errorf("Cannot extract X509 certificate from identity: Identity has authentication method %q (%q required)", i.AuthMethod, api.AuthenticationMethodTLS)
+		return nil, fmt.Errorf("Cannot get certificate metadata: Identity has authentication method %q (%q required)", i.AuthMethod, api.AuthenticationMethodTLS)
+	}
+
+	if i.Type == api.IdentityTypeCertificateClientPending {
+		return nil, fmt.Errorf("Cannot get certificate metadata: Identity is pending")
 	}
 
 	var metadata CertificateMetadata
@@ -282,6 +305,16 @@ func (i Identity) X509() (*x509.Certificate, error) {
 		return nil, fmt.Errorf("Failed to unmarshal certificate identity metadata: %w", err)
 	}
 
+	return &metadata, nil
+}
+
+// X509 returns an x509.Certificate from the identity metadata. The AuthMethod of the Identity must be api.AuthenticationMethodTLS.
+func (i Identity) X509() (*x509.Certificate, error) {
+	metadata, err := i.CertificateMetadata()
+	if err != nil {
+		return nil, err
+	}
+
 	return metadata.X509()
 }
 
@@ -305,6 +338,27 @@ func (i Identity) Subject() (string, error) {
 	return metadata.Subject, nil
 }
 
+// PendingTLSMetadata contains metadata for the pending TLS certificate identity type.
+type PendingTLSMetadata struct {
+	Secret string    `json:"secret"`
+	Expiry time.Time `json:"expiry"`
+}
+
+// PendingTLSMetadata returns the pending TLS identity metadata.
+func (i Identity) PendingTLSMetadata() (*PendingTLSMetadata, error) {
+	if i.Type != api.IdentityTypeCertificateClientPending {
+		return nil, api.NewStatusError(http.StatusBadRequest, "Cannot extract pending TLS identity secret: Identity is not pending")
+	}
+
+	var metadata PendingTLSMetadata
+	err := json.Unmarshal([]byte(i.Metadata), &metadata)
+	if err != nil {
+		return nil, api.StatusErrorf(http.StatusInternalServerError, "Failed to unmarshal pending TLS identity metadata: %w", err)
+	}
+
+	return &metadata, nil
+}
+
 // ToAPI converts an Identity to an api.Identity, executing database queries as necessary.
 func (i *Identity) ToAPI(ctx context.Context, tx *sql.Tx, canViewGroup auth.PermissionChecker) (*api.Identity, error) {
 	groups, err := GetAuthGroupsByIdentityID(ctx, tx, i.ID)
@@ -319,15 +373,85 @@ func (i *Identity) ToAPI(ctx context.Context, tx *sql.Tx, canViewGroup auth.Perm
 		}
 	}
 
+	var tlsCertificate string
+	if i.AuthMethod == api.AuthenticationMethodTLS && i.Type != api.IdentityTypeCertificateClientPending {
+		metadata, err := i.CertificateMetadata()
+		if err != nil {
+			return nil, err
+		}
+
+		tlsCertificate = metadata.Certificate
+	}
+
 	return &api.Identity{
 		AuthenticationMethod: string(i.AuthMethod),
 		Type:                 string(i.Type),
 		Identifier:           i.Identifier,
 		Name:                 i.Name,
 		Groups:               groupNames,
+		TLSCertificate:       tlsCertificate,
 	}, nil
 }
 
+// ActivateTLSIdentity updates a TLS identity to make it valid by adding the fingerprint, PEM encoded certificate, and setting
+// the type to api.IdentityTypeCertificateClient.
+func ActivateTLSIdentity(ctx context.Context, tx *sql.Tx, identifier uuid.UUID, cert *x509.Certificate) error {
+	fingerprint := shared.CertFingerprint(cert)
+	_, err := GetIdentityID(ctx, tx, api.AuthenticationMethodTLS, fingerprint)
+	if err == nil {
+		return api.StatusErrorf(http.StatusConflict, "Identity already exists")
+	}
+
+	metadata := CertificateMetadata{Certificate: string(pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: cert.Raw}))}
+	b, err := json.Marshal(metadata)
+	if err != nil {
+		return fmt.Errorf("Failed to encode certificate metadata: %w", err)
+	}
+
+	stmt := `UPDATE identities SET type = ?, identifier = ?, metadata = ? WHERE identifier = ? AND auth_method = ?`
+	res, err := tx.ExecContext(ctx, stmt, identityTypeCertificateClient, fingerprint, string(b), identifier.String(), authMethodTLS)
+	if err != nil {
+		return fmt.Errorf("Failed to activate TLS identity: %w", err)
+	}
+
+	n, err := res.RowsAffected()
+	if err != nil {
+		return fmt.Errorf("Failed to check for activated TLS identity: %w", err)
+	}
+
+	if n == 0 {
+		return api.StatusErrorf(http.StatusNotFound, "No pending TLS identity found with identifier %q", identifier)
+	} else if n > 1 {
+		return fmt.Errorf("Unknown error occurred when activating a TLS identity: %w", err)
+	}
+
+	return nil
+}
+
+var getPendingTLSIdentityByTokenSecretStmt = fmt.Sprintf(`
+SELECT identities.id, identities.auth_method, identities.type, identities.identifier, identities.name, identities.metadata
+	FROM identities
+	WHERE identities.type = %d
+	AND json_extract(identities.metadata, '$.secret') = ?
+`, identityTypeCertificateClientPending)
+
+// GetPendingTLSIdentityByTokenSecret gets a single identity of type identityTypeCertificateClientPending with the given
+// secret in its metadata. If no pending identity is found, an api.StatusError is returned with http.StatusNotFound.
+func GetPendingTLSIdentityByTokenSecret(ctx context.Context, tx *sql.Tx, secret string) (*Identity, error) {
+	identities, err := getIdentitysRaw(ctx, tx, getPendingTLSIdentityByTokenSecretStmt, secret)
+	if err != nil {
+		return nil, err
+	}
+
+	if len(identities) == 0 {
+		return nil, api.NewStatusError(http.StatusNotFound, "No pending identities found with given secret")
+	} else if len(identities) > 1 {
+		return nil, errors.New("Multiple pending identities found with given secret")
+	}
+
+	return &identities[0], nil
+}
+
 // GetAuthGroupsByIdentityID returns a slice of groups that the identity with the given ID is a member of.
 func GetAuthGroupsByIdentityID(ctx context.Context, tx *sql.Tx, identityID int) ([]AuthGroup, error) {
 	stmt := `
@@ -451,6 +575,24 @@ WHERE auth_groups.name IN %s
 	}
 
 	if int(rowsAffected) != len(groupNames) {
+		// No longer on happy "fast" path.
+		// Check each group name exists to return a nice error
+		missingGroups := make([]string, 0, len(groupNames))
+		for _, groupName := range groupNames {
+			exists, err := AuthGroupExists(ctx, tx, groupName)
+			if err != nil {
+				return err
+			}
+
+			if !exists {
+				missingGroups = append(missingGroups, `"`+groupName+`"`)
+			}
+		}
+
+		if len(missingGroups) > 0 {
+			return api.NewStatusError(http.StatusNotFound, "One or more groups were not found: "+strings.Join(missingGroups, ", "))
+		}
+
 		return fmt.Errorf("Failed to write expected number of rows to identity auth group association table (expected %d, got %d)", len(groupNames), rowsAffected)
 	}
 
