@@ -3,21 +3,17 @@ package cluster
 import (
 	"context"
 	"database/sql"
-	"errors"
 	"fmt"
-	"net/http"
 	"path/filepath"
 	"strings"
 	"sync/atomic"
 
-	"github.com/canonical/go-dqlite/v2/driver"
+	"github.com/canonical/go-dqlite/v3/driver"
 
-	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/db/query"
 	"github.com/canonical/lxd/lxd/db/schema"
 	"github.com/canonical/lxd/lxd/util"
 	"github.com/canonical/lxd/shared"
-	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/logger"
 	"github.com/canonical/lxd/shared/osarch"
 	"github.com/canonical/lxd/shared/version"
@@ -56,11 +52,14 @@ func Open(name string, store driver.NodeStore, options ...driver.Option) (*sql.D
 
 // EnsureSchema applies all relevant schema updates to the cluster database.
 //
-// Before actually doing anything, this function will make sure that all members in the cluster have a schema
-// version and a number of API extensions that match our one.
-// If it's not the case an api.StatusError with code http.StatusPreconditionFailed is returned,
-// this indicates that this member should wait for them to become aligned.
-func EnsureSchema(db *sql.DB, address string, dir string) error {
+// Before actually doing anything, this function will make sure that all nodes
+// in the cluster have a schema version and a number of API extensions that
+// match our one. If it's not the case, we either return an error (if some
+// nodes have version greater than us and we need to be upgraded), or return
+// false and no error (if some nodes have a lower version, and we need to wait
+// till they get upgraded and restarted).
+func EnsureSchema(db *sql.DB, address string, dir string) (bool, error) {
+	someNodesAreBehind := false
 	apiExtensions := version.APIExtensionsCount()
 
 	backupDone := false
@@ -84,7 +83,7 @@ func EnsureSchema(db *sql.DB, address string, dir string) error {
 
 		if n > 1 {
 			// This should never happen, since we only add cluster members with valid addresses.
-			return errors.New("Found more than one cluster member with a standalone address (0.0.0.0)")
+			return fmt.Errorf("Found more than one cluster member with a standalone address (0.0.0.0)")
 		} else if n == 1 {
 			clustered = false
 		}
@@ -94,7 +93,7 @@ func EnsureSchema(db *sql.DB, address string, dir string) error {
 		// non-clustered case, because otherwise the directory would be
 		// re-populated by replication.
 		if !clustered && !backupDone {
-			logger.Info("Updating the LXD global schema. Backup made as \"global.bak\"")
+			logger.Infof("Updating the LXD global schema. Backup made as \"global.bak\"")
 			err := shared.DirCopy(
 				filepath.Join(dir, "global"),
 				filepath.Join(dir, "global.bak"),
@@ -107,7 +106,7 @@ func EnsureSchema(db *sql.DB, address string, dir string) error {
 		}
 
 		if version == -1 {
-			logger.Debug("Running pre-update queries from file for global DB schema")
+			logger.Debugf("Running pre-update queries from file for global DB schema")
 		} else {
 			logger.Debugf("Updating global DB schema from %d to %d", version, version+1)
 		}
@@ -130,7 +129,7 @@ func EnsureSchema(db *sql.DB, address string, dir string) error {
 
 		if n > 1 {
 			// This should never happen, since we only add nodes with valid addresses.
-			return errors.New("Found more than one cluster member with a standalone address (0.0.0.0)")
+			return fmt.Errorf("Found more than one cluster member with a standalone address (0.0.0.0)")
 		} else if n == 1 {
 			address = "0.0.0.0" // We're not clustered
 		}
@@ -138,10 +137,16 @@ func EnsureSchema(db *sql.DB, address string, dir string) error {
 		// Update the schema and api_extension columns of ourselves.
 		err = updateNodeVersion(tx, address, apiExtensions)
 		if err != nil {
-			return fmt.Errorf("Failed to update cluster member version info for %q: %w", address, err)
+			return fmt.Errorf("Failed to update cluster member version info: %w", err)
 		}
 
-		return checkClusterIsUpgradable(ctx, tx, [2]int{len(updates), apiExtensions})
+		err = checkClusterIsUpgradable(ctx, tx, [2]int{len(updates), apiExtensions})
+		if err == errSomeNodesAreBehind {
+			someNodesAreBehind = true
+			return schema.ErrGracefulAbort
+		}
+
+		return err
 	}
 
 	schema := Schema()
@@ -166,8 +171,12 @@ func EnsureSchema(db *sql.DB, address string, dir string) error {
 
 		return err
 	})
+	if someNodesAreBehind {
+		return false, nil
+	}
+
 	if err != nil {
-		return err
+		return false, err
 	}
 
 	// When creating a database from scratch, insert an entry for node
@@ -176,10 +185,10 @@ func EnsureSchema(db *sql.DB, address string, dir string) error {
 	if initial == 0 {
 		arch, err := osarch.ArchitectureGetLocalID()
 		if err != nil {
-			return err
+			return false, err
 		}
 
-		err = query.Transaction(context.TODO(), db, func(_ context.Context, tx *sql.Tx) error {
+		err = query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
 			stmt := `
 INSERT INTO nodes(id, name, address, schema, api_extensions, arch, description) VALUES(1, 'none', '0.0.0.0', ?, ?, ?, '')
 `
@@ -202,19 +211,6 @@ INSERT INTO nodes(id, name, address, schema, api_extensions, arch, description)
 				return err
 			}
 
-			// Server administrators auth group
-			stmt = `INSERT INTO auth_groups (name, description) VALUES ('admins', 'Server administrators')`
-			_, err = tx.Exec(stmt)
-			if err != nil {
-				return err
-			}
-
-			stmt = `INSERT INTO auth_groups_permissions (auth_group_id, entity_type, entity_id, entitlement) VALUES (1, ?, 0, ?)`
-			_, err = tx.Exec(stmt, entityTypeCodeServer, string(auth.EntitlementAdmin))
-			if err != nil {
-				return err
-			}
-
 			// Default profile
 			stmt = `
 INSERT INTO profiles (name, description, project_id) VALUES ('default', 'Default LXD profile', 1)
@@ -237,11 +233,11 @@ INSERT INTO nodes_cluster_groups (node_id, group_id) VALUES(1, 1);
 			return nil
 		})
 		if err != nil {
-			return err
+			return false, err
 		}
 	}
 
-	return nil
+	return true, err
 }
 
 // Generate a new name for the dqlite driver registration. We need it to be
@@ -261,7 +257,7 @@ func checkClusterIsUpgradable(ctx context.Context, tx *sql.Tx, target [2]int) er
 	// Get the current versions in the nodes table.
 	versions, err := selectNodesVersions(ctx, tx)
 	if err != nil {
-		return fmt.Errorf("Failed fetching current member versions: %w", err)
+		return fmt.Errorf("failed to fetch current nodes versions: %w", err)
 	}
 
 	for _, version := range versions {
@@ -272,18 +268,25 @@ func checkClusterIsUpgradable(ctx context.Context, tx *sql.Tx, target [2]int) er
 
 		switch n {
 		case 0:
-			// Versions are equal, there's hope for the update.
-			// Let's check the next member.
+			// Versions are equal, there's hope for the
+			// update. Let's check the next node.
 			continue
 		case 1:
-			// Our version is ahead, we should wait for other members to align to our version.
-			return api.StatusErrorf(http.StatusPreconditionFailed, "A cluster member's version (%v) is behind this cluster member's version (%v), please ensure versions match", version, target)
+			// Our version is bigger, we should stop here
+			// and wait for other nodes to be upgraded and
+			// restarted.
+			return errSomeNodesAreBehind
 		case 2:
-			// Our version is behind, we should wait for other members to align to our version.
-			return api.StatusErrorf(http.StatusPreconditionFailed, "This cluster member's version (%v) is behind another member's version (%v), please ensure versions match", target, version)
+			// Another node has a version greater than ours
+			// and presumeably is waiting for other nodes
+			// to upgrade. Let's error out and shutdown
+			// since we need a greater version.
+			return fmt.Errorf("This cluster member's version is behind, please upgrade")
 		default:
 			panic("Unexpected return value from compareVersions")
 		}
 	}
 	return nil
 }
+
+var errSomeNodesAreBehind = fmt.Errorf("Some cluster members are behind this cluster member's version")
