@@ -1,19 +1,18 @@
-package cluster
+package cluster_test
 
 import (
 	"context"
 	"database/sql"
-	"net/http"
+	"fmt"
 	"os"
 	"path/filepath"
-	"strconv"
 	"testing"
 
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
+	"github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/db/query"
-	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/osarch"
 	"github.com/canonical/lxd/shared/version"
 )
@@ -25,20 +24,21 @@ func TestEnsureSchema_NoClustered(t *testing.T) {
 	assert.NoError(t, os.Mkdir(filepath.Join(dir, "global"), 0711))
 	db := newDB(t)
 	addNode(t, db, "0.0.0.0", 1, 1)
-	err := EnsureSchema(db, "1.2.3.4:666", dir)
+	ready, err := cluster.EnsureSchema(db, "1.2.3.4:666", dir)
+	assert.True(t, ready)
 	assert.NoError(t, err)
 }
 
 // Exercise EnsureSchema failures when the cluster can't be upgraded right now.
 func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
-	schema := SchemaVersion
+	schema := cluster.SchemaVersion
 	apiExtensions := len(version.APIExtensions)
 
 	cases := []struct {
-		title              string
-		setup              func(*testing.T, *sql.DB)
-		apiStatusErrorCode int
-		error              string
+		title string
+		setup func(*testing.T, *sql.DB)
+		ready bool
+		error string
 	}{
 		{
 			`a node's schema version is behind`,
@@ -46,8 +46,8 @@ func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
 				addNode(t, db, "1", schema, apiExtensions)
 				addNode(t, db, "2", schema-1, apiExtensions)
 			},
-			http.StatusPreconditionFailed, // The schema was not updated
-			"",
+			false, // The schema was not updated
+			"",    // No error is returned
 		},
 		{
 			`a node's number of API extensions is behind`,
@@ -55,8 +55,8 @@ func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
 				addNode(t, db, "1", schema, apiExtensions)
 				addNode(t, db, "2", schema, apiExtensions-1)
 			},
-			http.StatusPreconditionFailed, // The schema was not updated
-			"",
+			false, // The schema was not updated
+			"",    // No error is returned
 		},
 		{
 			`this node's schema is behind`,
@@ -64,8 +64,8 @@ func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
 				addNode(t, db, "1", schema, apiExtensions)
 				addNode(t, db, "2", schema+1, apiExtensions)
 			},
-			http.StatusPreconditionFailed,
-			"",
+			false,
+			"Failed to ensure schema: This cluster member's version is behind, please upgrade",
 		},
 		{
 			`this node's number of API extensions is behind`,
@@ -73,8 +73,8 @@ func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
 				addNode(t, db, "1", schema, apiExtensions)
 				addNode(t, db, "2", schema, apiExtensions+1)
 			},
-			http.StatusPreconditionFailed,
-			"",
+			false,
+			"Failed to ensure schema: This cluster member's version is behind, please upgrade",
 		},
 		{
 			`inconsistent schema version and API extensions number`,
@@ -82,7 +82,7 @@ func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
 				addNode(t, db, "1", schema, apiExtensions)
 				addNode(t, db, "2", schema+1, apiExtensions-1)
 			},
-			0,
+			false,
 			"Failed to ensure schema: Cluster members have inconsistent versions",
 		},
 	}
@@ -91,12 +91,9 @@ func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
 		t.Run(c.title, func(t *testing.T) {
 			db := newDB(t)
 			c.setup(t, db)
-			err := EnsureSchema(db, "1", "/unused/db/dir")
-			if c.apiStatusErrorCode > 0 {
-				statusErrCode, isStatusErr := api.StatusErrorMatch(err)
-				assert.True(t, isStatusErr)
-				assert.Equal(t, c.apiStatusErrorCode, statusErrCode)
-			} else if c.error == "" {
+			ready, err := cluster.EnsureSchema(db, "1", "/unused/db/dir")
+			assert.Equal(t, c.ready, ready)
+			if c.error == "" {
 				assert.NoError(t, err)
 			} else {
 				assert.EqualError(t, err, c.error)
@@ -108,28 +105,28 @@ func TestEnsureSchema_ClusterNotUpgradable(t *testing.T) {
 // Regardless of whether the schema could actually be upgraded or not, the
 // version of this node gets updated.
 func TestEnsureSchema_UpdateNodeVersion(t *testing.T) {
-	schema := SchemaVersion
+	schema := cluster.SchemaVersion
 	apiExtensions := len(version.APIExtensions)
 
 	cases := []struct {
-		setup              func(*testing.T, *sql.DB)
-		apiStatusErrorCode int
+		setup func(*testing.T, *sql.DB)
+		ready bool
 	}{
 		{
-			func(_ *testing.T, _ *sql.DB) {},
-			0,
+			func(t *testing.T, db *sql.DB) {},
+			true,
 		},
 		{
 			func(t *testing.T, db *sql.DB) {
 				// Add a node which is behind.
 				addNode(t, db, "2", schema, apiExtensions-1)
 			},
-			0,
+			true,
 		},
 	}
 
-	for i, c := range cases {
-		t.Run(strconv.Itoa(i), func(t *testing.T) {
+	for _, c := range cases {
+		t.Run(fmt.Sprint(c.ready), func(t *testing.T) {
 			db := newDB(t)
 
 			// Add ourselves with an older schema version and API
@@ -137,15 +134,9 @@ func TestEnsureSchema_UpdateNodeVersion(t *testing.T) {
 			addNode(t, db, "1", schema-1, apiExtensions-1)
 
 			// Ensure the schema.
-			err := EnsureSchema(db, "1", "/unused/db/dir")
-
-			statusErrCode, isStatusErr := api.StatusErrorMatch(err)
-			if c.apiStatusErrorCode > 0 {
-				assert.True(t, isStatusErr)
-				assert.Equal(t, c.apiStatusErrorCode, statusErrCode)
-			} else {
-				assert.NoError(t, err)
-			}
+			ready, err := cluster.EnsureSchema(db, "1", "/unused/db/dir")
+			assert.NoError(t, err)
+			assert.Equal(t, c.ready, ready)
 
 			// Check that the nodes table was updated with our new
 			// schema version and API extensions number.
@@ -167,7 +158,7 @@ CREATE TABLE schema (
     UNIQUE (version)
 );
 `
-	_, err = db.Exec(createTableSchema + FreshSchema())
+	_, err = db.Exec(createTableSchema + cluster.FreshSchema())
 	require.NoError(t, err)
 
 	return db
@@ -175,7 +166,7 @@ CREATE TABLE schema (
 
 // Add a new node with the given address, schema version and number of api extensions.
 func addNode(t *testing.T, db *sql.DB, address string, schema int, apiExtensions int) {
-	err := query.Transaction(context.TODO(), db, func(_ context.Context, tx *sql.Tx) error {
+	err := query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
 		stmt := `
 INSERT INTO nodes(name, address, schema, api_extensions, arch, description) VALUES (?, ?, ?, ?, ?, '')
 `
