@@ -7,7 +7,6 @@ import (
 	"database/sql"
 	"errors"
 	"fmt"
-	"net/http"
 	"os"
 	"path/filepath"
 	"regexp"
@@ -16,13 +15,12 @@ import (
 	"sync"
 	"time"
 
-	"github.com/canonical/go-dqlite/v2/driver"
+	"github.com/canonical/go-dqlite/v3/driver"
 
 	"github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/db/node"
 	"github.com/canonical/lxd/lxd/db/query"
 	"github.com/canonical/lxd/shared"
-	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/logger"
 )
 
@@ -38,9 +36,6 @@ type Node struct {
 	dir string  // Reference to the directory where the database file lives.
 }
 
-// Transactor is used to run transactions against the cluster database.
-type Transactor func(ctx context.Context, f func(context.Context, *ClusterTx) error) error
-
 // OpenNode creates a new Node object.
 //
 // The fresh hook parameter is used by the daemon to mark all known patch names
@@ -164,7 +159,7 @@ func OpenCluster(closingCtx context.Context, name string, store driver.NodeStore
 		logPriority := 1 // 0 is discard, 1 is Debug, 2 is Error
 		if i > 5 {
 			logPriority = 2
-			if i > 15 && (i%5) != 0 {
+			if i > 15 && !((i % 5) == 0) {
 				logPriority = 0
 			}
 		}
@@ -188,12 +183,12 @@ func OpenCluster(closingCtx context.Context, name string, store driver.NodeStore
 			logger.Error("Failed connecting to global database", logCtx)
 		}
 
-		err = connectCtx.Err()
-		if err != nil {
-			return nil, err
+		select {
+		case <-connectCtx.Done():
+			return nil, connectCtx.Err()
+		default:
+			time.Sleep(2 * time.Second)
 		}
-
-		time.Sleep(2 * time.Second)
 	}
 
 	// FIXME: https://github.com/canonical/dqlite/issues/163
@@ -203,8 +198,8 @@ func OpenCluster(closingCtx context.Context, name string, store driver.NodeStore
 	}
 
 	if dump != nil {
-		logger.Info("Migrating data from local to global database")
-		err := query.Transaction(closingCtx, db, func(ctx context.Context, tx *sql.Tx) error {
+		logger.Infof("Migrating data from local to global database")
+		err := query.Transaction(context.TODO(), db, func(ctx context.Context, tx *sql.Tx) error {
 			return importPreClusteringData(tx, dump)
 		})
 		if err != nil {
@@ -227,18 +222,18 @@ func OpenCluster(closingCtx context.Context, name string, store driver.NodeStore
 		}
 	}
 
-	err = cluster.EnsureSchema(db, address, dir)
+	nodesVersionsMatch, err := cluster.EnsureSchema(db, address, dir)
 	if err != nil {
-		if api.StatusErrorCheck(err, http.StatusPreconditionFailed) {
-			cluster := &Cluster{
-				db:         db,
-				closingCtx: closingCtx,
-			}
+		return nil, fmt.Errorf("failed to ensure schema: %w", err)
+	}
 
-			return cluster, err
+	if !nodesVersionsMatch {
+		cluster := &Cluster{
+			db:         db,
+			closingCtx: closingCtx,
 		}
 
-		return nil, fmt.Errorf("Failed to ensure schema: %w", err)
+		return cluster, ErrSomeNodesAreBehind
 	}
 
 	stmts, err := cluster.PrepareStmts(db, false)
@@ -253,7 +248,7 @@ func OpenCluster(closingCtx context.Context, name string, store driver.NodeStore
 		closingCtx: closingCtx,
 	}
 
-	err = clusterDB.Transaction(closingCtx, func(ctx context.Context, tx *ClusterTx) error {
+	err = clusterDB.Transaction(context.TODO(), func(ctx context.Context, tx *ClusterTx) error {
 		// Figure out the ID of this node.
 		members, err := tx.GetNodes(ctx)
 		if err != nil {
@@ -292,9 +287,13 @@ func OpenCluster(closingCtx context.Context, name string, store driver.NodeStore
 		return nil, err
 	}
 
-	return clusterDB, nil
+	return clusterDB, err
 }
 
+// ErrSomeNodesAreBehind is returned by OpenCluster if some of the nodes in the
+// cluster have a schema or API version that is less recent than this node.
+var ErrSomeNodesAreBehind = fmt.Errorf("some nodes are behind this node's version")
+
 // ForLocalInspection is a aid for the hack in initializeDbObject, which
 // sets the db-related Deamon attributes upfront, to be backward compatible
 // with the legacy patches that need to interact with the database.
@@ -338,45 +337,34 @@ func (c *Cluster) Transaction(ctx context.Context, f func(context.Context, *Clus
 	return c.transaction(ctx, f)
 }
 
-// RunExclusive acquires a lock on the cluster db and calls f.
-// Any successive call to Transaction() will block until f has returned.
-// f is passed a Transactor that can be used to run transactions against the locked database.
-func (c *Cluster) RunExclusive(f func(t Transactor) error) error {
-	logger.Info("Acquiring exclusive lock on cluster database")
-
-	unlock := func() {
-		logger.Info("Releasing exclusive lock on cluster database")
-		c.mu.Unlock()
-	}
-
-	timeout := 20 * time.Second
-	ctx, cancel := context.WithTimeout(context.Background(), timeout)
-	defer cancel()
+// EnterExclusive acquires a lock on the cluster db, so any successive call to
+// Transaction will block until ExitExclusive has been called.
+func (c *Cluster) EnterExclusive() error {
+	logger.Debug("Acquiring exclusive lock on cluster db")
 
 	ch := make(chan struct{})
 	go func() {
 		c.mu.Lock()
-
-		if ctx.Err() == nil {
-			// Close channel to indicate lock acquired. Safe even if outer function has returned.
-			close(ch)
-		} else {
-			// Release lock if outer function has timed out and returned.
-			// This avoids leaving the lock acquired permanently in the case of timeout.
-			unlock()
-		}
+		ch <- struct{}{}
 	}()
 
+	timeout := 20 * time.Second
 	select {
 	case <-ch:
-		err := f(c.transaction)
-		unlock()
-		return err
+		return nil
 	case <-time.After(timeout):
-		return fmt.Errorf("Timed out exclusively locking cluster database (%s)", timeout)
+		return fmt.Errorf("timeout (%s)", timeout)
 	}
 }
 
+// ExitExclusive runs the given transaction and then releases the lock acquired
+// with EnterExclusive.
+func (c *Cluster) ExitExclusive(ctx context.Context, f func(context.Context, *ClusterTx) error) error {
+	logger.Debug("Releasing exclusive lock on cluster db")
+	defer c.mu.Unlock()
+	return c.transaction(ctx, f)
+}
+
 func (c *Cluster) transaction(ctx context.Context, f func(context.Context, *ClusterTx) error) error {
 	clusterTx := &ClusterTx{
 		nodeID: c.nodeID,
@@ -434,7 +422,7 @@ func (c *Cluster) Begin() (*sql.Tx, error) {
 }
 
 func begin(db *sql.DB) (*sql.Tx, error) {
-	for range 1000 {
+	for i := 0; i < 1000; i++ {
 		tx, err := db.Begin()
 		if err == nil {
 			return tx, nil
@@ -450,7 +438,7 @@ func begin(db *sql.DB) (*sql.Tx, error) {
 
 	logger.Debug("DbBegin: DB still locked")
 	logger.Debug(logger.GetStack())
-	return nil, errors.New("DB is locked")
+	return nil, fmt.Errorf("DB is locked")
 }
 
 // TxCommit commits the given transaction.
@@ -512,3 +500,77 @@ func DqliteLatestSegment() (string, error) {
 func dbQueryRowScan(ctx context.Context, c *ClusterTx, q string, args []any, outargs []any) error {
 	return c.tx.QueryRowContext(ctx, q, args...).Scan(outargs...)
 }
+
+/*
+ * . db a reference to a sql.DB instance
+ * . q is the database query
+ * . inargs is an array of interfaces containing the query arguments
+ * . outfmt is an array of interfaces containing the right types of output
+ *   arguments, i.e.
+ *      var arg1 string
+ *      var arg2 int
+ *      outfmt := {}any{arg1, arg2}
+ *
+ * The result will be an array (one per output row) of arrays (one per output argument)
+ * of interfaces, containing pointers to the actual output arguments.
+ */
+func queryScan(ctx context.Context, c *ClusterTx, q string, inargs []any, outfmt []any) ([][]any, error) {
+	result := [][]any{}
+
+	rows, err := c.tx.QueryContext(ctx, q, inargs...)
+	if err != nil {
+		return [][]any{}, err
+	}
+
+	defer func() { _ = rows.Close() }()
+
+	for rows.Next() {
+		ptrargs := make([]any, len(outfmt))
+		for i := range outfmt {
+			switch t := outfmt[i].(type) {
+			case string:
+				str := ""
+				ptrargs[i] = &str
+			case int:
+				integer := 0
+				ptrargs[i] = &integer
+			case int64:
+				integer := int64(0)
+				ptrargs[i] = &integer
+			case bool:
+				boolean := bool(false)
+				ptrargs[i] = &boolean
+			default:
+				return [][]any{}, fmt.Errorf("Bad interface type: %s", t)
+			}
+		}
+		err = rows.Scan(ptrargs...)
+		if err != nil {
+			return [][]any{}, err
+		}
+
+		newargs := make([]any, len(outfmt))
+		for i := range ptrargs {
+			switch t := outfmt[i].(type) {
+			case string:
+				newargs[i] = *ptrargs[i].(*string)
+			case int:
+				newargs[i] = *ptrargs[i].(*int)
+			case int64:
+				newargs[i] = *ptrargs[i].(*int64)
+			case bool:
+				newargs[i] = *ptrargs[i].(*bool)
+			default:
+				return [][]any{}, fmt.Errorf("Bad interface type: %s", t)
+			}
+		}
+		result = append(result, newargs)
+	}
+
+	err = rows.Err()
+	if err != nil {
+		return [][]any{}, err
+	}
+
+	return result, nil
+}
