@@ -1,16 +1,16 @@
 package db
 
 import (
-	"errors"
+	"fmt"
 	"net/http"
 
-	"github.com/canonical/go-dqlite/v2/driver"
+	"github.com/canonical/go-dqlite/v3/driver"
 	"github.com/mattn/go-sqlite3"
 )
 
 var (
 	// ErrNoClusterMember is used to indicate no cluster member has been found for a resource.
-	ErrNoClusterMember = errors.New("No cluster member found")
+	ErrNoClusterMember = fmt.Errorf("No cluster member found")
 )
 
 // SmartErrors are used to return more appropriate errors to the caller.
