@@ -106,10 +106,10 @@ func destFunc(slice string, typ string, fields []*Field) string {
 	for i, field := range fields {
 		var arg string
 		if shared.IsTrue(field.Config.Get("marshal")) {
-			declVarName := lex.Minuscule(field.Name) + "Str"
+			declVarName := fmt.Sprintf("%sStr", lex.Minuscule(field.Name))
 			declVarNames = append(declVarNames, declVarName)
 			declVars[declVarName] = field
-			arg = "&" + declVarName
+			arg = fmt.Sprintf("&%s", declVarName)
 		} else {
 			arg = fmt.Sprintf("&%s.%s", varName, field.Name)
 		}
