@@ -145,6 +145,7 @@ func (c *ClusterTx) imageFill(ctx context.Context, id int, image *api.Image, cre
 	_, source, err := c.GetImageSource(ctx, id)
 	if err == nil {
 		image.UpdateSource = &source
+		image.UpdateSource.ImageType = image.Type
 	}
 
 	return nil
@@ -400,6 +401,7 @@ func (c *ClusterTx) GetImageByFingerprintPrefix(ctx context.Context, fingerprint
 	image.Cached = object.Cached
 	image.Public = object.Public
 	image.AutoUpdate = object.AutoUpdate
+	image.Project = object.Project
 
 	err = c.imageFill(
 		ctx, object.ID, &image,
@@ -526,8 +528,8 @@ SELECT nodes.address FROM nodes
   LEFT JOIN images ON images_nodes.image_id = images.id
 WHERE images.fingerprint = ?
 `
-	var localAddress string // Address of this node
-	var addresses []string  // Addresses of online nodes with the image
+	// Address of this node
+	var localAddress string
 
 	offlineThreshold, err := c.GetNodeOfflineThreshold(ctx)
 	if err != nil {
@@ -544,6 +546,9 @@ WHERE images.fingerprint = ?
 		return "", err
 	}
 
+	// Addresses of online nodes with the image
+	addresses := make([]string, 0, len(allAddresses))
+
 	for _, address := range allAddresses {
 		node, err := c.GetNodeByAddress(ctx, address)
 		if err != nil {
@@ -773,6 +778,15 @@ func (c *ClusterTx) SetImageCachedAndLastUseDate(ctx context.Context, projectNam
 	return err
 }
 
+// UnsetImageCached unsets the cached field of the image with the given fingerprint.
+func (c *ClusterTx) UnsetImageCached(ctx context.Context, projectName string, fingerprint string) error {
+	stmt := `UPDATE images SET cached=0 WHERE fingerprint=? AND project_id = (SELECT id FROM projects WHERE name = ? LIMIT 1)`
+
+	_, err := c.tx.ExecContext(ctx, stmt, fingerprint, projectName)
+
+	return err
+}
+
 // UpdateImage updates the image with the given ID.
 func (c *ClusterTx) UpdateImage(ctx context.Context, id int, fname string, sz int64, public bool, autoUpdate bool, architecture string, createdAt time.Time, expiresAt time.Time, properties map[string]string, project string, profileIDs []int64) error {
 	arch, err := osarch.ArchitectureId(architecture)
@@ -990,7 +1004,7 @@ func (c *ClusterTx) GetPoolNamesFromIDs(ctx context.Context, poolIDs []int64) ([
 		args[i] = id
 	}
 
-	q := fmt.Sprintf("SELECT name FROM storage_pools WHERE id IN (%s)", strings.Join(params, ","))
+	q := "SELECT name FROM storage_pools WHERE id IN (" + strings.Join(params, ",") + ")"
 
 	poolNames, err := query.SelectStrings(ctx, c.tx, q, args...)
 	if err != nil {
@@ -1101,8 +1115,6 @@ SELECT DISTINCT nodes.address FROM nodes WHERE nodes.address NOT IN (
 }
 
 func (c *ClusterTx) getNodesByImageFingerprint(ctx context.Context, stmt string, fingerprint string, autoUpdate *bool) ([]string, error) {
-	var addresses []string // Addresses of online nodes with the image
-
 	offlineThreshold, err := c.GetNodeOfflineThreshold(ctx)
 	if err != nil {
 		return nil, err
@@ -1120,6 +1132,9 @@ func (c *ClusterTx) getNodesByImageFingerprint(ctx context.Context, stmt string,
 		return nil, err
 	}
 
+	// Addresses of online nodes with the image
+	addresses := make([]string, 0, len(allAddresses))
+
 	for _, address := range allAddresses {
 		node, err := c.GetNodeByAddress(ctx, address)
 		if err != nil {
