@@ -8,6 +8,7 @@ import (
 	"fmt"
 	"net/http"
 	"sort"
+	"strconv"
 	"strings"
 	"time"
 
@@ -17,6 +18,7 @@ import (
 	"github.com/canonical/lxd/lxd/instance/instancetype"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/osarch"
 )
 
 // InstanceArgs is a value object holding all db-related details about an instance.
@@ -44,6 +46,37 @@ type InstanceArgs struct {
 	ExpiryDate   time.Time
 }
 
+// ToAPI converts InstanceArgs to api.Instance.
+// The returned Instance has ExpandedConfig and ExpandedDevices set.
+func (i *InstanceArgs) ToAPI() (*api.Instance, error) {
+	var err error
+
+	rslt := &api.Instance{
+		Name:        i.Name,
+		Description: i.Description,
+
+		CreatedAt:  i.CreationDate,
+		LastUsedAt: i.LastUsedDate,
+		Location:   i.Node,
+		Type:       i.Type.String(),
+		Project:    i.Project,
+		Ephemeral:  i.Ephemeral,
+		Stateful:   i.Stateful,
+
+		Config:  i.Config,
+		Devices: i.Devices.CloneNative(),
+	}
+
+	rslt.Architecture, err = osarch.ArchitectureName(i.Architecture)
+
+	rslt.Profiles = make([]string, 0, len(i.Profiles))
+	for _, profile := range i.Profiles {
+		rslt.Profiles = append(rslt.Profiles, profile.Name)
+	}
+
+	return rslt, err
+}
+
 // GetInstanceNames returns the names of all containers the given project.
 func (c *ClusterTx) GetInstanceNames(ctx context.Context, project string) ([]string, error) {
 	stmt := `
@@ -74,37 +107,34 @@ func (c *ClusterTx) GetNodeAddressOfInstance(ctx context.Context, project string
 		args = append(args, instType)
 	}
 
-	if strings.Contains(name, shared.SnapshotDelimiter) {
-		parts := strings.SplitN(name, shared.SnapshotDelimiter, 2)
-
+	instanceName, snapshotName, found := strings.Cut(name, shared.SnapshotDelimiter)
+	if found {
 		// Instance name filter.
 		filters.WriteString(" AND instances.name = ?")
-		args = append(args, parts[0])
+		args = append(args, instanceName)
 
 		// Snapshot name filter.
 		filters.WriteString(" AND instances_snapshots.name = ?")
-		args = append(args, parts[1])
+		args = append(args, snapshotName)
 
-		stmt = fmt.Sprintf(`
+		stmt = `
 SELECT nodes.id, nodes.address
   FROM nodes
   JOIN instances ON instances.node_id = nodes.id
   JOIN projects ON projects.id = instances.project_id
   JOIN instances_snapshots ON instances_snapshots.instance_id = instances.id
- WHERE %s
-`, filters.String())
+ WHERE ` + filters.String()
 	} else {
 		// Instance name filter.
 		filters.WriteString(" AND instances.name = ?")
-		args = append(args, name)
+		args = append(args, instanceName)
 
-		stmt = fmt.Sprintf(`
+		stmt = `
 SELECT nodes.id, nodes.address
   FROM nodes
   JOIN instances ON instances.node_id = nodes.id
   JOIN projects ON projects.id = instances.project_id
- WHERE %s
-`, filters.String())
+ WHERE ` + filters.String()
 	}
 
 	var address string
@@ -167,7 +197,7 @@ func (c *ClusterTx) GetInstancesByMemberAddress(ctx context.Context, offlineThre
 	`)
 
 	// Project filter.
-	q.WriteString(fmt.Sprintf("WHERE projects.name IN %s", query.Params(len(projects))))
+	q.WriteString("WHERE projects.name IN " + query.Params(len(projects)))
 	for _, project := range projects {
 		args = append(args, project)
 	}
@@ -216,8 +246,8 @@ func (c *ClusterTx) GetInstancesByMemberAddress(ctx context.Context, offlineThre
 	return memberAddressInstances, nil
 }
 
-// ErrInstanceListStop used as return value from InstanceList's instanceFunc when prematurely stopping the search.
-var ErrInstanceListStop = fmt.Errorf("search stopped")
+// ErrListStop used as return value from InstanceList's instanceFunc when prematurely stopping the search.
+var ErrListStop = fmt.Errorf("search stopped")
 
 // InstanceList loads all instances across all projects and for each instance runs the instanceFunc passing in the
 // instance and it's project and profiles. Accepts optional filter arguments to specify a subset of instances.
@@ -332,7 +362,7 @@ func (c *ClusterTx) instanceConfigFill(ctx context.Context, snapshotsMode bool,
 
 		first = false
 
-		q.WriteString(fmt.Sprintf("%d", instanceID))
+		q.WriteString(strconv.Itoa(instanceID))
 	}
 
 	q.WriteString(`)`)
@@ -412,7 +442,7 @@ func (c *ClusterTx) instanceDevicesFill(ctx context.Context, snapshotsMode bool,
 
 		first = false
 
-		q.WriteString(fmt.Sprintf("%d", instanceID))
+		q.WriteString(strconv.Itoa(instanceID))
 	}
 
 	q.WriteString(`)`)
@@ -461,6 +491,37 @@ func (c *ClusterTx) instanceDevicesFill(ctx context.Context, snapshotsMode bool,
 // instanceProfiles loads the profile IDs to apply to an instance (in the application order) for all
 // instanceIDs in a single query and then updates the instanceApplyProfileIDs and profilesByID maps.
 func (c *ClusterTx) instanceProfilesFill(ctx context.Context, snapshotsMode bool, instanceArgs *map[int]InstanceArgs) error {
+	profilesByID := make(map[int]*api.Profile)
+
+	// Get all profiles.
+	profiles, err := cluster.GetProfiles(ctx, c.Tx())
+	if err != nil {
+		return fmt.Errorf("Failed loading profiles: %w", err)
+	}
+
+	// Get all the profile configs.
+	profileConfigs, err := cluster.GetConfig(ctx, c.Tx(), "profile")
+	if err != nil {
+		return fmt.Errorf("Failed loading profile configs: %w", err)
+	}
+
+	// Get all the profile devices.
+	profileDevices, err := cluster.GetDevices(ctx, c.Tx(), "profile")
+	if err != nil {
+		return fmt.Errorf("Failed loading profile devices: %w", err)
+	}
+
+	for _, profile := range profiles {
+		profilesByID[profile.ID], err = profile.ToAPI(ctx, c.tx, profileConfigs, profileDevices)
+		if err != nil {
+			return err
+		}
+	}
+
+	return c.instanceProfilesFillWithProfiles(ctx, snapshotsMode, instanceArgs, profilesByID)
+}
+
+func (c *ClusterTx) instanceProfilesFillWithProfiles(ctx context.Context, snapshotsMode bool, instanceArgs *map[int]InstanceArgs, profilesByID map[int]*api.Profile) error {
 	instances := *instanceArgs
 
 	// Get profiles referenced by instances.
@@ -496,13 +557,12 @@ func (c *ClusterTx) instanceProfilesFill(ctx context.Context, snapshotsMode bool
 
 		first = false
 
-		q.WriteString(fmt.Sprintf("%d", instanceID))
+		q.WriteString(strconv.Itoa(instanceID))
 	}
 
 	q.WriteString(`)
 		ORDER BY instances_profiles.instance_id, instances_profiles.apply_order`)
 
-	profilesByID := make(map[int]*api.Profile)
 	instanceApplyProfileIDs := make(map[int64][]int, len(instances))
 
 	err := query.Scan(ctx, c.Tx(), q.String(), func(scan func(dest ...any) error) error {
@@ -516,39 +576,12 @@ func (c *ClusterTx) instanceProfilesFill(ctx context.Context, snapshotsMode bool
 
 		instanceApplyProfileIDs[instanceID] = append(instanceApplyProfileIDs[instanceID], profileID)
 
-		// Record that this profile is referenced by at least one instance in the list.
-		_, ok := profilesByID[profileID]
-		if !ok {
-			profilesByID[profileID] = nil
-		}
-
 		return nil
 	})
 	if err != nil {
 		return err
 	}
 
-	// Get all profiles.
-	profiles, err := cluster.GetProfiles(context.TODO(), c.Tx())
-	if err != nil {
-		return fmt.Errorf("Failed loading profiles: %w", err)
-	}
-
-	// Populate profilesByID map entry for referenced profiles.
-	// This way we only call ToAPI() on the profiles actually referenced by the instances in
-	// the list, which can reduce the number of queries run.
-	for _, profile := range profiles {
-		_, ok := profilesByID[profile.ID]
-		if !ok {
-			continue
-		}
-
-		profilesByID[profile.ID], err = profile.ToAPI(context.TODO(), c.tx)
-		if err != nil {
-			return err
-		}
-	}
-
 	// Populate instance profiles list in apply order.
 	for instanceID := range instances {
 		inst := instances[instanceID]
@@ -855,7 +888,7 @@ func (c *ClusterTx) GetInstancePool(ctx context.Context, projectName string, ins
 	// unique, and their storage volumes carry the same name, their storage
 	// volumes are unique too.
 	poolName := ""
-	query := fmt.Sprintf(`
+	query := `
 SELECT storage_pools.name FROM storage_pools
   JOIN storage_volumes_all ON storage_pools.id=storage_volumes_all.storage_pool_id
   JOIN instances ON instances.name=storage_volumes_all.name
@@ -864,7 +897,7 @@ SELECT storage_pools.name FROM storage_pools
    AND storage_volumes_all.name=?
    AND storage_volumes_all.type IN (?,?)
    AND storage_volumes_all.project_id = instances.project_id
-   AND (storage_volumes_all.node_id=? OR storage_volumes_all.node_id IS NULL AND storage_pools.driver IN %s)`, query.Params(len(remoteDrivers)))
+   AND (storage_volumes_all.node_id=? OR storage_volumes_all.node_id IS NULL AND storage_pools.driver IN ` + query.Params(len(remoteDrivers)) + `)`
 	inargs := []any{projectName, instanceName, cluster.StoragePoolVolumeTypeContainer, cluster.StoragePoolVolumeTypeVM, c.nodeID}
 	outargs := []any{&poolName}
 
@@ -886,12 +919,12 @@ SELECT storage_pools.name FROM storage_pools
 
 // DeleteInstance removes the instance with the given name from the database.
 func (c *ClusterTx) DeleteInstance(ctx context.Context, project, name string) error {
-	if strings.Contains(name, shared.SnapshotDelimiter) {
-		parts := strings.SplitN(name, shared.SnapshotDelimiter, 2)
-		return cluster.DeleteInstanceSnapshot(ctx, c.tx, project, parts[0], parts[1])
+	instance, snapshot, found := strings.Cut(name, shared.SnapshotDelimiter)
+	if found {
+		return cluster.DeleteInstanceSnapshot(ctx, c.tx, project, instance, snapshot)
 	}
 
-	return cluster.DeleteInstance(ctx, c.tx, project, name)
+	return cluster.DeleteInstance(ctx, c.tx, project, instance)
 }
 
 // GetInstanceProjectAndName returns the project and the name of the instance
@@ -991,7 +1024,7 @@ ORDER BY instances_snapshots.creation_date, instances_snapshots.id
 
 // GetNextInstanceSnapshotIndex returns the index that the next snapshot of the
 // instance with the given name and pattern should have.
-func (c *ClusterTx) GetNextInstanceSnapshotIndex(ctx context.Context, project string, name string, pattern string) int {
+func (c *ClusterTx) GetNextInstanceSnapshotIndex(ctx context.Context, project string, name string, pattern string) (nextIndex int) {
 	q := `
 SELECT instances_snapshots.name
   FROM instances_snapshots
@@ -1004,33 +1037,34 @@ ORDER BY instances_snapshots.creation_date, instances_snapshots.id
 	inargs := []any{project, name}
 	outfmt := []any{numstr}
 
+	// Check if pattern is valid.
+	if !strings.Contains(pattern, "%d") {
+		return 0
+	}
+
 	results, err := queryScan(ctx, c, q, inargs, outfmt)
 	if err != nil {
 		return 0
 	}
 
-	max := 0
-
 	for _, r := range results {
 		snapOnlyName, ok := r[0].(string)
 		if !ok {
 			continue
 		}
 
-		fields := strings.SplitN(pattern, "%d", 2)
-
 		var num int
-		count, err := fmt.Sscanf(snapOnlyName, fmt.Sprintf("%s%%d%s", fields[0], fields[1]), &num)
+		count, err := fmt.Sscanf(snapOnlyName, pattern, &num)
 		if err != nil || count != 1 {
 			continue
 		}
 
-		if num >= max {
-			max = num + 1
+		if num >= nextIndex {
+			nextIndex = num + 1
 		}
 	}
 
-	return max
+	return nextIndex
 }
 
 // DeleteReadyStateFromLocalInstances deletes the volatile.last_state.ready config key
