@@ -8,10 +8,10 @@ import (
 	"fmt"
 	"strings"
 
-	"github.com/lxc/lxd/lxd/db/cluster"
-	"github.com/lxc/lxd/lxd/db/query"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/db/cluster"
+	"github.com/canonical/lxd/lxd/db/query"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 // LoadPreClusteringData loads all the data that before the introduction of
@@ -172,7 +172,7 @@ func importPreClusteringData(tx *sql.Tx, dump *Dump) error {
 						continue
 					}
 
-					if column == "key" && shared.StringInSlice(value, keys) {
+					if column == "key" && shared.ValueInSlice(value, keys) {
 						skip = true
 					}
 				}
@@ -195,7 +195,7 @@ func importPreClusteringData(tx *sql.Tx, dump *Dump) error {
 					}
 				}
 				key := row[index].(string)
-				if !shared.StringInSlice(key, NodeSpecificNetworkConfig) {
+				if !shared.ValueInSlice(key, NodeSpecificNetworkConfig) {
 					nullNodeID = true
 					break
 				}
@@ -214,7 +214,7 @@ func importPreClusteringData(tx *sql.Tx, dump *Dump) error {
 					}
 				}
 				key := row[index].(string)
-				if !shared.StringInSlice(key, NodeSpecificStorageConfig) {
+				if !shared.ValueInSlice(key, NodeSpecificStorageConfig) {
 					nullNodeID = true
 					break
 				}
@@ -224,12 +224,12 @@ func importPreClusteringData(tx *sql.Tx, dump *Dump) error {
 				fallthrough
 			case "storage_pools":
 				columns = append(columns, "state")
-				row = append(row, storagePoolCreated)
+				row = append(row, StoragePoolCreated)
 			case "storage_volumes":
 				appendNodeID()
 			}
 
-			if shared.StringInSlice(table, preClusteringTablesRequiringProjectID) {
+			if shared.ValueInSlice(table, preClusteringTablesRequiringProjectID) {
 				// These tables have a project_id reference in the new schema.
 				columns = append(columns, "project_id")
 				row = append(row, 1) // Reference the default project.
@@ -252,7 +252,7 @@ func importPreClusteringData(tx *sql.Tx, dump *Dump) error {
 			}
 
 			// Also insert the image ID to node ID association.
-			if shared.StringInSlice(table, []string{"images", "networks", "storage_pools"}) {
+			if shared.ValueInSlice(table, []string{"images", "networks", "storage_pools"}) {
 				entity := table[:len(table)-1]
 				err := importNodeAssociation(entity, columns, row, tx)
 				if err != nil {
