@@ -7,7 +7,6 @@ import (
 	"fmt"
 
 	"github.com/canonical/lxd/lxd/db/cluster"
-	"github.com/canonical/lxd/lxd/db/query"
 	"github.com/canonical/lxd/shared/api"
 )
 
@@ -19,6 +18,8 @@ SELECT profiles.name
  JOIN projects ON projects.id = profiles.project_id
 WHERE projects.name = ?
 `
+	var result [][]any
+
 	enabled, err := cluster.ProjectHasProfiles(context.Background(), c.tx, project)
 	if err != nil {
 		return nil, fmt.Errorf("Check if project has profiles: %w", err)
@@ -28,24 +29,21 @@ WHERE projects.name = ?
 		project = "default"
 	}
 
-	profileNames := []string{}
-	err = query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
-		var profileName string
+	inargs := []any{project}
+	var name string
+	outfmt := []any{name}
 
-		err := scan(&profileName)
-		if err != nil {
-			return err
-		}
-
-		profileNames = append(profileNames, profileName)
-
-		return nil
-	}, project)
+	result, err = queryScan(ctx, c, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
 
-	return profileNames, nil
+	response := []string{}
+	for _, r := range result {
+		response = append(response, r[0].(string))
+	}
+
+	return response, nil
 }
 
 // GetProfile returns the profile with the given name.
@@ -115,6 +113,7 @@ func (c *ClusterTx) GetInstancesWithProfile(ctx context.Context, project, profil
 		   WHERE profiles.name=? AND projects.name=?)`
 
 	results := map[string][]string{}
+	var output [][]any
 
 	enabled, err := cluster.ProjectHasProfiles(context.Background(), c.tx, project)
 	if err != nil {
@@ -125,27 +124,23 @@ func (c *ClusterTx) GetInstancesWithProfile(ctx context.Context, project, profil
 		project = "default"
 	}
 
-	err = query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
-		var instanceName string
-		var projectName string
+	inargs := []any{profile, project}
+	var name string
+	outfmt := []any{name, name}
 
-		err := scan(&instanceName, &projectName)
-		if err != nil {
-			return err
-		}
-
-		if results[projectName] == nil {
-			results[projectName] = []string{}
-		}
-
-		results[projectName] = append(results[projectName], instanceName)
-
-		return nil
-	}, profile, project)
+	output, err = queryScan(ctx, c, q, inargs, outfmt)
 	if err != nil {
 		return nil, err
 	}
 
+	for _, r := range output {
+		if results[r[1].(string)] == nil {
+			results[r[1].(string)] = []string{}
+		}
+
+		results[r[1].(string)] = append(results[r[1].(string)], r[0].(string))
+	}
+
 	return results, nil
 }
 
