@@ -4,11 +4,10 @@ package db
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"net/http"
 
-	"github.com/canonical/go-dqlite/v2/client"
+	"github.com/canonical/go-dqlite/v3/client"
 
 	"github.com/canonical/lxd/lxd/db/query"
 	"github.com/canonical/lxd/shared/api"
@@ -82,7 +81,7 @@ func (n *NodeTx) GetRaftNodeAddress(ctx context.Context, id int64) (string, erro
 	default:
 		// This should never happen since we have a UNIQUE constraint
 		// on the raft_nodes.id column.
-		return "", errors.New("more than one match found")
+		return "", fmt.Errorf("more than one match found")
 	}
 }
 
@@ -100,7 +99,7 @@ func (n *NodeTx) CreateFirstRaftNode(address string, name string) error {
 	}
 
 	if id != 1 {
-		return errors.New("could not set raft node ID to 1")
+		return fmt.Errorf("could not set raft node ID to 1")
 	}
 
 	return nil
