@@ -6,8 +6,8 @@ import (
 
 	"github.com/stretchr/testify/require"
 
-	"github.com/lxc/lxd/lxd/db/schema"
-	"github.com/lxc/lxd/shared"
+	"github.com/canonical/lxd/lxd/db/schema"
+	"github.com/canonical/lxd/shared"
 )
 
 // A Go source file matching the given prefix is created in the calling
@@ -18,7 +18,7 @@ func TestDotGo(t *testing.T) {
 		2: updateInsertValue,
 	}
 
-	require.NoError(t, schema.DotGo(updates, "xyz"))
+	require.NoError(t, schema.DotGo(updates, "xyz", "xyz.go"))
 	require.Equal(t, true, shared.PathExists("xyz.go"))
 	require.NoError(t, os.Remove("xyz.go"))
 }
