@@ -8,10 +8,10 @@ import (
 	"errors"
 	"fmt"
 	"net/http"
-	"slices"
 	"strings"
 
 	"github.com/canonical/lxd/lxd/db/query"
+	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 )
 
@@ -57,7 +57,7 @@ func (c *ClusterTx) GetStoragePoolID(ctx context.Context, name string) (int64, e
 	case 1:
 		return int64(ids[0]), nil
 	default:
-		return -1, errors.New("More than one pool has the given name")
+		return -1, fmt.Errorf("More than one pool has the given name")
 	}
 }
 
@@ -75,7 +75,7 @@ func (c *ClusterTx) GetStoragePoolDriver(ctx context.Context, id int64) (string,
 	case 1:
 		return drivers[0], nil
 	default:
-		return "", errors.New("More than one pool has the given id")
+		return "", fmt.Errorf("More than one pool has the given id")
 	}
 }
 
@@ -142,7 +142,7 @@ func (c *ClusterTx) UpdateCephStoragePoolAfterNodeJoin(ctx context.Context, pool
 	}
 
 	if len(nodeIDs) == 0 {
-		return errors.New("ceph pool is not linked to any cluster member")
+		return fmt.Errorf("ceph pool is not linked to any cluster member")
 	}
 
 	otherNodeID := nodeIDs[0]
@@ -173,7 +173,7 @@ SELECT id FROM storage_volumes WHERE storage_pool_id=? AND node_id=?
 	}
 
 	if len(volumeIDs) != len(otherVolumeIDs) { // Quick check.
-		return errors.New("not all ceph volumes were copied")
+		return fmt.Errorf("not all ceph volumes were copied")
 	}
 
 	for i, otherVolumeID := range otherVolumeIDs {
@@ -274,7 +274,7 @@ func (c *ClusterTx) CreatePendingStoragePool(ctx context.Context, node string, n
 	err := query.Scan(ctx, c.tx, sql, func(scan func(dest ...any) error) error {
 		// Ensure that there is at most one pool with the given name.
 		if count != 0 {
-			return errors.New("more than one pool exists with the given name")
+			return fmt.Errorf("more than one pool exists with the given name")
 		}
 
 		count++
@@ -299,11 +299,11 @@ func (c *ClusterTx) CreatePendingStoragePool(ctx context.Context, node string, n
 		// Check that the existing pools matches the given driver and
 		// is in the pending state.
 		if pool.driver != driver {
-			return errors.New("Storage pool already exists with a different driver")
+			return fmt.Errorf("Storage pool already exists with a different driver")
 		}
 
 		if pool.state != StoragePoolPending {
-			return errors.New("Storage pool is not in pending state")
+			return fmt.Errorf("Storage pool is not in pending state")
 		}
 	}
 
@@ -446,7 +446,7 @@ func (c *ClusterTx) GetStoragePools(ctx context.Context, state *StoragePoolState
 			verb = "AND"
 		}
 
-		fmt.Fprintf(q, "%s storage_pools.name IN %s", verb, query.Params(len(poolNames)))
+		q.WriteString(fmt.Sprintf("%s storage_pools.name IN %s", verb, query.Params(len(poolNames))))
 		for _, poolName := range poolNames {
 			args = append(args, poolName)
 		}
@@ -526,7 +526,7 @@ WHERE storage_pools.id = ? AND storage_pools.state = ?
 	// Figure which nodes are missing
 	missing := []string{}
 	for _, node := range nodes {
-		if !slices.Contains(defined, node.Name) {
+		if !shared.ValueInSlice(node.Name, defined) {
 			missing = append(missing, node.Name)
 		}
 	}
@@ -586,64 +586,55 @@ func (c *ClusterTx) GetCreatedStoragePoolNames(ctx context.Context) ([]string, e
 
 // Get all storage pools matching the given WHERE filter (if given).
 func (c *ClusterTx) storagePools(ctx context.Context, where string, args ...any) ([]string, error) {
+	var name string
 	stmt := "SELECT name FROM storage_pools"
 	inargs := []any{}
+	outargs := []any{name}
 
 	if where != "" {
-		stmt += " WHERE " + where
+		stmt += fmt.Sprintf(" WHERE %s", where)
 		inargs = append(inargs, args...)
 	}
 
-	pools := []string{}
-	err := query.Scan(ctx, c.tx, stmt, func(scan func(dest ...any) error) error {
-		var poolName string
-
-		err := scan(&poolName)
-		if err != nil {
-			return err
-		}
-
-		pools = append(pools, poolName)
-
-		return nil
-	}, inargs...)
+	result, err := queryScan(ctx, c, stmt, inargs, outargs)
 	if err != nil {
-		return nil, err
+		return []string{}, err
 	}
 
-	if len(pools) == 0 {
+	if len(result) == 0 {
 		return []string{}, api.StatusErrorf(http.StatusNotFound, "Storage pool(s) not found")
 	}
 
+	pools := []string{}
+	for _, r := range result {
+		pools = append(pools, r[0].(string))
+	}
+
 	return pools, nil
 }
 
 // GetStorageDrivers returns the names of all storage drivers currently
 // being used by at least one storage pool.
 func (c *ClusterTx) GetStorageDrivers(ctx context.Context) ([]string, error) {
-	stmt := "SELECT DISTINCT driver FROM storage_pools"
+	var poolDriver string
+	query := "SELECT DISTINCT driver FROM storage_pools"
+	inargs := []any{}
+	outargs := []any{poolDriver}
 
-	drivers := []string{}
-	err := query.Scan(ctx, c.tx, stmt, func(scan func(dest ...any) error) error {
-		var poolDriver string
-
-		err := scan(&poolDriver)
-		if err != nil {
-			return err
-		}
-
-		drivers = append(drivers, poolDriver)
-
-		return nil
-	})
+	result, err := queryScan(ctx, c, query, inargs, outargs)
 	if err != nil {
-		return nil, err
+		return []string{}, err
 	}
 
-	if len(drivers) == 0 {
+	if len(result) == 0 {
 		return []string{}, api.StatusErrorf(http.StatusNotFound, "Storage pool(s) not found")
 	}
 
+	drivers := []string{}
+	for _, driver := range result {
+		drivers = append(drivers, driver[0].(string))
+	}
+
 	return drivers, nil
 }
 
@@ -665,7 +656,7 @@ func (c *ClusterTx) GetStoragePool(ctx context.Context, poolName string) (int64,
 		return poolID, &pool, poolMembers[poolID], err // Only single pool in map.
 	}
 
-	return -1, nil, nil, errors.New("Unexpected pool list size")
+	return -1, nil, nil, fmt.Errorf("Unexpected pool list size")
 }
 
 // GetStoragePoolInAnyState returns the storage pool with the given name.
@@ -685,7 +676,7 @@ func (c *ClusterTx) GetStoragePoolInAnyState(ctx context.Context, poolName strin
 		return poolID, &pool, poolMembers[poolID], err // Only single pool in map.
 	}
 
-	return -1, nil, nil, errors.New("Unexpected pool list size")
+	return -1, nil, nil, fmt.Errorf("Unexpected pool list size")
 }
 
 // GetStoragePoolWithID returns the storage pool with the given ID.
@@ -832,7 +823,7 @@ func storagePoolConfigAdd(tx *sql.Tx, poolID, nodeID int64, poolConfig map[strin
 		}
 
 		var nodeIDValue any
-		if !slices.Contains(NodeSpecificStorageConfig, k) {
+		if !shared.ValueInSlice(k, NodeSpecificStorageConfig) {
 			nodeIDValue = nil
 		} else {
 			nodeIDValue = nodeID
@@ -921,7 +912,7 @@ func (c *ClusterTx) IsRemoteStorage(ctx context.Context, poolID int64) (bool, er
 		return false, err
 	}
 
-	isRemoteStorage := slices.Contains(StorageRemoteDriverNames(), driver)
+	isRemoteStorage := shared.ValueInSlice(driver, StorageRemoteDriverNames())
 
 	return isRemoteStorage, nil
 }
