@@ -5,7 +5,6 @@ package db
 import (
 	"context"
 	"database/sql"
-	"errors"
 	"fmt"
 	"net/http"
 	"strings"
@@ -19,7 +18,7 @@ import (
 
 // CreateStorageVolumeSnapshot creates a new storage volume snapshot attached to a given
 // storage pool.
-func (c *ClusterTx) CreateStorageVolumeSnapshot(ctx context.Context, projectName string, volumeName string, volumeDescription string, volumeType cluster.StoragePoolVolumeType, poolID int64, volumeConfig map[string]string, creationDate time.Time, expiryDate time.Time) (int64, error) {
+func (c *ClusterTx) CreateStorageVolumeSnapshot(ctx context.Context, projectName string, volumeName string, volumeDescription string, volumeType int, poolID int64, volumeConfig map[string]string, creationDate time.Time, expiryDate time.Time) (int64, error) {
 	var volumeID int64
 
 	volumeName, snapshotName, _ := strings.Cut(volumeName, shared.SnapshotDelimiter)
@@ -55,11 +54,11 @@ func (c *ClusterTx) CreateStorageVolumeSnapshot(ctx context.Context, projectName
 }
 
 // UpdateStorageVolumeSnapshot updates the storage volume snapshot attached to a given storage pool.
-func (c *ClusterTx) UpdateStorageVolumeSnapshot(ctx context.Context, projectName string, volumeName string, volumeType cluster.StoragePoolVolumeType, poolID int64, volumeDescription string, volumeConfig map[string]string, expiryDate time.Time) error {
+func (c *ClusterTx) UpdateStorageVolumeSnapshot(ctx context.Context, projectName string, volumeName string, volumeType int, poolID int64, volumeDescription string, volumeConfig map[string]string, expiryDate time.Time) error {
 	var err error
 
 	if !strings.Contains(volumeName, shared.SnapshotDelimiter) {
-		return errors.New("Volume is not a snapshot")
+		return fmt.Errorf("Volume is not a snapshot")
 	}
 
 	volume, err := c.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, volumeName, true)
@@ -93,7 +92,6 @@ func (c *ClusterTx) UpdateStorageVolumeSnapshot(ctx context.Context, projectName
 // GetStorageVolumeSnapshotWithID returns the volume snapshot with the given ID.
 func (c *ClusterTx) GetStorageVolumeSnapshotWithID(ctx context.Context, snapshotID int) (StorageVolumeArgs, error) {
 	args := StorageVolumeArgs{}
-	rawVolumeType := int(-1)
 	q := `
 SELECT
 	volumes.id,
@@ -108,7 +106,7 @@ JOIN storage_pools ON storage_pools.id=volumes.storage_pool_id
 WHERE volumes.id=?
 `
 	arg1 := []any{snapshotID}
-	outfmt := []any{&args.ID, &args.Name, &args.CreationDate, &args.PoolName, &rawVolumeType, &args.ProjectName}
+	outfmt := []any{&args.ID, &args.Name, &args.CreationDate, &args.PoolName, &args.Type, &args.ProjectName}
 
 	err := dbQueryRowScan(ctx, c, q, arg1, outfmt)
 	if err != nil {
@@ -120,15 +118,10 @@ WHERE volumes.id=?
 	}
 
 	if !strings.Contains(args.Name, shared.SnapshotDelimiter) {
-		return args, errors.New("Volume is not a snapshot")
+		return args, fmt.Errorf("Volume is not a snapshot")
 	}
 
-	args.Type, err = cluster.StoragePoolVolumeTypeFromInt(rawVolumeType)
-	if err != nil {
-		return StorageVolumeArgs{}, err
-	}
-
-	args.TypeName = args.Type.String()
+	args.TypeName = cluster.StoragePoolVolumeTypeNames[args.Type]
 
 	return args, nil
 }
