@@ -8,7 +8,6 @@ import (
 	"errors"
 	"fmt"
 	"net/http"
-	"slices"
 	"strings"
 	"time"
 
@@ -22,7 +21,7 @@ import (
 // GetStoragePoolVolumesWithType return a list of all volumes of the given type.
 // If memberSpecific is true, then the search is restricted to volumes that belong to this member or belong to
 // all members.
-func (c *ClusterTx) GetStoragePoolVolumesWithType(ctx context.Context, volumeType cluster.StoragePoolVolumeType, memberSpecific bool) ([]StorageVolumeArgs, error) {
+func (c *ClusterTx) GetStoragePoolVolumesWithType(ctx context.Context, volumeType int, memberSpecific bool) ([]StorageVolumeArgs, error) {
 	var q strings.Builder
 	q.WriteString(`
 SELECT
@@ -75,7 +74,6 @@ WHERE storage_volumes.type = ?
 // GetStoragePoolVolumeWithID returns the volume with the given ID.
 func (c *ClusterTx) GetStoragePoolVolumeWithID(ctx context.Context, volumeID int) (StorageVolumeArgs, error) {
 	var response StorageVolumeArgs
-	var rawVolumeType = int(-1)
 
 	stmt := `
 SELECT
@@ -94,7 +92,7 @@ LEFT JOIN nodes ON nodes.id = storage_volumes.node_id
 WHERE storage_volumes.id = ?
 `
 
-	err := c.tx.QueryRowContext(ctx, stmt, volumeID).Scan(&response.ID, &response.Name, &response.Description, &response.CreationDate, &rawVolumeType, &response.NodeID, &response.PoolName, &response.ProjectName)
+	err := c.tx.QueryRowContext(ctx, stmt, volumeID).Scan(&response.ID, &response.Name, &response.Description, &response.CreationDate, &response.Type, &response.NodeID, &response.PoolName, &response.ProjectName)
 	if err != nil {
 		if err == sql.ErrNoRows {
 			return StorageVolumeArgs{}, api.StatusErrorf(http.StatusNotFound, "Storage pool volume not found")
@@ -108,19 +106,14 @@ WHERE storage_volumes.id = ?
 		return StorageVolumeArgs{}, err
 	}
 
-	response.Type, err = cluster.StoragePoolVolumeTypeFromInt(rawVolumeType)
-	if err != nil {
-		return StorageVolumeArgs{}, err
-	}
-
-	response.TypeName = response.Type.String()
+	response.TypeName = cluster.StoragePoolVolumeTypeNames[response.Type]
 
 	return response, nil
 }
 
 // StorageVolumeFilter used for filtering storage volumes with GetStorageVolumes().
 type StorageVolumeFilter struct {
-	Type    *cluster.StoragePoolVolumeType
+	Type    *int
 	Project *string
 	Name    *string
 	PoolID  *int64
@@ -164,11 +157,11 @@ func (c *ClusterTx) GetStorageVolumes(ctx context.Context, memberSpecific bool,
 		for i, filter := range filters {
 			// Validate filter.
 			if filter.Name != nil && filter.Type == nil {
-				return nil, errors.New("Cannot filter by volume name if volume type not specified")
+				return nil, fmt.Errorf("Cannot filter by volume name if volume type not specified")
 			}
 
 			if filter.Name != nil && filter.Project == nil {
-				return nil, errors.New("Cannot filter by volume name if volume project not specified")
+				return nil, fmt.Errorf("Cannot filter by volume name if volume project not specified")
 			}
 
 			var qFilters []string
@@ -194,7 +187,7 @@ func (c *ClusterTx) GetStorageVolumes(ctx context.Context, memberSpecific bool,
 			}
 
 			if qFilters == nil {
-				return nil, errors.New("Invalid storage volume filter")
+				return nil, fmt.Errorf("Invalid storage volume filter")
 			}
 
 			if i > 0 {
@@ -221,28 +214,25 @@ func (c *ClusterTx) GetStorageVolumes(ctx context.Context, memberSpecific bool,
 	var volumes []*StorageVolume
 
 	err = query.Scan(ctx, c.Tx(), q.String(), func(scan func(dest ...any) error) error {
-		var rawVolumeType = int(-1)
-		var rawContentType = int(-1)
+		var volumeType = int(-1)
+		var contentType = int(-1)
 		var vol StorageVolume
 
-		err := scan(&vol.Project, &vol.ID, &vol.Name, &vol.Location, &rawVolumeType, &rawContentType, &vol.Description, &vol.CreatedAt, &vol.Pool)
+		err := scan(&vol.Project, &vol.ID, &vol.Name, &vol.Location, &volumeType, &contentType, &vol.Description, &vol.CreatedAt, &vol.Pool)
 		if err != nil {
 			return err
 		}
 
-		volumeType, err := cluster.StoragePoolVolumeTypeFromInt(rawVolumeType)
+		vol.Type, err = storagePoolVolumeTypeToName(volumeType)
 		if err != nil {
 			return err
 		}
 
-		contentType, err := cluster.StoragePoolVolumeContentTypeFromInt(rawContentType)
+		vol.ContentType, err = storagePoolVolumeContentTypeToName(contentType)
 		if err != nil {
 			return err
 		}
 
-		vol.Type = volumeType.String()
-		vol.ContentType = contentType.String()
-
 		volumes = append(volumes, &vol)
 
 		return nil
@@ -263,7 +253,7 @@ func (c *ClusterTx) GetStorageVolumes(ctx context.Context, memberSpecific bool,
 }
 
 // GetStoragePoolVolume returns the storage volume attached to a given storage pool.
-func (c *ClusterTx) GetStoragePoolVolume(ctx context.Context, poolID int64, projectName string, volumeType cluster.StoragePoolVolumeType, volumeName string, memberSpecific bool) (*StorageVolume, error) {
+func (c *ClusterTx) GetStoragePoolVolume(ctx context.Context, poolID int64, projectName string, volumeType int, volumeName string, memberSpecific bool) (*StorageVolume, error) {
 	filters := []StorageVolumeFilter{{
 		Project: &projectName,
 		Type:    &volumeType,
@@ -287,7 +277,7 @@ func (c *ClusterTx) GetStoragePoolVolume(ctx context.Context, poolID int64, proj
 // GetLocalStoragePoolVolumeSnapshotsWithType get all snapshots of a storage volume
 // attached to a given storage pool of a given volume type, on the local member.
 // Returns snapshots slice ordered by when they were created, oldest first.
-func (c *ClusterTx) GetLocalStoragePoolVolumeSnapshotsWithType(ctx context.Context, projectName string, volumeName string, volumeType cluster.StoragePoolVolumeType, poolID int64) ([]StorageVolumeArgs, error) {
+func (c *ClusterTx) GetLocalStoragePoolVolumeSnapshotsWithType(ctx context.Context, projectName string, volumeName string, volumeType int, poolID int64) ([]StorageVolumeArgs, error) {
 	remoteDrivers := StorageRemoteDriverNames()
 
 	// ORDER BY creation_date and then id is important here as the users of this function can expect that the
@@ -321,9 +311,9 @@ func (c *ClusterTx) GetLocalStoragePoolVolumeSnapshotsWithType(ctx context.Conte
 		var s StorageVolumeArgs
 		var snapName string
 		var expiryDate sql.NullTime
-		var rawContentType = int(-1)
+		var contentType int
 
-		err := scan(&s.ID, &snapName, &s.Description, &s.CreationDate, &expiryDate, &rawContentType)
+		err := scan(&s.ID, &snapName, &s.Description, &s.CreationDate, &expiryDate, &contentType)
 		if err != nil {
 			return err
 		}
@@ -334,13 +324,11 @@ func (c *ClusterTx) GetLocalStoragePoolVolumeSnapshotsWithType(ctx context.Conte
 		s.Snapshot = true
 		s.ExpiryDate = expiryDate.Time // Convert null to zero.
 
-		contentType, err := cluster.StoragePoolVolumeContentTypeFromInt(rawContentType)
+		s.ContentType, err = storagePoolVolumeContentTypeToName(contentType)
 		if err != nil {
 			return err
 		}
 
-		s.ContentType = contentType.String()
-
 		snapshots = append(snapshots, s)
 
 		return nil
@@ -385,7 +373,7 @@ func storageVolumeSnapshotConfig(ctx context.Context, tx *ClusterTx, volumeSnaps
 }
 
 // UpdateStoragePoolVolume updates the storage volume attached to a given storage pool.
-func (c *ClusterTx) UpdateStoragePoolVolume(ctx context.Context, projectName string, volumeName string, volumeType cluster.StoragePoolVolumeType, poolID int64, volumeDescription string, volumeConfig map[string]string) error {
+func (c *ClusterTx) UpdateStoragePoolVolume(ctx context.Context, projectName string, volumeName string, volumeType int, poolID int64, volumeDescription string, volumeConfig map[string]string) error {
 	isSnapshot := strings.Contains(volumeName, shared.SnapshotDelimiter)
 
 	volume, err := c.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, volumeName, true)
@@ -413,7 +401,7 @@ func (c *ClusterTx) UpdateStoragePoolVolume(ctx context.Context, projectName str
 
 // RemoveStoragePoolVolume deletes the storage volume attached to a given storage
 // pool.
-func (c *ClusterTx) RemoveStoragePoolVolume(ctx context.Context, projectName string, volumeName string, volumeType cluster.StoragePoolVolumeType, poolID int64) error {
+func (c *ClusterTx) RemoveStoragePoolVolume(ctx context.Context, projectName string, volumeName string, volumeType int, poolID int64) error {
 	isSnapshot := strings.Contains(volumeName, shared.SnapshotDelimiter)
 	var stmt string
 	if isSnapshot {
@@ -436,7 +424,7 @@ func (c *ClusterTx) RemoveStoragePoolVolume(ctx context.Context, projectName str
 }
 
 // RenameStoragePoolVolume renames the storage volume attached to a given storage pool.
-func (c *ClusterTx) RenameStoragePoolVolume(ctx context.Context, projectName string, oldVolumeName string, newVolumeName string, volumeType cluster.StoragePoolVolumeType, poolID int64) error {
+func (c *ClusterTx) RenameStoragePoolVolume(ctx context.Context, projectName string, oldVolumeName string, newVolumeName string, volumeType int, poolID int64) error {
 	isSnapshot := strings.Contains(oldVolumeName, shared.SnapshotDelimiter)
 	var stmt string
 	if isSnapshot {
@@ -460,11 +448,11 @@ func (c *ClusterTx) RenameStoragePoolVolume(ctx context.Context, projectName str
 }
 
 // CreateStoragePoolVolume creates a new storage volume attached to a given storage pool.
-func (c *ClusterTx) CreateStoragePoolVolume(ctx context.Context, projectName string, volumeName string, volumeDescription string, volumeType cluster.StoragePoolVolumeType, poolID int64, volumeConfig map[string]string, contentType cluster.StoragePoolVolumeContentType, creationDate time.Time) (int64, error) {
+func (c *ClusterTx) CreateStoragePoolVolume(ctx context.Context, projectName string, volumeName string, volumeDescription string, volumeType int, poolID int64, volumeConfig map[string]string, contentType int, creationDate time.Time) (int64, error) {
 	var volumeID int64
 
 	if shared.IsSnapshot(volumeName) {
-		return -1, errors.New("Volume name may not be a snapshot")
+		return -1, fmt.Errorf("Volume name may not be a snapshot")
 	}
 
 	remoteDrivers := StorageRemoteDriverNames()
@@ -476,7 +464,7 @@ func (c *ClusterTx) CreateStoragePoolVolume(ctx context.Context, projectName str
 
 	var result sql.Result
 
-	if slices.Contains(remoteDrivers, driver) {
+	if shared.ValueInSlice(driver, remoteDrivers) {
 		result, err = c.tx.ExecContext(ctx, `
 INSERT INTO storage_volumes (storage_pool_id, type, name, description, project_id, content_type, creation_date)
  VALUES (?, ?, ?, ?, (SELECT id FROM projects WHERE name = ?), ?, ?)
@@ -509,7 +497,7 @@ INSERT INTO storage_volumes (storage_pool_id, node_id, type, name, description,
 
 // Return the ID of a storage volume on a given storage pool of a given storage
 // volume type, on the given node.
-func (c *ClusterTx) storagePoolVolumeGetTypeID(ctx context.Context, project string, volumeName string, volumeType cluster.StoragePoolVolumeType, poolID, nodeID int64) (int64, error) {
+func (c *ClusterTx) storagePoolVolumeGetTypeID(ctx context.Context, project string, volumeName string, volumeType int, poolID, nodeID int64) (int64, error) {
 	remoteDrivers := StorageRemoteDriverNames()
 
 	s := `
@@ -543,7 +531,7 @@ SELECT storage_volumes_all.id
 
 // GetStoragePoolNodeVolumeID gets the ID of a storage volume on a given storage pool
 // of a given storage volume type and project, on the current node.
-func (c *ClusterTx) GetStoragePoolNodeVolumeID(ctx context.Context, projectName string, volumeName string, volumeType cluster.StoragePoolVolumeType, poolID int64) (int64, error) {
+func (c *ClusterTx) GetStoragePoolNodeVolumeID(ctx context.Context, projectName string, volumeName string, volumeType int, poolID int64) (int64, error) {
 	return c.storagePoolVolumeGetTypeID(ctx, projectName, volumeName, volumeType, poolID, c.nodeID)
 }
 
@@ -554,7 +542,7 @@ type StorageVolumeArgs struct {
 	Name string
 
 	// At least one of Type or TypeName must be set.
-	Type     cluster.StoragePoolVolumeType
+	Type     int
 	TypeName string
 
 	// At least one of PoolID or PoolName must be set.
@@ -581,7 +569,7 @@ type StorageVolumeArgs struct {
 // The volume name can be either a regular name or a volume snapshot name.
 // If the volume is defined, but without a specific node, then the ErrNoClusterMember error is returned.
 // If the volume is not found then an api.StatusError with code set to http.StatusNotFound is returned.
-func (c *ClusterTx) GetStorageVolumeNodes(ctx context.Context, poolID int64, projectName string, volumeName string, volumeType cluster.StoragePoolVolumeType) ([]NodeInfo, error) {
+func (c *ClusterTx) GetStorageVolumeNodes(ctx context.Context, poolID int64, projectName string, volumeName string, volumeType int) ([]NodeInfo, error) {
 	nodes := []NodeInfo{}
 
 	sql := `
@@ -632,11 +620,11 @@ func (c *ClusterTx) GetStorageVolumeNodes(ctx context.Context, poolID int64, pro
 		// equivalent to db.ErrNoClusterMember that is used in newer schemas where a single remote volume
 		// DB record is created that is not associated to any single member.
 		if StorageRemoteDriverNames == nil {
-			return nil, errors.New("No remote storage drivers function defined")
+			return nil, fmt.Errorf("No remote storage drivers function defined")
 		}
 
 		remoteDrivers := StorageRemoteDriverNames()
-		if slices.Contains(remoteDrivers, driver) {
+		if shared.ValueInSlice(driver, remoteDrivers) {
 			return nil, ErrNoClusterMember
 		}
 	}
@@ -679,7 +667,7 @@ func (c *ClusterTx) storageVolumeConfigGet(ctx context.Context, volumeID int64,
 //
 // Note, the code below doesn't deal with snapshots of snapshots.
 // To do that, we'll need to weed out based on # slashes in names.
-func (c *ClusterTx) GetNextStorageVolumeSnapshotIndex(ctx context.Context, pool, name string, typ cluster.StoragePoolVolumeType, pattern string) (nextIndex int) {
+func (c *ClusterTx) GetNextStorageVolumeSnapshotIndex(ctx context.Context, pool, name string, typ int, pattern string) (nextIndex int) {
 	remoteDrivers := StorageRemoteDriverNames()
 
 	q := `
@@ -691,7 +679,10 @@ SELECT storage_volumes_snapshots.name FROM storage_volumes_snapshots
    AND storage_pools.name=?
    AND (storage_volumes.node_id=? OR storage_volumes.node_id IS NULL AND storage_pools.driver IN ` + query.Params(len(remoteDrivers)) + `)`
 
+	var numstr string
 	inargs := []any{typ, name, pool, c.nodeID}
+	outfmt := []any{numstr}
+
 	for _, driver := range remoteDrivers {
 		inargs = append(inargs, driver)
 	}
@@ -700,27 +691,26 @@ SELECT storage_volumes_snapshots.name FROM storage_volumes_snapshots
 		return 0
 	}
 
-	err := query.Scan(ctx, c.tx, q, func(scan func(dest ...any) error) error {
-		var substr string
-		err := scan(&substr)
-		if err != nil {
-			return err
+	results, err := queryScan(ctx, c, q, inargs, outfmt)
+	if err != nil {
+		return 0
+	}
+
+	for _, r := range results {
+		substr, ok := r[0].(string)
+		if !ok {
+			continue
 		}
 
 		var num int
 		count, err := fmt.Sscanf(substr, pattern, &num)
 		if err != nil || count != 1 {
-			return nil
+			continue
 		}
 
 		if num >= nextIndex {
 			nextIndex = num + 1
 		}
-
-		return nil
-	}, inargs...)
-	if err != nil {
-		return 0
 	}
 
 	return nextIndex
@@ -787,6 +777,36 @@ func storageVolumeConfigClear(tx *sql.Tx, volumeID int64, isSnapshot bool) error
 	return nil
 }
 
+// Convert a volume integer type code to its human-readable name.
+func storagePoolVolumeTypeToName(volumeType int) (string, error) {
+	switch volumeType {
+	case cluster.StoragePoolVolumeTypeContainer:
+		return cluster.StoragePoolVolumeTypeNameContainer, nil
+	case cluster.StoragePoolVolumeTypeVM:
+		return cluster.StoragePoolVolumeTypeNameVM, nil
+	case cluster.StoragePoolVolumeTypeImage:
+		return cluster.StoragePoolVolumeTypeNameImage, nil
+	case cluster.StoragePoolVolumeTypeCustom:
+		return cluster.StoragePoolVolumeTypeNameCustom, nil
+	}
+
+	return "", fmt.Errorf("Invalid storage volume type")
+}
+
+// Convert a volume integer content type code to its human-readable name.
+func storagePoolVolumeContentTypeToName(contentType int) (string, error) {
+	switch contentType {
+	case cluster.StoragePoolVolumeContentTypeFS:
+		return cluster.StoragePoolVolumeContentTypeNameFS, nil
+	case cluster.StoragePoolVolumeContentTypeBlock:
+		return cluster.StoragePoolVolumeContentTypeNameBlock, nil
+	case cluster.StoragePoolVolumeContentTypeISO:
+		return cluster.StoragePoolVolumeContentTypeNameISO, nil
+	}
+
+	return "", fmt.Errorf("Invalid storage volume content type")
+}
+
 // GetCustomVolumesInProject returns all custom volumes in the given project.
 func (c *ClusterTx) GetCustomVolumesInProject(ctx context.Context, project string) ([]StorageVolumeArgs, error) {
 	sql := `
@@ -865,7 +885,7 @@ func (c *ClusterTx) GetStorageVolumeURIs(ctx context.Context, project string) ([
 
 // UpdateStorageVolumeNode changes the name of a storage volume and the cluster member hosting it.
 // It's meant to be used when moving a storage volume backed by ceph from one cluster node to another.
-func (c *ClusterTx) UpdateStorageVolumeNode(ctx context.Context, projectName string, oldName string, newName string, newMemberName string, poolID int64, volumeType cluster.StoragePoolVolumeType) error {
+func (c *ClusterTx) UpdateStorageVolumeNode(ctx context.Context, projectName string, oldName string, newName string, newMemberName string, poolID int64, volumeType int) error {
 	volume, err := c.GetStoragePoolVolume(ctx, poolID, projectName, volumeType, oldName, false)
 	if err != nil {
 		return err
