@@ -2,11 +2,9 @@ package config
 
 import (
 	"fmt"
-	"maps"
 	"sort"
 	"strings"
 
-	"github.com/canonical/lxd/lxd/device/filters"
 	"github.com/canonical/lxd/shared/api"
 )
 
@@ -17,7 +15,9 @@ type Device map[string]string
 func (device Device) Clone() Device {
 	cpy := make(map[string]string, len(device))
 
-	maps.Copy(cpy, device)
+	for k, v := range device {
+		cpy[k] = v
+	}
 
 	return cpy
 }
@@ -87,7 +87,9 @@ func NewDevices(nativeSet map[string]map[string]string) Devices {
 
 	for devName, devConfig := range nativeSet {
 		newDev := Device{}
-		maps.Copy(newDev, devConfig)
+		for k, v := range devConfig {
+			newDev[k] = v
+		}
 
 		newDevices[devName] = newDev
 	}
@@ -100,7 +102,7 @@ func ApplyDeviceInitialValues(devices Devices, profiles []api.Profile) Devices {
 	for _, p := range profiles {
 		for devName, devConfig := range p.Devices {
 			// Apply only root disk device from profile devices to instance devices.
-			if !filters.IsRootDisk(devConfig) {
+			if devConfig["type"] != "disk" || devConfig["path"] != "/" || devConfig["source"] != "" {
 				continue
 			}
 
@@ -243,28 +245,3 @@ func (list Devices) Reversed() DevicesSortable {
 	sort.Sort(sort.Reverse(sortable))
 	return sortable
 }
-
-// Filter returns the devices matching the provided filters.
-// The list of filters is applied using the AND operator.
-// Combining filters using the OR operator can be done using the filters.Or function.
-func (list Devices) Filter(filters ...func(map[string]string) bool) Devices {
-	filteredDevices := Devices{}
-
-	for deviceName, device := range list {
-		allFiltersPassed := true
-
-		for _, filter := range filters {
-			if !filter(device) {
-				// The first filter returned false which means the remaining ones can be skipped.
-				allFiltersPassed = false
-				break
-			}
-		}
-
-		if allFiltersPassed {
-			filteredDevices[deviceName] = device
-		}
-	}
-
-	return filteredDevices
-}
