@@ -8,8 +8,6 @@ import (
 	"os"
 	"os/exec"
 	"path/filepath"
-	"slices"
-	"strconv"
 	"strings"
 	"time"
 
@@ -26,6 +24,32 @@ import (
 	"github.com/canonical/lxd/shared/version"
 )
 
+// DevSourcePath is a path on the LXD host.
+// See `deviceConfig.DevSource`.
+type DevSourcePath struct {
+	Path string
+}
+
+// DevSourceFD is a file descriptor held by the LXD process.
+// See `deviceConfig.DevSource`.
+type DevSourceFD struct {
+	FD   uintptr
+	Path string
+}
+
+// DevSourceRBD describes an RBD image.
+// See `deviceConfig.DevSource`.
+//
+// This structure roughly corresponds to a qmp BlockdevOptionsRbd:
+// https://www.qemu.org/docs/master/interop/qemu-storage-daemon-qmp-ref.html#qapidoc-708
+type DevSourceRBD struct {
+	ClusterName string
+	UserName    string
+	PoolName    string
+	ImageName   string
+	Snapshot    string
+}
+
 // BlockFsDetect detects the type of block device.
 func BlockFsDetect(dev string) (string, error) {
 	out, err := shared.RunCommandContext(context.TODO(), "blkid", "-s", "TYPE", "-o", "value", dev)
@@ -43,7 +67,7 @@ func DiskMount(srcPath string, dstPath string, recursive bool, propagation strin
 	flags, mountOptionsStr := filesystem.ResolveMountOptions(mountOptions)
 
 	var readonly bool
-	if slices.Contains(mountOptions, "ro") {
+	if shared.ValueInSlice("ro", mountOptions) {
 		readonly = true
 	}
 
@@ -137,7 +161,7 @@ func diskCephRbdMap(clusterName string, userName string, poolName string, volume
 
 	idx := strings.Index(devPath, "/dev/rbd")
 	if idx < 0 {
-		return "", errors.New("Failed to detect mapped device path")
+		return "", fmt.Errorf("Failed to detect mapped device path")
 	}
 
 	devPath = devPath[idx:]
@@ -264,7 +288,7 @@ func DiskVMVirtfsProxyStart(execPath string, pidPath string, sharePath string, i
 	}
 
 	if cmd == "" {
-		return nil, nil, errors.New(`Required binary "virtfs-proxy-helper" couldn't be found`)
+		return nil, nil, fmt.Errorf(`Required binary "virtfs-proxy-helper" couldn't be found`)
 	}
 
 	listener, err := net.Listen("unix", "")
@@ -283,7 +307,7 @@ func DiskVMVirtfsProxyStart(execPath string, pidPath string, sharePath string, i
 
 	cDialUnix, ok := cDial.(*net.UnixConn)
 	if !ok {
-		return nil, nil, errors.New("Dialled virtfs-proxy-helper connection isn't unix socket")
+		return nil, nil, fmt.Errorf("Dialled virtfs-proxy-helper connection isn't unix socket")
 	}
 
 	defer func() { _ = cDialUnix.Close() }()
@@ -304,7 +328,7 @@ func DiskVMVirtfsProxyStart(execPath string, pidPath string, sharePath string, i
 
 	cAcceptUnix, ok := cAccept.(*net.UnixConn)
 	if !ok {
-		return nil, nil, errors.New("Accepted virtfs-proxy-helper connection isn't unix socket")
+		return nil, nil, fmt.Errorf("Accepted virtfs-proxy-helper connection isn't unix socket")
 	}
 
 	defer func() { _ = cAcceptUnix.Close() }()
@@ -370,7 +394,7 @@ func DiskVMVirtfsProxyStop(pidPath string) error {
 // Returns UnsupportedError error if the host system or instance does not support virtiosfd, returns normal error
 // type if process cannot be started for other reasons.
 // Returns revert function and listener file handle on success.
-func DiskVMVirtiofsdStart(kernelVersion version.DottedVersion, inst instance.Instance, socketPath string, pidPath string, logPath string, sharePath string, idmaps []idmap.IdmapEntry, threadPoolSize uint16) (func(), net.Listener, error) {
+func DiskVMVirtiofsdStart(kernelVersion version.DottedVersion, inst instance.Instance, socketPath string, pidPath string, logPath string, sharePath string, idmaps []idmap.IdmapEntry) (func(), net.Listener, error) {
 	revert := revert.New()
 	defer revert.Fail()
 
@@ -433,7 +457,7 @@ func DiskVMVirtiofsdStart(kernelVersion version.DottedVersion, inst instance.Ins
 
 	unixListener, ok := listener.(*net.UnixListener)
 	if !ok {
-		return nil, nil, errors.New("Failed getting UnixListener for virtiofsd")
+		return nil, nil, fmt.Errorf("Failed getting UnixListener for virtiofsd")
 	}
 
 	revert.Add(func() {
@@ -450,11 +474,9 @@ func DiskVMVirtiofsdStart(kernelVersion version.DottedVersion, inst instance.Ins
 	// Start the virtiofsd process in non-daemon mode.
 	args := []string{
 		"--fd=3",
-		"--shared-dir", sharePath,
-		"--cache", "auto", // "never" and "metadata" modes do not allow execution at this time.
-		"--allow-direct-io",
-		"--thread-pool-size", strconv.FormatUint(uint64(threadPoolSize), 10),
-		"--xattr",
+		// use -o flags for support in wider versions of virtiofsd.
+		"-o", "xattr",
+		"-o", "source=" + sharePath,
 	}
 
 	// Virtiofsd defaults to namespace sandbox mode which requires pidfd_open support.
