@@ -7,7 +7,6 @@ import (
 	"net"
 	"net/netip"
 	"os"
-	"slices"
 	"strconv"
 	"strings"
 	"sync"
@@ -157,7 +156,7 @@ func networkSnapshotPhysicalNIC(hostName string, volatile map[string]string) err
 		return err
 	}
 
-	volatile["last_state.mtu"] = strconv.FormatUint(uint64(mtu), 10)
+	volatile["last_state.mtu"] = fmt.Sprint(mtu)
 
 	// Store current MAC for restoration on detach
 	mac, err := NetworkGetDevMAC(hostName)
@@ -449,7 +448,7 @@ func networkNICRouteAdd(routeDev string, routes ...string) error {
 // Logs any errors and continues to next route to remove.
 func networkNICRouteDelete(routeDev string, routes ...string) {
 	if routeDev == "" {
-		logger.Error("Failed removing static route, empty route device specified")
+		logger.Errorf("Failed removing static route, empty route device specified")
 		return
 	}
 
@@ -579,7 +578,7 @@ func networkSetupHostVethLimits(d *deviceCommon, oldConfig deviceConfig.Device,
 	if oldConfig == nil || oldConfig["limits.priority"] != d.config["limits.priority"] {
 		if networkPriority != 0 {
 			if bridged && d.state.Firewall.String() == "xtables" {
-				return errors.New("Failed to setup instance device network priority. The xtables firewall driver does not support required functionality.")
+				return fmt.Errorf("Failed to setup instance device network priority. The xtables firewall driver does not support required functionality.")
 			}
 
 			err = d.state.Firewall.InstanceSetupNetPrio(d.inst.Project().Name, d.inst.Name(), veth, uint32(networkPriority))
@@ -604,7 +603,7 @@ func networkClearHostVethLimits(d *deviceCommon) error {
 
 // networkValidGateway validates the gateway value.
 func networkValidGateway(value string) error {
-	if slices.Contains([]string{"none", "auto"}, value) {
+	if shared.ValueInSlice(value, []string{"none", "auto"}) {
 		return nil
 	}
 
@@ -710,9 +709,9 @@ func networkSRIOVSetupVF(d deviceCommon, vfParent string, vfDevice string, vfID
 	// Record properties of VF settings on the parent device.
 	volatile["last_state.vf.parent"] = vfParent
 	volatile["last_state.vf.hwaddr"] = vfInfo.Address
-	volatile["last_state.vf.id"] = strconv.Itoa(vfID)
-	volatile["last_state.vf.vlan"] = strconv.Itoa(vfInfo.VLANs[0]["vlan"])
-	volatile["last_state.vf.spoofcheck"] = strconv.FormatBool(vfInfo.SpoofCheck)
+	volatile["last_state.vf.id"] = fmt.Sprint(vfID)
+	volatile["last_state.vf.vlan"] = fmt.Sprint(vfInfo.VLANs[0]["vlan"])
+	volatile["last_state.vf.spoofcheck"] = fmt.Sprint(vfInfo.SpoofCheck)
 
 	// Record the host interface we represents the VF device which we will move into instance.
 	volatile["host_name"] = vfDevice
@@ -752,7 +751,7 @@ func networkSRIOVSetupVF(d deviceCommon, vfParent string, vfDevice string, vfID
 	// order of setup to allow LXD to set custom MACs when using spoof check mode.
 	if shared.IsTrue(d.config["security.mac_filtering"]) {
 		if !useSpoofCheck {
-			return pcidev.Device{}, 0, errors.New("security.mac_filtering cannot be enabled when VF spoof check not enabled")
+			return pcidev.Device{}, 0, fmt.Errorf("security.mac_filtering cannot be enabled when VF spoof check not enabled")
 		}
 
 		// If no MAC specified in config, use current VF interface MAC.
@@ -1095,7 +1094,7 @@ func isIPAvailable(ctx context.Context, address net.IP, parentInterface string)
 
 	netipAddr, ok := netip.AddrFromSlice(address)
 	if !ok {
-		return false, errors.New("Couldn't convert address to netip")
+		return false, fmt.Errorf("Couldn't convert address to netip")
 	}
 
 	solicitedNodeMulticast, err := ndp.SolicitedNodeMulticast(netipAddr)
