@@ -1,11 +1,9 @@
 package device
 
 import (
-	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
-	"slices"
 	"strconv"
 	"strings"
 
@@ -32,13 +30,12 @@ func unixDeviceAttributes(path string) (dType string, major uint32, minor uint32
 	}
 
 	// Check what kind of file it is
-	switch stat.Mode & unix.S_IFMT {
-	case unix.S_IFBLK:
+	if stat.Mode&unix.S_IFMT == unix.S_IFBLK {
 		dType = "b"
-	case unix.S_IFCHR:
+	} else if stat.Mode&unix.S_IFMT == unix.S_IFCHR {
 		dType = "c"
-	default:
-		return "", 0, 0, errors.New("Not a device")
+	} else {
+		return "", 0, 0, fmt.Errorf("Not a device")
 	}
 
 	// Return the device information
@@ -121,10 +118,9 @@ func UnixDeviceCreate(s *state.State, idmapSet *idmap.IdmapSet, devicesPath stri
 	d := UnixDevice{}
 
 	// Extra checks for nesting.
-	deviceProperties := []string{"major", "minor", "mode", "uid", "gid"}
 	if s.OS.RunningInUserNS {
 		for key, value := range m {
-			if value != "" && slices.Contains(deviceProperties, key) {
+			if shared.ValueInSlice(key, []string{"major", "minor", "mode", "uid", "gid"}) && value != "" {
 				return nil, fmt.Errorf("The %q property may not be set when adding a device to a nested container", key)
 			}
 		}
@@ -239,7 +235,7 @@ func UnixDeviceCreate(s *state.State, idmapSet *idmap.IdmapSet, devicesPath stri
 	// Create the new entry.
 	if !s.OS.RunningInUserNS {
 		if s.OS.Nodev {
-			return nil, errors.New("Can't create device as devices path is mounted nodev")
+			return nil, fmt.Errorf("Can't create device as devices path is mounted nodev")
 		}
 
 		devNum := int(unix.Mkdev(d.Major, d.Minor))
@@ -342,7 +338,7 @@ func unixDeviceSetup(s *state.State, devicesPath string, typePrefix string, devi
 
 	// Instruct LXD to perform the mount.
 	runConf.Mounts = append(runConf.Mounts, deviceConfig.MountEntryItem{
-		DevSource:  deviceConfig.DevSourcePath{Path: d.HostPath},
+		DevSource:  DevSourcePath{Path: d.HostPath},
 		TargetPath: d.RelativePath,
 		FSType:     "none",
 		Opts:       []string{"bind", "create=file"},
@@ -369,8 +365,8 @@ func unixDeviceSetupCharNum(s *state.State, devicesPath string, typePrefix strin
 	// Overriding these in the config copy should avoid the need for unixDeviceSetup to stat
 	// the origin device to ascertain this information.
 	configCopy["type"] = "unix-char"
-	configCopy["major"] = strconv.FormatUint(uint64(major), 10)
-	configCopy["minor"] = strconv.FormatUint(uint64(minor), 10)
+	configCopy["major"] = fmt.Sprintf("%d", major)
+	configCopy["minor"] = fmt.Sprintf("%d", minor)
 	configCopy["path"] = path
 
 	return unixDeviceSetup(s, devicesPath, typePrefix, deviceName, configCopy, defaultMode, runConf)
@@ -387,8 +383,8 @@ func unixDeviceSetupBlockNum(s *state.State, devicesPath string, typePrefix stri
 	// Overriding these in the config copy should avoid the need for unixDeviceSetup to stat
 	// the origin device to ascertain this information.
 	configCopy["type"] = "unix-block"
-	configCopy["major"] = strconv.FormatUint(uint64(major), 10)
-	configCopy["minor"] = strconv.FormatUint(uint64(minor), 10)
+	configCopy["major"] = fmt.Sprintf("%d", major)
+	configCopy["minor"] = fmt.Sprintf("%d", minor)
 	configCopy["path"] = path
 
 	return unixDeviceSetup(s, devicesPath, typePrefix, deviceName, configCopy, defaultMode, runConf)
@@ -470,7 +466,13 @@ func unixDeviceRemove(devicesPath string, typePrefix string, deviceName string,
 		ourEncRelDestFile := ourDev[idx+1:]
 
 		// Look for devices for other LXD devices that match the same path.
-		dupe := slices.Contains(encRelDevFiles, ourEncRelDestFile)
+		dupe := false
+		for _, encRelDevFile := range encRelDevFiles {
+			if encRelDevFile == ourEncRelDestFile {
+				dupe = true
+				break
+			}
+		}
 
 		// If a device has been found that points to the same device inside the instance
 		// then we cannot request it be umounted inside the instance as it's still in use.
@@ -551,7 +553,7 @@ func unixValidDeviceNum(value string) error {
 
 	_, err := strconv.ParseUint(value, 10, 32)
 	if err != nil {
-		return errors.New("Invalid value for a UNIX device number")
+		return fmt.Errorf("Invalid value for a UNIX device number")
 	}
 
 	return nil
@@ -565,7 +567,7 @@ func unixValidUserID(value string) error {
 
 	_, err := strconv.ParseUint(value, 10, 32)
 	if err != nil {
-		return errors.New("Invalid value for a UNIX ID")
+		return fmt.Errorf("Invalid value for a UNIX ID")
 	}
 
 	return nil
@@ -579,7 +581,7 @@ func unixValidOctalFileMode(value string) error {
 
 	_, err := strconv.ParseUint(value, 8, 32)
 	if err != nil {
-		return errors.New("Invalid value for an octal file mode")
+		return fmt.Errorf("Invalid value for an octal file mode")
 	}
 
 	return nil
