@@ -2,17 +2,20 @@ package device
 
 import (
 	"fmt"
+	"net"
 	"net/http"
+	"os"
+	"strconv"
 
-	deviceConfig "github.com/lxc/lxd/lxd/device/config"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/instance/instancetype"
-	"github.com/lxc/lxd/lxd/ip"
-	"github.com/lxc/lxd/lxd/network"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
+	deviceConfig "github.com/canonical/lxd/lxd/device/config"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/instance/instancetype"
+	"github.com/canonical/lxd/lxd/ip"
+	"github.com/canonical/lxd/lxd/network"
+	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/revert"
 )
 
 type nicMACVLAN struct {
@@ -63,9 +66,9 @@ func (d *nicMACVLAN) validateConfig(instConf instance.ConfigReader) error {
 		}
 
 		// If network property is specified, lookup network settings and apply them to the device's config.
-		// project.Default is used here as macvlan networks don't suppprt projects.
+		// api.ProjectDefaultName is used here as macvlan networks don't support projects.
 		var err error
-		d.network, err = network.LoadByName(d.state, project.Default, d.config["network"])
+		d.network, err = network.LoadByName(d.state, api.ProjectDefaultName, d.config["network"])
 		if err != nil {
 			return fmt.Errorf("Error loading network config for %q: %w", d.config["network"], err)
 		}
@@ -152,7 +155,10 @@ func (d *nicMACVLAN) Start() (*deviceConfig.RunConfig, error) {
 	actualParentName := network.GetHostDevice(d.config["parent"], d.config["vlan"])
 
 	// Record the temporary device name used for deletion later.
-	saveData["host_name"] = network.RandomDevName("mac")
+	saveData["host_name"], err = d.generateHostName("mac", d.config["hwaddr"])
+	if err != nil {
+		return nil, err
+	}
 
 	// Create VLAN parent device if needed.
 	statusDev, err := networkCreateVlanDeviceIfNeeded(d.state, d.config["parent"], actualParentName, d.config["vlan"], shared.IsTrue(d.config["gvrp"]))
@@ -169,33 +175,54 @@ func (d *nicMACVLAN) Start() (*deviceConfig.RunConfig, error) {
 		})
 	}
 
-	if d.inst.Type() == instancetype.Container {
-		// Create MACVLAN interface.
-		macvlan := &ip.Macvlan{
-			Link: ip.Link{
-				Name:   saveData["host_name"],
-				Parent: actualParentName,
-			},
-			Mode: "bridge",
+	// Create MACVLAN interface.
+	link := &ip.Macvlan{
+		Link: ip.Link{
+			Name:   saveData["host_name"],
+			Parent: actualParentName,
+		},
+		Mode: "bridge",
+	}
+
+	// Set the MAC address.
+	if d.config["hwaddr"] != "" {
+		hwaddr, err := net.ParseMAC(d.config["hwaddr"])
+		if err != nil {
+			return nil, fmt.Errorf("Failed parsing MAC address %q: %w", d.config["hwaddr"], err)
 		}
 
-		err = macvlan.Add()
+		link.Address = hwaddr
+	}
+
+	// Set the MTU.
+	if d.config["mtu"] != "" {
+		mtu, err := strconv.ParseUint(d.config["mtu"], 10, 32)
+		if err != nil {
+			return nil, fmt.Errorf("Invalid MTU specified %q: %w", d.config["mtu"], err)
+		}
+
+		link.MTU = uint32(mtu)
+	}
+
+	if d.inst.Type() == instancetype.VM {
+		// Enable all multicast processing which is required for IPv6 NDP functionality.
+		link.AllMutlicast = true
+
+		// Bring the interface up on host side.
+		link.Up = true
+
+		// Create macvtap interface using common macvlan settings.
+		link := &ip.Macvtap{
+			Macvlan: *link,
+		}
+
+		err = link.Add()
 		if err != nil {
 			return nil, err
 		}
-	} else if d.inst.Type() == instancetype.VM {
-		// Create MACVTAP interface.
-		macvtap := &ip.Macvtap{
-			Macvlan: ip.Macvlan{
-				Link: ip.Link{
-					Name:   saveData["host_name"],
-					Parent: actualParentName,
-				},
-				Mode: "bridge",
-			},
-		}
-
-		err = macvtap.Add()
+	} else {
+		// Create macvlan interface.
+		err = link.Add()
 		if err != nil {
 			return nil, err
 		}
@@ -203,30 +230,11 @@ func (d *nicMACVLAN) Start() (*deviceConfig.RunConfig, error) {
 
 	revert.Add(func() { _ = network.InterfaceRemove(saveData["host_name"]) })
 
-	// Set the MAC address.
-	if d.config["hwaddr"] != "" {
-		link := &ip.Link{Name: saveData["host_name"]}
-		err := link.SetAddress(d.config["hwaddr"])
-		if err != nil {
-			return nil, fmt.Errorf("Failed to set the MAC address: %s", err)
-		}
-	}
-
-	// Set the MTU.
-	if d.config["mtu"] != "" {
-		link := &ip.Link{Name: saveData["host_name"]}
-		err := link.SetMTU(d.config["mtu"])
-		if err != nil {
-			return nil, fmt.Errorf("Failed setting MTU %q on %q: %w", d.config["mtu"], saveData["host_name"], err)
-		}
-	}
-
 	if d.inst.Type() == instancetype.VM {
-		// Bring the interface up on host side.
-		link := &ip.Link{Name: saveData["host_name"]}
-		err := link.SetUp()
-		if err != nil {
-			return nil, fmt.Errorf("Failed to bring up interface %s: %w", saveData["host_name"], err)
+		// Disable IPv6 on host interface to avoid getting IPv6 link-local addresses unnecessarily.
+		err = util.SysctlSet(fmt.Sprintf("net/ipv6/conf/%s/disable_ipv6", link.Name), "1")
+		if err != nil && !os.IsNotExist(err) {
+			return nil, fmt.Errorf("Failed to disable IPv6 on host interface %q: %w", link.Name, err)
 		}
 	}
 
