@@ -2,11 +2,9 @@ package device
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"net"
 	"os"
-	"strconv"
 	"strings"
 	"time"
 
@@ -112,7 +110,7 @@ func (d *nicRouted) validateConfig(instConf instance.ConfigReader) error {
 		ips := make(map[string]struct{})
 
 		if d.config[key] != "" {
-			for addr := range strings.SplitSeq(d.config[key], ",") {
+			for _, addr := range strings.Split(d.config[key], ",") {
 				addr = strings.TrimSpace(addr)
 				_, dupe := ips[addr]
 				if dupe {
@@ -126,14 +124,14 @@ func (d *nicRouted) validateConfig(instConf instance.ConfigReader) error {
 
 	// Ensure that address is set if routes is set.
 	for _, keyPrefix := range []string{"ipv4", "ipv6"} {
-		if d.config[keyPrefix+".routes"] != "" && d.config[keyPrefix+".address"] == "" {
+		if d.config[fmt.Sprintf("%s.routes", keyPrefix)] != "" && d.config[fmt.Sprintf("%s.address", keyPrefix)] == "" {
 			return fmt.Errorf("%s.routes requires %s.address to be set", keyPrefix, keyPrefix)
 		}
 	}
 
 	// Ensure that VLAN setting is only used with parent setting.
 	if d.config["parent"] == "" && d.config["vlan"] != "" {
-		return errors.New("The vlan setting can only be used when combined with a parent interface")
+		return fmt.Errorf("The vlan setting can only be used when combined with a parent interface")
 	}
 
 	return nil
@@ -142,7 +140,7 @@ func (d *nicRouted) validateConfig(instConf instance.ConfigReader) error {
 // validateEnvironment checks the runtime environment for correctness.
 func (d *nicRouted) validateEnvironment() error {
 	if d.inst.Type() == instancetype.Container && d.config["name"] == "" {
-		return errors.New("Requires name property to start")
+		return fmt.Errorf("Requires name property to start")
 	}
 
 	if d.config["parent"] != "" {
@@ -199,7 +197,7 @@ func (d *nicRouted) validateEnvironment() error {
 
 			if sysctlVal != "1\n" {
 				// Replace . in parent name with / for sysctl formatting.
-				return fmt.Errorf("Routed mode requires sysctl net.ipv4.conf.%s.forwarding=1", strings.ReplaceAll(d.effectiveParentName, ".", "/"))
+				return fmt.Errorf("Routed mode requires sysctl net.ipv4.conf.%s.forwarding=1", strings.Replace(d.effectiveParentName, ".", "/", -1))
 			}
 		}
 
@@ -213,7 +211,7 @@ func (d *nicRouted) validateEnvironment() error {
 
 			if sysctlVal != "1\n" {
 				// Replace . in parent name with / for sysctl formatting.
-				return fmt.Errorf("Routed mode requires sysctl net.ipv6.conf.%s.forwarding=1", strings.ReplaceAll(d.effectiveParentName, ".", "/"))
+				return fmt.Errorf("Routed mode requires sysctl net.ipv6.conf.%s.forwarding=1", strings.Replace(d.effectiveParentName, ".", "/", -1))
 			}
 
 			ipv6ProxyNdpPath := fmt.Sprintf("net/ipv6/conf/%s/proxy_ndp", d.effectiveParentName)
@@ -224,7 +222,7 @@ func (d *nicRouted) validateEnvironment() error {
 
 			if sysctlVal != "1\n" {
 				// Replace . in parent name with / for sysctl formatting.
-				return fmt.Errorf("Routed mode requires sysctl net.ipv6.conf.%s.proxy_ndp=1", strings.ReplaceAll(d.effectiveParentName, ".", "/"))
+				return fmt.Errorf("Routed mode requires sysctl net.ipv6.conf.%s.proxy_ndp=1", strings.Replace(d.effectiveParentName, ".", "/", -1))
 			}
 		}
 	}
@@ -302,7 +300,7 @@ func (d *nicRouted) Start() (*deviceConfig.RunConfig, error) {
 		}
 
 		// Record whether we created this device or not so it can be removed on stop.
-		saveData["last_state.created"] = strconv.FormatBool(statusDev != "existing")
+		saveData["last_state.created"] = fmt.Sprintf("%t", statusDev != "existing")
 
 		// If we created a VLAN interface, we need to setup the sysctls on that interface.
 		if shared.IsTrue(saveData["last_state.created"]) {
@@ -393,7 +391,7 @@ func (d *nicRouted) Start() (*deviceConfig.RunConfig, error) {
 			ipFamilyArg = ip.FamilyV6
 		}
 
-		addresses := shared.SplitNTrimSpace(d.config[keyPrefix+".address"], ",", -1, true)
+		addresses := shared.SplitNTrimSpace(d.config[fmt.Sprintf("%s.address", keyPrefix)], ",", -1, true)
 
 		// Add host-side gateway addresses.
 		if len(addresses) > 0 {
@@ -437,11 +435,11 @@ func (d *nicRouted) Start() (*deviceConfig.RunConfig, error) {
 			// This is in addition to the static route added to the main routing table, which is still
 			// critical to ensure that reverse path filtering doesn't kick in blocking traffic from
 			// the instance.
-			if d.config[keyPrefix+".host_table"] != "" {
+			if d.config[fmt.Sprintf("%s.host_table", keyPrefix)] != "" {
 				r := ip.Route{
 					DevName: saveData["host_name"],
 					Route:   fmt.Sprintf("%s/%d", addrStr, subnetSize),
-					Table:   d.config[keyPrefix+".host_table"],
+					Table:   d.config[fmt.Sprintf("%s.host_table", keyPrefix)],
 					Family:  ipFamilyArg,
 				}
 
@@ -467,8 +465,8 @@ func (d *nicRouted) Start() (*deviceConfig.RunConfig, error) {
 			}
 		}
 
-		if d.config[keyPrefix+".routes"] != "" {
-			routes := shared.SplitNTrimSpace(d.config[keyPrefix+".routes"], ",", -1, true)
+		if d.config[fmt.Sprintf("%s.routes", keyPrefix)] != "" {
+			routes := shared.SplitNTrimSpace(d.config[fmt.Sprintf("%s.routes", keyPrefix)], ",", -1, true)
 
 			if len(addresses) == 0 {
 				return nil, fmt.Errorf("%s.routes requires %s.address to be set", keyPrefix, keyPrefix)
@@ -509,12 +507,12 @@ func (d *nicRouted) Start() (*deviceConfig.RunConfig, error) {
 
 	if d.inst.Type() == instancetype.Container {
 		for _, keyPrefix := range []string{"ipv4", "ipv6"} {
-			ipAddresses := shared.SplitNTrimSpace(d.config[keyPrefix+".address"], ",", -1, true)
+			ipAddresses := shared.SplitNTrimSpace(d.config[fmt.Sprintf("%s.address", keyPrefix)], ",", -1, true)
 
 			// Use a fixed address as the auto next-hop default gateway if using this IP family.
-			if len(ipAddresses) > 0 && nicHasAutoGateway(d.config[keyPrefix+".gateway"]) {
+			if len(ipAddresses) > 0 && nicHasAutoGateway(d.config[fmt.Sprintf("%s.gateway", keyPrefix)]) {
 				runConf.NetworkInterface = append(runConf.NetworkInterface,
-					deviceConfig.RunConfigItem{Key: keyPrefix + ".gateway", Value: d.ipHostAddress(keyPrefix)},
+					deviceConfig.RunConfigItem{Key: fmt.Sprintf("%s.gateway", keyPrefix), Value: d.ipHostAddress(keyPrefix)},
 				)
 			}
 
@@ -522,7 +520,7 @@ func (d *nicRouted) Start() (*deviceConfig.RunConfig, error) {
 				// Add addresses to instance NIC.
 				if keyPrefix == "ipv6" {
 					runConf.NetworkInterface = append(runConf.NetworkInterface,
-						deviceConfig.RunConfigItem{Key: "ipv6.address", Value: addrStr + "/128"},
+						deviceConfig.RunConfigItem{Key: "ipv6.address", Value: fmt.Sprintf("%s/128", addrStr)},
 					)
 				} else {
 					// Specify the broadcast address as 0.0.0.0 as there is no broadcast address on
@@ -530,7 +528,7 @@ func (d *nicRouted) Start() (*deviceConfig.RunConfig, error) {
 					// (and getting it wrong) which can prevent instances communicating with each other
 					// using adjacent IP addresses.
 					runConf.NetworkInterface = append(runConf.NetworkInterface,
-						deviceConfig.RunConfigItem{Key: "ipv4.address", Value: addrStr + "/32 0.0.0.0"},
+						deviceConfig.RunConfigItem{Key: "ipv4.address", Value: fmt.Sprintf("%s/32 0.0.0.0", addrStr)},
 					)
 				}
 			}
@@ -538,7 +536,7 @@ func (d *nicRouted) Start() (*deviceConfig.RunConfig, error) {
 	} else if d.inst.Type() == instancetype.VM {
 		runConf.NetworkInterface = append(runConf.NetworkInterface, []deviceConfig.RunConfigItem{
 			{Key: "devName", Value: d.name},
-			{Key: "mtu", Value: strconv.FormatUint(uint64(mtu), 10)},
+			{Key: "mtu", Value: fmt.Sprintf("%d", mtu)},
 		}...)
 	}
 
@@ -682,7 +680,7 @@ func (d *nicRouted) postStop() error {
 }
 
 func (d *nicRouted) ipHostAddress(ipFamily string) string {
-	key := ipFamily + ".host_address"
+	key := fmt.Sprintf("%s.host_address", ipFamily)
 	if d.config[key] != "" {
 		return d.config[key]
 	}
