@@ -8,7 +8,6 @@ import (
 	"net"
 	"os"
 	"path/filepath"
-	"slices"
 	"strconv"
 	"strings"
 	"time"
@@ -75,8 +74,8 @@ func (d *proxy) validateConfig(instConf instance.ConfigReader) error {
 	// Supported bind types are: "host" or "instance" (or "guest" or "container", legacy options equivalent to "instance").
 	// If an empty value is supplied the default behavior is to assume "host" bind mode.
 	validateBind := func(input string) error {
-		if !slices.Contains([]string{"host", "instance", "guest", "container"}, d.config["bind"]) {
-			return errors.New("Invalid binding side given. Must be \"host\" or \"instance\"")
+		if !shared.ValueInSlice(d.config["bind"], []string{"host", "instance", "guest", "container"}) {
+			return fmt.Errorf("Invalid binding side given. Must be \"host\" or \"instance\"")
 		}
 
 		return nil
@@ -169,7 +168,7 @@ func (d *proxy) validateConfig(instConf instance.ConfigReader) error {
 	}
 
 	if instConf.Type() == instancetype.VM && shared.IsFalseOrEmpty(d.config["nat"]) {
-		return errors.New("Only NAT mode is supported for proxies on VM instances")
+		return fmt.Errorf("Only NAT mode is supported for proxies on VM instances")
 	}
 
 	listenAddr, err := network.ProxyParseAddr(d.config["listen"])
@@ -189,16 +188,16 @@ func (d *proxy) validateConfig(instConf instance.ConfigReader) error {
 
 	if (listenAddr.ConnType != "unix" && len(connectAddr.Ports) > len(listenAddr.Ports)) || (listenAddr.ConnType == "unix" && len(connectAddr.Ports) > 1) {
 		// Cannot support single address (or port) -> multiple port.
-		return errors.New("Mismatch between listen port(s) and connect port(s) count")
+		return fmt.Errorf("Mismatch between listen port(s) and connect port(s) count")
 	}
 
 	if shared.IsTrue(d.config["proxy_protocol"]) && (!strings.HasPrefix(d.config["connect"], "tcp") || shared.IsTrue(d.config["nat"])) {
-		return errors.New("The PROXY header can only be sent to tcp servers in non-nat mode")
+		return fmt.Errorf("The PROXY header can only be sent to tcp servers in non-nat mode")
 	}
 
 	if (!strings.HasPrefix(d.config["listen"], "unix:") || strings.HasPrefix(d.config["listen"], "unix:@")) &&
 		(d.config["uid"] != "" || d.config["gid"] != "" || d.config["mode"] != "") {
-		return errors.New("Only proxy devices for non-abstract unix sockets can carry uid, gid, or mode properties")
+		return fmt.Errorf("Only proxy devices for non-abstract unix sockets can carry uid, gid, or mode properties")
 	}
 
 	if shared.IsTrue(d.config["nat"]) {
@@ -210,12 +209,12 @@ func (d *proxy) validateConfig(instConf instance.ConfigReader) error {
 				// Prevent use of NAT mode on non-default projects with networks feature.
 				// This is because OVN networks don't allow the host to communicate directly with
 				// instance NICs and so DNAT rules on the host won't work.
-				return errors.New("NAT mode cannot be used in projects that have the networks feature")
+				return fmt.Errorf("NAT mode cannot be used in projects that have the networks feature")
 			}
 		}
 
 		if d.config["bind"] != "" && d.config["bind"] != "host" {
-			return errors.New("Only host-bound proxies can use NAT")
+			return fmt.Errorf("Only host-bound proxies can use NAT")
 		}
 
 		// Support TCP <-> TCP and UDP <-> UDP only.
@@ -243,7 +242,7 @@ func (d *proxy) validateConfig(instConf instance.ConfigReader) error {
 		}
 
 		if listenIPVersion != connectIPVersion {
-			return errors.New("Cannot mix IP versions between listen and connect in nat mode")
+			return fmt.Errorf("Cannot mix IP versions between listen and connect in nat mode")
 		}
 	}
 
@@ -253,7 +252,7 @@ func (d *proxy) validateConfig(instConf instance.ConfigReader) error {
 // validateEnvironment checks the runtime environment for correctness.
 func (d *proxy) validateEnvironment() error {
 	if d.name == "" {
-		return errors.New("Device name cannot be empty")
+		return fmt.Errorf("Device name cannot be empty")
 	}
 
 	return nil
@@ -308,7 +307,7 @@ func (d *proxy) Start() (*deviceConfig.RunConfig, error) {
 				return err
 			}
 
-			devFileName := "proxy." + d.name
+			devFileName := fmt.Sprintf("proxy.%s", d.name)
 			pidPath := filepath.Join(d.inst.DevicesPath(), devFileName)
 			logFileName := fmt.Sprintf("proxy.%s.log", d.name)
 			logPath := filepath.Join(d.inst.LogPath(), logFileName)
@@ -354,7 +353,7 @@ func (d *proxy) Start() (*deviceConfig.RunConfig, error) {
 			}
 
 			// Poll log file a few times until we see "Started" to indicate successful start.
-			for range 10 {
+			for i := 0; i < 10; i++ {
 				started, err := d.checkProcStarted(logPath)
 				if err != nil {
 					_ = p.Stop()
@@ -437,7 +436,7 @@ func (d *proxy) Stop() (*deviceConfig.RunConfig, error) {
 		logger.Errorf("Failed to remove proxy NAT filters: %v", err)
 	}
 
-	devFileName := "proxy." + d.name
+	devFileName := fmt.Sprintf("proxy.%s", d.name)
 	devPath := filepath.Join(d.inst.DevicesPath(), devFileName)
 
 	if !shared.PathExists(devPath) {
@@ -489,7 +488,7 @@ func (d *proxy) setupNAT() error {
 		}
 
 		// Check if the instance has a NIC with a static IP that is reachable from the host.
-		if !slices.Contains([]string{"bridged", "routed"}, nicType) {
+		if !shared.ValueInSlice(nicType, []string{"bridged", "routed"}) {
 			continue
 		}
 
@@ -541,7 +540,7 @@ func (d *proxy) setupNAT() error {
 		}
 
 		if hostName == "" {
-			return errors.New("Proxy cannot find bridge port host_name to enable hairpin mode")
+			return fmt.Errorf("Proxy cannot find bridge port host_name to enable hairpin mode")
 		}
 
 		// br_netfilter is enabled, so we need to enable hairpin mode on instance's bridge port otherwise
@@ -619,22 +618,22 @@ func (d *proxy) setupProxyProcInfo() (*proxyProcInfo, error) {
 	switch d.config["bind"] {
 	case "host", "":
 		listenPid = lxdPid
-		listenPidFd = strconv.Itoa(lxdPidFd)
+		listenPidFd = fmt.Sprintf("%d", lxdPidFd)
 
 		connectPid = containerPid
-		connectPidFd = strconv.Itoa(containerPidFd)
+		connectPidFd = fmt.Sprintf("%d", containerPidFd)
 
 		listenAddr = d.rewriteHostAddr(listenAddr)
 	case "instance", "guest", "container":
 		listenPid = containerPid
-		listenPidFd = strconv.Itoa(containerPidFd)
+		listenPidFd = fmt.Sprintf("%d", containerPidFd)
 
 		connectPid = lxdPid
-		connectPidFd = strconv.Itoa(lxdPidFd)
+		connectPidFd = fmt.Sprintf("%d", lxdPidFd)
 
 		connectAddr = d.rewriteHostAddr(connectAddr)
 	default:
-		return nil, errors.New("Invalid binding side given. Must be \"host\" or \"instance\"")
+		return nil, fmt.Errorf("Invalid binding side given. Must be \"host\" or \"instance\"")
 	}
 
 	listenAddrMode := "0644"
@@ -686,9 +685,8 @@ func (d *proxy) killProxyProc(pidPath string) error {
 	}
 
 	// Remove socket file if needed.
-	// Unix non-abstract sockets are addressed to the host filesystem, not scoped inside the LXD snap.
-	if listenAddr.ConnType == "unix" && !listenAddr.Abstract && d.config["bind"] == "host" {
-		err = os.Remove(shared.HostPath(listenAddr.Address))
+	if listenAddr.ConnType == "unix" && !listenAddr.Abstract {
+		err = os.Remove(listenAddr.Address)
 		if err != nil && !errors.Is(err, os.ErrNotExist) {
 			return fmt.Errorf("Failed to remove socket file: %w", err)
 		}
