@@ -2,7 +2,6 @@ package device
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"net"
 	"os"
@@ -73,7 +72,7 @@ func (d *tpm) validateEnvironment() error {
 	// Validate the required binary.
 	_, err := exec.LookPath("swtpm")
 	if err != nil {
-		return fmt.Errorf("Required tool %q is missing", "swtpm")
+		return fmt.Errorf("Required tool '%s' is missing", "swtpm")
 	}
 
 	if d.inst.Type() == instancetype.Container {
@@ -97,7 +96,7 @@ func (d *tpm) Start() (*deviceConfig.RunConfig, error) {
 		return nil, fmt.Errorf("Failed to validate environment: %w", err)
 	}
 
-	tpmDevPath := filepath.Join(d.inst.Path(), "tpm."+filesystem.PathNameEncode(d.name))
+	tpmDevPath := filepath.Join(d.inst.Path(), fmt.Sprintf("tpm.%s", filesystem.PathNameEncode(d.name)))
 
 	if !shared.PathExists(tpmDevPath) {
 		err := os.Mkdir(tpmDevPath, 0700)
@@ -115,11 +114,11 @@ func (d *tpm) Start() (*deviceConfig.RunConfig, error) {
 
 func (d *tpm) startContainer() (*deviceConfig.RunConfig, error) {
 	escapedDeviceName := filesystem.PathNameEncode(d.name)
-	tpmDevPath := filepath.Join(d.inst.Path(), "tpm."+escapedDeviceName)
+	tpmDevPath := filepath.Join(d.inst.Path(), fmt.Sprintf("tpm.%s", escapedDeviceName))
 	logFileName := fmt.Sprintf("tpm.%s.log", escapedDeviceName)
 	logPath := filepath.Join(d.inst.LogPath(), logFileName)
 
-	proc, err := subprocess.NewProcess("swtpm", []string{"chardev", "--tpm2", "--tpmstate", "dir=" + tpmDevPath, "--vtpm-proxy"}, logPath, "")
+	proc, err := subprocess.NewProcess("swtpm", []string{"chardev", "--tpm2", "--tpmstate", fmt.Sprintf("dir=%s", tpmDevPath), "--vtpm-proxy"}, logPath, "")
 	if err != nil {
 		return nil, fmt.Errorf("Failed to create new process: %w", err)
 	}
@@ -135,7 +134,7 @@ func (d *tpm) startContainer() (*deviceConfig.RunConfig, error) {
 	// Stop the TPM emulator if anything goes wrong.
 	revert.Add(func() { _ = proc.Stop() })
 
-	pidPath := filepath.Join(d.inst.DevicesPath(), escapedDeviceName+".pid")
+	pidPath := filepath.Join(d.inst.DevicesPath(), fmt.Sprintf("%s.pid", escapedDeviceName))
 
 	err = proc.Save(pidPath)
 	if err != nil {
@@ -149,7 +148,7 @@ func (d *tpm) startContainer() (*deviceConfig.RunConfig, error) {
 	// We need to capture the output of the TPM emulator since it contains the device path. To do
 	// that, we wait until something has been written to the log file (stdout redirect), and then
 	// read it.
-	for range 20 {
+	for i := 0; i < 20; i++ {
 		fi, err := os.Stat(logPath)
 		if err != nil {
 			return nil, fmt.Errorf("Failed to stat %q: %w", logPath, err)
@@ -173,7 +172,7 @@ func (d *tpm) startContainer() (*deviceConfig.RunConfig, error) {
 	fields := strings.Split(string(line), " ")
 
 	if len(fields) < 7 {
-		return nil, errors.New("Failed to get TPM device information")
+		return nil, fmt.Errorf("Failed to get TPM device information")
 	}
 
 	_, err = fmt.Sscanf(fields[6], "%d/%d)", &major, &minor)
@@ -183,7 +182,7 @@ func (d *tpm) startContainer() (*deviceConfig.RunConfig, error) {
 
 	// Return error as we were unable to retrieve information regarding the TPM device.
 	if major == 0 && minor == 0 {
-		return nil, errors.New("Failed to get TPM device information")
+		return nil, fmt.Errorf("Failed to get TPM device information")
 	}
 
 	if minor == TPM_MINOR {
@@ -214,7 +213,7 @@ func (d *tpm) startVM() (*deviceConfig.RunConfig, error) {
 	defer revert.Fail()
 
 	escapedDeviceName := filesystem.PathNameEncode(d.name)
-	tpmDevPath := filepath.Join(d.inst.Path(), "tpm."+escapedDeviceName)
+	tpmDevPath := filepath.Join(d.inst.Path(), fmt.Sprintf("tpm.%s", escapedDeviceName))
 	socketPath := filepath.Join(tpmDevPath, fmt.Sprintf("swtpm-%s.sock", escapedDeviceName))
 	runConf := deviceConfig.RunConfig{
 		TPMDevice: []deviceConfig.RunConfigItem{
@@ -248,7 +247,7 @@ func (d *tpm) startVM() (*deviceConfig.RunConfig, error) {
 
 	unixListener, ok := listener.(*net.UnixListener)
 	if !ok {
-		return nil, errors.New("Failed getting UnixListener for swtpm")
+		return nil, fmt.Errorf("Failed getting UnixListener for swtpm")
 	}
 
 	revert.Add(func() {
@@ -265,7 +264,7 @@ func (d *tpm) startVM() (*deviceConfig.RunConfig, error) {
 
 	defer func() { _ = unixFile.Close() }()
 
-	proc, err := subprocess.NewProcess("swtpm", []string{"socket", "--tpm2", "--tpmstate", "dir=" + tpmDevPath, "--ctrl", "type=unixio,fd=3"}, "", "")
+	proc, err := subprocess.NewProcess("swtpm", []string{"socket", "--tpm2", "--tpmstate", fmt.Sprintf("dir=%s", tpmDevPath), "--ctrl", "type=unixio,fd=3"}, "", "")
 	if err != nil {
 		return nil, err
 	}
@@ -282,7 +281,7 @@ func (d *tpm) startVM() (*deviceConfig.RunConfig, error) {
 
 	revert.Add(func() { _ = proc.Stop() })
 
-	pidPath := filepath.Join(d.inst.DevicesPath(), escapedDeviceName+".pid")
+	pidPath := filepath.Join(d.inst.DevicesPath(), fmt.Sprintf("%s.pid", escapedDeviceName))
 
 	err = proc.Save(pidPath)
 	if err != nil {
@@ -297,7 +296,7 @@ func (d *tpm) startVM() (*deviceConfig.RunConfig, error) {
 
 // Stop terminates the TPM emulator.
 func (d *tpm) Stop() (*deviceConfig.RunConfig, error) {
-	pidPath := filepath.Join(d.inst.DevicesPath(), d.name+".pid")
+	pidPath := filepath.Join(d.inst.DevicesPath(), fmt.Sprintf("%s.pid", d.name))
 	runConf := deviceConfig.RunConfig{}
 
 	defer func() { _ = os.Remove(pidPath) }()
@@ -329,7 +328,7 @@ func (d *tpm) Stop() (*deviceConfig.RunConfig, error) {
 
 // Remove removes the TPM state file.
 func (d *tpm) Remove() error {
-	tpmDevPath := filepath.Join(d.inst.Path(), "tpm."+d.name)
+	tpmDevPath := filepath.Join(d.inst.Path(), fmt.Sprintf("tpm.%s", d.name))
 
 	return os.RemoveAll(tpmDevPath)
 }
