@@ -5,7 +5,6 @@ import (
 	"os"
 	"path"
 	"path/filepath"
-	"slices"
 	"sort"
 	"strconv"
 	"strings"
@@ -101,7 +100,7 @@ func deviceNetlinkListener() (chCPU chan []string, chNetwork chan []string, chUS
 			ueventParts := strings.Split(string(ueventBuf), "\x00")
 			for i, part := range ueventParts {
 				if strings.HasPrefix(part, "SEQNUM=") {
-					ueventParts = slices.Delete(ueventParts, i, i+1)
+					ueventParts = append(ueventParts[:i], ueventParts[i+1:]...)
 					break
 				}
 			}
@@ -323,7 +322,7 @@ func fillFixedInstances(fixedInstances map[int64][]instance.Instance, inst insta
 	// If the `targetCPUPool` has been manually specified (explicit CPU IDs/ranges specified with `limits.cpu`)
 	if len(targetCPUPool) == targetCPUNum && !loadBalancing {
 		for _, nr := range targetCPUPool {
-			if !slices.Contains(usableCpus, nr) {
+			if !shared.ValueInSlice(nr, usableCpus) {
 				logger.Warn("Instance using unavailable cpu", logger.Ctx{"project": inst.Project().Name, "instance": inst.Name(), "cpu": nr})
 				continue
 			}
@@ -346,7 +345,7 @@ func fillFixedInstances(fixedInstances map[int64][]instance.Instance, inst insta
 	for _, id := range targetCPUPool {
 		cpu := deviceTaskCPU{}
 		cpu.id = id
-		cpu.strID = strconv.FormatInt(id, 10)
+		cpu.strID = fmt.Sprint(id)
 
 		count := 0
 		_, ok := fixedInstances[id]
@@ -412,11 +411,11 @@ func getCPULists() (cpus []int64, isolCPUs []int64, err error) {
 	isolCPUs = resources.GetCPUIsolated()
 	effectiveCPUsSlice := []string{}
 	for _, id := range effectiveCPUsInt {
-		if slices.Contains(isolCPUs, id) {
+		if shared.ValueInSlice(id, isolCPUs) {
 			continue
 		}
 
-		effectiveCPUsSlice = append(effectiveCPUsSlice, strconv.FormatInt(id, 10))
+		effectiveCPUsSlice = append(effectiveCPUsSlice, fmt.Sprint(id))
 	}
 
 	effectiveCPUs = strings.Join(effectiveCPUsSlice, ",")
