@@ -1,7 +1,6 @@
 package endpoints
 
 import (
-	"errors"
 	"fmt"
 	"net"
 	"net/http"
@@ -120,23 +119,23 @@ type Config struct {
 // config.RestServer to it.
 func Up(config *Config) (*Endpoints, error) {
 	if config.Dir == "" {
-		return nil, errors.New("No directory configured")
+		return nil, fmt.Errorf("No directory configured")
 	}
 
 	if config.UnixSocket == "" {
-		return nil, errors.New("No unix socket configured")
+		return nil, fmt.Errorf("No unix socket configured")
 	}
 
 	if config.RestServer == nil {
-		return nil, errors.New("No REST server configured")
+		return nil, fmt.Errorf("No REST server configured")
 	}
 
 	if config.DevLxdServer == nil {
-		return nil, errors.New("No devlxd server configured")
+		return nil, fmt.Errorf("No devlxd server configured")
 	}
 
 	if config.Cert == nil {
-		return nil, errors.New("No TLS certificate configured")
+		return nil, fmt.Errorf("No TLS certificate configured")
 	}
 
 	endpoints := &Endpoints{
@@ -228,7 +227,7 @@ func (e *Endpoints) up(config *Config) error {
 	if config.NetworkAddress != "" {
 		listener, ok := e.listeners[network]
 		if ok {
-			logger.Info("Replacing inherited TCP socket with configured one")
+			logger.Infof("Replacing inherited TCP socket with configured one")
 			_ = listener.Close()
 			e.inherited[network] = false
 		}
