@@ -1,18 +1,17 @@
 package endpoints
 
 import (
-	"crypto/tls"
 	"errors"
 	"fmt"
 	"math"
 	"math/rand"
 	"net"
 
+	"github.com/mdlayher/vsock"
 	"golang.org/x/sys/unix"
 
-	"github.com/lxc/lxd/lxd/util"
-	lxdvsock "github.com/lxc/lxd/lxd/vsock"
-	"github.com/lxc/lxd/shared"
+	"github.com/canonical/lxd/lxd/endpoints/listeners"
+	"github.com/canonical/lxd/shared"
 )
 
 func createVsockListener(cert *shared.CertInfo) (net.Listener, error) {
@@ -20,7 +19,8 @@ func createVsockListener(cert *shared.CertInfo) (net.Listener, error) {
 		// Get random port between 1024 and 65535.
 		port := 1024 + rand.Int31n(math.MaxUint16-1024)
 
-		listener, err := lxdvsock.Listen(uint32(port))
+		// Setup listener on host context ID for inbound connections from lxd-agent running inside VMs.
+		listener, err := vsock.ListenContextID(vsock.Host, uint32(port), nil)
 		if err != nil {
 			// Try a different port.
 			if errors.Is(err, unix.EADDRINUSE) {
@@ -30,7 +30,7 @@ func createVsockListener(cert *shared.CertInfo) (net.Listener, error) {
 			return nil, err
 		}
 
-		return tls.NewListener(listener, util.ServerTLSConfig(cert)), nil
+		return listeners.NewFancyTLSListener(listener, cert), nil
 	}
 
 	return nil, fmt.Errorf("Failed finding free listen port for vsock listener")
@@ -41,7 +41,7 @@ func (e *Endpoints) VsockAddress() net.Addr {
 	e.mu.RLock()
 	defer e.mu.RUnlock()
 
-	listener := e.listeners[vsock]
+	listener := e.listeners[vmvsock]
 	if listener == nil {
 		return nil
 	}
