@@ -4,11 +4,9 @@ import (
 	"context"
 	"encoding/hex"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"net"
 	"os/exec"
-	"slices"
 	"strconv"
 	"strings"
 	"text/template"
@@ -51,7 +49,7 @@ func (d Nftables) Compat() (bool, error) {
 	// We require a >= 5.2 kernel to avoid weird conflicts with xtables and support for inet table NAT rules.
 	releaseLen := len(uname.Release)
 	if releaseLen > 1 {
-		verErr := errors.New("Kernel version does not meet minimum requirement of 5.2")
+		verErr := fmt.Errorf("Kernel version does not meet minimum requirement of 5.2")
 		releaseParts := strings.SplitN(uname.Release, ".", 3)
 		if len(releaseParts) < 2 {
 			return false, fmt.Errorf("Failed parsing kernel version number into parts: %w", err)
@@ -118,7 +116,7 @@ func (d Nftables) Compat() (bool, error) {
 	}
 
 	for _, item := range ruleset {
-		if item.itemType == "rule" {
+		if item.ItemType == "rule" {
 			return true, nil // At least one rule found indicates in use.
 		}
 	}
@@ -128,7 +126,7 @@ func (d Nftables) Compat() (bool, error) {
 
 // nftGenericItem represents some common fields amongst the different nftables types.
 type nftGenericItem struct {
-	itemType string // Type of item (table, chain or rule). Populated by LXD.
+	ItemType string `json:"-"`      // Type of item (table, chain or rule). Populated by LXD.
 	Family   string `json:"family"` // Family of item (ip, ip6, bridge etc).
 	Table    string `json:"table"`  // Table the item belongs to (for chains and rules).
 	Chain    string `json:"chain"`  // Chain the item belongs to (for rules).
@@ -167,13 +165,13 @@ func (d Nftables) nftParseRuleset() ([]nftGenericItem, error) {
 		chain, foundChain := item["chain"]
 		table, foundTable := item["table"]
 		if foundRule {
-			rule.itemType = "rule"
+			rule.ItemType = "rule"
 			items = append(items, rule)
 		} else if foundChain {
-			chain.itemType = "chain"
+			chain.ItemType = "chain"
 			items = append(items, chain)
 		} else if foundTable {
-			table.itemType = "table"
+			table.ItemType = "table"
 			items = append(items, table)
 		}
 	}
@@ -193,12 +191,8 @@ func (d Nftables) hostVersion() (*version.DottedVersion, error) {
 		return nil, fmt.Errorf("Failed to check nftables version: %w", err)
 	}
 
-	fields := strings.SplitN(string(output), " ", 3)
-	if len(fields) < 2 {
-		return nil, fmt.Errorf("Unexpected nft version output: %q", output)
-	}
-
-	return version.Parse(strings.TrimPrefix(fields[1], "v"))
+	lines := strings.Split(string(output), " ")
+	return version.Parse(strings.TrimPrefix(lines[1], "v"))
 }
 
 // networkSetupForwardingPolicy allows forwarding dependent on boolean argument.
@@ -467,22 +461,22 @@ func (d Nftables) InstanceClearBridgeFilter(projectName string, instanceName str
 // InstanceSetupProxyNAT creates DNAT rules for proxy devices.
 func (d Nftables) InstanceSetupProxyNAT(projectName string, instanceName string, deviceName string, forward *AddressForward) error {
 	if forward.ListenAddress == nil {
-		return errors.New("Listen address is required")
+		return fmt.Errorf("Listen address is required")
 	}
 
 	if forward.TargetAddress == nil {
-		return errors.New("Target address is required")
+		return fmt.Errorf("Target address is required")
 	}
 
 	listenPortsLen := len(forward.ListenPorts)
 	if listenPortsLen <= 0 {
-		return errors.New("At least 1 listen port must be supplied")
+		return fmt.Errorf("At least 1 listen port must be supplied")
 	}
 
 	// If multiple target ports supplied, check they match the listen port(s) count.
 	targetPortsLen := len(forward.TargetPorts)
 	if targetPortsLen != 1 && targetPortsLen != listenPortsLen {
-		return errors.New("Mismatch between listen port(s) and target port(s) count")
+		return fmt.Errorf("Mismatch between listen port(s) and target port(s) count")
 	}
 
 	ipFamily := "ip"
@@ -611,7 +605,7 @@ func (d Nftables) removeChains(families []string, chainSuffix string, chains ...
 	foundChains := make(map[string]nftGenericItem)
 	for _, family := range families {
 		for _, item := range ruleset {
-			if item.itemType == "chain" && item.Family == family && item.Table == nftablesNamespace && slices.Contains(fullChains, item.Name) {
+			if item.ItemType == "chain" && item.Family == family && item.Table == nftablesNamespace && shared.ValueInSlice(item.Name, fullChains) {
 				foundChains[item.Name] = item
 			}
 		}
@@ -725,12 +719,12 @@ func (d Nftables) NetworkApplyACLRules(networkName string, rules []ACLRule) erro
 			}
 
 			if nftRule == "" {
-				return errors.New("Invalid empty rule generated")
+				return fmt.Errorf("Invalid empty rule generated")
 			}
 
 			nftRules = append(nftRules, nftRule)
 		} else if nftRule == "" {
-			return errors.New("Invalid empty rule generated")
+			return fmt.Errorf("Invalid empty rule generated")
 		}
 	}
 
@@ -804,7 +798,7 @@ func (d Nftables) aclRuleCriteriaToRules(networkName string, ipVersion uint, rul
 	}
 
 	// Add protocol filters.
-	if slices.Contains([]string{"tcp", "udp"}, rule.Protocol) {
+	if shared.ValueInSlice(rule.Protocol, []string{"tcp", "udp"}) {
 		args = append(args, "meta", "l4proto", rule.Protocol)
 
 		if rule.SourcePort != "" {
@@ -814,7 +808,7 @@ func (d Nftables) aclRuleCriteriaToRules(networkName string, ipVersion uint, rul
 		if rule.DestinationPort != "" {
 			args = append(args, d.aclRulePortToACLMatch("dport", shared.SplitNTrimSpace(rule.DestinationPort, ",", -1, false)...)...)
 		}
-	} else if slices.Contains([]string{"icmp4", "icmp6"}, rule.Protocol) {
+	} else if shared.ValueInSlice(rule.Protocol, []string{"icmp4", "icmp6"}) {
 		var icmpIPVersion uint
 		var protoName string
 
@@ -947,7 +941,7 @@ func (d Nftables) aclRulePortToACLMatch(direction string, portCriteria ...string
 	for _, portCriterion := range portCriteria {
 		criterionParts := strings.SplitN(portCriterion, "-", 2)
 		if len(criterionParts) > 1 {
-			fieldParts = append(fieldParts, criterionParts[0]+"-"+criterionParts[1])
+			fieldParts = append(fieldParts, criterionParts[0]+criterionParts[1])
 		} else {
 			fieldParts = append(fieldParts, criterionParts[0])
 		}
