@@ -4,12 +4,10 @@ import (
 	"bufio"
 	"context"
 	"encoding/hex"
-	"errors"
 	"fmt"
 	"net"
 	"os"
 	"os/exec"
-	"slices"
 	"strconv"
 	"strings"
 	"sync"
@@ -85,7 +83,11 @@ func (d Xtables) xtablesIsNftables(cmd string) bool {
 		return false
 	}
 
-	return strings.Contains(output, "nf_tables")
+	if strings.Contains(output, "nf_tables") {
+		return true
+	}
+
+	return false
 }
 
 // iptablesInUse returns whether the specified iptables backend command has any rules defined.
@@ -121,7 +123,13 @@ func (d Xtables) iptablesInUse(iptablesCmd string) bool {
 		return false
 	}
 
-	return slices.ContainsFunc([]string{"filter", "nat", "mangle", "raw"}, tableIsUse)
+	for _, table := range []string{"filter", "nat", "mangle", "raw"} {
+		if tableIsUse(table) {
+			return true
+		}
+	}
+
+	return false
 }
 
 // ebtablesInUse returns whether the ebtables backend command has any rules defined.
@@ -254,12 +262,12 @@ func (d Xtables) networkSetupACLFilteringChains(networkName string) error {
 
 			// Allow core ICMPv4 to/from LXD host.
 			for _, icmpType := range []int{3, 11, 12} {
-				err = d.iptablesPrepend(ipVersion, comment, "filter", "INPUT", "-i", networkName, "-p", "icmp", "-m", "icmp", "--icmp-type", strconv.Itoa(icmpType), "-j", "ACCEPT")
+				err = d.iptablesPrepend(ipVersion, comment, "filter", "INPUT", "-i", networkName, "-p", "icmp", "-m", "icmp", "--icmp-type", fmt.Sprint(icmpType), "-j", "ACCEPT")
 				if err != nil {
 					return err
 				}
 
-				err = d.iptablesPrepend(ipVersion, comment, "filter", "OUTPUT", "-o", networkName, "-p", "icmp", "-m", "icmp", "--icmp-type", strconv.Itoa(icmpType), "-j", "ACCEPT")
+				err = d.iptablesPrepend(ipVersion, comment, "filter", "OUTPUT", "-o", networkName, "-p", "icmp", "-m", "icmp", "--icmp-type", fmt.Sprint(icmpType), "-j", "ACCEPT")
 				if err != nil {
 					return err
 				}
@@ -280,7 +288,7 @@ func (d Xtables) networkSetupACLFilteringChains(networkName string) error {
 
 			// Allow core ICMPv6 to/from LXD host.
 			for _, icmpType := range []int{1, 2, 3, 4, 133, 135, 136, 143} {
-				err = d.iptablesPrepend(ipVersion, comment, "filter", "INPUT", "-i", networkName, "-p", "icmpv6", "-m", "icmp6", "--icmpv6-type", strconv.Itoa(icmpType), "-j", "ACCEPT")
+				err = d.iptablesPrepend(ipVersion, comment, "filter", "INPUT", "-i", networkName, "-p", "icmpv6", "-m", "icmp6", "--icmpv6-type", fmt.Sprint(icmpType), "-j", "ACCEPT")
 				if err != nil {
 					return err
 				}
@@ -288,7 +296,7 @@ func (d Xtables) networkSetupACLFilteringChains(networkName string) error {
 
 			// Allow ICMPv6 ping from host into network as dnsmasq uses this to probe IP allocations.
 			for _, icmpType := range []int{1, 2, 3, 4, 128, 134, 135, 136, 143} {
-				err = d.iptablesPrepend(ipVersion, comment, "filter", "OUTPUT", "-o", networkName, "-p", "icmpv6", "-m", "icmp6", "--icmpv6-type", strconv.Itoa(icmpType), "-j", "ACCEPT")
+				err = d.iptablesPrepend(ipVersion, comment, "filter", "OUTPUT", "-o", networkName, "-p", "icmpv6", "-m", "icmp6", "--icmpv6-type", fmt.Sprint(icmpType), "-j", "ACCEPT")
 				if err != nil {
 					return err
 				}
@@ -379,8 +387,7 @@ func (d Xtables) networkSetupOutboundNAT(networkName string, subnet *net.IPNet,
 // networkSetupICMPDHCPDNSAccess sets up basic iptables overrides for ICMP, DHCP and DNS.
 func (d Xtables) networkSetupICMPDHCPDNSAccess(networkName string, networkAddress net.IP, ipVersion uint) error {
 	var rules [][]string
-	switch ipVersion {
-	case 4:
+	if ipVersion == 4 {
 		rules = [][]string{
 			{"4", networkName, "filter", "INPUT", "-i", networkName, "-p", "udp", "--dport", "67", "-j", "ACCEPT"},
 			// Prevent DNS requests to the bridge's dnsmasq except from lo and the bridge
@@ -397,11 +404,10 @@ func (d Xtables) networkSetupICMPDHCPDNSAccess(networkName string, networkAddres
 
 		// Allow core ICMPv4 to/from LXD host.
 		for _, icmpType := range []int{3, 11, 12} {
-			rules = append(rules, []string{"4", networkName, "filter", "INPUT", "-i", networkName, "-p", "icmp", "-m", "icmp", "--icmp-type", strconv.Itoa(icmpType), "-j", "ACCEPT"})
-			rules = append(rules, []string{"4", networkName, "filter", "OUTPUT", "-o", networkName, "-p", "icmp", "-m", "icmp", "--icmp-type", strconv.Itoa(icmpType), "-j", "ACCEPT"})
+			rules = append(rules, []string{"4", networkName, "filter", "INPUT", "-i", networkName, "-p", "icmp", "-m", "icmp", "--icmp-type", fmt.Sprint(icmpType), "-j", "ACCEPT"})
+			rules = append(rules, []string{"4", networkName, "filter", "OUTPUT", "-o", networkName, "-p", "icmp", "-m", "icmp", "--icmp-type", fmt.Sprint(icmpType), "-j", "ACCEPT"})
 		}
-
-	case 6:
+	} else if ipVersion == 6 {
 		rules = [][]string{
 			{"6", networkName, "filter", "INPUT", "-i", networkName, "-p", "udp", "--dport", "547", "-j", "ACCEPT"},
 			{"6", networkName, "filter", "INPUT", "-d", networkAddress.String(), "-p", "udp", "--dport", "53", "-j", "DROP"},
@@ -416,16 +422,15 @@ func (d Xtables) networkSetupICMPDHCPDNSAccess(networkName string, networkAddres
 
 		// Allow core ICMPv6 to/from LXD host.
 		for _, icmpType := range []int{1, 2, 3, 4, 133, 135, 136, 143} {
-			rules = append(rules, []string{"6", networkName, "filter", "INPUT", "-i", networkName, "-p", "icmpv6", "-m", "icmp6", "--icmpv6-type", strconv.Itoa(icmpType), "-j", "ACCEPT"})
+			rules = append(rules, []string{"6", networkName, "filter", "INPUT", "-i", networkName, "-p", "icmpv6", "-m", "icmp6", "--icmpv6-type", fmt.Sprint(icmpType), "-j", "ACCEPT"})
 		}
 
 		// Allow ICMPv6 ping from host into network as dnsmasq uses this to probe IP allocations.
 		for _, icmpType := range []int{1, 2, 3, 4, 128, 134, 135, 136, 143} {
-			rules = append(rules, []string{"6", networkName, "filter", "OUTPUT", "-o", networkName, "-p", "icmpv6", "-m", "icmp6", "--icmpv6-type", strconv.Itoa(icmpType), "-j", "ACCEPT"})
+			rules = append(rules, []string{"6", networkName, "filter", "OUTPUT", "-o", networkName, "-p", "icmpv6", "-m", "icmp6", "--icmpv6-type", fmt.Sprint(icmpType), "-j", "ACCEPT"})
 		}
-
-	default:
-		return errors.New("Invalid IP version")
+	} else {
+		return fmt.Errorf("Invalid IP version")
 	}
 
 	comment := d.networkIPTablesComment(networkName)
@@ -629,7 +634,7 @@ func (d Xtables) aclRuleCriteriaToArgs(networkName string, ipVersion uint, rule
 	}
 
 	// Add protocol filters.
-	if slices.Contains([]string{"tcp", "udp"}, rule.Protocol) {
+	if shared.ValueInSlice(rule.Protocol, []string{"tcp", "udp"}) {
 		args = append(args, "-p", rule.Protocol)
 
 		if rule.SourcePort != "" {
@@ -639,7 +644,7 @@ func (d Xtables) aclRuleCriteriaToArgs(networkName string, ipVersion uint, rule
 		if rule.DestinationPort != "" {
 			args = append(args, d.aclRulePortToACLMatch("dports", shared.SplitNTrimSpace(rule.DestinationPort, ",", -1, false)...)...)
 		}
-	} else if slices.Contains([]string{"icmp4", "icmp6"}, rule.Protocol) {
+	} else if shared.ValueInSlice(rule.Protocol, []string{"icmp4", "icmp6"}) {
 		var icmpIPVersion uint
 		var protoName string
 		var extName string
@@ -670,7 +675,7 @@ func (d Xtables) aclRuleCriteriaToArgs(networkName string, ipVersion uint, rule
 		}
 
 		if rule.ICMPCode != "" && rule.ICMPType == "" {
-			return nil, nil, errors.New("Invalid use of ICMP code without ICMP type")
+			return nil, nil, fmt.Errorf("Invalid use of ICMP code without ICMP type")
 		}
 
 		args = append(args, "-p", protoName)
@@ -871,7 +876,7 @@ func (d Xtables) InstanceClearBridgeFilter(projectName string, instanceName stri
 
 	errs := []error{}
 	// Iterate through each active rule on the host and try and match it to one the LXD rules.
-	for line := range strings.SplitSeq(out, "\n") {
+	for _, line := range strings.Split(out, "\n") {
 		line = strings.TrimSpace(line)
 		fields := strings.Fields(line)
 		fieldsLen := len(fields)
@@ -914,22 +919,22 @@ func (d Xtables) InstanceClearBridgeFilter(projectName string, instanceName stri
 // InstanceSetupProxyNAT creates DNAT rules for proxy devices.
 func (d Xtables) InstanceSetupProxyNAT(projectName string, instanceName string, deviceName string, forward *AddressForward) error {
 	if forward.ListenAddress == nil {
-		return errors.New("Listen address is required")
+		return fmt.Errorf("Listen address is required")
 	}
 
 	if forward.TargetAddress == nil {
-		return errors.New("Target address is required")
+		return fmt.Errorf("Target address is required")
 	}
 
 	listenPortsLen := len(forward.ListenPorts)
 	if listenPortsLen <= 0 {
-		return errors.New("At least 1 listen port must be supplied")
+		return fmt.Errorf("At least 1 listen port must be supplied")
 	}
 
 	// If multiple target ports supplied, check they match the listen port(s) count.
 	targetPortsLen := len(forward.TargetPorts)
 	if targetPortsLen != 1 && targetPortsLen != listenPortsLen {
-		return errors.New("Mismatch between listen port(s) and target port(s) count")
+		return fmt.Errorf("Mismatch between listen port(s) and target port(s) count")
 	}
 
 	ipVersion := uint(4)
@@ -1222,13 +1227,12 @@ func (d Xtables) matchEbtablesRule(activeRule []string, matchRule []string, dele
 // iptablesAdd adds an iptables rule.
 func (d Xtables) iptablesAdd(ipVersion uint, comment string, table string, method string, chain string, rule ...string) error {
 	var cmd string
-	switch ipVersion {
-	case 4:
+	if ipVersion == 4 {
 		cmd = "iptables"
-	case 6:
+	} else if ipVersion == 6 {
 		cmd = "ip6tables"
-	default:
-		return errors.New("Invalid IP version")
+	} else {
+		return fmt.Errorf("Invalid IP version")
 	}
 
 	_, err := exec.LookPath(cmd)
@@ -1262,15 +1266,14 @@ func (d Xtables) iptablesPrepend(ipVersion uint, comment string, table string, c
 func (d Xtables) iptablesClear(ipVersion uint, comments []string, fromTables ...string) error {
 	var cmd string
 	var tablesFile string
-	switch ipVersion {
-	case 4:
+	if ipVersion == 4 {
 		cmd = "iptables"
 		tablesFile = "/proc/self/net/ip_tables_names"
-	case 6:
+	} else if ipVersion == 6 {
 		cmd = "ip6tables"
 		tablesFile = "/proc/self/net/ip6_tables_names"
-	default:
-		return errors.New("Invalid IP version")
+	} else {
+		return fmt.Errorf("Invalid IP version")
 	}
 
 	// Detect kernels that lack IPv6 support.
@@ -1302,7 +1305,7 @@ func (d Xtables) iptablesClear(ipVersion uint, comments []string, fromTables ...
 	}
 
 	for _, fromTable := range fromTables {
-		if tables != nil && !slices.Contains(tables, fromTable) {
+		if tables != nil && !shared.ValueInSlice(fromTable, tables) {
 			// If we successfully opened the tables file, and the requested table is not present,
 			// then skip trying to get a list of rules from that table.
 			continue
@@ -1317,7 +1320,7 @@ func (d Xtables) iptablesClear(ipVersion uint, comments []string, fromTables ...
 			return fmt.Errorf("Failed to list IPv%d rules (table %s)", ipVersion, fromTable)
 		}
 
-		for line := range strings.SplitSeq(output, "\n") {
+		for _, line := range strings.Split(output, "\n") {
 			for _, comment := range comments {
 				if !strings.Contains(line, iptablesCommentPrefix+" "+comment) {
 					continue
@@ -1438,13 +1441,12 @@ func (d Xtables) InstanceClearNetPrio(projectName string, instanceName string, d
 // iptablesChainExists checks whether a chain exists in a table, and whether it has any rules.
 func (d Xtables) iptablesChainExists(ipVersion uint, table string, chain string) (exists, hasRules bool, err error) {
 	var cmd string
-	switch ipVersion {
-	case 4:
+	if ipVersion == 4 {
 		cmd = "iptables"
-	case 6:
+	} else if ipVersion == 6 {
 		cmd = "ip6tables"
-	default:
-		return false, false, errors.New("Invalid IP version")
+	} else {
+		return false, false, fmt.Errorf("Invalid IP version")
 	}
 
 	_, err = exec.LookPath(cmd)
@@ -1470,13 +1472,12 @@ func (d Xtables) iptablesChainExists(ipVersion uint, table string, chain string)
 // iptablesChainCreate creates a chain in a table.
 func (d Xtables) iptablesChainCreate(ipVersion uint, table string, chain string) error {
 	var cmd string
-	switch ipVersion {
-	case 4:
+	if ipVersion == 4 {
 		cmd = "iptables"
-	case 6:
+	} else if ipVersion == 6 {
 		cmd = "ip6tables"
-	default:
-		return errors.New("Invalid IP version")
+	} else {
+		return fmt.Errorf("Invalid IP version")
 	}
 
 	// Attempt to create chain in table.
@@ -1491,13 +1492,12 @@ func (d Xtables) iptablesChainCreate(ipVersion uint, table string, chain string)
 // iptablesChainDelete deletes a chain in a table.
 func (d Xtables) iptablesChainDelete(ipVersion uint, table string, chain string, flushFirst bool) error {
 	var cmd string
-	switch ipVersion {
-	case 4:
+	if ipVersion == 4 {
 		cmd = "iptables"
-	case 6:
+	} else if ipVersion == 6 {
 		cmd = "ip6tables"
-	default:
-		return errors.New("Invalid IP version")
+	} else {
+		return fmt.Errorf("Invalid IP version")
 	}
 
 	// Attempt to flush rules from chain in table.
