@@ -167,7 +167,7 @@ func addIdentityDetailsToContext(s *state.State, r *http.Request, authentication
 		return nil, err
 	}
 
-	request.SetContextValue(r, ctxClusterDBIdentity, id)
+	request.SetCtxValue(r, ctxClusterDBIdentity, id)
 	return id, nil
 }
 
@@ -203,7 +203,7 @@ func identityAccessHandler(authenticationMethod string, entitlement auth.Entitle
 //  Add a TLS identity
 //
 //  Adds a TLS identity as a trusted client.
-//  In this mode, the `trust_token` property must be set to the correct value.
+//  In this mode, the `token` property must be set to the correct value.
 //  The certificate that the client sent during the TLS handshake will be added.
 //  The `certificate` field must be omitted.
 //
@@ -250,7 +250,7 @@ func identityAccessHandler(authenticationMethod string, entitlement auth.Entitle
 //	    description: TLS Identity
 //	    required: true
 //	    schema:
-//	      $ref: "#/definitions/IdentitiesTLSPost"
+//	      $ref: "#/definitions/IdentitiesPostTLS"
 //	responses:
 //	  "201":
 //	    oneOf:
@@ -297,7 +297,7 @@ func createIdentityTLSUntrusted(ctx context.Context, s *state.State, peerCertifi
 
 	// If not trusted get the certificate from the request TLS config.
 	if len(peerCertificates) < 1 {
-		return response.BadRequest(errors.New("No client certificate provided"))
+		return response.BadRequest(fmt.Errorf("No client certificate provided"))
 	}
 
 	cert := peerCertificates[len(peerCertificates)-1]
@@ -347,17 +347,17 @@ func createIdentityTLSTrusted(ctx context.Context, s *state.State, networkCert *
 
 	// A name is required whether getting a token or directly creating the identity with a certificate.
 	if req.Name == "" {
-		return response.BadRequest(errors.New("Identity name must be provided"))
+		return response.BadRequest(fmt.Errorf("Identity name must be provided"))
 	}
 
 	// If the caller is trusted, they should not be providing a trust token
 	if req.TrustToken != "" {
-		return response.Conflict(errors.New("Client already trusted"))
+		return response.Conflict(fmt.Errorf("Client already trusted"))
 	}
 
 	// Can't request a token if a certificate is provided.
 	if req.Token && req.Certificate != "" {
-		return response.BadRequest(errors.New("Can't use certificate if token is requested"))
+		return response.BadRequest(fmt.Errorf("Can't use certificate if token is requested"))
 	}
 
 	// If a token is requested, create a pending TLS identity and return an api.CertificateAddToken.
@@ -408,12 +408,12 @@ func createIdentityTLSTrusted(ctx context.Context, s *state.State, networkCert *
 	return response.SyncResponseLocation(true, nil, lc.Source)
 }
 
-func createCertificateAddToken(s *state.State, clientName string, identityType string) (*api.CertificateAddToken, error) {
+func createIdentityTLSPending(ctx context.Context, s *state.State, req api.IdentitiesTLSPost, notify identityNotificationFunc) response.Response {
 	localHTTPSAddress := s.LocalConfig.HTTPSAddress()
 
 	// Tokens are useless if the server isn't listening (how will the untrusted client contact the server?)
 	if localHTTPSAddress == "" {
-		return nil, api.NewStatusError(http.StatusBadRequest, "Can't issue token when server isn't listening on network")
+		return response.BadRequest(fmt.Errorf("Can't issue token when server isn't listening on network"))
 	}
 
 	// Get all addresses the server is listening on. This is encoded in the certificate token,
@@ -421,7 +421,7 @@ func createCertificateAddToken(s *state.State, clientName string, identityType s
 	// through all these addresses until it can connect to one of them.
 	addresses, err := util.ListenAddresses(localHTTPSAddress)
 	if err != nil {
-		return nil, err
+		return response.InternalError(err)
 	}
 
 	// Generate join secret for new client. This will be stored inside the join token operation and will be
@@ -429,14 +429,14 @@ func createCertificateAddToken(s *state.State, clientName string, identityType s
 	// operation in order to validate the requested joining client name is correct and authorised.
 	joinSecret, err := shared.RandomCryptoString()
 	if err != nil {
-		return nil, err
+		return response.InternalError(err)
 	}
 
 	// Generate fingerprint of network certificate so joining member can automatically trust the correct
 	// certificate when it is presented during the join process.
 	fingerprint, err := shared.CertFingerprintStr(string(s.Endpoints.NetworkPublicKey()))
 	if err != nil {
-		return nil, err
+		return response.InternalError(err)
 	}
 
 	// Calculate an expiry for the pending TLS identity.
@@ -445,37 +445,15 @@ func createCertificateAddToken(s *state.State, clientName string, identityType s
 	if expiry != "" {
 		expiresAt, err = shared.GetExpiry(time.Now(), expiry)
 		if err != nil {
-			return nil, err
+			return response.InternalError(err)
 		}
 	}
 
-	// Return the CertificateAddToken.
-	token := api.CertificateAddToken{
-		ClientName:  clientName,
-		Fingerprint: fingerprint,
-		Addresses:   addresses,
-		Secret:      joinSecret,
-		ExpiresAt:   expiresAt,
-		// Set the Type field so that the client can differentiate
-		// between tokens meant for the certificates API and the auth API.
-		Type: identityType,
-	}
-
-	return &token, nil
-}
-
-func createIdentityTLSPending(ctx context.Context, s *state.State, req api.IdentitiesTLSPost, notify identityNotificationFunc) response.Response {
-	// Create CertificateAddToken token.
-	token, err := createCertificateAddToken(s, req.Name, api.IdentityTypeCertificateClient)
-	if err != nil {
-		return response.SmartError(fmt.Errorf("Failed to create certificate add token: %w", err))
-	}
-
 	// Generate an identifier for the identity and calculate its metadata.
 	identifier := uuid.New()
 	metadata := dbCluster.PendingTLSMetadata{
-		Secret: token.Secret,
-		Expiry: token.ExpiresAt,
+		Secret: joinSecret,
+		Expiry: expiresAt,
 	}
 
 	metadataJSON, err := json.Marshal(metadata)
@@ -506,6 +484,18 @@ func createIdentityTLSPending(ctx context.Context, s *state.State, req api.Ident
 		return response.InternalError(fmt.Errorf("Failed to create pending TLS identity: %w", err))
 	}
 
+	// Return the CertificateAddToken.
+	token := api.CertificateAddToken{
+		ClientName:  req.Name,
+		Fingerprint: fingerprint,
+		Addresses:   addresses,
+		Secret:      joinSecret,
+		ExpiresAt:   expiresAt,
+		// Set the Type field so that the client can differentiate
+		// between tokens meant for the certificates API and the auth API.
+		Type: api.IdentityTypeCertificateClient,
+	}
+
 	// Notify other members, update the cache, and send a lifecycle event.
 	lc, err := notify(lifecycle.IdentityCreated, api.AuthenticationMethodTLS, identifier.String(), false)
 	if err != nil {
@@ -1019,7 +1009,7 @@ func getIdentities(authenticationMethod string) func(d *Daemon, r *http.Request)
 //	  "500":
 //	    $ref: "#/responses/InternalServerError"
 func getIdentity(d *Daemon, r *http.Request) response.Response {
-	id, err := request.GetContextValue[*dbCluster.Identity](r.Context(), ctxClusterDBIdentity)
+	id, err := request.GetCtxValue[*dbCluster.Identity](r.Context(), ctxClusterDBIdentity)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1094,31 +1084,31 @@ func getIdentity(d *Daemon, r *http.Request) response.Response {
 //	  "500":
 //	    $ref: "#/responses/InternalServerError"
 func getCurrentIdentityInfo(d *Daemon, r *http.Request) response.Response {
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo == nil {
-		return response.SmartError(errors.New("Failed to get request info from the request"))
+	identifier, err := request.GetCtxValue[string](r.Context(), request.CtxUsername)
+	if err != nil {
+		return response.SmartError(fmt.Errorf("Failed to get identity identifier: %w", err))
 	}
 
-	if reqInfo.Username == "" {
-		return response.SmartError(errors.New("Failed to get identity identifier from request info"))
-	}
-
-	if reqInfo.Protocol == "" {
-		return response.SmartError(errors.New("Failed to get authentication method from request info"))
+	protocol, err := request.GetCtxValue[string](r.Context(), request.CtxProtocol)
+	if err != nil {
+		return response.SmartError(fmt.Errorf("Failed to get authentication method: %w", err))
 	}
 
 	// Must be a remote API request.
-	err := identity.ValidateAuthenticationMethod(reqInfo.Protocol)
+	err = identity.ValidateAuthenticationMethod(protocol)
 	if err != nil {
-		return response.BadRequest(errors.New("Current identity information must be requested via the HTTPS API"))
+		return response.BadRequest(fmt.Errorf("Current identity information must be requested via the HTTPS API"))
 	}
 
+	// Identity provider groups may not be present.
+	identityProviderGroupNames, _ := request.GetCtxValue[[]string](r.Context(), request.CtxIdentityProviderGroups)
+
 	s := d.State()
 	var apiIdentity *api.Identity
 	var effectiveGroups []string
 	var effectivePermissions []api.Permission
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		id, err := dbCluster.GetIdentity(ctx, tx.Tx(), dbCluster.AuthMethod(reqInfo.Protocol), reqInfo.Username)
+		id, err := dbCluster.GetIdentity(ctx, tx.Tx(), dbCluster.AuthMethod(protocol), identifier)
 		if err != nil {
 			return fmt.Errorf("Failed to get current identity from database: %w", err)
 		}
@@ -1131,13 +1121,13 @@ func getCurrentIdentityInfo(d *Daemon, r *http.Request) response.Response {
 		}
 
 		effectiveGroups = apiIdentity.Groups
-		mappedGroups, err := dbCluster.GetDistinctAuthGroupNamesFromIDPGroupNames(ctx, tx.Tx(), reqInfo.IdentityProviderGroups)
+		mappedGroups, err := dbCluster.GetDistinctAuthGroupNamesFromIDPGroupNames(ctx, tx.Tx(), identityProviderGroupNames)
 		if err != nil {
 			return fmt.Errorf("Failed to get effective groups: %w", err)
 		}
 
 		for _, mappedGroup := range mappedGroups {
-			if !slices.Contains(effectiveGroups, mappedGroup) {
+			if !shared.ValueInSlice(mappedGroup, effectiveGroups) {
 				effectiveGroups = append(effectiveGroups, mappedGroup)
 			}
 		}
@@ -1555,7 +1545,7 @@ func patchIdentityPrivileged(s *state.State, r *http.Request, id dbCluster.Ident
 		}
 
 		for _, groupName := range identityPut.Groups {
-			if !slices.Contains(apiIdentity.Groups, groupName) {
+			if !shared.ValueInSlice(groupName, apiIdentity.Groups) {
 				apiIdentity.Groups = append(apiIdentity.Groups, groupName)
 			}
 		}
@@ -1698,7 +1688,7 @@ func patchSelfIdentityUnprivileged(s *state.State, r *http.Request, id dbCluster
 //	  "501":
 //	    $ref: "#/responses/NotImplemented"
 func deleteIdentity(d *Daemon, r *http.Request) response.Response {
-	id, err := request.GetContextValue[*dbCluster.Identity](r.Context(), ctxClusterDBIdentity)
+	id, err := request.GetCtxValue[*dbCluster.Identity](r.Context(), ctxClusterDBIdentity)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1748,7 +1738,7 @@ func newIdentityNotificationFunc(s *state.State, r *http.Request, networkCert *s
 			s.UpdateIdentityCache()
 		}
 
-		lc := action.Event(authenticationMethod, identifier, request.CreateRequestor(r.Context()), nil)
+		lc := action.Event(authenticationMethod, identifier, request.CreateRequestor(r), nil)
 		s.Events.SendLifecycle(api.ProjectDefaultName, lc)
 
 		return &lc, nil
@@ -1855,7 +1845,7 @@ func updateIdentityCache(d *Daemon) {
 	identityCacheEntries := make([]identity.CacheEntry, 0, len(identities))
 	var localServerCerts []dbCluster.Certificate
 	for _, id := range identities {
-		if !slices.Contains(cacheableIdentityTypes, string(id.Type)) {
+		if !shared.ValueInSlice(string(id.Type), cacheableIdentityTypes) {
 			continue
 		}
 
