@@ -5,14 +5,12 @@ package idmap
 import (
 	"bufio"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"os"
 	"os/user"
 	"path"
 	"path/filepath"
 	"reflect"
-	"slices"
 	"sort"
 	"strconv"
 	"strings"
@@ -28,7 +26,7 @@ const VFS3FscapsSupported int32 = 1
 const VFS3FscapsUnknown int32 = -1
 
 var VFS3Fscaps int32 = VFS3FscapsUnknown
-var ErrNoUserMap = errors.New("No map found for user")
+var ErrNoUserMap = fmt.Errorf("No map found for user")
 
 type IdRange struct {
 	Isuid   bool
@@ -237,7 +235,7 @@ func (e *IdmapEntry) parse(s string) error {
 
 	// wraparound
 	if e.Hostid+e.Maprange < e.Hostid || e.Nsid+e.Maprange < e.Nsid {
-		return errors.New("Bad mapping: id wraparound")
+		return fmt.Errorf("Bad mapping: id wraparound")
 	}
 
 	return nil
@@ -250,7 +248,7 @@ func (e *IdmapEntry) parse(s string) error {
 func (e *IdmapEntry) shift_into_ns(id int64) (int64, error) {
 	if id < e.Nsid || id >= e.Nsid+e.Maprange {
 		// this mapping doesn't apply
-		return 0, errors.New("ID mapping doesn't apply")
+		return 0, fmt.Errorf("ID mapping doesn't apply")
 	}
 
 	return id - e.Nsid + e.Hostid, nil
@@ -263,7 +261,7 @@ func (e *IdmapEntry) shift_into_ns(id int64) (int64, error) {
 func (e *IdmapEntry) shift_from_ns(id int64) (int64, error) {
 	if id < e.Hostid || id >= e.Hostid+e.Maprange {
 		// this mapping doesn't apply
-		return 0, errors.New("ID mapping doesn't apply")
+		return 0, fmt.Errorf("ID mapping doesn't apply")
 	}
 
 	return id - e.Hostid + e.Nsid, nil
@@ -348,11 +346,21 @@ func (m IdmapSet) Less(i, j int) bool {
 }
 
 func (m IdmapSet) Intersects(i IdmapEntry) bool {
-	return slices.ContainsFunc(m.Idmap, i.Intersects)
+	for _, e := range m.Idmap {
+		if i.Intersects(e) {
+			return true
+		}
+	}
+	return false
 }
 
 func (m IdmapSet) HostidsIntersect(i IdmapEntry) bool {
-	return slices.ContainsFunc(m.Idmap, i.HostidsIntersect)
+	for _, e := range m.Idmap {
+		if i.HostidsIntersect(e) {
+			return true
+		}
+	}
+	return false
 }
 
 func (m IdmapSet) Usable() error {
@@ -407,7 +415,7 @@ func (m IdmapSet) ValidRanges() ([]*IdRange, error) {
 	return ranges, nil
 }
 
-var ErrHostIdIsSubId = errors.New("Host id is in the range of subids")
+var ErrHostIdIsSubId = fmt.Errorf("Host id is in the range of subids")
 
 /* AddSafe adds an entry to the idmap set, breaking apart any ranges that the
  * new idmap intersects with in the process.
@@ -495,7 +503,7 @@ func (m IdmapSet) ToLxcString() []string {
 	var lines []string
 	for _, e := range m.Idmap {
 		for _, l := range e.ToLxcString() {
-			if !slices.Contains(lines, l) {
+			if !shared.ValueInSlice(l, lines) {
 				lines = append(lines, l)
 			}
 		}
@@ -548,7 +556,7 @@ func (m IdmapSet) Append(s string) (IdmapSet, error) {
 	}
 
 	if m.Intersects(e) {
-		return m, errors.New("Conflicting id mapping")
+		return m, fmt.Errorf("Conflicting id mapping")
 	}
 
 	m.Idmap = Extend(m.Idmap, e)
@@ -635,8 +643,11 @@ func (set *IdmapSet) doUidshiftIntoContainer(dir string, testmode bool, how stri
 		}
 
 		if nlink >= 2 {
-			if slices.Contains(hardLinks, inode) {
-				return nil
+			for _, linkInode := range hardLinks {
+				// File was already shifted through hardlink
+				if linkInode == inode {
+					return nil
+				}
 			}
 
 			hardLinks = append(hardLinks, inode)
@@ -825,7 +836,7 @@ func getFromProc(fname string) ([][]int64, error) {
 	}
 
 	if len(entries) == 0 {
-		return nil, errors.New("Namespace doesn't have any map set")
+		return nil, fmt.Errorf("Namespace doesn't have any map set")
 	}
 
 	return entries, nil
@@ -1076,23 +1087,23 @@ func GetIdmapSet() *IdmapSet {
 	idmapSet, err := DefaultIdmapSet("", "")
 	if err != nil {
 		logger.Warn("Error reading default uid/gid map", map[string]any{"err": err.Error()})
-		logger.Warn("Only privileged containers will be able to run")
+		logger.Warnf("Only privileged containers will be able to run")
 		idmapSet = nil
 	} else {
 		kernelIdmapSet, err := CurrentIdmapSet()
 		if err == nil {
-			logger.Info("Kernel uid/gid map:")
+			logger.Infof("Kernel uid/gid map:")
 			for _, lxcmap := range kernelIdmapSet.ToLxcString() {
-				logger.Info(" - " + lxcmap)
+				logger.Infof(fmt.Sprintf(" - %s", lxcmap))
 			}
 		}
 
 		if len(idmapSet.Idmap) == 0 {
-			logger.Warn("No available uid/gid map could be found")
-			logger.Warn("Only privileged containers will be able to run")
+			logger.Warnf("No available uid/gid map could be found")
+			logger.Warnf("Only privileged containers will be able to run")
 			idmapSet = nil
 		} else {
-			logger.Info("Configured LXD uid/gid map:")
+			logger.Infof("Configured LXD uid/gid map:")
 			for _, lxcmap := range idmapSet.Idmap {
 				suffix := ""
 
@@ -1101,14 +1112,14 @@ func GetIdmapSet() *IdmapSet {
 				}
 
 				for _, lxcEntry := range lxcmap.ToLxcString() {
-					logger.Info(" - " + lxcEntry + suffix)
+					logger.Infof(" - %s%s", lxcEntry, suffix)
 				}
 			}
 
 			err = idmapSet.Usable()
 			if err != nil {
-				logger.Warn("One or more uid/gid map entry isn't usable (typically due to nesting)")
-				logger.Warn("Only privileged containers will be able to run")
+				logger.Warnf("One or more uid/gid map entry isn't usable (typically due to nesting)")
+				logger.Warnf("Only privileged containers will be able to run")
 				idmapSet = nil
 			}
 		}
