@@ -20,6 +20,7 @@ import (
 	"github.com/canonical/lxd/lxd/locking"
 	"github.com/canonical/lxd/lxd/operations"
 	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/project/limits"
 	"github.com/canonical/lxd/lxd/state"
 	storagePools "github.com/canonical/lxd/lxd/storage"
 	"github.com/canonical/lxd/lxd/task"
@@ -84,7 +85,7 @@ func instanceImageTransfer(s *state.State, r *http.Request, projectName string,
 	return nil
 }
 
-func ensureImageIsLocallyAvailable(s *state.State, r *http.Request, img *api.Image, projectName string, instanceType instancetype.Type) error {
+func ensureImageIsLocallyAvailable(s *state.State, r *http.Request, img *api.Image, projectName string) error {
 	// Check if the image is available locally or it's on another member.
 	// Ensure we are the only ones operating on this image. Otherwise another instance created at the same
 	// time may also arrive at the conclusion that the image doesn't exist on this cluster member and then
@@ -128,7 +129,7 @@ func ensureImageIsLocallyAvailable(s *state.State, r *http.Request, img *api.Ima
 }
 
 // instanceCreateFromImage creates an instance from a rootfs image.
-func instanceCreateFromImage(s *state.State, r *http.Request, img *api.Image, args db.InstanceArgs, op *operations.Operation) error {
+func instanceCreateFromImage(s *state.State, img *api.Image, args db.InstanceArgs, op *operations.Operation) error {
 	revert := revert.New()
 	defer revert.Fail()
 
@@ -145,7 +146,7 @@ func instanceCreateFromImage(s *state.State, r *http.Request, img *api.Image, ar
 	// Set the "image.*" keys.
 	if img.Properties != nil {
 		for k, v := range img.Properties {
-			args.Config[fmt.Sprintf("image.%s", k)] = v
+			args.Config["image."+k] = v
 		}
 	}
 
@@ -178,6 +179,15 @@ func instanceCreateFromImage(s *state.State, r *http.Request, img *api.Image, ar
 		return fmt.Errorf("Failed loading instance storage pool: %w", err)
 	}
 
+	// Lock this operation to ensure that concurrent image operations don't conflict.
+	// Other operations will wait for this one to finish.
+	unlock, err := imageOperationLock(img.Fingerprint)
+	if err != nil {
+		return err
+	}
+
+	defer unlock()
+
 	err = pool.CreateInstanceFromImage(inst, img.Fingerprint, op)
 	if err != nil {
 		return fmt.Errorf("Failed creating instance from image: %w", err)
@@ -205,7 +215,7 @@ func instanceRebuildFromImage(s *state.State, r *http.Request, inst instance.Ins
 		return fmt.Errorf("Requested image's type %q doesn't match instance type %q", imgType, inst.Type())
 	}
 
-	err = ensureImageIsLocallyAvailable(s, r, img, inst.Project().Name, inst.Type())
+	err = ensureImageIsLocallyAvailable(s, r, img, inst.Project().Name)
 	if err != nil {
 		return err
 	}
@@ -218,7 +228,7 @@ func instanceRebuildFromImage(s *state.State, r *http.Request, inst instance.Ins
 	return nil
 }
 
-func instanceRebuildFromEmpty(s *state.State, inst instance.Instance, op *operations.Operation) error {
+func instanceRebuildFromEmpty(inst instance.Instance, op *operations.Operation) error {
 	err := inst.Rebuild(nil, op) // Rebuild as empty.
 	if err != nil {
 		return fmt.Errorf("Failed rebuilding as an empty instance: %w", err)
@@ -376,8 +386,8 @@ func instanceCreateAsCopy(s *state.State, opts instanceCreateAsCopyOpts, op *ope
 				// leave alone so we don't prevent copy.
 			}
 
-			fields := strings.SplitN(srcSnap.Name(), shared.SnapshotDelimiter, 2)
-			newSnapName := fmt.Sprintf("%s/%s", inst.Name(), fields[1])
+			_, origSnapName, _ := strings.Cut(srcSnap.Name(), shared.SnapshotDelimiter)
+			newSnapName := inst.Name() + "/" + origSnapName
 			snapInstArgs := db.InstanceArgs{
 				Architecture: srcSnap.Architecture(),
 				Config:       srcSnap.LocalConfig(),
@@ -512,7 +522,7 @@ func autoCreateInstanceSnapshots(ctx context.Context, s *state.State, instances
 
 var instSnapshotsPruneRunning = sync.Map{}
 
-func pruneExpiredInstanceSnapshots(ctx context.Context, s *state.State, snapshots []instance.Instance) error {
+func pruneExpiredInstanceSnapshots(ctx context.Context, snapshots []instance.Instance) error {
 	// Find snapshots to delete
 	for _, snapshot := range snapshots {
 		err := ctx.Err()
@@ -619,7 +629,7 @@ func pruneExpiredAndAutoCreateInstanceSnapshotsTask(d *Daemon) (task.Func, task.
 
 		err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
 			return tx.InstanceList(ctx, func(dbInst db.InstanceArgs, p api.Project) error {
-				err = project.AllowSnapshotCreation(&p)
+				err = limits.AllowSnapshotCreation(&p)
 				if err != nil {
 					return nil
 				}
@@ -660,7 +670,7 @@ func pruneExpiredAndAutoCreateInstanceSnapshotsTask(d *Daemon) (task.Func, task.
 		// disk space.
 		if len(expiredSnapshotInstances) > 0 {
 			opRun := func(op *operations.Operation) error {
-				return pruneExpiredInstanceSnapshots(ctx, s, expiredSnapshotInstances)
+				return pruneExpiredInstanceSnapshots(ctx, expiredSnapshotInstances)
 			}
 
 			op, err := operations.OperationCreate(s, "", operations.OperationClassTask, operationtype.SnapshotsExpire, nil, nil, opRun, nil, nil, nil)
