@@ -5,6 +5,7 @@ import (
 	"errors"
 	"fmt"
 	"net/http"
+	"os"
 	"path/filepath"
 	"sort"
 	"strconv"
@@ -12,6 +13,7 @@ import (
 	"time"
 
 	"github.com/google/uuid"
+	"golang.org/x/sys/unix"
 
 	"github.com/canonical/lxd/lxd/backup"
 	"github.com/canonical/lxd/lxd/db"
@@ -434,7 +436,7 @@ func (d *common) StoragePool() (string, error) {
 // that it is removed then added immediately afterwards.
 func (d *common) deviceVolatileReset(devName string, oldConfig, newConfig deviceConfig.Device) error {
 	volatileClear := make(map[string]string)
-	devicePrefix := fmt.Sprintf("volatile.%s.", devName)
+	devicePrefix := "volatile." + devName + "."
 
 	newNICType, err := nictype.NICType(d.state, d.project.Name, newConfig)
 	if err != nil {
@@ -727,7 +729,16 @@ func (d *common) snapshotCommon(inst instance.Instance, name string, expiry time
 		return fmt.Errorf("Create instance snapshot: %w", err)
 	}
 
-	revert.Add(func() { _ = snap.Delete(true) })
+	revert.Add(func() {
+		switch s := snap.(type) {
+		case *lxc:
+			_ = s.delete(true)
+		case *qemu:
+			_ = s.delete(true)
+		default:
+			logger.Error("Failed to delete snapshot during revert", logger.Ctx{"instance": inst.Name(), "snapshot": snap.Name()})
+		}
+	})
 
 	// Mount volume for backup.yaml writing.
 	_, err = pool.MountInstance(inst, d.op)
@@ -1000,7 +1011,7 @@ func (d *common) maasDelete(inst instance.Instance) error {
 }
 
 // validateStartup checks any constraints that would prevent start up from succeeding under normal circumstances.
-func (d *common) validateStartup(stateful bool, statusCode api.StatusCode) error {
+func (d *common) validateStartup(statusCode api.StatusCode) error {
 	// Because the root disk is special and is mounted before the root disk device is setup we duplicate the
 	// pre-start check here before the isStartableStatusCode check below so that if there is a problem loading
 	// the instance status because the storage pool isn't available we don't mask the StatusServiceUnavailable
@@ -1024,6 +1035,30 @@ func (d *common) validateStartup(stateful bool, statusCode api.StatusCode) error
 	return nil
 }
 
+// Returns an api status code for any ongoing instance operations, or nil if no
+// operation is ongoing.
+func (d *common) operationStatusCode() *api.StatusCode {
+	op := operationlock.Get(d.Project().Name, d.Name())
+	if op != nil {
+		if op.Action() == operationlock.ActionStart {
+			stopped := api.Stopped
+			return &stopped
+		}
+
+		if op.Action() == operationlock.ActionStop {
+			if shared.IsTrue(d.LocalConfig()["volatile.last_state.ready"]) {
+				ready := api.Ready
+				return &ready
+			}
+
+			running := api.Running
+			return &running
+		}
+	}
+
+	return nil
+}
+
 // onStopOperationSetup creates or picks up the relevant operation. This is used in the stopns and stop hooks to
 // ensure that a lock on their activities is held before the instance process is stopped. This prevents a start
 // request run at the same time from overlapping with the stop process.
@@ -1166,7 +1201,7 @@ func (d *common) setCoreSched(pids []int) error {
 		args = append(args, strconv.Itoa(pid))
 	}
 
-	_, err := shared.RunCommand(d.state.OS.ExecPath, args...)
+	_, err := shared.RunCommandContext(context.Background(), d.state.OS.ExecPath, args...)
 	return err
 }
 
@@ -1220,6 +1255,13 @@ func (d *common) needsNewInstanceID(changedConfig []string, oldExpandedDevices d
 		}
 	}
 
+	// Additional SSH keys should also trigger an ID reset.
+	for _, key := range changedConfig {
+		if strings.HasPrefix(key, "cloud-init.ssh-keys.") {
+			return true
+		}
+	}
+
 	// Look for changes in network interface names.
 	getNICNames := func(devs deviceConfig.Devices) []string {
 		names := make([]string, 0, len(devs))
@@ -1297,6 +1339,21 @@ func (d *common) getStoragePool() (storagePools.Pool, error) {
 	return d.storagePool, nil
 }
 
+// getParentStoragePool retrieves the root disk device from the expanded devices.
+func (d *common) getParentStoragePool() (string, error) {
+	parentStoragePool := ""
+	parentLocalRootDiskDeviceKey, parentLocalRootDiskDevice, _ := instancetype.GetRootDiskDevice(d.ExpandedDevices().CloneNative())
+	if parentLocalRootDiskDeviceKey != "" {
+		parentStoragePool = parentLocalRootDiskDevice["pool"]
+	}
+
+	if parentStoragePool == "" {
+		return "", fmt.Errorf("Instance's root device is missing the pool property")
+	}
+
+	return parentStoragePool, nil
+}
+
 // deviceLoad instantiates and validates a new device and returns it along with enriched config.
 func (d *common) deviceLoad(inst instance.Instance, deviceName string, rawConfig deviceConfig.Device) (device.Device, error) {
 	var configCopy deviceConfig.Device
@@ -1419,13 +1476,13 @@ func (d *common) devicesRegister(inst instance.Instance) {
 }
 
 // devicesUpdate applies device changes to an instance.
-func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfig.Devices, addDevices deviceConfig.Devices, updateDevices deviceConfig.Devices, oldExpandedDevices deviceConfig.Devices, instanceRunning bool, userRequested bool) error {
+func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfig.Devices, addDevices deviceConfig.Devices, updateDevices deviceConfig.Devices, oldExpandedDevices deviceConfig.Devices, instanceRunning bool, userRequested bool) (devlxdEvents []map[string]any, err error) {
 	revert := revert.New()
 	defer revert.Fail()
 
 	dm, ok := inst.(deviceManager)
 	if !ok {
-		return fmt.Errorf("Instance is not compatible with deviceManager interface")
+		return nil, fmt.Errorf("Instance is not compatible with deviceManager interface")
 	}
 
 	// Remove devices in reverse order to how they were added.
@@ -1446,13 +1503,19 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 			if instanceRunning {
 				err = dm.deviceStop(dev, instanceRunning, "")
 				if err != nil {
-					return fmt.Errorf("Failed to stop device %q: %w", dev.Name(), err)
+					return nil, fmt.Errorf("Failed to stop device %q: %w", dev.Name(), err)
 				}
+
+				devlxdEvents = append(devlxdEvents, map[string]any{
+					"action": "removed",
+					"name":   entry.Name,
+					"config": entry.Config,
+				})
 			}
 
 			err = d.deviceRemove(dev, instanceRunning)
 			if err != nil && err != device.ErrUnsupportedDevType {
-				return fmt.Errorf("Failed to remove device %q: %w", dev.Name(), err)
+				return nil, fmt.Errorf("Failed to remove device %q: %w", dev.Name(), err)
 			}
 		}
 
@@ -1461,7 +1524,7 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 		// this device (as its an actual removal or a device type change).
 		err = d.deviceVolatileReset(entry.Name, entry.Config, addDevices[entry.Name])
 		if err != nil {
-			return fmt.Errorf("Failed to reset volatile data for device %q: %w", entry.Name, err)
+			return nil, fmt.Errorf("Failed to reset volatile data for device %q: %w", entry.Name, err)
 		}
 	}
 
@@ -1475,7 +1538,7 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 			}
 
 			if userRequested {
-				return fmt.Errorf("Failed add validation for device %q: %w", entry.Name, err)
+				return nil, fmt.Errorf("Failed add validation for device %q: %w", entry.Name, err)
 			}
 
 			// If update is non-user requested (i.e from a snapshot restore), there's nothing we can
@@ -1488,7 +1551,7 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 		err = d.deviceAdd(dev, instanceRunning)
 		if err != nil {
 			if userRequested {
-				return fmt.Errorf("Failed to add device %q: %w", dev.Name(), err)
+				return nil, fmt.Errorf("Failed to add device %q: %w", dev.Name(), err)
 			}
 
 			// If update is non-user requested (i.e from a snapshot restore), there's nothing we can
@@ -1501,15 +1564,47 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 		if instanceRunning {
 			err = dev.PreStartCheck()
 			if err != nil {
-				return fmt.Errorf("Failed pre-start check for device %q: %w", dev.Name(), err)
+				return nil, fmt.Errorf("Failed pre-start check for device %q: %w", dev.Name(), err)
 			}
 
-			_, err := dm.deviceStart(dev, instanceRunning)
+			runConf, err := dm.deviceStart(dev, instanceRunning)
 			if err != nil && err != device.ErrUnsupportedDevType {
-				return fmt.Errorf("Failed to start device %q: %w", dev.Name(), err)
+				return nil, fmt.Errorf("Failed to start device %q: %w", dev.Name(), err)
 			}
 
 			revert.Add(func() { _ = dm.deviceStop(dev, instanceRunning, "") })
+
+			event := map[string]any{
+				"action": "added",
+				"name":   entry.Name,
+				"config": entry.Config,
+			}
+
+			if runConf != nil && len(runConf.Mounts) > 0 {
+				for _, opt := range runConf.Mounts[0].Opts {
+					key, value, _ := strings.Cut(opt, "=")
+					if key != "mountTag" {
+						continue
+					}
+
+					if value == "" {
+						return nil, errors.New(`Empty "mountTag" on device's mount options`)
+					}
+
+					agentMount := instancetype.VMAgentMount{
+						Source: value,
+					}
+
+					if shared.IsTrue(dev.Config()["readonly"]) {
+						// Tell the agent to mount with "ro" option for consistency.
+						agentMount.Options = []string{"ro"}
+					}
+
+					event["mount"] = agentMount
+				}
+			}
+
+			devlxdEvents = append(devlxdEvents, event)
 		}
 	}
 
@@ -1522,7 +1617,7 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 			}
 
 			if userRequested {
-				return fmt.Errorf("Failed update validation for device %q: %w", entry.Name, err)
+				return nil, fmt.Errorf("Failed update validation for device %q: %w", entry.Name, err)
 			}
 
 			// If update is non-user requested (i.e from a snapshot restore), there's nothing we can
@@ -1542,6 +1637,12 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 					if err != nil {
 						l.Error("Failed to stop device after update validation failed", logger.Ctx{"err": err})
 					}
+
+					devlxdEvents = append(devlxdEvents, map[string]any{
+						"action": "updated",
+						"name":   entry.Name,
+						"config": entry.Config,
+					})
 				}
 
 				err = d.deviceRemove(dev, instanceRunning)
@@ -1555,12 +1656,12 @@ func (d *common) devicesUpdate(inst instance.Instance, removeDevices deviceConfi
 
 		err = dev.Update(oldExpandedDevices, instanceRunning)
 		if err != nil {
-			return fmt.Errorf("Failed to update device %q: %w", dev.Name(), err)
+			return nil, fmt.Errorf("Failed to update device %q: %w", dev.Name(), err)
 		}
 	}
 
 	revert.Success()
-	return nil
+	return devlxdEvents, nil
 }
 
 // devicesRemove runs device removal function for each device.
@@ -1615,3 +1716,116 @@ func (d *common) deleteSnapshots(deleteFunc func(snapInst instance.Instance) err
 
 	return nil
 }
+
+// checkRootVolumeNotInUse fails if the instance's root volume is in use on
+// another instance.
+func (d *common) checkRootVolumeNotInUse() error {
+	// Make sure that the instance's root volume is not attached to another instance
+	storagePool, err := d.getStoragePool()
+	if err != nil {
+		return err
+	}
+
+	rootVolumeType, err := storagePools.InstanceTypeToVolumeType(d.Type())
+	if err != nil {
+		return err
+	}
+
+	rootVolumeDBType, err := storagePools.VolumeTypeToDBType(rootVolumeType)
+	if err != nil {
+		return err
+	}
+
+	var rootVolume *db.StorageVolume
+	err = d.state.DB.Cluster.Transaction(d.state.ShutdownCtx, func(ctx context.Context, tx *db.ClusterTx) error {
+		rootVolume, err = tx.GetStoragePoolVolume(ctx, storagePool.ID(), d.Project().Name, rootVolumeDBType, d.Name(), true)
+		return err
+	})
+	if err != nil {
+		return err
+	}
+
+	err = storagePools.VolumeUsedByProfileDevices(d.state, storagePool.Name(), d.Project().Name, &rootVolume.StorageVolume, func(profileID int64, profile api.Profile, p api.Project, usedByDevices []string) error {
+		return fmt.Errorf(`"%s/%s" is attached to a profile`, rootVolume.Type, rootVolume.Name)
+	})
+	if err != nil {
+		return err
+	}
+
+	err = storagePools.VolumeUsedByInstanceDevices(d.state, storagePool.Name(), d.Project().Name, &rootVolume.StorageVolume, false, func(inst db.InstanceArgs, p api.Project, usedByDevices []string) error {
+		if inst.Name == d.Name() && inst.Project == d.Project().Name {
+			return nil
+		}
+
+		return fmt.Errorf(`"%s/%s" is attached to another instance`, rootVolume.Type, rootVolume.Name)
+	})
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// removeUnixDevices reads the devices path and remove all unix devices.
+func (d *common) removeUnixDevices() error {
+	// Check that we indeed have devices to remove
+	if !shared.PathExists(d.DevicesPath()) {
+		return nil
+	}
+
+	// Load the directory listing
+	dents, err := os.ReadDir(d.DevicesPath())
+	if err != nil {
+		return err
+	}
+
+	// Go through all the unix devices
+	for _, f := range dents {
+		// Skip non-Unix devices
+		if !strings.HasPrefix(f.Name(), "forkmknod.unix.") && !strings.HasPrefix(f.Name(), "unix.") && !strings.HasPrefix(f.Name(), device.IBDevPrefix) {
+			continue
+		}
+
+		// Remove the entry
+		devicePath := filepath.Join(d.DevicesPath(), f.Name())
+		err := os.Remove(devicePath)
+		if err != nil {
+			d.logger.Error("Failed removing unix device", logger.Ctx{"err": err, "path": devicePath})
+		}
+	}
+
+	return nil
+}
+
+func (d *common) removeDiskDevices() error {
+	// Check that we indeed have devices to remove
+	if !shared.PathExists(d.DevicesPath()) {
+		return nil
+	}
+
+	// Load the directory listing
+	dents, err := os.ReadDir(d.DevicesPath())
+	if err != nil {
+		return err
+	}
+
+	// Go through all the unix devices
+	for _, f := range dents {
+		// Skip non-disk devices
+		if !strings.HasPrefix(f.Name(), "disk.") {
+			continue
+		}
+
+		// Always try to unmount the host side
+		_ = unix.Unmount(filepath.Join(d.DevicesPath(), f.Name()), unix.MNT_DETACH)
+
+		// Remove the entry
+		diskPath := filepath.Join(d.DevicesPath(), f.Name())
+		err := os.Remove(diskPath)
+		if err != nil {
+			d.logger.Error("Failed to remove disk device path", logger.Ctx{"err": err, "path": diskPath})
+		}
+	}
+
+	return nil
+}
