@@ -1,13 +1,39 @@
 package drivers
 
 import (
+	"crypto/sha256"
+	"encoding/base64"
 	"fmt"
 	"strings"
 
 	"github.com/canonical/lxd/lxd/resources"
+	"github.com/canonical/lxd/lxd/storage/filesystem"
 	"github.com/canonical/lxd/shared/osarch"
 )
 
+// qemuDeviceNameOrID generates a QEMU device name or ID.
+// Respects the property length limit by hashing the device name when necessary. Also escapes / to -, and - to --.
+func qemuDeviceNameOrID(prefix string, deviceName string, suffix string, maxLength int) string {
+	baseName := filesystem.PathNameEncode(deviceName)
+	maxNameLength := maxLength - (len(prefix) + len(suffix))
+
+	if len(baseName) > maxNameLength {
+		// If the name is too long, hash it as SHA-256 (32 bytes).
+		// Then encode the SHA-256 binary hash as Base64 Raw URL format and trim down if needed.
+		hash := sha256.New()
+		hash.Write([]byte(baseName))
+		binaryHash := hash.Sum(nil)
+
+		// Raw URL avoids the use of "+" character and the padding "=" character which QEMU doesn't allow.
+		baseName = base64.RawURLEncoding.EncodeToString(binaryHash)
+		if len(baseName) > maxNameLength {
+			baseName = baseName[0:maxNameLength]
+		}
+	}
+
+	return prefix + baseName + suffix
+}
+
 type cfgEntry struct {
 	key   string
 	value string
@@ -24,15 +50,15 @@ func qemuStringifyCfg(cfg ...cfgSection) *strings.Builder {
 
 	for _, section := range cfg {
 		if section.comment != "" {
-			sb.WriteString(fmt.Sprintf("# %s\n", section.comment))
+			sb.WriteString("# " + section.comment + "\n")
 		}
 
-		sb.WriteString(fmt.Sprintf("[%s]\n", section.name))
+		sb.WriteString("[" + section.name + "]\n")
 
 		for _, entry := range section.entries {
 			value := entry.value
 			if value != "" {
-				sb.WriteString(fmt.Sprintf("%s = \"%s\"\n", entry.key, value))
+				sb.WriteString(entry.key + ` = "` + value + `"` + "\n")
 			}
 		}
 
@@ -48,7 +74,7 @@ func qemuMachineType(architecture int) string {
 	switch architecture {
 	case osarch.ARCH_64BIT_INTEL_X86:
 		machineType = "q35"
-	case osarch.ARCH_64BIT_ARMV8_LITTLE_ENDIAN:
+	case osarch.ARCH_32BIT_ARMV7_LITTLE_ENDIAN, osarch.ARCH_32BIT_ARMV8_LITTLE_ENDIAN, osarch.ARCH_64BIT_ARMV8_LITTLE_ENDIAN:
 		machineType = "virt"
 	case osarch.ARCH_64BIT_POWERPC_LITTLE_ENDIAN:
 		machineType = "pseries"
@@ -179,7 +205,7 @@ func qemuSerial(opts *qemuSerialOpts) []cfgSection {
 	}, {
 		// Ring buffer used by the lxd agent to report (write) its status to. LXD server will read
 		// its content via QMP using "ringbuf-read" command.
-		name:    fmt.Sprintf(`chardev "%s"`, opts.charDevName),
+		name:    `chardev "` + opts.charDevName + `"`,
 		comment: "LXD serial identifier",
 		entries: []cfgEntry{
 			{key: "backend", value: "ringbuf"},
@@ -257,7 +283,7 @@ func qemuPCIe(opts *qemuPCIeOpts) []cfgSection {
 	}
 
 	return []cfgSection{{
-		name:    fmt.Sprintf(`device "%s"`, opts.portName),
+		name:    `device "` + opts.portName + `"`,
 		entries: entries,
 	}}
 }
@@ -485,18 +511,18 @@ func qemuCPU(opts *qemuCPUOpts, pinning bool) []cfgSection {
 			return nil
 		}
 
-		// Cap the max number of CPUs to 64 unless directly assigned more.
-		max := 64
-		if int(cpu.Total) < max {
-			max = int(cpu.Total)
-		} else if opts.cpuRequested > max {
-			max = opts.cpuRequested
-		} else if opts.cpuCount > max {
-			max = opts.cpuCount
+		// Cap the maxCPU number of CPUs to 64 unless directly assigned more.
+		maxCPU := 64
+		if int(cpu.Total) < maxCPU {
+			maxCPU = int(cpu.Total)
+		} else if opts.cpuRequested > maxCPU {
+			maxCPU = opts.cpuRequested
+		} else if opts.cpuCount > maxCPU {
+			maxCPU = opts.cpuCount
 		}
 
 		entries = append(entries, cfgEntry{
-			key: "maxcpus", value: fmt.Sprintf("%d", max),
+			key: "maxcpus", value: fmt.Sprintf("%d", maxCPU),
 		})
 	}
 
@@ -630,8 +656,8 @@ func qemuDriveFirmware(opts *qemuDriveFirmwareOpts) []cfgSection {
 
 type qemuHostDriveOpts struct {
 	dev           qemuDevOpts
+	id            string
 	name          string
-	nameSuffix    string
 	comment       string
 	fsdriver      string
 	mountTag      string
@@ -656,7 +682,7 @@ func qemuHostDrive(opts *qemuHostDriveOpts) []cfgSection {
 		}
 
 		driveSection = cfgSection{
-			name:    fmt.Sprintf(`fsdev "%s"`, opts.name),
+			name:    `fsdev "` + opts.id + `"`,
 			comment: opts.comment,
 			entries: []cfgEntry{
 				{key: "fsdriver", value: opts.fsdriver},
@@ -672,11 +698,11 @@ func qemuHostDrive(opts *qemuHostDriveOpts) []cfgSection {
 
 		extraDeviceEntries = []cfgEntry{
 			{key: "mount_tag", value: opts.mountTag},
-			{key: "fsdev", value: opts.name},
+			{key: "fsdev", value: opts.id},
 		}
 	} else if opts.protocol == "virtio-fs" {
 		driveSection = cfgSection{
-			name:    fmt.Sprintf(`chardev "%s"`, opts.name),
+			name:    `chardev "` + opts.id + `"`,
 			comment: opts.comment,
 			entries: []cfgEntry{
 				{key: "backend", value: "socket"},
@@ -689,7 +715,7 @@ func qemuHostDrive(opts *qemuHostDriveOpts) []cfgSection {
 
 		extraDeviceEntries = []cfgEntry{
 			{key: "tag", value: opts.mountTag},
-			{key: "chardev", value: opts.name},
+			{key: "chardev", value: opts.id},
 		}
 	} else {
 		return []cfgSection{}
@@ -698,7 +724,7 @@ func qemuHostDrive(opts *qemuHostDriveOpts) []cfgSection {
 	return []cfgSection{
 		driveSection,
 		{
-			name:    fmt.Sprintf(`device "dev-%s%s-%s"`, opts.name, opts.nameSuffix, opts.protocol),
+			name:    `device "` + opts.id + `"`,
 			entries: append(qemuDeviceEntries(&deviceOpts), extraDeviceEntries...),
 		},
 	}
@@ -713,10 +739,10 @@ type qemuDriveConfigOpts struct {
 func qemuDriveConfig(opts *qemuDriveConfigOpts) []cfgSection {
 	return qemuHostDrive(&qemuHostDriveOpts{
 		dev: opts.dev,
+		id:  "dev-qemu_config-drive-" + opts.protocol,
 		// Devices use "qemu_" prefix indicating that this is a internally named device.
 		name:          "qemu_config",
-		nameSuffix:    "-drive",
-		comment:       fmt.Sprintf("Config drive (%s)", opts.protocol),
+		comment:       "Config drive (" + opts.protocol + ")",
 		mountTag:      "config",
 		protocol:      opts.protocol,
 		fsdriver:      "local",
@@ -739,9 +765,10 @@ type qemuDriveDirOpts struct {
 func qemuDriveDir(opts *qemuDriveDirOpts) []cfgSection {
 	return qemuHostDrive(&qemuHostDriveOpts{
 		dev: opts.dev,
+		id:  qemuDeviceNameOrID(qemuDeviceIDPrefix, opts.devName, "-"+opts.protocol, qemuDeviceIDMaxLength),
 		// Devices use "lxd_" prefix indicating that this is a user named device.
-		name:     fmt.Sprintf("lxd_%s", opts.devName),
-		comment:  fmt.Sprintf("%s drive (%s)", opts.devName, opts.protocol),
+		name:     "lxd_" + opts.devName,
+		comment:  opts.devName + " drive (" + opts.protocol + ")",
 		mountTag: opts.mountTag,
 		protocol: opts.protocol,
 		fsdriver: "proxy",
@@ -770,7 +797,7 @@ func qemuPCIPhysical(opts *qemuPCIPhysicalOpts) []cfgSection {
 
 	return []cfgSection{{
 		// Devices use "lxd_" prefix indicating that this is a user named device.
-		name:    fmt.Sprintf(`device "dev-lxd_%s"`, opts.devName),
+		name:    `device "` + qemuDeviceNameOrID(qemuDeviceIDPrefix, opts.devName, "", qemuDeviceIDMaxLength) + `"`,
 		comment: fmt.Sprintf(`PCI card ("%s" device)`, opts.devName),
 		entries: entries,
 	}}
@@ -794,7 +821,7 @@ func qemuGPUDevPhysical(opts *qemuGPUDevPhysicalOpts) []cfgSection {
 	entries := qemuDeviceEntries(&deviceOpts)
 
 	if opts.vgpu != "" {
-		sysfsdev := fmt.Sprintf("/sys/bus/mdev/devices/%s", opts.vgpu)
+		sysfsdev := "/sys/bus/mdev/devices/" + opts.vgpu
 		entries = append(entries, cfgEntry{key: "sysfsdev", value: sysfsdev})
 	} else {
 		entries = append(entries, cfgEntry{key: "host", value: opts.pciSlotName})
@@ -806,8 +833,8 @@ func qemuGPUDevPhysical(opts *qemuGPUDevPhysicalOpts) []cfgSection {
 
 	return []cfgSection{{
 		// Devices use "lxd_" prefix indicating that this is a user named device.
-		name:    fmt.Sprintf(`device "dev-lxd_%s"`, opts.devName),
-		comment: fmt.Sprintf(`GPU card ("%s" device)`, opts.devName),
+		name:    `device "` + qemuDeviceNameOrID(qemuDeviceIDPrefix, opts.devName, "", qemuDeviceIDMaxLength) + `"`,
+		comment: `GPU card ("` + opts.devName + `" device)`,
 		entries: entries,
 	}}
 }
@@ -842,7 +869,7 @@ func qemuUSB(opts *qemuUSBOpts) []cfgSection {
 	for i := 1; i <= 3; i++ {
 		chardev := fmt.Sprintf("qemu_spice-usb-chardev%d", i)
 		sections = append(sections, []cfgSection{{
-			name: fmt.Sprintf(`chardev "%s"`, chardev),
+			name: `chardev "` + chardev + `"`,
 			entries: []cfgEntry{
 				{key: "backend", value: "spicevmc"},
 				{key: "name", value: "usbredir"},
@@ -865,23 +892,24 @@ type qemuTPMOpts struct {
 }
 
 func qemuTPM(opts *qemuTPMOpts) []cfgSection {
-	chardev := fmt.Sprintf("qemu_tpm-chardev_%s", opts.devName)
-	tpmdev := fmt.Sprintf("qemu_tpm-tpmdev_%s", opts.devName)
+	chardev := qemuDeviceNameOrID("qemu_tpm-chardev_", opts.devName, "", qemuDeviceIDMaxLength)
+	tpmdev := qemuDeviceNameOrID("qemu_tpm-tpmdev_", opts.devName, "", qemuDeviceIDMaxLength)
+	device := qemuDeviceNameOrID(qemuDeviceIDPrefix, opts.devName, "", qemuDeviceIDMaxLength)
 
 	return []cfgSection{{
-		name: fmt.Sprintf(`chardev "%s"`, chardev),
+		name: `chardev "` + chardev + `"`,
 		entries: []cfgEntry{
 			{key: "backend", value: "socket"},
 			{key: "path", value: opts.path},
 		},
 	}, {
-		name: fmt.Sprintf(`tpmdev "%s"`, tpmdev),
+		name: `tpmdev "` + tpmdev + `"`,
 		entries: []cfgEntry{
 			{key: "type", value: "emulator"},
 			{key: "chardev", value: chardev},
 		},
 	}, {
-		name: fmt.Sprintf(`device "dev-lxd_%s"`, opts.devName),
+		name: `device "` + device + `"`,
 		entries: []cfgEntry{
 			{key: "driver", value: "tpm-crb"},
 			{key: "tpmdev", value: tpmdev},
