@@ -3,7 +3,6 @@ package drivers
 import (
 	"errors"
 	"fmt"
-	"slices"
 	"sync"
 
 	"github.com/canonical/lxd/lxd/db"
@@ -15,6 +14,7 @@ import (
 	"github.com/canonical/lxd/lxd/instance/instancetype"
 	"github.com/canonical/lxd/lxd/project"
 	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/logger"
 	"github.com/canonical/lxd/shared/revert"
@@ -57,13 +57,12 @@ func load(s *state.State, args db.InstanceArgs, p api.Project) (instance.Instanc
 	var inst instance.Instance
 	var err error
 
-	switch args.Type {
-	case instancetype.Container:
+	if args.Type == instancetype.Container {
 		inst, err = lxcLoad(s, args, p)
-	case instancetype.VM:
+	} else if args.Type == instancetype.VM {
 		inst, err = qemuLoad(s, args, p)
-	default:
-		return nil, fmt.Errorf("Invalid type for instance %q", args.Name)
+	} else {
+		return nil, fmt.Errorf("Invalid instance type for instance %s", args.Name)
 	}
 
 	if err != nil {
@@ -87,7 +86,7 @@ func validDevices(state *state.State, p api.Project, instanceType instancetype.T
 	checkDevices := func(devices deviceConfig.Devices, expanded bool) error {
 		// Check each device individually using the device package.
 		for deviceName, deviceConfig := range devices {
-			if expanded && slices.Contains(checkedDevices, deviceName) {
+			if expanded && shared.ValueInSlice(deviceName, checkedDevices) {
 				continue // Don't check the device twice if present in both local and expanded.
 			}
 
@@ -135,14 +134,13 @@ func validDevices(state *state.State, p api.Project, instanceType instancetype.T
 }
 
 func create(s *state.State, args db.InstanceArgs, p api.Project) (instance.Instance, revert.Hook, error) {
-	switch args.Type {
-	case instancetype.Container:
+	if args.Type == instancetype.Container {
 		return lxcCreate(s, args, p)
-	case instancetype.VM:
+	} else if args.Type == instancetype.VM {
 		return qemuCreate(s, args, p)
 	}
 
-	return nil, nil, errors.New("Instance type invalid")
+	return nil, nil, fmt.Errorf("Instance type invalid")
 }
 
 // DriverStatuses returns a map of DriverStatus structs for all instance type drivers.
@@ -171,7 +169,7 @@ func DriverStatuses() map[instancetype.Type]*DriverStatus {
 			driverStatus.Supported = false
 			driverStatus.Warning = &cluster.Warning{
 				TypeCode:    warningtype.InstanceTypeNotOperational,
-				LastMessage: driverInfo.Error.Error(),
+				LastMessage: fmt.Sprint(driverInfo.Error),
 			}
 		} else {
 			logger.Info("Instance type operational", logger.Ctx{"type": driverInfo.Type, "driver": driverInfo.Name, "features": driverInfo.Features})
