@@ -182,7 +182,7 @@ func (m *Monitor) SendFileWithFDSet(name string, file *os.File, readonly bool) (
 	}
 
 	req.Execute = "add-fd"
-	req.Arguments.Opaque = fmt.Sprintf("%s:%s", permissions, name)
+	req.Arguments.Opaque = permissions + ":" + name
 
 	reqJSON, err := json.Marshal(req)
 	if err != nil {
@@ -505,7 +505,7 @@ func (m *Monitor) RemoveBlockDevice(blockDevName string) error {
 		err := m.run("blockdev-del", blockDevName, nil)
 		if err != nil {
 			if strings.Contains(err.Error(), "is in use") {
-				return api.StatusErrorf(http.StatusLocked, err.Error())
+				return api.StatusErrorf(http.StatusLocked, "%w", err)
 			}
 
 			if strings.Contains(err.Error(), "Failed to find") {
