@@ -16,7 +16,6 @@ import (
 
 	"github.com/flosch/pongo2"
 	"github.com/google/uuid"
-	liblxc "github.com/lxc/go-lxc"
 
 	"github.com/canonical/lxd/client"
 	"github.com/canonical/lxd/lxd/backup"
@@ -112,10 +111,56 @@ func ValidConfig(sysOS *sys.OS, config map[string]string, expanded bool, instanc
 		return fmt.Errorf("nvidia.runtime is incompatible with privileged containers")
 	}
 
+	if sysOS.InUbuntuCore() && shared.IsTrue(config["nvidia.runtime"]) {
+		return fmt.Errorf("nvidia.runtime is incompatible with Ubuntu Core")
+	}
+
+	// Validate pinning strategy when limits.cpu specifies static pinning.
+	cpuPinStrategy := config["limits.cpu.pin_strategy"]
+	cpuLimit := config["limits.cpu"]
+	err = validate.IsStaticCPUPinning(cpuLimit)
+	if err == nil && !expanded && cpuPinStrategy == "auto" {
+		return fmt.Errorf(`CPU pinning specified, but pinning strategy is set to "auto"`)
+	}
+
 	return nil
 }
 
 func validConfigKey(os *sys.OS, key string, value string, instanceType instancetype.Type) error {
+	// Disallow keys with container-specific prefixes such as "linux.sysctl." and "limits.kernel." for VMs.
+	if instanceType == instancetype.VM && shared.StringHasPrefix(key, instancetype.ConfigKeyPrefixesContainer...) {
+		return fmt.Errorf("%q isn't supported for %q", key, instanceType)
+	}
+
+	// Check if the key is a valid prefix and whether or not it requires a subkey.
+	knownPrefixes := append(instancetype.ConfigKeyPrefixesAny, instancetype.ConfigKeyPrefixesContainer...)
+	if strings.HasSuffix(key, ".") {
+		if !(key == instancetype.ConfigVolatilePrefix || shared.ValueInSlice(key, knownPrefixes)) {
+			// Not a known prefix.
+			return fmt.Errorf("Unknown configuration key: %q", key)
+		}
+
+		return fmt.Errorf("%q requires a subkey", key)
+	}
+
+	// Validate the configuration key against instance type for containers and VMs.
+	// Ignore configuration keys with known prefixes since usage has already been validated, and ConfigKeyChecker validates keys syntactically.
+	if instanceType != instancetype.Any && !shared.StringHasPrefix(key, knownPrefixes...) && !strings.HasPrefix(key, instancetype.ConfigVolatilePrefix) {
+		// Ensure key is present in instance config key map based on type.
+		exists := false
+		switch instanceType {
+		case instancetype.VM:
+			_, exists = instancetype.InstanceConfigKeysVM[key]
+		case instancetype.Container:
+			_, exists = instancetype.InstanceConfigKeysContainer[key]
+		}
+
+		_, existsAny := instancetype.InstanceConfigKeysAny[key]
+		if !(exists || existsAny) {
+			return fmt.Errorf("%q isn't supported for %q", key, instanceType)
+		}
+	}
+
 	f, err := instancetype.ConfigKeyChecker(key, instanceType)
 	if err != nil {
 		return err
@@ -125,10 +170,6 @@ func validConfigKey(os *sys.OS, key string, value string, instanceType instancet
 		return err
 	}
 
-	if strings.HasPrefix(key, "limits.kernel.") && instanceType == instancetype.VM {
-		return fmt.Errorf("%s isn't supported for VMs", key)
-	}
-
 	if key == "raw.lxc" {
 		return lxcValidConfig(value)
 	}
@@ -210,33 +251,17 @@ func lxcValidConfig(rawLxc string) error {
 		}
 
 		networkKeyPrefix := "lxc.net."
-		if !liblxc.RuntimeLiblxcVersionAtLeast(liblxc.Version(), 2, 1, 0) {
-			networkKeyPrefix = "lxc.network."
-		}
-
 		if strings.HasPrefix(key, networkKeyPrefix) {
 			fields := strings.Split(key, ".")
 
-			if !liblxc.RuntimeLiblxcVersionAtLeast(liblxc.Version(), 2, 1, 0) {
-				// lxc.network.X.ipv4 or lxc.network.X.ipv6
-				if len(fields) == 4 && shared.ValueInSlice(fields[3], []string{"ipv4", "ipv6"}) {
-					continue
-				}
+			// lxc.net.X.ipv4.address or lxc.net.X.ipv6.address
+			if len(fields) == 5 && shared.ValueInSlice(fields[3], []string{"ipv4", "ipv6"}) && fields[4] == "address" {
+				continue
+			}
 
-				// lxc.network.X.ipv4.gateway or lxc.network.X.ipv6.gateway
-				if len(fields) == 5 && shared.ValueInSlice(fields[3], []string{"ipv4", "ipv6"}) && fields[4] == "gateway" {
-					continue
-				}
-			} else {
-				// lxc.net.X.ipv4.address or lxc.net.X.ipv6.address
-				if len(fields) == 5 && shared.ValueInSlice(fields[3], []string{"ipv4", "ipv6"}) && fields[4] == "address" {
-					continue
-				}
-
-				// lxc.net.X.ipv4.gateway or lxc.net.X.ipv6.gateway
-				if len(fields) == 5 && shared.ValueInSlice(fields[3], []string{"ipv4", "ipv6"}) && fields[4] == "gateway" {
-					continue
-				}
+			// lxc.net.X.ipv4.gateway or lxc.net.X.ipv6.gateway
+			if len(fields) == 5 && shared.ValueInSlice(fields[3], []string{"ipv4", "ipv6"}) && fields[4] == "gateway" {
+				continue
 			}
 
 			return fmt.Errorf("Only interface-specific ipv4/ipv6 %s keys are allowed", networkKeyPrefix)
@@ -388,50 +413,35 @@ func LoadNodeAll(s *state.State, instanceType instancetype.Type) ([]Instance, er
 	return instances, nil
 }
 
-// LoadFromBackup loads from a mounted instance's backup file.
-// If applyProfiles is false, then the profiles property will be cleared to prevent profile enrichment from DB.
-// Then the expanded config and expanded devices from the backup file will be applied to the local config and
-// local devices respectively. This is done to allow an expanded instance to be returned without needing the DB.
-func LoadFromBackup(s *state.State, projectName string, instancePath string, applyProfiles bool) (Instance, error) {
-	var inst Instance
-
+// LoadFromBackup loads from a mounted instance's backup file without needing the database.
+// Project config is not populated (as not in the backup file), however expanded config from backup file is applied
+// to avoid needing to expand config by loading profiles from database.
+func LoadFromBackup(s *state.State, projectName string, instancePath string) (Instance, error) {
 	backupYamlPath := filepath.Join(instancePath, "backup.yaml")
 	backupConf, err := backup.ParseConfigYamlFile(backupYamlPath)
 	if err != nil {
 		return nil, fmt.Errorf("Failed parsing instance backup file from %q: %w", backupYamlPath, err)
 	}
 
-	instDBArgs, err := backup.ConfigToInstanceDBArgs(s, backupConf, projectName, applyProfiles)
+	// Specify applyProfiles arg as false to avoid DB query.
+	instDBArgs, err := backup.ConfigToInstanceDBArgs(s, backupConf, projectName, false)
 	if err != nil {
 		return nil, err
 	}
 
-	if !applyProfiles {
-		// Stop instance.Load() from expanding profile config from DB, and apply expanded config from
-		// backup file to local config. This way we can still see the devices even if DB not available.
-		instDBArgs.Config = backupConf.Container.ExpandedConfig
-		instDBArgs.Devices = deviceConfig.NewDevices(backupConf.Container.ExpandedDevices)
+	// Stop instance.Load() from expanding profile config from DB, and apply expanded config from
+	// backup file to local config. This way we can still see the devices even if DB not available.
+	instDBArgs.Config = backupConf.Container.ExpandedConfig
+	instDBArgs.Devices = deviceConfig.NewDevices(backupConf.Container.ExpandedDevices)
+
+	// Set Node field to local node.
+	instDBArgs.Node = s.ServerName
+
+	p := api.Project{
+		Name: backupConf.Container.Project,
 	}
 
-	var p *api.Project
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		proj, err := cluster.GetProject(ctx, tx.Tx(), projectName)
-		if err != nil {
-			return err
-		}
-
-		p, err = proj.ToAPI(ctx, tx.Tx())
-		if err != nil {
-			return err
-		}
-
-		return nil
-	})
-	if err != nil {
-		return nil, err
-	}
-
-	inst, err = Load(s, *instDBArgs, *p)
+	inst, err := Load(s, *instDBArgs, p)
 	if err != nil {
 		return nil, fmt.Errorf("Failed loading instance from backup file %q: %w", backupYamlPath, err)
 	}
@@ -553,7 +563,7 @@ func ResolveImage(ctx context.Context, tx *db.ClusterTx, projectName string, sou
 // A nil list indicates that we can't tell at this stage, typically for private images.
 func SuitableArchitectures(ctx context.Context, s *state.State, tx *db.ClusterTx, projectName string, sourceInst *cluster.Instance, sourceImageRef string, req api.InstancesPost) ([]int, error) {
 	// Handle cases where the architecture is already provided.
-	if shared.ValueInSlice(req.Source.Type, []string{"migration", "none"}) && req.Architecture != "" {
+	if shared.ValueInSlice(req.Source.Type, []string{api.SourceTypeConversion, api.SourceTypeMigration, api.SourceTypeNone}) && req.Architecture != "" {
 		id, err := osarch.ArchitectureId(req.Architecture)
 		if err != nil {
 			return nil, err
@@ -562,23 +572,23 @@ func SuitableArchitectures(ctx context.Context, s *state.State, tx *db.ClusterTx
 		return []int{id}, nil
 	}
 
-	// For migration, an architecture must be specified in the req.
-	if req.Source.Type == "migration" && req.Architecture == "" {
-		return nil, api.StatusErrorf(http.StatusBadRequest, "An architecture must be specified in migration requests")
+	// For migration and conversion, an architecture must be specified in the req.
+	if shared.ValueInSlice(req.Source.Type, []string{api.SourceTypeConversion, api.SourceTypeMigration}) && req.Architecture == "" {
+		return nil, api.StatusErrorf(http.StatusBadRequest, "An architecture must be specified in migration or conversion requests")
 	}
 
 	// For none, allow any architecture.
-	if req.Source.Type == "none" {
+	if req.Source.Type == api.SourceTypeNone {
 		return []int{}, nil
 	}
 
 	// For copy, always use the source architecture.
-	if req.Source.Type == "copy" {
+	if req.Source.Type == api.SourceTypeCopy {
 		return []int{sourceInst.Architecture}, nil
 	}
 
 	// For image, things get a bit more complicated.
-	if req.Source.Type == "image" {
+	if req.Source.Type == api.SourceTypeImage {
 		// Handle local images.
 		if req.Source.Server == "" {
 			_, img, err := tx.GetImageByFingerprintPrefix(ctx, sourceImageRef, cluster.ImageFilter{Project: &projectName})
@@ -666,34 +676,6 @@ func SuitableArchitectures(ctx context.Context, s *state.State, tx *db.ClusterTx
 	return nil, api.StatusErrorf(http.StatusBadRequest, "Unknown instance source type %q", req.Source.Type)
 }
 
-// ValidName validates an instance name. There are different validation rules for instance snapshot names
-// so it takes an argument indicating whether the name is to be used for a snapshot or not.
-func ValidName(instanceName string, isSnapshot bool) error {
-	if isSnapshot {
-		parentName, snapshotName, _ := api.GetParentAndSnapshotName(instanceName)
-		err := validate.IsHostname(parentName)
-		if err != nil {
-			return fmt.Errorf("Invalid instance name: %w", err)
-		}
-
-		// Snapshot part is more flexible, but doesn't allow space or / character.
-		if strings.ContainsAny(snapshotName, " /") {
-			return fmt.Errorf("Invalid instance snapshot name: Cannot contain space or / characters")
-		}
-	} else {
-		if strings.Contains(instanceName, shared.SnapshotDelimiter) {
-			return fmt.Errorf("The character %q is reserved for snapshots", shared.SnapshotDelimiter)
-		}
-
-		err := validate.IsHostname(instanceName)
-		if err != nil {
-			return fmt.Errorf("Invalid instance name: %w", err)
-		}
-	}
-
-	return nil
-}
-
 // CreateInternal creates an instance record and storage volume record in the database and sets up devices.
 // Accepts a reverter that revert steps this function does will be added to. It is up to the caller to
 // call the revert's Fail() or Success() function as needed.
@@ -748,7 +730,7 @@ func CreateInternal(s *state.State, args db.InstanceArgs, clearLogDir bool) (Ins
 		args.Architecture = s.OS.Architectures[0]
 	}
 
-	err = ValidName(args.Name, args.Snapshot)
+	err = instancetype.ValidName(args.Name, args.Snapshot)
 	if err != nil {
 		return nil, nil, nil, err
 	}
@@ -967,7 +949,7 @@ func CreateInternal(s *state.State, args db.InstanceArgs, clearLogDir bool) (Ins
 		return nil
 	})
 	if err != nil {
-		if err == db.ErrAlreadyDefined {
+		if api.StatusErrorCheck(err, http.StatusConflict) {
 			thing := "Instance"
 			if shared.IsSnapshot(args.Name) {
 				thing = "Snapshot"
