@@ -8,6 +8,7 @@ import (
 	"time"
 
 	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/units"
 	"github.com/canonical/lxd/shared/validate"
 )
@@ -27,6 +28,53 @@ const (
 // ConfigVolatilePrefix indicates the prefix used for volatile config keys.
 const ConfigVolatilePrefix = "volatile."
 
+// ConfigKeyPrefixesAny indicates valid prefixes for configuration options.
+var ConfigKeyPrefixesAny = []string{"environment.", "user.", "image.", "cloud-init.ssh-keys."}
+
+// ConfigKeyPrefixesContainer indicates valid prefixes for container configuration options.
+var ConfigKeyPrefixesContainer = []string{"linux.sysctl.", "limits.kernel."}
+
+// ValidName validates an instance name. There are different validation rules for instance snapshot names
+// so it takes an argument indicating whether the name is to be used for a snapshot or not.
+func ValidName(instanceName string, isSnapshot bool) error {
+	if isSnapshot {
+		parentName, snapshotName, _ := api.GetParentAndSnapshotName(instanceName)
+		err := validate.IsHostname(parentName)
+		if err != nil {
+			return fmt.Errorf("Invalid instance name %q: %w", parentName, err)
+		}
+
+		err = ValidSnapName(snapshotName)
+		if err != nil {
+			return fmt.Errorf("Invalid instance snapshot name %q: %w", snapshotName, err)
+		}
+	} else {
+		err := validate.IsHostname(instanceName)
+		if err != nil {
+			return fmt.Errorf("Invalid instance name %q: %w", instanceName, err)
+		}
+	}
+
+	return nil
+}
+
+// ValidSnapName validates a snnapshot instance name which must not include the instance prefix.
+func ValidSnapName(snapshotName string) error {
+	if snapshotName == "" {
+		return fmt.Errorf("Invalid instance snapshot name, cannot be empty")
+	}
+
+	if snapshotName == ".." {
+		return fmt.Errorf("Invalid instance snapshot name %q", snapshotName)
+	}
+
+	if strings.ContainsAny(snapshotName, "* /\\") {
+		return fmt.Errorf("Invalid instance snapshot name %q: Cannot contain *, spaces, forward or back slashes", snapshotName)
+	}
+
+	return nil
+}
+
 // IsRootDiskDevice returns true if the given device representation is configured as root disk for
 // an instance. It typically get passed a specific entry of api.Instance.Devices.
 func IsRootDiskDevice(device map[string]string) bool {
@@ -88,7 +136,7 @@ var InstanceConfigKeysAny = map[string]func(value string) error{
 	// The number of seconds to wait after the instance started before starting the next one.
 	// ---
 	//  type: integer
-	//  defaultdesc: "0"
+	//  defaultdesc: `0`
 	//  liveupdate: no
 	//  shortdesc: Delay after starting the instance
 	"boot.autostart.delay": validate.Optional(validate.IsInt64),
@@ -97,7 +145,7 @@ var InstanceConfigKeysAny = map[string]func(value string) error{
 	// The instance with the highest value is started first.
 	// ---
 	//  type: integer
-	//  defaultdesc: "0"
+	//  defaultdesc: `0`
 	//  liveupdate: no
 	//  shortdesc: What order to start the instances in
 	"boot.autostart.priority": validate.Optional(validate.IsInt64),
@@ -106,7 +154,7 @@ var InstanceConfigKeysAny = map[string]func(value string) error{
 	// The instance with the highest value is shut down first.
 	// ---
 	//  type: integer
-	//  defaultdesc: "0"
+	//  defaultdesc: `0`
 	//  liveupdate: no
 	//  shortdesc: What order to shut down the instances in
 	"boot.stop.priority": validate.Optional(validate.IsInt64),
@@ -115,7 +163,7 @@ var InstanceConfigKeysAny = map[string]func(value string) error{
 	// Number of seconds to wait for the instance to shut down before it is force-stopped.
 	// ---
 	//  type: integer
-	//  defaultdesc: "30"
+	//  defaultdesc: `30`
 	//  liveupdate: yes
 	//  shortdesc: How long to wait for the instance to shut down
 	"boot.host_shutdown_timeout": validate.Optional(validate.IsInt64),
@@ -178,20 +226,14 @@ var InstanceConfigKeysAny = map[string]func(value string) error{
 	//  shortdesc: Legacy version of `cloud-init.vendor-data`
 
 	// lxdmeta:generate(entities=instance; group=miscellaneous; key=cluster.evacuate)
-	// The `cluster.evacuate` provides control over how instances are handled when a cluster member is being
-	// evacuated.
+	// The `cluster.evacuate` provides control over how instances are handled when a cluster member is being evacuated.
 	//
 	// Available Modes:
-	//   - `auto` *(default)*: The system will automatically decide the best evacuation method based on the
-	//      instance's type and configured devices:
+	//   - `auto` *(default)*: The system will automatically decide the best evacuation method based on the instance's type and configured devices:
 	//     + If any device is not suitable for migration, the instance will not be migrated (only stopped).
-	//     + Live migration will be used only for virtual machines with the `migration.stateful` setting
-	//       enabled and for which all its devices can be migrated as well.
-	//   - `live-migrate`: Instances are live-migrated to another node. This means the instance remains running
-	//      and operational during the migration process, ensuring minimal disruption.
-	//   - `migrate`: In this mode, instances are migrated to another node in the cluster. The migration
-	//      process will not be live, meaning there will be a brief downtime for the instance during the
-	//      migration.
+	//     + Live migration will be used only for virtual machines with the `migration.stateful` setting enabled and for which all its devices can be migrated as well.
+	//   - `live-migrate`: Instances are live-migrated to another node. This means the instance remains running and operational during the migration process, ensuring minimal disruption.
+	//   - `migrate`: In this mode, instances are migrated to another node in the cluster. The migration process will not be live, meaning there will be a brief downtime for the instance during the migration.
 	//   -  `stop`: Instances are not migrated. Instead, they are stopped on the current node.
 	//
 	// See {ref}`cluster-evacuate` for more information.
@@ -241,7 +283,7 @@ var InstanceConfigKeysAny = map[string]func(value string) error{
 	// See {ref}`instances-limit-units` for details.
 	// ---
 	//  type: string
-	//  defaultdesc: `1Gib` (VMs)
+	//  defaultdesc: `1GiB` (VMs)
 	//  liveupdate: yes
 	//  shortdesc: Usage limit for the host's memory
 	"limits.memory": func(value string) error {
@@ -302,15 +344,33 @@ var InstanceConfigKeysAny = map[string]func(value string) error{
 	//  shortdesc: Whether `/dev/lxd` is present in the instance
 	"security.devlxd": validate.Optional(validate.IsBool),
 
-	// lxdmeta:generate(entities=instance; group=security; key=security.protection.delete)
+	// lxdmeta:generate(entities=instance; group=security; key=security.devlxd.images)
 	//
 	// ---
 	//  type: bool
 	//  defaultdesc: `false`
 	//  liveupdate: yes
-	//  shortdesc: Prevents the instance from being deleted
+	//  shortdesc: Controls the availability of the `/1.0/images` API over `devlxd`
+	"security.devlxd.images": validate.Optional(validate.IsBool),
+
+	// lxdmeta:generate(entities=instance; group=security; key=security.protection.delete)
+	//
+	// ---
+	//  type: bool
+	//  defaultdesc: `false`
+	//  liveupdate: container
+	//  shortdesc: Whether to prevent the instance from being deleted
 	"security.protection.delete": validate.Optional(validate.IsBool),
 
+	// lxdmeta:generate(entities=instance; group=security; key=security.protection.start)
+	//
+	// ---
+	//  type: bool
+	//  defaultdesc: `false`
+	//  liveupdate: container
+	//  shortdesc: Whether to prevent the instance from being started
+	"security.protection.start": validate.Optional(validate.IsBool),
+
 	// lxdmeta:generate(entities=instance; group=snapshots; key=snapshots.schedule)
 	// Specify either a cron expression (`<minute> <hour> <dom> <month> <dow>`), a comma-separated list of schedule aliases (`@hourly`, `@daily`, `@midnight`, `@weekly`, `@monthly`, `@annually`, `@yearly`), or leave empty to disable automatic snapshots.
 	//
@@ -354,6 +414,20 @@ var InstanceConfigKeysAny = map[string]func(value string) error{
 		return err
 	},
 
+	// lxdmeta:generate(entities=instance; group=miscellaneous; key=ubuntu_pro.guest_attach)
+	// Indicate whether the guest should auto-attach Ubuntu Pro at start up.
+	// The allowed values are `off`, `on`, and `available`.
+	// If set to `off`, it will not be possible for the Ubuntu Pro client in the guest to obtain guest token via `devlxd`.
+	// If set to `available`, attachment via guest token is possible but will not be performed automatically by the Ubuntu Pro client in the guest at startup.
+	// If set to `on`, attachment will be performed automatically by the Ubuntu Pro client in the guest at startup.
+	// To allow guest attachment, the host must be an Ubuntu machine that is Pro attached, and guest attachment must be enabled via the Pro client.
+	// To do this, run `pro config set lxd_guest_attach=on`.
+	// ---
+	// type: string
+	// liveupdate: no
+	// shortdesc: Whether to auto-attach Ubuntu Pro.
+	"ubuntu_pro.guest_attach": validate.Optional(validate.IsOneOf("off", "on", "available")),
+
 	// Volatile keys.
 
 	// lxdmeta:generate(entities=instance; group=volatile; key=volatile.apply_template)
@@ -370,7 +444,7 @@ var InstanceConfigKeysAny = map[string]func(value string) error{
 	//  shortdesc: Hash of the base image
 	"volatile.base_image": validate.IsAny,
 
-	// lxdmeta:generate(entities=instance; group=volatile; key=volatile.cloud_init.instance-id)
+	// lxdmeta:generate(entities=instance; group=volatile; key=volatile.cloud-init.instance-id)
 	//
 	// ---
 	//  type: string
@@ -559,13 +633,26 @@ var InstanceConfigKeysContainer = map[string]func(value string) error{
 
 	// lxdmeta:generate(entities=instance; group=miscellaneous; key=linux.kernel_modules)
 	// Specify the kernel modules as a comma-separated list.
+	//
+	// The modules are loaded before the instance starts, or they can be loaded by a privileged user if {config:option}`instance-miscellaneous:linux.kernel_modules.load` is set to `ondemand`.
 	// ---
 	//  type: string
 	//  liveupdate: yes
 	//  condition: container
-	//  shortdesc: Kernel modules to load before starting the instance
+	//  shortdesc: Kernel modules to load or allow loading
 	"linux.kernel_modules": validate.IsAny,
 
+	// lxdmeta:generate(entities=instance; group=miscellaneous; key=linux.kernel_modules.load)
+	// This option specifies how to load the kernel modules that are specified in {config:option}`instance-miscellaneous:linux.kernel_modules`.
+	// Possible values are `boot` (load the modules when booting the container) and `ondemand` (intercept the `finit_modules()` syscall and allow a privileged user in the container's user namespace to load the modules).
+	// ---
+	//  type: string
+	//  defaultdesc: `boot`
+	//  liveupdate: no
+	//  condition: container
+	//  shortdesc: How to load kernel modules
+	"linux.kernel_modules.load": validate.Optional(validate.IsOneOf("boot", "ondemand")),
+
 	// lxdmeta:generate(entities=instance; group=migration; key=migration.incremental.memory)
 	// Using incremental memory transfer of the instance's memory can reduce downtime.
 	// ---
@@ -654,16 +741,6 @@ var InstanceConfigKeysContainer = map[string]func(value string) error{
 	//  shortdesc: Raw Seccomp configuration
 	"raw.seccomp": validate.IsAny,
 
-	// lxdmeta:generate(entities=instance; group=security; key=security.devlxd.images)
-	//
-	// ---
-	//  type: bool
-	//  defaultdesc: `false`
-	//  liveupdate: no
-	//  condition: container
-	//  shortdesc: Controls the availability of the `/1.0/images` API over `devlxd`
-	"security.devlxd.images": validate.Optional(validate.IsBool),
-
 	// lxdmeta:generate(entities=instance; group=security; key=security.idmap.base)
 	// Setting this option overrides auto-detection.
 	// ---
@@ -862,31 +939,37 @@ var InstanceConfigKeysContainer = map[string]func(value string) error{
 	"security.syscalls.intercept.sysinfo": validate.Optional(validate.IsBool),
 
 	// lxdmeta:generate(entities=instance; group=volatile; key=volatile.last_state.idmap)
-	//
+	// The UID/GID map that has been applied to the container's underlying storage.
+	// This is usually set for containers created on older kernels that don't
+	// support idmapped mounts.
 	// ---
 	//  type: string
-	//  shortdesc: Serialized instance UID/GID map
+	//  condition: container
+	//  shortdesc: On-disk UID/GID map for the container's rootfs
 	"volatile.last_state.idmap": validate.IsAny,
 
 	// lxdmeta:generate(entities=instance; group=volatile; key=volatile.idmap.base)
 	//
 	// ---
 	//  type: integer
-	//  shortdesc: The first ID in the instance's primary idmap range
+	//  condition: container
+	//  shortdesc: The first ID in the container's primary idmap range
 	"volatile.idmap.base": validate.IsAny,
 
 	// lxdmeta:generate(entities=instance; group=volatile; key=volatile.idmap.current)
 	//
 	// ---
 	//  type: string
-	//  shortdesc: The idmap currently in use by the instance
+	//  condition: container
+	//  shortdesc: The idmap currently in use by the container
 	"volatile.idmap.current": validate.IsAny,
 
 	// lxdmeta:generate(entities=instance; group=volatile; key=volatile.idmap.next)
 	//
 	// ---
 	//  type: string
-	//  shortdesc: The idmap to use the next time the instance starts
+	//  condition: container
+	//  shortdesc: The idmap to use the next time the container starts
 	"volatile.idmap.next": validate.IsAny,
 }
 
@@ -902,6 +985,19 @@ var InstanceConfigKeysVM = map[string]func(value string) error{
 	//  shortdesc: Whether to back the instance using huge pages
 	"limits.memory.hugepages": validate.Optional(validate.IsBool),
 
+	// lxdmeta:generate(entities=instance; group=resource-limits; key=limits.cpu.pin_strategy)
+	// Specify the strategy for VM CPU auto pinning.
+	// Possible values: `none` (disables CPU auto pinning) and `auto` (enables CPU auto pinning).
+	//
+	// See {ref}`instance-options-limits-cpu-vm` for more information.
+	// ---
+	//  type: string
+	//  defaultdesc: `none`
+	//  liveupdate: no
+	//	condition: virtual machine
+	//  shortdesc: VM CPU auto pinning strategy
+	"limits.cpu.pin_strategy": validate.Optional(validate.IsOneOf("none", "auto")),
+
 	// lxdmeta:generate(entities=instance; group=migration; key=migration.stateful)
 	// Enabling this option prevents the use of some features that are incompatible with it.
 	// ---
@@ -990,7 +1086,7 @@ var InstanceConfigKeysVM = map[string]func(value string) error{
 	//  liveupdate: no
 	//  condition: virtual machine
 	//  shortdesc: The guest owner's `base64`-encoded Diffie-Hellman key
-	"security.sev.session.dh": validate.Optional(validate.IsAny),
+	"security.sev.session.dh": validate.IsAny,
 
 	// lxdmeta:generate(entities=instance; group=security; key=security.sev.session.data)
 	//
@@ -1000,7 +1096,7 @@ var InstanceConfigKeysVM = map[string]func(value string) error{
 	//  liveupdate: no
 	//  condition: virtual machine
 	//  shortdesc: The guest owner's `base64`-encoded session blob
-	"security.sev.session.data": validate.Optional(validate.IsAny),
+	"security.sev.session.data": validate.IsAny,
 
 	// lxdmeta:generate(entities=instance; group=miscellaneous; key=user.*)
 	// User keys can be used in search.
@@ -1010,8 +1106,7 @@ var InstanceConfigKeysVM = map[string]func(value string) error{
 	//  shortdesc: Free-form user key/value storage
 
 	// lxdmeta:generate(entities=instance; group=miscellaneous; key=agent.nic_config)
-	// For containers, the name and MTU of the default network interfaces is used for the instance devices.
-	// For virtual machines, set this option to `true` to set the name and MTU of the default network interfaces to be the same as the instance devices.
+	// When set to true, the name and MTU of the default network interfaces inside the virtual machine will match those of the instance devices.
 	// ---
 	//  type: bool
 	//  defaultdesc: `false`
@@ -1069,6 +1164,21 @@ func ConfigKeyChecker(key string, instanceType Type) (func(value string) error,
 		}
 	}
 
+	// lxdmeta:generate(entities=instance; group=cloud-init; key=cloud-init.ssh-keys.KEYNAME)
+	// Represents an additional SSH public key to be merged into existing `cloud-init` seed data
+	// and injected into an instance. Has the format `{user}:{key}`, where {user} is a Linux username and
+	// {key} can be either a pure SSH public key or an import ID for a key hosted elsewhere.
+	// // For example: `root:gh:githubUser`, `myUser:ssh-keyAlg publicKeyHash`
+	// ---
+	//  type: string
+	//  liveupdate: no
+	//  condition: If supported by image
+	//  shortdesc: Additional SSH key to be injected on the instance by `cloud-init`
+	sshKeyName := strings.TrimPrefix(key, "cloud-init.ssh-keys.")
+	if sshKeyName != key && sshKeyName != "" {
+		return validate.Optional(validate.IsUserSSHKey), nil
+	}
+
 	if strings.HasPrefix(key, ConfigVolatilePrefix) {
 		// lxdmeta:generate(entities=instance; group=volatile; key=volatile.<name>.last_state.hwaddr)
 		// The original MAC that was used when moving a physical device into an instance.
@@ -1158,15 +1268,6 @@ func ConfigKeyChecker(key string, instanceType Type) (func(value string) error,
 			return validate.IsAny, nil
 		}
 
-		// lxdmeta:generate(entities=instance; group=volatile; key=volatile.<name>.last_state.ip_addresses)
-		// Comma-separated list of the last used IP addresses of the network device.
-		// ---
-		//  type: string
-		//  shortdesc: Last used IP addresses
-		if strings.HasSuffix(key, ".last_state.ip_addresses") {
-			return validate.IsListOf(validate.IsNetworkAddress), nil
-		}
-
 		// lxdmeta:generate(entities=instance; group=volatile; key=volatile.<name>.apply_quota)
 		// The disk quota is applied the next time the instance starts.
 		// ---
@@ -1189,6 +1290,24 @@ func ConfigKeyChecker(key string, instanceType Type) (func(value string) error,
 			return validate.IsAny, nil
 		}
 
+		// lxdmeta:generate(entities=network-physical; group=volatile; key=volatile.last_state.usb.bus)
+		//
+		// --
+		//  type: string
+		//  shortdesc: USB Bus Number
+		if strings.HasSuffix(key, ".bus") {
+			return validate.IsAny, nil
+		}
+
+		// lxdmeta:generate(entities=network-physical; group=volatile; key=volatile.last_state.usb.device)
+		//
+		// --
+		//  type: string
+		//  shortdesc: USB Device Number
+		if strings.HasSuffix(key, ".device") {
+			return validate.IsAny, nil
+		}
+
 		if strings.HasSuffix(key, ".uuid") {
 			return validate.IsAny, nil
 		}
@@ -1198,29 +1317,16 @@ func ConfigKeyChecker(key string, instanceType Type) (func(value string) error,
 		}
 	}
 
-	if strings.HasPrefix(key, "environment.") {
+	if (instanceType == Any || instanceType == Container) && strings.HasPrefix(key, "linux.sysctl.") {
 		return validate.IsAny, nil
 	}
 
-	if strings.HasPrefix(key, "user.") {
+	knownPrefixes := append(ConfigKeyPrefixesAny, ConfigKeyPrefixesContainer...)
+	if shared.StringHasPrefix(key, knownPrefixes...) {
 		return validate.IsAny, nil
 	}
 
-	if strings.HasPrefix(key, "image.") {
-		return validate.IsAny, nil
-	}
-
-	if strings.HasPrefix(key, "limits.kernel.") &&
-		(len(key) > len("limits.kernel.")) {
-		return validate.IsAny, nil
-	}
-
-	if (instanceType == Any || instanceType == Container) &&
-		strings.HasPrefix(key, "linux.sysctl.") {
-		return validate.IsAny, nil
-	}
-
-	return nil, fmt.Errorf("Unknown configuration key: %s", key)
+	return nil, fmt.Errorf("Unknown configuration key: %q", key)
 }
 
 // InstanceIncludeWhenCopying is used to decide whether to include a config item or not when copying an instance.
