@@ -3,11 +3,9 @@ package main
 import (
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
-	"path/filepath"
 	"strings"
 	"time"
 
@@ -15,7 +13,6 @@ import (
 
 	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/backup"
-	"github.com/canonical/lxd/lxd/backup/config"
 	"github.com/canonical/lxd/lxd/db"
 	"github.com/canonical/lxd/lxd/db/operationtype"
 	"github.com/canonical/lxd/lxd/instance"
@@ -140,11 +137,11 @@ func instanceBackupsGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(cname) {
-		return response.BadRequest(errors.New("Invalid instance name"))
+		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
 	// Handle requests targeted to a container on a different node
-	resp, err := forwardedResponseIfInstanceIsRemote(r.Context(), s, projectName, cname, instanceType)
+	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, cname, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -176,7 +173,7 @@ func instanceBackupsGet(d *Daemon, r *http.Request) response.Response {
 		_, backupName, ok := strings.Cut(backup.Name(), "/")
 		if !ok {
 			// Not adding the name to the error response here because we were unable to check if the caller is allowed to view it.
-			return response.InternalError(errors.New("Instance backup has invalid name"))
+			return response.InternalError(fmt.Errorf("Instance backup has invalid name"))
 		}
 
 		if !canView(entity.InstanceBackupURL(projectName, c.Name(), backupName)) {
@@ -247,10 +244,10 @@ func instanceBackupsPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(name) {
-		return response.BadRequest(errors.New("Invalid instance name"))
+		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		err := limits.AllowBackupCreation(tx, projectName)
 		return err
 	})
@@ -259,7 +256,7 @@ func instanceBackupsPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Handle requests targeted to a container on a different node.
-	resp, err := forwardedResponseIfInstanceIsRemote(r.Context(), s, projectName, name, instanceType)
+	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -331,15 +328,6 @@ func instanceBackupsPost(d *Daemon, r *http.Request) response.Response {
 		req.Name = fmt.Sprintf("backup%d", backupNo)
 	}
 
-	// In case no version was selected for the backup format use the globally set format by default.
-	// This allows staying backwards compatible with older CLIs which don't yet support
-	// sending this field.
-	if req.Version == 0 {
-		req.Version = config.DefaultMetadataVersion
-	} else if req.Version > config.MaxMetadataVersion {
-		return response.BadRequest(fmt.Errorf("Invalid backup format version %d", req.Version))
-	}
-
 	// Validate the name.
 	backupName, err := backup.ValidateBackupName(req.Name)
 	if err != nil {
@@ -361,7 +349,7 @@ func instanceBackupsPost(d *Daemon, r *http.Request) response.Response {
 			CompressionAlgorithm: req.CompressionAlgorithm,
 		}
 
-		err := backupCreate(s, args, inst, req.Version, op)
+		err := backupCreate(s, args, inst, op)
 		if err != nil {
 			return fmt.Errorf("Create backup: %w", err)
 		}
@@ -378,8 +366,8 @@ func instanceBackupsPost(d *Daemon, r *http.Request) response.Response {
 
 	resources["backups"] = []api.URL{*api.NewURL().Path(version.APIVersion, "instances", name, "backups", backupName)}
 
-	op, err := operations.OperationCreate(r.Context(), s, projectName, operations.OperationClassTask,
-		operationtype.BackupCreate, resources, nil, backup, nil, nil)
+	op, err := operations.OperationCreate(s, projectName, operations.OperationClassTask,
+		operationtype.BackupCreate, resources, nil, backup, nil, nil, r)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -442,7 +430,7 @@ func instanceBackupGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(name) {
-		return response.BadRequest(errors.New("Invalid instance name"))
+		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
 	backupName, err := url.PathUnescape(mux.Vars(r)["backupName"])
@@ -451,7 +439,7 @@ func instanceBackupGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Handle requests targeted to a container on a different node
-	resp, err := forwardedResponseIfInstanceIsRemote(r.Context(), s, projectName, name, instanceType)
+	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -516,7 +504,7 @@ func instanceBackupPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(name) {
-		return response.BadRequest(errors.New("Invalid instance name"))
+		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
 	backupName, err := url.PathUnescape(mux.Vars(r)["backupName"])
@@ -525,7 +513,7 @@ func instanceBackupPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Handle requests targeted to a container on a different node
-	resp, err := forwardedResponseIfInstanceIsRemote(r.Context(), s, projectName, name, instanceType)
+	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -572,8 +560,8 @@ func instanceBackupPost(d *Daemon, r *http.Request) response.Response {
 		resources["containers"] = resources["instances"]
 	}
 
-	op, err := operations.OperationCreate(r.Context(), s, projectName, operations.OperationClassTask,
-		operationtype.BackupRename, resources, nil, rename, nil, nil)
+	op, err := operations.OperationCreate(s, projectName, operations.OperationClassTask,
+		operationtype.BackupRename, resources, nil, rename, nil, nil, r)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -622,7 +610,7 @@ func instanceBackupDelete(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(name) {
-		return response.BadRequest(errors.New("Invalid instance name"))
+		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
 	backupName, err := url.PathUnescape(mux.Vars(r)["backupName"])
@@ -631,7 +619,7 @@ func instanceBackupDelete(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Handle requests targeted to a container on a different node
-	resp, err := forwardedResponseIfInstanceIsRemote(r.Context(), s, projectName, name, instanceType)
+	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -661,8 +649,8 @@ func instanceBackupDelete(d *Daemon, r *http.Request) response.Response {
 		resources["containers"] = resources["instances"]
 	}
 
-	op, err := operations.OperationCreate(r.Context(), s, projectName, operations.OperationClassTask,
-		operationtype.BackupRemove, resources, nil, remove, nil, nil)
+	op, err := operations.OperationCreate(s, projectName, operations.OperationClassTask,
+		operationtype.BackupRemove, resources, nil, remove, nil, nil, r)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -707,7 +695,7 @@ func instanceBackupExportGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(name) {
-		return response.BadRequest(errors.New("Invalid instance name"))
+		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
 	backupName, err := url.PathUnescape(mux.Vars(r)["backupName"])
@@ -716,7 +704,7 @@ func instanceBackupExportGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Handle requests targeted to a container on a different node
-	resp, err := forwardedResponseIfInstanceIsRemote(r.Context(), s, projectName, name, instanceType)
+	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -732,7 +720,7 @@ func instanceBackupExportGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	ent := response.FileResponseEntry{
-		Path: filepath.Join(d.State().BackupsStoragePath(), "instances", project.Instance(projectName, backup.Name())),
+		Path: shared.VarPath("backups", "instances", project.Instance(projectName, backup.Name())),
 	}
 
 	s.Events.SendLifecycle(projectName, lifecycle.InstanceBackupRetrieved.Event(fullName, backup.Instance(), nil))
