@@ -3,13 +3,11 @@ package main
 import (
 	"bytes"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"io"
 	"net/http"
 	"net/url"
 	"os"
-	"slices"
 	"strconv"
 	"sync"
 	"time"
@@ -81,25 +79,20 @@ func (s *consoleWs) Metadata() any {
 
 // Connect connects to the websocket.
 func (s *consoleWs) Connect(op *operations.Operation, r *http.Request, w http.ResponseWriter) error {
-	err := op.CheckRequestor(r)
-	if err != nil {
-		return err
-	}
-
 	switch s.protocol {
 	case instance.ConsoleTypeConsole:
-		return s.connectConsole(r, w)
+		return s.connectConsole(op, r, w)
 	case instance.ConsoleTypeVGA:
-		return s.connectVGA(r, w)
+		return s.connectVGA(op, r, w)
 	default:
 		return fmt.Errorf("Unknown protocol %q", s.protocol)
 	}
 }
 
-func (s *consoleWs) connectConsole(r *http.Request, w http.ResponseWriter) error {
+func (s *consoleWs) connectConsole(op *operations.Operation, r *http.Request, w http.ResponseWriter) error {
 	secret := r.FormValue("secret")
 	if secret == "" {
-		return errors.New("missing secret")
+		return fmt.Errorf("missing secret")
 	}
 
 	for fd, fdSecret := range s.fds {
@@ -109,8 +102,6 @@ func (s *consoleWs) connectConsole(r *http.Request, w http.ResponseWriter) error
 				return err
 			}
 
-			ws.StartKeepAlive(conn)
-
 			s.connsLock.Lock()
 			s.conns[fd] = conn
 			s.connsLock.Unlock()
@@ -139,10 +130,10 @@ func (s *consoleWs) connectConsole(r *http.Request, w http.ResponseWriter) error
 	return os.ErrPermission
 }
 
-func (s *consoleWs) connectVGA(r *http.Request, w http.ResponseWriter) error {
+func (s *consoleWs) connectVGA(op *operations.Operation, r *http.Request, w http.ResponseWriter) error {
 	secret := r.FormValue("secret")
 	if secret == "" {
-		return errors.New("missing secret")
+		return fmt.Errorf("missing secret")
 	}
 
 	for fd, fdSecret := range s.fds {
@@ -155,8 +146,6 @@ func (s *consoleWs) connectVGA(r *http.Request, w http.ResponseWriter) error {
 			return err
 		}
 
-		ws.StartKeepAlive(conn)
-
 		if fd == -1 {
 			logger.Debug("VGA control websocket connected")
 
@@ -203,18 +192,18 @@ func (s *consoleWs) connectVGA(r *http.Request, w http.ResponseWriter) error {
 }
 
 // Do connects to the websocket and executes the operation.
-func (s *consoleWs) Do(_ *operations.Operation) error {
+func (s *consoleWs) Do(op *operations.Operation) error {
 	switch s.protocol {
 	case instance.ConsoleTypeConsole:
-		return s.doConsole()
+		return s.doConsole(op)
 	case instance.ConsoleTypeVGA:
-		return s.doVGA()
+		return s.doVGA(op)
 	default:
 		return fmt.Errorf("Unknown protocol %q", s.protocol)
 	}
 }
 
-func (s *consoleWs) doConsole() error {
+func (s *consoleWs) doConsole(op *operations.Operation) error {
 	defer logger.Debug("Console websocket finished")
 	<-s.allConnected
 
@@ -235,7 +224,7 @@ func (s *consoleWs) doConsole() error {
 
 	// Wait for control socket to connect and then read messages from the remote side in a loop.
 	go func() {
-		defer logger.Debug("Console control websocket finished")
+		defer logger.Debugf("Console control websocket finished")
 		res := <-s.controlConnected
 		if !res {
 			return
@@ -349,14 +338,14 @@ func (s *consoleWs) doConsole() error {
 	return nil
 }
 
-func (s *consoleWs) doVGA() error {
+func (s *consoleWs) doVGA(op *operations.Operation) error {
 	defer logger.Debug("VGA websocket finished")
 
 	consoleDoneCh := make(chan struct{})
 
 	// The control socket is used to terminate the operation.
 	go func() {
-		defer logger.Debug("VGA control websocket finished")
+		defer logger.Debugf("VGA control websocket finished")
 		res := <-s.controlConnected
 		if !res {
 			return
@@ -443,7 +432,7 @@ func instanceConsolePost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(name) {
-		return response.BadRequest(errors.New("Invalid instance name"))
+		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
 	post := api.InstanceConsolePost{}
@@ -458,14 +447,14 @@ func instanceConsolePost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Forward the request if the container is remote.
-	client, err := cluster.ConnectIfInstanceIsRemote(r.Context(), s, projectName, name, instanceType)
+	client, err := cluster.ConnectIfInstanceIsRemote(s, projectName, name, r, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	if client != nil {
 		url := api.NewURL().Path(version.APIVersion, "instances", name, "console").Project(projectName)
-		resp, _, err := client.RawQuery(http.MethodPost, url.String(), post, "")
+		resp, _, err := client.RawQuery("POST", url.String(), post, "")
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -483,7 +472,7 @@ func instanceConsolePost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Basic parameter validation.
-	if !slices.Contains([]string{instance.ConsoleTypeConsole, instance.ConsoleTypeVGA}, post.Type) {
+	if !shared.ValueInSlice(post.Type, []string{instance.ConsoleTypeConsole, instance.ConsoleTypeVGA}) {
 		return response.BadRequest(fmt.Errorf("Unknown console type %q", post.Type))
 	}
 
@@ -493,15 +482,15 @@ func instanceConsolePost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if post.Type == instance.ConsoleTypeVGA && inst.Type() != instancetype.VM {
-		return response.BadRequest(errors.New("VGA console is only supported by virtual machines"))
+		return response.BadRequest(fmt.Errorf("VGA console is only supported by virtual machines"))
 	}
 
 	if !inst.IsRunning() {
-		return response.BadRequest(errors.New("Instance is not running"))
+		return response.BadRequest(fmt.Errorf("Instance is not running"))
 	}
 
 	if inst.IsFrozen() {
-		return response.BadRequest(errors.New("Instance is frozen"))
+		return response.BadRequest(fmt.Errorf("Instance is frozen"))
 	}
 
 	ws := &consoleWs{}
@@ -531,7 +520,7 @@ func instanceConsolePost(d *Daemon, r *http.Request) response.Response {
 		resources["containers"] = resources["instances"]
 	}
 
-	op, err := operations.OperationCreate(r.Context(), s, projectName, operations.OperationClassWebsocket, operationtype.ConsoleShow, resources, ws.Metadata(), ws.Do, nil, ws.Connect)
+	op, err := operations.OperationCreate(s, projectName, operations.OperationClassWebsocket, operationtype.ConsoleShow, resources, ws.Metadata(), ws.Do, nil, ws.Connect, r)
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -585,11 +574,11 @@ func instanceConsoleLogGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(name) {
-		return response.BadRequest(errors.New("Invalid instance name"))
+		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
 	// Forward the request if the container is remote.
-	resp, err := forwardedResponseIfInstanceIsRemote(r.Context(), s, projectName, name, instanceType)
+	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -604,12 +593,12 @@ func instanceConsoleLogGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if inst.Type() != instancetype.Container {
-		return response.SmartError(errors.New("Instance is not container type"))
+		return response.SmartError(fmt.Errorf("Instance is not container type"))
 	}
 
 	c, ok := inst.(instance.Container)
 	if !ok {
-		return response.SmartError(errors.New("Invalid instance type"))
+		return response.SmartError(fmt.Errorf("Invalid instance type"))
 	}
 
 	ent := response.FileResponseEntry{}
@@ -684,7 +673,7 @@ func instanceConsoleLogDelete(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(name) {
-		return response.BadRequest(errors.New("Invalid instance name"))
+		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
 	projectName := request.ProjectParam(r)
@@ -695,12 +684,12 @@ func instanceConsoleLogDelete(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if inst.Type() != instancetype.Container {
-		return response.SmartError(errors.New("Instance is not container type"))
+		return response.SmartError(fmt.Errorf("Instance is not container type"))
 	}
 
 	c, ok := inst.(instance.Container)
 	if !ok {
-		return response.SmartError(errors.New("Invalid instance type"))
+		return response.SmartError(fmt.Errorf("Invalid instance type"))
 	}
 
 	truncateConsoleLogFile := func(path string) error {
@@ -712,11 +701,11 @@ func instanceConsoleLogDelete(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if !st.Mode().IsRegular() {
-			return errors.New("The console log is not a regular file")
+			return fmt.Errorf("The console log is not a regular file")
 		}
 
 		if path == "" {
-			return errors.New("Container does not keep a console logfile")
+			return fmt.Errorf("Container does not keep a console logfile")
 		}
 
 		return os.Truncate(path, 0)
