@@ -33,6 +33,7 @@ import (
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/cancel"
 	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/tcp"
 	"github.com/canonical/lxd/shared/version"
 	"github.com/canonical/lxd/shared/ws"
 )
@@ -48,8 +49,8 @@ type execWs struct {
 	instance              instance.Instance
 	conns                 map[int]*websocket.Conn
 	connsLock             sync.Mutex
-	waitRequiredConnected cancel.Canceller
-	waitControlConnected  cancel.Canceller
+	waitRequiredConnected *cancel.Canceller
+	waitControlConnected  *cancel.Canceller
 	fds                   map[int]string
 	s                     *state.State
 }
@@ -77,12 +78,7 @@ func (s *execWs) Metadata() any {
 func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.ResponseWriter) error {
 	secret := r.FormValue("secret")
 	if secret == "" {
-		return errors.New("missing secret")
-	}
-
-	err := op.CheckRequestor(r)
-	if err != nil {
-		return err
+		return fmt.Errorf("missing secret")
 	}
 
 	for fd, fdSecret := range s.fds {
@@ -98,7 +94,30 @@ func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.Respo
 			if found && val == nil {
 				s.conns[fd] = conn
 
-				ws.StartKeepAlive(conn)
+				// Set TCP timeout options.
+				remoteTCP, _ := tcp.ExtractConn(conn.UnderlyingConn())
+				if remoteTCP != nil {
+					err = tcp.SetTimeouts(remoteTCP, 0)
+					if err != nil {
+						logger.Warn("Failed setting TCP timeouts on remote connection", logger.Ctx{"err": err})
+					}
+
+					// Start channel keep alive to run until channel is closed.
+					go func() {
+						pingInterval := time.Second * 10
+						t := time.NewTicker(pingInterval)
+						defer t.Stop()
+
+						for {
+							err := conn.WriteControl(websocket.PingMessage, []byte("keepalive"), time.Now().Add(5*time.Second))
+							if err != nil {
+								return
+							}
+
+							<-t.C
+						}
+					}()
+				}
 
 				if fd == execWSControl {
 					s.waitControlConnected.Cancel() // Control connection connected.
@@ -125,10 +144,10 @@ func (s *execWs) Connect(op *operations.Operation, r *http.Request, w http.Respo
 				s.connsLock.Unlock()
 				return nil
 			} else if !found {
-				return errors.New("Unknown websocket number")
+				return fmt.Errorf("Unknown websocket number")
 			}
 
-			return errors.New("Websocket number already connected")
+			return fmt.Errorf("Websocket number already connected")
 		}
 	}
 
@@ -156,7 +175,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 	select {
 	case <-s.waitRequiredConnected.Done():
 	case <-time.After(time.Second * 5):
-		return errors.New("Timed out waiting for websockets to connect")
+		return fmt.Errorf("Timed out waiting for websockets to connect")
 	}
 
 	var err error
@@ -178,7 +197,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 
 			c, ok := s.instance.(instance.Container)
 			if !ok {
-				return errors.New("Invalid instance type")
+				return fmt.Errorf("Invalid instance type")
 			}
 
 			idmapset, err := c.CurrentIdmap()
@@ -215,7 +234,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 			// For VMs we rely on the lxd-agent PTY running inside the VM guest.
 			ttys = make([]*os.File, 2)
 			ptys = make([]*os.File, 2)
-			for i := range ttys {
+			for i := 0; i < len(ttys); i++ {
 				ptys[i], ttys[i], err = os.Pipe()
 				if err != nil {
 					return err
@@ -228,7 +247,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 	} else {
 		ttys = make([]*os.File, 3)
 		ptys = make([]*os.File, 3)
-		for i := range ttys {
+		for i := 0; i < len(ttys); i++ {
 			ptys[i], ttys[i], err = os.Pipe()
 			if err != nil {
 				return err
@@ -295,14 +314,15 @@ func (s *execWs) Do(op *operations.Operation) error {
 	l := logger.AddContext(logger.Ctx{"project": s.instance.Project().Name, "instance": s.instance.Name(), "PID": cmd.PID(), "interactive": s.req.Interactive})
 	l.Debug("Instance process started")
 
-	cmdKill := sync.OnceFunc(func() {
+	var cmdKillOnce sync.Once
+	cmdKill := func() {
 		err := cmd.Signal(unix.SIGKILL)
 		if err != nil {
 			l.Debug("Failed to send SIGKILL signal", logger.Ctx{"err": err})
 		} else {
 			l.Debug("Sent SIGKILL signal")
 		}
-	})
+	}
 
 	// Now that process has started, we can start the control handler.
 	wgEOF.Add(1)
@@ -336,7 +356,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 					l.Warn("Failed getting exec control websocket reader, killing command", logger.Ctx{"err": err})
 				}
 
-				cmdKill()
+				cmdKillOnce.Do(cmdKill)
 
 				return
 			}
@@ -350,7 +370,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 
 				l.Warn("Failed reading control websocket message, killing command", logger.Ctx{"err": err})
 
-				cmdKill()
+				cmdKillOnce.Do(cmdKill)
 
 				return
 			}
@@ -424,7 +444,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 		}()
 	} else {
 		wgEOF.Add(len(ttys) - 1)
-		for i := range ttys {
+		for i := 0; i < len(ttys); i++ {
 			go func(i int) {
 				var err error
 				l.Debug("Exec mirror websocket started", logger.Ctx{"number": i})
@@ -444,7 +464,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 					// detect when the client disconnects to avoid leaving the command running
 					// in the background.
 					go func() {
-						_, _, err := conn.ReadMessage() // Consume pings from server.
+						_, _, err := conn.ReadMessage()
 
 						// If there is a control connection, then leave it to that handler
 						// to clean the command up. If there's no control connection, the
@@ -454,7 +474,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 						// then it is our responsibility to kill the command now.
 						if s.waitControlConnected.Err() == nil {
 							l.Warn("Unexpected read on stdout websocket, killing command", logger.Ctx{"number": i, "err": err})
-							cmdKill()
+							cmdKillOnce.Do(cmdKill)
 						}
 					}()
 				}
@@ -464,7 +484,7 @@ func (s *execWs) Do(op *operations.Operation) error {
 					// avoid a situation where we hit an inactivity timeout on
 					// stderr during long exec sessions
 					go func() {
-						_, _, _ = conn.ReadMessage() // Consume pings from server.
+						_, _, _ = conn.ReadMessage()
 					}()
 				}
 
@@ -539,7 +559,7 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if shared.IsSnapshot(name) {
-		return response.BadRequest(errors.New("Invalid instance name"))
+		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
 	post := api.InstanceExecPost{}
@@ -563,14 +583,14 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Forward the request if the container is remote.
-	client, err := cluster.ConnectIfInstanceIsRemote(r.Context(), s, projectName, name, instanceType)
+	client, err := cluster.ConnectIfInstanceIsRemote(s, projectName, name, r, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	if client != nil {
 		url := api.NewURL().Path(version.APIVersion, "instances", name, "exec").Project(projectName)
-		resp, _, err := client.RawQuery(http.MethodPost, url.String(), post, "")
+		resp, _, err := client.RawQuery("POST", url.String(), post, "")
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -589,11 +609,11 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if !inst.IsRunning() {
-		return response.BadRequest(errors.New("Instance is not running"))
+		return response.BadRequest(fmt.Errorf("Instance is not running"))
 	}
 
 	if inst.IsFrozen() {
-		return response.BadRequest(errors.New("Instance is frozen"))
+		return response.BadRequest(fmt.Errorf("Instance is frozen"))
 	}
 
 	// Process environment.
@@ -670,8 +690,8 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 			ws.conns[execWSStderr] = nil
 		}
 
-		ws.waitRequiredConnected = cancel.New()
-		ws.waitControlConnected = cancel.New()
+		ws.waitRequiredConnected = cancel.New(context.Background())
+		ws.waitControlConnected = cancel.New(context.Background())
 
 		for i := range ws.conns {
 			ws.fds[i], err = shared.RandomCryptoString()
@@ -690,7 +710,7 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 			resources["containers"] = resources["instances"]
 		}
 
-		op, err := operations.OperationCreate(r.Context(), s, projectName, operations.OperationClassWebsocket, operationtype.CommandExec, resources, ws.Metadata(), ws.Do, nil, ws.Connect)
+		op, err := operations.OperationCreate(s, projectName, operations.OperationClassWebsocket, operationtype.CommandExec, resources, ws.Metadata(), ws.Do, nil, ws.Connect, r)
 		if err != nil {
 			return response.InternalError(err)
 		}
@@ -766,7 +786,7 @@ func instanceExecPost(d *Daemon, r *http.Request) response.Response {
 		resources["containers"] = resources["instances"]
 	}
 
-	op, err := operations.OperationCreate(r.Context(), s, projectName, operations.OperationClassTask, operationtype.CommandExec, resources, nil, run, nil, nil)
+	op, err := operations.OperationCreate(s, projectName, operations.OperationClassTask, operationtype.CommandExec, resources, nil, run, nil, nil, r)
 	if err != nil {
 		return response.InternalError(err)
 	}
