@@ -13,7 +13,6 @@ import (
 	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/instance"
 	"github.com/canonical/lxd/lxd/lifecycle"
-	"github.com/canonical/lxd/lxd/project"
 	"github.com/canonical/lxd/lxd/request"
 	"github.com/canonical/lxd/lxd/response"
 	"github.com/canonical/lxd/lxd/storage"
@@ -24,8 +23,9 @@ import (
 )
 
 var instanceLogCmd = APIEndpoint{
-	Name: "instanceLog",
-	Path: "instances/{name}/logs/{file}",
+	Name:        "instanceLog",
+	Path:        "instances/{name}/logs/{file}",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerLog", Path: "containers/{name}/logs/{file}"},
 		{Name: "vmLog", Path: "virtual-machines/{name}/logs/{file}"},
@@ -36,8 +36,9 @@ var instanceLogCmd = APIEndpoint{
 }
 
 var instanceLogsCmd = APIEndpoint{
-	Name: "instanceLogs",
-	Path: "instances/{name}/logs",
+	Name:        "instanceLogs",
+	Path:        "instances/{name}/logs",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerLogs", Path: "containers/{name}/logs"},
 		{Name: "vmLogs", Path: "virtual-machines/{name}/logs"},
@@ -47,8 +48,9 @@ var instanceLogsCmd = APIEndpoint{
 }
 
 var instanceExecOutputCmd = APIEndpoint{
-	Name: "instanceExecOutput",
-	Path: "instances/{name}/logs/exec-output/{file}",
+	Name:        "instanceExecOutput",
+	Path:        "instances/{name}/logs/exec-output/{file}",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerExecOutput", Path: "containers/{name}/logs/exec-output/{file}"},
 		{Name: "vmExecOutput", Path: "virtual-machines/{name}/logs/exec-output/{file}"},
@@ -59,8 +61,9 @@ var instanceExecOutputCmd = APIEndpoint{
 }
 
 var instanceExecOutputsCmd = APIEndpoint{
-	Name: "instanceExecOutputs",
-	Path: "instances/{name}/logs/exec-output",
+	Name:        "instanceExecOutputs",
+	Path:        "instances/{name}/logs/exec-output",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerExecOutputs", Path: "containers/{name}/logs/exec-output"},
 		{Name: "vmExecOutputs", Path: "virtual-machines/{name}/logs/exec-output"},
@@ -143,8 +146,10 @@ func instanceLogsGet(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
+	s := d.State()
+
 	// Handle requests targeted to a container on a different node
-	resp, err := forwardedResponseIfInstanceIsRemote(d.State(), r, projectName, name, instanceType)
+	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -153,15 +158,15 @@ func instanceLogsGet(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
-	err = instance.ValidName(name, false)
+	// Ensure instance exists.
+	inst, err := instance.LoadByProjectAndName(s, projectName, name)
 	if err != nil {
-		return response.BadRequest(err)
+		return response.SmartError(err)
 	}
 
 	result := []string{}
 
-	fullName := project.Instance(projectName, name)
-	dents, err := os.ReadDir(shared.LogPath(fullName))
+	dents, err := os.ReadDir(inst.LogPath())
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -171,7 +176,7 @@ func instanceLogsGet(d *Daemon, r *http.Request) response.Response {
 			continue
 		}
 
-		result = append(result, fmt.Sprintf("/%s/instances/%s/logs/%s", version.APIVersion, name, f.Name()))
+		result = append(result, "/"+version.APIVersion+"/instances/"+name+"/logs/"+f.Name())
 	}
 
 	return response.SyncResponse(true, result)
@@ -227,12 +232,6 @@ func instanceLogGet(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
-	// Ensure instance exists.
-	inst, err := instance.LoadByProjectAndName(s, projectName, name)
-	if err != nil {
-		return response.SmartError(err)
-	}
-
 	// Handle requests targeted to a container on a different node
 	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
@@ -243,14 +242,15 @@ func instanceLogGet(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
-	file, err := url.PathUnescape(mux.Vars(r)["file"])
+	// Ensure instance exists.
+	inst, err := instance.LoadByProjectAndName(s, projectName, name)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	err = instance.ValidName(name, false)
+	file, err := url.PathUnescape(mux.Vars(r)["file"])
 	if err != nil {
-		return response.BadRequest(err)
+		return response.SmartError(err)
 	}
 
 	if !validLogFileName(file) {
@@ -258,13 +258,13 @@ func instanceLogGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	ent := response.FileResponseEntry{
-		Path:     shared.LogPath(project.Instance(projectName, name), file),
+		Path:     filepath.Join(inst.LogPath(), file),
 		Filename: file,
 	}
 
 	s.Events.SendLifecycle(projectName, lifecycle.InstanceLogRetrieved.Event(file, inst, request.CreateRequestor(r), nil))
 
-	return response.FileResponse(r, []response.FileResponseEntry{ent}, nil)
+	return response.FileResponse([]response.FileResponseEntry{ent}, nil)
 }
 
 // swagger:operation DELETE /1.0/instances/{name}/logs/{filename} instances instance_log_delete
@@ -311,12 +311,6 @@ func instanceLogDelete(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
-	// Ensure instance exists.
-	inst, err := instance.LoadByProjectAndName(s, projectName, name)
-	if err != nil {
-		return response.SmartError(err)
-	}
-
 	// Handle requests targeted to a container on a different node
 	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
@@ -327,14 +321,15 @@ func instanceLogDelete(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
-	file, err := url.PathUnescape(mux.Vars(r)["file"])
+	// Ensure instance exists.
+	inst, err := instance.LoadByProjectAndName(s, projectName, name)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	err = instance.ValidName(name, false)
+	file, err := url.PathUnescape(mux.Vars(r)["file"])
 	if err != nil {
-		return response.BadRequest(err)
+		return response.SmartError(err)
 	}
 
 	if !validLogFileName(file) {
@@ -345,7 +340,7 @@ func instanceLogDelete(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(fmt.Errorf("Only log files excluding qemu.log and lxc.log may be deleted"))
 	}
 
-	err = os.Remove(shared.LogPath(project.Instance(projectName, name), file))
+	err = os.Remove(filepath.Join(inst.LogPath(), file))
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -355,7 +350,7 @@ func instanceLogDelete(d *Daemon, r *http.Request) response.Response {
 	return response.EmptySyncResponse
 }
 
-// swagger:operation GET /1.0/instances/{name}/logs/exec-output instances instance_exec-outputs_get
+// swagger:operation GET /1.0/instances/{name}/logs/exec-output instances instance_exec-output_get
 //
 //	Get the exec record-output files
 //
@@ -423,12 +418,6 @@ func instanceExecOutputsGet(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
-	// Ensure instance exists.
-	inst, err := instance.LoadByProjectAndName(s, projectName, name)
-	if err != nil {
-		return response.SmartError(err)
-	}
-
 	// Handle requests targeted to a container on a different node
 	resp, err := forwardedResponseIfInstanceIsRemote(d.State(), r, projectName, name, instanceType)
 	if err != nil {
@@ -439,9 +428,10 @@ func instanceExecOutputsGet(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
-	err = instance.ValidName(name, false)
+	// Ensure instance exists.
+	inst, err := instance.LoadByProjectAndName(s, projectName, name)
 	if err != nil {
-		return response.BadRequest(err)
+		return response.SmartError(err)
 	}
 
 	// Mount the instance's root volume
@@ -469,7 +459,7 @@ func instanceExecOutputsGet(d *Daemon, r *http.Request) response.Response {
 			continue
 		}
 
-		result = append(result, fmt.Sprintf("/%s/instances/%s/logs/exec-output/%s", version.APIVersion, name, f.Name()))
+		result = append(result, "/"+version.APIVersion+"/instances/"+name+"/logs/exec-output/"+f.Name())
 	}
 
 	return response.SyncResponse(true, result)
@@ -528,12 +518,6 @@ func instanceExecOutputGet(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
-	// Ensure instance exists.
-	inst, err := instance.LoadByProjectAndName(s, projectName, name)
-	if err != nil {
-		return response.SmartError(err)
-	}
-
 	// Handle requests targeted to a container on a different node
 	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
@@ -544,14 +528,15 @@ func instanceExecOutputGet(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
-	file, err := url.PathUnescape(mux.Vars(r)["file"])
+	// Ensure instance exists.
+	inst, err := instance.LoadByProjectAndName(s, projectName, name)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	err = instance.ValidName(name, false)
+	file, err := url.PathUnescape(mux.Vars(r)["file"])
 	if err != nil {
-		return response.BadRequest(err)
+		return response.SmartError(err)
 	}
 
 	if !validExecOutputFileName(file) {
@@ -581,7 +566,7 @@ func instanceExecOutputGet(d *Daemon, r *http.Request) response.Response {
 
 	s.Events.SendLifecycle(projectName, lifecycle.InstanceLogRetrieved.Event(file, inst, request.CreateRequestor(r), nil))
 
-	return response.FileResponse(r, []response.FileResponseEntry{ent}, nil)
+	return response.FileResponse([]response.FileResponseEntry{ent}, nil)
 }
 
 // swagger:operation DELETE /1.0/instances/{name}/logs/exec-output/{filename} instances instance_exec-output_delete
@@ -628,12 +613,6 @@ func instanceExecOutputDelete(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(fmt.Errorf("Invalid instance name"))
 	}
 
-	// Ensure instance exists.
-	inst, err := instance.LoadByProjectAndName(s, projectName, name)
-	if err != nil {
-		return response.SmartError(err)
-	}
-
 	// Handle requests targeted to a container on a different node
 	resp, err := forwardedResponseIfInstanceIsRemote(s, r, projectName, name, instanceType)
 	if err != nil {
@@ -644,14 +623,15 @@ func instanceExecOutputDelete(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
-	file, err := url.PathUnescape(mux.Vars(r)["file"])
+	// Ensure instance exists.
+	inst, err := instance.LoadByProjectAndName(s, projectName, name)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	err = instance.ValidName(name, false)
+	file, err := url.PathUnescape(mux.Vars(r)["file"])
 	if err != nil {
-		return response.BadRequest(err)
+		return response.SmartError(err)
 	}
 
 	if !validExecOutputFileName(file) {
@@ -682,6 +662,10 @@ func instanceExecOutputDelete(d *Daemon, r *http.Request) response.Response {
 }
 
 func validLogFileName(fname string) bool {
+	if !shared.IsFileName(fname) {
+		return false
+	}
+
 	/* Let's just require that the paths be relative, so that we don't have
 	 * to deal with any escaping or whatever.
 	 */
@@ -694,6 +678,10 @@ func validLogFileName(fname string) bool {
 }
 
 func validExecOutputFileName(fName string) bool {
+	if !shared.IsFileName(fName) {
+		return false
+	}
+
 	return (strings.HasSuffix(fName, ".stdout") || strings.HasSuffix(fName, ".stderr")) &&
 		strings.HasPrefix(fName, "exec_")
 }
