@@ -8,7 +8,6 @@ import (
 	"net/url"
 	"os"
 	"path/filepath"
-	"strings"
 
 	"github.com/gorilla/mux"
 	"gopkg.in/yaml.v2"
@@ -472,12 +471,12 @@ func instanceMetadataTemplatesGet(d *Daemon, r *http.Request) response.Response
 	templateName := r.FormValue("path")
 	if templateName == "" {
 		templates := []string{}
-		if !shared.PathExists(filepath.Join(c.Path(), "templates")) {
+		templatesPath := c.TemplatesPath()
+		if !shared.PathExists(templatesPath) {
 			return response.SyncResponse(true, templates)
 		}
 
 		// List templates
-		templatesPath := filepath.Join(c.Path(), "templates")
 		entries, err := os.ReadDir(templatesPath)
 		if err != nil {
 			return response.InternalError(err)
@@ -534,7 +533,7 @@ func instanceMetadataTemplatesGet(d *Daemon, r *http.Request) response.Response
 
 	s.Events.SendLifecycle(projectName, lifecycle.InstanceMetadataTemplateRetrieved.Event(c, request.CreateRequestor(r), logger.Ctx{"path": templateName}))
 
-	return response.FileResponse(r, files, nil)
+	return response.FileResponse(files, nil)
 }
 
 // swagger:operation POST /1.0/instances/{name}/metadata/templates instances instance_metadata_templates_post
@@ -626,8 +625,9 @@ func instanceMetadataTemplatesPost(d *Daemon, r *http.Request) response.Response
 		return response.BadRequest(fmt.Errorf("missing path argument"))
 	}
 
-	if !shared.PathExists(filepath.Join(c.Path(), "templates")) {
-		err := os.MkdirAll(filepath.Join(c.Path(), "templates"), 0711)
+	templatesPath := c.TemplatesPath()
+	if !shared.PathExists(templatesPath) {
+		err := os.MkdirAll(templatesPath, 0711)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -767,9 +767,9 @@ func instanceMetadataTemplatesDelete(d *Daemon, r *http.Request) response.Respon
 
 // Return the full path of a container template.
 func getContainerTemplatePath(c instance.Instance, filename string) (string, error) {
-	if strings.Contains(filename, "/") {
-		return "", fmt.Errorf("Invalid template filename")
+	if !shared.IsFileName(filename) {
+		return "", fmt.Errorf("Invalid template filename: %q", filename)
 	}
 
-	return filepath.Join(c.Path(), "templates", filename), nil
+	return filepath.Join(c.TemplatesPath(), filename), nil
 }
