@@ -12,13 +12,14 @@ import (
 
 	"github.com/gorilla/mux"
 
+	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/db"
 	"github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/db/operationtype"
 	"github.com/canonical/lxd/lxd/instance"
 	"github.com/canonical/lxd/lxd/instance/instancetype"
 	"github.com/canonical/lxd/lxd/operations"
-	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/project/limits"
 	"github.com/canonical/lxd/lxd/request"
 	"github.com/canonical/lxd/lxd/response"
 	"github.com/canonical/lxd/lxd/state"
@@ -26,6 +27,7 @@ import (
 	"github.com/canonical/lxd/lxd/util"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/entity"
 	"github.com/canonical/lxd/shared/validate"
 	"github.com/canonical/lxd/shared/version"
 )
@@ -150,6 +152,11 @@ func instanceSnapshotsGet(d *Daemon, r *http.Request) response.Response {
 		return resp
 	}
 
+	canView, err := s.Authorizer.GetPermissionChecker(r.Context(), auth.EntitlementCanView, entity.TypeInstanceSnapshot)
+	if err != nil {
+		return response.SmartError(err)
+	}
+
 	recursion := util.IsRecursionRequest(r)
 	resultString := []string{}
 	resultMap := []*api.InstanceSnapshot{}
@@ -170,6 +177,11 @@ func instanceSnapshotsGet(d *Daemon, r *http.Request) response.Response {
 
 		for _, snap := range snaps {
 			_, snapName, _ := api.GetParentAndSnapshotName(snap)
+
+			if !canView(entity.InstanceSnapshotURL(projectName, cname, snapName)) {
+				continue
+			}
+
 			if projectName == api.ProjectDefaultName {
 				url := fmt.Sprintf("/%s/instances/%s/snapshots/%s", version.APIVersion, cname, snapName)
 				resultString = append(resultString, url)
@@ -190,12 +202,23 @@ func instanceSnapshotsGet(d *Daemon, r *http.Request) response.Response {
 		}
 
 		for _, snap := range snaps {
+			_, snapName, _ := api.GetParentAndSnapshotName(snap.Name())
+
+			if !canView(entity.InstanceSnapshotURL(projectName, cname, snapName)) {
+				continue
+			}
+
 			render, _, err := snap.Render(storagePools.RenderSnapshotUsage(s, snap))
 			if err != nil {
 				continue
 			}
 
-			resultMap = append(resultMap, render.(*api.InstanceSnapshot))
+			renderedSnap, ok := render.(*api.InstanceSnapshot)
+			if !ok {
+				return response.InternalError(fmt.Errorf("Render didn't return a snapshot"))
+			}
+
+			resultMap = append(resultMap, renderedSnap)
 		}
 	}
 
@@ -267,7 +290,7 @@ func instanceSnapshotsPost(d *Daemon, r *http.Request) response.Response {
 			return err
 		}
 
-		err = project.AllowSnapshotCreation(p)
+		err = limits.AllowSnapshotCreation(p)
 		if err != nil {
 			return err
 		}
@@ -389,7 +412,7 @@ func instanceSnapshotHandler(d *Daemon, r *http.Request) response.Response {
 
 	switch r.Method {
 	case "GET":
-		return snapshotGet(s, snapInst)
+		return snapshotGet(s, r, snapInst)
 	case "POST":
 		return snapshotPost(s, r, snapInst)
 	case "DELETE":
@@ -591,14 +614,19 @@ func snapshotPut(s *state.State, r *http.Request, snapInst instance.Instance) re
 //	    $ref: "#/responses/Forbidden"
 //	  "500":
 //	    $ref: "#/responses/InternalServerError"
-func snapshotGet(s *state.State, snapInst instance.Instance) response.Response {
+func snapshotGet(s *state.State, r *http.Request, snapInst instance.Instance) response.Response {
 	render, _, err := snapInst.Render(storagePools.RenderSnapshotUsage(s, snapInst))
 	if err != nil {
 		return response.SmartError(err)
 	}
 
+	renderedSnap, ok := render.(*api.InstanceSnapshot)
+	if !ok {
+		return response.InternalError(fmt.Errorf("Render didn't return a snapshot"))
+	}
+
 	etag := []any{snapInst.ExpiryDate()}
-	return response.SyncResponseETag(true, render.(*api.InstanceSnapshot), etag)
+	return response.SyncResponseETag(true, renderedSnap, etag)
 }
 
 // swagger:operation POST /1.0/instances/{name}/snapshots/{snapshot} instances instance_snapshot_post
