@@ -27,21 +27,23 @@ import (
 )
 
 var instancesCmd = APIEndpoint{
-	Name: "instances",
-	Path: "instances",
+	Name:        "instances",
+	Path:        "instances",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containers", Path: "containers"},
 		{Name: "vms", Path: "virtual-machines"},
 	},
 
-	Get:  APIEndpointAction{Handler: instancesGet, AccessHandler: allowAuthenticated},
+	Get:  APIEndpointAction{Handler: instancesGet, AccessHandler: allowProjectResourceList},
 	Post: APIEndpointAction{Handler: instancesPost, AccessHandler: allowPermission(entity.TypeProject, auth.EntitlementCanCreateInstances)},
-	Put:  APIEndpointAction{Handler: instancesPut, AccessHandler: allowAuthenticated},
+	Put:  APIEndpointAction{Handler: instancesPut, AccessHandler: allowProjectResourceList},
 }
 
 var instanceCmd = APIEndpoint{
-	Name: "instance",
-	Path: "instances/{name}",
+	Name:        "instance",
+	Path:        "instances/{name}",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "container", Path: "containers/{name}"},
 		{Name: "vm", Path: "virtual-machines/{name}"},
@@ -55,8 +57,9 @@ var instanceCmd = APIEndpoint{
 }
 
 var instanceUEFIVarsCmd = APIEndpoint{
-	Name: "instanceUEFIVars",
-	Path: "instances/{name}/uefi-vars",
+	Name:        "instanceUEFIVars",
+	Path:        "instances/{name}/uefi-vars",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "vmUEFIVars", Path: "virtual-machines/{name}/uefi-vars"},
 	},
@@ -66,8 +69,9 @@ var instanceUEFIVarsCmd = APIEndpoint{
 }
 
 var instanceRebuildCmd = APIEndpoint{
-	Name: "instanceRebuild",
-	Path: "instances/{name}/rebuild",
+	Name:        "instanceRebuild",
+	Path:        "instances/{name}/rebuild",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerRebuild", Path: "containers/{name}/rebuild"},
 		{Name: "vmRebuild", Path: "virtual-machines/{name}/rebuild"},
@@ -77,8 +81,9 @@ var instanceRebuildCmd = APIEndpoint{
 }
 
 var instanceStateCmd = APIEndpoint{
-	Name: "instanceState",
-	Path: "instances/{name}/state",
+	Name:        "instanceState",
+	Path:        "instances/{name}/state",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerState", Path: "containers/{name}/state"},
 		{Name: "vmState", Path: "virtual-machines/{name}/state"},
@@ -89,8 +94,9 @@ var instanceStateCmd = APIEndpoint{
 }
 
 var instanceSFTPCmd = APIEndpoint{
-	Name: "instanceFile",
-	Path: "instances/{name}/sftp",
+	Name:        "instanceFile",
+	Path:        "instances/{name}/sftp",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerFile", Path: "containers/{name}/sftp"},
 		{Name: "vmFile", Path: "virtual-machines/{name}/sftp"},
@@ -100,8 +106,9 @@ var instanceSFTPCmd = APIEndpoint{
 }
 
 var instanceFileCmd = APIEndpoint{
-	Name: "instanceFile",
-	Path: "instances/{name}/files",
+	Name:        "instanceFile",
+	Path:        "instances/{name}/files",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerFile", Path: "containers/{name}/files"},
 		{Name: "vmFile", Path: "virtual-machines/{name}/files"},
@@ -114,35 +121,38 @@ var instanceFileCmd = APIEndpoint{
 }
 
 var instanceSnapshotsCmd = APIEndpoint{
-	Name: "instanceSnapshots",
-	Path: "instances/{name}/snapshots",
+	Name:        "instanceSnapshots",
+	Path:        "instances/{name}/snapshots",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerSnapshots", Path: "containers/{name}/snapshots"},
 		{Name: "vmSnapshots", Path: "virtual-machines/{name}/snapshots"},
 	},
 
-	Get:  APIEndpointAction{Handler: instanceSnapshotsGet, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanView, "name")},
+	Get:  APIEndpointAction{Handler: instanceSnapshotsGet, AccessHandler: allowProjectResourceList},
 	Post: APIEndpointAction{Handler: instanceSnapshotsPost, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanManageSnapshots, "name")},
 }
 
 var instanceSnapshotCmd = APIEndpoint{
-	Name: "instanceSnapshot",
-	Path: "instances/{name}/snapshots/{snapshotName}",
+	Name:        "instanceSnapshot",
+	Path:        "instances/{name}/snapshots/{snapshotName}",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerSnapshot", Path: "containers/{name}/snapshots/{snapshotName}"},
 		{Name: "vmSnapshot", Path: "virtual-machines/{name}/snapshots/{snapshotName}"},
 	},
 
-	Get:    APIEndpointAction{Handler: instanceSnapshotHandler, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanView, "name")},
-	Post:   APIEndpointAction{Handler: instanceSnapshotHandler, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanManageSnapshots, "name")},
-	Delete: APIEndpointAction{Handler: instanceSnapshotHandler, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanManageSnapshots, "name")},
-	Patch:  APIEndpointAction{Handler: instanceSnapshotHandler, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanManageSnapshots, "name")},
-	Put:    APIEndpointAction{Handler: instanceSnapshotHandler, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanManageSnapshots, "name")},
+	Get:    APIEndpointAction{Handler: instanceSnapshotHandler, AccessHandler: allowPermission(entity.TypeInstanceSnapshot, auth.EntitlementCanView, "name", "snapshotName")},
+	Post:   APIEndpointAction{Handler: instanceSnapshotHandler, AccessHandler: allowPermission(entity.TypeInstanceSnapshot, auth.EntitlementCanEdit, "name", "snapshotName")},
+	Delete: APIEndpointAction{Handler: instanceSnapshotHandler, AccessHandler: allowPermission(entity.TypeInstanceSnapshot, auth.EntitlementCanDelete, "name", "snapshotName")},
+	Patch:  APIEndpointAction{Handler: instanceSnapshotHandler, AccessHandler: allowPermission(entity.TypeInstanceSnapshot, auth.EntitlementCanEdit, "name", "snapshotName")},
+	Put:    APIEndpointAction{Handler: instanceSnapshotHandler, AccessHandler: allowPermission(entity.TypeInstanceSnapshot, auth.EntitlementCanEdit, "name", "snapshotName")},
 }
 
 var instanceConsoleCmd = APIEndpoint{
-	Name: "instanceConsole",
-	Path: "instances/{name}/console",
+	Name:        "instanceConsole",
+	Path:        "instances/{name}/console",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerConsole", Path: "containers/{name}/console"},
 		{Name: "vmConsole", Path: "virtual-machines/{name}/console"},
@@ -154,8 +164,9 @@ var instanceConsoleCmd = APIEndpoint{
 }
 
 var instanceExecCmd = APIEndpoint{
-	Name: "instanceExec",
-	Path: "instances/{name}/exec",
+	Name:        "instanceExec",
+	Path:        "instances/{name}/exec",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerExec", Path: "containers/{name}/exec"},
 		{Name: "vmExec", Path: "virtual-machines/{name}/exec"},
@@ -165,8 +176,9 @@ var instanceExecCmd = APIEndpoint{
 }
 
 var instanceMetadataCmd = APIEndpoint{
-	Name: "instanceMetadata",
-	Path: "instances/{name}/metadata",
+	Name:        "instanceMetadata",
+	Path:        "instances/{name}/metadata",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerMetadata", Path: "containers/{name}/metadata"},
 		{Name: "vmMetadata", Path: "virtual-machines/{name}/metadata"},
@@ -178,8 +190,9 @@ var instanceMetadataCmd = APIEndpoint{
 }
 
 var instanceMetadataTemplatesCmd = APIEndpoint{
-	Name: "instanceMetadataTemplates",
-	Path: "instances/{name}/metadata/templates",
+	Name:        "instanceMetadataTemplates",
+	Path:        "instances/{name}/metadata/templates",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerMetadataTemplates", Path: "containers/{name}/metadata/templates"},
 		{Name: "vmMetadataTemplates", Path: "virtual-machines/{name}/metadata/templates"},
@@ -191,39 +204,42 @@ var instanceMetadataTemplatesCmd = APIEndpoint{
 }
 
 var instanceBackupsCmd = APIEndpoint{
-	Name: "instanceBackups",
-	Path: "instances/{name}/backups",
+	Name:        "instanceBackups",
+	Path:        "instances/{name}/backups",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerBackups", Path: "containers/{name}/backups"},
 		{Name: "vmBackups", Path: "virtual-machines/{name}/backups"},
 	},
 
-	Get:  APIEndpointAction{Handler: instanceBackupsGet, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanView, "name")},
+	Get:  APIEndpointAction{Handler: instanceBackupsGet, AccessHandler: allowProjectResourceList},
 	Post: APIEndpointAction{Handler: instanceBackupsPost, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanManageBackups, "name")},
 }
 
 var instanceBackupCmd = APIEndpoint{
-	Name: "instanceBackup",
-	Path: "instances/{name}/backups/{backupName}",
+	Name:        "instanceBackup",
+	Path:        "instances/{name}/backups/{backupName}",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerBackup", Path: "containers/{name}/backups/{backupName}"},
 		{Name: "vmBackup", Path: "virtual-machines/{name}/backups/{backupName}"},
 	},
 
-	Get:    APIEndpointAction{Handler: instanceBackupGet, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanView, "name")},
-	Post:   APIEndpointAction{Handler: instanceBackupPost, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanManageBackups, "name")},
-	Delete: APIEndpointAction{Handler: instanceBackupDelete, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanManageBackups, "name")},
+	Get:    APIEndpointAction{Handler: instanceBackupGet, AccessHandler: allowPermission(entity.TypeInstanceBackup, auth.EntitlementCanView, "name", "backupName")},
+	Post:   APIEndpointAction{Handler: instanceBackupPost, AccessHandler: allowPermission(entity.TypeInstanceBackup, auth.EntitlementCanEdit, "name", "backupName")},
+	Delete: APIEndpointAction{Handler: instanceBackupDelete, AccessHandler: allowPermission(entity.TypeInstanceBackup, auth.EntitlementCanEdit, "name", "backupName")},
 }
 
 var instanceBackupExportCmd = APIEndpoint{
-	Name: "instanceBackupExport",
-	Path: "instances/{name}/backups/{backupName}/export",
+	Name:        "instanceBackupExport",
+	Path:        "instances/{name}/backups/{backupName}/export",
+	MetricsType: entity.TypeInstance,
 	Aliases: []APIEndpointAlias{
 		{Name: "containerBackupExport", Path: "containers/{name}/backups/{backupName}/export"},
 		{Name: "vmBackupExport", Path: "virtual-machines/{name}/backups/{backupName}/export"},
 	},
 
-	Get: APIEndpointAction{Handler: instanceBackupExportGet, AccessHandler: allowPermission(entity.TypeInstance, auth.EntitlementCanManageBackups, "name")},
+	Get: APIEndpointAction{Handler: instanceBackupExportGet, AccessHandler: allowPermission(entity.TypeInstanceBackup, auth.EntitlementCanView, "name", "backupName")},
 }
 
 type instanceAutostartList []instance.Instance
@@ -252,21 +268,32 @@ func (slice instanceAutostartList) Swap(i, j int) {
 var instancesStartMu sync.Mutex
 
 // instanceShouldAutoStart returns whether the instance should be auto-started.
-// Returns true if boot.autostart is enabled or boot.autostart is not set and instance was previously running.
+// Returns true if the conditions below are all met:
+// 1. security.protection.start is not enabled or not set.
+// 2. boot.autostart is enabled or boot.autostart is not set and instance was previously running.
 func instanceShouldAutoStart(inst instance.Instance) bool {
 	config := inst.ExpandedConfig()
 	autoStart := config["boot.autostart"]
 	lastState := config["volatile.last_state.power"]
+	protectStart := config["security.protection.start"]
 
-	return shared.IsTrue(autoStart) || (autoStart == "" && lastState == instance.PowerStateRunning)
+	return shared.IsFalseOrEmpty(protectStart) && (shared.IsTrue(autoStart) || (autoStart == "" && lastState == instance.PowerStateRunning))
 }
 
 func instancesStart(s *state.State, instances []instance.Instance) {
+	// Check if the cluster is currently evacuated.
+	if s.DB.Cluster.LocalNodeIsEvacuated() {
+		return
+	}
+
+	// Acquire startup lock.
 	instancesStartMu.Lock()
 	defer instancesStartMu.Unlock()
 
+	// Sort based on instance boot priority.
 	sort.Sort(instanceAutostartList(instances))
 
+	// Let's make up to 3 attempts to start instances.
 	maxAttempts := 3
 
 	// Start the instances
@@ -301,7 +328,7 @@ func instancesStart(s *state.State, instances []instance.Instance) {
 				if attempt >= maxAttempts {
 					warnErr := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 						// If unable to start after 3 tries, record a warning.
-						return tx.UpsertWarningLocalNode(ctx, inst.Project().Name, entity.TypeInstance, inst.ID(), warningtype.InstanceAutostartFailure, fmt.Sprintf("%v", err))
+						return tx.UpsertWarningLocalNode(ctx, inst.Project().Name, entity.TypeInstance, inst.ID(), warningtype.InstanceAutostartFailure, fmt.Sprint(err))
 					})
 					if warnErr != nil {
 						instLogger.Warn("Failed to create instance autostart failure warning", logger.Ctx{"err": warnErr})
@@ -393,7 +420,7 @@ func instancesOnDisk(s *state.State) ([]instance.Instance, error) {
 			// Also generally it ensures that all devices are stopped cleanly too.
 			backupYamlPath := filepath.Join(instancePaths[instanceType], file.Name(), "backup.yaml")
 			if shared.PathExists(backupYamlPath) {
-				inst, err = instance.LoadFromBackup(s, projectName, filepath.Join(instancePaths[instanceType], file.Name()), false)
+				inst, err = instance.LoadFromBackup(s, projectName, filepath.Join(instancePaths[instanceType], file.Name()))
 				if err != nil {
 					logger.Warn("Failed loading instance", logger.Ctx{"project": projectName, "instance": instanceName, "backup_file": backupYamlPath, "err": err})
 				}
@@ -406,6 +433,7 @@ func instancesOnDisk(s *state.State) ([]instance.Instance, error) {
 					Type:    instanceType,
 					Project: projectName,
 					Name:    instanceName,
+					Node:    s.ServerName, // Set Node field to local node.
 					Config:  make(map[string]string),
 				}
 
@@ -427,7 +455,7 @@ func instancesOnDisk(s *state.State) ([]instance.Instance, error) {
 	return instances, nil
 }
 
-func instancesShutdown(s *state.State, instances []instance.Instance) {
+func instancesShutdown(instances []instance.Instance) {
 	sort.Sort(instanceStopList(instances))
 
 	// Limit shutdown concurrency to number of instances or number of CPU cores (which ever is less).
