@@ -2,7 +2,6 @@ package main
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"net"
 	"net/http"
@@ -247,10 +246,9 @@ func instancesGet(d *Daemon, r *http.Request) response.Response {
 
 	mustLoadObjects := recursion > 0 || (recursion == 0 && clauses != nil && len(clauses.Clauses) > 0)
 
-	projectName, allProjects, err := request.ProjectParams(r)
-	if err != nil {
-		return response.SmartError(err)
-	}
+	// Detect project mode.
+	projectName := request.QueryParam(r, "project")
+	allProjects := shared.IsTrue(r.FormValue("all-projects"))
 
 	// Detect if we want to also return entitlements for each instance.
 	withEntitlements, err := extractEntitlementsFromQuery(r, entity.TypeInstance, true)
@@ -258,6 +256,12 @@ func instancesGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
+	if allProjects && projectName != "" {
+		return response.BadRequest(fmt.Errorf("Cannot specify a project when requesting all projects"))
+	} else if !allProjects && projectName == "" {
+		projectName = api.ProjectDefaultName
+	}
+
 	// Get the list and location of all instances.
 	var filteredProjects []string
 	var memberAddressInstances map[string][]db.Instance
@@ -349,7 +353,7 @@ func instancesGet(d *Daemon, r *http.Request) response.Response {
 		// Mark instances on unavailable projectInstanceToNodeName as down.
 		if mustLoadObjects && memberAddress == "0.0.0.0" {
 			for _, inst := range instances {
-				resultErrListAppend(inst, errors.New("unavailable"))
+				resultErrListAppend(inst, fmt.Errorf("unavailable"))
 			}
 
 			continue
@@ -374,6 +378,7 @@ func instancesGet(d *Daemon, r *http.Request) response.Response {
 					}
 
 					for _, apiInst := range apiInsts {
+						apiInst := apiInst // Local variable for append.
 						resultFullListAppend(&api.InstanceFull{Instance: apiInst})
 					}
 
@@ -390,6 +395,7 @@ func instancesGet(d *Daemon, r *http.Request) response.Response {
 				}
 
 				for _, c := range cs {
+					c := c // Local variable for append.
 					resultFullListAppend(&c)
 				}
 			}(memberAddress, instances)
@@ -406,7 +412,10 @@ func instancesGet(d *Daemon, r *http.Request) response.Response {
 				}})
 			}
 		} else {
-			threads := min(len(instances), 4)
+			threads := 4
+			if len(instances) < threads {
+				threads = len(instances)
+			}
 
 			hostInterfaces, _ := net.Interfaces()
 
@@ -425,7 +434,7 @@ func instancesGet(d *Daemon, r *http.Request) response.Response {
 
 			queue := make(chan db.Instance, threads)
 
-			for range threads {
+			for i := 0; i < threads; i++ {
 				wg.Add(1)
 
 				go func() {
@@ -494,10 +503,9 @@ func instancesGet(d *Daemon, r *http.Request) response.Response {
 		for i := range resultFullList {
 			instancePath := "instances"
 			routeName := mux.CurrentRoute(r).GetName()
-			switch routeName {
-			case "container":
+			if routeName == "container" {
 				instancePath = "containers"
-			case "vm":
+			} else if routeName == "vm" {
 				instancePath = "virtual-machines"
 			}
 
@@ -537,7 +545,7 @@ func instancesGet(d *Daemon, r *http.Request) response.Response {
 // rest API and with a timeout of 30 seconds.
 func doContainersGetFromNode(projects []string, node string, allProjects bool, networkCert *shared.CertInfo, serverCert *shared.CertInfo, r *http.Request, instanceType instancetype.Type) ([]api.Instance, error) {
 	f := func() ([]api.Instance, error) {
-		client, err := cluster.Connect(r.Context(), node, networkCert, serverCert, true)
+		client, err := cluster.Connect(node, networkCert, serverCert, r, true)
 		if err != nil {
 			return nil, fmt.Errorf("Failed to connect to member %s: %w", node, err)
 		}
@@ -586,7 +594,7 @@ func doContainersGetFromNode(projects []string, node string, allProjects bool, n
 
 func doContainersFullGetFromNode(projects []string, node string, allProjects bool, networkCert *shared.CertInfo, serverCert *shared.CertInfo, r *http.Request, instanceType instancetype.Type) ([]api.InstanceFull, error) {
 	f := func() ([]api.InstanceFull, error) {
-		client, err := cluster.Connect(r.Context(), node, networkCert, serverCert, true)
+		client, err := cluster.Connect(node, networkCert, serverCert, r, true)
 		if err != nil {
 			return nil, fmt.Errorf("Failed to connect to member %s: %w", node, err)
 		}
