@@ -3,18 +3,21 @@ package main
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net/http"
 	"net/url"
 	"os"
-	"strings"
+	"slices"
+	"strconv"
 
 	petname "github.com/dustinkirkland/golang-petname"
 	"github.com/google/uuid"
 	"github.com/gorilla/websocket"
 
 	"github.com/canonical/lxd/lxd/archive"
+	"github.com/canonical/lxd/lxd/auth"
 	"github.com/canonical/lxd/lxd/backup"
 	"github.com/canonical/lxd/lxd/cluster"
 	"github.com/canonical/lxd/lxd/db"
@@ -26,6 +29,7 @@ import (
 	"github.com/canonical/lxd/lxd/instance/operationlock"
 	"github.com/canonical/lxd/lxd/operations"
 	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/project/limits"
 	"github.com/canonical/lxd/lxd/request"
 	"github.com/canonical/lxd/lxd/response"
 	"github.com/canonical/lxd/lxd/scriptlet"
@@ -40,7 +44,7 @@ import (
 	"github.com/canonical/lxd/shared/version"
 )
 
-func ensureDownloadedImageFitWithinBudget(s *state.State, r *http.Request, op *operations.Operation, p api.Project, img *api.Image, imgAlias string, source api.InstanceSource, imgType string) (*api.Image, error) {
+func ensureDownloadedImageFitWithinBudget(s *state.State, r *http.Request, op *operations.Operation, p api.Project, imgAlias string, source api.InstanceSource, imgType string) (*api.Image, error) {
 	var autoUpdate bool
 	var err error
 	if p.Config["images.auto_update_cached"] != "" {
@@ -50,8 +54,8 @@ func ensureDownloadedImageFitWithinBudget(s *state.State, r *http.Request, op *o
 	}
 
 	var budget int64
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
-		budget, err = project.GetImageSpaceBudget(s.GlobalConfig, tx, p.Name)
+	err = s.DB.Cluster.Transaction(s.ShutdownCtx, func(ctx context.Context, tx *db.ClusterTx) error {
+		budget, err = limits.GetImageSpaceBudget(ctx, s.GlobalConfig, tx, p.Name)
 		return err
 	})
 	if err != nil {
@@ -104,12 +108,12 @@ func createFromImage(s *state.State, r *http.Request, p api.Project, profiles []
 		}
 
 		if req.Source.Server != "" {
-			img, err = ensureDownloadedImageFitWithinBudget(s, r, op, p, img, imgAlias, req.Source, string(req.Type))
+			img, err = ensureDownloadedImageFitWithinBudget(s, r, op, p, imgAlias, req.Source, string(req.Type))
 			if err != nil {
 				return err
 			}
 		} else if img != nil {
-			err := ensureImageIsLocallyAvailable(s, r, img, args.Project, args.Type)
+			err := ensureImageIsLocallyAvailable(s, r, img, args.Project)
 			if err != nil {
 				return err
 			}
@@ -122,7 +126,13 @@ func createFromImage(s *state.State, r *http.Request, p api.Project, profiles []
 			return err
 		}
 
-		return instanceCreateFromImage(s, r, img, args, op)
+		// Actually create the instance.
+		err = instanceCreateFromImage(s, img, args, op)
+		if err != nil {
+			return err
+		}
+
+		return instanceCreateFinish(s, req, args)
 	}
 
 	resources := map[string][]api.URL{}
@@ -173,8 +183,13 @@ func createFromNone(s *state.State, r *http.Request, projectName string, profile
 	}
 
 	run := func(op *operations.Operation) error {
+		// Actually create the instance.
 		_, err := instanceCreateAsEmpty(s, args)
-		return err
+		if err != nil {
+			return err
+		}
+
+		return instanceCreateFinish(s, req, args)
 	}
 
 	resources := map[string][]api.URL{}
@@ -193,8 +208,19 @@ func createFromNone(s *state.State, r *http.Request, projectName string, profile
 }
 
 func createFromMigration(s *state.State, r *http.Request, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
-	if s.DB.Cluster.LocalNodeIsEvacuated() && r.Context().Value(request.CtxProtocol) != "cluster" {
-		return response.Forbidden(fmt.Errorf("Cluster member is evacuated"))
+	// The request can be nil (see `clusterCopyContainerInternal`).
+	if r != nil {
+		// If it isn't nil, get the protocol.
+		protocol, err := request.GetCtxValue[string](r.Context(), request.CtxProtocol)
+		if err != nil {
+			return response.SmartError(fmt.Errorf("Failed to check request origin: %w", err))
+		}
+
+		// If the protocol is not auth.AuthenticationMethodCluster (e.g. not an internal request) and the node has been
+		// evacuated, reject the request.
+		if s.DB.Cluster.LocalNodeIsEvacuated() && protocol != auth.AuthenticationMethodCluster {
+			return response.Forbidden(fmt.Errorf("Cluster member is evacuated"))
+		}
 	}
 
 	// Validate migration mode.
@@ -202,12 +228,6 @@ func createFromMigration(s *state.State, r *http.Request, projectName string, pr
 		return response.NotImplemented(fmt.Errorf("Mode %q not implemented", req.Source.Mode))
 	}
 
-	// Parse the architecture name
-	architecture, err := osarch.ArchitectureId(req.Architecture)
-	if err != nil {
-		return response.BadRequest(err)
-	}
-
 	dbType, err := instancetype.New(string(req.Type))
 	if err != nil {
 		return response.BadRequest(err)
@@ -217,57 +237,11 @@ func createFromMigration(s *state.State, r *http.Request, projectName string, pr
 		return response.BadRequest(fmt.Errorf("Instance type not supported %q", req.Type))
 	}
 
-	// Prepare the instance creation request.
-	args := db.InstanceArgs{
-		Project:      projectName,
-		Architecture: architecture,
-		BaseImage:    req.Source.BaseImage,
-		Config:       req.Config,
-		Type:         dbType,
-		Devices:      deviceConfig.NewDevices(req.Devices),
-		Description:  req.Description,
-		Ephemeral:    req.Ephemeral,
-		Name:         req.Name,
-		Profiles:     profiles,
-		Stateful:     req.Stateful,
-	}
-
-	storagePool, storagePoolProfile, localRootDiskDeviceKey, localRootDiskDevice, resp := instanceFindStoragePool(s, projectName, req)
+	storagePool, args, resp := setupInstanceArgs(s, dbType, projectName, profiles, req)
 	if resp != nil {
 		return resp
 	}
 
-	if storagePool == "" {
-		return response.BadRequest(fmt.Errorf("Can't find a storage pool for the instance to use"))
-	}
-
-	if localRootDiskDeviceKey == "" && storagePoolProfile == "" {
-		// Give the container it's own local root disk device with a pool property.
-		rootDev := map[string]string{}
-		rootDev["type"] = "disk"
-		rootDev["path"] = "/"
-		rootDev["pool"] = storagePool
-		if args.Devices == nil {
-			args.Devices = deviceConfig.Devices{}
-		}
-
-		// Make sure that we do not overwrite a device the user is currently using under the
-		// name "root".
-		rootDevName := "root"
-		for i := 0; i < 100; i++ {
-			if args.Devices[rootDevName] == nil {
-				break
-			}
-
-			rootDevName = fmt.Sprintf("root%d", i)
-			continue
-		}
-
-		args.Devices[rootDevName] = rootDev
-	} else if localRootDiskDeviceKey != "" && localRootDiskDevice["pool"] == "" {
-		args.Devices[localRootDiskDeviceKey]["pool"] = storagePool
-	}
-
 	var inst instance.Instance
 	var instOp *operationlock.InstanceOperation
 	var cleanup revert.Hook
@@ -302,7 +276,8 @@ func createFromMigration(s *state.State, r *http.Request, projectName string, pr
 	revert := revert.New()
 	defer revert.Fail()
 
-	instanceOnly := req.Source.InstanceOnly || req.Source.ContainerOnly
+	// We keep the ContainerOnly for backward compatibility.
+	instanceOnly := req.Source.InstanceOnly || req.Source.ContainerOnly //nolint:staticcheck,unused
 
 	if inst == nil {
 		_, err := storagePools.LoadByName(s, storagePool)
@@ -314,7 +289,7 @@ func createFromMigration(s *state.State, r *http.Request, projectName string, pr
 		// Note: At this stage we do not yet know if snapshots are going to be received and so we cannot
 		// create their DB records. This will be done if needed in the migrationSink.Do() function called
 		// as part of the operation below.
-		inst, instOp, cleanup, err = instance.CreateInternal(s, args, true)
+		inst, instOp, cleanup, err = instance.CreateInternal(s, *args, true)
 		if err != nil {
 			return response.InternalError(fmt.Errorf("Failed creating instance record: %w", err))
 		}
@@ -342,15 +317,15 @@ func createFromMigration(s *state.State, r *http.Request, projectName string, pr
 	}
 
 	migrationArgs := migrationSinkArgs{
-		URL:                   req.Source.Operation,
-		Dialer:                dialer,
-		Instance:              inst,
-		Secrets:               req.Source.Websockets,
-		Push:                  push,
-		Live:                  req.Source.Live,
-		InstanceOnly:          instanceOnly,
-		ClusterMoveSourceName: clusterMoveSourceName,
-		Refresh:               req.Source.Refresh,
+		url:                   req.Source.Operation,
+		dialer:                dialer,
+		instance:              inst,
+		secrets:               req.Source.Websockets,
+		push:                  push,
+		live:                  req.Source.Live,
+		instanceOnly:          instanceOnly,
+		clusterMoveSourceName: clusterMoveSourceName,
+		refresh:               req.Source.Refresh,
 	}
 
 	sink, err := newMigrationSink(&migrationArgs)
@@ -404,6 +379,112 @@ func createFromMigration(s *state.State, r *http.Request, projectName string, pr
 	return operations.OperationResponse(op)
 }
 
+// createFromConversion receives the root disk (container FS or VM block volume) from the client and creates an
+// instance from it. Conversion options also allow the uploaded image to be converted into a raw format.
+func createFromConversion(s *state.State, r *http.Request, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
+	if s.DB.Cluster.LocalNodeIsEvacuated() {
+		return response.Forbidden(fmt.Errorf("Cluster member is evacuated"))
+	}
+
+	// Validate migration mode.
+	if req.Source.Mode != "push" {
+		return response.NotImplemented(fmt.Errorf("Mode %q not implemented", req.Source.Mode))
+	}
+
+	dbType, err := instancetype.New(string(req.Type))
+	if err != nil {
+		return response.BadRequest(err)
+	}
+
+	// Only virtual machines support additional conversion options.
+	if dbType != instancetype.VM && len(req.Source.ConversionOptions) > 0 {
+		return response.BadRequest(fmt.Errorf("Conversion options can only be used with virtual machines. Instance type %q does not support conversion options", req.Type))
+	}
+
+	// Validate conversion options.
+	for _, opt := range req.Source.ConversionOptions {
+		if !slices.Contains([]string{"format", "virtio"}, opt) {
+			return response.BadRequest(fmt.Errorf("Invalid conversion option %q", opt))
+		}
+	}
+
+	storagePool, args, resp := setupInstanceArgs(s, dbType, projectName, profiles, req)
+	if resp != nil {
+		return resp
+	}
+
+	revert := revert.New()
+	defer revert.Fail()
+
+	_, err = storagePools.LoadByName(s, storagePool)
+	if err != nil {
+		return response.InternalError(err)
+	}
+
+	// Create the instance DB record for main instance.
+	inst, instOp, cleanup, err := instance.CreateInternal(s, *args, true)
+	if err != nil {
+		return response.InternalError(fmt.Errorf("Failed creating instance record: %w", err))
+	}
+
+	revert.Add(cleanup)
+	if err != nil {
+		return response.SmartError(fmt.Errorf("Failed getting exclusive access to instance: %w", err))
+	}
+
+	revert.Add(func() { instOp.Done(err) })
+
+	conversionArgs := conversionSinkArgs{
+		url:               req.Source.Operation,
+		secrets:           req.Source.Websockets,
+		sourceDiskSize:    req.Source.SourceDiskSize,
+		conversionOptions: req.Source.ConversionOptions,
+		instance:          inst,
+	}
+
+	sink, err := newConversionSink(&conversionArgs)
+	if err != nil {
+		return response.InternalError(err)
+	}
+
+	// Copy reverter so far so we can use it inside run after this function has finished.
+	runRevert := revert.Clone()
+
+	run := func(op *operations.Operation) error {
+		defer runRevert.Fail()
+
+		sink.instance.SetOperation(op)
+
+		// And finally run the migration.
+		err = sink.Do(s, instOp)
+		if err != nil {
+			err = fmt.Errorf("Error transferring instance data: %w", err)
+			instOp.Done(err) // Complete operation that was created earlier, to release lock.
+
+			return err
+		}
+
+		instOp.Done(nil) // Complete operation that was created earlier, to release lock.
+		runRevert.Success()
+		return nil
+	}
+
+	resources := map[string][]api.URL{}
+	resources["instances"] = []api.URL{*api.NewURL().Path(version.APIVersion, "instances", req.Name)}
+
+	if dbType == instancetype.Container {
+		resources["containers"] = resources["instances"]
+	}
+
+	op, err := operations.OperationCreate(s, projectName, operations.OperationClassWebsocket, operationtype.InstanceCreate, resources, sink.Metadata(), run, nil, sink.Connect, r)
+	if err != nil {
+		return response.InternalError(err)
+	}
+
+	revert.Success()
+	return operations.OperationResponse(op)
+}
+
 func createFromCopy(s *state.State, r *http.Request, projectName string, profiles []api.Profile, req *api.InstancesPost) response.Response {
 	if s.DB.Cluster.LocalNodeIsEvacuated() {
 		return response.Forbidden(fmt.Errorf("Cluster member is evacuated"))
@@ -535,10 +616,12 @@ func createFromCopy(s *state.State, r *http.Request, projectName string, profile
 	}
 
 	run := func(op *operations.Operation) error {
+		// Actually create the instance.
 		_, err := instanceCreateAsCopy(s, instanceCreateAsCopyOpts{
-			sourceInstance:       source,
-			targetInstance:       args,
-			instanceOnly:         req.Source.InstanceOnly || req.Source.ContainerOnly,
+			sourceInstance: source,
+			targetInstance: args,
+			// We keep the ContainerOnly for backward compatibility.
+			instanceOnly:         req.Source.InstanceOnly || req.Source.ContainerOnly, //nolint:staticcheck,unused
 			refresh:              req.Source.Refresh,
 			applyTemplateTrigger: true,
 			allowInconsistent:    req.Source.AllowInconsistent,
@@ -547,11 +630,18 @@ func createFromCopy(s *state.State, r *http.Request, projectName string, profile
 			return err
 		}
 
-		return nil
+		return instanceCreateFinish(s, req, args)
 	}
 
 	resources := map[string][]api.URL{}
-	resources["instances"] = []api.URL{*api.NewURL().Path(version.APIVersion, "instances", req.Name), *api.NewURL().Path(version.APIVersion, "instances", req.Source.Source)}
+	resources["instances"] = []api.URL{*api.NewURL().Path(version.APIVersion, "instances", req.Name)}
+
+	if shared.IsSnapshot(req.Source.Source) {
+		cName, sName, _ := api.GetParentAndSnapshotName(req.Source.Source)
+		resources["instances_snapshots"] = []api.URL{*api.NewURL().Path(version.APIVersion, "instances", cName, "snapshots", sName)}
+	} else {
+		resources["instances"] = append(resources["instances"], *api.NewURL().Path(version.APIVersion, "instances", req.Source.Source))
+	}
 
 	if dbType == instancetype.Container {
 		resources["containers"] = resources["instances"]
@@ -570,7 +660,7 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 	defer revert.Fail()
 
 	// Create temporary file to store uploaded backup data.
-	backupFile, err := os.CreateTemp(shared.VarPath("backups"), fmt.Sprintf("%s_", backup.WorkingDirPrefix))
+	backupFile, err := os.CreateTemp(shared.VarPath("backups"), backup.WorkingDirPrefix+"_")
 	if err != nil {
 		return response.InternalError(err)
 	}
@@ -600,7 +690,7 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 		decomArgs := append(decomArgs, backupFile.Name())
 
 		// Create temporary file to store the decompressed tarball in.
-		tarFile, err := os.CreateTemp(shared.VarPath("backups"), fmt.Sprintf("%s_decompress_", backup.WorkingDirPrefix))
+		tarFile, err := os.CreateTemp(shared.VarPath("backups"), backup.WorkingDirPrefix+"_decompress_")
 		if err != nil {
 			return response.InternalError(err)
 		}
@@ -633,16 +723,25 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 		return response.BadRequest(err)
 	}
 
+	if bInfo.Config == nil {
+		return response.BadRequest(errors.New("Backup config is missing"))
+	}
+
+	if bInfo.Config.Container == nil {
+		return response.BadRequest(errors.New("Instance definition in backup config is missing"))
+	}
+
 	// Check project permissions.
+	var req api.InstancesPost
 	err = s.DB.Cluster.Transaction(s.ShutdownCtx, func(ctx context.Context, tx *db.ClusterTx) error {
-		req := api.InstancesPost{
+		req = api.InstancesPost{
 			InstancePut: bInfo.Config.Container.Writable(),
 			Name:        bInfo.Name,
 			Source:      api.InstanceSource{}, // Only relevant for "copy" or "migration", but may not be nil.
 			Type:        api.InstanceType(bInfo.Config.Container.Type),
 		}
 
-		return project.AllowInstanceCreation(s.GlobalConfig, tx, projectName, req)
+		return limits.AllowInstanceCreation(ctx, s.GlobalConfig, tx, projectName, req)
 	})
 	if err != nil {
 		return response.SmartError(err)
@@ -771,7 +870,8 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 		}
 
 		runRevert.Success()
-		return nil
+
+		return instanceCreateFinish(s, &req, db.InstanceArgs{Name: bInfo.Name, Project: bInfo.Project})
 	}
 
 	resources := map[string][]api.URL{}
@@ -786,6 +886,68 @@ func createFromBackup(s *state.State, r *http.Request, projectName string, data
 	return operations.OperationResponse(op)
 }
 
+// setupInstanceArgs sets the database instance arguments and determines the storage pool to use.
+func setupInstanceArgs(s *state.State, instType instancetype.Type, projectName string, profiles []api.Profile, req *api.InstancesPost) (storagePool string, instArgs *db.InstanceArgs, resp response.Response) {
+	// Parse the architecture name
+	architecture, err := osarch.ArchitectureId(req.Architecture)
+	if err != nil {
+		return "", nil, response.BadRequest(err)
+	}
+
+	// Prepare the instance creation request.
+	args := db.InstanceArgs{
+		Project:      projectName,
+		Architecture: architecture,
+		BaseImage:    req.Source.BaseImage,
+		Config:       req.Config,
+		Type:         instType,
+		Devices:      deviceConfig.NewDevices(req.Devices),
+		Description:  req.Description,
+		Ephemeral:    req.Ephemeral,
+		Name:         req.Name,
+		Profiles:     profiles,
+		Stateful:     req.Stateful,
+	}
+
+	storagePool, storagePoolProfile, localRootDiskDeviceKey, localRootDiskDevice, resp := instanceFindStoragePool(s, projectName, req)
+	if resp != nil {
+		return "", nil, resp
+	}
+
+	if storagePool == "" {
+		return "", nil, response.BadRequest(fmt.Errorf("Can't find a storage pool for the instance to use"))
+	}
+
+	if localRootDiskDeviceKey == "" && storagePoolProfile == "" {
+		// Give the instance it's own local root disk device with a pool property.
+		rootDev := map[string]string{}
+		rootDev["type"] = "disk"
+		rootDev["path"] = "/"
+		rootDev["pool"] = storagePool
+		if args.Devices == nil {
+			args.Devices = deviceConfig.Devices{}
+		}
+
+		// Make sure that we do not overwrite a device the user is currently using
+		// under the name "root".
+		rootDevName := "root"
+		for i := 0; i < 100; i++ {
+			if args.Devices[rootDevName] == nil {
+				break
+			}
+
+			rootDevName = "root" + strconv.Itoa(i)
+			continue
+		}
+
+		args.Devices[rootDevName] = rootDev
+	} else if localRootDiskDeviceKey != "" && localRootDiskDevice["pool"] == "" {
+		args.Devices[localRootDiskDeviceKey]["pool"] = storagePool
+	}
+
+	return storagePool, &args, nil
+}
+
 // swagger:operation POST /1.0/instances instances instances_post
 //
 //	Create a new instance
@@ -941,7 +1103,7 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 			}
 
 			// Check if the given target is allowed and try to resolve the right member or group
-			targetMemberInfo, targetGroupName, err = project.CheckTarget(ctx, s.Authorizer, r, tx, targetProject, target, allMembers)
+			targetMemberInfo, targetGroupName, err = limits.CheckTarget(ctx, s.Authorizer, r, tx, targetProject, target, allMembers)
 			if err != nil {
 				return err
 			}
@@ -950,7 +1112,7 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 		profileProject := project.ProfileProjectFromRecord(targetProject)
 
 		switch req.Source.Type {
-		case "copy":
+		case api.SourceTypeCopy:
 			if req.Source.Source == "" {
 				return api.StatusErrorf(http.StatusBadRequest, "Must specify a source instance")
 			}
@@ -979,7 +1141,7 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 				}
 			}
 
-		case "image":
+		case api.SourceTypeImage:
 			// Check if the image has an entry in the database but fail only if the error
 			// is different than the image not being found.
 			sourceImage, err = getSourceImageFromInstanceSource(ctx, s, tx, targetProject.Name, req.Source, &sourceImageRef, string(req.Type))
@@ -1020,6 +1182,16 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 				return err
 			}
 
+			dbProfileConfigs, err := dbCluster.GetConfig(ctx, tx.Tx(), "profile")
+			if err != nil {
+				return err
+			}
+
+			dbProfileDevices, err := dbCluster.GetDevices(ctx, tx.Tx(), "profile")
+			if err != nil {
+				return err
+			}
+
 			profilesByName := make(map[string]dbCluster.Profile, len(dbProfiles))
 			for _, dbProfile := range dbProfiles {
 				profilesByName[dbProfile.Name] = dbProfile
@@ -1031,7 +1203,7 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 					return fmt.Errorf("Requested profile %q doesn't exist", profileName)
 				}
 
-				apiProfile, err := profile.ToAPI(ctx, tx.Tx())
+				apiProfile, err := profile.ToAPI(ctx, tx.Tx(), dbProfileConfigs, dbProfileDevices)
 				if err != nil {
 					return err
 				}
@@ -1050,7 +1222,7 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 			i := 0
 			for {
 				i++
-				req.Name = strings.ToLower(petname.Generate(2, "-"))
+				req.Name = petname.Generate(2, "-")
 				if !shared.ValueInSlice(req.Name, names) {
 					break
 				}
@@ -1089,20 +1261,18 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 				}
 			}
 
-			clusterGroupsAllowed := project.GetRestrictedClusterGroups(targetProject)
+			clusterGroupsAllowed := limits.GetRestrictedClusterGroups(targetProject)
 
 			candidateMembers, err = tx.GetCandidateMembers(ctx, allMembers, architectures, targetGroupName, clusterGroupsAllowed, s.GlobalConfig.OfflineThreshold())
 			if err != nil {
 				return err
 			}
-
-			return nil
 		}
 
 		if !clusterNotification {
 			// Check that the project's limits are not violated. Note this check is performed after
 			// automatically generated config values (such as ones from an InstanceType) have been set.
-			err = project.AllowInstanceCreation(s.GlobalConfig, tx, targetProjectName, req)
+			err = limits.AllowInstanceCreation(ctx, s.GlobalConfig, tx, targetProjectName, req)
 			if err != nil {
 				return err
 			}
@@ -1114,7 +1284,7 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	err = instance.ValidName(req.Name, false)
+	err = instancetype.ValidName(req.Name, false)
 	if err != nil {
 		return response.BadRequest(err)
 	}
@@ -1122,7 +1292,7 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 	if s.ServerClustered && !clusterNotification && targetMemberInfo == nil {
 		// Run instance placement scriptlet if enabled and no cluster member selected yet.
 		if s.GlobalConfig.InstancesPlacementScriptlet() != "" {
-			leaderAddress, err := d.gateway.LeaderAddress()
+			leaderInfo, err := s.LeaderInfo()
 			if err != nil {
 				return response.InternalError(err)
 			}
@@ -1142,7 +1312,7 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 			reqExpanded.Config = instancetype.ExpandInstanceConfig(globalConfigDump, reqExpanded.Config, profiles)
 			reqExpanded.Devices = instancetype.ExpandInstanceDevices(deviceConfig.NewDevices(reqExpanded.Devices), profiles).CloneNative()
 
-			targetMemberInfo, err = scriptlet.InstancePlacementRun(r.Context(), logger.Log, s, &reqExpanded, candidateMembers, leaderAddress)
+			targetMemberInfo, err = scriptlet.InstancePlacementRun(r.Context(), logger.Log, s, &reqExpanded, candidateMembers, leaderInfo.Address)
 			if err != nil {
 				return response.SmartError(fmt.Errorf("Failed instance placement scriptlet: %w", err))
 			}
@@ -1180,13 +1350,15 @@ func instancesPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	switch req.Source.Type {
-	case "image":
+	case api.SourceTypeImage:
 		return createFromImage(s, r, *targetProject, profiles, sourceImage, sourceImageRef, &req)
-	case "none":
+	case api.SourceTypeNone:
 		return createFromNone(s, r, targetProjectName, profiles, &req)
-	case "migration":
+	case api.SourceTypeMigration:
 		return createFromMigration(s, r, targetProjectName, profiles, &req)
-	case "copy":
+	case api.SourceTypeConversion:
+		return createFromConversion(s, r, targetProjectName, profiles, &req)
+	case api.SourceTypeCopy:
 		return createFromCopy(s, r, targetProjectName, profiles, &req)
 	default:
 		return response.BadRequest(fmt.Errorf("Unknown source type %s", req.Source.Type))
@@ -1318,7 +1490,8 @@ func clusterCopyContainerInternal(s *state.State, r *http.Request, source instan
 
 		opAPI = op.Get()
 	} else {
-		instanceOnly := req.Source.InstanceOnly || req.Source.ContainerOnly
+		// We keep the ContainerOnly for backward compatibility.
+		instanceOnly := req.Source.InstanceOnly || req.Source.ContainerOnly //nolint:staticcheck,unused
 		pullReq := api.InstancePost{
 			Migration:     true,
 			Live:          req.Source.Live,
@@ -1346,7 +1519,7 @@ func clusterCopyContainerInternal(s *state.State, r *http.Request, source instan
 	}
 
 	// Reset the source for a migration
-	req.Source.Type = "migration"
+	req.Source.Type = api.SourceTypeMigration
 	req.Source.Certificate = string(s.Endpoints.NetworkCert().PublicKey())
 	req.Source.Mode = "pull"
 	req.Source.Operation = fmt.Sprintf("https://%s/%s/operations/%s", nodeAddress, version.APIVersion, opAPI.ID)
@@ -1357,3 +1530,19 @@ func clusterCopyContainerInternal(s *state.State, r *http.Request, source instan
 	// Run the migration
 	return createFromMigration(s, nil, projectName, profiles, req)
 }
+
+// instanceCreateFinish finalizes the creation process of an instance by starting it based on
+// the Start field of the request.
+func instanceCreateFinish(s *state.State, req *api.InstancesPost, args db.InstanceArgs) error {
+	if req == nil || !req.Start {
+		return nil
+	}
+
+	// Start the instance.
+	inst, err := instance.LoadByProjectAndName(s, args.Project, args.Name)
+	if err != nil {
+		return fmt.Errorf("Failed to load the instance: %w", err)
+	}
+
+	return inst.Start(false)
+}
