@@ -12,14 +12,14 @@ import (
 	"os/exec"
 	"path/filepath"
 	"regexp"
-	"slices"
 	"sort"
 	"strings"
 	"time"
 
-	"gopkg.in/yaml.v2"
-
+	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
+
+	"gopkg.in/yaml.v2"
 )
 
 var (
@@ -77,7 +77,7 @@ func getSortedKeysFromMap[K string, V IterableAny](m map[K]V) []K {
 		keys = append(keys, k)
 	}
 
-	slices.Sort(keys)
+	sort.Slice(keys, func(i, j int) bool { return keys[i] < keys[j] })
 	return keys
 }
 
@@ -150,7 +150,7 @@ func parse(path string, outputJSONPath string, excludedPaths []string, substitut
 		}
 
 		// Skip excluded paths
-		if slices.Contains(excludedPaths, path) {
+		if shared.ValueInSlice(path, excludedPaths) {
 			if info.IsDir() {
 				log.Printf("Skipping excluded directory: %v", path)
 				return filepath.SkipDir
@@ -203,7 +203,7 @@ func parse(path string, outputJSONPath string, excludedPaths []string, substitut
 					mdKey := mdKVMatch[1]
 					mdValue := mdKVMatch[2]
 					// check that the metadata key is among the expected ones
-					if !slices.Contains(mdKeys, mdKey) {
+					if !shared.ValueInSlice(mdKey, mdKeys) {
 						continue
 					}
 
@@ -336,7 +336,7 @@ func writeDocFile(inputJSONPath, outputTxtPath string) error {
 	countMaxBackTicks := func(s string) int {
 		count, currCount := 0, 0
 		n := len(s)
-		for i := range n {
+		for i := 0; i < n; i++ {
 			if s[i] == '`' {
 				currCount++
 				continue
@@ -375,7 +375,7 @@ func writeDocFile(inputJSONPath, outputTxtPath string) error {
 		sortedGroupKeys := getSortedKeysFromMap(entityEntries)
 		for _, groupKey := range sortedGroupKeys {
 			groupEntries := entityEntries[groupKey]
-			fmt.Fprintf(buffer, "<!-- config group %s-%s start -->\n", entityKey, groupKey)
+			buffer.WriteString(fmt.Sprintf("<!-- config group %s-%s start -->\n", entityKey, groupKey))
 			for _, configEntryAny := range groupEntries["keys"] {
 				configEntry, ok := configEntryAny.(map[string]any)
 				if !ok {
@@ -404,7 +404,7 @@ func writeDocFile(inputJSONPath, outputTxtPath string) error {
 
 						configContentValueStr, ok := configContentValue.(string)
 						if ok {
-							if (strings.HasSuffix(configContentValueStr, "`") && strings.HasPrefix(configContentValueStr, "`")) || slices.Contains(specialChars, configContentValueStr) {
+							if (strings.HasSuffix(configContentValueStr, "`") && strings.HasPrefix(configContentValueStr, "`")) || shared.ValueInSlice(configContentValueStr, specialChars) {
 								configContentValueStr = fmt.Sprintf("\"%s\"", configContentValueStr)
 							}
 						} else {
@@ -412,7 +412,7 @@ func writeDocFile(inputJSONPath, outputTxtPath string) error {
 							case int, float64, bool:
 								configContentValueStr = fmt.Sprint(configEntryContentTyped)
 							case time.Time:
-								configContentValueStr = configEntryContentTyped.Format(time.RFC3339)
+								configContentValueStr = fmt.Sprint(configEntryContentTyped.Format(time.RFC3339))
 							}
 						}
 
@@ -432,36 +432,41 @@ func writeDocFile(inputJSONPath, outputTxtPath string) error {
 							quoteFormattedValue = fmt.Sprintf("\"%s\"", configContentValueStr)
 						}
 
-						fmt.Fprintf(kvBuffer,
-							":%s: %s\n",
-							configEntryContentKey,
-							quoteFormattedValue)
+						kvBuffer.WriteString(
+							fmt.Sprintf(
+								":%s: %s\n",
+								configEntryContentKey,
+								quoteFormattedValue,
+							),
+						)
 					}
 
 					if backticksCount < 3 {
-						fmt.Fprintf(buffer,
-							"```{config:option} %s %s-%s\n%s%s\n```\n\n",
-							configKey,
-							entityKey,
-							groupKey,
-							kvBuffer.String(),
-							strings.TrimLeft(longDescContent, "\n"))
+						buffer.WriteString(
+							fmt.Sprintf("```{config:option} %s %s-%s\n%s%s\n```\n\n",
+								configKey,
+								entityKey,
+								groupKey,
+								kvBuffer.String(),
+								strings.TrimLeft(longDescContent, "\n"),
+							))
 					} else {
 						configQuotes := strings.Repeat("`", backticksCount+1)
-						fmt.Fprintf(buffer,
-							"%s{config:option} %s %s-%s\n%s%s\n%s\n\n",
-							configQuotes,
-							configKey,
-							entityKey,
-							groupKey,
-							kvBuffer.String(),
-							strings.TrimLeft(longDescContent, "\n"),
-							configQuotes)
+						buffer.WriteString(
+							fmt.Sprintf("%s{config:option} %s %s-%s\n%s%s\n%s\n\n",
+								configQuotes,
+								configKey,
+								entityKey,
+								groupKey,
+								kvBuffer.String(),
+								strings.TrimLeft(longDescContent, "\n"),
+								configQuotes,
+							))
 					}
 				}
 			}
 
-			fmt.Fprintf(buffer, "<!-- config group %s-%s end -->\n", entityKey, groupKey)
+			buffer.WriteString(fmt.Sprintf("<!-- config group %s-%s end -->\n", entityKey, groupKey))
 		}
 	}
 
@@ -480,14 +485,14 @@ func writeDocFile(inputJSONPath, outputTxtPath string) error {
 
 	for _, entityName := range sortedEntityNames {
 		entity := entities[entityName]
-		fmt.Fprintf(buffer, "<!-- entity group %s start -->\n", entityName)
+		buffer.WriteString(fmt.Sprintf("<!-- entity group %s start -->\n", entityName))
 		for _, entitlement := range entity.Entitlements {
-			fmt.Fprintf(buffer, "`%s`\n", entitlement.Name)
-			fmt.Fprintf(buffer, ": %s\n\n", entitlement.Description)
+			buffer.WriteString(fmt.Sprintf("`%s`\n", entitlement.Name))
+			buffer.WriteString(fmt.Sprintf(": %s\n\n", entitlement.Description))
 		}
 
 		buffer.WriteString("\n")
-		fmt.Fprintf(buffer, "<!-- entity group %s end -->\n", entityName)
+		buffer.WriteString(fmt.Sprintf("<!-- entity group %s end -->\n", entityName))
 	}
 
 	err = os.WriteFile(outputTxtPath, buffer.Bytes(), 0644)
