@@ -7,8 +7,7 @@ import (
 
 	"github.com/juju/gomaasapi"
 
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/shared/api"
+	"github.com/canonical/lxd/shared/api"
 )
 
 // Instance is a MAAS specific instance interface.
@@ -116,7 +115,7 @@ func (c *Controller) getDomain(inst Instance) string {
 	fields := strings.Split(c.machine.FQDN(), ".")
 	domain := strings.Join(fields[1:], ".")
 
-	if inst.Project().Name == project.Default {
+	if inst.Project().Name == api.ProjectDefaultName {
 		return domain
 	}
 
