@@ -3,8 +3,7 @@ package main
 import (
 	"context"
 	"database/sql"
-	"errors"
-	"net/http"
+	"fmt"
 	"os"
 
 	sqlite3 "github.com/mattn/go-sqlite3"
@@ -17,7 +16,6 @@ import (
 	"github.com/canonical/lxd/lxd/instance"
 	"github.com/canonical/lxd/lxd/instance/instancetype"
 	"github.com/canonical/lxd/lxd/node"
-	"github.com/canonical/lxd/lxd/state"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/logger"
 )
@@ -30,7 +28,7 @@ type cmdActivateifneeded struct {
 	global *cmdGlobal
 }
 
-func (c *cmdActivateifneeded) command() *cobra.Command {
+func (c *cmdActivateifneeded) Command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = "activateifneeded"
 	cmd.Short = "Check if LXD should be started"
@@ -44,15 +42,15 @@ func (c *cmdActivateifneeded) command() *cobra.Command {
   If at least one of those is true, then a connection will be attempted to the
   LXD socket which will cause a socket-activated LXD to be spawned.
 `
-	cmd.RunE = c.run
+	cmd.RunE = c.Run
 
 	return cmd
 }
 
-func (c *cmdActivateifneeded) run(cmd *cobra.Command, args []string) error {
+func (c *cmdActivateifneeded) Run(cmd *cobra.Command, args []string) error {
 	// Only root should run this
 	if os.Geteuid() != 0 {
-		return errors.New("This must be run as root")
+		return fmt.Errorf("This must be run as root")
 	}
 
 	// Don't start a full daemon, we just need database access
@@ -61,7 +59,7 @@ func (c *cmdActivateifneeded) run(cmd *cobra.Command, args []string) error {
 	// Check if either the local database files exists.
 	path := d.os.LocalDatabasePath()
 	if !shared.PathExists(d.os.LocalDatabasePath()) {
-		logger.Debug("No local database, so no need to start the daemon now")
+		logger.Debugf("No local database, so no need to start the daemon now")
 		return nil
 	}
 
@@ -75,33 +73,22 @@ func (c *cmdActivateifneeded) run(cmd *cobra.Command, args []string) error {
 	d.db.Node = db.DirectAccess(sqldb)
 
 	// Load the configured address from the database
+	var localConfig *node.Config
 	err = d.db.Node.Transaction(context.TODO(), func(ctx context.Context, tx *db.NodeTx) error {
-		d.localConfig, err = node.ConfigLoad(ctx, tx)
+		localConfig, err = node.ConfigLoad(ctx, tx)
 		return err
 	})
 	if err != nil {
 		return err
 	}
 
-	localHTTPAddress := d.localConfig.HTTPSAddress()
-
-	startLXD := func() error {
-		d, err := lxd.ConnectLXDUnix("", &lxd.ConnectionArgs{
-			SkipGetServer: true, // Don't hit the /1.0 endpoint to avoid unnecessary work on server.
-		})
-		if err != nil {
-			return err
-		}
-
-		// Request / endpoint to make a minimum request sufficient to start LXD.
-		_, _, err = d.RawQuery(http.MethodGet, "/", nil, "")
-		return err
-	}
+	localHTTPAddress := localConfig.HTTPSAddress()
 
 	// Look for network socket
 	if localHTTPAddress != "" {
-		logger.Debug("Daemon has core.https_address set, activating...")
-		return startLXD()
+		logger.Debugf("Daemon has core.https_address set, activating...")
+		_, err := lxd.ConnectLXDUnix("", nil)
+		return err
 	}
 
 	// Load the idmap for unprivileged instances
@@ -113,7 +100,7 @@ func (c *cmdActivateifneeded) run(cmd *cobra.Command, args []string) error {
 	// Look for auto-started or previously started instances
 	path = d.os.GlobalDatabasePath()
 	if !shared.PathExists(path) {
-		logger.Debug("No global database, so no need to start the daemon now")
+		logger.Debugf("No global database, so no need to start the daemon now")
 		return nil
 	}
 
@@ -129,45 +116,36 @@ func (c *cmdActivateifneeded) run(cmd *cobra.Command, args []string) error {
 		return err
 	}
 
-	// Don't call d.State() as that will run instance driver feature checks (amongst other unnecessary things).
-	// We just need access to the DBs.
-	s := &state.State{
-		DB:          d.db,
-		LocalConfig: d.localConfig,
-	}
-
-	// Because we've not loaded the cluster member name from the global DB, this function will load all
-	// instances in the database and not filter by local member name. However as we only do this when
-	// core.https_address is unset (as there is an early check above) this is not inefficient because we will
-	// only get to this part for standalone servers.
-	instances, err := instance.LoadNodeAll(s, instancetype.Any)
+	instances, err := instance.LoadNodeAll(d.State(), instancetype.Any)
 	if err != nil {
 		return err
 	}
 
 	for _, inst := range instances {
 		if instanceShouldAutoStart(inst) {
-			logger.Debug("Daemon has auto-started instances, activating...")
-			return startLXD()
+			logger.Debugf("Daemon has auto-started instances, activating...")
+			_, err := lxd.ConnectLXDUnix("", nil)
+			return err
 		}
 
-		config := inst.ExpandedConfig()
-
-		if config["volatile.last_state.power"] == instance.PowerStateRunning {
-			logger.Debug("Daemon has running instances, activating...")
-			return startLXD()
+		if inst.IsRunning() {
+			logger.Debugf("Daemon has running instances, activating...")
+			_, err := lxd.ConnectLXDUnix("", nil)
+			return err
 		}
 
 		// Check for scheduled instance snapshots
+		config := inst.ExpandedConfig()
 		if config["snapshots.schedule"] != "" {
-			logger.Debug("Daemon has scheduled instance snapshots, activating...")
-			return startLXD()
+			logger.Debugf("Daemon has scheduled instance snapshots, activating...")
+			_, err := lxd.ConnectLXDUnix("", nil)
+			return err
 		}
 	}
 
 	// Check for scheduled volume snapshots
 	var volumes []db.StorageVolumeArgs
-	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = d.State().DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		volumes, err = tx.GetStoragePoolVolumesWithType(ctx, cluster.StoragePoolVolumeTypeCustom, false)
 		if err != nil {
 			return err
@@ -181,12 +159,13 @@ func (c *cmdActivateifneeded) run(cmd *cobra.Command, args []string) error {
 
 	for _, vol := range volumes {
 		if vol.Config["snapshots.schedule"] != "" {
-			logger.Debug("Daemon has scheduled volume snapshots, activating...")
-			return startLXD()
+			logger.Debugf("Daemon has scheduled volume snapshots, activating...")
+			_, err := lxd.ConnectLXDUnix("", nil)
+			return err
 		}
 	}
 
-	logger.Debug("No need to start the daemon now")
+	logger.Debugf("No need to start the daemon now")
 	return nil
 }
 
