@@ -3,15 +3,13 @@ package main
 import (
 	"bufio"
 	"context"
-	"errors"
 	"fmt"
 	"io"
-	"net/http"
 	"os"
 	"path/filepath"
 	"strings"
 
-	"github.com/canonical/go-dqlite/v2/client"
+	"github.com/canonical/go-dqlite/v3/client"
 	"github.com/spf13/cobra"
 	"golang.org/x/sys/unix"
 	"gopkg.in/yaml.v2"
@@ -34,7 +32,7 @@ func promptConfirmation(prompt string, opname string) error {
 	input, _ := reader.ReadString('\n')
 	input = strings.TrimSuffix(input, "\n")
 
-	if strings.ToLower(input) != "yes" {
+	if !shared.ValueInSlice(strings.ToLower(input), []string{"yes"}) {
 		return fmt.Errorf("%s operation aborted", opname)
 	}
 
@@ -126,7 +124,7 @@ const clusterEditPrompt = `You should run this command only if:
  - You are *absolutely* sure all LXD daemons are stopped
  - This instance has the most up to date database
 
-See https://documentation.ubuntu.com/lxd/latest/howto/cluster_recover/#reconfigure-the-cluster for more info.`
+See https://documentation.ubuntu.com/lxd/en/latest/howto/cluster_recover/#reconfigure-the-cluster for more info.`
 
 const clusterEditComment = `# Member roles can be modified. Unrecoverable nodes should be given the role "spare".
 #
@@ -162,7 +160,7 @@ func (c *cmdClusterEdit) Run(cmd *cobra.Command, args []string) error {
 	// Make sure that the daemon is not running.
 	_, err := lxd.ConnectLXDUnix("", nil)
 	if err == nil {
-		return errors.New("The LXD daemon is running, please stop it first.")
+		return fmt.Errorf("The LXD daemon is running, please stop it first.")
 	}
 
 	database, err := db.OpenNode(filepath.Join(sys.DefaultOS().VarDir, "database"), nil)
@@ -179,7 +177,7 @@ func (c *cmdClusterEdit) Run(cmd *cobra.Command, args []string) error {
 
 		clusterAddress := config.ClusterAddress()
 		if clusterAddress == "" {
-			return errors.New(`Can't edit cluster configuration as server isn't clustered (missing "cluster.https_address" config)`)
+			return fmt.Errorf(`Can't edit cluster configuration as server isn't clustered (missing "cluster.https_address" config)`)
 		}
 
 		nodes, err = tx.GetRaftNodes(ctx)
@@ -289,11 +287,11 @@ func (c *cmdClusterEdit) Run(cmd *cobra.Command, args []string) error {
 
 func validateNewConfig(oldNodes []db.RaftNode, newNodes []db.RaftNode) error {
 	if len(oldNodes) > len(newNodes) {
-		return errors.New("Removing cluster members is not supported")
+		return fmt.Errorf("Removing cluster members is not supported")
 	}
 
 	if len(oldNodes) < len(newNodes) {
-		return errors.New("Adding cluster members is not supported")
+		return fmt.Errorf("Adding cluster members is not supported")
 	}
 
 	numNewVoters := 0
@@ -302,12 +300,12 @@ func validateNewConfig(oldNodes []db.RaftNode, newNodes []db.RaftNode) error {
 
 		// IDs should not be reordered among cluster members.
 		if oldNode.ID != newNode.ID {
-			return errors.New("Changing cluster member ID is not supported")
+			return fmt.Errorf("Changing cluster member ID is not supported")
 		}
 
 		// If the name field could not be populated, just ignore the new value.
 		if oldNode.Name != "" && newNode.Name != "" && oldNode.Name != newNode.Name {
-			return errors.New("Changing cluster member name is not supported")
+			return fmt.Errorf("Changing cluster member name is not supported")
 		}
 
 		if oldNode.Role == db.RaftSpare && newNode.Role == db.RaftVoter {
@@ -441,7 +439,7 @@ database, so you can possibly inspect it for further recovery.
 You'll be able to permanently delete from the database all information about
 former cluster members by running "lxc cluster remove <member-name> --force".
 
-See https://documentation.ubuntu.com/lxd/latest/howto/cluster_recover/#recover-from-quorum-loss for more
+See https://documentation.ubuntu.com/lxd/en/latest/howto/cluster_recover/#recover-from-quorum-loss for more
 info.`
 
 type cmdClusterRecoverFromQuorumLoss struct {
@@ -467,7 +465,7 @@ func (c *cmdClusterRecoverFromQuorumLoss) Run(cmd *cobra.Command, args []string)
 	// Make sure that the daemon is not running.
 	_, err := lxd.ConnectLXDUnix("", nil)
 	if err == nil {
-		return errors.New("The LXD daemon is running, please stop it first.")
+		return fmt.Errorf("The LXD daemon is running, please stop it first.")
 	}
 
 	// Prompt for confirmation unless --quiet was passed.
@@ -514,7 +512,7 @@ func (c *cmdClusterRemoveRaftNode) Command() *cobra.Command {
 func (c *cmdClusterRemoveRaftNode) Run(cmd *cobra.Command, args []string) error {
 	if len(args) != 1 {
 		_ = cmd.Help()
-		return errors.New("Missing required arguments")
+		return fmt.Errorf("Missing required arguments")
 	}
 
 	address := util.CanonicalNetworkAddress(args[0], shared.HTTPSDefaultPort)
@@ -533,7 +531,7 @@ func (c *cmdClusterRemoveRaftNode) Run(cmd *cobra.Command, args []string) error
 	}
 
 	endpoint := "/internal/cluster/raft-node/" + address
-	_, _, err = client.RawQuery(http.MethodDelete, endpoint, nil, "")
+	_, _, err = client.RawQuery("DELETE", endpoint, nil, "")
 	if err != nil {
 		return err
 	}
