@@ -121,7 +121,6 @@ import "C"
 import (
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"net"
 	"os"
@@ -198,15 +197,15 @@ func (c *cmdForknet) RunDetach(cmd *cobra.Command, args []string) error {
 	hostName := args[3]
 
 	if lxdPID == "" {
-		return errors.New("LXD PID argument is required")
+		return fmt.Errorf("LXD PID argument is required")
 	}
 
 	if ifName == "" {
-		return errors.New("ifname argument is required")
+		return fmt.Errorf("ifname argument is required")
 	}
 
 	if hostName == "" {
-		return errors.New("hostname argument is required")
+		return fmt.Errorf("hostname argument is required")
 	}
 
 	// Check if the interface exists.
