@@ -241,7 +241,6 @@ void forkproxy(void)
 import "C"
 
 import (
-	"errors"
 	"fmt"
 	"io"
 	"net"
@@ -321,7 +320,7 @@ func listenerInstance(epFd C.int, lAddr *deviceConfig.ProxyAddress, cAddr *devic
 			connectPort = cAddr.Ports[(*lStruct).lAddrIndex]
 		}
 
-		connectAddr = net.JoinHostPort(cAddr.Address, strconv.FormatUint(connectPort, 10))
+		connectAddr = net.JoinHostPort(cAddr.Address, fmt.Sprint(connectPort))
 	}
 
 	if lAddr.ConnType == "udp" {
@@ -409,7 +408,7 @@ type lStruct struct {
 func (c *cmdForkproxy) Run(cmd *cobra.Command, args []string) error {
 	// Only root should run this
 	if os.Geteuid() != 0 {
-		return errors.New("This must be run as root")
+		return fmt.Errorf("This must be run as root")
 	}
 
 	// Quick checks.
@@ -420,12 +419,12 @@ func (c *cmdForkproxy) Run(cmd *cobra.Command, args []string) error {
 			return nil
 		}
 
-		return errors.New("Missing required arguments")
+		return fmt.Errorf("Missing required arguments")
 	}
 
 	// Check where we are in initialization
 	if C.whoami != C.FORKPROXY_PARENT && C.whoami != C.FORKPROXY_CHILD {
-		return errors.New("Failed to call forkproxy constructor")
+		return fmt.Errorf("Failed to call forkproxy constructor")
 	}
 
 	listenAddr := args[2]
@@ -441,7 +440,7 @@ func (c *cmdForkproxy) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	if (lAddr.ConnType == "udp" || lAddr.ConnType == "tcp") && cAddr.ConnType == "udp" || cAddr.ConnType == "tcp" {
-		err := errors.New("Invalid port range")
+		err := fmt.Errorf("Invalid port range")
 		if len(lAddr.Ports) > 1 && len(cAddr.Ports) > 1 && (len(cAddr.Ports) != len(lAddr.Ports)) {
 			fmt.Println(err)
 			return err
@@ -469,7 +468,7 @@ func (c *cmdForkproxy) Run(cmd *cobra.Command, args []string) error {
 			listenAddresses = make([]string, 0, listenPortCount)
 
 			for i := 0; i < listenPortCount; i++ {
-				listenAddresses = append(listenAddresses, net.JoinHostPort(lAddr.Address, strconv.FormatUint(lAddr.Ports[i], 10)))
+				listenAddresses = append(listenAddresses, net.JoinHostPort(lAddr.Address, fmt.Sprint(lAddr.Ports[i])))
 			}
 		}
 
@@ -628,7 +627,7 @@ func (c *cmdForkproxy) Run(cmd *cobra.Command, args []string) error {
 
 	epFd := C.epoll_create1(C.EPOLL_CLOEXEC)
 	if epFd < 0 {
-		return errors.New("Failed to create new epoll instance")
+		return fmt.Errorf("Failed to create new epoll instance")
 	}
 
 	// Wait for SIGTERM and close the listener in order to exit the loop below
@@ -663,7 +662,7 @@ func (c *cmdForkproxy) Run(cmd *cobra.Command, args []string) error {
 		*(*C.int)(unsafe.Pointer(&ev.data)) = C.int(f.Fd())
 		ret := C.epoll_ctl(epFd, C.EPOLL_CTL_ADD, C.int(f.Fd()), &ev)
 		if ret < 0 {
-			return errors.New("Error: Failed to add listener fd to epoll instance")
+			return fmt.Errorf("Error: Failed to add listener fd to epoll instance")
 		}
 	}
 
@@ -779,7 +778,7 @@ func proxyCopy(dst net.Conn, src net.Conn) error {
 				udpSessionsLock.Unlock()
 
 				if us == nil {
-					return errors.New("Connection expired")
+					return fmt.Errorf("Connection expired")
 				}
 
 				us.timerLock.Lock()
@@ -820,53 +819,41 @@ func proxyCopy(dst net.Conn, src net.Conn) error {
 }
 
 func genericRelay(dst net.Conn, src net.Conn) {
-	// relayer is a helper function that handles one-way data copying and signals completion through the provided channel
 	relayer := func(src net.Conn, dst net.Conn, ch chan error) {
-		err := proxyCopy(src, dst)
-		// Support half-closed TCP connections after copying completes. This allows data to still be received.
-		tcpConn, ok := src.(*net.TCPConn)
-		if ok {
-			_ = tcpConn.CloseWrite()
-		}
-
-		// Signal completion
-		ch <- err
+		ch <- proxyCopy(src, dst)
 		close(ch)
 	}
 
 	chSend := make(chan error)
 	chRecv := make(chan error)
 
-	// Start copying in both directions
 	go relayer(src, dst, chRecv)
 
 	_, isUDP := dst.(*net.UDPConn)
 	if !isUDP {
-		go relayer(dst, src, chSend) // Bidirectional copying is only supported for non-UDP connections
+		go relayer(dst, src, chSend)
 	}
 
-	// Wait for both copy operations to complete
-	for chSend != nil || chRecv != nil {
-		select {
-		case errSend := <-chSend:
-			if daemon.Debug && errSend != nil {
-				fmt.Printf("Warning: Error while sending data: %v\n", errSend)
-			}
+	select {
+	case errSnd := <-chSend:
+		if daemon.Debug && errSnd != nil {
+			fmt.Printf("Warning: Error while sending data: %v\n", errSnd)
+		}
 
-			chSend = nil // Sending complete
-
-		case errRecv := <-chRecv:
-			if daemon.Debug && errRecv != nil {
-				fmt.Printf("Warning: Error while reading data: %v\n", errRecv)
-			}
-
-			chRecv = nil // Receiving complete
+	case errRcv := <-chRecv:
+		if daemon.Debug && errRcv != nil {
+			fmt.Printf("Warning: Error while reading data: %v\n", errRcv)
 		}
 	}
 
-	// Fully close both connections once both directions are complete
 	_ = src.Close()
 	_ = dst.Close()
+
+	// Empty the channels
+	if !isUDP {
+		<-chSend
+	}
+	<-chRecv
 }
 
 func unixRelayer(src *net.UnixConn, dst *net.UnixConn, ch chan error) {
@@ -918,7 +905,7 @@ func unixRelayer(src *net.UnixConn, dst *net.UnixConn, ch chan error) {
 		}
 
 		if sData != tData || sOob != tOob {
-			ch <- errors.New("Lost oob data during transfer")
+			ch <- fmt.Errorf("Lost oob data during transfer")
 			return
 		}
 
@@ -1005,7 +992,7 @@ func tryListenUDP(protocol string, addr string) (*os.File, error) {
 	}
 
 	if UDPConn == nil {
-		return nil, errors.New("Failed to setup UDP listener")
+		return nil, fmt.Errorf("Failed to setup UDP listener")
 	}
 
 	file, err := UDPConn.File()
@@ -1030,7 +1017,7 @@ func getListenerFile(protocol string, addr string) (*os.File, error) {
 	case *net.UnixListener:
 		file, err = l.File()
 	default:
-		return nil, errors.New("Could not get listener file: invalid listener type")
+		return nil, fmt.Errorf("Could not get listener file: invalid listener type")
 	}
 
 	if err != nil {
