@@ -1,9 +1,7 @@
 package main
 
 import (
-	"context"
 	"encoding/pem"
-	"errors"
 	"fmt"
 	"os"
 
@@ -31,7 +29,6 @@ type cmdInit struct {
 	flagStorageDevice   string
 	flagStorageLoopSize int
 	flagStoragePool     string
-	flagTrustPassword   string
 
 	hostname string
 }
@@ -47,7 +44,7 @@ func (c *cmdInit) Command() *cobra.Command {
 	cmd.Example = `  init --minimal
   init --auto [--network-address=IP] [--network-port=8443] [--storage-backend=dir]
               [--storage-create-device=DEVICE] [--storage-create-loop=SIZE]
-              [--storage-pool=POOL] [--trust-password=PASSWORD]
+              [--storage-pool=POOL]
   init --preseed
   init --dump
 `
@@ -63,7 +60,6 @@ func (c *cmdInit) Command() *cobra.Command {
 	cmd.Flags().StringVar(&c.flagStorageDevice, "storage-create-device", "", "Setup device based storage using DEVICE"+"``")
 	cmd.Flags().IntVar(&c.flagStorageLoopSize, "storage-create-loop", -1, "Setup loop based storage with SIZE in GiB"+"``")
 	cmd.Flags().StringVar(&c.flagStoragePool, "storage-pool", "", "Storage pool to use or create"+"``")
-	cmd.Flags().StringVar(&c.flagTrustPassword, "trust-password", "", "Password required to add new clients"+"``")
 
 	return cmd
 }
@@ -72,30 +68,29 @@ func (c *cmdInit) Command() *cobra.Command {
 func (c *cmdInit) Run(cmd *cobra.Command, args []string) error {
 	// Quick checks.
 	if c.flagAuto && c.flagPreseed {
-		return errors.New("Can't use --auto and --preseed together")
+		return fmt.Errorf("Can't use --auto and --preseed together")
 	}
 
 	if c.flagMinimal && c.flagPreseed {
-		return errors.New("Can't use --minimal and --preseed together")
+		return fmt.Errorf("Can't use --minimal and --preseed together")
 	}
 
 	if c.flagMinimal && c.flagAuto {
-		return errors.New("Can't use --minimal and --auto together")
+		return fmt.Errorf("Can't use --minimal and --auto together")
 	}
 
 	if !c.flagAuto && (c.flagNetworkAddress != "" || c.flagNetworkPort != -1 ||
 		c.flagStorageBackend != "" || c.flagStorageDevice != "" ||
-		c.flagStorageLoopSize != -1 || c.flagStoragePool != "" ||
-		c.flagTrustPassword != "") {
-		return errors.New("Configuration flags require --auto")
+		c.flagStorageLoopSize != -1 || c.flagStoragePool != "") {
+		return fmt.Errorf("Configuration flags require --auto")
 	}
 
 	if c.flagDump && (c.flagAuto || c.flagMinimal ||
 		c.flagPreseed || c.flagNetworkAddress != "" ||
 		c.flagNetworkPort != -1 || c.flagStorageBackend != "" ||
 		c.flagStorageDevice != "" || c.flagStorageLoopSize != -1 ||
-		c.flagStoragePool != "" || c.flagTrustPassword != "") {
-		return errors.New("Can't use --dump with other flags")
+		c.flagStoragePool != "") {
+		return fmt.Errorf("Can't use --dump with other flags")
 	}
 
 	// Connect to LXD
@@ -178,7 +173,7 @@ func (c *cmdInit) Run(cmd *cobra.Command, args []string) error {
 			config.Cluster.ClusterAddress = util.CanonicalNetworkAddress(clusterAddress, shared.HTTPSDefaultPort)
 
 			// Cluster certificate
-			cert, err := shared.GetRemoteCertificate(context.Background(), "https://"+config.Cluster.ClusterAddress, version.UserAgent)
+			cert, err := shared.GetRemoteCertificate("https://"+config.Cluster.ClusterAddress, version.UserAgent)
 			if err != nil {
 				fmt.Printf("Error connecting to existing cluster member %q: %v\n", clusterAddress, err)
 				continue
@@ -195,11 +190,8 @@ func (c *cmdInit) Run(cmd *cobra.Command, args []string) error {
 		}
 
 		if config.Cluster.ClusterCertificate == "" {
-			return errors.New("Unable to connect to any of the cluster members specified in join token")
+			return fmt.Errorf("Unable to connect to any of the cluster members specified in join token")
 		}
-
-		// Raw join token used as cluster password so it can be validated.
-		config.Cluster.ClusterPassword = config.Cluster.ClusterToken
 	}
 
 	// If clustering is enabled, and no cluster.https_address network address
