@@ -3,12 +3,10 @@ package main
 import (
 	"context"
 	"encoding/pem"
-	"errors"
 	"fmt"
 	"net"
 	"os"
 	"os/exec"
-	"slices"
 	"strconv"
 	"strings"
 
@@ -46,7 +44,7 @@ func (c *cmdInit) RunInteractive(cmd *cobra.Command, args []string, d lxd.Instan
 	}
 
 	// Clustering
-	err := c.askClustering(&config, server)
+	err := c.askClustering(&config, d, server)
 	if err != nil {
 		return nil, err
 	}
@@ -60,7 +58,7 @@ func (c *cmdInit) RunInteractive(cmd *cobra.Command, args []string, d lxd.Instan
 		}
 
 		// MAAS
-		err = c.askMAAS(&config)
+		err = c.askMAAS(&config, d)
 		if err != nil {
 			return nil, err
 		}
@@ -72,7 +70,7 @@ func (c *cmdInit) RunInteractive(cmd *cobra.Command, args []string, d lxd.Instan
 		}
 
 		// Daemon config
-		err = c.askDaemon(&config, server)
+		err = c.askDaemon(&config, d, server)
 		if err != nil {
 			return nil, err
 		}
@@ -108,7 +106,7 @@ func (c *cmdInit) RunInteractive(cmd *cobra.Command, args []string, d lxd.Instan
 	return &config, nil
 }
 
-func (c *cmdInit) askClustering(config *api.InitPreseed, server *api.Server) error {
+func (c *cmdInit) askClustering(config *api.InitPreseed, d lxd.InstanceServer, server *api.Server) error {
 	clustering, err := c.global.asker.AskBool("Would you like to use LXD clustering? (yes/no) [default=no]: ", "no")
 	if err != nil {
 		return err
@@ -133,8 +131,8 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, server *api.Server) err
 			address := util.CanonicalNetworkAddress(value, shared.HTTPSDefaultPort)
 
 			host, _, _ := net.SplitHostPort(address)
-			if slices.Contains([]string{"", "[::]", "0.0.0.0"}, host) {
-				return errors.New("Invalid IP address or DNS name")
+			if shared.ValueInSlice(host, []string{"", "[::]", "0.0.0.0"}) {
+				return fmt.Errorf("Invalid IP address or DNS name")
 			}
 
 			if err == nil {
@@ -172,7 +170,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, server *api.Server) err
 
 			// Root is required to access the certificate files
 			if os.Geteuid() != 0 {
-				return errors.New("Joining an existing cluster requires root privileges")
+				return fmt.Errorf("Joining an existing cluster requires root privileges")
 			}
 
 			var joinToken *api.ClusterMemberJoinToken
@@ -187,118 +185,43 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, server *api.Server) err
 				return nil
 			}
 
-			validInput := func(input string) error {
-				if slices.Contains([]string{"yes", "y"}, strings.ToLower(input)) {
-					return nil
-				} else if slices.Contains([]string{"no", "n"}, strings.ToLower(input)) {
-					return nil
-				} else if validJoinToken(input) != nil {
-					return errors.New("Not yes/no, or invalid join token")
-				}
-
-				return nil
-			}
-
-			clusterJoinToken, err := c.global.asker.AskString("Do you have a join token? (yes/no/[token]) [default=no]: ", "no", validInput)
+			clusterJoinToken, err := c.global.asker.AskString("Please provide join token: ", "", validJoinToken)
 			if err != nil {
 				return err
 			}
 
-			if !slices.Contains([]string{"no", "n"}, strings.ToLower(clusterJoinToken)) {
-				if slices.Contains([]string{"yes", "y"}, strings.ToLower(clusterJoinToken)) {
-					clusterJoinToken, err = c.global.asker.AskString("Please provide join token: ", "", validJoinToken)
-					if err != nil {
-						return err
-					}
-				}
+			// Set server name from join token
+			config.Cluster.ServerName = joinToken.ServerName
 
-				// Set server name from join token
-				config.Cluster.ServerName = joinToken.ServerName
+			// Attempt to find a working cluster member to use for joining by retrieving the
+			// cluster certificate from each address in the join token until we succeed.
+			for _, clusterAddress := range joinToken.Addresses {
+				config.Cluster.ClusterAddress = util.CanonicalNetworkAddress(clusterAddress, shared.HTTPSDefaultPort)
 
-				// Attempt to find a working cluster member to use for joining by retrieving the
-				// cluster certificate from each address in the join token until we succeed.
-				for _, clusterAddress := range joinToken.Addresses {
-					config.Cluster.ClusterAddress = util.CanonicalNetworkAddress(clusterAddress, shared.HTTPSDefaultPort)
-
-					// Cluster certificate
-					cert, err := shared.GetRemoteCertificate(context.Background(), "https://"+config.Cluster.ClusterAddress, version.UserAgent)
-					if err != nil {
-						fmt.Printf("Error connecting to existing cluster member %q: %v\n", clusterAddress, err)
-						continue
-					}
-
-					certDigest := shared.CertFingerprint(cert)
-					if joinToken.Fingerprint != certDigest {
-						return fmt.Errorf("Certificate fingerprint mismatch between join token and cluster member %q", clusterAddress)
-					}
-
-					config.Cluster.ClusterCertificate = string(pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: cert.Raw}))
-
-					break // We've found a working cluster member.
-				}
-
-				if config.Cluster.ClusterCertificate == "" {
-					return errors.New("Unable to connect to any of the cluster members specified in join token")
-				}
-
-				// Raw join token used as cluster password so it can be validated.
-				config.Cluster.ClusterPassword = clusterJoinToken
-			} else {
-				// Ask for server name since no token is provided
-				err = askForServerName()
+				// Cluster certificate
+				cert, err := shared.GetRemoteCertificate("https://"+config.Cluster.ClusterAddress, version.UserAgent)
 				if err != nil {
-					return err
+					fmt.Printf("Error connecting to existing cluster member %q: %v\n", clusterAddress, err)
+					continue
 				}
 
-				for {
-					// Cluster URL
-					clusterAddress, err := c.global.asker.AskString("IP address or FQDN of an existing cluster member (may include port): ", "", nil)
-					if err != nil {
-						return err
-					}
-
-					config.Cluster.ClusterAddress = util.CanonicalNetworkAddress(clusterAddress, shared.HTTPSDefaultPort)
-
-					// Cluster certificate
-					cert, err := shared.GetRemoteCertificate(context.Background(), "https://"+config.Cluster.ClusterAddress, version.UserAgent)
-					if err != nil {
-						fmt.Printf("Error connecting to existing cluster member: %v\n", err)
-						continue
-					}
-
-					certDigest := shared.CertFingerprint(cert)
-					fmt.Println("Cluster fingerprint:", certDigest)
-					fmt.Println("You can validate this fingerprint by running \"lxc info\" locally on an existing cluster member.")
-
-					validator := func(input string) error {
-						if input == certDigest {
-							return nil
-						} else if slices.Contains([]string{"yes", "y"}, strings.ToLower(input)) {
-							return nil
-						} else if slices.Contains([]string{"no", "n"}, strings.ToLower(input)) {
-							return nil
-						}
-
-						return errors.New("Not yes/no or fingerprint")
-					}
-
-					fingerprintCorrect, err := c.global.asker.AskString("Is this the correct fingerprint? (yes/no/[fingerprint]) [default=no]: ", "no", validator)
-					if err != nil {
-						return err
-					}
-
-					if slices.Contains([]string{"no", "n"}, strings.ToLower(fingerprintCorrect)) {
-						return errors.New("User aborted configuration")
-					}
-
-					config.Cluster.ClusterCertificate = string(pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: cert.Raw}))
-
-					// Cluster password
-					config.Cluster.ClusterPassword = c.global.asker.AskPasswordOnce("Cluster trust password: ")
-					break
+				certDigest := shared.CertFingerprint(cert)
+				if joinToken.Fingerprint != certDigest {
+					return fmt.Errorf("Certificate fingerprint mismatch between join token and cluster member %q", clusterAddress)
 				}
+
+				config.Cluster.ClusterCertificate = string(pem.EncodeToMemory(&pem.Block{Type: "CERTIFICATE", Bytes: cert.Raw}))
+
+				break // We've found a working cluster member.
 			}
 
+			if config.Cluster.ClusterCertificate == "" {
+				return fmt.Errorf("Unable to connect to any of the cluster members specified in join token")
+			}
+
+			// Pass the raw join token.
+			config.Cluster.ClusterToken = clusterJoinToken
+
 			// Confirm wiping
 			clusterWipeMember, err := c.global.asker.AskBool("All existing data is lost when joining a cluster, continue? (yes/no) [default=no] ", "no")
 			if err != nil {
@@ -306,7 +229,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, server *api.Server) err
 			}
 
 			if !clusterWipeMember {
-				return errors.New("User aborted configuration")
+				return fmt.Errorf("User aborted configuration")
 			}
 
 			// Connect to existing cluster
@@ -322,7 +245,6 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, server *api.Server) err
 
 			// Now we have setup trust, don't send to server, othwerwise it will try and setup trust
 			// again and if using a one-time join token, will fail.
-			config.Cluster.ClusterPassword = ""
 			config.Cluster.ClusterToken = ""
 
 			// Client parameters to connect to the target cluster member.
@@ -345,7 +267,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, server *api.Server) err
 			}
 
 			for i, config := range cluster.MemberConfig {
-				question := fmt.Sprintf("Choose %s: ", config.Description)
+				question := "Choose " + config.Description + ": "
 
 				// Allow for empty values.
 				configValue, err := c.global.asker.AskString(question, "", validate.Optional())
@@ -369,7 +291,7 @@ func (c *cmdInit) askClustering(config *api.InitPreseed, server *api.Server) err
 	return nil
 }
 
-func (c *cmdInit) askMAAS(config *api.InitPreseed) error {
+func (c *cmdInit) askMAAS(config *api.InitPreseed, d lxd.InstanceServer) error {
 	maas, err := c.global.asker.AskBool("Would you like to connect to a MAAS server? (yes/no) [default=no]: ", "no")
 	if err != nil {
 		return err
@@ -543,7 +465,7 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 							return fmt.Errorf("The auto-detected underlay (%s) isn't a /16 or /24, please specify manually", subnet.String())
 						}
 
-						return errors.New("The underlay subnet must be a /16 or a /24")
+						return fmt.Errorf("The underlay subnet must be a /16 or a /24")
 					}
 
 					return nil
@@ -601,7 +523,7 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 
 		// IPv4
 		net.Config["ipv4.address"], err = c.global.asker.AskString("What IPv4 address should be used? (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
-			if slices.Contains([]string{"auto", "none"}, value) {
+			if shared.ValueInSlice(value, []string{"auto", "none"}) {
 				return nil
 			}
 
@@ -611,18 +533,18 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 			return err
 		}
 
-		if !slices.Contains([]string{"auto", "none"}, net.Config["ipv4.address"]) {
+		if !shared.ValueInSlice(net.Config["ipv4.address"], []string{"auto", "none"}) {
 			netIPv4UseNAT, err := c.global.asker.AskBool("Would you like LXD to NAT IPv4 traffic on your bridge? [default=yes]: ", "yes")
 			if err != nil {
 				return err
 			}
 
-			net.Config["ipv4.nat"] = strconv.FormatBool(netIPv4UseNAT)
+			net.Config["ipv4.nat"] = fmt.Sprint(netIPv4UseNAT)
 		}
 
 		// IPv6
 		net.Config["ipv6.address"], err = c.global.asker.AskString("What IPv6 address should be used? (CIDR subnet notation, “auto” or “none”) [default=auto]: ", "auto", func(value string) error {
-			if slices.Contains([]string{"auto", "none"}, value) {
+			if shared.ValueInSlice(value, []string{"auto", "none"}) {
 				return nil
 			}
 
@@ -632,13 +554,13 @@ func (c *cmdInit) askNetworking(config *api.InitPreseed, d lxd.InstanceServer) e
 			return err
 		}
 
-		if !slices.Contains([]string{"auto", "none"}, net.Config["ipv6.address"]) {
+		if !shared.ValueInSlice(net.Config["ipv6.address"], []string{"auto", "none"}) {
 			netIPv6UseNAT, err := c.global.asker.AskBool("Would you like LXD to NAT IPv6 traffic on your bridge? [default=yes]: ", "yes")
 			if err != nil {
 				return err
 			}
 
-			net.Config["ipv6.nat"] = strconv.FormatBool(netIPv6UseNAT)
+			net.Config["ipv6.nat"] = fmt.Sprint(netIPv6UseNAT)
 		}
 
 		// Add the new network
@@ -696,7 +618,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 
 	if len(availableBackends) == 0 {
 		if poolType != util.PoolTypeAny {
-			return errors.New("No storage backends available")
+			return fmt.Errorf("No storage backends available")
 		}
 
 		return fmt.Errorf("No %s storage backends available", poolType)
@@ -708,11 +630,11 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 	}
 
 	defaultStorage := "dir"
-	if backingFs == "btrfs" && slices.Contains(availableBackends, "btrfs") {
+	if backingFs == "btrfs" && shared.ValueInSlice("btrfs", availableBackends) {
 		defaultStorage = "btrfs"
-	} else if slices.Contains(availableBackends, "zfs") {
+	} else if shared.ValueInSlice("zfs", availableBackends) {
 		defaultStorage = "zfs"
-	} else if slices.Contains(availableBackends, "btrfs") {
+	} else if shared.ValueInSlice("btrfs", availableBackends) {
 		defaultStorage = "btrfs"
 	}
 
@@ -753,7 +675,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 		if len(availableBackends) > 1 {
 			defaultBackend := defaultStorage
 			if poolType == util.PoolTypeRemote {
-				if slices.Contains(availableBackends, "ceph") {
+				if shared.ValueInSlice("ceph", availableBackends) {
 					defaultBackend = "ceph"
 				} else {
 					defaultBackend = availableBackends[0] // Default to first remote driver.
@@ -811,8 +733,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 		}
 
 		if poolCreate {
-			switch pool.Driver {
-			case "ceph":
+			if pool.Driver == "ceph" {
 				// Ask for the name of the cluster
 				pool.Config["ceph.cluster_name"], err = c.global.asker.AskString("Name of the existing CEPH cluster [default=ceph]: ", "ceph", nil)
 				if err != nil {
@@ -830,8 +751,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 				if err != nil {
 					return err
 				}
-
-			case "cephfs":
+			} else if pool.Driver == "cephfs" {
 				// Ask for the name of the cluster
 				pool.Config["cephfs.cluster_name"], err = c.global.asker.AskString("Name of the existing CEPHfs cluster [default=ceph]: ", "ceph", nil)
 				if err != nil {
@@ -843,8 +763,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 				if err != nil {
 					return err
 				}
-
-			default:
+			} else {
 				useEmptyBlockDev, err := c.global.asker.AskBool("Would you like to use an existing empty block device (e.g. a disk or partition)? (yes/no) [default=no]: ", "no")
 				if err != nil {
 					return err
@@ -869,7 +788,14 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 					}
 
 					/* choose 5 GiB < x < 30GiB, where x is 20% of the disk size */
-					defaultSize := max(min(uint64(st.Frsize)*st.Blocks/(1024*1024*1024)/5, 30), 5)
+					defaultSize := uint64(st.Frsize) * st.Blocks / (1024 * 1024 * 1024) / 5
+					if defaultSize > 30 {
+						defaultSize = 30
+					}
+
+					if defaultSize < 5 {
+						defaultSize = 5
+					}
 
 					pool.Config["size"], err = c.global.asker.AskString(
 						fmt.Sprintf("Size in GiB of the new loop device (1GiB minimum) [default=%dGiB]: ", defaultSize),
@@ -883,7 +809,7 @@ func (c *cmdInit) askStoragePool(config *api.InitPreseed, d lxd.InstanceServer,
 							}
 
 							if result < 1 {
-								return errors.New("Minimum size is 1GiB")
+								return fmt.Errorf("Minimum size is 1GiB")
 							}
 
 							return nil
@@ -940,7 +866,7 @@ and make sure that your user can see and run the "thin_check" command before run
 				}
 
 				if !lvmContinueNoThin {
-					return errors.New("The LVM thin provisioning tools couldn't be found on the system")
+					return fmt.Errorf("The LVM thin provisioning tools couldn't be found on the system")
 				}
 
 				pool.Config["lvm.use_thinpool"] = "false"
@@ -954,7 +880,7 @@ and make sure that your user can see and run the "thin_check" command before run
 	return nil
 }
 
-func (c *cmdInit) askDaemon(config *api.InitPreseed, server *api.Server) error {
+func (c *cmdInit) askDaemon(config *api.InitPreseed, d lxd.InstanceServer, server *api.Server) error {
 	// Detect lack of uid/gid
 	idmapset, err := idmap.DefaultIdmapSet("", "")
 	if (err != nil || len(idmapset.Idmap) == 0 || idmapset.Usable() != nil) && shared.RunningInUserNS() {
@@ -1009,7 +935,7 @@ they otherwise would.
 				netAddr = "[" + netAddr + "]"
 			}
 
-			netPort, err := c.global.asker.AskInt(fmt.Sprintf("Port to bind LXD to [default=%d]: ", shared.HTTPSDefaultPort), 1, 65535, strconv.Itoa(shared.HTTPSDefaultPort), func(netPort int64) error {
+			netPort, err := c.global.asker.AskInt(fmt.Sprintf("Port to bind LXD to [default=%d]: ", shared.HTTPSDefaultPort), 1, 65535, fmt.Sprintf("%d", shared.HTTPSDefaultPort), func(netPort int64) error {
 				address := util.CanonicalNetworkAddressFromAddressAndPort(netAddr, netPort, shared.HTTPSDefaultPort)
 
 				if err == nil {
