@@ -6,18 +6,19 @@ import (
 	"io"
 	"os"
 
-	"github.com/olekukonko/tablewriter"
 	"github.com/spf13/cobra"
 
 	"github.com/canonical/lxd/client"
 	"github.com/canonical/lxd/shared"
+	cli "github.com/canonical/lxd/shared/cmd"
 )
 
-type cmdSql struct {
-	global *cmdGlobal
+type cmdSQL struct {
+	global     *cmdGlobal
+	flagFormat string
 }
 
-func (c *cmdSql) Command() *cobra.Command {
+func (c *cmdSQL) command() *cobra.Command {
 	cmd := &cobra.Command{}
 	cmd.Use = "sql <local|global> <query>"
 	cmd.Short = "Execute a SQL query against the LXD local or global database"
@@ -50,13 +51,14 @@ func (c *cmdSql) Command() *cobra.Command {
   This command targets the global LXD database and works in both local
   and cluster mode.
 `
-	cmd.RunE = c.Run
+	cmd.RunE = c.run
 	cmd.Hidden = true
+	cmd.Flags().StringVarP(&c.flagFormat, "format", "f", cli.TableFormatSQLResult, `Format (sql|csv|json|table|yaml|compact) (default "sql")`)
 
 	return cmd
 }
 
-func (c *cmdSql) Run(cmd *cobra.Command, args []string) error {
+func (c *cmdSQL) run(cmd *cobra.Command, args []string) error {
 	if len(args) != 2 {
 		_ = cmd.Help()
 
@@ -97,7 +99,7 @@ func (c *cmdSql) Run(cmd *cobra.Command, args []string) error {
 	}
 
 	if query == ".dump" || query == ".schema" {
-		url := fmt.Sprintf("/internal/sql?database=%s", database)
+		url := "/internal/sql?database=" + database
 		if query == ".schema" {
 			url += "&schema=1"
 		}
@@ -139,7 +141,10 @@ func (c *cmdSql) Run(cmd *cobra.Command, args []string) error {
 		}
 
 		if result.Type == "select" {
-			sqlPrintSelectResult(result)
+			err = sqlPrintSelectResult(c.flagFormat, result)
+			if err != nil {
+				return err
+			}
 		} else {
 			fmt.Printf("Rows affected: %d\n", result.RowsAffected)
 		}
@@ -151,20 +156,16 @@ func (c *cmdSql) Run(cmd *cobra.Command, args []string) error {
 	return nil
 }
 
-func sqlPrintSelectResult(result internalSQLResult) {
-	table := tablewriter.NewWriter(os.Stdout)
-	table.SetAlignment(tablewriter.ALIGN_LEFT)
-	table.SetAutoWrapText(false)
-	table.SetAutoFormatHeaders(false)
-	table.SetHeader(result.Columns)
+func sqlPrintSelectResult(format string, result internalSQLResult) error {
+	data := make([][]string, 0, len(result.Rows))
 	for _, row := range result.Rows {
-		data := []string{}
+		r := make([]string, 0, len(row))
 		for _, col := range row {
-			data = append(data, fmt.Sprintf("%v", col))
+			r = append(r, fmt.Sprint(col))
 		}
 
-		table.Append(data)
+		data = append(data, r)
 	}
 
-	table.Render()
+	return cli.RenderTable(format, result.Columns, data, result)
 }
