@@ -2,6 +2,7 @@ package main
 
 import (
 	"context"
+	"fmt"
 	"os"
 
 	"github.com/stretchr/testify/require"
@@ -72,8 +73,8 @@ func (suite *lxdTestSuite) SetupTest() {
 	poolConfig := map[string]string{}
 
 	// Create the database entry for the storage pool.
-	poolDescription := lxdTestSuiteDefaultStoragePool + " storage pool"
-	_, err = dbStoragePoolCreateAndUpdateCache(context.Background(), suite.d.State(), lxdTestSuiteDefaultStoragePool, poolDescription, "mock", poolConfig)
+	poolDescription := fmt.Sprintf("%s storage pool", lxdTestSuiteDefaultStoragePool)
+	_, err = dbStoragePoolCreateAndUpdateCache(suite.d.State(), lxdTestSuiteDefaultStoragePool, poolDescription, "mock", poolConfig)
 	if err != nil {
 		suite.T().Errorf("failed to create default storage pool: %v", err)
 	}
