@@ -2,17 +2,14 @@ package main
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"io"
 	"net/url"
-	"slices"
 	"strings"
 	"time"
 
 	"google.golang.org/protobuf/proto"
 
-	"github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/migration"
 	"github.com/canonical/lxd/lxd/operations"
 	"github.com/canonical/lxd/lxd/state"
@@ -74,7 +71,7 @@ func newStorageMigrationSource(volumeOnly bool, pushTarget *api.StorageVolumePos
 func (s *migrationSourceWs) DoStorage(state *state.State, projectName string, poolName string, volName string, migrateOp *operations.Operation) error {
 	l := logger.AddContext(logger.Ctx{"project": projectName, "pool": poolName, "volume": volName, "push": s.pushOperationURL != ""})
 
-	ctx, cancel := context.WithTimeout(state.ShutdownCtx, time.Second*10)
+	ctx, cancel := context.WithTimeout(context.TODO(), time.Second*10)
 	defer cancel()
 
 	l.Info("Waiting for migration connections on source")
@@ -100,21 +97,16 @@ func (s *migrationSourceWs) DoStorage(state *state.State, projectName string, po
 
 	srcConfig, err := pool.GenerateCustomVolumeBackupConfig(projectName, volName, !s.volumeOnly, migrateOp)
 	if err != nil {
-		return fmt.Errorf("Failed generating migration config of volume %q in pool %q and project %q: %w", volName, poolName, projectName, err)
-	}
-
-	customVol, err := srcConfig.CustomVolume()
-	if err != nil {
-		return fmt.Errorf("Failed getting the custom volume: %w", err)
+		return fmt.Errorf("Failed generating volume migration config: %w", err)
 	}
 
 	// The refresh argument passed to MigrationTypes() is always set
 	// to false here. The migration source/sender doesn't need to care whether
 	// or not it's doing a refresh as the migration sink/receiver will know
 	// this, and adjust the migration types accordingly.
-	poolMigrationTypes = pool.MigrationTypes(storageDrivers.ContentType(customVol.ContentType), false, !s.volumeOnly)
+	poolMigrationTypes = pool.MigrationTypes(storageDrivers.ContentType(srcConfig.Volume.ContentType), false, !s.volumeOnly)
 	if len(poolMigrationTypes) == 0 {
-		return errors.New("No source migration types available")
+		return fmt.Errorf("No source migration types available")
 	}
 
 	// Convert the pool's migration type options to an offer header to target.
@@ -126,19 +118,19 @@ func (s *migrationSourceWs) DoStorage(state *state.State, projectName string, po
 
 	// Only send snapshots when requested.
 	if !s.volumeOnly {
-		offerHeader.Snapshots = make([]*migration.Snapshot, 0, len(customVol.Snapshots))
-		offerHeader.SnapshotNames = make([]string, 0, len(customVol.Snapshots))
+		offerHeader.Snapshots = make([]*migration.Snapshot, 0, len(srcConfig.VolumeSnapshots))
+		offerHeader.SnapshotNames = make([]string, 0, len(srcConfig.VolumeSnapshots))
 
-		for i := range customVol.Snapshots {
-			offerHeader.SnapshotNames = append(offerHeader.SnapshotNames, customVol.Snapshots[i].Name)
-			offerHeader.Snapshots = append(offerHeader.Snapshots, volumeSnapshotToProtobuf(customVol.Snapshots[i]))
+		for i := range srcConfig.VolumeSnapshots {
+			offerHeader.SnapshotNames = append(offerHeader.SnapshotNames, srcConfig.VolumeSnapshots[i].Name)
+			offerHeader.Snapshots = append(offerHeader.Snapshots, volumeSnapshotToProtobuf(srcConfig.VolumeSnapshots[i]))
 		}
 	}
 
 	// Send offer to target.
 	err = s.send(offerHeader)
 	if err != nil {
-		logger.Error("Failed to send storage volume migration header")
+		logger.Errorf("Failed to send storage volume migration header")
 		s.sendControl(err)
 		return err
 	}
@@ -147,12 +139,12 @@ func (s *migrationSourceWs) DoStorage(state *state.State, projectName string, po
 	respHeader := &migration.MigrationHeader{}
 	err = s.recv(respHeader)
 	if err != nil {
-		logger.Error("Failed to receive storage volume migration header")
+		logger.Errorf("Failed to receive storage volume migration header")
 		s.sendControl(err)
 		return err
 	}
 
-	migrationTypes, err := migration.MatchTypes(respHeader, storagePools.FallbackMigrationType(storageDrivers.ContentType(customVol.ContentType)), poolMigrationTypes)
+	migrationTypes, err := migration.MatchTypes(respHeader, storagePools.FallbackMigrationType(storageDrivers.ContentType(srcConfig.Volume.ContentType)), poolMigrationTypes)
 	if err != nil {
 		logger.Errorf("Failed to negotiate migration type: %v", err)
 		s.sendControl(err)
@@ -161,11 +153,11 @@ func (s *migrationSourceWs) DoStorage(state *state.State, projectName string, po
 
 	volSourceArgs := &migration.VolumeSourceArgs{
 		IndexHeaderVersion: respHeader.GetIndexHeaderVersion(), // Enable index header frame if supported.
-		Name:               customVol.Name,
+		Name:               srcConfig.Volume.Name,
 		MigrationType:      migrationTypes[0],
 		Snapshots:          offerHeader.SnapshotNames,
 		TrackProgress:      true,
-		ContentType:        customVol.ContentType,
+		ContentType:        srcConfig.Volume.ContentType,
 		Info:               &migration.Info{Config: srcConfig},
 		VolumeOnly:         s.volumeOnly,
 	}
@@ -174,18 +166,18 @@ func (s *migrationSourceWs) DoStorage(state *state.State, projectName string, po
 	if respHeader.GetRefresh() {
 		volSourceArgs.Refresh = true
 		volSourceArgs.Snapshots = respHeader.GetSnapshotNames()
-		allSnapshots := customVol.Snapshots
+		allSnapshots := volSourceArgs.Info.Config.VolumeSnapshots
 
 		// Ensure that only the requested snapshots are included in the migration index header.
-		customVol.Snapshots = make([]*api.StorageVolumeSnapshot, 0, len(volSourceArgs.Snapshots))
+		volSourceArgs.Info.Config.VolumeSnapshots = make([]*api.StorageVolumeSnapshot, 0, len(volSourceArgs.Snapshots))
 		for i := range allSnapshots {
-			if slices.Contains(volSourceArgs.Snapshots, allSnapshots[i].Name) {
-				customVol.Snapshots = append(customVol.Snapshots, allSnapshots[i])
+			if shared.ValueInSlice(allSnapshots[i].Name, volSourceArgs.Snapshots) {
+				volSourceArgs.Info.Config.VolumeSnapshots = append(volSourceArgs.Info.Config.VolumeSnapshots, allSnapshots[i])
 			}
 		}
 	}
 
-	fsConn, err := s.conns[api.SecretNameFilesystem].WebsocketIO(state.ShutdownCtx)
+	fsConn, err := s.conns[api.SecretNameFilesystem].WebsocketIO(context.TODO())
 	if err != nil {
 		return err
 	}
@@ -199,16 +191,16 @@ func (s *migrationSourceWs) DoStorage(state *state.State, projectName string, po
 	msg := migration.MigrationControl{}
 	err = s.recv(&msg)
 	if err != nil {
-		logger.Error("Failed to receive storage volume migration control message")
+		logger.Errorf("Failed to receive storage volume migration control message")
 		return err
 	}
 
 	if !msg.GetSuccess() {
-		logger.Error("Failed to send storage volume")
-		return errors.New(msg.GetMessage())
+		logger.Errorf("Failed to send storage volume")
+		return fmt.Errorf(msg.GetMessage())
 	}
 
-	logger.Debug("Migration source finished transferring storage volume")
+	logger.Debugf("Migration source finished transferring storage volume")
 	return nil
 }
 
@@ -254,7 +246,7 @@ func newStorageMigrationSink(args *migrationSinkArgs) (*migrationSink, error) {
 func (c *migrationSink) DoStorage(state *state.State, projectName string, poolName string, req *api.StorageVolumesPost, op *operations.Operation) error {
 	l := logger.AddContext(logger.Ctx{"project": projectName, "pool": poolName, "volume": req.Name, "push": c.push})
 
-	ctx, cancel := context.WithTimeout(state.ShutdownCtx, time.Second*10)
+	ctx, cancel := context.WithTimeout(context.TODO(), time.Second*10)
 	defer cancel()
 
 	l.Info("Waiting for migration connections on target")
@@ -277,7 +269,7 @@ func (c *migrationSink) DoStorage(state *state.State, projectName string, poolNa
 	offerHeader := &migration.MigrationHeader{}
 	err := c.recv(offerHeader)
 	if err != nil {
-		logger.Error("Failed to receive storage volume migration header")
+		logger.Errorf("Failed to receive storage volume migration header")
 		c.sendControl(err)
 		return err
 	}
@@ -290,12 +282,15 @@ func (c *migrationSink) DoStorage(state *state.State, projectName string, poolNa
 		return err
 	}
 
-	dbContentType, err := cluster.StoragePoolVolumeContentTypeFromName(req.ContentType)
+	dbContentType, err := storagePools.VolumeContentTypeNameToContentType(req.ContentType)
 	if err != nil {
 		return err
 	}
 
-	contentType := storagePools.VolumeDBContentTypeToContentType(dbContentType)
+	contentType, err := storagePools.VolumeDBContentTypeToContentType(dbContentType)
+	if err != nil {
+		return err
+	}
 
 	// The source/sender will never set Refresh. However, to determine the correct migration type
 	// Refresh needs to be set.
@@ -315,7 +310,10 @@ func (c *migrationSink) DoStorage(state *state.State, projectName string, poolNa
 
 	// Respond with our maximum supported header version if the requested version is higher than ours.
 	// Otherwise just return the requested header version to the source.
-	indexHeaderVersion := min(offerHeader.GetIndexHeaderVersion(), migration.IndexHeaderVersion)
+	indexHeaderVersion := offerHeader.GetIndexHeaderVersion()
+	if indexHeaderVersion > migration.IndexHeaderVersion {
+		indexHeaderVersion = migration.IndexHeaderVersion
+	}
 
 	respHeader.IndexHeaderVersion = &indexHeaderVersion
 	respHeader.SnapshotNames = offerHeader.SnapshotNames
@@ -411,7 +409,7 @@ func (c *migrationSink) DoStorage(state *state.State, projectName string, poolNa
 
 	err = c.send(respHeader)
 	if err != nil {
-		logger.Error("Failed to send storage volume migration header")
+		logger.Errorf("Failed to send storage volume migration header")
 		c.sendControl(err)
 		return err
 	}
@@ -436,7 +434,7 @@ func (c *migrationSink) DoStorage(state *state.State, projectName string, poolNa
 				refresh:       c.refresh,
 			}
 
-			fsConn, err := c.conns[api.SecretNameFilesystem].WebsocketIO(state.ShutdownCtx)
+			fsConn, err := c.conns[api.SecretNameFilesystem].WebsocketIO(context.TODO())
 			if err != nil {
 				fsTransfer <- err
 				return
@@ -482,7 +480,7 @@ func (c *migrationSink) DoStorage(state *state.State, projectName string, poolNa
 			if !msg.GetSuccess() {
 				c.disconnect()
 
-				return errors.New(msg.GetMessage())
+				return fmt.Errorf(msg.GetMessage())
 			}
 
 			// The source can only tell us it failed (e.g. if
