@@ -4,11 +4,9 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"io"
 	"net"
-	"slices"
 	"strconv"
 	"strings"
 	"time"
@@ -71,7 +69,7 @@ func OVNNetworkPrefix(networkID int64) string {
 
 // OVNIntSwitchName returns the internal logical switch name for a Network ID.
 func OVNIntSwitchName(networkID int64) openvswitch.OVNSwitch {
-	return openvswitch.OVNSwitch(OVNNetworkPrefix(networkID) + "-ls-int")
+	return openvswitch.OVNSwitch(fmt.Sprintf("%s-ls-int", OVNNetworkPrefix(networkID)))
 }
 
 // OVNIntSwitchRouterPortName returns OVN logical internal switch router port name.
@@ -314,7 +312,7 @@ func ovnAddReferencedACLs(info *api.NetworkACL, referencedACLNames map[string]st
 				continue // Skip subjects already seen.
 			}
 
-			if slices.Contains(append(ruleSubjectInternalAliases, ruleSubjectExternalAliases...), subject) {
+			if shared.ValueInSlice(subject, append(ruleSubjectInternalAliases, ruleSubjectExternalAliases...)) {
 				continue // Skip special reserved subjects that are not ACL names.
 			}
 
@@ -419,8 +417,8 @@ func ovnApplyToPortGroup(l logger.Logger, client *openvswitch.OVN, aclInfo *api.
 
 		// Setup per-network dynamic replacements for @internal/@external subject port selectors.
 		matchReplace := map[string]string{
-			"@" + ruleSubjectInternal: fmt.Sprintf("@%s", OVNIntSwitchPortGroupName(aclNet.ID)),
-			"@" + ruleSubjectExternal: fmt.Sprintf(`"%s"`, OVNIntSwitchRouterPortName(aclNet.ID)),
+			fmt.Sprintf("@%s", ruleSubjectInternal): fmt.Sprintf("@%s", OVNIntSwitchPortGroupName(aclNet.ID)),
+			fmt.Sprintf("@%s", ruleSubjectExternal): fmt.Sprintf(`"%s"`, OVNIntSwitchRouterPortName(aclNet.ID)),
 		}
 
 		err = client.PortGroupSetACLRules(netPortGroupName, matchReplace, networkRules...)
@@ -496,7 +494,7 @@ func ovnRuleCriteriaToOVNACLRule(direction string, rule *api.NetworkACLRule, por
 	}
 
 	// Add protocol filters.
-	if slices.Contains([]string{"tcp", "udp"}, rule.Protocol) {
+	if shared.ValueInSlice(rule.Protocol, []string{"tcp", "udp"}) {
 		matchParts = append(matchParts, rule.Protocol)
 
 		if rule.SourcePort != "" {
@@ -506,7 +504,7 @@ func ovnRuleCriteriaToOVNACLRule(direction string, rule *api.NetworkACLRule, por
 		if rule.DestinationPort != "" {
 			matchParts = append(matchParts, ovnRulePortToOVNACLMatch(rule.Protocol, "dst", shared.SplitNTrimSpace(rule.DestinationPort, ",", -1, false)...))
 		}
-	} else if slices.Contains([]string{"icmp4", "icmp6"}, rule.Protocol) {
+	} else if shared.ValueInSlice(rule.Protocol, []string{"icmp4", "icmp6"}) {
 		matchParts = append(matchParts, rule.Protocol)
 
 		if rule.ICMPType != "" {
@@ -582,13 +580,13 @@ func ovnRuleSubjectToOVNACLMatch(direction string, aclNameIDs map[string]int64,
 			} else {
 				// If not valid IP subnet, check if subject is ACL name or network peer name.
 				var subjectPortSelector openvswitch.OVNPortGroup
-				if slices.Contains(ruleSubjectInternalAliases, subjectCriterion) {
+				if shared.ValueInSlice(subjectCriterion, ruleSubjectInternalAliases) {
 					// Use pseudo port group name for special reserved port selector types.
 					// These will be expanded later for each network specific rule.
 					// Convert deprecated #internal to non-deprecated @internal if needed.
 					subjectPortSelector = openvswitch.OVNPortGroup(ruleSubjectInternal)
 					networkSpecific = true
-				} else if slices.Contains(ruleSubjectExternalAliases, subjectCriterion) {
+				} else if shared.ValueInSlice(subjectCriterion, ruleSubjectExternalAliases) {
 					// Use pseudo port group name for special reserved port selector types.
 					// These will be expanded later for each network specific rule.
 					// Convert deprecated #external to non-deprecated @external if needed.
@@ -843,7 +841,7 @@ func OVNPortGroupDeleteIfUnused(s *state.State, l logger.Logger, client *openvsw
 			ignoreInst, isIgnoreInst := ignoreUsageType.(instance.Instance)
 
 			if isIgnoreInst && ignoreUsageNicName == "" {
-				return errors.New("ignoreUsageNicName should be specified when providing an instance in ignoreUsageType")
+				return fmt.Errorf("ignoreUsageNicName should be specified when providing an instance in ignoreUsageType")
 			}
 
 			// If an ignore instance was provided, then skip the device that the ACLs were just removed
@@ -871,7 +869,7 @@ func OVNPortGroupDeleteIfUnused(s *state.State, l logger.Logger, client *openvsw
 			ignoreNet, isIgnoreNet := ignoreUsageType.(*api.Network)
 
 			if isIgnoreNet && ignoreUsageNicName != "" {
-				return errors.New("ignoreUsageNicName should be empty when providing a network in ignoreUsageType")
+				return fmt.Errorf("ignoreUsageNicName should be empty when providing a network in ignoreUsageType")
 			}
 
 			// If an ignore network was provided, then skip the network that the ACLs were just removed
@@ -899,7 +897,7 @@ func OVNPortGroupDeleteIfUnused(s *state.State, l logger.Logger, client *openvsw
 			ignoreProfile, isIgnoreProfile := ignoreUsageType.(cluster.Profile)
 
 			if isIgnoreProfile && ignoreUsageNicName == "" {
-				return errors.New("ignoreUsageNicName should be specified when providing a profile in ignoreUsageType")
+				return fmt.Errorf("ignoreUsageNicName should be specified when providing a profile in ignoreUsageType")
 			}
 
 			// If an ignore profile was provided, then skip the device that the ACLs were just removed
@@ -930,7 +928,7 @@ func OVNPortGroupDeleteIfUnused(s *state.State, l logger.Logger, client *openvsw
 					aclUsedACLS[matchedACLName] = make([]string, 0, 1)
 				}
 
-				if !slices.Contains(aclUsedACLS[matchedACLName], u.Name) {
+				if !shared.ValueInSlice(u.Name, aclUsedACLS[matchedACLName]) {
 					// Record as in use by another ACL entity.
 					aclUsedACLS[matchedACLName] = append(aclUsedACLS[matchedACLName], u.Name)
 				}
@@ -998,11 +996,11 @@ func OVNPortGroupInstanceNICSchedule(portUUID openvswitch.OVNSwitchPortUUID, cha
 
 // OVNApplyInstanceNICDefaultRules applies instance NIC default rules to per-network port group.
 func OVNApplyInstanceNICDefaultRules(client *openvswitch.OVN, switchPortGroup openvswitch.OVNPortGroup, logPrefix string, nicPortName openvswitch.OVNSwitchPort, ingressAction string, ingressLogged bool, egressAction string, egressLogged bool) error {
-	if !slices.Contains(ValidActions, ingressAction) {
+	if !shared.ValueInSlice(ingressAction, ValidActions) {
 		return fmt.Errorf("Invalid ingress action %q", ingressAction)
 	}
 
-	if !slices.Contains(ValidActions, egressAction) {
+	if !shared.ValueInSlice(egressAction, ValidActions) {
 		return fmt.Errorf("Invalid egress action %q", egressAction)
 	}
 
@@ -1011,7 +1009,7 @@ func OVNApplyInstanceNICDefaultRules(client *openvswitch.OVN, switchPortGroup op
 			Direction: "to-lport",
 			Action:    egressAction,
 			Log:       egressLogged,
-			LogName:   logPrefix + "-egress", // Max 63 chars.
+			LogName:   fmt.Sprintf("%s-egress", logPrefix), // Max 63 chars.
 			Priority:  ovnACLPriorityNICDefaultActionEgress,
 			Match:     fmt.Sprintf(`inport == "%s"`, nicPortName), // From NIC.
 		},
@@ -1019,7 +1017,7 @@ func OVNApplyInstanceNICDefaultRules(client *openvswitch.OVN, switchPortGroup op
 			Direction: "to-lport",
 			Action:    ingressAction,
 			Log:       ingressLogged,
-			LogName:   logPrefix + "-ingress", // Max 63 chars.
+			LogName:   fmt.Sprintf("%s-ingress", logPrefix), // Max 63 chars.
 			Priority:  ovnACLPriorityNICDefaultActionIngress,
 			Match:     fmt.Sprintf(`outport == "%s"`, nicPortName), // To NIC.
 		},
