@@ -3,11 +3,9 @@ package acl
 import (
 	"bufio"
 	"context"
-	"errors"
 	"fmt"
 	"net"
 	"os"
-	"slices"
 	"sort"
 	"strings"
 	"sync"
@@ -114,7 +112,6 @@ func (d *common) Info() *api.NetworkACL {
 	info.Egress = append(make([]api.NetworkACLRule, 0, len(d.info.Egress)), d.info.Egress...)
 	info.Config = util.CopyConfig(d.info.Config)
 	info.UsedBy = nil // To indicate its not populated (use Usedby() function to populate).
-	info.Project = d.projectName
 
 	return &info
 }
@@ -130,28 +127,28 @@ func (d *common) usedBy(firstOnly bool) ([]string, error) {
 		case db.InstanceArgs:
 			uri := fmt.Sprintf("/%s/instances/%s", version.APIVersion, u.Name)
 			if u.Project != api.ProjectDefaultName {
-				uri += "?project=" + u.Project
+				uri += fmt.Sprintf("?project=%s", u.Project)
 			}
 
 			usedBy = append(usedBy, uri)
 		case *api.Network:
 			uri := fmt.Sprintf("/%s/networks/%s", version.APIVersion, u.Name)
 			if d.projectName != api.ProjectDefaultName {
-				uri += "?project=" + d.projectName
+				uri += fmt.Sprintf("?project=%s", d.projectName)
 			}
 
 			usedBy = append(usedBy, uri)
 		case dbCluster.Profile:
 			uri := fmt.Sprintf("/%s/profiles/%s", version.APIVersion, u.Name)
 			if u.Project != api.ProjectDefaultName {
-				uri += "?project=" + u.Project
+				uri += fmt.Sprintf("?project=%s", u.Project)
 			}
 
 			usedBy = append(usedBy, uri)
 		case *api.NetworkACL:
 			uri := fmt.Sprintf("/%s/network-acls/%s", version.APIVersion, u.Name)
 			if d.projectName != api.ProjectDefaultName {
-				uri += "?project=" + d.projectName
+				uri += fmt.Sprintf("?project=%s", d.projectName)
 			}
 
 			usedBy = append(usedBy, uri)
@@ -292,13 +289,13 @@ func (d *common) validateConfigMap(config map[string]string, rules map[string]fu
 // validateRule validates the rule supplied.
 func (d *common) validateRule(direction ruleDirection, rule api.NetworkACLRule) error {
 	// Validate Action field (required).
-	if !slices.Contains(ValidActions, rule.Action) {
+	if !shared.ValueInSlice(rule.Action, ValidActions) {
 		return fmt.Errorf("Action must be one of: %s", strings.Join(ValidActions, ", "))
 	}
 
 	// Validate State field (required).
 	validStates := []string{"enabled", "disabled", "logged"}
-	if !slices.Contains(validStates, rule.State) {
+	if !shared.ValueInSlice(rule.State, validStates) {
 		return fmt.Errorf("State must be one of: %s", strings.Join(validStates, ", "))
 	}
 
@@ -349,26 +346,26 @@ func (d *common) validateRule(direction ruleDirection, rule api.NetworkACLRule)
 			(dstHasIPv4 && !srcHasIPv4 && !srcHasName) ||
 			(srcHasIPv6 && !dstHasIPv6 && !dstHasName) ||
 			(dstHasIPv6 && !srcHasIPv6 && !srcHasName) {
-			return errors.New("Conflicting IP family types used for Source and Destination")
+			return fmt.Errorf("Conflicting IP family types used for Source and Destination")
 		}
 	}
 
 	// Validate Protocol field.
 	if rule.Protocol != "" {
 		validProtocols := []string{"icmp4", "icmp6", "tcp", "udp"}
-		if !slices.Contains(validProtocols, rule.Protocol) {
+		if !shared.ValueInSlice(rule.Protocol, validProtocols) {
 			return fmt.Errorf("Protocol must be one of: %s", strings.Join(validProtocols, ", "))
 		}
 	}
 
 	// Validate protocol dependent fields.
-	if slices.Contains([]string{"tcp", "udp"}, rule.Protocol) {
+	if shared.ValueInSlice(rule.Protocol, []string{"tcp", "udp"}) {
 		if rule.ICMPType != "" {
-			return errors.New("ICMP type cannot be used with non-ICMP protocol")
+			return fmt.Errorf("ICMP type cannot be used with non-ICMP protocol")
 		}
 
 		if rule.ICMPCode != "" {
-			return errors.New("ICMP code cannot be used with non-ICMP protocol")
+			return fmt.Errorf("ICMP code cannot be used with non-ICMP protocol")
 		}
 
 		// Validate SourcePort field.
@@ -386,7 +383,7 @@ func (d *common) validateRule(direction ruleDirection, rule api.NetworkACLRule)
 				return fmt.Errorf("Invalid Destination port: %w", err)
 			}
 		}
-	} else if slices.Contains([]string{"icmp4", "icmp6"}, rule.Protocol) {
+	} else if shared.ValueInSlice(rule.Protocol, []string{"icmp4", "icmp6"}) {
 		if rule.SourcePort != "" {
 			return fmt.Errorf("Source port cannot be used with %q protocol", rule.Protocol)
 		}
@@ -395,8 +392,7 @@ func (d *common) validateRule(direction ruleDirection, rule api.NetworkACLRule)
 			return fmt.Errorf("Destination port cannot be used with %q protocol", rule.Protocol)
 		}
 
-		switch rule.Protocol {
-		case "icmp4":
+		if rule.Protocol == "icmp4" {
 			if srcHasIPv6 {
 				return fmt.Errorf("Cannot use IPv6 source addresses with %q protocol", rule.Protocol)
 			}
@@ -404,8 +400,7 @@ func (d *common) validateRule(direction ruleDirection, rule api.NetworkACLRule)
 			if dstHasIPv6 {
 				return fmt.Errorf("Cannot use IPv6 destination addresses with %q protocol", rule.Protocol)
 			}
-
-		case "icmp6":
+		} else if rule.Protocol == "icmp6" {
 			if srcHasIPv4 {
 				return fmt.Errorf("Cannot use IPv4 source addresses with %q protocol", rule.Protocol)
 			}
@@ -432,19 +427,19 @@ func (d *common) validateRule(direction ruleDirection, rule api.NetworkACLRule)
 		}
 	} else {
 		if rule.ICMPType != "" {
-			return errors.New("ICMP type cannot be used without specifying protocol")
+			return fmt.Errorf("ICMP type cannot be used without specifying protocol")
 		}
 
 		if rule.ICMPCode != "" {
-			return errors.New("ICMP code cannot be used without specifying protocol")
+			return fmt.Errorf("ICMP code cannot be used without specifying protocol")
 		}
 
 		if rule.SourcePort != "" {
-			return errors.New("Source port cannot be used without specifying protocol")
+			return fmt.Errorf("Source port cannot be used without specifying protocol")
 		}
 
 		if rule.DestinationPort != "" {
-			return errors.New("Destination port cannot be used without specifying protocol")
+			return fmt.Errorf("Destination port cannot be used without specifying protocol")
 		}
 	}
 
@@ -456,7 +451,10 @@ func (d *common) validateRule(direction ruleDirection, rule api.NetworkACLRule)
 // Returns whether the subjects include names, IPv4 and IPv6 addresses respectively.
 func (d *common) validateRuleSubjects(fieldName string, direction ruleDirection, subjects []string, validSubjectNames []string) (hasName bool, hasIPv4 bool, hasIPv6 bool, err error) {
 	// Check if named subjects are allowed in field/direction combination.
-	allowSubjectNames := (fieldName == "Source" && direction == ruleDirectionIngress) || (fieldName == "Destination" && direction == ruleDirectionEgress)
+	allowSubjectNames := false
+	if (fieldName == "Source" && direction == ruleDirectionIngress) || (fieldName == "Destination" && direction == ruleDirectionEgress) {
+		allowSubjectNames = true
+	}
 
 	isNetworkAddress := func(value string) (uint, error) {
 		ip := net.ParseIP(value)
@@ -494,7 +492,7 @@ func (d *common) validateRuleSubjects(fieldName string, direction ruleDirection,
 
 		ips := strings.SplitN(value, "-", 2)
 		if len(ips) != 2 {
-			return 0, errors.New("IP range must contain start and end IP addresses")
+			return 0, fmt.Errorf("IP range must contain start and end IP addresses")
 		}
 
 		ip := net.ParseIP(ips[0])
@@ -704,7 +702,7 @@ func (d *common) Update(config *api.NetworkACLPut, clientType request.ClientType
 func (d *common) Rename(newName string) error {
 	_, err := LoadByName(d.state, d.projectName, newName)
 	if err == nil {
-		return errors.New("An ACL by that name exists already")
+		return fmt.Errorf("An ACL by that name exists already")
 	}
 
 	isUsed, err := d.isUsed()
@@ -713,7 +711,7 @@ func (d *common) Rename(newName string) error {
 	}
 
 	if isUsed {
-		return errors.New("Cannot rename an ACL that is in use")
+		return fmt.Errorf("Cannot rename an ACL that is in use")
 	}
 
 	err = d.validateName(newName)
@@ -742,7 +740,7 @@ func (d *common) Delete() error {
 	}
 
 	if isUsed {
-		return errors.New("Cannot delete an ACL that is in use")
+		return fmt.Errorf("Cannot delete an ACL that is in use")
 	}
 
 	return d.state.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
@@ -769,7 +767,7 @@ func (d *common) GetLog(ctx context.Context, clientType request.ClientType) (str
 		prefix := fmt.Sprintf("lxd_acl%d-", d.id)
 		logPath := shared.HostPath("/var/log/ovn/ovn-controller.log")
 		if !shared.PathExists(logPath) {
-			return "", errors.New("Only OVN log entries may be retrieved at this time")
+			return "", fmt.Errorf("Only OVN log entries may be retrieved at this time")
 		}
 
 		// Open the log file.
