@@ -15,6 +15,7 @@ import (
 	"github.com/canonical/lxd/lxd/db"
 	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/network/acl"
+	"github.com/canonical/lxd/lxd/project/limits"
 	"github.com/canonical/lxd/lxd/resources"
 	"github.com/canonical/lxd/lxd/state"
 	"github.com/canonical/lxd/shared"
@@ -63,6 +64,7 @@ const (
 	subnetUsageNetworkLoadBalancer
 	subnetUsageInstance
 	subnetUsageProxy
+	subnetUsageVolatileIP
 )
 
 // externalSubnetUsage represents usage of a subnet by a network or NIC.
@@ -382,7 +384,7 @@ func (n *common) update(applyNetwork api.NetworkPut, targetNode string, clientTy
 				sendNetwork.Config[k] = v
 			}
 
-			err = notifier(func(client lxd.InstanceServer) error {
+			err = notifier(func(member db.NodeInfo, client lxd.InstanceServer) error {
 				return client.UseProject(n.project).UpdateNetwork(n.name, sendNetwork, "")
 			})
 			if err != nil {
@@ -492,7 +494,7 @@ func (n *common) warningsDelete() error {
 }
 
 // delete the network on local server.
-func (n *common) delete(clientType request.ClientType) error {
+func (n *common) delete() error {
 	// Delete any persistent warnings for network.
 	err := n.warningsDelete()
 	if err != nil {
@@ -609,7 +611,7 @@ func (n *common) bgpValidationRules(config map[string]string) (map[string]func(v
 		case "asn":
 			rules[k] = validate.Optional(validate.IsInRange(1, 4294967294))
 		case "password":
-			rules[k] = validate.Optional(validate.IsAny)
+			rules[k] = validate.IsAny
 		case "holdtime":
 			rules[k] = validate.Optional(validate.IsInRange(9, 65535))
 		}
@@ -828,6 +830,28 @@ func (n *common) bgpGetPeers(config map[string]string) []string {
 	return peers
 }
 
+// projectUplinkIPQuotaAvailable checks if a project has quota available to assign new uplink IPs in a certain network.
+func (n *common) projectUplinkIPQuotaAvailable(ctx context.Context, tx *db.ClusterTx, p *api.Project, uplinkName string) (ipv4QuotaAvailable bool, ipv6QuotaAvailable bool, err error) {
+	rawIPV4Quota, hasIPV4Quota := p.Config["limits.networks.uplink_ips.ipv4."+uplinkName]
+	rawIPV6Quota, hasIPV6Quota := p.Config["limits.networks.uplink_ips.ipv6."+uplinkName]
+
+	// Will be 0 if the limit is not set.
+	ipv4AddressLimit, _ := strconv.Atoi(rawIPV4Quota)
+	ipv6AddressLimit, _ := strconv.Atoi(rawIPV6Quota)
+
+	var ipv4QuotaMet bool
+	var ipv6QuotaMet bool
+
+	// If limit-1 is exceeded, than that means we have no quota available.
+	ipv4QuotaMet, ipv6QuotaMet, err = limits.UplinkAddressQuotasExceeded(ctx, tx, p.Name, uplinkName, ipv4AddressLimit-1, ipv6AddressLimit-1, nil)
+	if err != nil {
+		return false, false, err
+	}
+
+	// Undefined quotas are always available.
+	return !hasIPV4Quota || !ipv4QuotaMet, !hasIPV6Quota || !ipv6QuotaMet, nil
+}
+
 // forwardValidate validates the forward request.
 func (n *common) forwardValidate(listenAddress net.IP, forward api.NetworkForwardPut) ([]*forwardPortMap, error) {
 	if listenAddress == nil {
@@ -991,8 +1015,8 @@ func (n *common) forwardValidate(listenAddress net.IP, forward api.NetworkForwar
 }
 
 // ForwardCreate returns ErrNotImplemented for drivers that do not support forwards.
-func (n *common) ForwardCreate(forward api.NetworkForwardsPost, clientType request.ClientType) error {
-	return ErrNotImplemented
+func (n *common) ForwardCreate(forward api.NetworkForwardsPost, clientType request.ClientType) (net.IP, error) {
+	return nil, ErrNotImplemented
 }
 
 // ForwardUpdate returns ErrNotImplemented for drivers that do not support forwards.
@@ -1304,8 +1328,8 @@ func (n *common) loadBalancerValidate(listenAddress net.IP, forward api.NetworkL
 }
 
 // LoadBalancerCreate returns ErrNotImplemented for drivers that do not support load balancers.
-func (n *common) LoadBalancerCreate(loadBalancer api.NetworkLoadBalancersPost, clientType request.ClientType) error {
-	return ErrNotImplemented
+func (n *common) LoadBalancerCreate(loadBalancer api.NetworkLoadBalancersPost, clientType request.ClientType) (net.IP, error) {
+	return nil, ErrNotImplemented
 }
 
 // LoadBalancerUpdate returns ErrNotImplemented for drivers that do not support load balancers..
