@@ -6,13 +6,11 @@ import (
 	"context"
 	cryptorand "crypto/rand"
 	"encoding/hex"
-	"errors"
 	"fmt"
 	"math/big"
 	"math/rand"
 	"net"
 	"os"
-	"slices"
 	"strconv"
 	"strings"
 	"sync"
@@ -76,7 +74,7 @@ func RandomDevName(prefix string) string {
 // MACDevName returns interface name with prefix 'lxd' and MAC without leading 2 digits.
 func MACDevName(mac net.HardwareAddr) string {
 	devName := strings.Join(strings.Split(mac.String(), ":"), "")
-	return "lxd" + devName[2:]
+	return fmt.Sprintf("lxd%s", devName[2:])
 }
 
 // UsedByInstanceDevices looks for instance NIC devices using the network and runs the supplied usageFunc for each.
@@ -362,7 +360,7 @@ func DefaultGatewaySubnetV4() (*net.IPNet, string, error) {
 	}
 
 	if ifaceName == "" {
-		return nil, "", errors.New("No default gateway for IPv4")
+		return nil, "", fmt.Errorf("No default gateway for IPv4")
 	}
 
 	iface, err := net.InterfaceByName(ifaceName)
@@ -388,14 +386,14 @@ func DefaultGatewaySubnetV4() (*net.IPNet, string, error) {
 		}
 
 		if subnet != nil {
-			return nil, "", errors.New("More than one IPv4 subnet on default interface")
+			return nil, "", fmt.Errorf("More than one IPv4 subnet on default interface")
 		}
 
 		subnet = addrNet
 	}
 
 	if subnet == nil {
-		return nil, "", errors.New("No IPv4 subnet on default interface")
+		return nil, "", fmt.Errorf("No IPv4 subnet on default interface")
 	}
 
 	return subnet, ifaceName, nil
@@ -452,7 +450,7 @@ func UpdateDNSMasqStatic(s *state.State, networkName string) error {
 			}
 
 			// Skip devices not connected to managed networks.
-			if !slices.Contains(networks, d["parent"]) {
+			if !shared.ValueInSlice(d["parent"], networks) {
 				continue
 			}
 
@@ -605,7 +603,7 @@ func ForkdnsServersList(networkName string) ([]string, error) {
 }
 
 func randomSubnetV4() (string, error) {
-	for range 100 {
+	for i := 0; i < 100; i++ {
 		cidr := fmt.Sprintf("10.%d.%d.1/24", rand.Intn(255), rand.Intn(255))
 		_, subnet, err := net.ParseCIDR(cidr)
 		if err != nil {
@@ -623,11 +621,11 @@ func randomSubnetV4() (string, error) {
 		return cidr, nil
 	}
 
-	return "", errors.New("Failed to automatically find an unused IPv4 subnet, manual configuration required")
+	return "", fmt.Errorf("Failed to automatically find an unused IPv4 subnet, manual configuration required")
 }
 
 func randomSubnetV6() (string, error) {
-	for range 100 {
+	for i := 0; i < 100; i++ {
 		cidr := fmt.Sprintf("fd42:%x:%x:%x::1/64", rand.Intn(65535), rand.Intn(65535), rand.Intn(65535))
 		_, subnet, err := net.ParseCIDR(cidr)
 		if err != nil {
@@ -645,7 +643,7 @@ func randomSubnetV6() (string, error) {
 		return cidr, nil
 	}
 
-	return "", errors.New("Failed to automatically find an unused IPv6 subnet, manual configuration required")
+	return "", fmt.Errorf("Failed to automatically find an unused IPv6 subnet, manual configuration required")
 }
 
 // noAvailableAddressErr is used by randomAddressInSubnet to indicate that the subnet was exhausted while searching for
@@ -775,7 +773,7 @@ func inRoutingTable(subnet *net.IPNet) bool {
 		filename = "ipv6_route"
 	}
 
-	file, err := os.Open("/proc/net/" + filename)
+	file, err := os.Open(fmt.Sprintf("/proc/net/%s", filename))
 	if err != nil {
 		return false
 	}
@@ -860,7 +858,7 @@ func pingIP(ctx context.Context, ip net.IP) error {
 		timeout = time.Until(deadline)
 	}
 
-	_, err := shared.RunCommandContext(ctx, cmd, "-n", "-q", ip.String(), "-c", "1", "-w", strconv.Itoa(int(timeout.Seconds())))
+	_, err := shared.RunCommandContext(ctx, cmd, "-n", "-q", ip.String(), "-c", "1", "-w", fmt.Sprintf("%d", int(timeout.Seconds())))
 
 	return err
 }
@@ -886,7 +884,7 @@ func pingSubnet(subnet *net.IPNet) bool {
 	poke := func(ip net.IP) {
 		defer wgChecks.Done()
 
-		addr := ip.String() + ":22"
+		addr := fmt.Sprintf("%s:22", ip.String())
 		if ip.To4() == nil {
 			addr = fmt.Sprintf("[%s]:22", ip.String())
 		}
@@ -998,7 +996,7 @@ func GetLeaseAddresses(networkName string, hwaddr string) ([]net.IP, error) {
 
 	addresses := []net.IP{}
 
-	for lease := range strings.SplitSeq(string(content), "\n") {
+	for _, lease := range strings.Split(string(content), "\n") {
 		fields := strings.Fields(lease)
 		if len(fields) < 5 {
 			continue
@@ -1152,7 +1150,7 @@ func InterfaceRemove(nic string) error {
 
 // InterfaceExists returns true if network interface exists.
 func InterfaceExists(nic string) bool {
-	if nic != "" && shared.PathExists("/sys/class/net/"+nic) {
+	if nic != "" && shared.PathExists(fmt.Sprintf("/sys/class/net/%s", nic)) {
 		return true
 	}
 
@@ -1310,7 +1308,7 @@ func ParsePortRange(r string) (base int64, size int64, err error) {
 		}
 
 		if size <= base {
-			return -1, -1, errors.New("End port should be higher than start port")
+			return -1, -1, fmt.Errorf("End port should be higher than start port")
 		}
 
 		size -= base
@@ -1383,10 +1381,10 @@ func BridgeNetfilterEnabled(ipVersion uint) error {
 		sysctlName = "ip6tables"
 	}
 
-	sysctlPath := "net/bridge/bridge-nf-call-" + sysctlName
+	sysctlPath := fmt.Sprintf("net/bridge/bridge-nf-call-%s", sysctlName)
 	sysctlVal, err := util.SysctlGet(sysctlPath)
 	if err != nil {
-		return errors.New("br_netfilter kernel module not loaded")
+		return fmt.Errorf("br_netfilter kernel module not loaded")
 	}
 
 	sysctlVal = strings.TrimSpace(sysctlVal)
@@ -1402,12 +1400,12 @@ func ProxyParseAddr(data string) (*deviceConfig.ProxyAddress, error) {
 	// Split into <protocol> and <address>.
 	fields := strings.SplitN(data, ":", 2)
 
-	if !slices.Contains([]string{"tcp", "udp", "unix"}, fields[0]) {
+	if !shared.ValueInSlice(fields[0], []string{"tcp", "udp", "unix"}) {
 		return nil, fmt.Errorf("Unknown protocol type %q", fields[0])
 	}
 
 	if len(fields) < 2 || fields[1] == "" {
-		return nil, errors.New("Missing address")
+		return nil, fmt.Errorf("Missing address")
 	}
 
 	newProxyAddr := &deviceConfig.ProxyAddress{
@@ -1429,7 +1427,7 @@ func ProxyParseAddr(data string) (*deviceConfig.ProxyAddress, error) {
 	}
 
 	// Validate that it's a valid address.
-	if slices.Contains([]string{"udp", "tcp"}, newProxyAddr.ConnType) {
+	if shared.ValueInSlice(newProxyAddr.ConnType, []string{"udp", "tcp"}) {
 		err := validate.Optional(validate.IsNetworkAddress)(address)
 		if err != nil {
 			return nil, err
@@ -1439,7 +1437,7 @@ func ProxyParseAddr(data string) (*deviceConfig.ProxyAddress, error) {
 	newProxyAddr.Address = address
 
 	// Split <ports> into individual ports and port ranges.
-	ports := strings.Split(port, ",")
+	ports := strings.SplitN(port, ",", -1)
 
 	newProxyAddr.Ports = make([]uint64, 0, len(ports))
 
@@ -1449,13 +1447,13 @@ func ProxyParseAddr(data string) (*deviceConfig.ProxyAddress, error) {
 			return nil, err
 		}
 
-		for i := range portRange {
+		for i := int64(0); i < portRange; i++ {
 			newProxyAddr.Ports = append(newProxyAddr.Ports, uint64(portFirst+i))
 		}
 	}
 
 	if len(newProxyAddr.Ports) <= 0 {
-		return nil, errors.New("At least one port is required")
+		return nil, fmt.Errorf("At least one port is required")
 	}
 
 	return newProxyAddr, nil
@@ -1494,7 +1492,7 @@ func AllowedUplinkNetworks(ctx context.Context, tx *db.ClusterTx, projectConfig
 	allowedRestrictedUplinks := shared.SplitNTrimSpace(projectConfig["restricted.networks.uplinks"], ",", -1, false)
 
 	for _, allowedRestrictedUplink := range allowedRestrictedUplinks {
-		if slices.Contains(uplinkNetworkNames, allowedRestrictedUplink) {
+		if shared.ValueInSlice(allowedRestrictedUplink, uplinkNetworkNames) {
 			allowedUplinkNetworkNames = append(allowedUplinkNetworkNames, allowedRestrictedUplink)
 		}
 	}
