@@ -27,14 +27,16 @@ import (
 )
 
 var networkACLsCmd = APIEndpoint{
-	Path: "network-acls",
+	Path:        "network-acls",
+	MetricsType: entity.TypeNetwork,
 
-	Get:  APIEndpointAction{Handler: networkACLsGet, AccessHandler: allowAuthenticated},
+	Get:  APIEndpointAction{Handler: networkACLsGet, AccessHandler: allowProjectResourceList},
 	Post: APIEndpointAction{Handler: networkACLsPost, AccessHandler: allowPermission(entity.TypeProject, auth.EntitlementCanCreateNetworkACLs)},
 }
 
 var networkACLCmd = APIEndpoint{
-	Path: "network-acls/{name}",
+	Path:        "network-acls/{name}",
+	MetricsType: entity.TypeNetwork,
 
 	Delete: APIEndpointAction{Handler: networkACLDelete, AccessHandler: allowPermission(entity.TypeNetworkACL, auth.EntitlementCanDelete, "name")},
 	Get:    APIEndpointAction{Handler: networkACLGet, AccessHandler: allowPermission(entity.TypeNetworkACL, auth.EntitlementCanView, "name")},
@@ -44,7 +46,8 @@ var networkACLCmd = APIEndpoint{
 }
 
 var networkACLLogCmd = APIEndpoint{
-	Path: "network-acls/{name}/log",
+	Path:        "network-acls/{name}/log",
+	MetricsType: entity.TypeNetwork,
 
 	Get: APIEndpointAction{Handler: networkACLLogGet, AccessHandler: allowPermission(entity.TypeNetworkACL, auth.EntitlementCanView, "name")},
 }
@@ -146,12 +149,17 @@ var networkACLLogCmd = APIEndpoint{
 func networkACLsGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	projectName, _, err := project.NetworkProject(s.DB.Cluster, request.ProjectParam(r))
+	requestProjectName := request.ProjectParam(r)
+	effectiveProjectName, _, err := project.NetworkProject(s.DB.Cluster, requestProjectName)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	recursion := util.IsRecursionRequest(r)
+	withEntitlements, err := extractEntitlementsFromQuery(r, entity.TypeNetworkACL, true)
+	if err != nil {
+		return response.SmartError(err)
+	}
 
 	var aclNames []string
 
@@ -159,7 +167,7 @@ func networkACLsGet(d *Daemon, r *http.Request) response.Response {
 		var err error
 
 		// Get list of Network ACLs.
-		aclNames, err = tx.GetNetworkACLs(ctx, projectName)
+		aclNames, err = tx.GetNetworkACLs(ctx, effectiveProjectName)
 
 		return err
 	})
@@ -167,23 +175,24 @@ func networkACLsGet(d *Daemon, r *http.Request) response.Response {
 		return response.InternalError(err)
 	}
 
-	request.SetCtxValue(r, request.CtxEffectiveProjectName, projectName)
-	userHasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), r, auth.EntitlementCanView, entity.TypeNetworkACL)
+	request.SetCtxValue(r, request.CtxEffectiveProjectName, effectiveProjectName)
+	userHasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), auth.EntitlementCanView, entity.TypeNetworkACL)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	resultString := []string{}
-	resultMap := []api.NetworkACL{}
+	resultMap := []*api.NetworkACL{}
+	urlToNetworkACL := make(map[*api.URL]auth.EntitlementReporter)
 	for _, aclName := range aclNames {
-		if !userHasPermission(entity.NetworkACLURL(projectName, aclName)) {
+		if !userHasPermission(entity.NetworkACLURL(requestProjectName, aclName)) {
 			continue
 		}
 
 		if !recursion {
 			resultString = append(resultString, fmt.Sprintf("/%s/network-acls/%s", version.APIVersion, aclName))
 		} else {
-			netACL, err := acl.LoadByName(s, projectName, aclName)
+			netACL, err := acl.LoadByName(s, effectiveProjectName, aclName)
 			if err != nil {
 				continue
 			}
@@ -192,7 +201,8 @@ func networkACLsGet(d *Daemon, r *http.Request) response.Response {
 			netACLInfo.UsedBy, _ = netACL.UsedBy() // Ignore errors in UsedBy, will return nil.
 			netACLInfo.UsedBy = project.FilterUsedBy(s.Authorizer, r, netACLInfo.UsedBy)
 
-			resultMap = append(resultMap, *netACLInfo)
+			resultMap = append(resultMap, netACLInfo)
+			urlToNetworkACL[entity.NetworkACLURL(requestProjectName, aclName)] = netACLInfo
 		}
 	}
 
@@ -200,6 +210,13 @@ func networkACLsGet(d *Daemon, r *http.Request) response.Response {
 		return response.SyncResponse(true, resultString)
 	}
 
+	if len(withEntitlements) > 0 {
+		err = reportEntitlements(r.Context(), s.Authorizer, s.IdentityCache, entity.TypeNetworkACL, withEntitlements, urlToNetworkACL)
+		if err != nil {
+			return response.SmartError(err)
+		}
+	}
+
 	return response.SyncResponse(true, resultMap)
 }
 
@@ -377,6 +394,11 @@ func networkACLGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
+	withEntitlements, err := extractEntitlementsFromQuery(r, entity.TypeNetworkACL, false)
+	if err != nil {
+		return response.SmartError(err)
+	}
+
 	netACL, err := acl.LoadByName(s, projectName, aclName)
 	if err != nil {
 		return response.SmartError(err)
@@ -389,6 +411,13 @@ func networkACLGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	info.UsedBy = project.FilterUsedBy(s.Authorizer, r, info.UsedBy)
+	if len(withEntitlements) > 0 {
+		err = reportEntitlements(r.Context(), s.Authorizer, s.IdentityCache, entity.TypeNetworkACL, withEntitlements, map[*api.URL]auth.EntitlementReporter{entity.NetworkACLURL(projectName, aclName): info})
+		if err != nil {
+			return response.SmartError(err)
+		}
+	}
+
 	return response.SyncResponseETag(true, info, netACL.Etag())
 }
 
@@ -633,7 +662,7 @@ func networkACLLogGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	clientType := clusterRequest.UserAgentClientType(r.Header.Get("User-Agent"))
-	log, err := netACL.GetLog(clientType)
+	log, err := netACL.GetLog(r.Context(), clientType)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -643,5 +672,5 @@ func networkACLLogGet(d *Daemon, r *http.Request) response.Response {
 	ent.FileModified = time.Now()
 	ent.FileSize = int64(len(log))
 
-	return response.FileResponse(r, []response.FileResponseEntry{ent}, nil)
+	return response.FileResponse([]response.FileResponseEntry{ent}, nil)
 }
