@@ -6,7 +6,6 @@ import (
 	"fmt"
 	"net"
 	"net/http"
-	"slices"
 	"strings"
 
 	"github.com/canonical/lxd/lxd/auth"
@@ -79,24 +78,20 @@ var networkAllocationsCmd = APIEndpoint{
 func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	requestProjectName, allProjects, err := request.ProjectParams(r)
+	requestProjectName := request.ProjectParam(r)
+	effectiveProjectName, _, err := project.NetworkProject(d.State().DB.Cluster, requestProjectName)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	reqInfo := request.GetContextInfo(r.Context())
-	if !allProjects {
-		reqInfo.EffectiveProjectName, _, err = project.NetworkProject(s.DB.Cluster, requestProjectName)
-		if err != nil {
-			return response.SmartError(err)
-		}
-	}
+	request.SetCtxValue(r, request.CtxEffectiveProjectName, effectiveProjectName)
+	allProjects := shared.IsTrue(request.QueryParam(r, "all-projects"))
 
 	var projectNames []string
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Figure out the projects to retrieve.
 		if !allProjects {
-			projectNames = []string{reqInfo.EffectiveProjectName}
+			projectNames = []string{effectiveProjectName}
 		} else {
 			// Get all project names if no specific project requested.
 			projectNames, err = dbCluster.GetProjectNames(ctx, tx.Tx())
@@ -120,29 +115,15 @@ func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if ip.To4() != nil {
-			return ip.String() + "/32", shared.IsTrue(netConf["ipv4.nat"]), nil
+			return fmt.Sprintf("%s/32", ip.String()), shared.IsTrue(netConf["ipv4.nat"]), nil
 		}
 
-		return ip.String() + "/128", shared.IsTrue(netConf["ipv6.nat"]), nil
+		return fmt.Sprintf("%s/128", ip.String()), shared.IsTrue(netConf["ipv6.nat"]), nil
 	}
 
 	result := make([]api.NetworkAllocations, 0)
 
-	canViewNetwork, err := s.Authorizer.GetPermissionChecker(r.Context(), auth.EntitlementCanView, entity.TypeNetwork)
-	if err != nil {
-		return response.SmartError(err)
-	}
-
-	// If project "foo" is provided but "foo" has `features.networks=false`, then we'll be returning IP allocations
-	// from the default project. In this case, "default" is set as the effective project on the request.Info in the
-	// context of the auth check. This tells the fine-grained auth driver to overwrite "foo" in the URL with "default"
-	// so it can find the actual entity.
-	//
-	// When performing auth checks for instances, the network feature has no relevance, so we need the authorizer to
-	// ignore the effective project. If we didn't do this, URLs would have the project parameter rewritten to "default"
-	// and the authorizer would either not be able to find an entity with that URL, or perform an auth check against an
-	// incorrect entity.
-	canViewInstanceIgnoringEffectiveProject, err := s.Authorizer.GetPermissionCheckerWithoutEffectiveProject(r.Context(), auth.EntitlementCanView, entity.TypeInstance)
+	userHasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), auth.EntitlementCanView, entity.TypeNetwork)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -158,7 +139,7 @@ func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 
 		var networkNames []string
 
-		err := s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err := d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 			var err error
 
 			networkNames, err = tx.GetNetworks(ctx, projectName)
@@ -171,11 +152,11 @@ func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 
 		// Get all the networks, their attached instances, their network forwards and their network load balancers.
 		for _, networkName := range networkNames {
-			if !canViewNetwork(entity.NetworkURL(authCheckProjectName, networkName)) {
+			if !userHasPermission(entity.NetworkURL(authCheckProjectName, networkName)) {
 				continue
 			}
 
-			n, err := network.LoadByName(s, projectName, networkName)
+			n, err := network.LoadByName(d.State(), projectName, networkName)
 			if err != nil {
 				return response.SmartError(fmt.Errorf("Failed loading network %q in project %q: %w", networkName, projectName, err))
 			}
@@ -183,7 +164,7 @@ func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 			netConf := n.Config()
 
 			for _, keyPrefix := range []string{"ipv4", "ipv6"} {
-				ipNet, _ := network.ParseIPCIDRToNet(netConf[keyPrefix+".address"])
+				ipNet, _ := network.ParseIPCIDRToNet(netConf[fmt.Sprintf("%s.address", keyPrefix)])
 				if ipNet == nil {
 					continue
 				}
@@ -192,7 +173,7 @@ func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 					Address: ipNet.String(),
 					UsedBy:  api.NewURL().Path(version.APIVersion, "networks", networkName).Project(projectName).String(),
 					Type:    "network",
-					NAT:     shared.IsTrue(netConf[keyPrefix+".nat"]),
+					NAT:     shared.IsTrue(netConf[fmt.Sprintf("%s.nat", keyPrefix)]),
 					Network: networkName,
 				})
 			}
@@ -202,9 +183,8 @@ func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 				return response.SmartError(fmt.Errorf("Failed getting leases for network %q: %w", networkName, err))
 			}
 
-			leaseTypes := []string{"static", "dynamic", "uplink"}
 			for _, lease := range leases {
-				if slices.Contains(leaseTypes, lease.Type) {
+				if shared.ValueInSlice(lease.Type, []string{"static", "dynamic", "uplink"}) {
 					cidrAddr, nat, err := ipToCIDR(lease.Address, netConf)
 					if err != nil {
 						return response.SmartError(err)
@@ -214,20 +194,10 @@ func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 					if lease.Type == "uplink" {
 						allocationType = "uplink"
 						networkName := strings.TrimSuffix(strings.TrimPrefix(lease.Hostname, lease.Project+"-"), ".uplink")
-						usedByURL := api.NewURL().Path(version.APIVersion, "networks", networkName).Project(lease.Project)
-						if !canViewNetwork(usedByURL) {
-							continue
-						}
-
-						usedBy = usedByURL.String()
+						usedBy = api.NewURL().Path(version.APIVersion, "networks", networkName).Project(lease.Project).String()
 					} else {
 						allocationType = "instance"
-						usedByURL := api.NewURL().Path(version.APIVersion, "instances", lease.Hostname).Project(lease.Project)
-						if !canViewInstanceIgnoringEffectiveProject(usedByURL) {
-							continue
-						}
-
-						usedBy = usedByURL.String()
+						usedBy = api.NewURL().Path(version.APIVersion, "instances", lease.Hostname).Project(lease.Project).String()
 					}
 
 					result = append(result, api.NetworkAllocations{
@@ -243,7 +213,7 @@ func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 
 			var forwards map[int64]*api.NetworkForward
 
-			err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+			err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 				forwards, err = tx.GetNetworkForwards(ctx, n.ID(), false)
 
 				return err
@@ -262,7 +232,6 @@ func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 					result,
 					api.NetworkAllocations{
 						Address: cidrAddr,
-						// No auth check here, the caller can view the network forward because they can view the network.
 						UsedBy:  api.NewURL().Path(version.APIVersion, "networks", networkName, "forwards", forward.ListenAddress).Project(projectName).String(),
 						Type:    "network-forward",
 						NAT:     false, // Network forwards are ingress and so aren't affected by SNAT.
@@ -273,7 +242,7 @@ func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 
 			var loadBalancers map[int64]*api.NetworkLoadBalancer
 
-			err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+			err = d.db.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 				loadBalancers, err = tx.GetNetworkLoadBalancers(ctx, n.ID(), false)
 
 				return err
@@ -292,7 +261,6 @@ func networkAllocationsGet(d *Daemon, r *http.Request) response.Response {
 					result,
 					api.NetworkAllocations{
 						Address: cidrAddr,
-						// No auth check here, the caller can view the load balancer because they can view the network.
 						UsedBy:  api.NewURL().Path(version.APIVersion, "networks", networkName, "load-balancers", loadBalancer.ListenAddress).Project(projectName).String(),
 						Type:    "network-load-balancer",
 						NAT:     false, // Network load-balancers are ingress and so aren't affected by SNAT.
