@@ -137,13 +137,12 @@ var networkPeerCmd = APIEndpoint{
 func networkPeersGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkDetails](r.Context(), ctxNetworkDetails)
+	details, err := request.GetCtxValue[networkDetails](r.Context(), ctxNetworkDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -177,7 +176,7 @@ func networkPeersGet(d *Daemon, r *http.Request) response.Response {
 		peers := make([]*api.NetworkPeer, 0, len(records))
 		for _, record := range records {
 			record.UsedBy, _ = n.PeerUsedBy(record.Name)
-			record.UsedBy = project.FilterUsedBy(r.Context(), s.Authorizer, record.UsedBy)
+			record.UsedBy = project.FilterUsedBy(s.Authorizer, r, record.UsedBy)
 			peers = append(peers, record)
 		}
 
@@ -240,19 +239,17 @@ func networkPeersGet(d *Daemon, r *http.Request) response.Response {
 func networkPeersPost(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkDetails](r.Context(), ctxNetworkDetails)
+	details, err := request.GetCtxValue[networkDetails](r.Context(), ctxNetworkDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -283,7 +280,7 @@ func networkPeersPost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Failed creating peer: %w", err))
 	}
 
-	lc := lifecycle.NetworkPeerCreated.Event(n, req.Name, request.CreateRequestor(r.Context()), nil)
+	lc := lifecycle.NetworkPeerCreated.Event(n, req.Name, request.CreateRequestor(r), nil)
 	s.Events.SendLifecycle(effectiveProjectName, lc)
 
 	return response.SyncResponseLocation(true, nil, lc.Source)
@@ -316,19 +313,17 @@ func networkPeersPost(d *Daemon, r *http.Request) response.Response {
 func networkPeerDelete(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkDetails](r.Context(), ctxNetworkDetails)
+	details, err := request.GetCtxValue[networkDetails](r.Context(), ctxNetworkDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -357,7 +352,7 @@ func networkPeerDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Failed deleting peer: %w", err))
 	}
 
-	s.Events.SendLifecycle(effectiveProjectName, lifecycle.NetworkPeerDeleted.Event(n, peerName, request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(effectiveProjectName, lifecycle.NetworkPeerDeleted.Event(n, peerName, request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
@@ -405,19 +400,17 @@ func networkPeerDelete(d *Daemon, r *http.Request) response.Response {
 func networkPeerGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkDetails](r.Context(), ctxNetworkDetails)
+	details, err := request.GetCtxValue[networkDetails](r.Context(), ctxNetworkDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -453,7 +446,7 @@ func networkPeerGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	peer.UsedBy, _ = n.PeerUsedBy(peer.Name)
-	peer.UsedBy = project.FilterUsedBy(r.Context(), s.Authorizer, peer.UsedBy)
+	peer.UsedBy = project.FilterUsedBy(s.Authorizer, r, peer.UsedBy)
 
 	return response.SyncResponseETag(true, peer, peer.Etag())
 }
@@ -530,19 +523,17 @@ func networkPeerGet(d *Daemon, r *http.Request) response.Response {
 func networkPeerPut(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkDetails](r.Context(), ctxNetworkDetails)
+	details, err := request.GetCtxValue[networkDetails](r.Context(), ctxNetworkDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -578,7 +569,7 @@ func networkPeerPut(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Failed updating peer: %w", err))
 	}
 
-	s.Events.SendLifecycle(effectiveProjectName, lifecycle.NetworkPeerUpdated.Event(n, peerName, request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(effectiveProjectName, lifecycle.NetworkPeerUpdated.Event(n, peerName, request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
