@@ -20,6 +20,7 @@ import (
 	"github.com/canonical/lxd/lxd/response"
 	"github.com/canonical/lxd/lxd/state"
 	"github.com/canonical/lxd/lxd/util"
+	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/entity"
 	"github.com/canonical/lxd/shared/version"
@@ -53,7 +54,7 @@ type networkZoneDetails struct {
 	requestProject api.Project
 }
 
-// addNetworkZoneDetailsToRequestContext sets the effective project in the request.Info and sets ctxNetworkZoneDetails (networkZoneDetails)
+// addNetworkZoneDetailsToRequestContext sets request.CtxEffectiveProjectName (string) and ctxNetworkZoneDetails (networkZoneDetails)
 // in the request context.
 func addNetworkZoneDetailsToRequestContext(s *state.State, r *http.Request) error {
 	zoneName, err := url.PathUnescape(mux.Vars(r)["zone"])
@@ -67,10 +68,8 @@ func addNetworkZoneDetailsToRequestContext(s *state.State, r *http.Request) erro
 		return fmt.Errorf("Failed to check project %q network feature: %w", requestProjectName, err)
 	}
 
-	reqInfo := request.GetContextInfo(r.Context())
-	reqInfo.EffectiveProjectName = effectiveProjectName
-
-	request.SetContextValue(r, ctxNetworkZoneDetails, networkZoneDetails{
+	request.SetCtxValue(r, request.CtxEffectiveProjectName, effectiveProjectName)
+	request.SetCtxValue(r, ctxNetworkZoneDetails, networkZoneDetails{
 		zoneName:       zoneName,
 		requestProject: *requestProject,
 	})
@@ -88,7 +87,7 @@ func networkZoneAccessHandler(entitlement auth.Entitlement) func(d *Daemon, r *h
 			return response.SmartError(err)
 		}
 
-		details, err := request.GetContextValue[networkZoneDetails](r.Context(), ctxNetworkZoneDetails)
+		details, err := request.GetCtxValue[networkZoneDetails](r.Context(), ctxNetworkZoneDetails)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -209,21 +208,29 @@ func networkZoneAccessHandler(entitlement auth.Entitlement) func(d *Daemon, r *h
 func networkZonesGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	requestProjectName, allProjects, err := request.ProjectParams(r)
-	if err != nil {
-		return response.SmartError(err)
+	allProjects := shared.IsTrue(request.QueryParam(r, "all-projects"))
+	requestProjectName := request.QueryParam(r, "project")
+
+	// requestProjectName is only valid for project specific requests.
+	if allProjects && requestProjectName != "" {
+		return response.BadRequest(errors.New("Cannot specify a project when requesting all projects"))
 	}
 
-	reqInfo := request.GetContextInfo(r.Context())
+	var effectiveProjectName string
+	var err error
 	if !allProjects {
+		if requestProjectName == "" {
+			requestProjectName = api.ProjectDefaultName
+		}
+
 		// Project specific requests require an effective project, when "features.networks.zones" is enabled this is the requested project, otherwise it is the default project.
-		effectiveProjectName, _, err := project.NetworkZoneProject(s.DB.Cluster, requestProjectName)
+		effectiveProjectName, _, err = project.NetworkZoneProject(s.DB.Cluster, requestProjectName)
 		if err != nil {
 			return response.SmartError(err)
 		}
 
 		// If the request is project specific, then set effective project name in the request context so that the authorizer can generate the correct URL.
-		reqInfo.EffectiveProjectName = effectiveProjectName
+		request.SetCtxValue(r, request.CtxEffectiveProjectName, effectiveProjectName)
 	}
 
 	recursion := util.IsRecursionRequest(r)
@@ -238,7 +245,7 @@ func networkZonesGet(d *Daemon, r *http.Request) response.Response {
 			zoneNamesMap, err = tx.GetNetworkZones(ctx)
 		} else {
 			// Get list of Network zones.
-			zoneNames, err := tx.GetNetworkZonesByProject(ctx, reqInfo.EffectiveProjectName)
+			zoneNames, err := tx.GetNetworkZonesByProject(ctx, effectiveProjectName)
 			if err != nil {
 				return err
 			}
@@ -275,7 +282,7 @@ func networkZonesGet(d *Daemon, r *http.Request) response.Response {
 		} else {
 			var netzone zone.NetworkZone
 			if !allProjects {
-				netzone, err = zone.LoadByNameAndProject(s, reqInfo.EffectiveProjectName, zoneName)
+				netzone, err = zone.LoadByNameAndProject(s, effectiveProjectName, zoneName)
 			} else {
 				netzone, err = zone.LoadByNameAndProject(s, projectName, zoneName)
 			}
@@ -286,7 +293,7 @@ func networkZonesGet(d *Daemon, r *http.Request) response.Response {
 
 			netzoneInfo := netzone.Info()
 			netzoneInfo.UsedBy, _ = netzone.UsedBy() // Ignore errors in UsedBy, will return nil.
-			netzoneInfo.UsedBy = project.FilterUsedBy(r.Context(), s.Authorizer, netzoneInfo.UsedBy)
+			netzoneInfo.UsedBy = project.FilterUsedBy(s.Authorizer, r, netzoneInfo.UsedBy)
 			netzoneInfo.Project = projectName
 
 			resultMap = append(resultMap, netzoneInfo)
@@ -359,7 +366,7 @@ func networkZonesPost(d *Daemon, r *http.Request) response.Response {
 	// Create the zone.
 	err = zone.Exists(s, req.Name)
 	if err == nil {
-		return response.BadRequest(errors.New("The network zone already exists"))
+		return response.BadRequest(fmt.Errorf("The network zone already exists"))
 	}
 
 	err = zone.Create(s, projectName, &req)
@@ -372,7 +379,7 @@ func networkZonesPost(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	lc := lifecycle.NetworkZoneCreated.Event(netzone, request.CreateRequestor(r.Context()), nil)
+	lc := lifecycle.NetworkZoneCreated.Event(netzone, request.CreateRequestor(r), nil)
 	s.Events.SendLifecycle(projectName, lc)
 
 	return response.SyncResponseLocation(true, nil, lc.Source)
@@ -405,13 +412,12 @@ func networkZonesPost(d *Daemon, r *http.Request) response.Response {
 func networkZoneDelete(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkZoneDetails](r.Context(), ctxNetworkZoneDetails)
+	details, err := request.GetCtxValue[networkZoneDetails](r.Context(), ctxNetworkZoneDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -426,7 +432,7 @@ func networkZoneDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	s.Events.SendLifecycle(effectiveProjectName, lifecycle.NetworkZoneDeleted.Event(netzone, request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(effectiveProjectName, lifecycle.NetworkZoneDeleted.Event(netzone, request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
@@ -474,13 +480,12 @@ func networkZoneDelete(d *Daemon, r *http.Request) response.Response {
 func networkZoneGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkZoneDetails](r.Context(), ctxNetworkZoneDetails)
+	details, err := request.GetCtxValue[networkZoneDetails](r.Context(), ctxNetworkZoneDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -501,7 +506,7 @@ func networkZoneGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	info.UsedBy = project.FilterUsedBy(r.Context(), s.Authorizer, info.UsedBy)
+	info.UsedBy = project.FilterUsedBy(s.Authorizer, r, info.UsedBy)
 
 	if len(withEntitlements) > 0 {
 		err = reportEntitlements(r.Context(), s.Authorizer, s.IdentityCache, entity.TypeNetworkZone, withEntitlements, map[*api.URL]auth.EntitlementReporter{entity.NetworkZoneURL(effectiveProjectName, details.zoneName): info})
@@ -585,13 +590,12 @@ func networkZoneGet(d *Daemon, r *http.Request) response.Response {
 func networkZonePut(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkZoneDetails](r.Context(), ctxNetworkZoneDetails)
+	details, err := request.GetCtxValue[networkZoneDetails](r.Context(), ctxNetworkZoneDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -634,7 +638,7 @@ func networkZonePut(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	s.Events.SendLifecycle(effectiveProjectName, lifecycle.NetworkZoneUpdated.Event(netzone, request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(effectiveProjectName, lifecycle.NetworkZoneUpdated.Event(netzone, request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
