@@ -3,13 +3,10 @@ package main
 import (
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
-	"maps"
 	"net"
 	"net/http"
 	"net/url"
-	"slices"
 	"strconv"
 	"strings"
 	"sync"
@@ -90,7 +87,7 @@ type networkDetails struct {
 	requestProject api.Project
 }
 
-// addNetworkDetailsToRequestContext sets the effective project on the request.Info and sets ctxNetworkDetails (networkDetails)
+// addNetworkDetailsToRequestContext sets request.CtxEffectiveProjectName (string) and ctxNetworkDetails (networkDetails)
 // in the request context.
 func addNetworkDetailsToRequestContext(s *state.State, r *http.Request) error {
 	networkName, err := url.PathUnescape(mux.Vars(r)["networkName"])
@@ -104,10 +101,8 @@ func addNetworkDetailsToRequestContext(s *state.State, r *http.Request) error {
 		return fmt.Errorf("Failed to check project %q network feature: %w", requestProjectName, err)
 	}
 
-	info := request.GetContextInfo(r.Context())
-	info.EffectiveProjectName = effectiveProjectName
-
-	request.SetContextValue(r, ctxNetworkDetails, networkDetails{
+	request.SetCtxValue(r, request.CtxEffectiveProjectName, effectiveProjectName)
+	request.SetCtxValue(r, ctxNetworkDetails, networkDetails{
 		networkName:    networkName,
 		requestProject: *requestProject,
 	})
@@ -125,7 +120,7 @@ func networkAccessHandler(entitlement auth.Entitlement) func(d *Daemon, r *http.
 			return response.SmartError(err)
 		}
 
-		details, err := request.GetContextValue[networkDetails](r.Context(), ctxNetworkDetails)
+		details, err := request.GetCtxValue[networkDetails](r.Context(), ctxNetworkDetails)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -156,11 +151,6 @@ func networkAccessHandler(entitlement auth.Entitlement) func(d *Daemon, r *http.
 //      description: Project name
 //      type: string
 //      example: default
-//    - in: query
-//      name: all-projects
-//      description: Retrieve networks from all projects
-//      type: boolean
-//      example: true
 //  responses:
 //    "200":
 //      description: API endpoints
@@ -215,11 +205,6 @@ func networkAccessHandler(entitlement auth.Entitlement) func(d *Daemon, r *http.
 //	    description: Cluster member name
 //	    type: string
 //	    example: lxd01
-//	  - in: query
-//	    name: all-projects
-//	    description: Retrieve networks from all projects
-//	    type: boolean
-//	    example: true
 //	responses:
 //	  "200":
 //	    description: API endpoints
@@ -252,26 +237,18 @@ func networksGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	// If a target was specified, forward the request to the relevant node.
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	requestProjectName, allProjects, err := request.ProjectParams(r)
+	requestProjectName := request.ProjectParam(r)
+	effectiveProjectName, reqProject, err := project.NetworkProject(s.DB.Cluster, requestProjectName)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	reqInfo := request.GetContextInfo(r.Context())
-	var reqProject *api.Project
-	if !allProjects {
-		// Project specific requests require an effective project, when "features.networks" is enabled this is the requested project, otherwise it is the default project.
-		reqInfo.EffectiveProjectName, reqProject, err = project.NetworkProject(s.DB.Cluster, requestProjectName)
-		if err != nil {
-			return response.SmartError(err)
-		}
-	}
+	request.SetCtxValue(r, request.CtxEffectiveProjectName, effectiveProjectName)
 
 	recursion := util.IsRecursionRequest(r)
 	withEntitlements, err := extractEntitlementsFromQuery(r, entity.TypeNetwork, true)
@@ -281,47 +258,25 @@ func networksGet(d *Daemon, r *http.Request) response.Response {
 
 	// networks holds the network names of the managed and unmanaged networks. They are in two different slices so that
 	// we can perform access control checks differently.
-	var networks [2]map[string][]string
+	var networks [2][]string
 	const (
 		managed = iota
 		unmanaged
 	)
 
-	projectConfigs := make(map[string]map[string]string)
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
-		networks[managed] = map[string][]string{}
-		networks[unmanaged] = map[string][]string{}
-
-		if allProjects {
-			// Get list of managed networks from all projects.
-			networks[managed], err = tx.GetNetworksAllProjects(ctx)
-			if err != nil {
-				return err
-			}
-
-			projectConfigs, err = dbCluster.GetAllProjectsConfig(ctx, tx.Tx())
-			if err != nil {
-				return err
-			}
-		} else {
-			// Get list of managed networks (that may or may not have network interfaces on the host).
-			networkNames, err := tx.GetNetworks(ctx, reqInfo.EffectiveProjectName)
-			if err != nil {
-				return err
-			}
-
-			networks[managed][requestProjectName] = networkNames
-		}
+		// Get list of managed networks (that may or may not have network interfaces on the host).
+		networks[managed], err = tx.GetNetworks(ctx, effectiveProjectName)
 
 		return err
 	})
 	if err != nil {
-		return response.SmartError(err)
+		return response.InternalError(err)
 	}
 
 	// Get list of actual network interfaces on the host if the effective project is default and the caller has permission.
 	var getUnmanagedNetworks bool
-	if reqInfo.EffectiveProjectName == api.ProjectDefaultName || allProjects {
+	if effectiveProjectName == api.ProjectDefaultName {
 		err := s.Authorizer.CheckPermission(r.Context(), entity.ServerURL(), auth.EntitlementCanViewUnmanagedNetworks)
 		if err == nil {
 			getUnmanagedNetworks = true
@@ -342,14 +297,9 @@ func networksGet(d *Daemon, r *http.Request) response.Response {
 				continue
 			}
 
-			unmanagedNetworkProject := requestProjectName
-			if allProjects {
-				unmanagedNetworkProject = api.ProjectDefaultName
-			}
-
 			// Append to the list of networks if a managed network of same name doesn't exist.
-			if !slices.Contains(networks[managed][unmanagedNetworkProject], iface.Name) {
-				networks[unmanaged][unmanagedNetworkProject] = append(networks[unmanaged][unmanagedNetworkProject], iface.Name)
+			if !shared.ValueInSlice(iface.Name, networks[managed]) {
+				networks[unmanaged] = append(networks[unmanaged], iface.Name)
 			}
 		}
 	}
@@ -363,32 +313,23 @@ func networksGet(d *Daemon, r *http.Request) response.Response {
 	resultString := []string{}
 	resultMap := []*api.Network{}
 	urlToNetwork := make(map[*api.URL]auth.EntitlementReporter)
-	for kind, projectNetworks := range networks {
-		for projectName, networkNames := range projectNetworks {
-			for _, networkName := range networkNames {
-				// Filter out managed networks that the caller doesn't have permission to view.
-				if kind == managed && !userHasPermission(entity.NetworkURL(projectName, networkName)) {
+	for kind, networkNames := range networks {
+		for _, networkName := range networkNames {
+			// Filter out managed networks that the caller doesn't have permission to view.
+			if kind == managed && !userHasPermission(entity.NetworkURL(requestProjectName, networkName)) {
+				continue
+			}
+
+			if !recursion {
+				resultString = append(resultString, "/"+version.APIVersion+"/networks/"+networkName)
+			} else {
+				net, err := doNetworkGet(s, r, s.ServerClustered, requestProjectName, reqProject.Config, networkName)
+				if err != nil {
 					continue
 				}
 
-				if !recursion {
-					resultString = append(resultString, api.NewURL().Path(version.APIVersion, "networks", networkName).String())
-				} else {
-					var projectConfig map[string]string
-					if allProjects {
-						projectConfig = projectConfigs[projectName]
-					} else {
-						projectConfig = reqProject.Config
-					}
-
-					net, err := doNetworkGet(s, r, s.ServerClustered, projectName, projectConfig, networkName)
-					if err != nil {
-						continue
-					}
-
-					resultMap = append(resultMap, &net)
-					urlToNetwork[entity.NetworkURL(projectName, networkName)] = &net
-				}
+				resultMap = append(resultMap, &net)
+				urlToNetwork[entity.NetworkURL(requestProjectName, networkName)] = &net
 			}
 		}
 	}
@@ -466,7 +407,7 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 
 	// Quick checks.
 	if req.Name == "" {
-		return response.BadRequest(errors.New("No name provided"))
+		return response.BadRequest(fmt.Errorf("No name provided"))
 	}
 
 	// Check if project allows access to network.
@@ -498,7 +439,7 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 
 	netTypeInfo := netType.Info()
 	if projectName != api.ProjectDefaultName && !netTypeInfo.Projects {
-		return response.BadRequest(errors.New("Network type does not support non-default projects"))
+		return response.BadRequest(fmt.Errorf("Network type does not support non-default projects"))
 	}
 
 	// Check if project has limits.network and if so check we are allowed to create another network.
@@ -522,8 +463,8 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 		// Only check network limits if the new network name doesn't exist already in networks list.
 		// If it does then this create request will either be for adding a target node to an existing
 		// pending network or it will fail anyway as it is a duplicate.
-		if !slices.Contains(networks, req.Name) && len(networks) >= networksLimit {
-			return response.BadRequest(errors.New("Networks limit has been reached for project"))
+		if !shared.ValueInSlice(req.Name, networks) && len(networks) >= networksLimit {
+			return response.BadRequest(fmt.Errorf("Networks limit has been reached for project"))
 		}
 	}
 
@@ -541,7 +482,7 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 
 		// This is an internal request which triggers the actual creation of the network across all nodes
 		// after they have been previously defined.
-		err = doNetworksCreate(r.Context(), s, n, clientType)
+		err = doNetworksCreate(s, n, clientType)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -558,12 +499,12 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 		// A targetNode was specified, let's just define the node's network without actually creating it.
 		// Check that only NodeSpecificNetworkConfig keys are specified.
 		for key := range req.Config {
-			if !slices.Contains(db.NodeSpecificNetworkConfig, key) {
+			if !shared.ValueInSlice(key, db.NodeSpecificNetworkConfig) {
 				return response.BadRequest(fmt.Errorf("Config key %q may not be used as member-specific key", key))
 			}
 		}
 
-		err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			return tx.CreatePendingNetwork(ctx, targetNode, projectName, req.Name, netType.DBType(), req.Config)
 		})
 		if err != nil {
@@ -601,7 +542,7 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 		// Simulate adding pending node network config when the driver doesn't support per-node config.
 		if !netTypeInfo.NodeSpecificConfig && clientType != clusterRequest.ClientTypeJoiner {
 			// Create pending entry for each node.
-			err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+			err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 				members, err := tx.GetNodes(ctx)
 				if err != nil {
 					return fmt.Errorf("Failed getting cluster members: %w", err)
@@ -627,11 +568,11 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 				return response.SmartError(fmt.Errorf("Failed loading network: %w", err))
 			}
 
-			requestor := request.CreateRequestor(r.Context())
+			requestor := request.CreateRequestor(r)
 			s.Events.SendLifecycle(projectName, lifecycle.NetworkCreated.Event(n, requestor, nil))
 		}
 
-		err = networksPostCluster(r.Context(), s, projectName, netInfo, req, clientType, netType)
+		err = networksPostCluster(s, projectName, netInfo, req, clientType, netType)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -641,7 +582,7 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 
 	// Non-clustered network creation.
 	if netInfo != nil {
-		return response.BadRequest(errors.New("The network already exists"))
+		return response.BadRequest(fmt.Errorf("The network already exists"))
 	}
 
 	revert := revert.New()
@@ -666,7 +607,7 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	revert.Add(func() {
-		_ = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		_ = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			return tx.DeleteNetwork(ctx, projectName, req.Name)
 		})
 	})
@@ -676,12 +617,12 @@ func networksPost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Failed loading network: %w", err))
 	}
 
-	err = doNetworksCreate(r.Context(), s, n, clientType)
+	err = doNetworksCreate(s, n, clientType)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	s.Events.SendLifecycle(projectName, lifecycle.NetworkCreated.Event(n, requestor, nil))
 
 	revert.Success()
@@ -700,7 +641,7 @@ func networkPartiallyCreated(netInfo *api.Network) bool {
 	// If the network has global config keys, then it has previously been created by having its global config
 	// inserted, and this means it is partialled created.
 	for key := range netInfo.Config {
-		if !slices.Contains(db.NodeSpecificNetworkConfig, key) {
+		if !shared.ValueInSlice(key, db.NodeSpecificNetworkConfig) {
 			return true
 		}
 	}
@@ -711,10 +652,10 @@ func networkPartiallyCreated(netInfo *api.Network) bool {
 // networksPostCluster checks that there is a pending network in the database and then attempts to setup the
 // network on each node. If all nodes are successfully setup then the network's state is set to created.
 // Accepts an optional existing network record, which will exist when performing subsequent re-create attempts.
-func networksPostCluster(ctx context.Context, s *state.State, projectName string, netInfo *api.Network, req api.NetworksPost, clientType clusterRequest.ClientType, netType network.Type) error {
+func networksPostCluster(s *state.State, projectName string, netInfo *api.Network, req api.NetworksPost, clientType clusterRequest.ClientType, netType network.Type) error {
 	// Check that no node-specific config key has been supplied in request.
 	for key := range req.Config {
-		if slices.Contains(db.NodeSpecificNetworkConfig, key) {
+		if shared.ValueInSlice(key, db.NodeSpecificNetworkConfig) {
 			return fmt.Errorf("Config key %q is cluster member specific", key)
 		}
 	}
@@ -723,7 +664,7 @@ func networksPostCluster(ctx context.Context, s *state.State, projectName string
 	if netInfo != nil {
 		// Check network isn't already created.
 		if netInfo.Status == api.NetworkStatusCreated {
-			return errors.New("The network is already created")
+			return fmt.Errorf("The network is already created")
 		}
 
 		// Check the requested network type matches the type created when adding the local member config.
@@ -734,11 +675,11 @@ func networksPostCluster(ctx context.Context, s *state.State, projectName string
 
 	// Check that the network is properly defined, get the node-specific configs and merge with global config.
 	var nodeConfigs map[string]map[string]string
-	err := s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Check if any global config exists already, if so we should not create global config again.
 		if netInfo != nil && networkPartiallyCreated(netInfo) {
 			if len(req.Config) > 0 {
-				return errors.New("Network already partially created. Please do not specify any global config when re-running create")
+				return fmt.Errorf("Network already partially created. Please do not specify any global config when re-running create")
 			}
 
 			logger.Debug("Skipping global network create as global config already partially created", logger.Ctx{"project": projectName, "network": req.Name})
@@ -774,7 +715,7 @@ func networksPostCluster(ctx context.Context, s *state.State, projectName string
 	})
 	if err != nil {
 		if response.IsNotFoundError(err) {
-			return errors.New("Network not pending on any node (use --target <node> first)")
+			return fmt.Errorf("Network not pending on any node (use --target <node> first)")
 		}
 
 		return err
@@ -794,7 +735,7 @@ func networksPostCluster(ctx context.Context, s *state.State, projectName string
 
 	netConfig := n.Config()
 
-	err = doNetworksCreate(ctx, s, n, clientType)
+	err = doNetworksCreate(s, n, clientType)
 	if err != nil {
 		return err
 	}
@@ -811,10 +752,14 @@ func networksPostCluster(ctx context.Context, s *state.State, projectName string
 		// Clone the network config for this node so we don't modify it and potentially end up sending
 		// this node's config to another node.
 		nodeConfig := make(map[string]string, len(netConfig))
-		maps.Copy(nodeConfig, netConfig)
+		for k, v := range netConfig {
+			nodeConfig[k] = v
+		}
 
 		// Merge node specific config items into global config.
-		maps.Copy(nodeConfig, nodeConfigs[member.Name])
+		for key, value := range nodeConfigs[member.Name] {
+			nodeConfig[key] = value
+		}
 
 		// Create fresh request based on existing network to send to node.
 		nodeReq := api.NetworksPost{
@@ -840,7 +785,7 @@ func networksPostCluster(ctx context.Context, s *state.State, projectName string
 	}
 
 	// Mark network global status as networkCreated now that all nodes have succeeded.
-	err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		return tx.NetworkCreated(projectName, req.Name)
 	})
 	if err != nil {
@@ -854,7 +799,7 @@ func networksPostCluster(ctx context.Context, s *state.State, projectName string
 
 // Create the network on the system. The clusterNotification flag is used to indicate whether creation request
 // is coming from a cluster notification (and if so we should not delete the database record on error).
-func doNetworksCreate(ctx context.Context, s *state.State, n network.Network, clientType clusterRequest.ClientType) error {
+func doNetworksCreate(s *state.State, n network.Network, clientType clusterRequest.ClientType) error {
 	revert := revert.New()
 	defer revert.Fail()
 
@@ -893,7 +838,7 @@ func doNetworksCreate(ctx context.Context, s *state.State, n network.Network, cl
 	}
 
 	// Mark local as status as networkCreated.
-	err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		return tx.NetworkNodeCreated(n.ID())
 	})
 	if err != nil {
@@ -955,8 +900,7 @@ func networkGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	// If a target was specified, forward the request to the relevant node.
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
@@ -966,12 +910,15 @@ func networkGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkDetails](r.Context(), ctxNetworkDetails)
+	details, err := request.GetCtxValue[networkDetails](r.Context(), ctxNetworkDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	allNodes := s.ServerClustered && request.QueryParam(r, "target") == ""
+	allNodes := false
+	if s.ServerClustered && request.QueryParam(r, "target") == "" {
+		allNodes = true
+	}
 
 	n, err := doNetworkGet(s, r, allNodes, details.requestProject.Name, details.requestProject.Config, details.networkName)
 	if err != nil {
@@ -994,11 +941,9 @@ func networkGet(d *Daemon, r *http.Request) response.Response {
 // If the network being requested is a managed network and allNodes is true then node specific config is removed.
 // Otherwise if allNodes is false then the network's local status is returned.
 func doNetworkGet(s *state.State, r *http.Request, allNodes bool, requestProjectName string, reqProjectConfig map[string]string, networkName string) (api.Network, error) {
-	// Effective project may not be set if getting networks for all projects.
-	effectiveProjectName := requestProjectName
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil && reqInfo.EffectiveProjectName != "" {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return api.Network{}, err
 	}
 
 	// Ignore veth pairs (for performance reasons).
@@ -1034,7 +979,6 @@ func doNetworkGet(s *state.State, r *http.Request, allNodes bool, requestProject
 	apiNet.Name = networkName
 	apiNet.UsedBy = []string{}
 	apiNet.Config = map[string]string{}
-	apiNet.Project = requestProjectName
 
 	// Set the device type as needed.
 	if n != nil {
@@ -1088,7 +1032,7 @@ func doNetworkGet(s *state.State, r *http.Request, allNodes bool, requestProject
 			return api.Network{}, err
 		}
 
-		apiNet.UsedBy = project.FilterUsedBy(r.Context(), s.Authorizer, usedBy)
+		apiNet.UsedBy = project.FilterUsedBy(s.Authorizer, r, usedBy)
 	}
 
 	if n != nil {
@@ -1131,13 +1075,12 @@ func doNetworkGet(s *state.State, r *http.Request, allNodes bool, requestProject
 func networkDelete(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkDetails](r.Context(), ctxNetworkDetails)
+	details, err := request.GetCtxValue[networkDetails](r.Context(), ctxNetworkDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1164,7 +1107,7 @@ func networkDelete(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if inUse {
-			return response.BadRequest(errors.New("The network is currently in use"))
+			return response.BadRequest(fmt.Errorf("The network is currently in use"))
 		}
 	}
 
@@ -1206,7 +1149,7 @@ func networkDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	s.Events.SendLifecycle(effectiveProjectName, lifecycle.NetworkDeleted.Event(n, requestor, nil))
 
 	return response.EmptySyncResponse
@@ -1255,16 +1198,15 @@ func networkPost(d *Daemon, r *http.Request) response.Response {
 	//        network is not yet renamed inthe db when the notified node
 	//        runs network.Start).
 	if s.ServerClustered {
-		return response.BadRequest(errors.New("Renaming clustered network not supported"))
+		return response.BadRequest(fmt.Errorf("Renaming clustered network not supported"))
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkDetails](r.Context(), ctxNetworkDetails)
+	details, err := request.GetCtxValue[networkDetails](r.Context(), ctxNetworkDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1289,12 +1231,12 @@ func networkPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if n.Status() != api.NetworkStatusCreated {
-		return response.BadRequest(errors.New("Cannot rename network when not in created state"))
+		return response.BadRequest(fmt.Errorf("Cannot rename network when not in created state"))
 	}
 
 	// Ensure new name is supplied.
 	if req.Name == "" {
-		return response.BadRequest(errors.New("New network name not provided"))
+		return response.BadRequest(fmt.Errorf("New network name not provided"))
 	}
 
 	err = n.ValidateName(req.Name)
@@ -1309,7 +1251,7 @@ func networkPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if inUse {
-		return response.BadRequest(errors.New("Network is currently in use"))
+		return response.BadRequest(fmt.Errorf("Network is currently in use"))
 	}
 
 	var networks []string
@@ -1324,7 +1266,7 @@ func networkPost(d *Daemon, r *http.Request) response.Response {
 		return response.InternalError(err)
 	}
 
-	if slices.Contains(networks, req.Name) {
+	if shared.ValueInSlice(req.Name, networks) {
 		return response.Conflict(fmt.Errorf("Network %q already exists", req.Name))
 	}
 
@@ -1334,7 +1276,7 @@ func networkPost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	lc := lifecycle.NetworkRenamed.Event(n, requestor, map[string]any{"old_name": details.networkName})
 	s.Events.SendLifecycle(effectiveProjectName, lc)
 
@@ -1384,19 +1326,17 @@ func networkPut(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	// If a target was specified, forward the request to the relevant node.
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[networkDetails](r.Context(), ctxNetworkDetails)
+	details, err := request.GetCtxValue[networkDetails](r.Context(), ctxNetworkDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1415,7 +1355,7 @@ func networkPut(d *Daemon, r *http.Request) response.Response {
 	targetNode := request.QueryParam(r, "target")
 
 	if targetNode == "" && n.Status() != api.NetworkStatusCreated {
-		return response.BadRequest(errors.New("Cannot update network global config when not in created state"))
+		return response.BadRequest(fmt.Errorf("Cannot update network global config when not in created state"))
 	}
 
 	// Duplicate config for etag modification and generation.
@@ -1450,7 +1390,7 @@ func networkPut(d *Daemon, r *http.Request) response.Response {
 		if targetNode == "" {
 			// If no target is specified, then ensure only non-node-specific config keys are changed.
 			for k := range req.Config {
-				if slices.Contains(db.NodeSpecificNetworkConfig, k) {
+				if shared.ValueInSlice(k, db.NodeSpecificNetworkConfig) {
 					return response.BadRequest(fmt.Errorf("Config key %q is cluster member specific", k))
 				}
 			}
@@ -1459,7 +1399,7 @@ func networkPut(d *Daemon, r *http.Request) response.Response {
 
 			// If a target is specified, then ensure only node-specific config keys are changed.
 			for k, v := range req.Config {
-				if !slices.Contains(db.NodeSpecificNetworkConfig, k) && curConfig[k] != v {
+				if !shared.ValueInSlice(k, db.NodeSpecificNetworkConfig) && curConfig[k] != v {
 					return response.BadRequest(fmt.Errorf("Config key %q may not be used as member-specific key", k))
 				}
 			}
@@ -1470,7 +1410,7 @@ func networkPut(d *Daemon, r *http.Request) response.Response {
 
 	response := doNetworkUpdate(n, req, targetNode, clientType, r.Method, s.ServerClustered)
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	s.Events.SendLifecycle(effectiveProjectName, lifecycle.NetworkUpdated.Event(n, requestor, nil))
 
 	return response
@@ -1533,7 +1473,7 @@ func doNetworkUpdate(n network.Network, req api.NetworkPut, targetNode string, c
 		// node-specific network config with the submitted config to allow validation.
 		// This allows removal of non-node specific keys when they are absent from request config.
 		for k, v := range n.Config() {
-			if slices.Contains(db.NodeSpecificNetworkConfig, k) {
+			if shared.ValueInSlice(k, db.NodeSpecificNetworkConfig) {
 				req.Config[k] = v
 			}
 		}
@@ -1644,14 +1584,14 @@ func networkLeasesGet(d *Daemon, r *http.Request) response.Response {
 	return response.SyncResponse(true, leases)
 }
 
-func networkStartup(stateFunc func() *state.State, restoreOnly bool) error {
+func networkStartup(stateFunc func() *state.State) error {
 	var err error
 
-	// Build a list of networks to start, keyed by project and network name.
+	// Build a list of networks to initialise, keyed by project and network name.
 	const networkPriorityStandalone = 0 // Start networks not dependent on any other network first.
 	const networkPriorityPhysical = 1   // Start networks dependent on physical interfaces second.
 	const networkPriorityLogical = 2    // Start networks dependent logical networks third.
-	startNetworks := []map[network.ProjectNetwork]struct{}{
+	initNetworks := []map[network.ProjectNetwork]struct{}{
 		networkPriorityStandalone: make(map[network.ProjectNetwork]struct{}),
 		networkPriorityPhysical:   make(map[network.ProjectNetwork]struct{}),
 		networkPriorityLogical:    make(map[network.ProjectNetwork]struct{}),
@@ -1679,35 +1619,14 @@ func networkStartup(stateFunc func() *state.State, restoreOnly bool) error {
 			NetworkName: n.Name(),
 		}
 
-		delete(startNetworks[priority], pn)
+		delete(initNetworks[priority], pn)
 
 		_ = warnings.ResolveWarningsByLocalNodeAndProjectAndTypeAndEntity(s.DB.Cluster, n.Project(), warningtype.NetworkUnvailable, entity.TypeNetwork, int(n.ID()))
 
 		return nil
 	}
 
-	restoreNetwork := func(n network.Network, priority int) error {
-		if n.LocalStatus() != api.NetworkStatusCreated {
-			return fmt.Errorf("Cannot restore network %q when not in created state", n.Name())
-		}
-
-		err = n.Restore()
-		if err != nil {
-			return fmt.Errorf("Failed restoring network: %w", err)
-		}
-
-		// Network restored successfully so remove it from the list.
-		// Otherwise the network startup might enter a retry loop which is not desired when restoring a network.
-		pn := network.ProjectNetwork{
-			ProjectName: n.Project(),
-			NetworkName: n.Name(),
-		}
-
-		delete(startNetworks[priority], pn)
-		return nil
-	}
-
-	loadAndStartupNetwork := func(s *state.State, pn network.ProjectNetwork, priority int, firstPass bool, restoreOnly bool) error {
+	loadAndInitNetwork := func(s *state.State, pn network.ProjectNetwork, priority int, firstPass bool) error {
 		var err error
 		var n network.Network
 
@@ -1720,7 +1639,7 @@ func networkStartup(stateFunc func() *state.State, restoreOnly bool) error {
 				if api.StatusErrorCheck(err, http.StatusNotFound) {
 					// Network has been deleted since we began trying to start it so delete
 					// entry.
-					delete(startNetworks[priority], pn)
+					delete(initNetworks[priority], pn)
 
 					return nil
 				}
@@ -1739,31 +1658,30 @@ func networkStartup(stateFunc func() *state.State, restoreOnly bool) error {
 		if netConfig["parent"] != "" && priority != networkPriorityPhysical {
 			// Start networks that depend on physical interfaces existing after
 			// non-dependent networks.
-			delete(startNetworks[priority], pn)
-			startNetworks[networkPriorityPhysical][pn] = struct{}{}
+			delete(initNetworks[priority], pn)
+			initNetworks[networkPriorityPhysical][pn] = struct{}{}
 
 			return nil
-		} else if (netConfig["network"] != "" || netConfig["bridge.external_interfaces"] != "") && priority != networkPriorityLogical {
-			// Start networks that depend on other logical networks after
+		} else if netConfig["network"] != "" && priority != networkPriorityLogical {
+			// Start networks that depend on other logical networks after networks after
 			// non-dependent networks and networks that depend on physical interfaces.
-			delete(startNetworks[priority], pn)
-			startNetworks[networkPriorityLogical][pn] = struct{}{}
+			delete(initNetworks[priority], pn)
+			initNetworks[networkPriorityLogical][pn] = struct{}{}
 
 			return nil
 		}
 
-		// When restoring a network don't enter the initNetwork function and simply run the network's Restore.
-		// The init takes care of e.g. clearing warnings related to the overall start of the network.
-		if restoreOnly {
-			return restoreNetwork(n, priority)
+		err = initNetwork(s, n, priority)
+		if err != nil {
+			return err
 		}
 
-		return initNetwork(s, n, priority)
+		return nil
 	}
 
 	remainingNetworksCount := func() int {
 		remainingNetworks := 0
-		for _, projectNetworks := range startNetworks {
+		for _, projectNetworks := range initNetworks {
 			remainingNetworks += len(projectNetworks)
 		}
 
@@ -1793,7 +1711,7 @@ func networkStartup(stateFunc func() *state.State, restoreOnly bool) error {
 					}
 
 					// Assume all networks are networkPriorityStandalone initially.
-					startNetworks[networkPriorityStandalone][pn] = struct{}{}
+					initNetworks[networkPriorityStandalone][pn] = struct{}{}
 				}
 			}
 
@@ -1804,16 +1722,10 @@ func networkStartup(stateFunc func() *state.State, restoreOnly bool) error {
 		}
 
 		// Try initializing networks in priority order.
-		for priority := range startNetworks {
-			for pn := range startNetworks[priority] {
-				err := loadAndStartupNetwork(s, pn, priority, true, restoreOnly)
+		for priority := range initNetworks {
+			for pn := range initNetworks[priority] {
+				err := loadAndInitNetwork(s, pn, priority, true)
 				if err != nil {
-					// When restoring a network the operation is not allowed to fail.
-					// The network is already started at this stage which might have taken multiple retries.
-					if restoreOnly {
-						return err
-					}
-
 					logger.Error("Failed initializing network", logger.Ctx{"project": pn.ProjectName, "network": pn.NetworkName, "err": err})
 
 					continue
@@ -1842,9 +1754,9 @@ func networkStartup(stateFunc func() *state.State, restoreOnly bool) error {
 					tryInstancesStart := false
 
 					// Try initializing networks in priority order.
-					for priority := range startNetworks {
-						for pn := range startNetworks[priority] {
-							err := loadAndStartupNetwork(s, pn, priority, false, restoreOnly)
+					for priority := range initNetworks {
+						for pn := range initNetworks[priority] {
+							err := loadAndInitNetwork(s, pn, priority, false)
 							if err != nil {
 								logger.Error("Failed initializing network", logger.Ctx{"project": pn.ProjectName, "network": pn.NetworkName, "err": err})
 
@@ -1872,35 +1784,19 @@ func networkStartup(stateFunc func() *state.State, restoreOnly bool) error {
 					}
 
 					if remainingNetworks <= 0 {
-						// All networks are ready now after performing some retries.
-						// This unblocks any waitready caller using the --network flag.
-						s.NetworkReady.Cancel()
-
 						return // Our job here is done.
 					}
 				}
 			}
 		}()
 	} else {
-		// All networks are ready.
-		// This unblocks any waitready caller using the --network flag.
-		// In case there aren't any networks, this just cancels the canceller.
-		stateFunc().NetworkReady.Cancel()
-
-		logger.Info("All networks started")
+		logger.Info("All networks initialized")
 	}
 
 	return nil
 }
 
-func networkStop(s *state.State, evacuateOnly bool) {
-	if s.DB.Cluster == nil {
-		logger.Warn("Skipping networks stop due to global database not being available")
-		return
-	}
-
-	logger.Info("Stopping networks")
-
+func networkShutdown(s *state.State) {
 	var err error
 
 	// Get a list of projects.
@@ -1937,19 +1833,9 @@ func networkStop(s *state.State, evacuateOnly bool) {
 				continue
 			}
 
-			if evacuateOnly {
-				if n.LocalStatus() != api.NetworkStatusCreated {
-					logger.Error("Failed evacuating network, not in created state", logger.Ctx{"network": name, "project": projectName})
-					continue
-				}
-
-				err = n.Evacuate()
-			} else {
-				err = n.Stop()
-			}
-
+			err = n.Stop()
 			if err != nil {
-				logger.Error("Failed to bring down network", logger.Ctx{"err": err, "project": projectName, "name": name, "evacuate": evacuateOnly})
+				logger.Error("Failed to bring down network", logger.Ctx{"err": err, "project": projectName, "name": name})
 			}
 		}
 	}
@@ -1957,12 +1843,12 @@ func networkStop(s *state.State, evacuateOnly bool) {
 
 // networkRestartOVN is used to trigger a restart of all OVN networks.
 func networkRestartOVN(s *state.State) error {
-	logger.Info("Restarting OVN networks")
+	logger.Infof("Restarting OVN networks")
 
 	// Get a list of projects.
 	var projectNames []string
 	var err error
-	err = s.DB.Cluster.Transaction(s.ShutdownCtx, func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		projectNames, err = dbCluster.GetProjectNames(ctx, tx.Tx())
 		return err
 	})
@@ -1974,7 +1860,7 @@ func networkRestartOVN(s *state.State) error {
 	for _, projectName := range projectNames {
 		var networkNames []string
 
-		err := s.DB.Cluster.Transaction(s.ShutdownCtx, func(ctx context.Context, tx *db.ClusterTx) error {
+		err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 			networkNames, err = tx.GetCreatedNetworkNamesByProject(ctx, projectName)
 
 			return err
@@ -2055,8 +1941,7 @@ func networkStateGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	// If a target was specified, forward the request to the relevant node.
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
