@@ -2,7 +2,6 @@ package main
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
@@ -30,26 +29,30 @@ import (
 )
 
 var operationCmd = APIEndpoint{
-	Path: "operations/{id}",
+	Path:        "operations/{id}",
+	MetricsType: entity.TypeOperation,
 
 	Delete: APIEndpointAction{Handler: operationDelete, AccessHandler: allowAuthenticated},
 	Get:    APIEndpointAction{Handler: operationGet, AccessHandler: allowAuthenticated},
 }
 
 var operationsCmd = APIEndpoint{
-	Path: "operations",
+	Path:        "operations",
+	MetricsType: entity.TypeOperation,
 
-	Get: APIEndpointAction{Handler: operationsGet, AccessHandler: allowAuthenticated},
+	Get: APIEndpointAction{Handler: operationsGet, AccessHandler: allowProjectResourceList},
 }
 
 var operationWait = APIEndpoint{
-	Path: "operations/{id}/wait",
+	Path:        "operations/{id}/wait",
+	MetricsType: entity.TypeOperation,
 
 	Get: APIEndpointAction{Handler: operationWaitGet, AllowUntrusted: true},
 }
 
 var operationWebsocket = APIEndpoint{
-	Path: "operations/{id}/websocket",
+	Path:        "operations/{id}/websocket",
+	MetricsType: entity.TypeOperation,
 
 	Get: APIEndpointAction{Handler: operationWebsocketGet, AllowUntrusted: true},
 }
@@ -66,7 +69,7 @@ func waitForOperations(ctx context.Context, cluster *db.Cluster, consoleShutdown
 				logger.Error("Failed cleaning up operations")
 			}
 
-			return nil //nolint:revive // False positive: raises "return in a defer function has no effect".
+			return nil
 		})
 	}()
 
@@ -94,7 +97,7 @@ func waitForOperations(ctx context.Context, cluster *db.Cluster, consoleShutdown
 
 			_, opAPI, err := op.Render()
 			if err != nil {
-				logger.Warn("Failed to render operation", logger.Ctx{"operation": op, "err": err})
+				logger.Warn("Failed to render operation", logger.Ctx{"operation ID": op.ID(), "operation class": op.Class().String(), "operation status": op.Status().String(), "err": err})
 			} else if opAPI.MayCancel {
 				_, _ = op.Cancel()
 			}
@@ -277,7 +280,7 @@ func operationDelete(d *Daemon, r *http.Request) response.Response {
 		for entityType, urls := range urlsByEntityType {
 			// If only one entry of this type, check directly.
 			if len(urls) == 1 {
-				err := s.Authorizer.CheckPermission(r.Context(), r, &urls[0], entitlement)
+				err := s.Authorizer.CheckPermission(r.Context(), &urls[0], entitlement)
 				if err != nil {
 					return response.SmartError(err)
 				}
@@ -286,7 +289,7 @@ func operationDelete(d *Daemon, r *http.Request) response.Response {
 			}
 
 			// Otherwise get a permission checker for the entity type.
-			hasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), r, entitlement, entityType)
+			hasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), entitlement, entityType)
 			if err != nil {
 				return response.SmartError(err)
 			}
@@ -518,7 +521,7 @@ func operationsGet(d *Daemon, r *http.Request) response.Response {
 		projectName = api.ProjectDefaultName
 	}
 
-	userHasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), r, auth.EntitlementCanViewOperations, entity.TypeProject)
+	userHasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), auth.EntitlementCanViewOperations, entity.TypeProject)
 	if err != nil {
 		return response.InternalError(fmt.Errorf("Failed to get operation permission checker: %w", err))
 	}
@@ -983,17 +986,17 @@ func operationWaitGet(d *Daemon, r *http.Request) response.Response {
 			// Wait for the operation.
 			err = op.Wait(ctx)
 			if err != nil {
-				_ = response.SmartError(err).Render(w)
+				_ = response.SmartError(err).Render(w, r)
 				return nil
 			}
 
 			_, body, err := op.Render()
 			if err != nil {
-				_ = response.SmartError(err).Render(w)
+				_ = response.SmartError(err).Render(w, r)
 				return nil
 			}
 
-			_ = response.SyncResponse(true, body).Render(w)
+			_ = response.SyncResponse(true, body).Render(w, r)
 			return nil
 		}
 
@@ -1034,32 +1037,6 @@ func operationWaitGet(d *Daemon, r *http.Request) response.Response {
 	return response.ForwardedResponse(client, r)
 }
 
-type operationWebSocket struct {
-	req *http.Request
-	op  *operations.Operation
-}
-
-// Render implements response.Response for operationWebSocket.
-func (r *operationWebSocket) Render(w http.ResponseWriter) error {
-	chanErr, err := r.op.Connect(r.req, w)
-	if err != nil {
-		return err
-	}
-
-	err = <-chanErr
-	return err
-}
-
-// String implements fmt.Stringer for operationWebSocket.
-func (r *operationWebSocket) String() string {
-	_, md, err := r.op.Render()
-	if err != nil {
-		return fmt.Sprintf("error: %s", err)
-	}
-
-	return md.ID
-}
-
 // swagger:operation GET /1.0/operations/{id}/websocket?public operations operation_websocket_get_untrusted
 //
 //  Get the websocket stream
@@ -1125,7 +1102,7 @@ func operationWebsocketGet(d *Daemon, r *http.Request) response.Response {
 	// First check if the query is for a local operation from this node
 	op, err := operations.OperationGetInternal(id)
 	if err == nil {
-		return &operationWebSocket{r, op}
+		return operations.OperationWebSocket(op)
 	}
 
 	// Then check if the query is from an operation on another node, and, if so, forward it
@@ -1169,26 +1146,24 @@ func operationWebsocketGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	return operations.ForwardedOperationWebSocket(r, id, source)
+	return operations.ForwardedOperationWebSocket(id, source)
 }
 
 func autoRemoveOrphanedOperationsTask(d *Daemon) (task.Func, task.Schedule) {
 	f := func(ctx context.Context) {
 		s := d.State()
 
-		localClusterAddress := s.LocalConfig.ClusterAddress()
-
-		leader, err := d.gateway.LeaderAddress()
+		leaderInfo, err := s.LeaderInfo()
 		if err != nil {
-			if errors.Is(err, cluster.ErrNodeIsNotClustered) {
-				return // No error if not clustered.
-			}
-
 			logger.Error("Failed to get leader cluster member address", logger.Ctx{"err": err})
 			return
 		}
 
-		if localClusterAddress != leader {
+		if !leaderInfo.Clustered {
+			return
+		}
+
+		if !leaderInfo.Leader {
 			logger.Debug("Skipping remove orphaned operations task since we're not leader")
 			return
 		}
