@@ -2,12 +2,10 @@ package main
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"net/http"
 	"os"
 	"path/filepath"
-	"slices"
 	"sort"
 	"strings"
 	"time"
@@ -93,6 +91,7 @@ var patches = []patch{
 	{name: "storage_set_volume_uuid_v2", stage: patchPostDaemonStorage, run: patchStorageSetVolumeUUIDV2},
 	{name: "storage_move_custom_iso_block_volumes_v2", stage: patchPostDaemonStorage, run: patchStorageRenameCustomISOBlockVolumesV2},
 	{name: "storage_unset_invalid_block_settings_v2", stage: patchPostDaemonStorage, run: patchStorageUnsetInvalidBlockSettingsV2},
+	{name: "config_remove_core_trust_password", stage: patchPreLoadClusterConfig, run: patchRemoveCoreTrustPassword},
 	{name: "entity_type_instance_snapshot_on_delete_trigger_typo_fix", stage: patchPreLoadClusterConfig, run: patchEntityTypeInstanceSnapshotOnDeleteTriggerTypoFix},
 	{name: "instance_remove_volatile_last_state_ip_addresses", stage: patchPostDaemonStorage, run: patchInstanceRemoveVolatileLastStateIPAddresses},
 	{name: "entity_type_identity_certificate_split", stage: patchPreLoadClusterConfig, run: patchSplitIdentityCertificateEntityTypes},
@@ -151,7 +150,7 @@ func patchesApply(d *Daemon, stage patchStage) error {
 			continue
 		}
 
-		if slices.Contains(appliedPatches, patch.name) {
+		if shared.ValueInSlice(patch.name, appliedPatches) {
 			continue
 		}
 
@@ -194,7 +193,7 @@ func selectedPatchClusterMember(s *state.State) (bool, error) {
 	}
 
 	if len(clusterMembers) == 0 {
-		return false, errors.New("Clustered but no member found")
+		return false, fmt.Errorf("Clustered but no member found")
 	}
 
 	// Sort the cluster members by name.
@@ -320,7 +319,7 @@ func patchClusteringServerCertTrust(name string, d *Daemon) error {
 			continue
 		}
 
-		logger.Info("Trusted server certificates found in trust store for all cluster members")
+		logger.Infof("Trusted server certificates found in trust store for all cluster members")
 		break
 	}
 
@@ -739,21 +738,20 @@ func patchNetworkOVNEnableNAT(name string, d *Daemon) error {
 
 // Moves backups from shared.VarPath("backups") to shared.VarPath("backups", "instances").
 func patchMoveBackupsInstances(name string, d *Daemon) error {
-	backupsPathBase := d.State().BackupsStoragePath()
-	if !shared.PathExists(backupsPathBase) {
+	if !shared.PathExists(shared.VarPath("backups")) {
 		return nil // Nothing to do, no backups directory.
 	}
 
-	backupsPath := filepath.Join(backupsPathBase, "instances")
+	backupsPath := shared.VarPath("backups", "instances")
 
 	err := os.MkdirAll(backupsPath, 0700)
 	if err != nil {
 		return fmt.Errorf("Failed creating instances backup directory %q: %w", backupsPath, err)
 	}
 
-	backups, err := os.ReadDir(backupsPathBase)
+	backups, err := os.ReadDir(shared.VarPath("backups"))
 	if err != nil {
-		return fmt.Errorf("Failed listing existing backup directory %q: %w", backupsPathBase, err)
+		return fmt.Errorf("Failed listing existing backup directory %q: %w", shared.VarPath("backups"), err)
 	}
 
 	for _, backupDir := range backups {
@@ -761,7 +759,7 @@ func patchMoveBackupsInstances(name string, d *Daemon) error {
 			continue // Don't try and move our new instances directory or temporary directories.
 		}
 
-		oldPath := filepath.Join(backupsPathBase, backupDir.Name())
+		oldPath := shared.VarPath("backups", backupDir.Name())
 		newPath := filepath.Join(backupsPath, backupDir.Name())
 		logger.Debugf("Moving backup from %q to %q", oldPath, newPath)
 		err = os.Rename(oldPath, newPath)
@@ -925,7 +923,7 @@ func patchZfsSetContentTypeUserProperty(name string, d *Daemon) error {
 
 			zfsVolName := fmt.Sprintf("%s/%s/%s", poolName, storageDrivers.VolumeTypeCustom, project.StorageVolume(vol.Project, vol.Name))
 
-			_, err = shared.RunCommandContext(d.shutdownCtx, "zfs", "set", "lxd:content_type="+vol.ContentType, zfsVolName)
+			_, err = shared.RunCommandContext(d.shutdownCtx, "zfs", "set", fmt.Sprintf("lxd:content_type=%s", vol.ContentType), zfsVolName)
 			if err != nil {
 				logger.Debug("Failed setting lxd:content_type property", logger.Ctx{"name": zfsVolName, "err": err})
 			}
@@ -1010,7 +1008,7 @@ func patchStorageZfsUnsetInvalidBlockSettingsV2(_ string, d *Daemon) error {
 		return err
 	}
 
-	var volType dbCluster.StoragePoolVolumeType
+	var volType int
 
 	for pool, volumes := range poolVolumes {
 		for _, vol := range volumes {
@@ -1040,12 +1038,12 @@ func patchStorageZfsUnsetInvalidBlockSettingsV2(_ string, d *Daemon) error {
 				continue
 			}
 
-			switch vol.Type {
-			case dbCluster.StoragePoolVolumeTypeNameVM:
+			if vol.Type == dbCluster.StoragePoolVolumeTypeNameVM {
 				volType = volTypeVM
-			case dbCluster.StoragePoolVolumeTypeNameCustom:
+			} else if vol.Type == dbCluster.StoragePoolVolumeTypeNameCustom {
 				volType = volTypeCustom
-			default:
+			} else {
+				// Should not happen.
 				continue
 			}
 
@@ -1297,7 +1295,7 @@ func patchStorageRenameCustomISOBlockVolumesV2(name string, d *Daemon) error {
 			// We need to use ContentTypeBlock here in order for the driver to figure out the correct (old) location.
 			oldVol := storageDrivers.NewVolume(p.Driver(), p.Name(), storageDrivers.VolumeTypeCustom, storageDrivers.ContentTypeBlock, project.StorageVolume(vol.Project, vol.Name), nil, nil)
 
-			err = p.Driver().RenameVolume(oldVol, oldVol.Name()+".iso", nil)
+			err = p.Driver().RenameVolume(oldVol, fmt.Sprintf("%s.iso", oldVol.Name()), nil)
 			if err != nil {
 				return fmt.Errorf("Failed to rename volume %q in pool %q: %w", oldVol.Name(), p.Name(), err)
 			}
@@ -1326,6 +1324,21 @@ DELETE FROM storage_volumes_config
 	return err
 }
 
+// patchRemoveCoreTrustPassword removes the core.trust_password config key from the cluster.
+func patchRemoveCoreTrustPassword(_ string, d *Daemon) error {
+	s := d.State()
+	err := s.DB.Cluster.Transaction(s.ShutdownCtx, func(ctx context.Context, tx *db.ClusterTx) error {
+		return tx.UpdateClusterConfig(map[string]string{
+			"core.trust_password": "",
+		})
+	})
+	if err != nil {
+		return fmt.Errorf("Failed to remove core.trust_password config key: %w", err)
+	}
+
+	return nil
+}
+
 // patchEntityTypeInstanceSnapshotOnDeleteTriggerTypoFix drops the misspelled on_instance_snaphot_delete trigger on all cluster members, if it exists.
 func patchEntityTypeInstanceSnapshotOnDeleteTriggerTypoFix(_ string, d *Daemon) error {
 	var err error
@@ -1447,18 +1460,18 @@ func patchOIDCGroupsClaimScope(_ string, d *Daemon) error {
 		}
 
 		// Get the groups claim and scopes (these will just be the default values at the time of the patch)
-		_, _, _, scopes, _, groupsClaim := globalConfig.OIDCServer()
+		_, _, scopes, _, groupsClaim := globalConfig.OIDCServer()
 
 		// If the groups claim is not set, or this patch was already run on another member and the groups claim is
 		// already present in the list of scopes, then there is nothing to do.
-		if groupsClaim == "" || slices.Contains(scopes, groupsClaim) {
+		if groupsClaim == "" || shared.ValueInSlice(groupsClaim, scopes) {
 			return nil
 		}
 
 		// Add the groups claim as an additional scope.
 		// The groups claim still needs to be set to extract group values from the token claims or userinfo.
 		oidcScopes := append(scopes, groupsClaim)
-		_, err = globalConfig.Patch(tx, map[string]any{
+		_, err = globalConfig.Patch(map[string]any{
 			"oidc.scopes": strings.Join(oidcScopes, " "),
 		})
 
