@@ -9,7 +9,6 @@ import (
 	"io"
 	"net/http"
 	"net/url"
-	"slices"
 	"strings"
 
 	"github.com/gorilla/mux"
@@ -64,7 +63,7 @@ type profileDetails struct {
 	effectiveProject api.Project
 }
 
-// addProfileDetailsToRequestContext sets the effective project in the request.Info and sets ctxProfileDetails (profileDetails)
+// addProfileDetailsToRequestContext sets request.CtxEffectiveProjectName (string) and ctxProfileDetails (profileDetails)
 // in the request context.
 func addProfileDetailsToRequestContext(s *state.State, r *http.Request) error {
 	profileName, err := url.PathUnescape(mux.Vars(r)["name"])
@@ -78,10 +77,8 @@ func addProfileDetailsToRequestContext(s *state.State, r *http.Request) error {
 		return fmt.Errorf("Failed to check project %q profile feature: %w", requestProjectName, err)
 	}
 
-	reqInfo := request.GetContextInfo(r.Context())
-	reqInfo.EffectiveProjectName = effectiveProject.Name
-
-	request.SetContextValue(r, ctxProfileDetails, profileDetails{
+	request.SetCtxValue(r, request.CtxEffectiveProjectName, effectiveProject.Name)
+	request.SetCtxValue(r, ctxProfileDetails, profileDetails{
 		profileName:      profileName,
 		effectiveProject: *effectiveProject,
 	})
@@ -99,7 +96,7 @@ func profileAccessHandler(entitlement auth.Entitlement) func(d *Daemon, r *http.
 			return response.SmartError(err)
 		}
 
-		details, err := request.GetContextValue[profileDetails](r.Context(), ctxProfileDetails)
+		details, err := request.GetCtxValue[profileDetails](r.Context(), ctxProfileDetails)
 		if err != nil {
 			return response.SmartError(err)
 		}
@@ -218,19 +215,17 @@ func profileAccessHandler(entitlement auth.Entitlement) func(d *Daemon, r *http.
 func profilesGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	requestProjectName, allProjects, err := request.ProjectParams(r)
-	if err != nil {
-		return response.SmartError(err)
+	requestProjectName := request.ProjectParam(r)
+	allProjects := shared.IsTrue(request.QueryParam(r, "all-projects"))
+
+	// requestProjectName is only valid for project specific requests.
+	if allProjects && requestProjectName != api.ProjectDefaultName {
+		return response.BadRequest(errors.New("Cannot specify a project when requesting all projects"))
 	}
 
-	reqInfo := request.GetContextInfo(r.Context())
-	if !allProjects {
-		p, err := project.ProfileProject(s.DB.Cluster, requestProjectName)
-		if err != nil {
-			return response.SmartError(err)
-		}
-
-		reqInfo.EffectiveProjectName = p.Name
+	p, err := project.ProfileProject(s.DB.Cluster, requestProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
 	recursion := util.IsRecursionRequest(r)
@@ -239,6 +234,7 @@ func profilesGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
+	request.SetCtxValue(r, request.CtxEffectiveProjectName, p.Name)
 	userHasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), auth.EntitlementCanView, entity.TypeProfile)
 	if err != nil {
 		return response.InternalError(err)
@@ -251,7 +247,7 @@ func profilesGet(d *Daemon, r *http.Request) response.Response {
 		var profiles []dbCluster.Profile
 		if !allProjects {
 			filter := dbCluster.ProfileFilter{
-				Project: &reqInfo.EffectiveProjectName,
+				Project: &p.Name,
 			}
 
 			profiles, err = dbCluster.GetProfiles(ctx, tx.Tx(), filter)
@@ -316,7 +312,7 @@ func profilesGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	for _, apiProfile := range apiProfiles {
-		apiProfile.UsedBy = project.FilterUsedBy(r.Context(), s.Authorizer, apiProfile.UsedBy)
+		apiProfile.UsedBy = project.FilterUsedBy(s.Authorizer, r, apiProfile.UsedBy)
 	}
 
 	if len(withEntitlements) > 0 {
@@ -393,14 +389,14 @@ func profilesPost(d *Daemon, r *http.Request) response.Response {
 
 	// Quick checks.
 	if req.Name == "" {
-		return response.BadRequest(errors.New("No name provided"))
+		return response.BadRequest(fmt.Errorf("No name provided"))
 	}
 
 	if strings.Contains(req.Name, "/") {
-		return response.BadRequest(errors.New("Profile names may not contain slashes"))
+		return response.BadRequest(fmt.Errorf("Profile names may not contain slashes"))
 	}
 
-	if slices.Contains([]string{".", ".."}, req.Name) {
+	if shared.ValueInSlice(req.Name, []string{".", ".."}) {
 		return response.BadRequest(fmt.Errorf("Invalid profile name %q", req.Name))
 	}
 
@@ -416,7 +412,7 @@ func profilesPost(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// Update DB entry.
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		devices, err := dbCluster.APIToDevices(req.Devices)
 		if err != nil {
 			return err
@@ -424,7 +420,7 @@ func profilesPost(d *Daemon, r *http.Request) response.Response {
 
 		current, _ := dbCluster.GetProfile(ctx, tx.Tx(), p.Name, req.Name)
 		if current != nil {
-			return errors.New("The profile already exists")
+			return fmt.Errorf("The profile already exists")
 		}
 
 		profile := dbCluster.Profile{
@@ -454,7 +450,7 @@ func profilesPost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Error inserting %q into database: %w", req.Name, err))
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	lc := lifecycle.ProfileCreated.Event(req.Name, p.Name, requestor, nil)
 	s.Events.SendLifecycle(p.Name, lc)
 
@@ -504,7 +500,7 @@ func profilesPost(d *Daemon, r *http.Request) response.Response {
 func profileGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	details, err := request.GetContextValue[profileDetails](r.Context(), ctxProfileDetails)
+	details, err := request.GetCtxValue[profileDetails](r.Context(), ctxProfileDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -516,7 +512,7 @@ func profileGet(d *Daemon, r *http.Request) response.Response {
 
 	var resp *api.Profile
 
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		profile, err := dbCluster.GetProfile(ctx, tx.Tx(), details.effectiveProject.Name, details.profileName)
 		if err != nil {
 			return fmt.Errorf("Fetch profile: %w", err)
@@ -548,7 +544,7 @@ func profileGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	resp.UsedBy = project.FilterUsedBy(r.Context(), s.Authorizer, resp.UsedBy)
+	resp.UsedBy = project.FilterUsedBy(s.Authorizer, r, resp.UsedBy)
 
 	if len(withEntitlements) > 0 {
 		err = reportEntitlements(r.Context(), s.Authorizer, s.IdentityCache, entity.TypeProfile, withEntitlements, map[*api.URL]auth.EntitlementReporter{entity.ProfileURL(details.effectiveProject.Name, details.profileName): resp})
@@ -598,7 +594,7 @@ func profileGet(d *Daemon, r *http.Request) response.Response {
 func profilePut(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	details, err := request.GetContextValue[profileDetails](r.Context(), ctxProfileDetails)
+	details, err := request.GetCtxValue[profileDetails](r.Context(), ctxProfileDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -612,14 +608,14 @@ func profilePut(d *Daemon, r *http.Request) response.Response {
 			return response.BadRequest(err)
 		}
 
-		err = doProfileUpdateCluster(r.Context(), s, details.effectiveProject.Name, details.profileName, old)
+		err = doProfileUpdateCluster(s, details.effectiveProject.Name, details.profileName, old)
 		return response.SmartError(err)
 	}
 
 	var id int64
 	var profile *api.Profile
 
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		current, err := dbCluster.GetProfile(ctx, tx.Tx(), details.effectiveProject.Name, details.profileName)
 		if err != nil {
 			return fmt.Errorf("Failed to retrieve profile %q: %w", details.profileName, err)
@@ -651,7 +647,7 @@ func profilePut(d *Daemon, r *http.Request) response.Response {
 		return response.BadRequest(err)
 	}
 
-	err = doProfileUpdate(r.Context(), s, details.effectiveProject, details.profileName, id, profile, req)
+	err = doProfileUpdate(s, details.effectiveProject, details.profileName, id, profile, req)
 
 	if err == nil && !isClusterNotification(r) {
 		// Notify all other nodes. If a node is down, it will be ignored.
@@ -660,7 +656,7 @@ func profilePut(d *Daemon, r *http.Request) response.Response {
 			return response.SmartError(err)
 		}
 
-		err = notifier(func(_ db.NodeInfo, client lxd.InstanceServer) error {
+		err = notifier(func(member db.NodeInfo, client lxd.InstanceServer) error {
 			return client.UseProject(details.effectiveProject.Name).UpdateProfile(details.profileName, profile.Writable(), "")
 		})
 		if err != nil {
@@ -668,7 +664,7 @@ func profilePut(d *Daemon, r *http.Request) response.Response {
 		}
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	s.Events.SendLifecycle(details.effectiveProject.Name, lifecycle.ProfileUpdated.Event(details.profileName, details.effectiveProject.Name, requestor, nil))
 
 	return response.SmartError(err)
@@ -711,7 +707,7 @@ func profilePut(d *Daemon, r *http.Request) response.Response {
 func profilePatch(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	details, err := request.GetContextValue[profileDetails](r.Context(), ctxProfileDetails)
+	details, err := request.GetCtxValue[profileDetails](r.Context(), ctxProfileDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -719,7 +715,7 @@ func profilePatch(d *Daemon, r *http.Request) response.Response {
 	var id int64
 	var profile *api.Profile
 
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		current, err := dbCluster.GetProfile(ctx, tx.Tx(), details.effectiveProject.Name, details.profileName)
 		if err != nil {
 			return fmt.Errorf("Failed to retrieve profile=%q: %w", details.profileName, err)
@@ -795,10 +791,10 @@ func profilePatch(d *Daemon, r *http.Request) response.Response {
 		}
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	s.Events.SendLifecycle(details.effectiveProject.Name, lifecycle.ProfileUpdated.Event(details.profileName, details.effectiveProject.Name, requestor, nil))
 
-	return response.SmartError(doProfileUpdate(r.Context(), s, details.effectiveProject, details.profileName, id, profile, req))
+	return response.SmartError(doProfileUpdate(s, details.effectiveProject, details.profileName, id, profile, req))
 }
 
 // swagger:operation POST /1.0/profiles/{name} profiles profile_post
@@ -836,7 +832,7 @@ func profilePatch(d *Daemon, r *http.Request) response.Response {
 func profilePost(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	details, err := request.GetContextValue[profileDetails](r.Context(), ctxProfileDetails)
+	details, err := request.GetCtxValue[profileDetails](r.Context(), ctxProfileDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -853,18 +849,18 @@ func profilePost(d *Daemon, r *http.Request) response.Response {
 
 	// Quick checks.
 	if req.Name == "" {
-		return response.BadRequest(errors.New("No name provided"))
+		return response.BadRequest(fmt.Errorf("No name provided"))
 	}
 
 	if strings.Contains(req.Name, "/") {
-		return response.BadRequest(errors.New("Profile names may not contain slashes"))
+		return response.BadRequest(fmt.Errorf("Profile names may not contain slashes"))
 	}
 
-	if slices.Contains([]string{".", ".."}, req.Name) {
+	if shared.ValueInSlice(req.Name, []string{".", ".."}) {
 		return response.BadRequest(fmt.Errorf("Invalid profile name %q", req.Name))
 	}
 
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		// Check that the name isn't already in use.
 		_, err = dbCluster.GetProfile(ctx, tx.Tx(), details.effectiveProject.Name, req.Name)
 		if err == nil {
@@ -877,7 +873,7 @@ func profilePost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	lc := lifecycle.ProfileRenamed.Event(req.Name, details.effectiveProject.Name, requestor, logger.Ctx{"old_name": details.profileName})
 	s.Events.SendLifecycle(details.effectiveProject.Name, lc)
 
@@ -911,7 +907,7 @@ func profilePost(d *Daemon, r *http.Request) response.Response {
 func profileDelete(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	details, err := request.GetContextValue[profileDetails](r.Context(), ctxProfileDetails)
+	details, err := request.GetCtxValue[profileDetails](r.Context(), ctxProfileDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -920,7 +916,7 @@ func profileDelete(d *Daemon, r *http.Request) response.Response {
 		return response.Forbidden(errors.New(`The "default" profile cannot be deleted`))
 	}
 
-	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		profile, err := dbCluster.GetProfile(ctx, tx.Tx(), details.effectiveProject.Name, details.profileName)
 		if err != nil {
 			return err
@@ -932,7 +928,7 @@ func profileDelete(d *Daemon, r *http.Request) response.Response {
 		}
 
 		if len(usedBy) > 0 {
-			return errors.New("Profile is currently in use")
+			return fmt.Errorf("Profile is currently in use")
 		}
 
 		return dbCluster.DeleteProfile(ctx, tx.Tx(), details.effectiveProject.Name, details.profileName)
@@ -941,7 +937,7 @@ func profileDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
-	requestor := request.CreateRequestor(r.Context())
+	requestor := request.CreateRequestor(r)
 	s.Events.SendLifecycle(details.effectiveProject.Name, lifecycle.ProfileDeleted.Event(details.profileName, details.effectiveProject.Name, requestor, nil))
 
 	return response.EmptySyncResponse
