@@ -3,9 +3,7 @@ package project
 import (
 	"context"
 	"database/sql"
-	"errors"
 	"fmt"
-	"slices"
 	"strings"
 
 	"github.com/canonical/lxd/lxd/db"
@@ -20,11 +18,11 @@ const separator = "_"
 // ValidName validates a project name.
 func ValidName(name string) error {
 	if name == "" {
-		return errors.New("No name provided")
+		return fmt.Errorf("No name provided")
 	}
 
 	if name == "*" {
-		return errors.New("Reserved project name")
+		return fmt.Errorf("Reserved project name")
 	}
 
 	if name == "." || name == ".." {
@@ -32,23 +30,23 @@ func ValidName(name string) error {
 	}
 
 	if strings.Contains(name, "\\") {
-		return errors.New("Project names may not contain back slashes")
+		return fmt.Errorf("Project names may not contain back slashes")
 	}
 
 	if strings.Contains(name, "/") {
-		return errors.New("Project names may not contain slashes")
+		return fmt.Errorf("Project names may not contain slashes")
 	}
 
 	if strings.Contains(name, " ") {
-		return errors.New("Project names may not contain spaces")
+		return fmt.Errorf("Project names may not contain spaces")
 	}
 
 	if strings.Contains(name, "_") {
-		return errors.New("Project names may not contain underscores")
+		return fmt.Errorf("Project names may not contain underscores")
 	}
 
 	if strings.Contains(name, "'") || strings.Contains(name, `"`) {
-		return errors.New("Project names may not contain quotes")
+		return fmt.Errorf("Project names may not contain quotes")
 	}
 
 	return nil
@@ -112,7 +110,7 @@ func StorageVolumeParts(projectStorageVolumeName string) (projectName string, st
 // For custom volume type, if the project specified has the "features.storage.volumes" flag enabled then the
 // project name is returned, otherwise the default project name is returned.
 // For all other volume types the supplied project name is returned.
-func StorageVolumeProject(c *db.Cluster, projectName string, volumeType cluster.StoragePoolVolumeType) (string, error) {
+func StorageVolumeProject(c *db.Cluster, projectName string, volumeType int) (string, error) {
 	// Image volumes are effectively a cache and so are always linked to default project.
 	// Optimisation to avoid loading project record.
 	if volumeType == cluster.StoragePoolVolumeTypeImage {
@@ -142,7 +140,7 @@ func StorageVolumeProject(c *db.Cluster, projectName string, volumeType cluster.
 // For custom volume type, if the project supplied has the "features.storage.volumes" flag enabled then the
 // project name is returned, otherwise the default project name is returned.
 // For all other volume types the supplied project's name is returned.
-func StorageVolumeProjectFromRecord(p *api.Project, volumeType cluster.StoragePoolVolumeType) string {
+func StorageVolumeProjectFromRecord(p *api.Project, volumeType int) string {
 	// Image volumes are effectively a cache and so are always linked to default project.
 	if volumeType == cluster.StoragePoolVolumeTypeImage {
 		return api.ProjectDefaultName
@@ -248,7 +246,7 @@ func NetworkAllowed(reqProjectConfig map[string]string, networkName string, isMa
 	}
 
 	// Don't allow access to unmanaged networks if only managed network access is allowed.
-	if !isManaged && slices.Contains([]string{"managed", ""}, reqProjectConfig["restricted.devices.nic"]) {
+	if shared.ValueInSlice(reqProjectConfig["restricted.devices.nic"], []string{"managed", ""}) && !isManaged {
 		return false
 	}
 
@@ -259,7 +257,7 @@ func NetworkAllowed(reqProjectConfig map[string]string, networkName string, isMa
 
 	// Check if requested network is in list of allowed networks.
 	allowedRestrictedNetworks := shared.SplitNTrimSpace(reqProjectConfig["restricted.networks.access"], ",", -1, false)
-	return slices.Contains(allowedRestrictedNetworks, networkName)
+	return shared.ValueInSlice(networkName, allowedRestrictedNetworks)
 }
 
 // ProfileProject returns the effective project to use for the profile based on the requested project.
