@@ -5,6 +5,7 @@ import (
 	"bytes"
 	"fmt"
 	"os"
+	"path"
 	"path/filepath"
 	"strconv"
 	"strings"
@@ -12,11 +13,12 @@ import (
 
 	"golang.org/x/sys/unix"
 
-	"github.com/canonical/lxd/lxd/storage/block"
+	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 )
 
 var devDiskByPath = "/dev/disk/by-path"
+var devDiskByID = "/dev/disk/by-id"
 var runUdevData = "/run/udev/data"
 var sysClassBlock = "/sys/class/block"
 var procSelfMountInfo = "/proc/self/mountinfo"
@@ -41,7 +43,7 @@ func storageAddDriveInfo(devicePath string, disk *api.ResourcesStorageDisk) erro
 	}
 
 	// Retrieve udev information
-	udevInfo := filepath.Join(runUdevData, "b"+disk.Device)
+	udevInfo := filepath.Join(runUdevData, fmt.Sprintf("b%s", disk.Device))
 	if sysfsExists(udevInfo) {
 		// Get the udev information
 		f, err := os.Open(udevInfo)
@@ -358,12 +360,6 @@ func GetStorage() (*api.ResourcesStorage, error) {
 
 				partition.Size = partitionSize * 512
 
-				// Pull device filesystem UUID information.
-				partition.DeviceFSUUID, err = block.DiskFSUUID(filepath.Join("/dev", subEntryName))
-				if err != nil {
-					return nil, err
-				}
-
 				// Add to list
 				disk.Partitions = append(disk.Partitions, partition)
 			}
@@ -391,15 +387,15 @@ func GetStorage() (*api.ResourcesStorage, error) {
 			}
 
 			// Try to find the udev device id
-			if sysfsExists(block.DevDiskByID) {
-				links, err := os.ReadDir(block.DevDiskByID)
+			if sysfsExists(devDiskByID) {
+				links, err := os.ReadDir(devDiskByID)
 				if err != nil {
-					return nil, fmt.Errorf("Failed to list the links in %q: %w", block.DevDiskByID, err)
+					return nil, fmt.Errorf("Failed to list the links in %q: %w", devDiskByID, err)
 				}
 
 				for _, link := range links {
 					linkName := link.Name()
-					linkPath := filepath.Join(block.DevDiskByID, linkName)
+					linkPath := filepath.Join(devDiskByID, linkName)
 
 					linkTarget, err := filepath.EvalSymlinks(linkPath)
 					if err != nil {
@@ -427,12 +423,6 @@ func GetStorage() (*api.ResourcesStorage, error) {
 				}
 			}
 
-			// Pull device filesystem UUID information.
-			disk.DeviceFSUUID, err = block.DiskFSUUID(filepath.Join("/dev", entryName))
-			if err != nil {
-				return nil, err
-			}
-
 			// Add to list
 			storage.Disks = append(storage.Disks, disk)
 		}
@@ -449,3 +439,23 @@ func GetStorage() (*api.ResourcesStorage, error) {
 
 	return &storage, nil
 }
+
+// GetDisksByID returns all disks whose ID contains the filter prefix.
+func GetDisksByID(filterPrefix string) ([]string, error) {
+	disks, err := os.ReadDir(devDiskByID)
+	if err != nil {
+		return nil, fmt.Errorf("Failed getting disks by ID: %w", err)
+	}
+
+	var filteredDisks []string
+	for _, disk := range disks {
+		// Skip the disk if it does not have the prefix.
+		if !shared.StringHasPrefix(disk.Name(), filterPrefix) {
+			continue
+		}
+
+		filteredDisks = append(filteredDisks, path.Join(devDiskByID, disk.Name()))
+	}
+
+	return filteredDisks, nil
+}
