@@ -6,12 +6,15 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
+	"io/fs"
 	"mime/multipart"
 	"net/http"
 	"os"
+	"strconv"
 	"time"
 
 	"github.com/canonical/lxd/client"
+	"github.com/canonical/lxd/lxd/metrics"
 	"github.com/canonical/lxd/lxd/ucred"
 	"github.com/canonical/lxd/lxd/util"
 	"github.com/canonical/lxd/shared/api"
@@ -21,14 +24,24 @@ import (
 
 var debug bool
 
-// Init sets the debug variable to the provided value.
-func Init(d bool) {
+// Init sets the debug variable to the provided value and registers any additional smart error mappings.
+func Init(d bool, smartErrors map[int][]error) {
 	debug = d
+
+	for code, additionalErrors := range smartErrors {
+		existingErrs, ok := httpResponseErrors[code]
+		if ok {
+			httpResponseErrors[code] = append(existingErrs, additionalErrors...)
+			continue
+		}
+
+		httpResponseErrors[code] = additionalErrors
+	}
 }
 
 // Response represents an API response.
 type Response interface {
-	Render(w http.ResponseWriter) error
+	Render(w http.ResponseWriter, r *http.Request) error
 	String() string
 }
 
@@ -39,11 +52,10 @@ type devLxdResponse struct {
 	contentType string
 }
 
-func (r *devLxdResponse) Render(w http.ResponseWriter) error {
-	var err error
-
+// Render renders a response for requests against the /dev/lxd socket.
+func (r *devLxdResponse) Render(w http.ResponseWriter, req *http.Request) (err error) {
 	if r.code != http.StatusOK {
-		http.Error(w, fmt.Sprintf("%s", r.content), r.code)
+		http.Error(w, fmt.Sprint(r.content), r.code)
 	} else if r.contentType == "json" {
 		w.Header().Set("Content-Type", "application/json")
 
@@ -59,11 +71,7 @@ func (r *devLxdResponse) Render(w http.ResponseWriter) error {
 		_, err = fmt.Fprint(w, r.content.(string))
 	}
 
-	if err != nil {
-		return err
-	}
-
-	return nil
+	return err
 }
 
 func (r *devLxdResponse) String() string {
@@ -144,12 +152,13 @@ func SyncResponsePlain(success bool, compress bool, metadata string) Response {
 	return &syncResponse{success: success, metadata: metadata, plaintext: true, compress: compress}
 }
 
-func (r *syncResponse) Render(w http.ResponseWriter) error {
+// Render renders a synchronous response.
+func (r *syncResponse) Render(w http.ResponseWriter, req *http.Request) error {
 	// Set an appropriate ETag header
 	if r.etag != nil {
 		etag, err := util.EtagHash(r.etag)
 		if err == nil {
-			w.Header().Set("ETag", fmt.Sprintf("\"%s\"", etag))
+			w.Header().Set("ETag", `"`+etag+`"`)
 		}
 	}
 
@@ -196,10 +205,19 @@ func (r *syncResponse) Render(w http.ResponseWriter) error {
 		// to propagate the data and preserve the status code.
 		err, ok := r.metadata.(error)
 		if ok {
-			return SmartError(err).Render(w)
+			return SmartError(err).Render(w, req)
 		}
 	}
 
+	// defer calling the callback function after possibly considering the response a SmartError.
+	defer func() {
+		if r.success {
+			metrics.UseMetricsCallback(req, metrics.Success)
+		} else {
+			metrics.UseMetricsCallback(req, metrics.ErrorServer)
+		}
+	}()
+
 	// Handle plain text responses.
 	if r.plaintext {
 		if r.metadata != nil {
@@ -327,7 +345,8 @@ func (r *errorResponse) String() string {
 	return r.msg
 }
 
-func (r *errorResponse) Render(w http.ResponseWriter) error {
+// Render renders a response that indicates an error on the request handling.
+func (r *errorResponse) Render(w http.ResponseWriter, req *http.Request) error {
 	var output io.Writer
 
 	buf := &bytes.Buffer{}
@@ -344,6 +363,17 @@ func (r *errorResponse) Render(w http.ResponseWriter) error {
 		Code:  r.code, // Set the error code in the Code field of the response body.
 	}
 
+	defer func() {
+		// Use the callback function to count the request for the API metrics.
+		if r.code >= 400 && r.code < 500 {
+			// 4* codes are considered client errors on HTTP.
+			metrics.UseMetricsCallback(req, metrics.ErrorClient)
+		} else {
+			// Any other status code here shoud be higher than or equal to 500 and is considered a server error.
+			metrics.UseMetricsCallback(req, metrics.ErrorServer)
+		}
+	}()
+
 	err := json.NewEncoder(output).Encode(resp)
 
 	if err != nil {
@@ -386,23 +416,31 @@ type FileResponseEntry struct {
 }
 
 type fileResponse struct {
-	req     *http.Request
 	files   []FileResponseEntry
 	headers map[string]string
 }
 
 // FileResponse returns a new file response.
-func FileResponse(r *http.Request, files []FileResponseEntry, headers map[string]string) Response {
-	return &fileResponse{r, files, headers}
+func FileResponse(files []FileResponseEntry, headers map[string]string) Response {
+	return &fileResponse{files, headers}
 }
 
-func (r *fileResponse) Render(w http.ResponseWriter) error {
+// Render renders a file response.
+func (r *fileResponse) Render(w http.ResponseWriter, req *http.Request) error {
 	if r.headers != nil {
 		for k, v := range r.headers {
 			w.Header().Set(k, v)
 		}
 	}
 
+	var err error
+	defer func() {
+		if err == nil {
+			// If there was an error on Render, the callback function will be called during the error handling.
+			metrics.UseMetricsCallback(req, metrics.Success)
+		}
+	}()
+
 	// No file, well, it's easy then
 	if len(r.files) == 0 {
 		return nil
@@ -410,11 +448,11 @@ func (r *fileResponse) Render(w http.ResponseWriter) error {
 
 	// For a single file, return it inline
 	if len(r.files) == 1 {
-		remoteConn := ucred.GetConnFromContext(r.req.Context())
+		remoteConn := ucred.GetConnFromContext(req.Context())
 		remoteTCP, _ := tcp.ExtractConn(remoteConn)
 		if remoteTCP != nil {
 			// Apply TCP timeouts if remote connection is TCP (rather than Unix).
-			err := tcp.SetTimeouts(remoteTCP, 10*time.Second)
+			err = tcp.SetTimeouts(remoteTCP, 10*time.Second)
 			if err != nil {
 				return api.StatusErrorf(http.StatusInternalServerError, "Failed setting TCP timeouts on remote connection: %w", err)
 			}
@@ -433,14 +471,16 @@ func (r *fileResponse) Render(w http.ResponseWriter) error {
 			mt = r.files[0].FileModified
 			sz = r.files[0].FileSize
 		} else {
-			f, err := os.Open(r.files[0].Path)
+			var f *os.File
+			f, err = os.Open(r.files[0].Path)
 			if err != nil {
 				return err
 			}
 
 			defer func() { _ = f.Close() }()
 
-			fi, err := f.Stat()
+			var fi fs.FileInfo
+			fi, err = f.Stat()
 			if err != nil {
 				return err
 			}
@@ -451,10 +491,10 @@ func (r *fileResponse) Render(w http.ResponseWriter) error {
 		}
 
 		w.Header().Set("Content-Type", "application/octet-stream")
-		w.Header().Set("Content-Length", fmt.Sprintf("%d", sz))
-		w.Header().Set("Content-Disposition", fmt.Sprintf("inline;filename=%s", r.files[0].Filename))
+		w.Header().Set("Content-Length", strconv.FormatInt(sz, 10))
+		w.Header().Set("Content-Disposition", "inline;filename="+r.files[0].Filename)
 
-		http.ServeContent(w, r.req, r.files[0].Filename, mt, rs)
+		http.ServeContent(w, req, r.files[0].Filename, mt, rs)
 
 		return nil
 	}
@@ -471,7 +511,8 @@ func (r *fileResponse) Render(w http.ResponseWriter) error {
 		if entry.File != nil {
 			rd = entry.File
 		} else {
-			fd, err := os.Open(entry.Path)
+			var fd *os.File
+			fd, err = os.Open(entry.Path)
 			if err != nil {
 				return err
 			}
@@ -481,12 +522,13 @@ func (r *fileResponse) Render(w http.ResponseWriter) error {
 			rd = fd
 		}
 
-		fw, err := mw.CreateFormFile(entry.Identifier, entry.Filename)
+		var fw io.Writer
+		fw, err = mw.CreateFormFile(entry.Identifier, entry.Filename)
 		if err != nil {
 			return err
 		}
 
-		_, err = io.Copy(fw, rd)
+		_, err := io.Copy(fw, rd)
 		if err != nil {
 			return err
 		}
@@ -496,41 +538,42 @@ func (r *fileResponse) Render(w http.ResponseWriter) error {
 		}
 	}
 
-	return mw.Close()
+	err = mw.Close()
+
+	return err
 }
 
 func (r *fileResponse) String() string {
-	return fmt.Sprintf("%d files", len(r.files))
+	return strconv.FormatInt(int64(len(r.files)), 10) + " files"
 }
 
 type forwardedResponse struct {
-	client  lxd.InstanceServer
-	request *http.Request
+	client lxd.InstanceServer
 }
 
 // ForwardedResponse takes a request directed to a node and forwards it to
 // another node, writing back the response it gegs.
 func ForwardedResponse(client lxd.InstanceServer, request *http.Request) Response {
 	return &forwardedResponse{
-		client:  client,
-		request: request,
+		client: client,
 	}
 }
 
-func (r *forwardedResponse) Render(w http.ResponseWriter) error {
+// Render renders a response for a forwarded request.
+func (r *forwardedResponse) Render(w http.ResponseWriter, req *http.Request) error {
 	info, err := r.client.GetConnectionInfo()
 	if err != nil {
 		return err
 	}
 
-	url := fmt.Sprintf("%s%s", info.Addresses[0], r.request.URL.RequestURI())
-	forwarded, err := http.NewRequest(r.request.Method, url, r.request.Body)
+	url := info.Addresses[0] + req.URL.RequestURI()
+	forwarded, err := http.NewRequest(req.Method, url, req.Body)
 	if err != nil {
 		return err
 	}
 
-	for key := range r.request.Header {
-		forwarded.Header.Set(key, r.request.Header.Get(key))
+	for key := range req.Header {
+		forwarded.Header.Set(key, req.Header.Get(key))
 	}
 
 	httpClient, err := r.client.GetHTTPClient()
@@ -556,7 +599,7 @@ func (r *forwardedResponse) Render(w http.ResponseWriter) error {
 }
 
 func (r *forwardedResponse) String() string {
-	return fmt.Sprintf("request to %s", r.request.URL)
+	return "forwarded response"
 }
 
 type manualResponse struct {
@@ -568,8 +611,16 @@ func ManualResponse(hook func(w http.ResponseWriter) error) Response {
 	return &manualResponse{hook: hook}
 }
 
-func (r *manualResponse) Render(w http.ResponseWriter) error {
-	return r.hook(w)
+// Render renders a manual response.
+func (r *manualResponse) Render(w http.ResponseWriter, req *http.Request) error {
+	err := r.hook(w)
+
+	if err == nil {
+		// If there was an error on Render, the callback function will be called during the error handling.
+		metrics.UseMetricsCallback(req, metrics.Success)
+	}
+
+	return err
 }
 
 func (r *manualResponse) String() string {
