@@ -23,7 +23,7 @@ func SmartError(err error) Response {
 
 	statusCode, found := api.StatusErrorMatch(err)
 	if found {
-		return &errorResponse{statusCode, err}
+		return &errorResponse{statusCode, err.Error()}
 	}
 
 	for httpStatusCode, checkErrs := range httpResponseErrors {
@@ -31,16 +31,17 @@ func SmartError(err error) Response {
 			if errors.Is(err, checkErr) {
 				if err != checkErr {
 					// If the error has been wrapped return the top-level error message.
-					return &errorResponse{httpStatusCode, err}
+					return &errorResponse{httpStatusCode, err.Error()}
 				}
 
-				// If the error hasn't been wrapped, use a generic error.
-				return &errorResponse{httpStatusCode, nil}
+				// If the error hasn't been wrapped, replace the error message with the generic
+				// HTTP status text.
+				return &errorResponse{httpStatusCode, http.StatusText(httpStatusCode)}
 			}
 		}
 	}
 
-	return &errorResponse{http.StatusInternalServerError, err}
+	return &errorResponse{http.StatusInternalServerError, err.Error()}
 }
 
 // IsNotFoundError returns true if the error is considered a Not Found error.
