@@ -2,7 +2,6 @@ package scriptlet
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"strconv"
 	"strings"
@@ -41,11 +40,11 @@ func InstancePlacementRun(ctx context.Context, l logger.Logger, s *state.State,
 
 		switch b.Name() {
 		case "log_info":
-			l.Info("Instance placement scriptlet: " + sb.String())
+			l.Info(fmt.Sprintf("Instance placement scriptlet: %s", sb.String()))
 		case "log_warn":
-			l.Warn("Instance placement scriptlet: " + sb.String())
+			l.Warn(fmt.Sprintf("Instance placement scriptlet: %s", sb.String()))
 		default:
-			l.Error("Instance placement scriptlet: " + sb.String())
+			l.Error(fmt.Sprintf("Instance placement scriptlet: %s", sb.String()))
 		}
 
 		return starlark.None, nil
@@ -108,7 +107,7 @@ func InstancePlacementRun(ctx context.Context, l logger.Logger, s *state.State,
 				return starlark.String("Invalid member name"), nil
 			}
 
-			client, err := cluster.Connect(context.Background(), targetMember.Address, s.Endpoints.NetworkCert(), s.ServerCert(), true)
+			client, err := cluster.Connect(targetMember.Address, s.Endpoints.NetworkCert(), s.ServerCert(), nil, true)
 			if err != nil {
 				return nil, err
 			}
@@ -157,7 +156,7 @@ func InstancePlacementRun(ctx context.Context, l logger.Logger, s *state.State,
 				return starlark.String("Invalid member name"), nil
 			}
 
-			client, err := cluster.Connect(context.Background(), targetMember.Address, s.Endpoints.NetworkCert(), s.ServerCert(), true)
+			client, err := cluster.Connect(targetMember.Address, s.Endpoints.NetworkCert(), s.ServerCert(), nil, true)
 			if err != nil {
 				return nil, err
 			}
@@ -272,9 +271,9 @@ func InstancePlacementRun(ctx context.Context, l logger.Logger, s *state.State,
 			return fmt.Errorf("Failed loading member failure domains: %w", err)
 		}
 
-		members, err := tx.GetNodes(ctx)
+		maxVersion, err := tx.GetNodeMaxVersion(ctx)
 		if err != nil {
-			return fmt.Errorf("Failed getting cluster members: %w", err)
+			return fmt.Errorf("Failed getting max member version: %w", err)
 		}
 
 		args := db.NodeInfoArgs{
@@ -282,7 +281,7 @@ func InstancePlacementRun(ctx context.Context, l logger.Logger, s *state.State,
 			FailureDomains:       failureDomains,
 			MemberFailureDomains: memberFailureDomains,
 			OfflineThreshold:     s.GlobalConfig.OfflineThreshold(),
-			Members:              members,
+			MaxMemberVersion:     maxVersion,
 			RaftNodes:            raftNodes,
 		}
 
@@ -333,7 +332,7 @@ func InstancePlacementRun(ctx context.Context, l logger.Logger, s *state.State,
 	// Retrieve a global variable from starlark environment.
 	instancePlacement := globals["instance_placement"]
 	if instancePlacement == nil {
-		return nil, errors.New("Scriptlet missing instance_placement function")
+		return nil, fmt.Errorf("Scriptlet missing instance_placement function")
 	}
 
 	rv, err := StarlarkMarshal(req)
