@@ -64,11 +64,6 @@ static int seccomp_notify_get_sizes(struct seccomp_notif_sizes *sizes)
 	return 0;
 }
 
-static bool is_whiteout(dev_t dev, mode_t mode)
-{
-	return ((mode & S_IFMT) == S_IFCHR) && (dev == makedev(0, 0));
-}
-
 static int device_allowed(dev_t dev, mode_t mode)
 {
 	switch (mode & S_IFMT) {
@@ -478,10 +473,8 @@ import "C"
 import (
 	"bytes"
 	"context"
-	"errors"
 	"fmt"
 	"io"
-	"math"
 	"net"
 	"os"
 	"path/filepath"
@@ -1005,7 +998,7 @@ func (siov *Iovec) ReceiveSeccompIovec(fd int) (uint64, error) {
 // IsValidSeccompIovec checks whether a seccomp iovec is valid.
 func (siov *Iovec) IsValidSeccompIovec(size uint64) bool {
 	if size < uint64(C.SECCOMP_MSG_SIZE_MIN) {
-		logger.Warn("Disconnected from seccomp socket after incomplete receive")
+		logger.Warnf("Disconnected from seccomp socket after incomplete receive")
 		return false
 	}
 
@@ -1047,7 +1040,7 @@ retry:
 	bytes, err := C.sendmsg(C.int(fd), &msghdr, C.MSG_NOSIGNAL)
 	if bytes < 0 {
 		if err == unix.EINTR {
-			logger.Debug("Caught EINTR, retrying...")
+			logger.Debugf("Caught EINTR, retrying...")
 			goto retry
 		}
 
@@ -1068,7 +1061,7 @@ retry:
 func NewSeccompServer(s *state.State, path string, findPID func(pid int32, state *state.State) (Instance, error)) (*Server, error) {
 	ret := C.seccomp_notify_get_sizes(&C.expected_sizes)
 	if ret < 0 {
-		return nil, errors.New("Failed to query kernel for seccomp notifier sizes")
+		return nil, fmt.Errorf("Failed to query kernel for seccomp notifier sizes")
 	}
 
 	// Cleanup existing sockets
@@ -1116,7 +1109,7 @@ func NewSeccompServer(s *state.State, path string, findPID func(pid int32, state
 
 				unixFile, err := c.(*net.UnixConn).File()
 				if err != nil {
-					logger.Debug("Failed to turn unix socket client into file")
+					logger.Debugf("Failed to turn unix socket client into file")
 					return
 				}
 
@@ -1258,7 +1251,7 @@ func FindTGID(procFd int) (uint32, error) {
 		}
 	}
 
-	return 0, errors.New("Task group leader ID not found")
+	return 0, fmt.Errorf("Task group leader ID not found")
 }
 
 // isCapableInCtInitUserns checks if intercepted syscall's caller has a (cap) effective
@@ -1364,7 +1357,7 @@ func (s *Server) doDeviceSyscall(c Instance, args *MknodArgs, siov *Iovec) int {
 		return int(-C.EPERM)
 	}
 
-	if !hasCapability && !bool(C.is_whiteout(args.cDev, args.cMode)) {
+	if !hasCapability {
 		l.Error("Requestor process creds lacks CAP_MKNOD")
 		return int(-C.EPERM)
 	}
@@ -1453,6 +1446,7 @@ func (s *Server) HandleMknodatSyscall(c Instance, siov *Iovec) int {
 	// built on 64bit userspace correctly.
 	if int32(siov.req.data.args[0]) != int32(C.AT_FDCWD) {
 		ctx["err"] = "Non AT_FDCWD mknodat calls are not allowed"
+		logger.Debug("bla", ctx)
 		if s.s.OS.SeccompListenerContinue {
 			ctx["syscall_continue"] = "true"
 			C.seccomp_notify_update_response(siov.resp, 0, C.uint32_t(seccompUserNotifFlagContinue))
@@ -1869,12 +1863,7 @@ func (s *Server) HandleSysinfoSyscall(c Instance, siov *Iovec) int {
 		return 0
 	}
 
-	// Limit process count to MaxUint16 to avoid integer overflow.
-	if pids > math.MaxUint16 {
-		instMetrics.Procs = math.MaxUint16
-	} else {
-		instMetrics.Procs = uint16(pids)
-	}
+	instMetrics.Procs = uint16(pids)
 
 	// Get instance memory stats.
 	memStats, err := cg.GetMemoryStats()
@@ -2063,10 +2052,10 @@ func (s *Server) HandleFinitModuleSyscall(c Instance, siov *Iovec) int {
 
 	forksyscallgoargs := []string{
 		"forksyscallgo",
-		"finit_module_parse", // <syscall_operation>
-		strconv.FormatUint(uint64(siov.req.pid), 10), // <PID>
-		"0", // <PidFd>
-		"3", // <module_fd>
+		"finit_module_parse",                 // <syscall_operation>
+		fmt.Sprintf("%d", int(siov.req.pid)), // <PID>
+		fmt.Sprintf("%d", 0),                 // <PidFd>
+		fmt.Sprintf("%d", 3),                 // <module_fd>
 	}
 
 	var buffer bytes.Buffer
@@ -2119,7 +2108,7 @@ func (s *Server) HandleFinitModuleSyscall(c Instance, siov *Iovec) int {
 		return int(-C.EPERM)
 	}
 
-	if shared.PathExists("/sys/module/" + moduleName) {
+	if shared.PathExists(fmt.Sprintf("/sys/module/%s", moduleName)) {
 		return int(-C.EEXIST)
 	}
 
@@ -2467,22 +2456,22 @@ func (s *Server) HandleMountSyscall(c Instance, siov *Iovec) int {
 			util.GetExecPath(),
 			"forksyscall",
 			"mount",
-			strconv.Itoa(args.pid),
-			strconv.Itoa(pidFdNr),
+			fmt.Sprintf("%d", args.pid),
+			fmt.Sprintf("%d", pidFdNr),
 			"0",
 			args.source,
 			args.target,
 			args.fstype,
-			strconv.Itoa(args.flags),
+			fmt.Sprintf("%d", args.flags),
 			string(args.idmapType),
-			strconv.FormatInt(args.uid, 10),
-			strconv.FormatInt(args.gid, 10),
-			strconv.FormatInt(args.fsuid, 10),
-			strconv.FormatInt(args.fsgid, 10),
-			strconv.FormatInt(args.nsuid, 10),
-			strconv.FormatInt(args.nsgid, 10),
-			strconv.FormatInt(args.nsfsuid, 10),
-			strconv.FormatInt(args.nsfsgid, 10),
+			fmt.Sprintf("%d", args.uid),
+			fmt.Sprintf("%d", args.gid),
+			fmt.Sprintf("%d", args.fsuid),
+			fmt.Sprintf("%d", args.fsgid),
+			fmt.Sprintf("%d", args.nsuid),
+			fmt.Sprintf("%d", args.nsgid),
+			fmt.Sprintf("%d", args.nsfsuid),
+			fmt.Sprintf("%d", args.nsfsgid),
 			args.data)
 	}
 
@@ -2618,7 +2607,7 @@ func lxcSupportSeccompNotifyContinue(state *state.State) error {
 	}
 
 	if !state.OS.SeccompListenerContinue {
-		return errors.New("Seccomp notify doesn't support continuing syscalls")
+		return fmt.Errorf("Seccomp notify doesn't support continuing syscalls")
 	}
 
 	return nil
@@ -2631,11 +2620,11 @@ func lxcSupportSeccompNotifyAddfd(state *state.State) error {
 	}
 
 	if !state.OS.SeccompListenerContinue {
-		return errors.New("Seccomp notify doesn't support continuing syscalls")
+		return fmt.Errorf("Seccomp notify doesn't support continuing syscalls")
 	}
 
 	if !state.OS.SeccompListenerAddfd {
-		return errors.New("Seccomp notify doesn't support adding file descriptors")
+		return fmt.Errorf("Seccomp notify doesn't support adding file descriptors")
 	}
 
 	return nil
@@ -2643,16 +2632,16 @@ func lxcSupportSeccompNotifyAddfd(state *state.State) error {
 
 func lxcSupportSeccompNotify(state *state.State) error {
 	if !state.OS.SeccompListener {
-		return errors.New("Seccomp notify not supported")
+		return fmt.Errorf("Seccomp notify not supported")
 	}
 
 	if !state.OS.LXCFeatures["seccomp_notify"] {
-		return errors.New("LXC doesn't support seccomp notify")
+		return fmt.Errorf("LXC doesn't support seccomp notify")
 	}
 
 	c, err := liblxc.NewContainer("test-seccomp", state.OS.LxcPath)
 	if err != nil {
-		return errors.New("Failed to load seccomp notify test container")
+		return fmt.Errorf("Failed to load seccomp notify test container")
 	}
 
 	err = c.SetConfigItem("lxc.seccomp.notify.proxy", "unix:"+shared.VarPath("seccomp.socket"))
