@@ -46,7 +46,7 @@ func readStoragePoolDriversCache() ([]api.ServerStorageDriverInfo, map[string]st
 	return driverInfo, driversUsed
 }
 
-func storageStartup(s *state.State, forceCheck bool) error {
+func storageStartup(s *state.State) error {
 	// Update the storage drivers supported and used cache in api_1.0.go.
 	storagePoolDriversCacheUpdate(s)
 
@@ -180,7 +180,7 @@ func storagePoolDriversCacheUpdate(s *state.State) {
 	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
-		drivers, err = tx.GetStoragePoolDrivers(ctx)
+		drivers, err = tx.GetStorageDrivers(ctx)
 
 		return err
 	})
@@ -209,7 +209,7 @@ func storagePoolDriversCacheUpdate(s *state.State) {
 	// Prepare the cache entries.
 	backends := []string{}
 	for k, v := range usedDrivers {
-		backends = append(backends, fmt.Sprintf("%s %s", k, v))
+		backends = append(backends, k+" "+v)
 	}
 
 	// Update the user agent.
