@@ -2,7 +2,6 @@ package drivers
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"os"
 	"os/exec"
@@ -64,7 +63,7 @@ func (d *btrfs) load() error {
 
 		count, err := fmt.Sscanf(strings.SplitN(out, " ", 2)[1], "v%s\n", &btrfsVersion)
 		if err != nil || count != 1 {
-			return errors.New("The 'btrfs' tool isn't working properly")
+			return fmt.Errorf("The 'btrfs' tool isn't working properly")
 		}
 	}
 
@@ -231,7 +230,7 @@ func (d *btrfs) Create() error {
 
 			// Check that the provided subvolume is empty.
 			if hasSubvolumes {
-				return errors.New("Requested btrfs subvolume exists but is not empty")
+				return fmt.Errorf("Requested btrfs subvolume exists but is not empty")
 			}
 		} else {
 			// New btrfs subvolume on existing btrfs filesystem.
@@ -269,7 +268,7 @@ func (d *btrfs) Create() error {
 			}
 		}
 	} else {
-		return errors.New(`Invalid "source" property`)
+		return fmt.Errorf(`Invalid "source" property`)
 	}
 
 	revert.Success()
@@ -371,7 +370,7 @@ func (d *btrfs) Update(changedConfig map[string]string) error {
 		mntFlags, mntOptions := filesystem.ResolveMountOptions(strings.Split(d.getMountOptions(), ","))
 		mntFlags |= unix.MS_REMOUNT
 
-		err := TryMount(context.TODO(), "", GetPoolMountPath(d.name), "none", mntFlags, mntOptions)
+		err := TryMount("", GetPoolMountPath(d.name), "none", mntFlags, mntOptions)
 		if err != nil {
 			return err
 		}
@@ -383,7 +382,7 @@ func (d *btrfs) Update(changedConfig map[string]string) error {
 		loopPath := loopFilePath(d.name)
 
 		if d.config["source"] != loopPath {
-			return errors.New("Cannot resize non-loopback pools")
+			return fmt.Errorf("Cannot resize non-loopback pools")
 		}
 
 		// Resize loop file
@@ -466,7 +465,7 @@ func (d *btrfs) Mount() (bool, error) {
 	// Handle bind-mounts first.
 	if mntFilesystem == "none" {
 		// Setup the bind-mount itself.
-		err := TryMount(context.TODO(), mntSrc, mntDst, mntFilesystem, unix.MS_BIND, "")
+		err := TryMount(mntSrc, mntDst, mntFilesystem, unix.MS_BIND, "")
 		if err != nil {
 			return false, err
 		}
@@ -478,7 +477,7 @@ func (d *btrfs) Mount() (bool, error) {
 
 		// Now apply the custom options.
 		mntFlags |= unix.MS_REMOUNT
-		err = TryMount(context.TODO(), "", mntDst, mntFilesystem, mntFlags, mntOptions)
+		err = TryMount("", mntDst, mntFilesystem, mntFlags, mntOptions)
 		if err != nil {
 			return false, err
 		}
@@ -487,7 +486,7 @@ func (d *btrfs) Mount() (bool, error) {
 	}
 
 	// Handle traditional mounts.
-	err = TryMount(context.TODO(), mntSrc, mntDst, mntFilesystem, mntFlags, mntOptions)
+	err = TryMount(mntSrc, mntDst, mntFilesystem, mntFlags, mntOptions)
 	if err != nil {
 		return false, err
 	}
