@@ -4,7 +4,6 @@ import (
 	"bufio"
 	"bytes"
 	"context"
-	"errors"
 	"fmt"
 	"io"
 	"io/fs"
@@ -30,8 +29,8 @@ import (
 )
 
 // Errors.
-var errBtrfsNoQuota = errors.New("Quotas disabled on filesystem")
-var errBtrfsNoQGroup = errors.New("Unable to find quota group")
+var errBtrfsNoQuota = fmt.Errorf("Quotas disabled on filesystem")
+var errBtrfsNoQGroup = fmt.Errorf("Unable to find quota group")
 
 // btrfsISOVolSuffix suffix used for iso content type volumes.
 const btrfsISOVolSuffix = ".iso"
@@ -315,7 +314,7 @@ func (d *btrfs) getQGroup(path string) (string, int64, error) {
 	// Parse to extract the qgroup identifier.
 	var qgroup string
 	usage := int64(-1)
-	for line := range strings.SplitSeq(output, "\n") {
+	for _, line := range strings.Split(output, "\n") {
 		// Use case-insensitive field title match because BTRFS tooling changed casing between versions.
 		if line == "" || strings.HasPrefix(strings.ToLower(line), "qgroupid") || strings.HasPrefix(line, "-") {
 			continue
@@ -405,7 +404,7 @@ func (d *btrfs) setSubvolumeReadonlyProperty(path string, readonly bool) error {
 		args = append(args, "-f")
 	}
 
-	args = append(args, "-ts", path, "ro", strconv.FormatBool(readonly))
+	args = append(args, "-ts", path, "ro", fmt.Sprint(readonly))
 
 	_, err := shared.RunCommandContext(context.TODO(), "btrfs", args...)
 	return err
@@ -572,7 +571,7 @@ func (d *btrfs) loadOptimizedBackupHeader(r io.ReadSeeker, mountPath string) (*B
 	header := BTRFSMetaDataHeader{}
 
 	// Extract.
-	tr, cancelFunc, err := backup.TarReader(d.state, r, mountPath)
+	tr, cancelFunc, err := backup.TarReader(r, d.state.OS, mountPath)
 	if err != nil {
 		return nil, err
 	}
@@ -600,7 +599,7 @@ func (d *btrfs) loadOptimizedBackupHeader(r io.ReadSeeker, mountPath string) (*B
 		}
 	}
 
-	return nil, errors.New("Optimized backup header file not found")
+	return nil, fmt.Errorf("Optimized backup header file not found")
 }
 
 // receiveSubVolume receives a subvolume from an io.Reader into the receivePath and returns the path to the received subvolume.
@@ -650,7 +649,7 @@ func (d *btrfs) receiveSubVolume(r io.Reader, receivePath string, tracker *iopro
 	}
 
 	if filename == "" {
-		return "", errors.New("Failed to determine received subvolume")
+		return "", fmt.Errorf("Failed to determine received subvolume")
 	}
 
 	subVolPath := filepath.Join(receivePath, filename)
