@@ -10,8 +10,6 @@ import (
 	"io"
 	"os"
 	"path/filepath"
-	"slices"
-	"strconv"
 	"strings"
 	"time"
 
@@ -80,7 +78,7 @@ func (d *btrfs) CreateVolume(vol Volume, filler *VolumeFiller, op *operations.Op
 		// data being referenced.
 		//
 		// An exception is made for when compression is enabled on the underlying storage.
-		if !slices.Contains(mountOptions, "datacow") && !strings.Contains(mountinfo[len(mountinfo)-1], "compress") {
+		if !shared.ValueInSlice("datacow", mountOptions) && !strings.Contains(mountinfo[len(mountinfo)-1], "compress") {
 			_, err = shared.RunCommandContext(context.TODO(), "chattr", "+C", volPath)
 			if err != nil {
 				return fmt.Errorf("Failed setting nodatacow on %q: %w", volPath, err)
@@ -154,7 +152,7 @@ func (d *btrfs) CreateVolume(vol Volume, filler *VolumeFiller, op *operations.Op
 func (d *btrfs) CreateVolumeFromBackup(vol VolumeCopy, srcBackup backup.Info, srcData io.ReadSeeker, op *operations.Operation) (VolumePostHook, revert.Hook, error) {
 	// Handle the non-optimized tarballs through the generic unpacker.
 	if !*srcBackup.OptimizedStorage {
-		return genericVFSBackupUnpack(d, d.state, vol, srcBackup.Snapshots, srcData, op)
+		return genericVFSBackupUnpack(d, d.state.OS, vol, srcBackup.Snapshots, srcData, op)
 	}
 
 	volExists, err := d.HasVolume(vol.Volume)
@@ -163,7 +161,7 @@ func (d *btrfs) CreateVolumeFromBackup(vol VolumeCopy, srcBackup backup.Info, sr
 	}
 
 	if volExists {
-		return nil, nil, errors.New("Cannot restore volume, already exists on target")
+		return nil, nil, fmt.Errorf("Cannot restore volume, already exists on target")
 	}
 
 	revert := revert.New()
@@ -238,7 +236,7 @@ func (d *btrfs) CreateVolumeFromBackup(vol VolumeCopy, srcBackup backup.Info, sr
 
 	// unpackSubVolume unpacks a subvolume file from a backup tarball file.
 	unpackSubVolume := func(r io.ReadSeeker, unpacker []string, srcFile string, targetPath string) (string, error) {
-		tr, cancelFunc, err := archive.CompressedTarReader(d.state, context.Background(), r, unpacker, targetPath)
+		tr, cancelFunc, err := archive.CompressedTarReader(context.Background(), r, unpacker, d.state.OS, targetPath)
 		if err != nil {
 			return "", err
 		}
@@ -338,14 +336,12 @@ func (d *btrfs) CreateVolumeFromBackup(vol VolumeCopy, srcBackup backup.Info, sr
 			snapVol, _ := vol.NewSnapshot(snapName)
 			snapDir := "snapshots"
 			srcFilePrefix := snapName
-			switch vol.volType {
-			case VolumeTypeVM:
+			if vol.volType == VolumeTypeVM {
 				snapDir = "virtual-machine-snapshots"
 				if vol.contentType == ContentTypeFS {
 					srcFilePrefix = snapName + "-config"
 				}
-
-			case VolumeTypeCustom:
+			} else if vol.volType == VolumeTypeCustom {
 				snapDir = "volume-snapshots"
 			}
 
@@ -359,15 +355,13 @@ func (d *btrfs) CreateVolumeFromBackup(vol VolumeCopy, srcBackup backup.Info, sr
 
 	// Extract main volume.
 	srcFilePrefix := "container"
-	switch vol.volType {
-	case VolumeTypeVM:
+	if vol.volType == VolumeTypeVM {
 		if vol.contentType == ContentTypeFS {
 			srcFilePrefix = "virtual-machine-config"
 		} else {
 			srcFilePrefix = "virtual-machine"
 		}
-
-	case VolumeTypeCustom:
+	} else if vol.volType == VolumeTypeCustom {
 		srcFilePrefix = "volume"
 	}
 
@@ -400,7 +394,7 @@ func (d *btrfs) CreateVolumeFromBackup(vol VolumeCopy, srcBackup backup.Info, sr
 
 		v := vol.Volume
 		if subVol.Snapshot != "" {
-			v, _ = vol.NewSnapshot(subVol.Snapshot)
+			v, _ = vol.Volume.NewSnapshot(subVol.Snapshot)
 		}
 
 		path := filepath.Join(v.MountPath(), subVol.Path)
@@ -514,7 +508,7 @@ func (d *btrfs) createVolumeFromCopy(vol VolumeCopy, srcVol VolumeCopy, allowInc
 				// Skip snapshots that shouldn't be refreshed on the target volume.
 				// This could be either because the snapshot itself isn't in the list of target volume snapshots
 				// inside of the DB or the snapshot already exists on the target volume.
-				if !found || slices.Contains(targetSnapshots, snapName) {
+				if !found || shared.ValueInSlice(snapName, targetSnapshots) {
 					continue
 				}
 			}
@@ -565,7 +559,7 @@ func (d *btrfs) CreateVolumeFromMigration(vol VolumeCopy, conn io.ReadWriteClose
 	var syncSubvolumes []BTRFSSubVolume
 
 	// Inspect negotiated features to see if we are expecting to get a metadata migration header frame.
-	if slices.Contains(volTargetArgs.MigrationType.Features, migration.BTRFSFeatureMigrationHeader) {
+	if shared.ValueInSlice(migration.BTRFSFeatureMigrationHeader, volTargetArgs.MigrationType.Features) {
 		buf, err := io.ReadAll(conn)
 		if err != nil {
 			return fmt.Errorf("Failed reading BTRFS migration header: %w", err)
@@ -594,7 +588,7 @@ func (d *btrfs) CreateVolumeFromMigration(vol VolumeCopy, conn io.ReadWriteClose
 		})
 	}
 
-	if volTargetArgs.Refresh && slices.Contains(volTargetArgs.MigrationType.Features, migration.BTRFSFeatureSubvolumeUUIDs) {
+	if volTargetArgs.Refresh && shared.ValueInSlice(migration.BTRFSFeatureSubvolumeUUIDs, volTargetArgs.MigrationType.Features) {
 		snapshots, err := d.volumeSnapshotsSorted(vol.Volume, op)
 		if err != nil {
 			return err
@@ -845,7 +839,7 @@ func (d *btrfs) DeleteVolume(vol Volume, op *operations.Operation) error {
 	}
 
 	if len(snapshots) > 0 {
-		return errors.New("Cannot remove a volume that has snapshots")
+		return fmt.Errorf("Cannot remove a volume that has snapshots")
 	}
 
 	volName := vol.name
@@ -992,11 +986,10 @@ func (d *btrfs) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool,
 			}
 
 			id := ""
-			for line := range strings.SplitSeq(output, "\n") {
+			for _, line := range strings.Split(output, "\n") {
 				_, value, found := strings.Cut(line, "Subvolume ID:")
 				if found {
 					id = strings.TrimSpace(value)
-					break
 				}
 			}
 
@@ -1035,7 +1028,7 @@ func (d *btrfs) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool,
 		}
 
 		// Apply the limit to referenced data in qgroup.
-		_, err = shared.RunCommandContext(context.TODO(), "btrfs", "qgroup", "limit", strconv.FormatInt(sizeBytes, 10), qgroup, volPath)
+		_, err = shared.RunCommandContext(context.TODO(), "btrfs", "qgroup", "limit", fmt.Sprint(sizeBytes), qgroup, volPath)
 		if err != nil {
 			return err
 		}
@@ -1189,7 +1182,7 @@ func (d *btrfs) MigrateVolume(vol VolumeCopy, conn io.ReadWriteCloser, volSrcArg
 	// Handle btrfs send/receive migration.
 	if volSrcArgs.MultiSync || volSrcArgs.FinalSync {
 		// This is not needed if the migration is performed using btrfs send/receive.
-		return errors.New("MultiSync should not be used with optimized migration")
+		return fmt.Errorf("MultiSync should not be used with optimized migration")
 	}
 
 	var snapshots []string
@@ -1210,16 +1203,16 @@ func (d *btrfs) MigrateVolume(vol VolumeCopy, conn io.ReadWriteCloser, volSrcArg
 
 	// If we haven't negotiated subvolume support, check if we have any subvolumes in source and fail,
 	// otherwise we would end up not materialising all of the source's files on the target.
-	if !slices.Contains(volSrcArgs.MigrationType.Features, migration.BTRFSFeatureMigrationHeader) || !slices.Contains(volSrcArgs.MigrationType.Features, migration.BTRFSFeatureSubvolumes) {
+	if !shared.ValueInSlice(migration.BTRFSFeatureMigrationHeader, volSrcArgs.MigrationType.Features) || !shared.ValueInSlice(migration.BTRFSFeatureSubvolumes, volSrcArgs.MigrationType.Features) {
 		for _, subVol := range migrationHeader.Subvolumes {
 			if subVol.Path != string(filepath.Separator) {
-				return errors.New("Subvolumes detected in source but target does not support receiving subvolumes")
+				return fmt.Errorf("Subvolumes detected in source but target does not support receiving subvolumes")
 			}
 		}
 	}
 
 	// Send metadata migration header frame with subvolume info if we have negotiated that feature.
-	if slices.Contains(volSrcArgs.MigrationType.Features, migration.BTRFSFeatureMigrationHeader) {
+	if shared.ValueInSlice(migration.BTRFSFeatureMigrationHeader, volSrcArgs.MigrationType.Features) {
 		headerJSON, err := json.Marshal(migrationHeader)
 		if err != nil {
 			return fmt.Errorf("Failed encoding BTRFS migration header: %w", err)
@@ -1238,7 +1231,7 @@ func (d *btrfs) MigrateVolume(vol VolumeCopy, conn io.ReadWriteCloser, volSrcArg
 		d.logger.Debug("Sent migration meta data header", logger.Ctx{"name": vol.name})
 	}
 
-	if volSrcArgs.Refresh && slices.Contains(volSrcArgs.MigrationType.Features, migration.BTRFSFeatureSubvolumeUUIDs) {
+	if volSrcArgs.Refresh && shared.ValueInSlice(migration.BTRFSFeatureSubvolumeUUIDs, volSrcArgs.MigrationType.Features) {
 		migrationHeader = &BTRFSMetaDataHeader{}
 
 		buf, err := io.ReadAll(conn)
@@ -1292,7 +1285,7 @@ func (d *btrfs) migrateVolumeOptimized(vol Volume, conn io.ReadWriteCloser, volS
 				continue // Only sending subvolumes related to snapshot name (empty for main vol).
 			}
 
-			if subVolume.Path != string(filepath.Separator) && !slices.Contains(volSrcArgs.MigrationType.Features, migration.BTRFSFeatureSubvolumes) {
+			if subVolume.Path != string(filepath.Separator) && !shared.ValueInSlice(migration.BTRFSFeatureSubvolumes, volSrcArgs.MigrationType.Features) {
 				continue // Skip sending subvolumes of volume if subvolumes feature not negotiated.
 			}
 
@@ -1483,7 +1476,8 @@ func (d *btrfs) BackupVolume(vol VolumeCopy, tarWriter *instancewriter.InstanceT
 		args = append(args, path)
 
 		// Create temporary file to store output of btrfs send.
-		tmpFile, err := os.CreateTemp(d.state.BackupsStoragePath(), backup.WorkingDirPrefix+"_btrfs")
+		backupsPath := shared.VarPath("backups")
+		tmpFile, err := os.CreateTemp(backupsPath, backup.WorkingDirPrefix+"_btrfs")
 		if err != nil {
 			return fmt.Errorf("Failed to open temporary file for BTRFS backup: %w", err)
 		}
@@ -1592,14 +1586,12 @@ func (d *btrfs) BackupVolume(vol VolumeCopy, tarWriter *instancewriter.InstanceT
 		// Make a binary btrfs backup.
 		snapDir := "snapshots"
 		fileName := snapName
-		switch vol.volType {
-		case VolumeTypeVM:
+		if vol.volType == VolumeTypeVM {
 			snapDir = "virtual-machine-snapshots"
 			if vol.contentType == ContentTypeFS {
 				fileName = snapName + "-config"
 			}
-
-		case VolumeTypeCustom:
+		} else if vol.volType == VolumeTypeCustom {
 			snapDir = "volume-snapshots"
 		}
 
@@ -1644,15 +1636,13 @@ func (d *btrfs) BackupVolume(vol VolumeCopy, tarWriter *instancewriter.InstanceT
 
 	// Dump the instance to a file.
 	fileNamePrefix := "container"
-	switch vol.volType {
-	case VolumeTypeVM:
+	if vol.volType == VolumeTypeVM {
 		if vol.contentType == ContentTypeFS {
 			fileNamePrefix = "virtual-machine-config"
 		} else {
 			fileNamePrefix = "virtual-machine"
 		}
-
-	case VolumeTypeCustom:
+	} else if vol.volType == VolumeTypeCustom {
 		fileNamePrefix = "volume"
 	}
 
