@@ -4,10 +4,8 @@ import (
 	"bytes"
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"os/exec"
-	"slices"
 	"strconv"
 	"strings"
 
@@ -48,7 +46,7 @@ func (d *ceph) load() error {
 	for _, tool := range []string{"ceph", "rbd"} {
 		_, err := exec.LookPath(tool)
 		if err != nil {
-			return fmt.Errorf("Required tool %q is missing", tool)
+			return fmt.Errorf("Required tool '%s' is missing", tool)
 		}
 	}
 
@@ -150,7 +148,7 @@ func (d *ceph) Create() error {
 
 	// Quick check.
 	if d.config["source"] != "" && d.config["ceph.osd.pool_name"] != "" && d.config["source"] != d.config["ceph.osd.pool_name"] {
-		return errors.New(`The "source" and "ceph.osd.pool_name" property must not differ for Ceph OSD storage pools`)
+		return fmt.Errorf(`The "source" and "ceph.osd.pool_name" property must not differ for Ceph OSD storage pools`)
 	}
 
 	// Use an existing OSD pool.
@@ -407,33 +405,13 @@ func (d *ceph) Validate(config map[string]string) error {
 		"volatile.pool.pristine": validate.IsAny,
 	}
 
-	immutableOptions := []string{
-		// Changing the cluster name does not work as the volume's won't be moved to the new cluster.
-		"ceph.cluster_name",
-		// Changing the pool name whilst having active volumes does not work as the volumes won't be moved to the new pool.
-		"ceph.osd.pool_name",
-	}
-
-	for configOption, configOptionValue := range config {
-		oldValue, ok := d.config[configOption]
-
-		// Skip config settings which weren't populated before.
-		if !ok {
-			continue
-		}
-
-		if oldValue != configOptionValue && slices.Contains(immutableOptions, configOption) {
-			return fmt.Errorf("Cannot update %q", configOption)
-		}
-	}
-
 	return d.validatePool(config, rules, d.commonVolumeRules())
 }
 
 // Update applies any driver changes required from a configuration change.
 func (d *ceph) Update(changedConfig map[string]string) error {
-	// applyPool applies a OSD pool level setting.
-	applyPool := func(key string, value string) error {
+	newSize, changed := changedConfig["ceph.osd.pool_size"]
+	if changed {
 		_, err := shared.TryRunCommand("ceph",
 			"--name", "client."+d.config["ceph.user.name"],
 			"--cluster", d.config["ceph.cluster_name"],
@@ -441,24 +419,9 @@ func (d *ceph) Update(changedConfig map[string]string) error {
 			"pool",
 			"set",
 			d.config["ceph.osd.pool_name"],
-			key,
-			value,
-			// Not all settings require this flag but we can set it nonetheless.
+			"size",
+			newSize,
 			"--yes-i-really-mean-it")
-		return err
-	}
-
-	newSize, changed := changedConfig["ceph.osd.pool_size"]
-	if changed {
-		err := applyPool("size", newSize)
-		if err != nil {
-			return err
-		}
-	}
-
-	newPgNum, changed := changedConfig["ceph.osd.pg_num"]
-	if changed {
-		err := applyPool("pg_num", newPgNum)
 		if err != nil {
 			return err
 		}
@@ -476,7 +439,7 @@ func (d *ceph) Mount() (bool, error) {
 	}
 
 	if !volExists {
-		return false, errors.New("Placeholder volume does not exist")
+		return false, fmt.Errorf("Placeholder volume does not exist")
 	}
 
 	return true, nil
@@ -533,7 +496,7 @@ func (d *ceph) GetResources() (*api.ResourcesStoragePool, error) {
 	}
 
 	if pool == nil {
-		return nil, errors.New("OSD pool missing in df output")
+		return nil, fmt.Errorf("OSD pool missing in df output")
 	}
 
 	spaceUsed := uint64(pool.Stats.BytesUsed)
