@@ -1,6 +1,7 @@
 package drivers
 
 import (
+	"context"
 	"encoding/json"
 	"fmt"
 	"io"
@@ -47,9 +48,10 @@ var cephVolTypePrefixes = map[VolumeType]string{
 
 // osdPoolExists checks whether a given OSD pool exists.
 func (d *ceph) osdPoolExists() (bool, error) {
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"ceph",
-		"--name", fmt.Sprintf("client.%s", d.config["ceph.user.name"]),
+		"--name", "client."+d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
 		"osd",
 		"pool",
@@ -81,9 +83,10 @@ func (d *ceph) osdPoolExists() (bool, error) {
 //     that this call actually deleted an OSD pool it needs to check for the
 //     existence of the pool first.
 func (d *ceph) osdDeletePool() error {
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"ceph",
-		"--name", fmt.Sprintf("client.%s", d.config["ceph.user.name"]),
+		"--name", "client."+d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
 		"osd",
 		"pool",
@@ -133,7 +136,7 @@ func (d *ceph) rbdCreateVolume(vol Volume, size string) error {
 		"create",
 		d.getRBDVolumeName(vol, "", false, false))
 
-	_, err = shared.RunCommand("rbd", cmd...)
+	_, err = shared.RunCommandContext(context.TODO(), "rbd", cmd...)
 	return err
 }
 
@@ -143,7 +146,8 @@ func (d *ceph) rbdCreateVolume(vol Volume, size string) error {
 //     to be sure that this call actually deleted an RBD storage volume it needs
 //     to check for the existence of the pool first.
 func (d *ceph) rbdDeleteVolume(vol Volume) error {
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -162,7 +166,8 @@ func (d *ceph) rbdDeleteVolume(vol Volume) error {
 // in the /dev directory and is therefore necessary in order to mount it.
 func (d *ceph) rbdMapVolume(vol Volume) (string, error) {
 	rbdName := d.getRBDVolumeName(vol, "", false, false)
-	devPath, err := shared.RunCommand(
+	devPath, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -193,7 +198,8 @@ func (d *ceph) rbdUnmapVolume(vol Volume, unmapUntilEINVAL bool) error {
 	ourDeactivate := false
 
 again:
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -245,7 +251,8 @@ again:
 // This is a precondition in order to delete an RBD snapshot can.
 func (d *ceph) rbdUnmapVolumeSnapshot(vol Volume, snapshotName string, unmapUntilEINVAL bool) error {
 again:
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -276,7 +283,8 @@ again:
 
 // rbdCreateVolumeSnapshot creates a read-write snapshot of a given RBD storage volume.
 func (d *ceph) rbdCreateVolumeSnapshot(vol Volume, snapshotName string) error {
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -295,7 +303,8 @@ func (d *ceph) rbdCreateVolumeSnapshot(vol Volume, snapshotName string) error {
 // rbdProtectVolumeSnapshot protects a given snapshot from being deleted.
 // This is a precondition to be able to create RBD clones from a given snapshot.
 func (d *ceph) rbdProtectVolumeSnapshot(vol Volume, snapshotName string) error {
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -326,7 +335,8 @@ func (d *ceph) rbdProtectVolumeSnapshot(vol Volume, snapshotName string) error {
 // - This is a precondition to be able to delete an RBD snapshot.
 // - This command will only succeed if the snapshot does not have any clones.
 func (d *ceph) rbdUnprotectVolumeSnapshot(vol Volume, snapshotName string) error {
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -377,7 +387,7 @@ func (d *ceph) rbdCreateClone(sourceVol Volume, sourceSnapshotName string, targe
 		d.getRBDVolumeName(sourceVol, sourceSnapshotName, false, true),
 		d.getRBDVolumeName(targetVol, "", false, true))
 
-	_, err := shared.RunCommand("rbd", cmd...)
+	_, err := shared.RunCommandContext(context.TODO(), "rbd", cmd...)
 	if err != nil {
 		return err
 	}
@@ -387,7 +397,8 @@ func (d *ceph) rbdCreateClone(sourceVol Volume, sourceSnapshotName string, targe
 
 // rbdListSnapshotClones list all clones of an RBD snapshot.
 func (d *ceph) rbdListSnapshotClones(vol Volume, snapshotName string) ([]string, error) {
-	msg, err := shared.RunCommand(
+	msg, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -421,7 +432,8 @@ func (d *ceph) rbdMarkVolumeDeleted(vol Volume, newVolumeName string) error {
 	newVol := NewVolume(d, d.name, vol.volType, vol.contentType, newVolumeName, vol.config, vol.poolConfig)
 	deletedName := d.getRBDVolumeName(newVol, "", true, true)
 
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -446,7 +458,8 @@ func (d *ceph) rbdRenameVolume(vol Volume, newVolumeName string) error {
 	// new volume name generated in getRBDVolumeName.
 	newVol := NewVolume(d, d.name, vol.volType, vol.contentType, newVolumeName, vol.config, vol.poolConfig)
 
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -468,7 +481,8 @@ func (d *ceph) rbdRenameVolume(vol Volume, newVolumeName string) error {
 // original name and the caller maps it under its new name the snapshot will be
 // mapped twice. This will prevent it from being deleted.
 func (d *ceph) rbdRenameVolumeSnapshot(vol Volume, oldSnapshotName string, newSnapshotName string) error {
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -491,7 +505,8 @@ func (d *ceph) rbdRenameVolumeSnapshot(vol Volume, oldSnapshotName string, newSn
 //     The caller will usually want to parse this according to its needs. This
 //     helper library provides two small functions to do this but see below.
 func (d *ceph) rbdGetVolumeParent(vol Volume) (string, error) {
-	msg, err := shared.RunCommand(
+	msg, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -525,7 +540,8 @@ func (d *ceph) rbdGetVolumeParent(vol Volume) (string, error) {
 // This requires that the snapshot does not have any clones and is unmapped and
 // unprotected.
 func (d *ceph) rbdDeleteVolumeSnapshot(vol Volume, snapshotName string) error {
-	_, err := shared.RunCommand(
+	_, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -547,7 +563,8 @@ func (d *ceph) rbdDeleteVolumeSnapshot(vol Volume, snapshotName string) error {
 // this will only return
 // <rbd-snapshot-name>.
 func (d *ceph) rbdListVolumeSnapshots(vol Volume) ([]string, error) {
-	msg, err := shared.RunCommand(
+	msg, err := shared.RunCommandContext(
+		context.TODO(),
 		"rbd",
 		"--id", d.config["ceph.user.name"],
 		"--cluster", d.config["ceph.cluster_name"],
@@ -589,6 +606,31 @@ func (d *ceph) rbdListVolumeSnapshots(vol Volume) ([]string, error) {
 	return snapshots, nil
 }
 
+// getOSDPoolDefaultSize gets the global OSD default pool size that is used for
+// all pools created without an explicit OSD pool size.
+func (d *ceph) getOSDPoolDefaultSize() (int, error) {
+	defaultSize, err := shared.TryRunCommand("ceph",
+		"--name", "client."+d.config["ceph.user.name"],
+		"--cluster", d.config["ceph.cluster_name"],
+		"config",
+		"get",
+		"mon",
+		"osd_pool_default_size",
+		"--format",
+		"json")
+	if err != nil {
+		return -1, err
+	}
+
+	var defaultSizeInt int
+	err = json.Unmarshal([]byte(defaultSize), &defaultSizeInt)
+	if err != nil {
+		return -1, err
+	}
+
+	return defaultSizeInt, nil
+}
+
 // copyVolumeDiff creates a sparse copy of a volume by exporting and importing the diff
 // between `sourceVolumeName` and its optional `sourceParentSnapshot` onto `targetVolumeName`.
 // This does not introduce a dependency relation between the source RBD storage
@@ -950,16 +992,16 @@ func (d *ceph) parseParent(parent string) (Volume, string, error) {
 // <osd-pool-name>/<lxd-specific-prefix>_<rbd-storage-volume>
 // will be split into
 // <osd-pool-name>, <lxd-specific-prefix>, <rbd-storage-volume>.
-func (d *ceph) parseClone(clone string) (string, string, string, error) {
+func (d *ceph) parseClone(clone string) (poolName string, volumeType string, volumeName string, err error) {
 	idx := strings.Index(clone, "/")
 	if idx == -1 {
 		return "", "", "", fmt.Errorf("Unexpected parsing error")
 	}
 
 	slider := clone[(idx + 1):]
-	poolName := clone[:idx]
+	poolName = clone[:idx]
 
-	volumeType := slider
+	volumeType = slider
 	idx = strings.Index(slider, "zombie_")
 	if idx == 0 {
 		idx += len("zombie_")
@@ -983,7 +1025,7 @@ func (d *ceph) parseClone(clone string) (string, string, string, error) {
 		return "", "", "", fmt.Errorf("Unexpected parsing error")
 	}
 
-	volumeName := slider
+	volumeName = slider
 	idx = strings.Index(volumeName, "_")
 	if idx == -1 {
 		return "", "", "", fmt.Errorf("Unexpected parsing error")
@@ -1106,15 +1148,22 @@ func (d *ceph) generateUUID(fsType string, devPath string) error {
 }
 
 func (d *ceph) getRBDVolumeName(vol Volume, snapName string, zombie bool, withPoolName bool) string {
-	out := CephGetRBDImageName(vol, snapName, zombie)
+	imageName, volSnapName := CephGetRBDImageName(vol, zombie)
+
+	// see `man 8 rbd` for "snap-spec"
+	if snapName != "" {
+		imageName += "@" + snapName
+	} else if volSnapName != "" {
+		imageName += "@" + volSnapName
+	}
 
 	// If needed, the output will be prefixed with the pool name, e.g.
 	// <pool>/<type>_<volname>@<snapname>.
 	if withPoolName {
-		out = fmt.Sprintf("%s/%s", d.config["ceph.osd.pool_name"], out)
+		imageName = d.config["ceph.osd.pool_name"] + "/" + imageName
 	}
 
-	return out
+	return imageName
 }
 
 // Let's say we want to send the a container "a" including snapshots "snap0" and
