@@ -4,12 +4,10 @@ import (
 	"bufio"
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"io"
 	"os"
 	"os/exec"
-	"slices"
 	"strings"
 	"time"
 
@@ -98,7 +96,7 @@ func (d *ceph) CreateVolume(vol Volume, filler *VolumeFiller, op *operations.Ope
 			// be restored in the future and a new cached image volume will be created instead.
 			if volSizeBytes != poolVolSizeBytes {
 				d.logger.Debug("Renaming deleted cached image volume so that regeneration is used", logger.Ctx{"fingerprint": vol.Name()})
-				randomVol := NewVolume(d, d.name, deletedVol.volType, deletedVol.contentType, strings.ReplaceAll(uuid.New().String(), "-", ""), deletedVol.config, deletedVol.poolConfig)
+				randomVol := NewVolume(d, d.name, deletedVol.volType, deletedVol.contentType, strings.Replace(uuid.New().String(), "-", "", -1), deletedVol.config, deletedVol.poolConfig)
 				err = renameVolume(d.getRBDVolumeName(deletedVol, "", false, true), d.getRBDVolumeName(randomVol, "", false, true))
 				if err != nil {
 					return err
@@ -319,7 +317,7 @@ func (d *ceph) getVolumeSize(volumeName string) (int64, error) {
 
 // CreateVolumeFromBackup re-creates a volume from its exported state.
 func (d *ceph) CreateVolumeFromBackup(vol VolumeCopy, srcBackup backup.Info, srcData io.ReadSeeker, op *operations.Operation) (VolumePostHook, revert.Hook, error) {
-	return genericVFSBackupUnpack(d, d.state, vol, srcBackup.Snapshots, srcData, op)
+	return genericVFSBackupUnpack(d, d.state.OS, vol, srcBackup.Snapshots, srcData, op)
 }
 
 // CreateVolumeFromCopy provides same-pool volume copying functionality.
@@ -419,11 +417,11 @@ func (d *ceph) CreateVolumeFromCopy(vol VolumeCopy, srcVol VolumeCopy, allowInco
 			snapshotName := "readonly"
 
 			if srcVol.volType != VolumeTypeImage {
-				snapshotName = "zombie_snapshot_" + uuid.New().String()
+				snapshotName = fmt.Sprintf("zombie_snapshot_%s", uuid.New().String())
 
 				if srcVol.IsSnapshot() {
 					srcParentName, srcSnapOnlyName, _ := api.GetParentAndSnapshotName(srcVol.name)
-					snapshotName = "snapshot_" + srcSnapOnlyName
+					snapshotName = fmt.Sprintf("snapshot_%s", srcSnapOnlyName)
 					parentVol = NewVolumeCopy(NewVolume(d, d.name, srcVol.volType, srcVol.contentType, srcParentName, nil, nil))
 				} else {
 					// Create snapshot.
@@ -484,10 +482,10 @@ func (d *ceph) CreateVolumeFromCopy(vol VolumeCopy, srcVol VolumeCopy, allowInco
 	for i, snap := range snapshots {
 		prev := ""
 		if i > 0 {
-			prev = "snapshot_" + snapshots[i-1]
+			prev = fmt.Sprintf("snapshot_%s", snapshots[i-1])
 		}
 
-		lastSnap = "snapshot_" + snap
+		lastSnap = fmt.Sprintf("snapshot_%s", snap)
 		sourceVolumeName := d.getRBDVolumeName(srcVol.Volume, lastSnap, false, true)
 		err = d.copyVolumeDiff(sourceVolumeName, targetVolumeName, prev)
 		if err != nil {
@@ -555,7 +553,7 @@ func (d *ceph) createVolumeFromMigration(vol VolumeCopy, conn io.ReadWriteCloser
 
 			// Delete all of the snapshots after the last common snapshot.
 			if lastCommonSnapshotFound {
-				ok, err := d.hasVolume(d.getRBDVolumeName(vol.Volume, "snapshot_"+targetSnapshotName, false, false))
+				ok, err := d.hasVolume(d.getRBDVolumeName(vol.Volume, fmt.Sprintf("snapshot_%s", targetSnapshotName), false, false))
 				if err != nil {
 					return nil, err
 				}
@@ -567,7 +565,7 @@ func (d *ceph) createVolumeFromMigration(vol VolumeCopy, conn io.ReadWriteCloser
 
 				// Delete the snapshot if its order is out of sync.
 				// This happens if not the latest snapshot on the target side gets deleted and requires refresh.
-				_, err = d.deleteVolumeSnapshot(vol.Volume, "snapshot_"+targetSnapshotName)
+				_, err = d.deleteVolumeSnapshot(vol.Volume, fmt.Sprintf("snapshot_%s", targetSnapshotName))
 				if err != nil {
 					return nil, err
 				}
@@ -603,7 +601,7 @@ func (d *ceph) createVolumeFromMigration(vol VolumeCopy, conn io.ReadWriteCloser
 		}
 	}
 
-	err := vol.EnsureMountPath()
+	err := vol.Volume.EnsureMountPath()
 	if err != nil {
 		return nil, err
 	}
@@ -651,7 +649,7 @@ func (d *ceph) createVolumeFromMigration(vol VolumeCopy, conn io.ReadWriteCloser
 
 			// Ensure to cleanup the snapshot on the target volume in case of error.
 			// When retrying the migration there shouldn't be any left over snapshot from before.
-			revert.Add(func() { _, _ = d.deleteVolumeSnapshot(vol.Volume, "snapshot_"+targetSnapshotName) })
+			revert.Add(func() { _, _ = d.deleteVolumeSnapshot(vol.Volume, fmt.Sprintf("snapshot_%s", targetSnapshotName)) })
 		}
 	}
 
@@ -704,14 +702,14 @@ func (d *ceph) CreateVolumeFromMigration(vol VolumeCopy, conn io.ReadWriteCloser
 	}
 
 	// Handle simple RSYNC and BLOCK_AND_RSYNC through the generic function.
-	if slices.Contains([]migration.MigrationFSType{migration.MigrationFSType_RSYNC, migration.MigrationFSType_BLOCK_AND_RSYNC}, volTargetArgs.MigrationType.FSType) || volTargetArgs.MigrationType.FSType == migration.MigrationFSType_RBD_AND_RSYNC && vol.contentType == ContentTypeFS {
+	if shared.ValueInSlice(volTargetArgs.MigrationType.FSType, []migration.MigrationFSType{migration.MigrationFSType_RSYNC, migration.MigrationFSType_BLOCK_AND_RSYNC}) || volTargetArgs.MigrationType.FSType == migration.MigrationFSType_RBD_AND_RSYNC && vol.contentType == ContentTypeFS {
 		_, err := genericVFSCreateVolumeFromMigration(d, nil, vol, conn, volTargetArgs, preFiller, op)
 		if err != nil {
 			return err
 		}
 
 		return nil
-	} else if !slices.Contains([]migration.MigrationFSType{migration.MigrationFSType_RBD, migration.MigrationFSType_RBD_AND_RSYNC}, volTargetArgs.MigrationType.FSType) {
+	} else if !shared.ValueInSlice(volTargetArgs.MigrationType.FSType, []migration.MigrationFSType{migration.MigrationFSType_RBD, migration.MigrationFSType_RBD_AND_RSYNC}) {
 		return ErrNotSupported
 	}
 
@@ -781,7 +779,7 @@ func (d *ceph) refreshVolume(vol VolumeCopy, srcVol VolumeCopy, refreshSnapshots
 
 			// Delete all of the snapshots after the last common snapshot.
 			if lastCommonSnapshotFound {
-				ok, err := d.hasVolume(d.getRBDVolumeName(vol.Volume, "snapshot_"+targetSnapshotName, false, false))
+				ok, err := d.hasVolume(d.getRBDVolumeName(vol.Volume, fmt.Sprintf("snapshot_%s", targetSnapshotName), false, false))
 				if err != nil {
 					return nil, err
 				}
@@ -795,7 +793,7 @@ func (d *ceph) refreshVolume(vol VolumeCopy, srcVol VolumeCopy, refreshSnapshots
 				// This happens if not the latest snapshot on the target side gets deleted and requires refresh.
 				// The VMs filesystem volume snapshot will not be deleted.
 				// It already got refreshed using the generic approach.
-				_, err = d.deleteVolumeSnapshot(vol.Volume, "snapshot_"+targetSnapshotName)
+				_, err = d.deleteVolumeSnapshot(vol.Volume, fmt.Sprintf("snapshot_%s", targetSnapshotName))
 				if err != nil {
 					return nil, err
 				}
@@ -850,7 +848,7 @@ func (d *ceph) refreshVolume(vol VolumeCopy, srcVol VolumeCopy, refreshSnapshots
 		// If sourceVol is a snapshot append the prefix to the snapshots name to match the name of the actual snapshot.
 		_, sourceSnapName, _ := api.GetParentAndSnapshotName(sourceVol.Name())
 		if sourceSnapName != "" {
-			sourceSnapName = "snapshot_" + sourceSnapName
+			sourceSnapName = fmt.Sprintf("snapshot_%s", sourceSnapName)
 		}
 
 		fullSourceSnapName := d.getRBDVolumeName(sourceVol, sourceSnapName, false, true)
@@ -892,7 +890,7 @@ func (d *ceph) refreshVolume(vol VolumeCopy, srcVol VolumeCopy, refreshSnapshots
 			// This also happens if the snapshot doesn't yet exist on the target.
 			if i > 0 {
 				_, sourceParentSnapshotName, _ = api.GetParentAndSnapshotName(vol.Snapshots[i-1].name)
-				sourceParentSnapshotName = "snapshot_" + sourceParentSnapshotName
+				sourceParentSnapshotName = fmt.Sprintf("snapshot_%s", sourceParentSnapshotName)
 			}
 
 			lastSnap = sourceParentSnapshotName
@@ -904,7 +902,7 @@ func (d *ceph) refreshVolume(vol VolumeCopy, srcVol VolumeCopy, refreshSnapshots
 
 			// Ensure to cleanup the snapshot on the target volume in case of error.
 			// When retrying the refresh there shouldn't be any left over snapshot from before.
-			revert.Add(func() { _, _ = d.deleteVolumeSnapshot(vol.Volume, "snapshot_"+sourceSnapshotName) })
+			revert.Add(func() { _, _ = d.deleteVolumeSnapshot(vol.Volume, fmt.Sprintf("snapshot_%s", sourceSnapshotName)) })
 		}
 	}
 
@@ -915,7 +913,7 @@ func (d *ceph) refreshVolume(vol VolumeCopy, srcVol VolumeCopy, refreshSnapshots
 	// After refreshing the snapshots, the last common snapshot has now been changed to the latest one present on the target.
 	if len(vol.Snapshots) > 0 {
 		_, lastCommonSnapshotName, _ := api.GetParentAndSnapshotName(vol.Snapshots[len(vol.Snapshots)-1].name)
-		lastSnap = "snapshot_" + lastCommonSnapshotName
+		lastSnap = fmt.Sprintf("snapshot_%s", lastCommonSnapshotName)
 	}
 
 	// Apply the diff on the target volume.
@@ -1237,7 +1235,7 @@ func (d *ceph) GetVolumeUsage(vol Volume) (int64, error) {
 	// Running rbd du can be resource intensive, so users may want to miss disk usage
 	// data for stopped instances instead of dealing with the performance hit
 	if shared.IsFalse(d.config["ceph.rbd.du"]) {
-		return -1, errors.New("Cannot get disk usage of unmounted volume when ceph.rbd.du is false")
+		return -1, fmt.Errorf("Cannot get disk usage of unmounted volume when ceph.rbd.du is false")
 	}
 
 	// If not mounted (or not mountable), query the usage from ceph directly.
@@ -1282,7 +1280,7 @@ func (d *ceph) GetVolumeUsage(vol Volume) (int64, error) {
 	}
 
 	_, snapName, _ := api.GetParentAndSnapshotName(vol.Name())
-	snapName = "snapshot_" + snapName
+	snapName = fmt.Sprintf("snapshot_%s", snapName)
 
 	// rbd du gives the output of all related rbd images, snapshots included.
 	for _, image := range result.Images {
@@ -1463,7 +1461,7 @@ func (d *ceph) ListVolumes() ([]Volume, error) {
 				continue // Unknown volume type.
 			}
 
-			prefix = prefix + "_"
+			prefix = fmt.Sprintf("%s_", prefix)
 
 			if strings.HasPrefix(rawName, prefix) {
 				volType = volumeType
@@ -1520,7 +1518,7 @@ func (d *ceph) ListVolumes() ([]Volume, error) {
 		return nil, fmt.Errorf("Failed getting volume list: %v: %w", strings.TrimSpace(string(errMsg)), err)
 	}
 
-	volList := make([]Volume, 0, len(vols))
+	volList := make([]Volume, len(vols))
 	for _, v := range vols {
 		volList = append(volList, v)
 	}
@@ -1550,8 +1548,7 @@ func (d *ceph) MountVolume(vol Volume, op *operations.Operation) error {
 		revert.Add(func() { _ = d.rbdUnmapVolume(vol, true) })
 	}
 
-	switch vol.contentType {
-	case ContentTypeFS:
+	if vol.contentType == ContentTypeFS {
 		mountPath := vol.MountPath()
 		if !filesystem.IsMountPoint(mountPath) {
 			err := vol.EnsureMountPath()
@@ -1569,15 +1566,14 @@ func (d *ceph) MountVolume(vol Volume, op *operations.Operation) error {
 			}
 
 			mountFlags, mountOptions := filesystem.ResolveMountOptions(strings.Split(vol.ConfigBlockMountOptions(), ","))
-			err = TryMount(context.TODO(), volDevPath, mountPath, fsType, mountFlags, mountOptions)
+			err = TryMount(volDevPath, mountPath, fsType, mountFlags, mountOptions)
 			if err != nil {
 				return err
 			}
 
 			d.logger.Debug("Mounted RBD volume", logger.Ctx{"volName": vol.name, "dev": volDevPath, "path": mountPath, "options": mountOptions})
 		}
-
-	case ContentTypeBlock:
+	} else if vol.contentType == ContentTypeBlock {
 		// For VMs, mount the filesystem volume.
 		if vol.IsVMBlock() {
 			fsVol := vol.NewVMBlockFilesystemVolume()
@@ -1707,7 +1703,7 @@ func (d *ceph) MigrateVolume(vol VolumeCopy, conn io.ReadWriteCloser, volSrcArgs
 	}
 
 	// Handle simple rsync and block_and_rsync through generic.
-	if slices.Contains([]migration.MigrationFSType{migration.MigrationFSType_RSYNC, migration.MigrationFSType_BLOCK_AND_RSYNC}, volSrcArgs.MigrationType.FSType) || volSrcArgs.MigrationType.FSType == migration.MigrationFSType_RBD_AND_RSYNC && vol.contentType == ContentTypeFS {
+	if shared.ValueInSlice(volSrcArgs.MigrationType.FSType, []migration.MigrationFSType{migration.MigrationFSType_RSYNC, migration.MigrationFSType_BLOCK_AND_RSYNC}) || volSrcArgs.MigrationType.FSType == migration.MigrationFSType_RBD_AND_RSYNC && vol.contentType == ContentTypeFS {
 		// TODO this should take a temporary snapshot.
 		// Before doing a generic volume migration, we need to ensure volume (or snap volume parent) is
 		// activated to avoid issues activating the snapshot volume device.
@@ -1721,14 +1717,14 @@ func (d *ceph) MigrateVolume(vol VolumeCopy, conn io.ReadWriteCloser, volSrcArgs
 		defer func() { _, _ = d.UnmountVolume(parentVol, false, op) }()
 
 		return genericVFSMigrateVolume(d, d.state, vol, conn, volSrcArgs, op)
-	} else if !slices.Contains([]migration.MigrationFSType{migration.MigrationFSType_RBD, migration.MigrationFSType_RBD_AND_RSYNC}, volSrcArgs.MigrationType.FSType) {
+	} else if !shared.ValueInSlice(volSrcArgs.MigrationType.FSType, []migration.MigrationFSType{migration.MigrationFSType_RBD, migration.MigrationFSType_RBD_AND_RSYNC}) {
 		return ErrNotSupported
 	}
 
 	// Handle rbd export-diff/import-diff migration.
 	if volSrcArgs.MultiSync || volSrcArgs.FinalSync {
 		// This is not needed if the migration is performed using rbd export-diff/import-diff.
-		return errors.New("MultiSync should not be used with optimized migration")
+		return fmt.Errorf("MultiSync should not be used with optimized migration")
 	}
 
 	// Migrate (send) the VMs filesystem volume too.
@@ -1757,9 +1753,9 @@ func (d *ceph) MigrateVolume(vol VolumeCopy, conn io.ReadWriteCloser, volSrcArgs
 		defer unlock()
 
 		parentName, snapOnlyName, _ := api.GetParentAndSnapshotName(vol.name)
-		snapOnlyName = "snapshot_" + snapOnlyName
+		snapOnlyName = fmt.Sprintf("snapshot_%s", snapOnlyName)
 		parentVol := NewVolume(d, vol.pool, vol.volType, vol.contentType, parentName, nil, nil)
-		cloneVol := NewVolume(d, vol.pool, vol.volType, vol.contentType, parentName+"_clone", nil, nil)
+		cloneVol := NewVolume(d, vol.pool, vol.volType, vol.contentType, fmt.Sprintf("%s_clone", parentName), nil, nil)
 
 		// Ensure the snapshot is protected so that it allows creating a clone from it.
 		err = d.rbdProtectVolumeSnapshot(parentVol, snapOnlyName)
@@ -1823,7 +1819,7 @@ func (d *ceph) MigrateVolume(vol VolumeCopy, conn io.ReadWriteCloser, volSrcArgs
 			// This also happens if the snapshot doesn't yet exist on the target.
 			if i > 0 {
 				_, sourceParentSnapshotName, _ = api.GetParentAndSnapshotName(vol.Snapshots[i-1].name)
-				sourceParentSnapshotName = "snapshot_" + sourceParentSnapshotName
+				sourceParentSnapshotName = fmt.Sprintf("snapshot_%s", sourceParentSnapshotName)
 			}
 
 			lastSnap = sourceParentSnapshotName
@@ -1835,7 +1831,7 @@ func (d *ceph) MigrateVolume(vol VolumeCopy, conn io.ReadWriteCloser, volSrcArgs
 				wrapper = migration.ProgressTracker(op, "fs_progress", targetSnapshot.name)
 			}
 
-			sendSnapName := d.getRBDVolumeName(vol.Volume, "snapshot_"+targetSnapshotName, false, true)
+			sendSnapName := d.getRBDVolumeName(vol.Volume, fmt.Sprintf("snapshot_%s", targetSnapshotName), false, true)
 
 			err := d.sendVolume(conn, sendSnapName, lastSnap, wrapper)
 			if err != nil {
@@ -1848,7 +1844,7 @@ func (d *ceph) MigrateVolume(vol VolumeCopy, conn io.ReadWriteCloser, volSrcArgs
 	// Don't try to create a diff from the last snapshot in case only the volume gets migrated.
 	if len(vol.Snapshots) > 0 && !volSrcArgs.VolumeOnly {
 		_, lastCommonSnapshotName, _ := api.GetParentAndSnapshotName(vol.Snapshots[len(vol.Snapshots)-1].name)
-		lastSnap = "snapshot_" + lastCommonSnapshotName
+		lastSnap = fmt.Sprintf("snapshot_%s", lastCommonSnapshotName)
 	}
 
 	// Setup progress tracking.
@@ -1857,7 +1853,7 @@ func (d *ceph) MigrateVolume(vol VolumeCopy, conn io.ReadWriteCloser, volSrcArgs
 		wrapper = migration.ProgressTracker(op, "fs_progress", vol.name)
 	}
 
-	runningSnapName := "migration-send-" + uuid.New().String()
+	runningSnapName := fmt.Sprintf("migration-send-%s", uuid.New().String())
 
 	err := d.rbdCreateVolumeSnapshot(vol.Volume, runningSnapName)
 	if err != nil {
@@ -1882,7 +1878,7 @@ func (d *ceph) CreateVolumeSnapshot(snapVol Volume, op *operations.Operation) er
 
 	parentName, snapshotOnlyName, _ := api.GetParentAndSnapshotName(snapVol.name)
 	sourcePath := GetVolumeMountPath(d.name, snapVol.volType, parentName)
-	snapshotName := "snapshot_" + snapshotOnlyName
+	snapshotName := fmt.Sprintf("snapshot_%s", snapshotOnlyName)
 
 	if filesystem.IsMountPoint(sourcePath) {
 		// Attempt to sync and freeze filesystem, but do not error if not able to freeze (as filesystem
@@ -1946,7 +1942,7 @@ func (d *ceph) DeleteVolumeSnapshot(snapVol Volume, op *operations.Operation) er
 	}
 
 	parentName, snapshotOnlyName, _ := api.GetParentAndSnapshotName(snapVol.name)
-	snapshotName := "snapshot_" + snapshotOnlyName
+	snapshotName := fmt.Sprintf("snapshot_%s", snapshotOnlyName)
 
 	parentVol := NewVolume(d, d.name, snapVol.volType, snapVol.contentType, parentName, nil, nil)
 
@@ -2008,7 +2004,7 @@ func (d *ceph) MountVolumeSnapshot(snapVol Volume, op *operations.Operation) err
 		}
 
 		parentName, snapshotOnlyName, _ := api.GetParentAndSnapshotName(snapVol.name)
-		prefixedSnapOnlyName := "snapshot_" + snapshotOnlyName
+		prefixedSnapOnlyName := fmt.Sprintf("snapshot_%s", snapshotOnlyName)
 
 		parentVol := NewVolume(d, d.name, snapVol.volType, snapVol.contentType, parentName, nil, nil)
 
@@ -2041,11 +2037,8 @@ func (d *ceph) MountVolumeSnapshot(snapVol Volume, op *operations.Operation) err
 
 		RBDFilesystem := snapVol.ConfigBlockFilesystem()
 		mountFlags, mountOptions := filesystem.ResolveMountOptions(strings.Split(snapVol.ConfigBlockMountOptions(), ","))
-		mountOptions = addNoRecoveryMountOption(mountOptions, RBDFilesystem)
 
 		if renegerateFilesystemUUIDNeeded(RBDFilesystem) {
-			// When mounting XFS filesystems temporarily we can use the nouuid option rather than fully
-			// regenerating the filesystem UUID.
 			if RBDFilesystem == "xfs" {
 				idx := strings.Index(mountOptions, "nouuid")
 				if idx < 0 {
@@ -2059,7 +2052,7 @@ func (d *ceph) MountVolumeSnapshot(snapVol Volume, op *operations.Operation) err
 			}
 		}
 
-		err = TryMount(context.TODO(), rbdDevPath, mountPath, RBDFilesystem, mountFlags, mountOptions)
+		err = TryMount(rbdDevPath, mountPath, RBDFilesystem, mountFlags, mountOptions)
 		if err != nil {
 			return err
 		}
@@ -2215,7 +2208,7 @@ func (d *ceph) restoreVolume(vol Volume, snapVol Volume, op *operations.Operatio
 		"--pool", d.config["ceph.osd.pool_name"],
 		"snap",
 		"rollback",
-		"--snap", "snapshot_"+snapshotName,
+		"--snap", fmt.Sprintf("snapshot_%s", snapshotName),
 		d.getRBDVolumeName(vol, "", false, false))
 	if err != nil {
 		return err
@@ -2267,8 +2260,8 @@ func (d *ceph) RenameVolumeSnapshot(snapVol Volume, newSnapshotName string, op *
 	defer revert.Fail()
 
 	parentName, snapshotOnlyName, _ := api.GetParentAndSnapshotName(snapVol.name)
-	oldSnapOnlyName := "snapshot_" + snapshotOnlyName
-	newSnapOnlyName := "snapshot_" + newSnapshotName
+	oldSnapOnlyName := fmt.Sprintf("snapshot_%s", snapshotOnlyName)
+	newSnapOnlyName := fmt.Sprintf("snapshot_%s", newSnapshotName)
 
 	parentVol := NewVolume(d, d.name, snapVol.volType, snapVol.contentType, parentName, nil, nil)
 
