@@ -1,7 +1,6 @@
 package drivers
 
 import (
-	"context"
 	"errors"
 	"fmt"
 	"os"
@@ -46,7 +45,7 @@ func (d *cephfs) load() error {
 	for _, tool := range []string{"ceph", "rbd"} {
 		_, err := exec.LookPath(tool)
 		if err != nil {
-			return fmt.Errorf("Required tool %q is missing", tool)
+			return fmt.Errorf("Required tool '%s' is missing", tool)
 		}
 	}
 
@@ -131,11 +130,11 @@ func (d *cephfs) Create() error {
 
 	// Config validation.
 	if d.config["source"] == "" {
-		return errors.New("Missing required source name/path")
+		return fmt.Errorf("Missing required source name/path")
 	}
 
 	if d.config["cephfs.path"] != "" && d.config["cephfs.path"] != d.config["source"] {
-		return errors.New("cephfs.path must match the source")
+		return fmt.Errorf("cephfs.path must match the source")
 	}
 
 	d.config["cephfs.path"] = d.config["source"]
@@ -156,7 +155,7 @@ func (d *cephfs) Create() error {
 
 	if fsExists {
 		for _, key := range []string{"create_missing", "osd_pg_num", "meta_pool", "data_pool"} {
-			cephfsSourceKey := "cephfs." + key
+			cephfsSourceKey := fmt.Sprintf("cephfs.%s", key)
 			if d.config[cephfsSourceKey] != "" {
 				return fmt.Errorf("Invalid config key %q: CephFS filesystem already exists", cephfsSourceKey)
 			}
@@ -302,7 +301,7 @@ func (d *cephfs) Create() error {
 
 	// Mount the pool.
 	srcPath := strings.Join(monAddresses, ",") + ":/"
-	err = TryMount(context.TODO(), srcPath, mountPoint, "ceph", 0, d.getMountOptions(d.config["cephfs.user.name"], userSecret, fsName))
+	err = TryMount(srcPath, mountPoint, "ceph", 0, fmt.Sprintf("name=%v,secret=%v,mds_namespace=%v", d.config["cephfs.user.name"], userSecret, fsName))
 	if err != nil {
 		return err
 	}
@@ -318,7 +317,7 @@ func (d *cephfs) Create() error {
 	// Check that the existing path is empty.
 	ok, _ := shared.PathIsEmpty(filepath.Join(mountPoint, fsPath))
 	if !ok {
-		return errors.New("Only empty CephFS paths can be used as a LXD storage pool")
+		return fmt.Errorf("Only empty CephFS paths can be used as a LXD storage pool")
 	}
 
 	revert.Success()
@@ -363,7 +362,7 @@ func (d *cephfs) Delete(op *operations.Operation) error {
 
 	// Mount the pool.
 	srcPath := strings.Join(monAddresses, ",") + ":/"
-	err = TryMount(context.TODO(), srcPath, mountPoint, "ceph", 0, d.getMountOptions(d.config["cephfs.user.name"], userSecret, fsName))
+	err = TryMount(srcPath, mountPoint, "ceph", 0, fmt.Sprintf("name=%v,secret=%v,mds_namespace=%v", d.config["cephfs.user.name"], userSecret, fsName))
 	if err != nil {
 		return err
 	}
@@ -546,14 +545,14 @@ func (d *cephfs) Mount() (bool, error) {
 	}
 
 	// Mount options.
-	options := d.getMountOptions(d.config["cephfs.user.name"], userSecret, fsName)
+	options := fmt.Sprintf("name=%s,secret=%s,mds_namespace=%s", d.config["cephfs.user.name"], userSecret, fsName)
 	if shared.IsTrue(d.config["cephfs.fscache"]) {
 		options += ",fsc"
 	}
 
 	// Mount the pool.
 	srcPath := strings.Join(monAddresses, ",") + ":/" + fsPath
-	err = TryMount(context.TODO(), srcPath, GetPoolMountPath(d.name), "ceph", 0, options)
+	err = TryMount(srcPath, GetPoolMountPath(d.name), "ceph", 0, options)
 	if err != nil {
 		return false, err
 	}
