@@ -2,7 +2,6 @@ package drivers
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"net/http"
 	"os/exec"
@@ -158,7 +157,7 @@ func (d *cephobject) FillConfig() error {
 	}
 
 	if d.config["cephobject.radosgw.endpoint"] == "" {
-		return errors.New(`"cephobject.radosgw.endpoint" option is required`)
+		return fmt.Errorf(`"cephobject.radosgw.endpoint" option is required`)
 	}
 
 	return nil
@@ -214,7 +213,7 @@ func (d *cephobject) Update(changedConfig map[string]string) error {
 
 		for _, bucketName := range buckets {
 			if strings.HasPrefix(bucketName, d.config["cephobject.bucket.name_prefix"]) {
-				return errors.New(`Cannot change "cephobject.bucket.name_prefix" when there are existing buckets`)
+				return fmt.Errorf(`Cannot change "cephobject.bucket.name_prefix" when there are existing buclets`)
 			}
 		}
 	}
