@@ -1,6 +1,7 @@
 package drivers
 
 import (
+	"context"
 	"fmt"
 	"io"
 	"net/url"
@@ -45,6 +46,17 @@ func (d *common) isRemote() bool {
 	return false
 }
 
+// defaultVMBlockFilesystemSize returns the default size for VM block filesystem
+// devices.
+func (d *common) defaultVMBlockFilesystemSize() string {
+	return defaultVMBlockFilesystemSize
+}
+
+// defaultBlockVolumeSize returns the default size for block volumes in this pool.
+func (d *common) defaultBlockVolumeSize() string {
+	return defaultBlockSize
+}
+
 // validatePool validates a pool config against common rules and optional driver specific rules.
 func (d *common) validatePool(config map[string]string, driverRules map[string]func(value string) error, volumeRules map[string]func(value string) error) error {
 	checkedFields := map[string]struct{}{}
@@ -124,6 +136,11 @@ func (d *common) fillVolumeConfig(vol *Volume, excludedKeys ...string) error {
 			continue
 		}
 
+		// security.shared is only relevant for custom block volumes.
+		if (vol.Type() != VolumeTypeCustom || vol.ContentType() != ContentTypeBlock) && (volKey == "security.shared") {
+			continue
+		}
+
 		if vol.config[volKey] == "" {
 			vol.config[volKey] = d.config[k]
 		}
@@ -275,7 +292,7 @@ func (d *common) moveGPTAltHeader(devPath string) error {
 		return nil
 	}
 
-	_, err = shared.RunCommand(path, "--move-second-header", devPath)
+	_, err = shared.RunCommandContext(context.TODO(), path, "--move-second-header", devPath)
 	if err == nil {
 		d.logger.Debug("Moved GPT alternative header to end of disk", logger.Ctx{"dev": devPath})
 		return nil
@@ -549,23 +566,12 @@ func (d *common) DeleteBucketKey(bucket Volume, keyName string, op *operations.O
 	return nil
 }
 
-// roundVolumeBlockSizeBytes returns size rounded to the nearest multiple of MinBlockBoundary bytes that is equal
-// to or larger than sizeBytes.
-func (d *common) roundVolumeBlockSizeBytes(sizeBytes int64) int64 {
+// roundVolumeBlockSizeBytes returns sizeBytes rounded up to the next multiple
+// of MinBlockBoundary.
+func (d *common) roundVolumeBlockSizeBytes(vol Volume, sizeBytes int64) int64 {
 	// QEMU requires image files to be in traditional storage block boundaries.
 	// We use 8k here to ensure our images are compatible with all of our backend drivers.
-	if sizeBytes < MinBlockBoundary {
-		sizeBytes = MinBlockBoundary
-	}
-
-	roundedSizeBytes := int64(sizeBytes/MinBlockBoundary) * MinBlockBoundary
-
-	// Ensure the rounded size is at least the size specified in sizeBytes.
-	if roundedSizeBytes < sizeBytes {
-		roundedSizeBytes += MinBlockBoundary
-	}
-
-	return roundedSizeBytes
+	return roundAbove(MinBlockBoundary, sizeBytes)
 }
 
 func (d *common) isBlockBacked(vol Volume) bool {
@@ -579,7 +585,7 @@ func (d *common) filesystemFreeze(path string) (func() error, error) {
 		return nil, fmt.Errorf("Failed syncing filesystem %q: %w", path, err)
 	}
 
-	_, err = shared.RunCommand("fsfreeze", "--freeze", path)
+	_, err = shared.RunCommandContext(context.TODO(), "fsfreeze", "--freeze", path)
 	if err != nil {
 		return nil, fmt.Errorf("Failed freezing filesystem %q: %w", path, err)
 	}
@@ -587,7 +593,7 @@ func (d *common) filesystemFreeze(path string) (func() error, error) {
 	d.logger.Info("Filesystem frozen", logger.Ctx{"path": path})
 
 	unfreezeFS := func() error {
-		_, err := shared.RunCommand("fsfreeze", "--unfreeze", path)
+		_, err := shared.RunCommandContext(context.TODO(), "fsfreeze", "--unfreeze", path)
 		if err != nil {
 			return fmt.Errorf("Failed unfreezing filesystem %q: %w", path, err)
 		}
