@@ -1,6 +1,7 @@
 package drivers
 
 import (
+	"context"
 	"fmt"
 	"net/http"
 	"os"
@@ -10,7 +11,6 @@ import (
 	"strings"
 	"time"
 
-	deviceConfig "github.com/canonical/lxd/lxd/device/config"
 	"github.com/canonical/lxd/lxd/operations"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
@@ -54,7 +54,7 @@ func (d *lvm) load() error {
 
 	// Detect and record the version.
 	if lvmVersion == "" {
-		output, err := shared.RunCommand("lvm", "version")
+		output, err := shared.RunCommandContext(d.state.ShutdownCtx, "lvm", "version")
 		if err != nil {
 			return fmt.Errorf("Error getting LVM version: %w", err)
 		}
@@ -87,7 +87,8 @@ func (d *lvm) Info() Info {
 	return Info{
 		Name:                         "lvm",
 		Version:                      lvmVersion,
-		DefaultVMBlockFilesystemSize: deviceConfig.DefaultVMBlockFilesystemSize,
+		DefaultBlockSize:             d.defaultBlockVolumeSize(),
+		DefaultVMBlockFilesystemSize: d.defaultVMBlockFilesystemSize(),
 		OptimizedImages:              d.usesThinpool(), // Only thinpool pools support optimized images.
 		PreservesInodes:              false,
 		Remote:                       d.isRemote(),
@@ -98,6 +99,7 @@ func (d *lvm) Info() Info {
 		IOUring:                      true,
 		MountedRoot:                  false,
 		Buckets:                      true,
+		PopulateParentVolumeUUID:     false,
 	}
 }
 
@@ -147,7 +149,7 @@ func (d *lvm) Create() error {
 				return err
 			}
 
-			d.config["size"] = fmt.Sprintf("%dGiB", defaultSize)
+			d.config["size"] = fmt.Sprint(defaultSize) + "GiB"
 		}
 
 		size, err := units.ParseByteSizeString(d.config["size"])
@@ -215,7 +217,7 @@ func (d *lvm) Create() error {
 
 		// Wipe if requested.
 		if shared.IsTrue(d.config["source.wipe"]) {
-			err := wipeBlockHeaders(d.config["source"])
+			err := wipeBlockHeaders(srcPath)
 			if err != nil {
 				return fmt.Errorf("Failed to wipe headers from disk %q: %w", d.config["source"], err)
 			}
@@ -503,6 +505,8 @@ func (d *lvm) Delete(op *operations.Operation) error {
 	return nil
 }
 
+// Validate checks that all provide keys are supported and that no conflicting
+// or missing configuration is present.
 func (d *lvm) Validate(config map[string]string) error {
 	rules := map[string]func(value string) error{
 		"size": validate.Optional(validate.IsSize),
@@ -512,6 +516,7 @@ func (d *lvm) Validate(config map[string]string) error {
 		//  type: string
 		//  defaultdesc: name of the pool
 		//  shortdesc: Name of the volume group to create
+		//  scope: local
 		"lvm.vg_name": validate.IsAny,
 		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=lvm.thinpool_name)
 		//
@@ -519,6 +524,7 @@ func (d *lvm) Validate(config map[string]string) error {
 		//  type: string
 		//  defaultdesc: `LXDThinPool`
 		//  shortdesc: Thin pool where volumes are created
+		//  scope: local
 		"lvm.thinpool_name": validate.IsAny,
 		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=lvm.thinpool_metadata_size)
 		// By default, LVM calculates an appropriate size.
@@ -526,6 +532,7 @@ func (d *lvm) Validate(config map[string]string) error {
 		//  type: string
 		//  defaultdesc: `0` (auto)
 		//  shortdesc: The size of the thin pool metadata volume
+		//  scope: global
 		"lvm.thinpool_metadata_size": validate.Optional(validate.IsSize),
 		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=lvm.use_thinpool)
 		//
@@ -533,6 +540,7 @@ func (d *lvm) Validate(config map[string]string) error {
 		//  type: bool
 		//  defaultdesc: `true`
 		//  shortdesc: Whether the storage pool uses a thin pool for logical volumes
+		//  scope: global
 		"lvm.use_thinpool": validate.Optional(validate.IsBool),
 		// lxdmeta:generate(entities=storage-lvm; group=pool-conf; key=lvm.vg.force_reuse)
 		//
@@ -540,6 +548,7 @@ func (d *lvm) Validate(config map[string]string) error {
 		//  type: bool
 		//  defaultdesc: `false`
 		//  shortdesc: Force using an existing non-empty volume group
+		//  scope: global
 		"lvm.vg.force_reuse": validate.Optional(validate.IsBool),
 	}
 
@@ -636,7 +645,7 @@ func (d *lvm) Update(changedConfig map[string]string) error {
 		}
 
 		// Resize physical volume so that lvresize is able to resize as well.
-		_, err = shared.RunCommand("pvresize", "-y", loopDevPath)
+		_, err = shared.RunCommandContext(context.TODO(), "pvresize", "-y", loopDevPath)
 		if err != nil {
 			return err
 		}
@@ -645,9 +654,9 @@ func (d *lvm) Update(changedConfig map[string]string) error {
 			lvPath := d.lvmDevPath(d.config["lvm.vg_name"], "", "", d.thinpoolName())
 
 			// Use the remaining space in the volume group.
-			_, err = shared.RunCommand("lvresize", "-f", "-l", "+100%FREE", lvPath)
+			_, err = shared.RunCommandContext(context.TODO(), "lvresize", "-f", "-l", "+100%FREE", lvPath)
 			if err != nil {
-				return err
+				return fmt.Errorf("Error resizing LV named %q: %w", lvPath, err)
 			}
 		}
 	}
@@ -754,7 +763,7 @@ func (d *lvm) GetResources() (*api.ResourcesStoragePool, error) {
 			"-o", "vg_size,vg_free",
 		}
 
-		out, err := shared.RunCommand("vgs", args...)
+		out, err := shared.RunCommandContext(d.state.ShutdownCtx, "vgs", args...)
 		if err != nil {
 			return nil, err
 		}
@@ -782,21 +791,10 @@ func (d *lvm) GetResources() (*api.ResourcesStoragePool, error) {
 	return &res, nil
 }
 
-// roundVolumeBlockSizeBytes returns size rounded to the nearest multiple of the volume group extent size that is
-// equal to or larger than sizeBytes.
-func (d *lvm) roundVolumeBlockSizeBytes(sizeBytes int64) int64 {
+// roundVolumeBlockSizeBytes returns sizeBytes rounded up to the next multiple
+// of the volume group extent size.
+func (d *lvm) roundVolumeBlockSizeBytes(vol Volume, sizeBytes int64) int64 {
 	// Get the volume group's physical extent size, and use that as minimum size.
 	vgExtentSize, _ := d.volumeGroupExtentSize(d.config["lvm.vg_name"])
-	if sizeBytes < vgExtentSize {
-		sizeBytes = vgExtentSize
-	}
-
-	roundedSizeBytes := int64(sizeBytes/vgExtentSize) * vgExtentSize
-
-	// Ensure the rounded size is at least the size specified in sizeBytes.
-	if roundedSizeBytes < sizeBytes {
-		roundedSizeBytes += vgExtentSize
-	}
-
-	return roundedSizeBytes
+	return roundAbove(vgExtentSize, sizeBytes)
 }
