@@ -11,20 +11,24 @@ import (
 	"strings"
 	"time"
 
-	"github.com/lxc/lxd/lxd/locking"
-	"github.com/lxc/lxd/lxd/operations"
-	"github.com/lxc/lxd/lxd/revert"
-	"github.com/lxc/lxd/lxd/storage/filesystem"
-	"github.com/lxc/lxd/shared"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
-	"github.com/lxc/lxd/shared/units"
-	"github.com/lxc/lxd/shared/version"
+	"github.com/canonical/lxd/lxd/locking"
+	"github.com/canonical/lxd/lxd/operations"
+	"github.com/canonical/lxd/lxd/storage/block"
+	"github.com/canonical/lxd/lxd/storage/filesystem"
+	"github.com/canonical/lxd/shared"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
+	"github.com/canonical/lxd/shared/revert"
+	"github.com/canonical/lxd/shared/units"
+	"github.com/canonical/lxd/shared/version"
 )
 
 // lvmBlockVolSuffix suffix used for block content type volumes.
 const lvmBlockVolSuffix = ".block"
 
+// lvmISOVolSuffix suffix used for iso content type volumes.
+const lvmISOVolSuffix = ".iso"
+
 // lvmSnapshotSeparator separator character used between volume name and snaphot name in logical volume names.
 const lvmSnapshotSeparator = "-"
 
@@ -56,7 +60,11 @@ func (d *lvm) openLoopFile(source string) (string, error) {
 	}
 
 	if filepath.IsAbs(source) && !shared.IsBlockdevPath(source) {
-		unlock := locking.Lock(context.TODO(), OperationLockName("openLoopFile", d.name, "", "", ""))
+		unlock, err := locking.Lock(context.TODO(), OperationLockName("openLoopFile", d.name, "", "", ""))
+		if err != nil {
+			return "", err
+		}
+
 		defer unlock()
 
 		loopDeviceName, err := loopDeviceSetup(source)
@@ -88,7 +96,7 @@ func (d *lvm) isLVMNotFoundExitError(err error) bool {
 
 // pysicalVolumeExists checks if an LVM Physical Volume exists.
 func (d *lvm) pysicalVolumeExists(pvName string) (bool, error) {
-	_, err := shared.RunCommand("pvs", "--noheadings", "-o", "pv_name", pvName)
+	_, err := shared.RunCommandContext(context.TODO(), "pvs", "--noheadings", "-o", "pv_name", pvName)
 	if err != nil {
 		if d.isLVMNotFoundExitError(err) {
 			return false, nil
@@ -102,7 +110,7 @@ func (d *lvm) pysicalVolumeExists(pvName string) (bool, error) {
 
 // volumeGroupExists checks if an LVM Volume Group exists and returns any tags on that volume group.
 func (d *lvm) volumeGroupExists(vgName string) (bool, []string, error) {
-	output, err := shared.RunCommand("vgs", "--noheadings", "-o", "vg_tags", vgName)
+	output, err := shared.RunCommandContext(context.TODO(), "vgs", "--noheadings", "-o", "vg_tags", vgName)
 	if err != nil {
 		if d.isLVMNotFoundExitError(err) {
 			return false, nil, nil
@@ -119,7 +127,7 @@ func (d *lvm) volumeGroupExists(vgName string) (bool, []string, error) {
 
 // volumeGroupExtentSize gets the volume group's physical extent size in bytes.
 func (d *lvm) volumeGroupExtentSize(vgName string) (int64, error) {
-	output, err := shared.RunCommand("vgs", "--noheadings", "--nosuffix", "--units", "b", "-o", "vg_extent_size", vgName)
+	output, err := shared.RunCommandContext(context.TODO(), "vgs", "--noheadings", "--nosuffix", "--units", "b", "-o", "vg_extent_size", vgName)
 	if err != nil {
 		if d.isLVMNotFoundExitError(err) {
 			return -1, api.StatusErrorf(http.StatusNotFound, "LVM volume group not found")
@@ -134,7 +142,7 @@ func (d *lvm) volumeGroupExtentSize(vgName string) (int64, error) {
 
 // countLogicalVolumes gets the count of volumes (both normal and thin) in a volume group.
 func (d *lvm) countLogicalVolumes(vgName string) (int, error) {
-	output, err := shared.RunCommand("vgs", "--noheadings", "-o", "lv_count", vgName)
+	output, err := shared.RunCommandContext(context.TODO(), "vgs", "--noheadings", "-o", "lv_count", vgName)
 	if err != nil {
 		if d.isLVMNotFoundExitError(err) {
 			return -1, api.StatusErrorf(http.StatusNotFound, "LVM volume group not found")
@@ -149,7 +157,7 @@ func (d *lvm) countLogicalVolumes(vgName string) (int, error) {
 
 // countThinVolumes gets the count of thin volumes in a thin pool.
 func (d *lvm) countThinVolumes(vgName, poolName string) (int, error) {
-	output, err := shared.RunCommand("lvs", "--noheadings", "-o", "thin_count", fmt.Sprintf("%s/%s", vgName, poolName))
+	output, err := shared.RunCommandContext(context.TODO(), "lvs", "--noheadings", "-o", "thin_count", vgName+"/"+poolName)
 	if err != nil {
 		if d.isLVMNotFoundExitError(err) {
 			return -1, api.StatusErrorf(http.StatusNotFound, "LVM volume group not found")
@@ -164,7 +172,7 @@ func (d *lvm) countThinVolumes(vgName, poolName string) (int, error) {
 
 // thinpoolExists checks whether the specified thinpool exists in a volume group.
 func (d *lvm) thinpoolExists(vgName string, poolName string) (bool, error) {
-	output, err := shared.RunCommand("lvs", "--noheadings", "-o", "lv_attr", fmt.Sprintf("%s/%s", vgName, poolName))
+	output, err := shared.RunCommandContext(context.TODO(), "lvs", "--noheadings", "-o", "lv_attr", vgName+"/"+poolName)
 	if err != nil {
 		if d.isLVMNotFoundExitError(err) {
 			return false, nil
@@ -184,7 +192,7 @@ func (d *lvm) thinpoolExists(vgName string, poolName string) (bool, error) {
 
 // logicalVolumeExists checks whether the specified logical volume exists.
 func (d *lvm) logicalVolumeExists(volDevPath string) (bool, error) {
-	_, err := shared.RunCommand("lvs", "--noheadings", "-o", "lv_name", volDevPath)
+	_, err := shared.RunCommandContext(context.TODO(), "lvs", "--noheadings", "-o", "lv_name", volDevPath)
 	if err != nil {
 		if d.isLVMNotFoundExitError(err) {
 			return false, nil
@@ -196,16 +204,18 @@ func (d *lvm) logicalVolumeExists(volDevPath string) (bool, error) {
 	return true, nil
 }
 
-// createDefaultThinPool creates the default thinpool as 100% the free size of the volume group.
-// Accepts thinpoolMetadataSize argument, which if >0 will manually set metadata size for the thinpool, otherwise
-// LVM will pick an appropriate size.
-func (d *lvm) createDefaultThinPool(lvmVersion, vgName, thinPoolName string, thinpoolMetadataSize string) error {
+// createDefaultThinPool creates the default thinpool in the pool's volume group.
+// If thinpoolSizeBytes >0 will manually set the thinpool volume size. Otherwise it will use 100% of the free space
+// in the volume group.
+// If pool lvm.thinpool_metadata_size setting >0 will manually set metadata size for the thinpool, otherwise LVM
+// will pick an appropriate size.
+func (d *lvm) createDefaultThinPool(lvmVersion, thinPoolName string, thinpoolSizeBytes int64) error {
 	isRecent, err := d.lvmVersionIsAtLeast(lvmVersion, "2.02.99")
 	if err != nil {
 		return fmt.Errorf("Error checking LVM version: %w", err)
 	}
 
-	lvmThinPool := fmt.Sprintf("%s/%s", vgName, thinPoolName)
+	lvmThinPool := d.config["lvm.vg_name"] + "/" + thinPoolName
 
 	args := []string{
 		"--yes",
@@ -213,16 +223,18 @@ func (d *lvm) createDefaultThinPool(lvmVersion, vgName, thinPoolName string, thi
 		"--thinpool", lvmThinPool,
 	}
 
-	thinpoolMetadataSizeBytes, err := d.roundedSizeBytesString(thinpoolMetadataSize)
+	thinpoolMetadataSizeBytes, err := d.roundedSizeBytesString(d.config["lvm.thinpool_metadata_size"])
 	if err != nil {
-		return fmt.Errorf("Invalid thinpool metadata size %q: %w", thinpoolMetadataSize, err)
+		return fmt.Errorf("Invalid lvm.thinpool_metadata_size: %w", err)
 	}
 
 	if thinpoolMetadataSizeBytes > 0 {
-		args = append(args, "--poolmetadatasize", fmt.Sprintf("%db", thinpoolMetadataSizeBytes))
+		args = append(args, "--poolmetadatasize", strconv.FormatInt(thinpoolMetadataSizeBytes, 10)+"b")
 	}
 
-	if isRecent {
+	if thinpoolSizeBytes > 0 {
+		args = append(args, "--size", strconv.FormatInt(thinpoolSizeBytes, 10)+"b")
+	} else if isRecent {
 		args = append(args, "--extents", "100%FREE")
 	} else {
 		args = append(args, "--size", "1G")
@@ -239,7 +251,7 @@ func (d *lvm) createDefaultThinPool(lvmVersion, vgName, thinPoolName string, thi
 				return fmt.Errorf("Invalid volume stripe size %q: %w", d.config["volume.lvm.stripes.size"], err)
 			}
 
-			args = append(args, "--stripesize", fmt.Sprintf("%db", stripSizeBytes))
+			args = append(args, "--stripesize", strconv.FormatInt(stripSizeBytes, 10)+"b")
 		}
 	}
 
@@ -249,7 +261,7 @@ func (d *lvm) createDefaultThinPool(lvmVersion, vgName, thinPoolName string, thi
 		return fmt.Errorf("Error creating LVM thin pool named %q: %w", thinPoolName, err)
 	}
 
-	if !isRecent {
+	if !isRecent && thinpoolSizeBytes <= 0 {
 		// Grow it to the maximum VG size (two step process required by old LVM).
 		_, err = shared.TryRunCommand("lvextend", "--alloc", "anywhere", "-l", "100%FREE", lvmThinPool)
 		if err != nil {
@@ -322,15 +334,15 @@ func (d *lvm) createLogicalVolume(vgName, thinPoolName string, vol Volume, makeT
 	}
 
 	if makeThinLv {
-		targetVg := fmt.Sprintf("%s/%s", vgName, thinPoolName)
+		targetVg := vgName + "/" + thinPoolName
 		args = append(args,
 			"--thin",
-			"--virtualsize", fmt.Sprintf("%db", lvSizeBytes),
+			"--virtualsize", strconv.FormatInt(lvSizeBytes, 10)+"b",
 			targetVg,
 		)
 	} else {
 		args = append(args,
-			"--size", fmt.Sprintf("%db", lvSizeBytes),
+			"--size", strconv.FormatInt(lvSizeBytes, 10)+"b",
 			vgName,
 		)
 
@@ -346,7 +358,7 @@ func (d *lvm) createLogicalVolume(vgName, thinPoolName string, vol Volume, makeT
 					return fmt.Errorf("Invalid volume stripe size %q: %w", stripeSize, err)
 				}
 
-				args = append(args, "--stripesize", fmt.Sprintf("%db", stripSizeBytes))
+				args = append(args, "--stripesize", strconv.FormatInt(stripSizeBytes, 10)+"b")
 			}
 		}
 	}
@@ -363,6 +375,12 @@ func (d *lvm) createLogicalVolume(vgName, thinPoolName string, vol Volume, makeT
 		if err != nil {
 			return fmt.Errorf("Error making filesystem on LVM logical volume: %w", err)
 		}
+	} else if !d.usesThinpool() {
+		// Make sure we get an empty LV.
+		err := block.ClearBlock(volDevPath, 0)
+		if err != nil {
+			return fmt.Errorf("Error clearing LVM logical volume: %w", err)
+		}
 	}
 
 	isRecent, err := d.lvmVersionIsAtLeast(lvmVersion, "2.02.99")
@@ -373,13 +391,13 @@ func (d *lvm) createLogicalVolume(vgName, thinPoolName string, vol Volume, makeT
 	if isRecent {
 		// Disable auto activation of volume on LVM versions that support it.
 		// Must be done after volume create so that zeroing and signature wiping can take place.
-		_, err := shared.RunCommand("lvchange", "--setactivationskip", "y", volDevPath)
+		_, err := shared.RunCommandContext(context.TODO(), "lvchange", "--setactivationskip", "y", volDevPath)
 		if err != nil {
 			return fmt.Errorf("Failed to set activation skip on LVM logical volume %q: %w", volDevPath, err)
 		}
 	}
 
-	d.logger.Debug("Logical volume created", logger.Ctx{"vg_name": vgName, "lv_name": lvFullName, "size": fmt.Sprintf("%db", lvSizeBytes), "fs": vol.ConfigBlockFilesystem()})
+	d.logger.Debug("Logical volume created", logger.Ctx{"vg_name": vgName, "lv_name": lvFullName, "size": strconv.FormatInt(lvSizeBytes, 10) + "b", "fs": vol.ConfigBlockFilesystem()})
 	return nil
 }
 
@@ -417,7 +435,7 @@ func (d *lvm) createLogicalVolumeSnapshot(vgName string, srcVol Volume, snapVol
 
 	_, err = shared.TryRunCommand("lvcreate", args...)
 	if err != nil {
-		return "", err
+		return "", fmt.Errorf("Error creating LV snapshot named %q: %w", snapLvName, err)
 	}
 
 	d.logger.Debug("Logical volume snapshot created", logCtx)
@@ -468,12 +486,14 @@ func (d *lvm) lvmFullVolumeName(volType VolumeType, contentType ContentType, vol
 	contentTypeSuffix := ""
 	if contentType == ContentTypeBlock {
 		contentTypeSuffix = lvmBlockVolSuffix
+	} else if contentType == ContentTypeISO {
+		contentTypeSuffix = lvmISOVolSuffix
 	}
 
 	// Escape the volume name to a name suitable for using as a logical volume.
 	lvName := strings.Replace(strings.Replace(volName, "-", lvmEscapedHyphen, -1), shared.SnapshotDelimiter, lvmSnapshotSeparator, -1)
 
-	return fmt.Sprintf("%s_%s%s", volType, lvName, contentTypeSuffix)
+	return string(volType) + "_" + lvName + contentTypeSuffix
 }
 
 // lvmDevPath returns the path to the LVM volume device. Empty string is returned if invalid volType supplied.
@@ -483,22 +503,32 @@ func (d *lvm) lvmDevPath(vgName string, volType VolumeType, contentType ContentT
 		return "" // Invalid volType supplied.
 	}
 
-	return fmt.Sprintf("/dev/%s/%s", vgName, fullVolName)
+	return "/dev/" + vgName + "/" + fullVolName
 }
 
 // resizeLogicalVolume resizes an LVM logical volume. This function does not resize any filesystem inside the LV.
 func (d *lvm) resizeLogicalVolume(lvPath string, sizeBytes int64) error {
-	_, err := shared.TryRunCommand("lvresize", "-L", fmt.Sprintf("%db", sizeBytes), "-f", lvPath)
+	isRecent, err := d.lvmVersionIsAtLeast(lvmVersion, "2.03.17")
+	if err != nil {
+		return fmt.Errorf("Error checking LVM version: %w", err)
+	}
+
+	args := []string{"-L", strconv.FormatInt(sizeBytes, 10) + "b", "-f", lvPath}
+	if isRecent {
+		args = append(args, "--fs=ignore")
+	}
+
+	_, err = shared.TryRunCommand("lvresize", args...)
 	if err != nil {
 		return err
 	}
 
-	d.logger.Debug("Logical volume resized", logger.Ctx{"dev": lvPath, "size": fmt.Sprintf("%db", sizeBytes)})
+	d.logger.Debug("Logical volume resized", logger.Ctx{"dev": lvPath, "size": strconv.FormatInt(sizeBytes, 10) + "b"})
 	return nil
 }
 
 // copyThinpoolVolume makes an optimised copy of a thinpool volume by using thinpool snapshots.
-func (d *lvm) copyThinpoolVolume(vol, srcVol Volume, srcSnapshots []Volume, refresh bool) error {
+func (d *lvm) copyThinpoolVolume(vol, srcVol Volume, srcSnapshots []string, refresh bool) error {
 	revert := revert.New()
 	defer revert.Fail()
 
@@ -513,8 +543,7 @@ func (d *lvm) copyThinpoolVolume(vol, srcVol Volume, srcSnapshots []Volume, refr
 		}
 
 		for _, srcSnapshot := range srcSnapshots {
-			_, snapName, _ := api.GetParentAndSnapshotName(srcSnapshot.name)
-			newFullSnapName := GetSnapshotVolumeName(vol.name, snapName)
+			newFullSnapName := GetSnapshotVolumeName(vol.name, srcSnapshot)
 			newSnapVol := NewVolume(d, d.Name(), vol.volType, vol.contentType, newFullSnapName, vol.config, vol.poolConfig)
 
 			volExists, err := d.HasVolume(newSnapVol)
@@ -534,6 +563,11 @@ func (d *lvm) copyThinpoolVolume(vol, srcVol Volume, srcSnapshots []Volume, refr
 
 			revert.Add(func() { _ = os.RemoveAll(newSnapVolPath) })
 
+			srcSnapshot, err := srcVol.NewSnapshot(srcSnapshot)
+			if err != nil {
+				return err
+			}
+
 			// We do not modify the original snapshot so as to avoid damaging if it is corrupted for
 			// some reason. If the filesystem needs to have a unique UUID generated in order to mount
 			// this will be done at restore time to be safe.
@@ -555,27 +589,27 @@ func (d *lvm) copyThinpoolVolume(vol, srcVol Volume, srcSnapshots []Volume, refr
 	}
 
 	if volExists {
-		if refresh {
-			newVolDevPath := d.lvmDevPath(d.config["lvm.vg_name"], vol.volType, vol.contentType, vol.name)
-			tmpVolName := fmt.Sprintf("%s%s", vol.name, tmpVolSuffix)
-			tmpVolDevPath := d.lvmDevPath(d.config["lvm.vg_name"], vol.volType, vol.contentType, tmpVolName)
-
-			// Rename existing volume to temporary new name so we can revert if needed.
-			err := d.renameLogicalVolume(newVolDevPath, tmpVolDevPath)
-			if err != nil {
-				return fmt.Errorf("Error temporarily renaming original LVM logical volume: %w", err)
-			}
-
-			// Record this volume to be removed at the very end.
-			removeVols = append(removeVols, tmpVolName)
-
-			revert.Add(func() {
-				// Rename the original volume back to the original name.
-				_ = d.renameLogicalVolume(tmpVolDevPath, newVolDevPath)
-			})
-		} else {
+		if !refresh {
 			return fmt.Errorf("LVM volume already exists %q", vol.name)
 		}
+
+		newVolDevPath := d.lvmDevPath(d.config["lvm.vg_name"], vol.volType, vol.contentType, vol.name)
+		tmpVolName := vol.name + tmpVolSuffix
+		tmpVolDevPath := d.lvmDevPath(d.config["lvm.vg_name"], vol.volType, vol.contentType, tmpVolName)
+
+		// Rename existing volume to temporary new name so we can revert if needed.
+		err := d.renameLogicalVolume(newVolDevPath, tmpVolDevPath)
+		if err != nil {
+			return fmt.Errorf("Error temporarily renaming original LVM logical volume: %w", err)
+		}
+
+		// Record this volume to be removed at the very end.
+		removeVols = append(removeVols, tmpVolName)
+
+		revert.Add(func() {
+			// Rename the original volume back to the original name.
+			_ = d.renameLogicalVolume(tmpVolDevPath, newVolDevPath)
+		})
 	} else {
 		volPath := vol.MountPath()
 		err := vol.EnsureMountPath()
@@ -645,7 +679,7 @@ func (d *lvm) copyThinpoolVolume(vol, srcVol Volume, srcSnapshots []Volume, refr
 
 // logicalVolumeSize gets the size in bytes of a logical volume.
 func (d *lvm) logicalVolumeSize(volDevPath string) (int64, error) {
-	output, err := shared.RunCommand("lvs", "--noheadings", "--nosuffix", "--units", "b", "-o", "lv_size", volDevPath)
+	output, err := shared.RunCommandContext(context.TODO(), "lvs", "--noheadings", "--nosuffix", "--units", "b", "-o", "lv_size", volDevPath)
 	if err != nil {
 		if d.isLVMNotFoundExitError(err) {
 			return -1, api.StatusErrorf(http.StatusNotFound, "LVM volume not found")
@@ -658,7 +692,7 @@ func (d *lvm) logicalVolumeSize(volDevPath string) (int64, error) {
 	return strconv.ParseInt(output, 10, 64)
 }
 
-func (d *lvm) thinPoolVolumeUsage(volDevPath string) (uint64, uint64, error) {
+func (d *lvm) thinPoolVolumeUsage(volDevPath string) (totalSize uint64, usedSize uint64, err error) {
 	args := []string{
 		volDevPath,
 		"--noheadings",
@@ -668,7 +702,7 @@ func (d *lvm) thinPoolVolumeUsage(volDevPath string) (uint64, uint64, error) {
 		"-o", "lv_size,data_percent,metadata_percent",
 	}
 
-	out, err := shared.RunCommand("lvs", args...)
+	out, err := shared.RunCommandContext(context.TODO(), "lvs", args...)
 	if err != nil {
 		return 0, 0, err
 	}
@@ -678,13 +712,11 @@ func (d *lvm) thinPoolVolumeUsage(volDevPath string) (uint64, uint64, error) {
 		return 0, 0, fmt.Errorf("Unexpected output from lvs command")
 	}
 
-	total, err := strconv.ParseUint(parts[0], 10, 64)
+	totalSize, err = strconv.ParseUint(parts[0], 10, 64)
 	if err != nil {
 		return 0, 0, fmt.Errorf("Failed parsing thin volume total size (%q): %w", parts[0], err)
 	}
 
-	totalSize := total
-
 	// Used percentage is not available if thin volume isn't activated.
 	if parts[1] == "" {
 		return 0, 0, ErrNotSupported
@@ -705,7 +737,7 @@ func (d *lvm) thinPoolVolumeUsage(volDevPath string) (uint64, uint64, error) {
 		}
 	}
 
-	usedSize := uint64(float64(total) * ((dataPerc + metaPerc) / 100))
+	usedSize = uint64(float64(totalSize) * ((dataPerc + metaPerc) / 100))
 
 	return totalSize, usedSize, nil
 }
@@ -730,10 +762,10 @@ func (d *lvm) parseLogicalVolumeSnapshot(parent Volume, lvmVolName string) strin
 	}
 
 	// Prefix we would expect for a snapshot of the parent volume.
-	snapPrefix := fmt.Sprintf("%s%s", fullVolName, lvmSnapshotSeparator)
+	snapPrefix := fullVolName + lvmSnapshotSeparator
 
 	// Prefix used when escaping "-" in volume names. Doesn't indicate a snapshot of parent.
-	badPrefix := fmt.Sprintf("%s%s", fullVolName, lvmEscapedHyphen)
+	badPrefix := fullVolName + lvmEscapedHyphen
 
 	// Check the volume matches the snapshot prefix, but doesn't match the prefix that indicates a similarly
 	// named volume that just has escaped "-" characters in it.
@@ -758,7 +790,7 @@ func (d *lvm) activateVolume(vol Volume) (bool, error) {
 	}
 
 	if !shared.PathExists(volDevPath) {
-		_, err := shared.RunCommand("lvchange", "--activate", "y", "--ignoreactivationskip", volDevPath)
+		_, err := shared.RunCommandContext(context.TODO(), "lvchange", "--activate", "y", "--ignoreactivationskip", volDevPath)
 		if err != nil {
 			return false, fmt.Errorf("Failed to activate LVM logical volume %q: %w", volDevPath, err)
 		}
@@ -796,7 +828,7 @@ func (d *lvm) deactivateVolume(vol Volume) (bool, error) {
 		// Keep trying to deactivate a few times in case the device is still being flushed.
 		var err error
 		for i := 0; i < 20; i++ {
-			_, err = shared.RunCommand("lvchange", "--activate", "n", "--ignoreactivationskip", volDevPath)
+			_, err = shared.RunCommandContext(context.TODO(), "lvchange", "--activate", "n", "--ignoreactivationskip", volDevPath)
 			if err == nil {
 				break
 			}
