@@ -7,6 +7,7 @@ import (
 	"math"
 	"os"
 	"os/exec"
+	"strconv"
 	"strings"
 
 	"golang.org/x/sys/unix"
@@ -16,6 +17,7 @@ import (
 	"github.com/canonical/lxd/lxd/migration"
 	"github.com/canonical/lxd/lxd/operations"
 	"github.com/canonical/lxd/lxd/rsync"
+	"github.com/canonical/lxd/lxd/storage/block"
 	"github.com/canonical/lxd/lxd/storage/filesystem"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
@@ -67,6 +69,21 @@ func (d *lvm) CreateVolume(vol Volume, filler *VolumeFiller, op *operations.Oper
 				if err != nil {
 					return err
 				}
+
+				// Check the block size for image volumes.
+				if vol.volType == VolumeTypeImage {
+					blockSize, err := block.DiskBlockSize(devPath)
+					if err != nil {
+						return err
+					}
+
+					// Our images are all built using 512 bytes physical block sizes.
+					// When those are written to a 4k physical block size device,
+					// the partition table makes no sense and leads to an unbootable VM.
+					if blockSize != 512 {
+						return fmt.Errorf("Underlying storage uses %d bytes sector size when virtual machine images require 512 bytes", blockSize)
+					}
+				}
 			}
 
 			allowUnsafeResize := false
@@ -313,6 +330,7 @@ func (d *lvm) commonVolumeRules() map[string]func(value string) error {
 		//  type: string
 		//  defaultdesc: same as `volume.lvm.stripes`
 		//  shortdesc: Number of stripes to use for new volumes (or thin pool volume)
+		//  scope: global
 		"lvm.stripes": validate.Optional(validate.IsUint32),
 		// lxdmeta:generate(entities=storage-lvm; group=volume-conf; key=lvm.stripes.size)
 		// The size must be at least 4096 bytes, and a multiple of 512 bytes.
@@ -320,6 +338,7 @@ func (d *lvm) commonVolumeRules() map[string]func(value string) error {
 		//  type: string
 		//  defaultdesc: same as `volume.lvm.stripes.size`
 		//  shortdesc: Size of stripes to use
+		//  scope: global
 		"lvm.stripes.size": validate.Optional(validate.IsSize),
 	}
 }
@@ -447,17 +466,7 @@ func (d *lvm) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool, op
 		return nil
 	}
 
-	l := d.logger.AddContext(logger.Ctx{"dev": volDevPath, "size": fmt.Sprintf("%db", sizeBytes)})
-
-	// Activate volume if needed.
-	activated, err := d.activateVolume(vol)
-	if err != nil {
-		return err
-	}
-
-	if activated {
-		defer func() { _, _ = d.deactivateVolume(vol) }()
-	}
+	l := d.logger.AddContext(logger.Ctx{"dev": volDevPath, "size": strconv.FormatInt(sizeBytes, 10) + "b"})
 
 	inUse := vol.MountInUse()
 
@@ -474,6 +483,18 @@ func (d *lvm) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool, op
 				return ErrInUse // We don't allow online shrinking of filesytem volumes.
 			}
 
+			// Activate volume if needed.
+			activated, err := d.activateVolume(vol)
+			if err != nil {
+				return err
+			}
+
+			if !activated {
+				defer func() {
+					_, _ = d.activateVolume(vol)
+				}()
+			}
+
 			// Shrink filesystem first.
 			// Pass allowUnsafeResize to allow disabling of filesystem resize safety checks.
 			// We do this as a separate step rather than passing -r to lvresize in resizeLogicalVolume
@@ -481,6 +502,13 @@ func (d *lvm) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool, op
 			// otherwise by passing -f to lvresize (required for other reasons) this would then pass
 			// -f onto resize2fs as well.
 			err = shrinkFileSystem(fsType, volDevPath, vol, sizeBytes, allowUnsafeResize)
+			if err != nil {
+				_, _ = d.deactivateVolume(vol)
+				return err
+			}
+
+			// Deactivate the volume for resizing.
+			_, err = d.deactivateVolume(vol)
 			if err != nil {
 				return err
 			}
@@ -499,6 +527,18 @@ func (d *lvm) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool, op
 				return err
 			}
 
+			// Activate the volume for resizing.
+			activated, err := d.activateVolume(vol)
+			if err != nil {
+				return err
+			}
+
+			if activated {
+				defer func() {
+					_, _ = d.deactivateVolume(vol)
+				}()
+			}
+
 			// Grow the filesystem to fill block device.
 			err = growFileSystem(fsType, volDevPath, vol)
 			if err != nil {
@@ -525,9 +565,42 @@ func (d *lvm) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool, op
 			return err
 		}
 
+		// The new blocks in a grown volume will need clearing if using a thick pool.
+		needsClearing := !d.usesThinpool() && (oldSizeBytes < sizeBytes)
+
+		// VM block volumes need the GPT header moved on normal resize scenarios.
+		needsGPTHeaderMove := vol.IsVMBlock() && !allowUnsafeResize
+
+		// Need to activate the volume to clear it or to move the GPT header.
+		needsActivating := needsClearing || needsGPTHeaderMove
+
+		if needsActivating {
+			// Activate the volume for clearing blocks and/or moving GPT header.
+			activated, err := d.activateVolume(vol)
+			if err != nil {
+				return err
+			}
+
+			if activated {
+				defer func() {
+					_, _ = d.deactivateVolume(vol)
+				}()
+			}
+		}
+
+		// On thick pools, discard the blocks in the additional space when the volume is grown.
+		if needsClearing {
+			// Discard blocks from the end of the old volume's size.
+			err := block.ClearBlock(volDevPath, oldSizeBytes)
+			if err != nil {
+				return err
+			}
+		}
+
 		// Move the VM GPT alt header to end of disk if needed (not needed in unsafe resize mode as it is
 		// expected the caller will do all necessary post resize actions themselves).
-		if vol.IsVMBlock() && !allowUnsafeResize {
+		// Do this after the new blocks have been cleared.
+		if needsGPTHeaderMove {
 			err = d.moveGPTAltHeader(volDevPath)
 			if err != nil {
 				return err
@@ -575,7 +648,7 @@ func (d *lvm) ListVolumes() ([]Volume, error) {
 		var volName string
 
 		for _, volumeType := range d.Info().VolumeTypes {
-			prefix := fmt.Sprintf("%s_", volumeType)
+			prefix := string(volumeType) + "_"
 			if strings.HasPrefix(rawName, prefix) {
 				volType = volumeType
 				volName = strings.TrimPrefix(rawName, prefix)
@@ -1004,7 +1077,7 @@ func (d *lvm) MountVolumeSnapshot(snapVol Volume, op *operations.Operation) erro
 		regenerateFSUUID := renegerateFilesystemUUIDNeeded(snapVol.ConfigBlockFilesystem())
 		if regenerateFSUUID {
 			// Instantiate a new volume to be the temporary writable snapshot.
-			tmpVolName := fmt.Sprintf("%s%s", snapVol.name, tmpVolSuffix)
+			tmpVolName := snapVol.name + tmpVolSuffix
 			tmpVol := NewVolume(d, d.name, snapVol.volType, snapVol.contentType, tmpVolName, snapVol.config, snapVol.poolConfig)
 
 			// Create writable snapshot from source snapshot named with a tmpVolSuffix suffix.
@@ -1107,7 +1180,7 @@ func (d *lvm) UnmountVolumeSnapshot(snapVol Volume, op *operations.Operation) (b
 		d.logger.Debug("Unmounted logical volume snapshot", logger.Ctx{"path": mountPath})
 
 		// Check if a temporary snapshot exists, and if so remove it.
-		tmpVolName := fmt.Sprintf("%s%s", snapVol.name, tmpVolSuffix)
+		tmpVolName := snapVol.name + tmpVolSuffix
 		tmpVolDevPath := d.lvmDevPath(d.config["lvm.vg_name"], snapVol.volType, snapVol.contentType, tmpVolName)
 		exists, err := d.logicalVolumeExists(tmpVolDevPath)
 		if err != nil {
@@ -1221,7 +1294,7 @@ func (d *lvm) RestoreVolume(vol Volume, snapVol Volume, op *operations.Operation
 		}
 
 		originalVolDevPath := d.lvmDevPath(d.config["lvm.vg_name"], restoreVol.volType, restoreVol.contentType, restoreVol.name)
-		tmpVolName := fmt.Sprintf("%s%s", restoreVol.name, tmpVolSuffix)
+		tmpVolName := restoreVol.name + tmpVolSuffix
 		tmpVolDevPath := d.lvmDevPath(d.config["lvm.vg_name"], restoreVol.volType, restoreVol.contentType, tmpVolName)
 
 		reverter := revert.New()
@@ -1337,7 +1410,7 @@ func (d *lvm) RestoreVolume(vol Volume, snapVol Volume, op *operations.Operation
 		snapLVPath := d.lvmDevPath(d.config["lvm.vg_name"], snapVol.volType, ContentTypeFS, snapVol.name)
 		_, err = shared.TryRunCommand("lvresize", "-l", "+100%ORIGIN", "-f", snapLVPath)
 		if err != nil {
-			return err
+			return fmt.Errorf("Error resizing LV snapshot named %q: %w", snapLVPath, err)
 		}
 	}
 
@@ -1345,7 +1418,7 @@ func (d *lvm) RestoreVolume(vol Volume, snapVol Volume, op *operations.Operation
 		snapLVPath := d.lvmDevPath(d.config["lvm.vg_name"], snapVol.volType, ContentTypeBlock, snapVol.name)
 		_, err = shared.TryRunCommand("lvresize", "-l", "+100%ORIGIN", "-f", snapLVPath)
 		if err != nil {
-			return err
+			return fmt.Errorf("Error resizing LV snapshot named %q: %w", snapLVPath, err)
 		}
 	}
 
