@@ -1,15 +1,12 @@
 package drivers
 
 import (
-	"context"
 	"errors"
 	"fmt"
 	"io"
 	"math"
 	"net/http"
 	"os"
-	"slices"
-	"strconv"
 	"strings"
 
 	"golang.org/x/sys/unix"
@@ -155,7 +152,7 @@ func (d *powerflex) CreateVolume(vol Volume, filler *VolumeFiller, op *operation
 
 // CreateVolumeFromBackup re-creates a volume from its exported state.
 func (d *powerflex) CreateVolumeFromBackup(vol VolumeCopy, srcBackup backup.Info, srcData io.ReadSeeker, op *operations.Operation) (VolumePostHook, revert.Hook, error) {
-	return genericVFSBackupUnpack(d, d.state, vol, srcBackup.Snapshots, srcData, op)
+	return genericVFSBackupUnpack(d, d.state.OS, vol, srcBackup.Snapshots, srcData, op)
 }
 
 // CreateVolumeFromCopy provides same-pool volume copying functionality.
@@ -488,7 +485,7 @@ func (d *powerflex) ValidateVolume(vol Volume, removeUnknownKeys bool) error {
 			multipleCount++
 		}
 
-		vol.SetConfigSize(strconv.FormatInt(multipleCount*factorGiB*8, 10))
+		vol.SetConfigSize(fmt.Sprintf("%d", multipleCount*factorGiB*8))
 	}
 
 	commonRules := d.commonVolumeRules()
@@ -710,8 +707,7 @@ func (d *powerflex) MountVolume(vol Volume, op *operations.Operation) error {
 
 	revert.Add(cleanup)
 
-	switch vol.contentType {
-	case ContentTypeFS:
+	if vol.contentType == ContentTypeFS {
 		mountPath := vol.MountPath()
 		if !filesystem.IsMountPoint(mountPath) {
 			err = vol.EnsureMountPath()
@@ -729,15 +725,14 @@ func (d *powerflex) MountVolume(vol Volume, op *operations.Operation) error {
 			}
 
 			mountFlags, mountOptions := filesystem.ResolveMountOptions(strings.Split(vol.ConfigBlockMountOptions(), ","))
-			err = TryMount(context.TODO(), volDevPath, mountPath, fsType, mountFlags, mountOptions)
+			err = TryMount(volDevPath, mountPath, fsType, mountFlags, mountOptions)
 			if err != nil {
 				return err
 			}
 
 			d.logger.Debug("Mounted PowerFlex volume", logger.Ctx{"volName": vol.name, "dev": volDevPath, "path": mountPath, "options": mountOptions})
 		}
-
-	case ContentTypeBlock:
+	} else if vol.contentType == ContentTypeBlock {
 		// For VMs, mount the filesystem volume.
 		if vol.IsVMBlock() {
 			fsVol := vol.NewVMBlockFilesystemVolume()
@@ -1045,14 +1040,14 @@ func (d *powerflex) CheckVolumeSnapshots(vol Volume, snapVols []Volume, op *oper
 
 	// Check if the provided list of volume snapshots matches the ones from storage.
 	for _, wantedSnapshotName := range wantedSnapshotNames {
-		if !slices.Contains(storageSnapshotNames, wantedSnapshotName) {
+		if !shared.ValueInSlice(wantedSnapshotName, storageSnapshotNames) {
 			return fmt.Errorf("Snapshot %q expected but not in storage", wantedSnapshotName)
 		}
 	}
 
 	// Check if the snapshots in storage match the ones from the provided list.
 	for _, storageSnapshotName := range storageSnapshotNames {
-		if !slices.Contains(wantedSnapshotNames, storageSnapshotName) {
+		if !shared.ValueInSlice(storageSnapshotName, wantedSnapshotNames) {
 			return fmt.Errorf("Snapshot %q in storage but not expected", storageSnapshotName)
 		}
 	}
