@@ -1,13 +1,11 @@
 package drivers
 
 import (
-	"context"
 	"fmt"
 	"io"
 	"net/http"
 	"os"
 	"slices"
-	"strconv"
 	"strings"
 
 	"golang.org/x/sys/unix"
@@ -173,7 +171,7 @@ func (d *pure) CreateVolume(vol Volume, filler *VolumeFiller, op *operations.Ope
 
 // CreateVolumeFromBackup re-creates a volume from its exported state.
 func (d *pure) CreateVolumeFromBackup(vol VolumeCopy, srcBackup backup.Info, srcData io.ReadSeeker, op *operations.Operation) (VolumePostHook, revert.Hook, error) {
-	return genericVFSBackupUnpack(d, d.state, vol, srcBackup.Snapshots, srcData, op)
+	return genericVFSBackupUnpack(d, d.state.OS, vol, srcBackup.Snapshots, srcData, op)
 }
 
 // CreateVolumeFromCopy provides same-pool volume copying functionality.
@@ -260,7 +258,7 @@ func (d *pure) CreateVolumeFromCopy(vol VolumeCopy, srcVol VolumeCopy, allowInco
 			// Find the corresponding source snapshot.
 			var srcSnapshot *Volume
 			for _, srcSnap := range srcVol.Snapshots {
-				_, srcSnapshotShortName, _ := api.GetParentAndSnapshotName(srcSnap.name)
+				_, srcSnapshotShortName, _ := api.GetParentAndSnapshotName(snapshot.name)
 				if snapshotShortName == srcSnapshotShortName {
 					srcSnapshot = &srcSnap
 					break
@@ -305,7 +303,7 @@ func (d *pure) CreateVolumeFromCopy(vol VolumeCopy, srcVol VolumeCopy, allowInco
 	// Finally, copy the source volume (or snapshot) into destination volume snapshots.
 	if srcVol.IsSnapshot() {
 		// Get snapshot parent volume name.
-		srcParentVol := srcVol.GetParent()
+		srcParentVol := srcVol.Volume.GetParent()
 		srcParentVolName, err := d.getVolumeName(srcParentVol)
 		if err != nil {
 			return err
@@ -535,7 +533,7 @@ func (d *pure) refreshVolume(vol VolumeCopy, srcVol VolumeCopy, refreshSnapshots
 	// Finally, copy the source volume (or snapshot) into destination volume snapshots.
 	if srcVol.IsSnapshot() {
 		// Find snapshot parent volume.
-		srcParentVol := srcVol.GetParent()
+		srcParentVol := srcVol.Volume.GetParent()
 		srcParentVolName, err := d.getVolumeName(srcParentVol)
 		if err != nil {
 			return nil, err
@@ -730,7 +728,7 @@ func (d *pure) ValidateVolume(vol Volume, removeUnknownKeys bool) error {
 		remainder := sizeBytes % 512
 		if remainder > 0 {
 			sizeBytes = (sizeBytes/512 + 1) * 512
-			vol.SetConfigSize(strconv.FormatInt(sizeBytes, 10))
+			vol.SetConfigSize(fmt.Sprintf("%d", sizeBytes))
 		}
 	}
 
@@ -779,6 +777,9 @@ func (d *pure) GetVolumeUsage(vol Volume) (int64, error) {
 // SetVolumeQuota applies a size limit on volume.
 // Does nothing if supplied with an non-positive size.
 func (d *pure) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool, op *operations.Operation) error {
+	revert := revert.New()
+	defer revert.Fail()
+
 	// Convert to bytes.
 	sizeBytes, err := units.ParseByteSizeString(size)
 	if err != nil {
@@ -832,7 +833,7 @@ func (d *pure) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool, o
 				return err
 			}
 
-			defer cleanup()
+			revert.Add(cleanup)
 
 			// Shrink filesystem first.
 			err = shrinkFileSystem(fsType, devPath, vol, sizeBytes, allowUnsafeResize)
@@ -857,7 +858,7 @@ func (d *pure) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool, o
 				return err
 			}
 
-			defer cleanup()
+			revert.Add(cleanup)
 
 			// Ensure the block device is resized before growing the filesystem.
 			// This should succeed immediately, but if volume was already mapped,
@@ -903,7 +904,7 @@ func (d *pure) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool, o
 				return err
 			}
 
-			defer cleanup()
+			revert.Add(cleanup)
 
 			// Wait for the block device to be resized before moving GPT alt header.
 			// This ensures that the GPT alt header is not moved before the actual
@@ -921,6 +922,7 @@ func (d *pure) SetVolumeQuota(vol Volume, size string, allowUnsafeResize bool, o
 		}
 	}
 
+	revert.Success()
 	return nil
 }
 
@@ -959,8 +961,7 @@ func (d *pure) MountVolume(vol Volume, op *operations.Operation) error {
 
 	revert.Add(cleanup)
 
-	switch vol.contentType {
-	case ContentTypeFS:
+	if vol.contentType == ContentTypeFS {
 		mountPath := vol.MountPath()
 		if !filesystem.IsMountPoint(mountPath) {
 			err = vol.EnsureMountPath()
@@ -978,15 +979,14 @@ func (d *pure) MountVolume(vol Volume, op *operations.Operation) error {
 			}
 
 			mountFlags, mountOptions := filesystem.ResolveMountOptions(strings.Split(vol.ConfigBlockMountOptions(), ","))
-			err = TryMount(context.TODO(), volDevPath, mountPath, fsType, mountFlags, mountOptions)
+			err = TryMount(volDevPath, mountPath, fsType, mountFlags, mountOptions)
 			if err != nil {
 				return err
 			}
 
 			d.logger.Debug("Mounted Pure Storage volume", logger.Ctx{"volName": vol.name, "dev": volDevPath, "path": mountPath, "options": mountOptions})
 		}
-
-	case ContentTypeBlock:
+	} else if vol.contentType == ContentTypeBlock {
 		// For VMs, mount the filesystem volume.
 		if vol.IsVMBlock() {
 			fsVol := vol.NewVMBlockFilesystemVolume()
