@@ -1,6 +1,7 @@
 package drivers
 
 import (
+	"context"
 	"fmt"
 	"os"
 	"os/exec"
@@ -8,7 +9,6 @@ import (
 	"strconv"
 	"strings"
 
-	deviceConfig "github.com/canonical/lxd/lxd/device/config"
 	"github.com/canonical/lxd/lxd/migration"
 	"github.com/canonical/lxd/lxd/operations"
 	"github.com/canonical/lxd/lxd/util"
@@ -119,7 +119,8 @@ func (d *zfs) Info() Info {
 	info := Info{
 		Name:                         "zfs",
 		Version:                      zfsVersion,
-		DefaultVMBlockFilesystemSize: deviceConfig.DefaultVMBlockFilesystemSize,
+		DefaultBlockSize:             d.defaultBlockVolumeSize(),
+		DefaultVMBlockFilesystemSize: d.defaultVMBlockFilesystemSize(),
 		OptimizedImages:              true,
 		OptimizedBackups:             true,
 		PreservesInodes:              true,
@@ -130,6 +131,7 @@ func (d *zfs) Info() Info {
 		DirectIO:                     zfsDirectIO,
 		MountedRoot:                  false,
 		Buckets:                      true,
+		PopulateParentVolumeUUID:     false,
 	}
 
 	return info
@@ -139,17 +141,24 @@ func (d *zfs) Info() Info {
 // Accepts warnOnExistingPolicyApplyError argument, if true will warn rather than fail if applying current policy
 // to an existing dataset fails.
 func (d zfs) ensureInitialDatasets(warnOnExistingPolicyApplyError bool) error {
-	args := make([]string, 0, len(zfsDefaultSettings))
+	properties := make([]string, 0, len(zfsDefaultSettings))
 	for k, v := range zfsDefaultSettings {
-		args = append(args, fmt.Sprintf("%s=%s", k, v))
+		properties = append(properties, k+"="+v)
 	}
 
-	err := d.setDatasetProperties(d.config["zfs.pool_name"], args...)
+	properties, err := d.filterRedundantOptions(d.config["zfs.pool_name"], properties...)
 	if err != nil {
-		if warnOnExistingPolicyApplyError {
+		return err
+	}
+
+	if len(properties) > 0 {
+		err := d.setDatasetProperties(d.config["zfs.pool_name"], properties...)
+		if err != nil {
+			if !warnOnExistingPolicyApplyError {
+				return fmt.Errorf("Failed applying policy to existing dataset %q: %w", d.config["zfs.pool_name"], err)
+			}
+
 			d.logger.Warn("Failed applying policy to existing dataset", logger.Ctx{"dataset": d.config["zfs.pool_name"], "err": err})
-		} else {
-			return fmt.Errorf("Failed applying policy to existing dataset %q: %w", d.config["zfs.pool_name"], err)
 		}
 	}
 
@@ -166,12 +175,19 @@ func (d zfs) ensureInitialDatasets(warnOnExistingPolicyApplyError bool) error {
 		}
 
 		if exists {
-			err = d.setDatasetProperties(datasetPath, properties...)
+			properties, err = d.filterRedundantOptions(datasetPath, properties...)
 			if err != nil {
-				if warnOnExistingPolicyApplyError {
+				return err
+			}
+
+			if len(properties) > 0 {
+				err = d.setDatasetProperties(datasetPath, properties...)
+				if err != nil {
+					if !warnOnExistingPolicyApplyError {
+						return fmt.Errorf("Failed applying policy to existing dataset %q: %w", datasetPath, err)
+					}
+
 					d.logger.Warn("Failed applying policy to existing dataset", logger.Ctx{"dataset": datasetPath, "err": err})
-				} else {
-					return fmt.Errorf("Failed applying policy to existing dataset %q: %w", datasetPath, err)
 				}
 			}
 		} else {
@@ -204,7 +220,7 @@ func (d *zfs) FillConfig() error {
 				return err
 			}
 
-			d.config["size"] = fmt.Sprintf("%dGiB", defaultSize)
+			d.config["size"] = fmt.Sprint(defaultSize) + "GiB"
 		}
 	} else if filepath.IsAbs(d.config["source"]) {
 		// Set default pool_name.
@@ -257,14 +273,14 @@ func (d *zfs) Create() error {
 		}
 
 		// Create the zpool.
-		_, err = shared.RunCommand("zpool", "create", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], loopPath)
+		_, err = shared.RunCommandContext(d.state.ShutdownCtx, "zpool", "create", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], loopPath)
 		if err != nil {
 			return err
 		}
 
 		// Apply auto-trim if supported.
 		if zfsTrim {
-			_, err := shared.RunCommand("zpool", "set", "autotrim=on", d.config["zfs.pool_name"])
+			_, err := shared.RunCommandContext(d.state.ShutdownCtx, "zpool", "set", "autotrim=on", d.config["zfs.pool_name"])
 			if err != nil {
 				return err
 			}
@@ -290,13 +306,13 @@ func (d *zfs) Create() error {
 			d.config["source.wipe"] = ""
 
 			// Create the zpool.
-			_, err = shared.RunCommand("zpool", "create", "-f", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], d.config["source"])
+			_, err = shared.RunCommandContext(d.state.ShutdownCtx, "zpool", "create", "-f", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], d.config["source"])
 			if err != nil {
 				return err
 			}
 		} else {
 			// Create the zpool.
-			_, err := shared.RunCommand("zpool", "create", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], d.config["source"])
+			_, err := shared.RunCommandContext(d.state.ShutdownCtx, "zpool", "create", "-m", "none", "-O", "compression=on", d.config["zfs.pool_name"], d.config["source"])
 			if err != nil {
 				return err
 			}
@@ -304,7 +320,7 @@ func (d *zfs) Create() error {
 
 		// Apply auto-trim if supported.
 		if zfsTrim {
-			_, err := shared.RunCommand("zpool", "set", "autotrim=on", d.config["zfs.pool_name"])
+			_, err := shared.RunCommandContext(d.state.ShutdownCtx, "zpool", "set", "autotrim=on", d.config["zfs.pool_name"])
 			if err != nil {
 				return err
 			}
@@ -400,13 +416,13 @@ func (d *zfs) Delete(op *operations.Operation) error {
 
 	if strings.Contains(d.config["zfs.pool_name"], "/") {
 		// Delete the dataset.
-		_, err := shared.RunCommand("zfs", "destroy", "-r", d.config["zfs.pool_name"])
+		_, err := shared.RunCommandContext(context.TODO(), "zfs", "destroy", "-r", d.config["zfs.pool_name"])
 		if err != nil {
 			return err
 		}
 	} else {
 		// Delete the pool.
-		_, err := shared.RunCommand("zpool", "destroy", d.config["zfs.pool_name"])
+		_, err := shared.RunCommandContext(context.TODO(), "zpool", "destroy", d.config["zfs.pool_name"])
 		if err != nil {
 			return err
 		}
@@ -438,6 +454,7 @@ func (d *zfs) Validate(config map[string]string) error {
 		//  type: string
 		//  defaultdesc: name of the pool
 		//  shortdesc: Name of the zpool
+		//  scope: local
 		"zfs.pool_name": validate.IsAny,
 		// lxdmeta:generate(entities=storage-zfs; group=pool-conf; key=zfs.clone_copy)
 		// Set this option to `true` or `false` to enable or disable using ZFS lightweight clones rather
@@ -447,6 +464,7 @@ func (d *zfs) Validate(config map[string]string) error {
 		//  type: string
 		//  defaultdesc: `true`
 		//  shortdesc: Whether to use ZFS lightweight clones
+		//  scope: global
 		"zfs.clone_copy": validate.Optional(func(value string) error {
 			if value == "rebase" {
 				return nil
@@ -459,7 +477,8 @@ func (d *zfs) Validate(config map[string]string) error {
 		// ---
 		//  type: bool
 		//  defaultdesc: `true`
-		//  shortdesc: Disable zpool export while an unmount is being performed
+		//  shortdesc: Whether to export the zpool when an unmount is being performed
+		//  scope: global
 		"zfs.export": validate.Optional(validate.IsBool),
 	}
 
@@ -497,7 +516,7 @@ func (d *zfs) Update(changedConfig map[string]string) error {
 			return err
 		}
 
-		_, err = shared.RunCommand("zpool", "online", "-e", d.config["zfs.pool_name"], loopPath)
+		_, err = shared.RunCommandContext(d.state.ShutdownCtx, "zpool", "online", "-e", d.config["zfs.pool_name"], loopPath)
 		if err != nil {
 			return err
 		}
@@ -536,12 +555,12 @@ func (d *zfs) importPool() (bool, error) {
 	// Import the pool.
 	if filepath.IsAbs(d.config["source"]) {
 		disksPath := shared.VarPath("disks")
-		_, err := shared.RunCommand("zpool", "import", "-f", "-d", disksPath, poolName)
+		_, err := shared.RunCommandContext(d.state.ShutdownCtx, "zpool", "import", "-f", "-d", disksPath, poolName)
 		if err != nil {
 			return false, err
 		}
 	} else {
-		_, err := shared.RunCommand("zpool", "import", poolName)
+		_, err := shared.RunCommandContext(d.state.ShutdownCtx, "zpool", "import", poolName)
 		if err != nil {
 			return false, err
 		}
@@ -601,7 +620,7 @@ func (d *zfs) Unmount() (bool, error) {
 
 	// Export the pool.
 	poolName := strings.Split(d.config["zfs.pool_name"], "/")[0]
-	_, err = shared.RunCommand("zpool", "export", poolName)
+	_, err = shared.RunCommandContext(context.TODO(), "zpool", "export", poolName)
 	if err != nil {
 		return false, err
 	}
@@ -609,6 +628,7 @@ func (d *zfs) Unmount() (bool, error) {
 	return true, nil
 }
 
+// GetResources returns utilization statistics for the storage pool.
 func (d *zfs) GetResources() (*api.ResourcesStoragePool, error) {
 	// Get the total amount of space.
 	availableStr, err := d.getDatasetProperty(d.config["zfs.pool_name"], "available")
@@ -641,7 +661,8 @@ func (d *zfs) GetResources() (*api.ResourcesStoragePool, error) {
 	return &res, nil
 }
 
-// MigrationType returns the type of transfer methods to be used when doing migrations between pools in preference order.
+// MigrationTypes returns the type of transfer methods to be used when doing
+// migrations between pools in preference order.
 func (d *zfs) MigrationTypes(contentType ContentType, refresh bool, copySnapshots bool) []migration.Type {
 	var rsyncFeatures []string
 
@@ -701,13 +722,13 @@ func (d *zfs) patchDropBlockVolumeFilesystemExtension() error {
 		poolName = d.name
 	}
 
-	out, err := shared.RunCommand("zfs", "list", "-H", "-r", "-o", "name", "-t", "volume", fmt.Sprintf("%s/images", poolName))
+	out, err := shared.RunCommandContext(d.state.ShutdownCtx, "zfs", "list", "-H", "-r", "-o", "name", "-t", "volume", poolName+"/images")
 	if err != nil {
 		return fmt.Errorf("Failed listing images: %w", err)
 	}
 
 	for _, volume := range strings.Split(out, "\n") {
-		fields := strings.SplitN(volume, fmt.Sprintf("%s/images/", poolName), 2)
+		fields := strings.SplitN(volume, poolName+"/images/", 2)
 
 		if len(fields) != 2 || fields[1] == "" {
 			continue
@@ -719,9 +740,9 @@ func (d *zfs) patchDropBlockVolumeFilesystemExtension() error {
 		}
 
 		// Rename zfs dataset. Snapshots will automatically be renamed.
-		newName := fmt.Sprintf("%s/images/%s.block", poolName, strings.Split(fields[1], "_")[0])
+		newName := poolName + "/images/" + strings.Split(fields[1], "_")[0] + ".block"
 
-		_, err = shared.RunCommand("zfs", "rename", volume, newName)
+		_, err = shared.RunCommandContext(context.TODO(), "zfs", "rename", volume, newName)
 		if err != nil {
 			return fmt.Errorf("Failed renaming zfs dataset: %w", err)
 		}
@@ -729,3 +750,17 @@ func (d *zfs) patchDropBlockVolumeFilesystemExtension() error {
 
 	return nil
 }
+
+// roundVolumeBlockSizeBytes returns sizeBytes rounded up to the next multiple
+// of `vol`'s "zfs.blocksize".
+func (d *zfs) roundVolumeBlockSizeBytes(vol Volume, sizeBytes int64) int64 {
+	minBlockSize, err := units.ParseByteSizeString(vol.ExpandedConfig("zfs.blocksize"))
+
+	// minBlockSize will be 0 if zfs.blocksize=""
+	if minBlockSize <= 0 || err != nil {
+		// 16KiB is the default volblocksize
+		minBlockSize = 16 * 1024
+	}
+
+	return roundAbove(minBlockSize, sizeBytes)
+}
