@@ -1,6 +1,7 @@
 package drivers
 
 import (
+	"context"
 	"fmt"
 	"io"
 	"os"
@@ -38,20 +39,20 @@ func (d *zfs) dataset(vol Volume, deleted bool) string {
 	name, snapName, _ := api.GetParentAndSnapshotName(vol.name)
 
 	if vol.volType == VolumeTypeImage && vol.contentType == ContentTypeFS && d.isBlockBacked(vol) {
-		name = fmt.Sprintf("%s_%s", name, vol.ConfigBlockFilesystem())
+		name = name + "_" + vol.ConfigBlockFilesystem()
 	}
 
 	if (vol.volType == VolumeTypeVM || vol.volType == VolumeTypeImage) && vol.contentType == ContentTypeBlock {
-		name = fmt.Sprintf("%s%s", name, zfsBlockVolSuffix)
+		name = name + zfsBlockVolSuffix
 	} else if vol.volType == VolumeTypeCustom && vol.contentType == ContentTypeISO {
-		name = fmt.Sprintf("%s%s", name, zfsISOVolSuffix)
+		name = name + zfsISOVolSuffix
 	}
 
 	if snapName != "" {
 		if deleted {
-			name = fmt.Sprintf("%s@deleted-%s", name, uuid.New().String())
+			name = name + "@deleted-" + uuid.New().String()
 		} else {
-			name = fmt.Sprintf("%s@snapshot-%s", name, snapName)
+			name = name + "@snapshot-" + snapName
 		}
 	} else if deleted {
 		if vol.volType != VolumeTypeImage {
@@ -73,7 +74,7 @@ func (d *zfs) createDataset(dataset string, options ...string) error {
 
 	args = append(args, dataset)
 
-	_, err := shared.RunCommand("zfs", args...)
+	_, err := shared.RunCommandContext(context.TODO(), "zfs", args...)
 	if err != nil {
 		return err
 	}
@@ -82,8 +83,6 @@ func (d *zfs) createDataset(dataset string, options ...string) error {
 }
 
 func (d *zfs) createVolume(dataset string, size int64, options ...string) error {
-	size = d.roundVolumeBlockSizeBytes(size)
-
 	args := []string{"create", "-s", "-V", fmt.Sprintf("%d", size)}
 	for _, option := range options {
 		args = append(args, "-o")
@@ -92,7 +91,7 @@ func (d *zfs) createVolume(dataset string, size int64, options ...string) error
 
 	args = append(args, dataset)
 
-	_, err := shared.RunCommand("zfs", args...)
+	_, err := shared.RunCommandContext(context.TODO(), "zfs", args...)
 	if err != nil {
 		return err
 	}
@@ -101,7 +100,7 @@ func (d *zfs) createVolume(dataset string, size int64, options ...string) error
 }
 
 func (d *zfs) datasetExists(dataset string) (bool, error) {
-	out, err := shared.RunCommand("zfs", "get", "-H", "-o", "name", "name", dataset)
+	out, err := shared.RunCommandContext(context.TODO(), "zfs", "get", "-H", "-o", "name", "name", dataset)
 	if err != nil {
 		return false, nil
 	}
@@ -155,7 +154,7 @@ func (d *zfs) deleteDatasetRecursive(dataset string) error {
 }
 
 func (d *zfs) getClones(dataset string) ([]string, error) {
-	out, err := shared.RunCommand("zfs", "get", "-H", "-p", "-r", "-o", "value", "clones", dataset)
+	out, err := shared.RunCommandContext(context.TODO(), "zfs", "get", "-H", "-p", "-r", "-o", "value", "clones", dataset)
 	if err != nil {
 		return nil, err
 	}
@@ -167,7 +166,7 @@ func (d *zfs) getClones(dataset string) ([]string, error) {
 			continue
 		}
 
-		line = strings.TrimPrefix(line, fmt.Sprintf("%s/", dataset))
+		line = strings.TrimPrefix(line, dataset+"/")
 		clones = append(clones, line)
 	}
 
@@ -175,7 +174,7 @@ func (d *zfs) getClones(dataset string) ([]string, error) {
 }
 
 func (d *zfs) getDatasets(dataset string, types string) ([]string, error) {
-	out, err := shared.RunCommand("zfs", "get", "-H", "-r", "-o", "name", "-t", types, "name", dataset)
+	out, err := shared.RunCommandContext(context.TODO(), "zfs", "get", "-H", "-r", "-o", "name", "-t", types, "name", dataset)
 	if err != nil {
 		return nil, err
 	}
@@ -194,12 +193,50 @@ func (d *zfs) getDatasets(dataset string, types string) ([]string, error) {
 	return children, nil
 }
 
+// filterRedundantOptions filters out options for setting dataset properties that match with the values already set.
+func (d *zfs) filterRedundantOptions(dataset string, options ...string) ([]string, error) {
+	keys := make([]string, 0, len(options))
+	values := make([]string, 0, len(options))
+
+	// Extract keys and values from options.
+	for _, option := range options {
+		key, value, found := strings.Cut(option, "=")
+		if !found {
+			return nil, fmt.Errorf("Wrongly formatted option %q", option)
+		}
+
+		keys = append(keys, key)
+		values = append(values, value)
+	}
+
+	// Get current values for the keys.
+	currentProperties, err := d.getDatasetProperties(dataset, keys...)
+	if err != nil {
+		return nil, err
+	}
+
+	var resultantOptions []string
+
+	// Change property values that are different from the current value.
+	for propertyIndex := range keys {
+		if currentProperties[keys[propertyIndex]] == "posix" && values[propertyIndex] == "posixacl" { // "posixacl" is an alias for "posix"
+			continue
+		}
+
+		if currentProperties[keys[propertyIndex]] != values[propertyIndex] {
+			resultantOptions = append(resultantOptions, options[propertyIndex])
+		}
+	}
+
+	return resultantOptions, nil
+}
+
 func (d *zfs) setDatasetProperties(dataset string, options ...string) error {
 	args := []string{"set"}
 	args = append(args, options...)
 	args = append(args, dataset)
 
-	_, err := shared.RunCommand("zfs", args...)
+	_, err := shared.RunCommandContext(context.TODO(), "zfs", args...)
 	if err != nil {
 		return err
 	}
@@ -236,7 +273,7 @@ func (d *zfs) setBlocksize(vol Volume, size int64) error {
 }
 
 func (d *zfs) getDatasetProperty(dataset string, key string) (string, error) {
-	output, err := shared.RunCommand("zfs", "get", "-H", "-p", "-o", "value", key, dataset)
+	output, err := shared.RunCommandContext(context.TODO(), "zfs", "get", "-H", "-p", "-o", "value", key, dataset)
 	if err != nil {
 		return "", err
 	}
@@ -245,7 +282,7 @@ func (d *zfs) getDatasetProperty(dataset string, key string) (string, error) {
 }
 
 func (d *zfs) getDatasetProperties(dataset string, keys ...string) (map[string]string, error) {
-	output, err := shared.RunCommand("zfs", "get", "-H", "-p", "-o", "property,value", strings.Join(keys, ","), dataset)
+	output, err := shared.RunCommandContext(context.TODO(), "zfs", "get", "-H", "-p", "-o", "property,value", strings.Join(keys, ","), dataset)
 	if err != nil {
 		return nil, err
 	}
@@ -271,7 +308,7 @@ func (d *zfs) getDatasetProperties(dataset string, keys ...string) (map[string]s
 func (d *zfs) version() (string, error) {
 	// This function is only really ever relevant on Ubuntu as the only
 	// distro that ships out of sync tools and kernel modules
-	out, err := shared.RunCommand("dpkg-query", "--showformat=${Version}", "--show", "zfsutils-linux")
+	out, err := shared.RunCommandContext(context.TODO(), "dpkg-query", "--showformat=${Version}", "--show", "zfsutils-linux")
 	if out != "" && err == nil {
 		return strings.TrimSpace(string(out)), nil
 	}
@@ -285,7 +322,7 @@ func (d *zfs) version() (string, error) {
 	}
 
 	// Module information version
-	out, err = shared.RunCommand("modinfo", "-F", "version", "zfs")
+	out, err = shared.RunCommandContext(context.TODO(), "modinfo", "-F", "version", "zfs")
 	if err == nil {
 		return strings.TrimSpace(string(out)), nil
 	}
@@ -486,11 +523,11 @@ func (d *zfs) datasetHeader(vol Volume, snapshots []string) (*ZFSMetaDataHeader,
 }
 
 func (d *zfs) randomVolumeName(vol Volume) string {
-	return fmt.Sprintf("%s_%s", vol.name, uuid.New().String())
+	return vol.name + "_" + uuid.New().String()
 }
 
 func (d *zfs) delegateDataset(vol Volume, pid int) error {
-	_, err := shared.RunCommand("zfs", "zone", fmt.Sprintf("/proc/%d/ns/user", pid), d.dataset(vol, false))
+	_, err := shared.RunCommandContext(context.TODO(), "zfs", "zone", fmt.Sprintf("/proc/%d/ns/user", pid), d.dataset(vol, false))
 	if err != nil {
 		return err
 	}
