@@ -2,26 +2,22 @@ package drivers
 
 import (
 	"context"
-	"errors"
 	"fmt"
 	"io"
 	"io/fs"
 	"os"
 	"os/exec"
 	"path/filepath"
-	"slices"
 	"sort"
 	"strconv"
 	"strings"
 	"time"
-	"unicode"
 
 	"golang.org/x/sys/unix"
 
 	"github.com/canonical/lxd/lxd/idmap"
 	"github.com/canonical/lxd/lxd/locking"
 	"github.com/canonical/lxd/lxd/operations"
-	"github.com/canonical/lxd/lxd/storage/block"
 	"github.com/canonical/lxd/lxd/storage/filesystem"
 	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
@@ -107,13 +103,13 @@ func mountReadOnly(srcPath string, dstPath string) (bool, error) {
 	}
 
 	// Create a mount entry.
-	err := TryMount(context.TODO(), srcPath, dstPath, "none", unix.MS_BIND, "")
+	err := TryMount(srcPath, dstPath, "none", unix.MS_BIND, "")
 	if err != nil {
 		return false, err
 	}
 
 	// Make it read-only.
-	err = TryMount(context.TODO(), "", dstPath, "none", unix.MS_BIND|unix.MS_RDONLY|unix.MS_REMOUNT, "")
+	err = TryMount("", dstPath, "none", unix.MS_BIND|unix.MS_RDONLY|unix.MS_REMOUNT, "")
 	if err != nil {
 		_, _ = forceUnmount(dstPath)
 		return false, err
@@ -166,36 +162,23 @@ func sameMount(srcPath string, dstPath string) bool {
 }
 
 // TryMount tries mounting a filesystem multiple times. This is useful for unreliable backends.
-// By default the mount operation will be retried until the context expires.
-// If no deadline is configured, a default timeout of 10 seconds is used.
-// In case the mount operation doesn't return within the defined deadline,
-// TryMount waits and doesn't return in case the context expired before that.
-func TryMount(ctx context.Context, src string, dst string, fs string, flags uintptr, options string) error {
-	var cancel context.CancelFunc
+func TryMount(src string, dst string, fs string, flags uintptr, options string) error {
+	var err error
 
-	// Set a default timeout in case it's not set by the caller.
-	// Attempt 20 mounts over 10s.
-	_, ok := ctx.Deadline()
-	if !ok {
-		ctx, cancel = context.WithTimeout(ctx, 10*time.Second)
-		defer cancel()
-	}
-
-mountLoop:
-	for {
-		select {
-		case <-ctx.Done():
-			return fmt.Errorf("Failed to mount %q on %q using %q: %w", src, dst, fs, ctx.Err())
-		default:
-			err := unix.Mount(src, dst, fs, flags, options)
-			if err == nil {
-				break mountLoop
-			}
+	// Attempt 20 mounts over 10s
+	for i := 0; i < 20; i++ {
+		err = unix.Mount(src, dst, fs, flags, options)
+		if err == nil {
+			break
 		}
 
 		time.Sleep(500 * time.Millisecond)
 	}
 
+	if err != nil {
+		return fmt.Errorf("Failed to mount %q on %q using %q: %w", src, dst, fs, err)
+	}
+
 	return nil
 }
 
@@ -203,7 +186,7 @@ mountLoop:
 func TryUnmount(path string, flags int) error {
 	var err error
 
-	for i := range 20 {
+	for i := 0; i < 20; i++ {
 		err = unix.Unmount(path, flags)
 		if err == nil {
 			break
@@ -240,7 +223,7 @@ func tryExists(ctx context.Context, path string) bool {
 // fsUUID returns the filesystem UUID for the given block path.
 // error is returned if the given block device exists but has no UUID.
 func fsUUID(path string) (string, error) {
-	val, err := block.DiskFSUUID(path)
+	val, err := shared.RunCommandContext(context.TODO(), "blkid", "-s", "UUID", "-o", "value", path)
 	if err != nil {
 		return "", err
 	}
@@ -251,7 +234,7 @@ func fsUUID(path string) (string, error) {
 		return "", fmt.Errorf("No UUID for device %q", path)
 	}
 
-	return val, nil
+	return strings.TrimSpace(val), nil
 }
 
 // fsProbe returns the filesystem type for the given block path.
@@ -356,7 +339,7 @@ func ensureSparseFile(filePath string, sizeBytes int64) error {
 // instead return ErrNotSupported.
 func ensureVolumeBlockFile(vol Volume, path string, sizeBytes int64, allowUnsafeResize bool, unsupportedResizeTypes ...VolumeType) (bool, error) {
 	if sizeBytes <= 0 {
-		return false, errors.New("Size cannot be zero")
+		return false, fmt.Errorf("Size cannot be zero")
 	}
 
 	// Get rounded block size to avoid QEMU boundary issues.
@@ -379,8 +362,10 @@ func ensureVolumeBlockFile(vol Volume, path string, sizeBytes int64, allowUnsafe
 			// Reject if would try and resize a volume type that is not supported.
 			// This needs to come before the ErrCannotBeShrunk check below so that any resize attempt
 			// is blocked with ErrNotSupported error.
-			if slices.Contains(unsupportedResizeTypes, vol.volType) {
-				return false, ErrNotSupported
+			for _, unsupportedType := range unsupportedResizeTypes {
+				if unsupportedType == vol.volType {
+					return false, ErrNotSupported
+				}
 			}
 
 			if sizeBytes < oldSizeBytes {
@@ -451,7 +436,7 @@ func filesystemTypeCanBeShrunk(fsType string) bool {
 		fsType = DefaultFilesystem
 	}
 
-	if slices.Contains([]string{"ext4", "btrfs"}, fsType) {
+	if shared.ValueInSlice(fsType, []string{"ext4", "btrfs"}) {
 		return true
 	}
 
@@ -580,7 +565,7 @@ func regenerateFilesystemUUID(fsType string, devPath string) error {
 		return regenerateFilesystemXFSUUID(devPath)
 	}
 
-	return errors.New("Filesystem not supported")
+	return fmt.Errorf("Filesystem not supported")
 }
 
 // regenerateFilesystemBTRFSUUID changes the BTRFS filesystem UUID to a new randomly generated one.
@@ -592,10 +577,7 @@ func regenerateFilesystemBTRFSUUID(devPath string) error {
 		return err
 	}
 
-	// `-m` modifies the metadata_uuid which is much faster than `-u` that rewrites all metadata blocks.
-	// The resulting filesystem needs kernel 5.0+ to be mounted or running `btrfstune -u` to regain compat
-	// with older kernels.
-	_, err = shared.RunCommandContext(context.TODO(), "btrfstune", "-f", "-m", devPath)
+	_, err = shared.RunCommandContext(context.TODO(), "btrfstune", "-f", "-u", devPath)
 	if err != nil {
 		return err
 	}
@@ -628,41 +610,6 @@ func regenerateFilesystemXFSUUID(devPath string) error {
 	return nil
 }
 
-// addNoRecoveryMountOption adds the `norecovery` mount option to the provided mount options.
-// btrfs/ext4/xfs can do some form of recovery if the filesystem was not cleanly unmounted.
-// To prevent this kind of write access, we mount the filesystem with the ro,norecovery mount options.
-// For ext4, norecovery is a synonym for noload, which prevents the journal from being loaded on mounting.
-func addNoRecoveryMountOption(mountOptions string, filesystem string) string {
-	// If the filesystem is not one of the supported types, return unchanged.
-	if !slices.Contains(blockBackedAllowedFilesystems, filesystem) {
-		return mountOptions
-	}
-
-	// Add norecovery as the first mount options.
-	if mountOptions == "" {
-		return "norecovery"
-	}
-
-	// Already contains norecovery, so return unchanged.
-	options := strings.Split(mountOptions, ",")
-
-	// For ext4, noload is a synonym for norecovery.
-	if filesystem == "ext4" {
-		for i, option := range options {
-			if option == "noload" {
-				options[i] = "norecovery"
-				break
-			}
-		}
-	}
-
-	if !slices.Contains(options, "norecovery") {
-		options = append(options, "norecovery")
-	}
-
-	return strings.Join(options, ",")
-}
-
 // copyDevice copies one device path to another using dd running at low priority.
 // It expects outputPath to exist already, so will not create it.
 func copyDevice(inputPath string, outputPath string) error {
@@ -825,7 +772,7 @@ func loopFileSizeDefault() (uint64, error) {
 		return gibAvailable, nil // Need at least 5GiB free.
 	}
 
-	return 0, errors.New("Insufficient free space to create default sized 5GiB pool")
+	return 0, fmt.Errorf("Insufficient free space to create default sized 5GiB pool")
 }
 
 // loopFileSetup sets up a loop device for the provided sourcePath.
@@ -836,7 +783,7 @@ func loopDeviceSetup(sourcePath string) (string, error) {
 		return strings.TrimSpace(out), nil
 	}
 
-	if !strings.Contains(err.Error(), "direct io") && !strings.Contains(err.Error(), "Invalid argument") {
+	if !(strings.Contains(err.Error(), "direct io") || strings.Contains(err.Error(), "Invalid argument")) {
 		return "", err
 	}
 
@@ -932,59 +879,5 @@ func remoteVolumeMapLock(connectorName string, driverName string) (locking.Unloc
 	l.Debug("Acquiring lock for remote volume map")
 	defer l.Debug("Lock acquired for remote volume map")
 
-	return locking.Lock(context.TODO(), "RemoteVolumeMap_"+connectorName+"_"+driverName)
-}
-
-// ValidPoolName validates a pool name.
-func ValidPoolName(poolName string) error {
-	if poolName == "" {
-		return errors.New("Cannot be empty")
-	}
-
-	if strings.HasPrefix(poolName, ".") {
-		return errors.New("Cannot start with a dot")
-	}
-
-	if strings.HasPrefix(poolName, "-") {
-		return errors.New("Cannot start with a hyphen")
-	}
-
-	if strings.Contains(poolName, shared.SnapshotDelimiter) {
-		return errors.New("Cannot contain slashes")
-	}
-
-	for _, r := range poolName {
-		if unicode.IsSpace(r) {
-			return errors.New("Cannot contain white space")
-		}
-	}
-
-	return nil
-}
-
-// ValidVolumeName validates a volume name.
-func ValidVolumeName(volumeName string) error {
-	if volumeName == "" {
-		return errors.New("Cannot be empty")
-	}
-
-	if volumeName == ".." {
-		return errors.New(`Cannot be ".."`)
-	}
-
-	if strings.Contains(volumeName, "\\") {
-		return errors.New("Cannot contain backslashes")
-	}
-
-	if strings.Contains(volumeName, shared.SnapshotDelimiter) {
-		return errors.New("Cannot contain slashes")
-	}
-
-	for _, r := range volumeName {
-		if unicode.IsSpace(r) {
-			return errors.New("Cannot contain white space")
-		}
-	}
-
-	return nil
+	return locking.Lock(context.TODO(), fmt.Sprintf("RemoteVolumeMap_%s_%s", connectorName, driverName))
 }
