@@ -129,7 +129,7 @@ func IsMountPoint(path string) bool {
 
 	// Btrfs annoyingly uses a different Dev id for different subvolumes on the same mount.
 	// So for btrfs, we require a matching mount entry in mountinfo.
-	fs, _ := Detect(path)
+	fs, err := Detect(path)
 	if err == nil && fs == "btrfs" {
 		return false
 	}
@@ -247,7 +247,7 @@ func GetMountinfo(path string) ([]string, error) {
 			continue
 		}
 
-		if tokens[0] == fmt.Sprintf("%d", stat.Mnt_id) {
+		if tokens[0] == fmt.Sprint(stat.Mnt_id) {
 			return tokens, nil
 		}
 	}
