@@ -4,14 +4,14 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/lxc/lxd/lxd/db"
-	"github.com/lxc/lxd/lxd/instance"
-	"github.com/lxc/lxd/lxd/project"
-	"github.com/lxc/lxd/lxd/response"
-	"github.com/lxc/lxd/lxd/state"
-	"github.com/lxc/lxd/lxd/storage/drivers"
-	"github.com/lxc/lxd/shared/api"
-	"github.com/lxc/lxd/shared/logger"
+	"github.com/canonical/lxd/lxd/db"
+	"github.com/canonical/lxd/lxd/instance"
+	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/response"
+	"github.com/canonical/lxd/lxd/state"
+	"github.com/canonical/lxd/lxd/storage/drivers"
+	"github.com/canonical/lxd/shared/api"
+	"github.com/canonical/lxd/shared/logger"
 )
 
 // PoolIDTemporary is used to indicate a temporary pool instance that is not in the database.
@@ -33,7 +33,7 @@ func volIDFuncMake(state *state.State, poolID int64) func(volType drivers.Volume
 		// format <project>_<volume>. However not all volume types currently use this
 		// encoding format, so if there is no underscore in the volume name then we assume
 		// the project is default.
-		projectName := project.Default
+		projectName := api.ProjectDefaultName
 
 		// Currently only Containers, VMs and custom volumes support project level volumes.
 		// This means that other volume types may have underscores in their names that don't
@@ -75,7 +75,7 @@ func NewTemporary(state *state.State, info *api.StoragePool) (Pool, error) {
 		pool := mockBackend{}
 		pool.name = info.Name
 		pool.state = state
-		pool.logger = logger.AddContext(logger.Log, logger.Ctx{"driver": "mock", "pool": pool.name})
+		pool.logger = logger.AddContext(logger.Ctx{"driver": "mock", "pool": pool.name})
 		driver, err := drivers.Load(state, "mock", "", nil, pool.logger, nil, nil)
 		if err != nil {
 			return nil, err
@@ -93,7 +93,7 @@ func NewTemporary(state *state.State, info *api.StoragePool) (Pool, error) {
 		info.Config = map[string]string{}
 	}
 
-	logger := logger.AddContext(logger.Log, logger.Ctx{"driver": info.Driver, "pool": info.Name})
+	logger := logger.AddContext(logger.Ctx{"driver": info.Driver, "pool": info.Name})
 
 	// Load the storage driver.
 	driver, err := drivers.Load(state, info.Driver, info.Name, info.Config, logger, volIDFuncMake(state, poolID), commonRules())
@@ -116,7 +116,7 @@ func NewTemporary(state *state.State, info *api.StoragePool) (Pool, error) {
 
 // LoadByType loads a network by driver type.
 func LoadByType(state *state.State, driverType string) (Type, error) {
-	logger := logger.AddContext(logger.Log, logger.Ctx{"driver": driverType})
+	logger := logger.AddContext(logger.Ctx{"driver": driverType})
 
 	driver, err := drivers.Load(state, driverType, "", nil, logger, nil, commonRules())
 	if err != nil {
@@ -133,16 +133,45 @@ func LoadByType(state *state.State, driverType string) (Type, error) {
 	return &pool, nil
 }
 
+// LoadByRecord instantiates a pool from its record and returns a Pool interface.
+// If the pool's driver is not recognised then drivers.ErrUnknownDriver is returned.
+func LoadByRecord(s *state.State, poolID int64, poolInfo api.StoragePool, poolMembers map[int64]db.StoragePoolNode) (Pool, error) {
+	// Ensure a config map exists.
+	if poolInfo.Config == nil {
+		poolInfo.Config = map[string]string{}
+	}
+
+	logger := logger.AddContext(logger.Ctx{"driver": poolInfo.Driver, "pool": poolInfo.Name})
+
+	// Load the storage driver.
+	driver, err := drivers.Load(s, poolInfo.Driver, poolInfo.Name, poolInfo.Config, logger, volIDFuncMake(s, poolID), commonRules())
+	if err != nil {
+		return nil, err
+	}
+
+	// Setup the pool struct.
+	pool := lxdBackend{}
+	pool.driver = driver
+	pool.id = poolID
+	pool.db = poolInfo
+	pool.name = poolInfo.Name
+	pool.state = s
+	pool.logger = logger
+	pool.nodes = poolMembers
+
+	return &pool, nil
+}
+
 // LoadByName retrieves the pool from the database by its name and returns a Pool interface.
 // If the pool's driver is not recognised then drivers.ErrUnknownDriver is returned.
-func LoadByName(state *state.State, name string) (Pool, error) {
+func LoadByName(s *state.State, name string) (Pool, error) {
 	// Handle mock requests.
-	if state.OS.MockMode {
+	if s.OS.MockMode {
 		pool := mockBackend{}
 		pool.name = name
-		pool.state = state
-		pool.logger = logger.AddContext(logger.Log, logger.Ctx{"driver": "mock", "pool": pool.name})
-		driver, err := drivers.Load(state, "mock", "", nil, pool.logger, nil, nil)
+		pool.state = s
+		pool.logger = logger.AddContext(logger.Ctx{"driver": "mock", "pool": pool.name})
+		driver, err := drivers.Load(s, "mock", "", nil, pool.logger, nil, nil)
 		if err != nil {
 			return nil, err
 		}
@@ -152,36 +181,23 @@ func LoadByName(state *state.State, name string) (Pool, error) {
 		return &pool, nil
 	}
 
-	// Load the database record.
-	poolID, dbPool, poolNodes, err := state.DB.Cluster.GetStoragePoolInAnyState(name)
+	var poolID int64
+	var dbPool *api.StoragePool
+	var poolNodes map[int64]db.StoragePoolNode
+
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		// Load the database record.
+		poolID, dbPool, poolNodes, err = tx.GetStoragePoolInAnyState(ctx, name)
+
+		return err
+	})
 	if err != nil {
 		return nil, err
 	}
 
-	// Ensure a config map exists.
-	if dbPool.Config == nil {
-		dbPool.Config = map[string]string{}
-	}
-
-	logger := logger.AddContext(logger.Log, logger.Ctx{"driver": dbPool.Driver, "pool": dbPool.Name})
-
-	// Load the storage driver.
-	driver, err := drivers.Load(state, dbPool.Driver, dbPool.Name, dbPool.Config, logger, volIDFuncMake(state, poolID), commonRules())
-	if err != nil {
-		return nil, err
-	}
-
-	// Setup the pool struct.
-	pool := lxdBackend{}
-	pool.driver = driver
-	pool.id = poolID
-	pool.db = *dbPool
-	pool.name = dbPool.Name
-	pool.state = state
-	pool.logger = logger
-	pool.nodes = poolNodes
-
-	return &pool, nil
+	return LoadByRecord(s, poolID, *dbPool, poolNodes)
 }
 
 // LoadByInstance retrieves the pool from the database using the instance's pool.
@@ -240,8 +256,16 @@ func Patch(s *state.State, patchName string) error {
 
 	unavailablePoolsMu.Unlock()
 
-	// Load all the pools.
-	pools, err := s.DB.Cluster.GetStoragePoolNames()
+	var pools []string
+
+	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		// Load all the pools.
+		pools, err = tx.GetStoragePoolNames(ctx)
+
+		return err
+	})
 	if err != nil {
 		if response.IsNotFoundError(err) {
 			return nil
