@@ -3,24 +3,18 @@ package storage
 import (
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"os"
 	"path/filepath"
-	"slices"
-	"strconv"
 	"strings"
 	"time"
 
-	"github.com/flosch/pongo2"
 	"golang.org/x/sys/unix"
 
 	"github.com/canonical/lxd/lxd/apparmor"
 	"github.com/canonical/lxd/lxd/archive"
 	"github.com/canonical/lxd/lxd/db"
 	"github.com/canonical/lxd/lxd/db/cluster"
-	deviceConfig "github.com/canonical/lxd/lxd/device/config"
-	"github.com/canonical/lxd/lxd/device/filters"
 	"github.com/canonical/lxd/lxd/instance"
 	"github.com/canonical/lxd/lxd/instance/instancetype"
 	"github.com/canonical/lxd/lxd/migration"
@@ -40,8 +34,6 @@ import (
 	"github.com/canonical/lxd/shared/validate"
 )
 
-const defaultSnapshotPattern = "snap%d"
-
 // ConfigDiff returns a diff of the provided configs. Additionally, it returns whether or not
 // only user properties have been changed.
 func ConfigDiff(oldConfig map[string]string, newConfig map[string]string) ([]string, bool) {
@@ -53,7 +45,7 @@ func ConfigDiff(oldConfig map[string]string, newConfig map[string]string) ([]str
 				userOnly = false
 			}
 
-			if !slices.Contains(changedConfig, key) {
+			if !shared.ValueInSlice(key, changedConfig) {
 				changedConfig = append(changedConfig, key)
 			}
 		}
@@ -65,7 +57,7 @@ func ConfigDiff(oldConfig map[string]string, newConfig map[string]string) ([]str
 				userOnly = false
 			}
 
-			if !slices.Contains(changedConfig, key) {
+			if !shared.ValueInSlice(key, changedConfig) {
 				changedConfig = append(changedConfig, key)
 			}
 		}
@@ -79,8 +71,24 @@ func ConfigDiff(oldConfig map[string]string, newConfig map[string]string) ([]str
 	return changedConfig, userOnly
 }
 
+// VolumeTypeNameToDBType converts a volume type string to internal volume type DB code.
+func VolumeTypeNameToDBType(volumeTypeName string) (int, error) {
+	switch volumeTypeName {
+	case cluster.StoragePoolVolumeTypeNameContainer:
+		return cluster.StoragePoolVolumeTypeContainer, nil
+	case cluster.StoragePoolVolumeTypeNameVM:
+		return cluster.StoragePoolVolumeTypeVM, nil
+	case cluster.StoragePoolVolumeTypeNameImage:
+		return cluster.StoragePoolVolumeTypeImage, nil
+	case cluster.StoragePoolVolumeTypeNameCustom:
+		return cluster.StoragePoolVolumeTypeCustom, nil
+	}
+
+	return -1, fmt.Errorf("Invalid storage volume type name")
+}
+
 // VolumeTypeToDBType converts volume type to internal volume type DB code.
-func VolumeTypeToDBType(volType drivers.VolumeType) (cluster.StoragePoolVolumeType, error) {
+func VolumeTypeToDBType(volType drivers.VolumeType) (int, error) {
 	switch volType {
 	case drivers.VolumeTypeContainer:
 		return cluster.StoragePoolVolumeTypeContainer, nil
@@ -96,19 +104,19 @@ func VolumeTypeToDBType(volType drivers.VolumeType) (cluster.StoragePoolVolumeTy
 }
 
 // VolumeDBTypeToType converts internal volume type DB code to storage driver volume type.
-func VolumeDBTypeToType(volDBType cluster.StoragePoolVolumeType) drivers.VolumeType {
+func VolumeDBTypeToType(volDBType int) (drivers.VolumeType, error) {
 	switch volDBType {
 	case cluster.StoragePoolVolumeTypeContainer:
-		return drivers.VolumeTypeContainer
+		return drivers.VolumeTypeContainer, nil
 	case cluster.StoragePoolVolumeTypeVM:
-		return drivers.VolumeTypeVM
+		return drivers.VolumeTypeVM, nil
 	case cluster.StoragePoolVolumeTypeImage:
-		return drivers.VolumeTypeImage
+		return drivers.VolumeTypeImage, nil
 	case cluster.StoragePoolVolumeTypeCustom:
-		return drivers.VolumeTypeCustom
+		return drivers.VolumeTypeCustom, nil
 	}
 
-	return drivers.VolumeTypeCustom
+	return "", fmt.Errorf("Invalid storage volume DB type: %d", volDBType)
 }
 
 // InstanceTypeToVolumeType converts instance type to storage driver volume type.
@@ -120,7 +128,7 @@ func InstanceTypeToVolumeType(instType instancetype.Type) (drivers.VolumeType, e
 		return drivers.VolumeTypeVM, nil
 	}
 
-	return "", errors.New("Invalid instance type")
+	return "", fmt.Errorf("Invalid instance type")
 }
 
 // VolumeTypeToAPIInstanceType converts storage driver volume type to API instance type type.
@@ -132,11 +140,11 @@ func VolumeTypeToAPIInstanceType(volType drivers.VolumeType) (api.InstanceType,
 		return api.InstanceTypeVM, nil
 	}
 
-	return api.InstanceTypeAny, errors.New("Volume type doesn't have equivalent instance type")
+	return api.InstanceTypeAny, fmt.Errorf("Volume type doesn't have equivalent instance type")
 }
 
 // VolumeContentTypeToDBContentType converts volume type to internal code.
-func VolumeContentTypeToDBContentType(contentType drivers.ContentType) (cluster.StoragePoolVolumeContentType, error) {
+func VolumeContentTypeToDBContentType(contentType drivers.ContentType) (int, error) {
 	switch contentType {
 	case drivers.ContentTypeBlock:
 		return cluster.StoragePoolVolumeContentTypeBlock, nil
@@ -146,28 +154,42 @@ func VolumeContentTypeToDBContentType(contentType drivers.ContentType) (cluster.
 		return cluster.StoragePoolVolumeContentTypeISO, nil
 	}
 
-	return -1, errors.New("Invalid volume content type")
+	return -1, fmt.Errorf("Invalid volume content type")
 }
 
 // VolumeDBContentTypeToContentType converts internal content type DB code to driver representation.
-func VolumeDBContentTypeToContentType(volDBType cluster.StoragePoolVolumeContentType) drivers.ContentType {
+func VolumeDBContentTypeToContentType(volDBType int) (drivers.ContentType, error) {
 	switch volDBType {
 	case cluster.StoragePoolVolumeContentTypeBlock:
-		return drivers.ContentTypeBlock
+		return drivers.ContentTypeBlock, nil
 	case cluster.StoragePoolVolumeContentTypeFS:
-		return drivers.ContentTypeFS
+		return drivers.ContentTypeFS, nil
 	case cluster.StoragePoolVolumeContentTypeISO:
-		return drivers.ContentTypeISO
+		return drivers.ContentTypeISO, nil
 	}
 
-	return drivers.ContentTypeFS
+	return "", fmt.Errorf("Invalid volume content type")
+}
+
+// VolumeContentTypeNameToContentType converts volume content type string internal code.
+func VolumeContentTypeNameToContentType(contentTypeName string) (int, error) {
+	switch contentTypeName {
+	case cluster.StoragePoolVolumeContentTypeNameFS:
+		return cluster.StoragePoolVolumeContentTypeFS, nil
+	case cluster.StoragePoolVolumeContentTypeNameBlock:
+		return cluster.StoragePoolVolumeContentTypeBlock, nil
+	case cluster.StoragePoolVolumeContentTypeNameISO:
+		return cluster.StoragePoolVolumeContentTypeISO, nil
+	}
+
+	return -1, fmt.Errorf("Invalid volume content type name")
 }
 
 // VolumeDBGet loads a volume from the database.
 func VolumeDBGet(pool Pool, projectName string, volumeName string, volumeType drivers.VolumeType) (*db.StorageVolume, error) {
 	p, ok := pool.(*lxdBackend)
 	if !ok {
-		return nil, errors.New("Pool is not a lxdBackend")
+		return nil, fmt.Errorf("Pool is not a lxdBackend")
 	}
 
 	volDBType, err := VolumeTypeToDBType(volumeType)
@@ -202,12 +224,12 @@ func VolumeDBGet(pool Pool, projectName string, volumeName string, volumeType dr
 func VolumeDBCreate(pool Pool, projectName string, volumeName string, volumeDescription string, volumeType drivers.VolumeType, snapshot bool, volumeConfig map[string]string, creationDate time.Time, expiryDate time.Time, contentType drivers.ContentType, removeUnknownKeys bool, hasSource bool) error {
 	p, ok := pool.(*lxdBackend)
 	if !ok {
-		return errors.New("Pool is not a lxdBackend")
+		return fmt.Errorf("Pool is not a lxdBackend")
 	}
 
 	// Prevent using this function to create storage volume bucket records.
 	if volumeType == drivers.VolumeTypeBucket {
-		return errors.New("Cannot store volume using bucket type")
+		return fmt.Errorf("Cannot store volume using bucket type")
 	}
 
 	// If the volumeType represents an instance type then check that the volumeConfig doesn't contain any of
@@ -237,7 +259,11 @@ func VolumeDBCreate(pool Pool, projectName string, volumeName string, volumeDesc
 		volumeConfig = map[string]string{}
 	}
 
-	volType := VolumeDBTypeToType(volDBType)
+	volType, err := VolumeDBTypeToType(volDBType)
+	if err != nil {
+		return err
+	}
+
 	vol := drivers.NewVolume(pool.Driver(), pool.Name(), volType, contentType, volumeName, volumeConfig, pool.Driver().Config())
 
 	// Set source indicator.
@@ -276,7 +302,7 @@ func VolumeDBCreate(pool Pool, projectName string, volumeName string, volumeDesc
 func VolumeDBDelete(pool Pool, projectName string, volumeName string, volumeType drivers.VolumeType) error {
 	p, ok := pool.(*lxdBackend)
 	if !ok {
-		return errors.New("Pool is not a lxdBackend")
+		return fmt.Errorf("Pool is not a lxdBackend")
 	}
 
 	// Convert the volume type to our internal integer representation.
@@ -299,7 +325,7 @@ func VolumeDBDelete(pool Pool, projectName string, volumeName string, volumeType
 func VolumeDBSnapshotsGet(pool Pool, projectName string, volume string, volumeType drivers.VolumeType) ([]db.StorageVolumeArgs, error) {
 	p, ok := pool.(*lxdBackend)
 	if !ok {
-		return nil, errors.New("Pool is not a lxdBackend")
+		return nil, fmt.Errorf("Pool is not a lxdBackend")
 	}
 
 	volDBType, err := VolumeTypeToDBType(volumeType)
@@ -325,7 +351,7 @@ func VolumeDBSnapshotsGet(pool Pool, projectName string, volume string, volumeTy
 func BucketDBGet(pool Pool, projectName string, bucketName string, memberSpecific bool) (*db.StorageBucket, error) {
 	p, ok := pool.(*lxdBackend)
 	if !ok {
-		return nil, errors.New("Pool is not a lxdBackend")
+		return nil, fmt.Errorf("Pool is not a lxdBackend")
 	}
 
 	var err error
@@ -357,7 +383,7 @@ func BucketDBGet(pool Pool, projectName string, bucketName string, memberSpecifi
 func BucketDBCreate(ctx context.Context, pool Pool, projectName string, memberSpecific bool, bucket *api.StorageBucketsPost) (int64, error) {
 	p, ok := pool.(*lxdBackend)
 	if !ok {
-		return -1, errors.New("Pool is not a lxdBackend")
+		return -1, fmt.Errorf("Pool is not a lxdBackend")
 	}
 
 	// Make sure that we don't pass a nil to the next function.
@@ -405,7 +431,7 @@ func BucketDBCreate(ctx context.Context, pool Pool, projectName string, memberSp
 func BucketDBDelete(ctx context.Context, pool Pool, bucketID int64) error {
 	p, ok := pool.(*lxdBackend)
 	if !ok {
-		return errors.New("Pool is not a lxdBackend")
+		return fmt.Errorf("Pool is not a lxdBackend")
 	}
 
 	err := p.state.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
@@ -684,7 +710,7 @@ func validateVolumeCommonRules(vol drivers.Volume) map[string]func(string) error
 // VM Format A: Separate metadata tarball and root qcow2 file.
 //   - Unpack metadata tarball into mountPath.
 //   - Check rootBlockPath is a file and convert qcow2 file into raw format in rootBlockPath.
-func ImageUnpack(s *state.State, imageFile string, vol drivers.Volume, destBlockFile string, allowUnsafeResize bool, tracker *ioprogress.ProgressTracker) (int64, error) {
+func ImageUnpack(imageFile string, vol drivers.Volume, destBlockFile string, sysOS *sys.OS, allowUnsafeResize bool, tracker *ioprogress.ProgressTracker) (int64, error) {
 	l := logger.Log.AddContext(logger.Ctx{"imageFile": imageFile, "volName": vol.Name()})
 	l.Info("Image unpack started")
 	defer l.Info("Image unpack stopped")
@@ -698,7 +724,7 @@ func ImageUnpack(s *state.State, imageFile string, vol drivers.Volume, destBlock
 		rootfsPath := filepath.Join(destPath, "rootfs")
 
 		// Unpack the main image file.
-		err := archive.Unpack(s, imageFile, destPath, vol.IsBlockBacked(), tracker)
+		err := archive.Unpack(imageFile, destPath, vol.IsBlockBacked(), sysOS, tracker)
 		if err != nil {
 			return -1, err
 		}
@@ -707,10 +733,10 @@ func ImageUnpack(s *state.State, imageFile string, vol drivers.Volume, destBlock
 		if shared.PathExists(imageRootfsFile) {
 			err = os.MkdirAll(rootfsPath, 0755)
 			if err != nil {
-				return -1, errors.New("Error creating rootfs directory")
+				return -1, fmt.Errorf("Error creating rootfs directory")
 			}
 
-			err = archive.Unpack(s, imageRootfsFile, rootfsPath, vol.IsBlockBacked(), tracker)
+			err = archive.Unpack(imageRootfsFile, rootfsPath, vol.IsBlockBacked(), sysOS, tracker)
 			if err != nil {
 				return -1, err
 			}
@@ -741,7 +767,7 @@ func ImageUnpack(s *state.State, imageFile string, vol drivers.Volume, destBlock
 	// convertBlockImage converts the qcow2 block image file into a raw block device. If needed it will attempt
 	// to enlarge the destination volume to accommodate the unpacked qcow2 image file.
 	convertBlockImage := func(imgPath string, dstPath string, tracker *ioprogress.ProgressTracker) (int64, error) {
-		imgFormat, imgVirtualSize, err := qemuImageInfo(s.OS, imgPath, tracker)
+		imgFormat, imgVirtualSize, err := qemuImageInfo(sysOS, imgPath, tracker)
 		if err != nil {
 			return -1, err
 		}
@@ -754,7 +780,7 @@ func ImageUnpack(s *state.State, imageFile string, vol drivers.Volume, destBlock
 		// Check whether image is allowed to be unpacked into pool volume. Create a partial image volume
 		// struct and then use it to check that target volume size can be set as needed.
 		imgVolConfig := map[string]string{
-			"volatile.rootfs.size": strconv.FormatInt(imgVirtualSize, 10),
+			"volatile.rootfs.size": fmt.Sprint(imgVirtualSize),
 		}
 
 		imgVol := drivers.NewVolume(nil, "", drivers.VolumeTypeImage, drivers.ContentTypeBlock, "", imgVolConfig, nil)
@@ -814,7 +840,7 @@ func ImageUnpack(s *state.State, imageFile string, vol drivers.Volume, destBlock
 
 		cmd = append(cmd, imgPath, dstPath)
 
-		_, err = apparmor.QemuImg(s.OS, cmd, imgPath, dstPath, tracker)
+		_, err = apparmor.QemuImg(sysOS, cmd, imgPath, dstPath, tracker)
 		if err != nil {
 			return -1, fmt.Errorf("Failed converting image to raw at %q: %w", dstPath, err)
 		}
@@ -826,7 +852,7 @@ func ImageUnpack(s *state.State, imageFile string, vol drivers.Volume, destBlock
 
 	if shared.PathExists(imageRootfsFile) {
 		// Unpack the main image file.
-		err := archive.Unpack(s, imageFile, destPath, vol.IsBlockBacked(), tracker)
+		err := archive.Unpack(imageFile, destPath, vol.IsBlockBacked(), sysOS, tracker)
 		if err != nil {
 			return -1, err
 		}
@@ -838,7 +864,7 @@ func ImageUnpack(s *state.State, imageFile string, vol drivers.Volume, destBlock
 		}
 	} else {
 		// Dealing with unified tarballs require an initial unpack to a temporary directory.
-		tempDir, err := os.MkdirTemp(s.ImagesStoragePath(), "lxd_image_unpack_")
+		tempDir, err := os.MkdirTemp(shared.VarPath("images"), "lxd_image_unpack_")
 		if err != nil {
 			return -1, err
 		}
@@ -846,7 +872,7 @@ func ImageUnpack(s *state.State, imageFile string, vol drivers.Volume, destBlock
 		defer func() { _ = os.RemoveAll(tempDir) }()
 
 		// Unpack the whole image.
-		err = archive.Unpack(s, imageFile, tempDir, vol.IsBlockBacked(), tracker)
+		err = archive.Unpack(imageFile, tempDir, vol.IsBlockBacked(), sysOS, tracker)
 		if err != nil {
 			return -1, err
 		}
@@ -916,11 +942,9 @@ func InstanceContentType(inst instance.Instance) drivers.ContentType {
 	return contentType
 }
 
-// volumeIsUsedByDevice; true when vol is referred to by dev, assumes the volume
-// belongs to the correct project to be referenced by the instance.
-// instanceType=instanceType.Any indicates the device is used by a profile.
-// The instanceName argument is only used if instanceType != instanceType.Any.
-func volumeIsUsedByDevice(vol api.StorageVolume, instanceType instancetype.Type, instanceName string, dev map[string]string) (bool, error) {
+// volumeIsUsedByDevice; true when vol is referred to by dev
+// inst is the instance dev belongs to, or nil if dev is part of a profile.
+func volumeIsUsedByDevice(vol api.StorageVolume, inst *db.InstanceArgs, dev map[string]string) (bool, error) {
 	if dev["type"] != cluster.TypeDisk.String() {
 		return false, nil
 	}
@@ -929,8 +953,8 @@ func volumeIsUsedByDevice(vol api.StorageVolume, instanceType instancetype.Type,
 		return false, nil
 	}
 
-	if instanceType != instancetype.Any && filters.IsRootDisk(dev) {
-		rootVolumeType, err := InstanceTypeToVolumeType(instanceType)
+	if inst != nil && instancetype.IsRootDiskDevice(dev) {
+		rootVolumeType, err := InstanceTypeToVolumeType(inst.Type)
 		if err != nil {
 			return false, err
 		}
@@ -940,7 +964,7 @@ func volumeIsUsedByDevice(vol api.StorageVolume, instanceType instancetype.Type,
 			return false, err
 		}
 
-		if instanceName == vol.Name && rootVolumeDBType.String() == vol.Type {
+		if inst.Name == vol.Name && cluster.StoragePoolVolumeTypeNames[rootVolumeDBType] == vol.Type {
 			return true, nil
 		}
 	}
@@ -975,7 +999,7 @@ func volumeIsUsedByDevice(vol api.StorageVolume, instanceType instancetype.Type,
 // the volume.
 func VolumeUsedByProfileDevices(s *state.State, poolName string, projectName string, vol *api.StorageVolume, profileFunc func(profileID int64, profile api.Profile, project api.Project, usedByDevices []string) error) error {
 	// Convert the volume type name to our internal integer representation.
-	volumeType, err := cluster.StoragePoolVolumeTypeFromName(vol.Type)
+	volumeType, err := VolumeTypeNameToDBType(vol.Type)
 	if err != nil {
 		return err
 	}
@@ -1054,7 +1078,7 @@ func VolumeUsedByProfileDevices(s *state.State, poolName string, projectName str
 		// Iterate through each of the profiles's devices, looking for disks in the same pool as volume.
 		// Then try and match the volume name against the profile device's "source" property.
 		for name, dev := range profile.Devices {
-			usesVol, err := volumeIsUsedByDevice(*vol, instancetype.Any, "", dev)
+			usesVol, err := volumeIsUsedByDevice(*vol, nil, dev)
 			if err != nil {
 				return err
 			}
@@ -1082,7 +1106,7 @@ func VolumeUsedByProfileDevices(s *state.State, poolName string, projectName str
 // names that are using the volume.
 func VolumeUsedByInstanceDevices(s *state.State, poolName string, projectName string, vol *api.StorageVolume, expandDevices bool, instanceFunc func(inst db.InstanceArgs, project api.Project, usedByDevices []string) error) error {
 	// Convert the volume type name to our internal integer representation.
-	volumeType, err := cluster.StoragePoolVolumeTypeFromName(vol.Type)
+	volumeType, err := VolumeTypeNameToDBType(vol.Type)
 	if err != nil {
 		return err
 	}
@@ -1117,7 +1141,7 @@ func VolumeUsedByInstanceDevices(s *state.State, poolName string, projectName st
 			// Iterate through each of the instance's devices, looking for disks in the same pool as volume.
 			// Then try and match the volume name against the instance device's "source" property.
 			for devName, dev := range devices {
-				usesVol, err := volumeIsUsedByDevice(*vol, inst.Type, inst.Name, dev)
+				usesVol, err := volumeIsUsedByDevice(*vol, &inst, dev)
 				if err != nil {
 					return err
 				}
@@ -1275,19 +1299,13 @@ func InstanceDiskBlockSize(pool Pool, inst instance.Instance, op *operations.Ope
 
 	defer func() { _ = InstanceUnmount(pool, inst, op) }()
 
-	devSource, isPath := mountInfo.DevSource.(deviceConfig.DevSourcePath)
-
-	if !isPath {
-		return -1, fmt.Errorf("Unhandled DevSource type %T", mountInfo.DevSource)
+	if mountInfo.DiskPath == "" {
+		return -1, fmt.Errorf("No disk path available from mount")
 	}
 
-	if devSource.Path == "" {
-		return -1, errors.New("No disk path available from mount")
-	}
-
-	blockDiskSize, err := block.DiskSizeBytes(devSource.Path)
+	blockDiskSize, err := block.DiskSizeBytes(mountInfo.DiskPath)
 	if err != nil {
-		return -1, fmt.Errorf("Error getting block disk size %q: %w", devSource.Path, err)
+		return -1, fmt.Errorf("Error getting block disk size %q: %w", mountInfo.DiskPath, err)
 	}
 
 	return blockDiskSize, nil
@@ -1347,6 +1365,23 @@ func CompareSnapshots(sourceSnapshots []ComparableSnapshot, targetSnapshots []Co
 	return syncFromSource, deleteFromTarget
 }
 
+// ValidVolumeName validates a volume name.
+func ValidVolumeName(volumeName string) error {
+	if volumeName == "" {
+		return fmt.Errorf("Invalid volume name: Cannot be empty")
+	}
+
+	if strings.Contains(volumeName, "\\") {
+		return fmt.Errorf("Invalid volume name %q: Cannot contain backslashes", volumeName)
+	}
+
+	if strings.Contains(volumeName, shared.SnapshotDelimiter) {
+		return fmt.Errorf("Invalid volume name %q: Cannot contain slashes", volumeName)
+	}
+
+	return nil
+}
+
 // GetPoolDefaultBlockSize returns the default block size for the specified storage pool according to its driver.
 func GetPoolDefaultBlockSize(s *state.State, poolName string) (string, error) {
 	pool, err := LoadByName(s, poolName)
@@ -1356,91 +1391,3 @@ func GetPoolDefaultBlockSize(s *state.State, poolName string) (string, error) {
 
 	return pool.Driver().Info().DefaultBlockSize, nil
 }
-
-// VolumeDetermineNextSnapshotName determines a name for next snapshot of a volume
-// following the volume's snapshots.pattern or the provided default pattern.
-func VolumeDetermineNextSnapshotName(ctx context.Context, s *state.State, pool string, volumeName string, volumeConfig map[string]string) (string, error) {
-	var err error
-
-	pattern, ok := volumeConfig["snapshots.pattern"]
-	if !ok {
-		pattern = defaultSnapshotPattern
-	}
-
-	pattern, err = shared.RenderTemplate(pattern, pongo2.Context{
-		"creation_date": time.Now(),
-	})
-	if err != nil {
-		return "", err
-	}
-
-	count := strings.Count(pattern, "%d")
-	if count > 1 {
-		return "", errors.New("Snapshot pattern may contain '%%d' only once")
-	} else if count == 1 {
-		var i int
-		_ = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
-			i = tx.GetNextStorageVolumeSnapshotIndex(ctx, pool, volumeName, cluster.StoragePoolVolumeTypeCustom, pattern)
-
-			return nil
-		})
-
-		return strings.Replace(pattern, "%d", strconv.Itoa(i), 1), nil
-	}
-
-	snapshotExists := false
-
-	var snapshots []db.StorageVolumeArgs
-	var projects []string
-	var pools []string
-
-	err = s.DB.Cluster.Transaction(ctx, func(ctx context.Context, tx *db.ClusterTx) error {
-		projects, err = cluster.GetProjectNames(ctx, tx.Tx())
-		if err != nil {
-			return err
-		}
-
-		pools, err = tx.GetStoragePoolNames(ctx)
-		if err != nil {
-			return err
-		}
-
-		for _, pool := range pools {
-			var poolID int64
-			poolID, err = tx.GetStoragePoolID(ctx, pool)
-			if err != nil {
-				return err
-			}
-
-			for _, project := range projects {
-				snaps, err := tx.GetLocalStoragePoolVolumeSnapshotsWithType(ctx, project, volumeName, cluster.StoragePoolVolumeTypeCustom, poolID)
-				if err != nil {
-					return err
-				}
-
-				snapshots = append(snapshots, snaps...)
-			}
-		}
-
-		for _, snap := range snapshots {
-			_, snapOnlyName, _ := api.GetParentAndSnapshotName(snap.Name)
-
-			if snapOnlyName == pattern {
-				snapshotExists = true
-				break
-			}
-		}
-
-		if snapshotExists {
-			i := tx.GetNextStorageVolumeSnapshotIndex(ctx, pool, volumeName, cluster.StoragePoolVolumeTypeCustom, pattern)
-			pattern = strings.Replace(pattern, "%d", strconv.Itoa(i), 1)
-		}
-
-		return nil
-	})
-	if err != nil {
-		return "", err
-	}
-
-	return pattern, nil
-}
