@@ -3,7 +3,6 @@ package main
 import (
 	"context"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"net/http"
 	"net/url"
@@ -72,7 +71,7 @@ func storageBucketAccessHandler(entitlement auth.Entitlement) func(d *Daemon, r
 			return response.SmartError(err)
 		}
 
-		details, err := request.GetContextValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
+		details, err := request.GetCtxValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
 		if err != nil {
 			return nil
 		}
@@ -111,11 +110,6 @@ func storageBucketAccessHandler(entitlement auth.Entitlement) func(d *Daemon, r
 //      description: Project name
 //      type: string
 //      example: default
-//    - in: query
-//      name: all-projects
-//      description: Retrieve storage pool buckets from all projects
-//      type: boolean
-//      example: true
 //  responses:
 //    "200":
 //      description: API endpoints
@@ -165,11 +159,6 @@ func storageBucketAccessHandler(entitlement auth.Entitlement) func(d *Daemon, r
 //	    description: Project name
 //	    type: string
 //	    example: default
-//	  - in: query
-//	    name: all-projects
-//	    description: Retrieve storage pool buckets from all projects
-//	    type: boolean
-//	    example: true
 //	responses:
 //	  "200":
 //	    description: API endpoints
@@ -201,21 +190,12 @@ func storageBucketAccessHandler(entitlement auth.Entitlement) func(d *Daemon, r
 func storagePoolBucketsGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	requestProjectName, allProjects, err := request.ProjectParams(r)
+	requestProjectName := request.ProjectParam(r)
+	bucketProjectName, err := project.StorageBucketProject(r.Context(), s.DB.Cluster, requestProjectName)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
-	reqInfo := request.GetContextInfo(r.Context())
-	if !allProjects {
-		// Project specific requests require an effective project, when "features.storage.buckets" is enabled this is the requested project, otherwise it is the default project.
-		// If the request is project specific, then set effective project name in the request info so that the authorizer can generate the correct URL.
-		reqInfo.EffectiveProjectName, err = project.StorageBucketProject(r.Context(), s.DB.Cluster, requestProjectName)
-		if err != nil {
-			return response.SmartError(err)
-		}
-	}
-
 	withEntitlements, err := extractEntitlementsFromQuery(r, entity.TypeStorageBucket, true)
 	if err != nil {
 		return response.SmartError(err)
@@ -239,18 +219,15 @@ func storagePoolBucketsGet(d *Daemon, r *http.Request) response.Response {
 	memberSpecific := false // Get buckets for all cluster members.
 
 	var dbBuckets []*db.StorageBucket
+
 	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 		poolID := pool.ID()
+		filters := []db.StorageBucketFilter{{
+			PoolID:  &poolID,
+			Project: &bucketProjectName,
+		}}
 
-		filter := db.StorageBucketFilter{
-			PoolID: &poolID,
-		}
-
-		if !allProjects {
-			filter.Project = &reqInfo.EffectiveProjectName
-		}
-
-		dbBuckets, err = tx.GetStoragePoolBuckets(ctx, memberSpecific, filter)
+		dbBuckets, err = tx.GetStoragePoolBuckets(ctx, memberSpecific, filters...)
 		if err != nil {
 			return fmt.Errorf("Failed loading storage buckets: %w", err)
 		}
@@ -261,6 +238,7 @@ func storagePoolBucketsGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
+	request.SetCtxValue(r, request.CtxEffectiveProjectName, bucketProjectName)
 	userHasPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), auth.EntitlementCanView, entity.TypeStorageBucket)
 	if err != nil {
 		return response.SmartError(err)
@@ -358,19 +336,17 @@ func storagePoolBucketsGet(d *Daemon, r *http.Request) response.Response {
 func storagePoolBucketGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
+	details, err := request.GetCtxValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -381,7 +357,7 @@ func storagePoolBucketGet(d *Daemon, r *http.Request) response.Response {
 	}
 
 	if !details.pool.Driver().Info().Buckets {
-		return response.BadRequest(errors.New("Storage pool does not support buckets"))
+		return response.BadRequest(fmt.Errorf("Storage pool does not support buckets"))
 	}
 
 	targetMember := request.QueryParam(r, "target")
@@ -446,8 +422,7 @@ func storagePoolBucketGet(d *Daemon, r *http.Request) response.Response {
 func storagePoolBucketsPost(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
@@ -498,7 +473,7 @@ func storagePoolBucketsPost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Failed creating storage bucket admin key: %w", err))
 	}
 
-	s.Events.SendLifecycle(bucketProjectName, lifecycle.StorageBucketCreated.Event(pool, bucketProjectName, req.Name, request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(bucketProjectName, lifecycle.StorageBucketCreated.Event(pool, bucketProjectName, req.Name, request.CreateRequestor(r), nil))
 
 	u := api.NewURL().Path(version.APIVersion, "storage-pools", pool.Name(), "buckets", req.Name)
 
@@ -588,19 +563,17 @@ func storagePoolBucketsPost(d *Daemon, r *http.Request) response.Response {
 func storagePoolBucketPut(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
+	details, err := request.GetCtxValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -647,7 +620,7 @@ func storagePoolBucketPut(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Failed updating storage bucket: %w", err))
 	}
 
-	s.Events.SendLifecycle(effectiveProjectName, lifecycle.StorageBucketUpdated.Event(details.pool, effectiveProjectName, details.bucketName, request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(effectiveProjectName, lifecycle.StorageBucketUpdated.Event(details.pool, effectiveProjectName, details.bucketName, request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
@@ -684,19 +657,17 @@ func storagePoolBucketPut(d *Daemon, r *http.Request) response.Response {
 func storagePoolBucketDelete(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
+	details, err := request.GetCtxValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -706,7 +677,7 @@ func storagePoolBucketDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Failed deleting storage bucket: %w", err))
 	}
 
-	s.Events.SendLifecycle(effectiveProjectName, lifecycle.StorageBucketDeleted.Event(details.pool, effectiveProjectName, details.bucketName, request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(effectiveProjectName, lifecycle.StorageBucketDeleted.Event(details.pool, effectiveProjectName, details.bucketName, request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
@@ -808,19 +779,17 @@ func storagePoolBucketDelete(d *Daemon, r *http.Request) response.Response {
 func storagePoolBucketKeysGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
+	details, err := request.GetCtxValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -905,19 +874,17 @@ func storagePoolBucketKeysGet(d *Daemon, r *http.Request) response.Response {
 func storagePoolBucketKeysPost(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
+	details, err := request.GetCtxValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -934,7 +901,7 @@ func storagePoolBucketKeysPost(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Failed creating storage bucket key: %w", err))
 	}
 
-	lc := lifecycle.StorageBucketKeyCreated.Event(details.pool, effectiveProjectName, details.pool.Name(), req.Name, request.CreateRequestor(r.Context()), nil)
+	lc := lifecycle.StorageBucketKeyCreated.Event(details.pool, effectiveProjectName, details.pool.Name(), req.Name, request.CreateRequestor(r), nil)
 	s.Events.SendLifecycle(effectiveProjectName, lc)
 
 	return response.SyncResponseLocation(true, key, lc.Source)
@@ -972,19 +939,17 @@ func storagePoolBucketKeysPost(d *Daemon, r *http.Request) response.Response {
 func storagePoolBucketKeyDelete(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
+	details, err := request.GetCtxValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -999,7 +964,7 @@ func storagePoolBucketKeyDelete(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Failed deleting storage bucket key: %w", err))
 	}
 
-	s.Events.SendLifecycle(effectiveProjectName, lifecycle.StorageBucketKeyDeleted.Event(details.pool, effectiveProjectName, details.pool.Name(), details.bucketName, request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(effectiveProjectName, lifecycle.StorageBucketKeyDeleted.Event(details.pool, effectiveProjectName, details.pool.Name(), details.bucketName, request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
@@ -1047,25 +1012,23 @@ func storagePoolBucketKeyDelete(d *Daemon, r *http.Request) response.Response {
 func storagePoolBucketKeyGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
+	details, err := request.GetCtxValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
 
 	if !details.pool.Driver().Info().Buckets {
-		return response.BadRequest(errors.New("Storage pool does not support buckets"))
+		return response.BadRequest(fmt.Errorf("Storage pool does not support buckets"))
 	}
 
 	keyName, err := url.PathUnescape(mux.Vars(r)["keyName"])
@@ -1139,19 +1102,17 @@ func storagePoolBucketKeyGet(d *Daemon, r *http.Request) response.Response {
 func storagePoolBucketKeyPut(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
-	target := request.QueryParam(r, "target")
-	resp := forwardedResponseToNode(r.Context(), s, target)
+	resp := forwardedResponseIfTargetIsRemote(s, r)
 	if resp != nil {
 		return resp
 	}
 
-	var effectiveProjectName string
-	reqInfo := request.GetContextInfo(r.Context())
-	if reqInfo != nil {
-		effectiveProjectName = reqInfo.EffectiveProjectName
+	effectiveProjectName, err := request.GetCtxValue[string](r.Context(), request.CtxEffectiveProjectName)
+	if err != nil {
+		return response.SmartError(err)
 	}
 
-	details, err := request.GetContextValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
+	details, err := request.GetCtxValue[storageBucketDetails](r.Context(), ctxStorageBucketDetails)
 	if err != nil {
 		return response.SmartError(err)
 	}
@@ -1173,7 +1134,7 @@ func storagePoolBucketKeyPut(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(fmt.Errorf("Failed updating storage bucket key: %w", err))
 	}
 
-	s.Events.SendLifecycle(effectiveProjectName, lifecycle.StorageBucketKeyUpdated.Event(details.pool, effectiveProjectName, details.pool.Name(), details.bucketName, request.CreateRequestor(r.Context()), nil))
+	s.Events.SendLifecycle(effectiveProjectName, lifecycle.StorageBucketKeyUpdated.Event(details.pool, effectiveProjectName, details.pool.Name(), details.bucketName, request.CreateRequestor(r), nil))
 
 	return response.EmptySyncResponse
 }
@@ -1193,11 +1154,11 @@ type storageBucketDetails struct {
 
 // addStorageBucketDetailsToContext extracts storageBucketDetails from the http.Request and adds it to the
 // request context with the ctxStorageBucketDetails request.CtxKey. Additionally, the effective project of the storage
-// bucket is added to the request.Info.
+// bucket is added to the request context under request.CtxEffectiveProjectName.
 func addStorageBucketDetailsToContext(d *Daemon, r *http.Request) error {
 	var details storageBucketDetails
 	defer func() {
-		request.SetContextValue(r, ctxStorageBucketDetails, details)
+		request.SetCtxValue(r, ctxStorageBucketDetails, details)
 	}()
 
 	s := d.State()
@@ -1209,8 +1170,7 @@ func addStorageBucketDetailsToContext(d *Daemon, r *http.Request) error {
 		return err
 	}
 
-	reqInfo := request.GetContextInfo(r.Context())
-	reqInfo.EffectiveProjectName = effectiveProjectName
+	request.SetCtxValue(r, request.CtxEffectiveProjectName, effectiveProjectName)
 
 	poolName, err := url.PathUnescape(mux.Vars(r)["poolName"])
 	if err != nil {
