@@ -6,6 +6,7 @@ import (
 	"fmt"
 	"net/http"
 	"net/url"
+	"slices"
 	"strings"
 	"sync"
 
@@ -19,6 +20,7 @@ import (
 	dbCluster "github.com/canonical/lxd/lxd/db/cluster"
 	"github.com/canonical/lxd/lxd/lifecycle"
 	"github.com/canonical/lxd/lxd/project"
+	"github.com/canonical/lxd/lxd/project/limits"
 	"github.com/canonical/lxd/lxd/request"
 	"github.com/canonical/lxd/lxd/response"
 	"github.com/canonical/lxd/lxd/state"
@@ -35,17 +37,19 @@ import (
 var storagePoolCreateLock sync.Mutex
 
 var storagePoolsCmd = APIEndpoint{
-	Path: "storage-pools",
+	Path:        "storage-pools",
+	MetricsType: entity.TypeStoragePool,
 
 	Get:  APIEndpointAction{Handler: storagePoolsGet, AccessHandler: allowAuthenticated},
 	Post: APIEndpointAction{Handler: storagePoolsPost, AccessHandler: allowPermission(entity.TypeServer, auth.EntitlementCanCreateStoragePools)},
 }
 
 var storagePoolCmd = APIEndpoint{
-	Path: "storage-pools/{poolName}",
+	Path:        "storage-pools/{poolName}",
+	MetricsType: entity.TypeStoragePool,
 
 	Delete: APIEndpointAction{Handler: storagePoolDelete, AccessHandler: allowPermission(entity.TypeStoragePool, auth.EntitlementCanDelete, "poolName")},
-	Get:    APIEndpointAction{Handler: storagePoolGet, AccessHandler: allowPermission(entity.TypeStoragePool, auth.EntitlementCanView, "poolName")},
+	Get:    APIEndpointAction{Handler: storagePoolGet, AccessHandler: allowAuthenticated},
 	Patch:  APIEndpointAction{Handler: storagePoolPatch, AccessHandler: allowPermission(entity.TypeStoragePool, auth.EntitlementCanEdit, "poolName")},
 	Put:    APIEndpointAction{Handler: storagePoolPut, AccessHandler: allowPermission(entity.TypeStoragePool, auth.EntitlementCanEdit, "poolName")},
 }
@@ -146,28 +150,49 @@ func storagePoolsGet(d *Daemon, r *http.Request) response.Response {
 	s := d.State()
 
 	recursion := util.IsRecursionRequest(r)
+	withEntitlements, err := extractEntitlementsFromQuery(r, entity.TypeStoragePool, true)
+	if err != nil {
+		return response.SmartError(err)
+	}
 
 	var poolNames []string
+	var hiddenPoolNames []string
 
-	err := s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
+	err = s.DB.Cluster.Transaction(context.TODO(), func(ctx context.Context, tx *db.ClusterTx) error {
 		var err error
 
+		// Load the pool names.
 		poolNames, err = tx.GetStoragePoolNames(ctx)
+		if err != nil {
+			return err
+		}
 
-		return err
+		// Load the project limits.
+		hiddenPoolNames, err = limits.HiddenStoragePools(ctx, tx, request.ProjectParam(r))
+		if err != nil {
+			return err
+		}
+
+		return nil
 	})
 	if err != nil && !response.IsNotFoundError(err) {
 		return response.SmartError(err)
 	}
 
-	hasEditPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), r, auth.EntitlementCanEdit, entity.TypeStoragePool)
+	hasEditPermission, err := s.Authorizer.GetPermissionChecker(r.Context(), auth.EntitlementCanEdit, entity.TypeStoragePool)
 	if err != nil {
 		return response.InternalError(err)
 	}
 
 	resultString := []string{}
-	resultMap := []api.StoragePool{}
+	resultMap := []*api.StoragePool{}
+	urlToPool := make(map[*api.URL]auth.EntitlementReporter)
 	for _, poolName := range poolNames {
+		// Hide storage pools with a 0 project limit.
+		if slices.Contains(hiddenPoolNames, poolName) {
+			continue
+		}
+
 		if !recursion {
 			resultString = append(resultString, fmt.Sprintf("/%s/storage-pools/%s", version.APIVersion, poolName))
 		} else {
@@ -200,7 +225,8 @@ func storagePoolsGet(d *Daemon, r *http.Request) response.Response {
 				poolAPI.Status = pool.LocalStatus()
 			}
 
-			resultMap = append(resultMap, poolAPI)
+			resultMap = append(resultMap, &poolAPI)
+			urlToPool[entity.StoragePoolURL(poolName)] = &poolAPI
 		}
 	}
 
@@ -208,6 +234,13 @@ func storagePoolsGet(d *Daemon, r *http.Request) response.Response {
 		return response.SyncResponse(true, resultString)
 	}
 
+	if len(withEntitlements) > 0 {
+		err = reportEntitlements(r.Context(), s.Authorizer, s.IdentityCache, entity.TypeStorageVolume, withEntitlements, urlToPool)
+		if err != nil {
+			return response.SmartError(err)
+		}
+	}
+
 	return response.SyncResponse(true, resultMap)
 }
 
@@ -340,7 +373,7 @@ func storagePoolsPost(d *Daemon, r *http.Request) response.Response {
 			return tx.CreatePendingStoragePool(ctx, targetNode, req.Name, req.Driver, req.Config)
 		})
 		if err != nil {
-			if err == db.ErrAlreadyDefined {
+			if api.StatusErrorCheck(err, http.StatusConflict) {
 				return response.BadRequest(fmt.Errorf("The storage pool already defined on member %q", targetNode))
 			}
 
@@ -506,12 +539,7 @@ func storagePoolsPostCluster(s *state.State, pool *api.StoragePool, req api.Stor
 	}
 
 	// Notify all other nodes to create the pool.
-	err = notifier(func(client lxd.InstanceServer) error {
-		server, _, err := client.GetServer()
-		if err != nil {
-			return err
-		}
-
+	err = notifier(func(member db.NodeInfo, client lxd.InstanceServer) error {
 		nodeReq := req
 
 		// Clone fresh node config so we don't modify req.Config with this node's specific config which
@@ -522,7 +550,7 @@ func storagePoolsPostCluster(s *state.State, pool *api.StoragePool, req api.Stor
 		}
 
 		// Merge node specific config items into global config.
-		for key, value := range configs[server.Environment.ServerName] {
+		for key, value := range configs[member.Name] {
 			nodeReq.Config[key] = value
 		}
 
@@ -531,7 +559,7 @@ func storagePoolsPostCluster(s *state.State, pool *api.StoragePool, req api.Stor
 			return err
 		}
 
-		logger.Debug("Created storage pool on cluster member", logger.Ctx{"pool": req.Name, "member": server.Environment.ServerName})
+		logger.Debug("Created storage pool on cluster member", logger.Ctx{"pool": req.Name, "member": member.Name})
 
 		return nil
 	})
@@ -611,11 +639,37 @@ func storagePoolGet(d *Daemon, r *http.Request) response.Response {
 		return response.SmartError(err)
 	}
 
+	withEntitlements, err := extractEntitlementsFromQuery(r, entity.TypeStoragePool, false)
+	if err != nil {
+		return response.SmartError(err)
+	}
+
 	memberSpecific := false
 	if request.QueryParam(r, "target") != "" {
 		memberSpecific = true
 	}
 
+	var hiddenPoolNames []string
+	err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
+		var err error
+
+		// Load the project limits.
+		hiddenPoolNames, err = limits.HiddenStoragePools(ctx, tx, request.ProjectParam(r))
+		if err != nil {
+			return err
+		}
+
+		return nil
+	})
+	if err != nil {
+		return response.SmartError(err)
+	}
+
+	// Hide storage pools with a 0 project limit.
+	if slices.Contains(hiddenPoolNames, poolName) {
+		return response.NotFound(nil)
+	}
+
 	// Get the existing storage pool.
 	pool, err := storagePools.LoadByName(s, poolName)
 	if err != nil {
@@ -631,7 +685,7 @@ func storagePoolGet(d *Daemon, r *http.Request) response.Response {
 	poolAPI := pool.ToAPI()
 	poolAPI.UsedBy = project.FilterUsedBy(s.Authorizer, r, poolUsedBy)
 
-	err = s.Authorizer.CheckPermission(r.Context(), r, entity.StoragePoolURL(poolName), auth.EntitlementCanEdit)
+	err = s.Authorizer.CheckPermission(r.Context(), entity.StoragePoolURL(poolName), auth.EntitlementCanEdit)
 	if err != nil && !auth.IsDeniedError(err) {
 		return response.SmartError(err)
 	} else if err != nil {
@@ -649,6 +703,13 @@ func storagePoolGet(d *Daemon, r *http.Request) response.Response {
 		poolAPI.Status = pool.LocalStatus()
 	}
 
+	if len(withEntitlements) > 0 {
+		err = reportEntitlements(r.Context(), s.Authorizer, s.IdentityCache, entity.TypeStoragePool, withEntitlements, map[*api.URL]auth.EntitlementReporter{entity.StoragePoolURL(poolName): &poolAPI})
+		if err != nil {
+			return response.SmartError(err)
+		}
+	}
+
 	etag := []any{pool.Name(), pool.Driver().Info().Name, pool.Description(), poolAPI.Config}
 
 	return response.SyncResponseETag(true, &poolAPI, etag)
@@ -880,7 +941,7 @@ func doStoragePoolUpdate(s *state.State, pool storagePools.Pool, req api.Storage
 			sendPool.Config[k] = v
 		}
 
-		err = notifier(func(client lxd.InstanceServer) error {
+		err = notifier(func(member db.NodeInfo, client lxd.InstanceServer) error {
 			return client.UpdateStoragePool(pool.Name(), sendPool, "")
 		})
 		if err != nil {
@@ -962,7 +1023,8 @@ func storagePoolDelete(d *Daemon, r *http.Request) response.Response {
 		err = s.DB.Cluster.Transaction(r.Context(), func(ctx context.Context, tx *db.ClusterTx) error {
 			// Get all the volumes using the storage pool on this server.
 			// Only image volumes should remain now.
-			volumes, err := tx.GetStoragePoolVolumes(ctx, pool.ID(), true)
+			poolID := pool.ID() // Create local variable to get the pointer.
+			volumes, err := tx.GetStorageVolumes(ctx, true, db.StorageVolumeFilter{PoolID: &poolID})
 			if err != nil {
 				return fmt.Errorf("Failed loading storage volumes: %w", err)
 			}
@@ -1003,12 +1065,7 @@ func storagePoolDelete(d *Daemon, r *http.Request) response.Response {
 	}
 
 	// If we are clustered, also notify all other nodes.
-	err = notifier(func(client lxd.InstanceServer) error {
-		_, _, err := client.GetServer()
-		if err != nil {
-			return err
-		}
-
+	err = notifier(func(member db.NodeInfo, client lxd.InstanceServer) error {
 		return client.DeleteStoragePool(pool.Name())
 	})
 	if err != nil {
