@@ -76,14 +76,11 @@ type OS struct {
 	AppArmorStacked   bool
 	AppArmorStacking  bool
 	AppArmorFeatures  AppArmorFeaturesInfo
-	AppArmorCacheLoc  string
-	AppArmorCacheDir  string // Based on AppArmorCacheLoc, but may also point to a subdirectory influenced by features.
 
 	// Cgroup features
 	CGInfo cgroup.Info
 
 	// Kernel features
-	BPFToken                bool // BPFToken indicates support for BPF token delegation mechanism.
 	CloseRange              bool // CloseRange indicates support for the close_range syscall.
 	ContainerCoreScheduling bool // ContainerCoreScheduling indicates LXC and kernel support for core scheduling.
 	CoreScheduling          bool // CoreScheduling indicates support for core scheduling syscalls.
@@ -103,13 +100,10 @@ type OS struct {
 	LXCFeatures map[string]bool
 
 	// OS info
-	ReleaseInfo map[string]string
-	Uname       *shared.Utsname
-	BootTime    time.Time
-
-	// Version info
-	KernelVersion   version.DottedVersion
-	AppArmorVersion *version.DottedVersion // AppArmorVersion is nil if AppArmorAvailable is false.
+	ReleaseInfo   map[string]string
+	KernelVersion version.DottedVersion
+	Uname         *shared.Utsname
+	BootTime      time.Time
 }
 
 // DefaultOS returns a fresh uninitialized OS instance with default values.
@@ -215,7 +209,7 @@ func (s *OS) Init() ([]cluster.Warning, error) {
 	}
 
 	btime := int64(0)
-	for line := range strings.SplitSeq(string(out), "\n") {
+	for _, line := range strings.Split(string(out), "\n") {
 		if !strings.HasPrefix(line, "btime ") {
 			continue
 		}
@@ -253,13 +247,3 @@ func (s *OS) InUbuntuCore() bool {
 
 	return false
 }
-
-// GetUnixSocket returns the full path to the unix.socket file that this daemon is listening on. Used by tests.
-func (s *OS) GetUnixSocket() string {
-	path := os.Getenv("LXD_SOCKET")
-	if path != "" {
-		return path
-	}
-
-	return filepath.Join(s.VarDir, "unix.socket")
-}
