@@ -4,12 +4,12 @@ import (
 	"bytes"
 	"fmt"
 	"io"
-	"io/ioutil"
+	"os"
 	"path/filepath"
 	"strings"
 )
 
-// ChrootLoader is a pong2 compatible file loader which restricts all accesses to a directory
+// ChrootLoader is a pong2 compatible file loader which restricts all accesses to a directory.
 type ChrootLoader struct {
 	Path string
 }
@@ -20,7 +20,7 @@ type ChrootLoader struct {
 // might be a path of a template which includes another template) or
 // the current working directory.
 func (l ChrootLoader) Abs(base string, name string) string {
-	return filepath.Clean(fmt.Sprintf("%s/%s", l.Path, name))
+	return filepath.Clean(l.Path + "/" + name)
 }
 
 // Get reads the path's content from your local filesystem.
@@ -38,11 +38,11 @@ func (l ChrootLoader) Get(path string) (io.Reader, error) {
 
 	// Validate that we're under the expected prefix
 	if !strings.HasPrefix(path, basePath) {
-		return nil, fmt.Errorf("Attempting to access a file outside the container")
+		return nil, fmt.Errorf("Attempting to access a file outside the instance")
 	}
 
 	// Open and read the file
-	buf, err := ioutil.ReadFile(path)
+	buf, err := os.ReadFile(path)
 	if err != nil {
 		return nil, err
 	}
