@@ -6,10 +6,8 @@ import (
 	"crypto/sha256"
 	"crypto/subtle"
 	"crypto/x509"
-	"encoding/hex"
 	"encoding/json"
 	"encoding/pem"
-	"errors"
 	"fmt"
 	"io"
 	"net"
@@ -74,7 +72,7 @@ func EtagHash(data any) (string, error) {
 		return "", err
 	}
 
-	return hex.EncodeToString(etag.Sum(nil)), nil
+	return fmt.Sprintf("%x", etag.Sum(nil)), nil
 }
 
 // EtagCheck validates the hash of the current state with the hash
@@ -93,7 +91,7 @@ func EtagCheck(r *http.Request, data any) error {
 	}
 
 	if hash != match {
-		return api.StatusErrorf(http.StatusPreconditionFailed, "ETag doesn't match: %s vs %s. The configuration has been modified since this change began. Please retrieve the updated configuration before proceeding.", hash, match)
+		return api.StatusErrorf(http.StatusPreconditionFailed, "ETag doesn't match: %s vs %s", hash, match)
 	}
 
 	return nil
@@ -107,7 +105,7 @@ func HTTPClient(certificate string, proxy proxyFunc) (*http.Client, error) {
 	if certificate != "" {
 		certBlock, _ := pem.Decode([]byte(certificate))
 		if certBlock == nil {
-			return nil, errors.New("Invalid certificate")
+			return nil, fmt.Errorf("Invalid certificate")
 		}
 
 		cert, err = x509.ParseCertificate(certBlock.Bytes)
