@@ -62,6 +62,7 @@ func (l *inMemoryListener) Addr() net.Addr {
 type inMemoryAddr struct {
 }
 
+// Network returns the name of the network.
 func (a *inMemoryAddr) Network() string {
 	return "memory"
 }
@@ -80,15 +81,15 @@ func CanonicalNetworkAddress(address string, defaultPort int64) string {
 		if ip != nil {
 			// If the input address is a bare IP address, then convert it to a proper listen address
 			// using the canonical IP with default port and wrap IPv6 addresses in square brackets.
-			address = net.JoinHostPort(ip.String(), fmt.Sprintf("%d", defaultPort))
+			address = net.JoinHostPort(ip.String(), fmt.Sprint(defaultPort))
 		} else {
 			// Otherwise assume this is either a host name or a partial address (e.g `[::]`) without
 			// a port number, so append the default port.
-			address = fmt.Sprintf("%s:%d", address, defaultPort)
+			address = address + ":" + fmt.Sprint(defaultPort)
 		}
 	} else if port == "" && address[len(address)-1] == ':' {
 		// An address that ends with a trailing colon will be parsed as having an empty port.
-		address = net.JoinHostPort(host, fmt.Sprintf("%d", defaultPort))
+		address = net.JoinHostPort(host, fmt.Sprint(defaultPort))
 	}
 
 	return address
@@ -99,7 +100,7 @@ func CanonicalNetworkAddress(address string, defaultPort int64) string {
 func CanonicalNetworkAddressFromAddressAndPort(address string, port int64, defaultPort int64) string {
 	// Because we accept just the host part of an IPv6 listen address (e.g. `[::]`) don't use net.JoinHostPort.
 	// If a bare IP address is supplied then CanonicalNetworkAddress will use net.JoinHostPort if needed.
-	return CanonicalNetworkAddress(fmt.Sprintf("%s:%d", address, port), defaultPort)
+	return CanonicalNetworkAddress(address+":"+fmt.Sprint(port), defaultPort)
 }
 
 // ServerTLSConfig returns a new server-side tls.Config generated from the give
@@ -116,7 +117,7 @@ func ServerTLSConfig(cert *shared.CertInfo) *tls.Config {
 		config.RootCAs = pool
 		config.ClientCAs = pool
 
-		logger.Infof("LXD is in CA mode, only CA-signed certificates will be allowed")
+		logger.Infof("LXD is in CA mode, only CA-signed client certificates will be allowed")
 	}
 
 	return config
@@ -271,7 +272,7 @@ func IsWildCardAddress(address string) bool {
 // SysctlGet retrieves the value of a sysctl file in /proc/sys.
 func SysctlGet(path string) (string, error) {
 	// Read the current content
-	content, err := os.ReadFile(fmt.Sprintf("/proc/sys/%s", path))
+	content, err := os.ReadFile("/proc/sys/" + path)
 	if err != nil {
 		return "", err
 	}
@@ -298,7 +299,7 @@ func SysctlSet(parts ...string) error {
 			return nil
 		}
 
-		err = os.WriteFile(fmt.Sprintf("/proc/sys/%s", path), []byte(newValue), 0)
+		err = os.WriteFile("/proc/sys/"+path, []byte(newValue), 0)
 		if err != nil {
 			return err
 		}
