@@ -45,8 +45,7 @@ type Event struct {
 
 // ToLogging creates log record for the event.
 func (event *Event) ToLogging() (EventLogRecord, error) {
-	switch event.Type {
-	case EventTypeLogging, EventTypeOVN:
+	if event.Type == EventTypeLogging || event.Type == EventTypeOVN {
 		e := &EventLogging{}
 		err := json.Unmarshal(event.Metadata, &e)
 		if err != nil {
@@ -67,7 +66,7 @@ func (event *Event) ToLogging() (EventLogRecord, error) {
 		}
 
 		return record, nil
-	case EventTypeLifecycle:
+	} else if event.Type == EventTypeLifecycle {
 		e := &EventLifecycle{}
 		err := json.Unmarshal(event.Metadata, &e)
 		if err != nil {
@@ -94,7 +93,7 @@ func (event *Event) ToLogging() (EventLogRecord, error) {
 		}
 
 		return record, nil
-	case EventTypeOperation:
+	} else if event.Type == EventTypeOperation {
 		e := &Operation{}
 		err := json.Unmarshal(event.Metadata, &e)
 		if err != nil {
