@@ -6,7 +6,7 @@ package api
 //
 // API extension: network.
 type NetworksPost struct {
-	NetworkPut `yaml:",inline"` //nolint:musttag
+	NetworkPut `yaml:",inline"`
 
 	// The name of the new network
 	// Example: lxdbr1
@@ -111,12 +111,6 @@ type Network struct {
 	//
 	// API extension: clustering
 	Locations []string `json:"locations" yaml:"locations"`
-
-	// Project name
-	// Example: project1
-	//
-	// API extension: networks_all_projects
-	Project string `json:"project" yaml:"project"`
 }
 
 // Writable converts a full Network struct into a NetworkPut struct (filters read-only fields).
