@@ -1,7 +1,6 @@
 package api
 
 import (
-	"errors"
 	"fmt"
 	"time"
 )
@@ -79,7 +78,7 @@ func (op *Operation) ToCertificateAddToken() (*CertificateAddToken, error) {
 
 	clientName, ok := req["name"].(string)
 	if !ok {
-		return nil, errors.New("Failed to get client name")
+		return nil, fmt.Errorf("Failed to get client name")
 	}
 
 	secret, ok := op.Metadata["secret"].(string)
