@@ -40,7 +40,7 @@ type StorageBucketPut struct {
 //
 // API extension: storage_buckets.
 type StorageBucket struct {
-	WithEntitlements `yaml:",inline"` //nolint:musttag
+	WithEntitlements `yaml:",inline"`
 
 	// Bucket name
 	// Example: foo
@@ -71,12 +71,6 @@ type StorageBucket struct {
 	//
 	// API extension: storage_buckets
 	Config map[string]string `json:"config" yaml:"config"`
-
-	// Project name
-	// Example: project1
-	//
-	// API extension: storage_buckets_all_projects
-	Project string `json:"project" yaml:"project"`
 }
 
 // Etag returns the values used for etag generation.
