@@ -102,7 +102,7 @@ type StorageVolumePostTarget struct {
 //
 // API extension: storage.
 type StorageVolume struct {
-	WithEntitlements `yaml:",inline"` //nolint:musttag
+	WithEntitlements `yaml:",inline"`
 
 	// Volume name
 	// Example: foo
