@@ -1,6 +1,7 @@
 package osarch
 
 import (
+	"fmt"
 	"testing"
 
 	"github.com/stretchr/testify/suite"
@@ -23,7 +24,7 @@ VERSION_ID="16.04"
 	defer cleanup()
 
 	lsbRelease, err := getLSBRelease(filename)
-	s.NoError(err)
+	s.Nil(err)
 	s.Equal(
 		map[string]string{
 			"NAME":       "Ubuntu",
@@ -38,7 +39,7 @@ func (s *releaseTestSuite) TestGetLSBReleaseSingleQuotes() {
 	defer cleanup()
 
 	lsbRelease, err := getLSBRelease(filename)
-	s.NoError(err)
+	s.Nil(err)
 	s.Equal(map[string]string{"NAME": "Ubuntu"}, lsbRelease)
 }
 
@@ -48,7 +49,7 @@ func (s *releaseTestSuite) TestGetLSBReleaseNoQuotes() {
 	defer cleanup()
 
 	lsbRelease, err := getLSBRelease(filename)
-	s.NoError(err)
+	s.Nil(err)
 	s.Equal(map[string]string{"NAME": "Ubuntu"}, lsbRelease)
 }
 
@@ -64,7 +65,7 @@ VERSION_ID="16.04"
 	defer cleanup()
 
 	lsbRelease, err := getLSBRelease(filename)
-	s.NoError(err)
+	s.Nil(err)
 	s.Equal(
 		map[string]string{
 			"NAME":       "Ubuntu",
@@ -83,5 +84,5 @@ ID="ubuntu"
 	defer cleanup()
 
 	_, err := getLSBRelease(filename)
-	s.EqualError(err, filename+": invalid format on line 3")
+	s.EqualError(err, fmt.Sprintf("%s: invalid format on line 3", filename))
 }
