@@ -2,10 +2,10 @@ package simplestreams
 
 import (
 	"fmt"
+	"slices"
 	"strings"
 	"time"
 
-	"github.com/canonical/lxd/shared"
 	"github.com/canonical/lxd/shared/api"
 	"github.com/canonical/lxd/shared/osarch"
 )
@@ -70,7 +70,8 @@ func (s *Products) ToLXD() ([]api.Image, map[string][][]string) {
 	downloads := map[string][][]string{}
 
 	images := []api.Image{}
-	nameLayout := "20060102"
+	nameLayoutLong := "20060102_1504" // Date and time.
+	nameLayoutShort := "20060102"     // Date only.
 	eolLayout := "2006-01-02"
 
 	for _, product := range s.Products {
@@ -91,18 +92,24 @@ func (s *Products) ToLXD() ([]api.Image, map[string][][]string) {
 				continue
 			}
 
-			creationDate, err := time.Parse(nameLayout, name[0:8])
+			// Parse the creation date from the version name. First, check if the version contains
+			// both date and upload time. If it doesn't, try parsing just the date part. If the date
+			// cannot be fetched, skip that version instead of erroring out.
+			creationDate, err := time.Parse(nameLayoutLong, name)
 			if err != nil {
-				continue
+				creationDate, err = time.Parse(nameLayoutShort, name[0:8])
+				if err != nil {
+					continue
+				}
 			}
 
 			// Image processing function
 			addImage := func(meta *ProductVersionItem, root *ProductVersionItem) error {
 				// Look for deltas
 				deltas := []ProductVersionItem{}
-				if root != nil && shared.ValueInSlice(root.FileType, []string{"squashfs", "disk-kvm.img"}) {
+				if root != nil && slices.Contains([]string{"squashfs", "disk-kvm.img"}, root.FileType) {
 					for _, item := range version.Items {
-						if item.FileType == fmt.Sprintf("%s.vcdiff", root.FileType) {
+						if item.FileType == root.FileType+".vcdiff" {
 							deltas = append(deltas, item)
 						}
 					}
@@ -149,12 +156,12 @@ func (s *Products) ToLXD() ([]api.Image, map[string][][]string) {
 				filename := fields[len(fields)-1]
 
 				// Generate the actual image entry
-				description := fmt.Sprintf("%s %s %s", product.OperatingSystem, product.ReleaseTitle, product.Architecture)
+				description := product.OperatingSystem + " " + product.ReleaseTitle + " " + product.Architecture
 				if version.Label != "" {
-					description = fmt.Sprintf("%s (%s)", description, version.Label)
+					description += " (" + version.Label + ")"
 				}
 
-				description = fmt.Sprintf("%s (%s)", description, name)
+				description += " (" + name + ")"
 
 				image := api.Image{}
 				image.Architecture = architectureName
@@ -226,11 +233,11 @@ func (s *Products) ToLXD() ([]api.Image, map[string][][]string) {
 				// Set the file list
 				var imgDownloads [][]string
 				if root == nil {
-					imgDownloads = [][]string{{meta.Path, meta.HashSha256, "meta", fmt.Sprintf("%d", meta.Size)}}
+					imgDownloads = [][]string{{meta.Path, meta.HashSha256, "meta", fmt.Sprint(meta.Size)}}
 				} else {
 					imgDownloads = [][]string{
-						{meta.Path, meta.HashSha256, "meta", fmt.Sprintf("%d", meta.Size)},
-						{root.Path, root.HashSha256, "root", fmt.Sprintf("%d", root.Size)}}
+						{meta.Path, meta.HashSha256, "meta", fmt.Sprint(meta.Size)},
+						{root.Path, root.HashSha256, "root", fmt.Sprint(root.Size)}}
 				}
 
 				// Add the deltas
@@ -244,16 +251,23 @@ func (s *Products) ToLXD() ([]api.Image, map[string][][]string) {
 					// Locate source image fingerprint
 					var srcFingerprint string
 					for _, item := range srcImage.Items {
-						if !shared.ValueInSlice(item.FileType, lxdCompatItems) {
+						if !slices.Contains(lxdCompatItems, item.FileType) {
 							continue
 						}
 
-						srcFingerprint = item.LXDHashSha256SquashFs
+						// Take correct source image fingerprint based on
+						// delta file type.
+						if delta.FileType == "disk-kvm.img.vcdiff" {
+							srcFingerprint = item.LXDHashSha256DiskKvmImg
+						} else if delta.FileType == "squashfs.vcdiff" {
+							srcFingerprint = item.LXDHashSha256SquashFs
+						}
+
 						break
 					}
 
 					if srcFingerprint == "" {
-						// Couldn't find the image
+						// Couldn't find the source image
 						continue
 					}
 
@@ -261,8 +275,8 @@ func (s *Products) ToLXD() ([]api.Image, map[string][][]string) {
 					imgDownloads = append(imgDownloads, []string{
 						delta.Path,
 						delta.HashSha256,
-						fmt.Sprintf("root.delta-%s", srcFingerprint),
-						fmt.Sprintf("%d", delta.Size)})
+						"root.delta-" + srcFingerprint,
+						fmt.Sprint(delta.Size)})
 				}
 
 				// Add the image
@@ -274,15 +288,15 @@ func (s *Products) ToLXD() ([]api.Image, map[string][][]string) {
 
 			// Locate a valid LXD image
 			for _, item := range version.Items {
-				if shared.ValueInSlice(item.FileType, lxdCompatCombinedItems) {
+				if slices.Contains(lxdCompatCombinedItems, item.FileType) {
 					err := addImage(&item, nil)
 					if err != nil {
 						continue
 					}
-				} else if shared.ValueInSlice(item.FileType, lxdCompatItems) {
+				} else if slices.Contains(lxdCompatItems, item.FileType) {
 					// Locate the root files
 					for _, subItem := range version.Items {
-						if shared.ValueInSlice(subItem.FileType, []string{"disk1.img", "disk-kvm.img", "uefi1.img", "root.tar.xz", "squashfs"}) {
+						if slices.Contains([]string{"disk1.img", "disk-kvm.img", "uefi1.img", "root.tar.xz", "squashfs"}, subItem.FileType) {
 							err := addImage(&item, &subItem)
 							if err != nil {
 								continue
