@@ -9,11 +9,9 @@ import (
 	"encoding/gob"
 	"encoding/hex"
 	"encoding/json"
-	"errors"
 	"fmt"
 	"hash"
 	"io"
-	"maps"
 	"net/http"
 	"net/url"
 	"os"
@@ -23,7 +21,6 @@ import (
 	"reflect"
 	"regexp"
 	"runtime"
-	"slices"
 	"strconv"
 	"strings"
 	"time"
@@ -335,7 +332,7 @@ func ParseLXDFileHeaders(headers http.Header) (*LXDFileHeaders, error) {
 		filetype = "file"
 	}
 
-	if !slices.Contains([]string{"file", "symlink", "directory"}, filetype) {
+	if !ValueInSlice(filetype, []string{"file", "symlink", "directory"}) {
 		return nil, fmt.Errorf("Invalid file type: %q", filetype)
 	}
 
@@ -347,7 +344,7 @@ func ParseLXDFileHeaders(headers http.Header) (*LXDFileHeaders, error) {
 		write = "overwrite"
 	}
 
-	if !slices.Contains([]string{"overwrite", "append"}, write) {
+	if !ValueInSlice(write, []string{"overwrite", "append"}) {
 		return nil, fmt.Errorf("Invalid file write mode: %q", write)
 	}
 
@@ -357,14 +354,13 @@ func ParseLXDFileHeaders(headers http.Header) (*LXDFileHeaders, error) {
 
 	modifyPermHeader := headers.Get("X-LXD-modify-perm")
 
-	modifyPermFields := []string{"uid", "gid", "mode"}
 	if modifyPermHeader != "" {
-		for perm := range strings.SplitSeq(modifyPermHeader, ",") {
+		for _, perm := range strings.Split(modifyPermHeader, ",") {
 			UIDModifyExisting = UIDModifyExisting || perm == "uid"
 			GIDModifyExisting = GIDModifyExisting || perm == "gid"
 			modeModifyExisting = modeModifyExisting || perm == "mode"
 
-			if !slices.Contains(modifyPermFields, perm) {
+			if !ValueInSlice(perm, []string{"uid", "gid", "mode"}) {
 				return nil, fmt.Errorf("Invalid modify-perm field: %q", perm)
 			}
 		}
@@ -428,7 +424,7 @@ func RandomCryptoString() (string, error) {
 	}
 
 	if n != len(buf) {
-		return "", errors.New("not enough random bytes read")
+		return "", fmt.Errorf("not enough random bytes read")
 	}
 
 	return hex.EncodeToString(buf), nil
@@ -597,7 +593,7 @@ func DirCopy(source string, dest string) error {
 	}
 
 	if !info.IsDir() {
-		return errors.New("source is not a directory")
+		return fmt.Errorf("source is not a directory")
 	}
 
 	// Remove dest if it already exists.
@@ -673,7 +669,7 @@ func MkdirAllOwner(path string, perm os.FileMode, uid int, gid int) error {
 			return nil
 		}
 
-		return errors.New("path exists but isn't a directory")
+		return fmt.Errorf("path exists but isn't a directory")
 	}
 
 	// Slow path: make sure parent exists and then call Mkdir for path.
@@ -724,6 +720,17 @@ func HasKey[K comparable, V any](key K, m map[K]V) bool {
 	return found
 }
 
+// ValueInSlice returns true if key is in list.
+func ValueInSlice[T comparable](key T, list []T) bool {
+	for _, entry := range list {
+		if entry == key {
+			return true
+		}
+	}
+
+	return false
+}
+
 // StringPrefixInSlice returns true if any element in the list has the given prefix.
 func StringPrefixInSlice(key string, list []string) bool {
 	for _, entry := range list {
@@ -737,22 +744,20 @@ func StringPrefixInSlice(key string, list []string) bool {
 
 // RemoveElementsFromSlice returns a slice equivalent to removing the given elements from the given list.
 // Elements not present in the list are ignored.
-// The input slice is cloned to avoid modifying the original slice.
 func RemoveElementsFromSlice[T comparable](list []T, elements ...T) []T {
-	list = slices.Clone(list)
 	for i := len(elements) - 1; i >= 0; i-- {
 		element := elements[i]
 		match := false
 		for j := len(list) - 1; j >= 0; j-- {
 			if element == list[j] {
 				match = true
-				list = slices.Delete(list, j, j+1)
+				list = append(list[:j], list[j+1:]...)
 				break
 			}
 		}
 
 		if match {
-			elements = slices.Delete(elements, i, i+1)
+			elements = append(elements[:i], elements[i+1:]...)
 		}
 	}
 
@@ -775,7 +780,7 @@ func IsTrue(value string) bool {
 		return true
 	}
 
-	return slices.Contains([]string{"true", "yes", "on"}, strings.ToLower(value))
+	return ValueInSlice(strings.ToLower(value), []string{"true", "yes", "on"})
 }
 
 // IsTrueOrEmpty returns true if value is empty or if IsTrue() returns true.
@@ -789,7 +794,7 @@ func IsFalse(value string) bool {
 		return true
 	}
 
-	return slices.Contains([]string{"false", "no", "off"}, strings.ToLower(value))
+	return ValueInSlice(strings.ToLower(value), []string{"false", "no", "off"})
 }
 
 // IsFalseOrEmpty returns true if value is empty or if IsFalse() returns true.
@@ -898,7 +903,7 @@ func TextEditor(inPath string, inContent []byte) ([]byte, error) {
 				}
 			}
 			if editor == "" {
-				return []byte{}, errors.New("No text editor found, please set the EDITOR environment variable")
+				return []byte{}, fmt.Errorf("No text editor found, please set the EDITOR environment variable")
 			}
 		}
 	}
@@ -965,7 +970,7 @@ func ParseMetadata(metadata any) (map[string]any, error) {
 	if s.Kind() == reflect.Map {
 		for _, k := range s.MapKeys() {
 			if k.Kind() != reflect.String {
-				return nil, errors.New("Invalid metadata provided (key isn't a string)")
+				return nil, fmt.Errorf("Invalid metadata provided (key isn't a string)")
 			}
 
 			newMetadata[k.String()] = s.MapIndex(k).Interface()
@@ -973,7 +978,7 @@ func ParseMetadata(metadata any) (map[string]any, error) {
 	} else if s.Kind() == reflect.Ptr && !s.Elem().IsValid() {
 		return nil, nil
 	} else {
-		return nil, errors.New("Invalid metadata provided (type isn't a map)")
+		return nil, fmt.Errorf("Invalid metadata provided (type isn't a map)")
 	}
 
 	return newMetadata, nil
@@ -991,7 +996,7 @@ func RemoveDuplicatesFromString(s string, sep string) string {
 
 	dup := sep + sep
 	for strings.Contains(s, dup) {
-		s = strings.ReplaceAll(s, dup, sep)
+		s = strings.Replace(s, dup, sep, -1)
 	}
 
 	return s
@@ -1129,7 +1134,7 @@ func TryRunCommand(name string, arg ...string) (string, error) {
 	var err error
 	var output string
 
-	for range 20 {
+	for i := 0; i < 20; i++ {
 		output, err = RunCommand(name, arg...)
 		if err == nil {
 			break
@@ -1214,7 +1219,7 @@ func DownloadFileHash(ctx context.Context, httpClient *http.Client, useragent st
 	if ctx != nil {
 		req, err = http.NewRequestWithContext(ctx, "GET", url, nil)
 	} else {
-		req, err = http.NewRequest(http.MethodGet, url, nil)
+		req, err = http.NewRequest("GET", url, nil)
 	}
 
 	if err != nil {
@@ -1264,7 +1269,7 @@ func DownloadFileHash(ctx context.Context, httpClient *http.Client, useragent st
 			return -1, err
 		}
 
-		result := hex.EncodeToString(hashFunc.Sum(nil))
+		result := fmt.Sprintf("%x", hashFunc.Sum(nil))
 		if result != hash {
 			return -1, fmt.Errorf("Hash mismatch for %s: %s != %s", url, result, hash)
 		}
@@ -1325,52 +1330,25 @@ func (r *ReadSeeker) Seek(offset int64, whence int) (int64, error) {
 }
 
 // RenderTemplate renders a pongo2 template.
-func RenderTemplate(template string, ctx pongo2.Context) (output string, err error) {
-	defer func() {
-		// Capture panics in the pongo2 template rendering.
-		// This is to prevent the server from crashing due to a template error.
-		r := recover()
-		if r != nil {
-			err = fmt.Errorf("Panic while rendering template: %v", r)
-		}
-	}()
-
-	// Create custom TemplateSet
-	set := pongo2.NewSet("restricted", pongo2.DefaultLoader)
-
-	// Ban tags that could be used to access the host's filesystem.
-	for _, tag := range []string{"extends", "import", "include", "ssi"} {
-		err := set.BanTag(tag)
-		if err != nil {
-			return "", fmt.Errorf("Failed to ban tag %q: %w", tag, err)
-		}
+func RenderTemplate(template string, ctx pongo2.Context) (string, error) {
+	// Load template from string
+	tpl, err := pongo2.FromString("{% autoescape off %}" + template + "{% endautoescape %}")
+	if err != nil {
+		return "", err
 	}
 
-	// Prevent unbounded recursion while rendering templates. Normal use should not
-	// require more than 1 or 2 levels of recursion.
-	for range 3 {
-		// Load template from string
-		tpl, err := set.FromString("{% autoescape off %}" + template + "{% endautoescape %}")
-		if err != nil {
-			return "", err
-		}
-
-		// Get rendered template
-		ret, err := tpl.Execute(ctx)
-		if err != nil {
-			return "", err
-		}
-
-		// Check if another pass is needed.
-		if !strings.Contains(ret, "{{") && !strings.Contains(ret, "{%") {
-			return ret, nil
-		}
-
-		// Prepare for another pass.
-		template = ret
+	// Get rendered template
+	ret, err := tpl.Execute(ctx)
+	if err != nil {
+		return ret, err
 	}
 
-	return "", errors.New("Recursion limit reached while rendering template")
+	// Looks like we're nesting templates so run pongo again
+	if strings.Contains(ret, "{{") || strings.Contains(ret, "{%") {
+		return RenderTemplate(ret, ctx)
+	}
+
+	return ret, err
 }
 
 // GetExpiry returns the expiry date based on the reference date and a length of time.
@@ -1407,12 +1385,12 @@ func GetExpiry(refDate time.Time, s string) (time.Time, error) {
 	for _, value := range values {
 		fields := re.FindStringSubmatch(value)
 		if fields == nil {
-			return time.Time{}, errors.New("Invalid expiry expression")
+			return time.Time{}, fmt.Errorf("Invalid expiry expression")
 		}
 
 		if expiry[fields[2]] > 0 {
 			// We don't allow fields to be set multiple times
-			return time.Time{}, errors.New("Invalid expiry expression")
+			return time.Time{}, fmt.Errorf("Invalid expiry expression")
 		}
 
 		val, err := strconv.Atoi(fields[1])
@@ -1482,19 +1460,19 @@ func JoinTokenDecode(input string) (*api.ClusterMemberJoinToken, error) {
 	}
 
 	if j.ServerName == "" {
-		return nil, errors.New("No server name in join token")
+		return nil, fmt.Errorf("No server name in join token")
 	}
 
 	if len(j.Addresses) < 1 {
-		return nil, errors.New("No cluster member addresses in join token")
+		return nil, fmt.Errorf("No cluster member addresses in join token")
 	}
 
 	if j.Secret == "" {
-		return nil, errors.New("No secret in join token")
+		return nil, fmt.Errorf("No secret in join token")
 	}
 
 	if j.Fingerprint == "" {
-		return nil, errors.New("No certificate fingerprint in join token")
+		return nil, fmt.Errorf("No certificate fingerprint in join token")
 	}
 
 	return &j, nil
@@ -1503,9 +1481,8 @@ func JoinTokenDecode(input string) (*api.ClusterMemberJoinToken, error) {
 // TargetDetect returns either target node or group based on the provided prefix:
 // An invocation with `target=h1` returns "h1", "" and `target=@g1` returns "", "g1".
 func TargetDetect(target string) (targetNode string, targetGroup string) {
-	after, found := strings.CutPrefix(target, "@")
-	if found {
-		targetGroup = after
+	if strings.HasPrefix(target, "@") {
+		targetGroup = strings.TrimPrefix(target, "@")
 	} else {
 		targetNode = target
 	}
@@ -1522,7 +1499,9 @@ func ApplyDeviceOverrides(localDevices map[string]map[string]string, profileDevi
 		_, isLocalDevice := localDevices[deviceName]
 		if isLocalDevice {
 			// Apply overrides to local device.
-			maps.Copy(localDevices[deviceName], deviceOverrides[deviceName])
+			for k, v := range deviceOverrides[deviceName] {
+				localDevices[deviceName][k] = v
+			}
 		} else {
 			// Check device exists in expanded profile devices.
 			profileDeviceConfig, found := profileDevices[deviceName]
@@ -1530,7 +1509,9 @@ func ApplyDeviceOverrides(localDevices map[string]map[string]string, profileDevi
 				return nil, fmt.Errorf("Cannot override config for device %q: Device not found in profile devices", deviceName)
 			}
 
-			maps.Copy(profileDeviceConfig, deviceOverrides[deviceName])
+			for k, v := range deviceOverrides[deviceName] {
+				profileDeviceConfig[k] = v
+			}
 
 			localDevices[deviceName] = profileDeviceConfig
 		}
