@@ -12,7 +12,9 @@ import (
 	"os/exec"
 	"path/filepath"
 	"reflect"
+	"strconv"
 	"strings"
+	"syscall"
 	"time"
 	"unsafe"
 
@@ -20,16 +22,17 @@ import (
 	"golang.org/x/sys/unix"
 
 	"github.com/canonical/lxd/shared/revert"
-	"github.com/canonical/lxd/shared/units"
 )
 
 // --- pure Go functions ---
 
+// GetFileStat retrieves the UID, GID, major and minor device numbers, inode, and number of hard links for
+// the given file path.
 func GetFileStat(p string) (uid int, gid int, major uint32, minor uint32, inode uint64, nlink int, err error) {
 	var stat unix.Stat_t
 	err = unix.Lstat(p, &stat)
 	if err != nil {
-		return
+		return 0, 0, 0, 0, 0, 0, err
 	}
 
 	uid = int(stat.Uid)
@@ -41,7 +44,7 @@ func GetFileStat(p string) (uid int, gid int, major uint32, minor uint32, inode
 		minor = unix.Minor(uint64(stat.Rdev))
 	}
 
-	return
+	return uid, gid, major, minor, inode, nlink, nil
 }
 
 // GetPathMode returns a os.FileMode for the provided path.
@@ -55,6 +58,7 @@ func GetPathMode(path string) (os.FileMode, error) {
 	return mode, nil
 }
 
+// SetSize sets the terminal size to the specified width and height for the given file descriptor.
 func SetSize(fd int, width int, height int) (err error) {
 	var dimensions [4]uint16
 	dimensions[0] = uint16(height)
@@ -94,8 +98,10 @@ func GetAllXattr(path string) (map[string]string, error) {
 	return xattrs, nil
 }
 
-var ObjectFound = fmt.Errorf("Found requested object")
+// ErrObjectFound indicates that the requested object was found.
+var ErrObjectFound = fmt.Errorf("Found requested object")
 
+// LookupUUIDByBlockDevPath finds and returns the UUID of a block device by its path.
 func LookupUUIDByBlockDevPath(diskDevice string) (string, error) {
 	uuid := ""
 	readUUID := func(path string, info os.FileInfo, err error) error {
@@ -117,14 +123,14 @@ func LookupUUIDByBlockDevPath(diskDevice string) (string, error) {
 				uuid = path
 				// Will allows us to avoid needlessly travers
 				// the whole directory.
-				return ObjectFound
+				return ErrObjectFound
 			}
 		}
 		return nil
 	}
 
 	err := filepath.Walk("/dev/disk/by-uuid", readUUID)
-	if err != nil && err != ObjectFound {
+	if err != nil && err != ErrObjectFound {
 		return "", fmt.Errorf("Failed to detect UUID: %s", err)
 	}
 
@@ -136,7 +142,9 @@ func LookupUUIDByBlockDevPath(diskDevice string) (string, error) {
 	return uuid[lastSlash+1:], nil
 }
 
-// Detect whether err is an errno.
+// GetErrno detects whether the error is an errno.
+//
+//revive:disable:error-return Error is returned first because this is similar to assertion.
 func GetErrno(err error) (errno error, iserrno bool) {
 	sysErr, ok := err.(*os.SyscallError)
 	if ok {
@@ -220,10 +228,12 @@ func intArrayToString(arr any) string {
 	return s
 }
 
+// DeviceTotalMemory returns the total memory of the device by reading /proc/meminfo.
 func DeviceTotalMemory() (int64, error) {
 	return GetMeminfo("MemTotal")
 }
 
+// GetMeminfo retrieves the memory information for the specified field from /proc/meminfo.
 func GetMeminfo(field string) (int64, error) {
 	// Open /proc/meminfo
 	f, err := os.Open("/proc/meminfo")
@@ -238,38 +248,41 @@ func GetMeminfo(field string) (int64, error) {
 	for scan.Scan() {
 		line := scan.Text()
 
-		// We only care about MemTotal
-		if !strings.HasPrefix(line, field+":") {
+		// We only care about the requested field
+		rightHandSide, found := strings.CutPrefix(line, field+":")
+		if !found {
 			continue
 		}
 
-		// Extract the before last (value) and last (unit) fields
-		fields := strings.Split(line, " ")
-		value := fields[len(fields)-2] + fields[len(fields)-1]
+		// Most lines end with " kB" to indicate the value is in kilobytes
+		multiplier := int64(1)
+		value, found := strings.CutSuffix(rightHandSide, " kB")
+		if found {
+			multiplier = 1024
+		}
 
-		// Feed the result to units.ParseByteSizeString to get an int value
-		valueBytes, err := units.ParseByteSizeString(value)
+		// Remove spaces and convert to int.
+		valueInt, err := strconv.ParseInt(strings.TrimLeft(value, " "), 10, 64)
 		if err != nil {
 			return -1, err
 		}
 
-		return valueBytes, nil
+		// Multiply the value by the multiplier
+		return valueInt * multiplier, nil
 	}
 
 	return -1, fmt.Errorf("Couldn't find %s", field)
 }
 
 // OpenPtyInDevpts creates a new PTS pair, configures them and returns them.
-func OpenPtyInDevpts(devpts_fd int, uid, gid int64) (*os.File, *os.File, error) {
+func OpenPtyInDevpts(devptsFD int, uid, gid int64) (ptx *os.File, pty *os.File, err error) {
 	revert := revert.New()
 	defer revert.Fail()
 	var fd int
-	var ptx *os.File
-	var err error
 
 	// Create a PTS pair.
-	if devpts_fd >= 0 {
-		fd, err = unix.Openat(devpts_fd, "ptmx", unix.O_RDWR|unix.O_CLOEXEC|unix.O_NOCTTY, 0)
+	if devptsFD >= 0 {
+		fd, err = unix.Openat(devptsFD, "ptmx", unix.O_RDWR|unix.O_CLOEXEC|unix.O_NOCTTY, 0)
 	} else {
 		fd, err = unix.Openat(-1, "/dev/ptmx", unix.O_RDWR|unix.O_CLOEXEC|unix.O_NOCTTY, 0)
 	}
@@ -288,7 +301,6 @@ func OpenPtyInDevpts(devpts_fd int, uid, gid int64) (*os.File, *os.File, error)
 		return nil, nil, unix.Errno(errno)
 	}
 
-	var pty *os.File
 	ptyFd, _, errno := unix.Syscall(unix.SYS_IOCTL, uintptr(ptx.Fd()), unix.TIOCGPTPEER, uintptr(unix.O_NOCTTY|unix.O_CLOEXEC|os.O_RDWR))
 	// We can only fallback to looking up the fd in /dev/pts when we aren't dealing with the container's devpts instance.
 	if errno == 0 {
@@ -301,7 +313,7 @@ func OpenPtyInDevpts(devpts_fd int, uid, gid int64) (*os.File, *os.File, error)
 
 		pty = os.NewFile(ptyFd, fmt.Sprintf("/dev/pts/%d", id))
 	} else {
-		if devpts_fd >= 0 {
+		if devptsFD >= 0 {
 			return nil, nil, fmt.Errorf("TIOCGPTPEER required but not available")
 		}
 
@@ -370,7 +382,7 @@ func OpenPtyInDevpts(devpts_fd int, uid, gid int64) (*os.File, *os.File, error)
 }
 
 // OpenPty creates a new PTS pair, configures them and returns them.
-func OpenPty(uid, gid int64) (*os.File, *os.File, error) {
+func OpenPty(uid, gid int64) (ptx *os.File, pty *os.File, err error) {
 	return OpenPtyInDevpts(-1, uid, gid)
 }
 
@@ -388,7 +400,7 @@ func ExitStatus(err error) (int, error) {
 	// Detect and extract ExitError to check the embedded exit status.
 	if errors.As(err, &exitErr) {
 		// If the process was signaled, extract the signal.
-		status, isWaitStatus := exitErr.Sys().(unix.WaitStatus)
+		status, isWaitStatus := exitErr.Sys().(syscall.WaitStatus)
 		if isWaitStatus && status.Signaled() {
 			return 128 + int(status.Signal()), nil // 128 + n == Fatal error signal "n"
 		}
@@ -401,7 +413,7 @@ func ExitStatus(err error) (int, error) {
 }
 
 // GetPollRevents poll for events on provided fd.
-func GetPollRevents(fd int, timeout int, flags int) (int, int, error) {
+func GetPollRevents(fd int, timeout int, flags int) (n int, revents int, err error) {
 	pollFd := unix.PollFd{
 		Fd:      int32(fd),
 		Events:  int16(flags),
@@ -411,7 +423,7 @@ func GetPollRevents(fd int, timeout int, flags int) (int, int, error) {
 	pollFds := []unix.PollFd{pollFd}
 
 again:
-	n, err := unix.Poll(pollFds, timeout)
+	n, err = unix.Poll(pollFds, timeout)
 	if err != nil {
 		if err == unix.EAGAIN || err == unix.EINTR {
 			goto again
@@ -498,10 +510,12 @@ func (w *execWrapper) Read(p []byte) (int, error) {
 	return n, opErr
 }
 
+// Write writes data to the underlying os.File.
 func (w *execWrapper) Write(p []byte) (int, error) {
 	return w.f.Write(p)
 }
 
+// Close closes the underlying os.File.
 func (w *execWrapper) Close() error {
 	return w.f.Close()
 }
