@@ -1,5 +1,4 @@
 btrfs_setup() {
-  # shellcheck disable=2039
   local LXD_DIR
 
   LXD_DIR=$1
@@ -8,19 +7,17 @@ btrfs_setup() {
 }
 
 btrfs_configure() {
-  # shellcheck disable=2039
   local LXD_DIR
 
   LXD_DIR=$1
 
-  lxc storage create "lxdtest-$(basename "${LXD_DIR}")" btrfs size=100GB
+  lxc storage create "lxdtest-$(basename "${LXD_DIR}")" btrfs size=1GiB
   lxc profile device add default root disk path="/" pool="lxdtest-$(basename "${LXD_DIR}")"
 
   echo "==> Configuring btrfs backend in ${LXD_DIR}"
 }
 
 btrfs_teardown() {
-  # shellcheck disable=2039
   local LXD_DIR
 
   LXD_DIR=$1
