@@ -10,14 +10,14 @@ lxc_remote() {
     local injected cmd arg
 
     injected=0
-    # _LXC contains the path to lxc binary, not the shell wrapper function
-    cmd="${_LXC}"
+    # find the path to lxc binary, not the shell wrapper function
+    cmd=$(unset -f lxc; command -v lxc)
 
     # shellcheck disable=SC2048,SC2068
     for arg in "$@"; do
         if [ "${arg}" = "--" ]; then
             injected=1
-            cmd="${cmd} ${CLIENT_DEBUG:-}"
+            cmd="${cmd} ${DEBUG:-}"
             [ -n "${LXC_LOCAL}" ] && cmd="${cmd} --force-local"
             cmd="${cmd} --"
         elif [ "${arg}" = "--force-local" ]; then
@@ -28,9 +28,9 @@ lxc_remote() {
     done
 
     if [ "${injected}" = "0" ]; then
-        cmd="${cmd} ${CLIENT_DEBUG-}"
+        cmd="${cmd} ${DEBUG-}"
     fi
-    if [ -n "${SHELL_TRACING:-}" ]; then
+    if [ -n "${DEBUG:-}" ]; then
         eval "set -x;timeout --foreground 120 ${cmd}"
     else
         eval "timeout --foreground 120 ${cmd}"
