@@ -9,7 +9,6 @@ spawn_lxd() {
 
     lxddir=${1}
     shift
-    chmod +x "${lxddir}"
 
     storage=${1}
     shift
@@ -31,16 +30,17 @@ spawn_lxd() {
     echo "$lxd_backend" > "${lxddir}/lxd.backend"
 
     echo "==> Spawning lxd in ${lxddir}"
+    # shellcheck disable=SC2086
 
     # Set ulimit to ensure core dump is outputted.
     ulimit -c unlimited
 
     if [ "${LXD_NETNS}" = "" ]; then
-        LXD_DIR="${lxddir}" lxd --logfile "${lxddir}/lxd.log" "${SERVER_DEBUG-}" "$@" 2>&1 &
+        LXD_DIR="${lxddir}" lxd --logfile "${lxddir}/lxd.log" "${DEBUG-}" "$@" 2>&1 &
     else
         # shellcheck disable=SC2153
         read -r pid < "${TEST_DIR}/ns/${LXD_NETNS}/PID"
-        LXD_DIR="${lxddir}" nsenter -n -m -t "${pid}" lxd --logfile "${lxddir}/lxd.log" "${SERVER_DEBUG-}" "$@" 2>&1 &
+        LXD_DIR="${lxddir}" nsenter -n -m -t "${pid}" lxd --logfile "${lxddir}/lxd.log" "${DEBUG-}" "$@" 2>&1 &
     fi
     LXD_PID=$!
     echo "${LXD_PID}" > "${lxddir}/lxd.pid"
@@ -62,9 +62,7 @@ spawn_lxd() {
         done
     fi
 
-    echo "==> Setting trust password"
-    LXD_DIR="${lxddir}" lxc config set core.trust_password foo
-    if [ -n "${SHELL_TRACING:-}" ]; then
+    if [ -n "${DEBUG:-}" ]; then
         set -x
     fi
 
@@ -93,11 +91,12 @@ respawn_lxd() {
     shift
 
     echo "==> Spawning lxd in ${lxddir}"
+    # shellcheck disable=SC2086
     if [ "${LXD_NETNS}" = "" ]; then
-        LXD_DIR="${lxddir}" lxd --logfile "${lxddir}/lxd.log" "${SERVER_DEBUG-}" "$@" 2>&1 &
+        LXD_DIR="${lxddir}" lxd --logfile "${lxddir}/lxd.log" "${DEBUG-}" "$@" 2>&1 &
     else
         read -r pid < "${TEST_DIR}/ns/${LXD_NETNS}/PID"
-        LXD_DIR="${lxddir}" nsenter -n -m -t "${pid}" lxd --logfile "${lxddir}/lxd.log" "${SERVER_DEBUG-}" "$@" 2>&1 &
+        LXD_DIR="${lxddir}" nsenter -n -m -t "${pid}" lxd --logfile "${lxddir}/lxd.log" "${DEBUG-}" "$@" 2>&1 &
     fi
     LXD_PID=$!
     echo "${LXD_PID}" > "${lxddir}/lxd.pid"
@@ -108,7 +107,7 @@ respawn_lxd() {
         LXD_DIR="${lxddir}" lxd waitready --timeout=300 || (echo "Killing PID ${LXD_PID}" ; kill -9 "${LXD_PID}" ; false)
     fi
 
-    if [ -n "${SHELL_TRACING:-}" ]; then
+    if [ -n "${DEBUG:-}" ]; then
         set -x
     fi
 }
@@ -127,16 +126,16 @@ kill_lxd() {
       return
     fi
 
-    daemon_pid=$(< "${daemon_dir}/lxd.pid")
+    daemon_pid=$(cat "${daemon_dir}/lxd.pid")
     check_leftovers="false"
     lxd_backend=$(storage_backend "$daemon_dir")
     echo "==> Killing LXD at ${daemon_dir} (${daemon_pid})"
 
     if [ -e "${daemon_dir}/unix.socket" ]; then
         # Delete all containers
-        echo "==> Deleting all instances"
-        for instance in $(timeout -k 2 2 lxc list --force-local --format csv --columns n); do
-            timeout -k 10 10 lxc delete "${instance}" --force-local -f || true
+        echo "==> Deleting all containers"
+        for container in $(timeout -k 2 2 lxc list --force-local --format csv --columns n); do
+            timeout -k 10 10 lxc delete "${container}" --force-local -f || true
         done
 
         # Delete all images
@@ -147,9 +146,7 @@ kill_lxd() {
 
         # Delete all profiles
         echo "==> Deleting all profiles"
-        for profile in $(timeout -k 2 2 lxc profile list --force-local --format csv --columns n); do
-            # default cannot be deleted.
-            [ "${profile}" = "default" ] && continue
+        for profile in $(timeout -k 2 2 lxc profile list --force-local --format csv | cut -d, -f1); do
             timeout -k 10 10 lxc profile delete "${profile}" --force-local || true
         done
 
@@ -199,9 +196,9 @@ kill_lxd() {
         check_leftovers="true"
     fi
 
-    # If SERVER_DEBUG is set, check for panics in the daemon logs
-    if [ -n "${SERVER_DEBUG:-}" ]; then
-      "${MAIN_DIR}/deps/panic-checker" "${daemon_dir}/lxd.log"
+    # If DEBUG is set, check for panics in the daemon logs
+    if [ -n "${DEBUG:-}" ]; then
+      deps/panic-checker "${daemon_dir}/lxd.log"
     fi
 
     if [ -n "${LXD_LOGS:-}" ]; then
@@ -216,7 +213,7 @@ kill_lxd() {
         rm -f "${daemon_dir}/containers/lxc-monitord.log"
 
         # Support AppArmor policy cache directory
-        apparmor_cache_dir="$(apparmor_parser --cache-loc "${daemon_dir}"/security/apparmor/cache --print-cache-dir)"
+        apparmor_cache_dir="$(apparmor_parser -L "${daemon_dir}"/security/apparmor/cache --print-cache-dir)"
         rm -f "${apparmor_cache_dir}/.features"
         check_empty "${daemon_dir}/containers/"
         check_empty "${daemon_dir}/devices/"
@@ -230,16 +227,16 @@ kill_lxd() {
         check_empty "${daemon_dir}/snapshots/"
 
         echo "==> Checking for leftover DB entries"
-        check_empty_table "${daemon_dir}/database/global/db.bin" "images"
-        check_empty_table "${daemon_dir}/database/global/db.bin" "images_aliases"
-        check_empty_table "${daemon_dir}/database/global/db.bin" "images_nodes"
-        check_empty_table "${daemon_dir}/database/global/db.bin" "images_properties"
-        check_empty_table "${daemon_dir}/database/global/db.bin" "images_source"
         check_empty_table "${daemon_dir}/database/global/db.bin" "instances"
         check_empty_table "${daemon_dir}/database/global/db.bin" "instances_config"
         check_empty_table "${daemon_dir}/database/global/db.bin" "instances_devices"
         check_empty_table "${daemon_dir}/database/global/db.bin" "instances_devices_config"
         check_empty_table "${daemon_dir}/database/global/db.bin" "instances_profiles"
+        check_empty_table "${daemon_dir}/database/global/db.bin" "images"
+        check_empty_table "${daemon_dir}/database/global/db.bin" "images_aliases"
+        check_empty_table "${daemon_dir}/database/global/db.bin" "images_properties"
+        check_empty_table "${daemon_dir}/database/global/db.bin" "images_source"
+        check_empty_table "${daemon_dir}/database/global/db.bin" "images_nodes"
         check_empty_table "${daemon_dir}/database/global/db.bin" "networks"
         check_empty_table "${daemon_dir}/database/global/db.bin" "networks_config"
         check_empty_table "${daemon_dir}/database/global/db.bin" "profiles"
@@ -247,8 +244,8 @@ kill_lxd() {
         check_empty_table "${daemon_dir}/database/global/db.bin" "profiles_devices"
         check_empty_table "${daemon_dir}/database/global/db.bin" "profiles_devices_config"
         check_empty_table "${daemon_dir}/database/global/db.bin" "storage_pools"
-        check_empty_table "${daemon_dir}/database/global/db.bin" "storage_pools_config"
         check_empty_table "${daemon_dir}/database/global/db.bin" "storage_pools_nodes"
+        check_empty_table "${daemon_dir}/database/global/db.bin" "storage_pools_config"
         check_empty_table "${daemon_dir}/database/global/db.bin" "storage_volumes"
         check_empty_table "${daemon_dir}/database/global/db.bin" "storage_volumes_config"
     fi
@@ -272,7 +269,7 @@ shutdown_lxd() {
     daemon_dir=${1}
     # shellcheck disable=2034
     LXD_DIR=${daemon_dir}
-    daemon_pid=$(< "${daemon_dir}/lxd.pid")
+    daemon_pid=$(cat "${daemon_dir}/lxd.pid")
     echo "==> Shutting down LXD at ${daemon_dir} (${daemon_pid})"
 
     # Shutting down the daemon
@@ -300,6 +297,12 @@ wipe() {
         fi
     fi
 
+    local pid
+    # shellcheck disable=SC2009
+    ps aux | grep lxc-monitord | grep "${1}" | awk '{print $2}' | while read -r pid; do
+        kill -9 "${pid}" || true
+    done
+
     if mountpoint -q "${1}"; then
         umount -l "${1}"
     fi
@@ -308,19 +311,17 @@ wipe() {
 }
 
 panic_checker() {
-  # Only run if SERVER_DEBUG is set (e.g. LXD_VERBOSE or LXD_DEBUG is set)
+  # Only run if DEBUG is set (e.g. LXD_VERBOSE or LXD_DEBUG is set)
   # Panics are logged at info level, which won't be outputted unless this is set.
-  if [ -z "${SERVER_DEBUG:-}" ]; then
-    return 0
+  if [ -z "${DEBUG:-}" ]; then
+    return
   fi
 
   local test_dir daemon_dir
   test_dir="${1}"
 
-  [ -e "${test_dir}/daemons" ] || return
-
   while read -r daemon_dir; do
-    "${MAIN_DIR}/deps/panic-checker" "${daemon_dir}/lxd.log"
+    deps/panic-checker "${daemon_dir}/lxd.log"
   done < "${test_dir}/daemons"
 }
 
@@ -330,11 +331,9 @@ cleanup_lxds() {
     test_dir="$1"
 
     # Kill all LXD instances
-    if [ -s "${test_dir}/daemons" ]; then
-      while read -r daemon_dir; do
-          kill_lxd "${daemon_dir}"
-      done < "${test_dir}/daemons"
-    fi
+    while read -r daemon_dir; do
+        kill_lxd "${daemon_dir}"
+    done < "${test_dir}/daemons"
 
     # Cleanup leftover networks
     # shellcheck disable=SC2009
@@ -354,66 +353,3 @@ cleanup_lxds() {
 
     umount_loops "$test_dir"
 }
-
-lxd_shutdown_restart() {
-    local scenario LXD_DIR
-    scenario=${1}
-    LXD_DIR=${2}
-
-    daemon_pid=$(< "${LXD_DIR}/lxd.pid")
-    echo "==> Shutting down LXD at ${LXD_DIR} (${daemon_pid})"
-
-    local logfile="${scenario}.log"
-    echo "Starting LXD log capture in $logfile using lxc monitor..."
-    lxc monitor --pretty > "$logfile" 2>&1 &
-    local monitor_pid=$!
-
-    # Give monitor a moment to connect
-    sleep 2
-    echo "Monitor PID: $monitor_pid"
-    echo "LXD daemon PID: $daemon_pid"
-    echo "Starting LXD shutdown sequence..."
-    if ! kill -SIGPWR "$daemon_pid" 2>/dev/null; then
-        echo "Failed to signal LXD to shutdown" | tee -a "$logfile"
-        return 1
-    fi
-
-    echo "Waiting for LXD to shutdown gracefully..." | tee -a "$logfile"
-    for _ in $(seq 540); do
-        if ! kill -0 "$daemon_pid" 2>/dev/null; then
-            sleep 5 # Give the monitor a moment to catch up
-            break
-        fi
-        sleep 1
-    done
-
-    echo "LXD shutdown sequence completed."
-    respawn_lxd "${LXD_DIR}" true
-}
-
-# create_instances creates a specified number of instances in the background.
-# The instance are called i1, i2, i3, etc.
-create_instances() {
-  local n="$1"  # Number of instances to create.
-
-  for i in $(seq 1 "$n"); do
-    echo "Creating instance i$i..."
-    lxc launch testimage "i${i}" -d "${SMALL_ROOT_DISK}"
-  done
-
-  echo "All instances created successfully."
-  return 0
-}
-
-# delete_instances deletes a specified number of instances in the background.
-# The instances should be called i1, i2, i3, etc.
-delete_instances() {
-  local n="$1"  # Number of instances to delete.
-
-  for i in $(seq 1 "$n"); do
-    echo "Deleting i$i..."
-    lxc delete "i$i" --force
-  done
-
-  return 0
-}
