@@ -13,7 +13,6 @@ EOF
         return
     fi
 
-    # shellcheck disable=SC2039,3043
     local port pid
 
     while true; do
@@ -29,7 +28,8 @@ EOF
 
 # Certificate-aware curl wrapper
 my_curl() {
-    curl -k -s --cert "${LXD_CONF}/client.crt" --key "${LXD_CONF}/client.key" "$@"
+    CERTNAME="${CERTNAME:-"client"}"
+    curl --insecure --silent --cert "${LXD_CONF}/${CERTNAME}.crt" --key "${LXD_CONF}/${CERTNAME}.key" "$@"
 }
 
 # Wait for duplicate address detection to complete.
