@@ -1,16 +1,5 @@
 #!/bin/bash
 set -eu
-set -o pipefail
-if [ -z "${GOPATH:-}" ] && command -v go >/dev/null; then
-    GOPATH="$(go env GOPATH)"
-fi
-
-# Avoid accidental re-execution
-if [ -n "${LXD_INSPECT_INPROGRESS:-}" ]; then
-    echo "Refusing to run tests from inside a LXD_INSPECT session" >&2
-    exit 1
-fi
-
 [ -n "${GOPATH:-}" ] && export "PATH=${GOPATH}/bin:${PATH}"
 
 # Don't translate lxc output for parsing in it in tests.
@@ -24,33 +13,21 @@ if [ -z "${NO_PROXY:-}" ]; then
   export NO_PROXY="127.0.0.1"
 fi
 
-export CLIENT_DEBUG=""
-export SERVER_DEBUG=""
-export SHELL_TRACING=""
-if [ "${LXD_VERBOSE:-0}" != "0" ]; then
-  if [ "${LXD_VERBOSE}" = "client" ]; then
-      CLIENT_DEBUG="--verbose"
-  elif [ "${LXD_VERBOSE}" = "server" ]; then
-      SERVER_DEBUG="--verbose"
-  else
-      CLIENT_DEBUG="--verbose"
-      SERVER_DEBUG="--verbose"
-  fi
-
-  SHELL_TRACING=1
+export DEBUG=""
+if [ -n "${LXD_VERBOSE:-}" ]; then
+  DEBUG="--verbose"
 fi
 
-if [ "${LXD_DEBUG:-0}" != "0" ]; then
-  if [ "${LXD_DEBUG}" = "client" ]; then
-      CLIENT_DEBUG="--debug"
-  elif [ "${LXD_DEBUG}" = "server" ]; then
-      SERVER_DEBUG="--debug"
-  else
-      CLIENT_DEBUG="--debug"
-      SERVER_DEBUG="--debug"
-  fi
+if [ -n "${LXD_DEBUG:-}" ]; then
+  DEBUG="--debug"
+fi
 
-  SHELL_TRACING=1
+if [ -n "${DEBUG:-}" ]; then
+  set -x
+fi
+
+if [ -z "${LXD_BACKEND:-}" ]; then
+    LXD_BACKEND="dir"
 fi
 
 # shellcheck disable=SC2034
@@ -65,30 +42,10 @@ import_subdir_files() {
     done
 }
 
-# `main.sh` needs to be executed from inside the `test/` directory
-if [ "${PWD}" != "$(dirname "${0}")" ]; then
-    cd "$(dirname "${0}")"
-fi
-MAIN_DIR="${PWD}"
-export MAIN_DIR
 import_subdir_files includes
 
-# Install needed storage driver tools
-install_storage_driver_tools
-
-# Install needed instance drivers
-install_instance_drivers
-
 echo "==> Checking for dependencies"
-check_dependencies lxd lxc curl busybox dnsmasq iptables jq nc ping yq git s3cmd sqlite3 rsync shuf setfacl setfattr socat swtpm dig xz
-if [ "${LXD_VM_TESTS:-0}" = "1" ]; then
-  check_dependencies qemu-img "qemu-system-$(uname -m)" sgdisk
-fi
-
-# find the path to lxc binary, not the shell wrapper function
-_LXC="$(unset -f lxc; command -v lxc)"
-readonly _LXC
-export _LXC
+check_dependencies lxd lxc curl dnsmasq jq git sqlite3 msgmerge msgfmt shuf setfacl socat swtpm dig
 
 if [ "${USER:-'root'}" != "root" ]; then
   echo "The testsuite must be run as root." >&2
@@ -114,55 +71,31 @@ echo "==> Using storage backend ${LXD_BACKEND}"
 import_storage_backends
 
 cleanup() {
-  # Stop tracing everything
-  { set +x; } 2>/dev/null
-  if [ -z "${SHELL_TRACING:-}" ]; then
-    echo "cleanup"
-  fi
-
-  # Avoid reentry by removing the traps
-  trap - EXIT HUP INT TERM
-
   # Before setting +e, run the panic checker for any running LXD daemons.
   panic_checker "${TEST_DIR}"
 
-  # Allow for failures
-  set +e
-  unset CLIENT_DEBUG
-  unset SERVER_DEBUG
-  unset SHELL_TRACING
+  # Allow for failures and stop tracing everything
+  set +ex
+  DEBUG=
 
   # Allow for inspection
   if [ -n "${LXD_INSPECT:-}" ]; then
     if [ "${TEST_RESULT}" != "success" ]; then
-      echo "==> FAILED TEST: ${TEST_CURRENT#test_} (${TEST_CURRENT_DESCRIPTION})"
-      # red
-      PS1_PREFIX="\[\033[0;31m\]LXD-TEST\[\033[0m\]"
-    else
-      # green
-      PS1_PREFIX="\[\033[0;32m\]LXD-TEST\[\033[0m\]"
+      echo "==> TEST DONE: ${TEST_CURRENT_DESCRIPTION}"
     fi
     echo "==> Test result: ${TEST_RESULT}"
 
-    # Re-execution prevention
-    export LXD_INSPECT_INPROGRESS=true
-
-    echo -e "\033[0;33mDropping to a shell for inspection.\nOnce done, exit (Ctrl-D) to continue\033[0m"
-    export PS1="${PS1_PREFIX} ${PS1:-\u@\h:\w\$ }"
-    bash --norc
+    # shellcheck disable=SC2086
+    printf "To poke around, use:\\n LXD_DIR=%s LXD_CONF=%s sudo -E %s/bin/lxc COMMAND\\n" "${LXD_DIR}" "${LXD_CONF}" ${GOPATH:-}
+    echo "Tests Completed (${TEST_RESULT}): hit enter to continue"
+    read -r _
   fi
 
+  echo ""
+  echo "df -h output:"
+  df -h
+
   if [ "${TEST_RESULT}" != "success" ]; then
-    echo ""
-    echo "df -h output:"
-    df -h
-
-    if command -v ceph >/dev/null; then
-      echo "::group::ceph status"
-      ceph status --connect-timeout 5 || true
-      echo "::endgroup::"
-    fi
-
     # dmesg may contain oops, IO errors, crashes, etc
     # If there's a kernel stack trace, don't generate a collapsible group
 
@@ -187,29 +120,19 @@ cleanup() {
   else
     echo "==> Cleaning up"
 
-    [ -e "${LXD_TEST_IMAGE:-}" ] && rm "${LXD_TEST_IMAGE}"
-
     kill_oidc
-    clear_ovn_nb_db
-    mountpoint -q "${TEST_DIR}/dev" && umount -l "${TEST_DIR}/dev"
+    umount -l "${TEST_DIR}/dev"
     cleanup_lxds "$TEST_DIR"
-
-    mountpoint -q "${TEST_DIR}" && umount -l "${TEST_DIR}"
-    rm -rf "${TEST_DIR}"
   fi
 
   echo ""
   echo ""
   if [ "${TEST_RESULT}" != "success" ]; then
-    echo "==> FAILED TEST: ${TEST_CURRENT#test_} (${TEST_CURRENT_DESCRIPTION})"
+    echo "==> TEST DONE: ${TEST_CURRENT_DESCRIPTION}"
   fi
   echo "==> Test result: ${TEST_RESULT}"
 }
 
-if [ -n "${SHELL_TRACING:-}" ]; then
-  set -x
-fi
-
 # Must be set before cleanup()
 TEST_CURRENT=setup
 TEST_CURRENT_DESCRIPTION=setup
@@ -221,25 +144,12 @@ trap cleanup EXIT HUP INT TERM
 # Import all the testsuites
 import_subdir_files suites
 
-if [ -n "${SHELL_TRACING:-}" ]; then
-  set -x
-fi
-
 # Setup test directory
-TEST_DIR="$(mktemp -d -t lxd-test.tmp.XXXX)"
+TEST_DIR=$(mktemp -d -p "$(pwd)" tmp.XXX)
 chmod +x "${TEST_DIR}"
 
-# Verify the dir chain is accessible for other users (other's execute bit has to be `x` or `t` (sticky))
-# This is to catch if `sudo chmod +x ~` was not run and the TEST_DIR is under `~`
-INACCESSIBLE_DIRS="$(namei -m "${TEST_DIR}" | awk '/^ d/ {print $1}' | grep -v '[xt]$' || true)"
-if [ -n "${INACCESSIBLE_DIRS:-}" ]; then
-    echo "Some directories are not accessible by other users" >&2
-    namei -m "${TEST_DIR}"
-    exit 1
-fi
-
-if [ "${LXD_TMPFS:-0}" = "1" ]; then
-  mount -t tmpfs tmpfs "${TEST_DIR}" -o mode=0751 -o size=7G
+if [ -n "${LXD_TMPFS:-}" ]; then
+  mount -t tmpfs tmpfs "${TEST_DIR}" -o mode=0751 -o size=6G
 fi
 
 mkdir -p "${TEST_DIR}/dev"
@@ -253,21 +163,24 @@ LXD_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
 export LXD_DIR
 chmod +x "${LXD_DIR}"
 spawn_lxd "${LXD_DIR}" true
-LXD_ADDR=$(< "${LXD_DIR}/lxd.addr")
+LXD_ADDR=$(cat "${LXD_DIR}/lxd.addr")
 export LXD_ADDR
 
-export LXD_SKIP_TESTS="${LXD_SKIP_TESTS:-}"
+LXD_SKIP_TESTS="${LXD_SKIP_TESTS:-}"
+export LXD_SKIP_TESTS
 
-export LXD_REQUIRED_TESTS="${LXD_REQUIRED_TESTS:-}"
+LXD_REQUIRED_TESTS="${LXD_REQUIRED_TESTS:-}"
+export LXD_REQUIRED_TESTS
 
 # This must be enough to accomodate the busybox testimage
-export SMALL_ROOT_DISK="${SMALL_ROOT_DISK:-"root,size=32MiB"}"
+SMALL_ROOT_DISK="${SMALL_ROOT_DISK:-"root,size=32MiB"}"
+export SMALL_ROOT_DISK
 
 run_test() {
   TEST_CURRENT=${1}
-  TEST_CURRENT_DESCRIPTION=${2:-${1#test_}}
+  TEST_CURRENT_DESCRIPTION=${2:-${1}}
   TEST_UNMET_REQUIREMENT=""
-  cwd="${PWD}"
+  cwd="$(pwd)"
 
   echo "==> TEST BEGIN: ${TEST_CURRENT_DESCRIPTION}"
   START_TIME=$(date +%s)
@@ -310,8 +223,7 @@ run_test() {
   DURATION=$((END_TIME-START_TIME))
   cd "${cwd}"
 
-  # output duration in blue
-  echo -e "==> TEST DONE: ${TEST_CURRENT_DESCRIPTION} (\033[0;34m${DURATION}s\033[0m)"
+  echo "==> TEST DONE: ${TEST_CURRENT_DESCRIPTION} (${DURATION}s)"
 
   if [ -n "${GITHUB_ACTIONS:-}" ]; then
       # strip the "test_" prefix to save the shorten test name along with its duration
@@ -326,43 +238,13 @@ if [ -n "${GITHUB_ACTIONS:-}" ]; then
 fi
 
 # allow for running a specific set of tests
-if [ "$#" -gt 0 ] && [ "$1" != "all" ] && [ "$1" != "cluster" ] && [ "$1" != "standalone" ] && [ "$1" != "test-shell" ]; then
+if [ "$#" -gt 0 ] && [ "$1" != "all" ] && [ "$1" != "cluster" ] && [ "$1" != "standalone" ]; then
   run_test "test_${1}"
   # shellcheck disable=SC2034
   TEST_RESULT=success
   exit
 fi
 
-# Spawn an interactive test shell when invoked as `./main.sh test-shell`.
-# This is useful for quick interactions with LXD and its test suite.
-if [ "${1:-"all"}" = "test-shell" ]; then
-  # yellow
-  export PS1="\[\033[0;33mLXD-TEST\033[0m ${PS1:-\u@\h:\w\$ }\]"
-
-  # The `cleanup` handler must run when exiting a `test-shell` session but if the
-  # last command returned non-0 (like `false`), we don't want to output the debug
-  # information accompanying normal failures.
-  #
-  # If a test script runs into an error, the `cleanup` handler will already have
-  # reported the relevant debug info so there is no need to repeat it when exiting
-  # the `test-shell` environment.
-  #
-  # To do so, swallow any error code returned from the interactive \`test-shell\`.
-  bash --rcfile test-shell.bashrc || true
-
-  # shellcheck disable=SC2034
-  TEST_RESULT=success
-  exit
-else
-  # Since we are executing more than one test, cache the busybox testimage for reuse
-  deps/import-busybox --save-image
-
-  # Avoid `.tar.xz` extension that may conflict with some tests
-  mv busybox.tar.xz busybox.tar.xz.cache
-  export LXD_TEST_IMAGE="busybox.tar.xz.cache"
-  echo "==> Saving testimage for reuse (${LXD_TEST_IMAGE})"
-fi
-
 if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_check_deps "checking dependencies"
     run_test test_database_restore "database restore"
@@ -370,18 +252,15 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_sql "lxd sql"
     run_test test_tls_restrictions "TLS restrictions"
     run_test test_tls_version "TLS version"
-    run_test test_completions "CLI completions"
     run_test test_oidc "OpenID Connect"
     run_test test_authorization "Authorization"
     run_test test_certificate_edit "Certificate edit"
     run_test test_basic_usage "basic usage"
-    run_test test_basic_version "basic version"
     run_test test_server_info "server info"
     run_test test_remote_url "remote url handling"
     run_test test_remote_url_with_token "remote token handling"
     run_test test_remote_admin "remote administration"
     run_test test_remote_usage "remote usage"
-    run_test test_vm_empty "Empty VM"
 fi
 
 if [ "${1:-"all"}" != "standalone" ]; then
@@ -403,7 +282,6 @@ if [ "${1:-"all"}" != "standalone" ]; then
     run_test test_clustering_address "clustering address"
     run_test test_clustering_image_replication "clustering image replication"
     run_test test_clustering_dns "clustering DNS"
-    run_test test_clustering_fan "clustering FAN"
     run_test test_clustering_recover "clustering recovery"
     run_test test_clustering_handover "clustering handover"
     run_test test_clustering_rebalance "clustering rebalance"
@@ -418,12 +296,10 @@ if [ "${1:-"all"}" != "standalone" ]; then
     run_test test_clustering_autotarget "clustering autotarget member"
     run_test test_clustering_upgrade "clustering upgrade"
     run_test test_clustering_upgrade_large "clustering upgrade_large"
-    run_test test_clustering_downgrade "clustering downgrade"
     run_test test_clustering_groups "clustering groups"
     run_test test_clustering_events "clustering events"
     run_test test_clustering_uuid "clustering uuid"
     run_test test_clustering_trust_add "clustering trust add"
-    run_test test_clustering_waitready "clustering waitready"
 fi
 
 if [ "${1:-"all"}" != "cluster" ]; then
@@ -476,12 +352,11 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_image_import_existing_alias "import existing image from alias"
     run_test test_image_refresh "image refresh"
     run_test test_image_acl "image acl"
-    run_test test_images_public "public images"
     run_test test_cloud_init "cloud-init"
     run_test test_exec "exec"
     run_test test_exec_exit_code "exec exit code"
-    run_test test_lxd_benchmark_basic "lxd-benchmark basic init/start/stop/delete"
-    run_test test_shutdown "lxd shutdown sequence"
+    run_test test_concurrent_exec "concurrent exec"
+    run_test test_concurrent "concurrent startup"
     run_test test_snapshots "container snapshots"
     run_test test_snap_restore "snapshot restores"
     run_test test_snap_expiry "snapshot expiry"
@@ -496,7 +371,6 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_container_snapshot_config "container snapshot configuration"
     run_test test_server_config "server configuration"
     run_test test_filemanip "file manipulations"
-    run_test test_filemanip_req_content_type "request content-type header verification during file push"
     run_test test_network "network management"
     run_test test_network_acl "network ACL management"
     run_test test_network_forward "network address forwards"
@@ -508,6 +382,7 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_devlxd "/dev/lxd"
     run_test test_fuidshift "fuidshift"
     run_test test_migration "migration"
+    run_test test_lxc_to_lxd "LXC to LXD"
     run_test test_fdleak "fd leak"
     run_test test_storage "storage"
     run_test test_storage_volume_snapshots "storage volume snapshots"
@@ -539,15 +414,12 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_backup_rename "backup rename"
     run_test test_backup_volume_export "backup volume export"
     run_test test_backup_export_import_instance_only "backup export and import instance only"
-    run_test test_backup_metadata "backup metadata checks for containers and custom storage volumes"
     run_test test_backup_volume_rename_delete "backup volume rename and delete"
     run_test test_backup_instance_uuid "backup instance and check instance UUIDs"
     run_test test_backup_volume_expiry "backup volume expiry"
     run_test test_backup_export_import_recover "backup export, import, and recovery"
     run_test test_container_local_cross_pool_handling "container local cross pool handling"
     run_test test_incremental_copy "incremental container copy"
-    run_test test_concurrent_exec "concurrent exec"
-    run_test test_concurrent "concurrent startup"
     run_test test_profiles_project_default "profiles in default project"
     run_test test_profiles_project_images_profiles "profiles in project with images and profiles enabled"
     run_test test_profiles_project_images "profiles in project with images enabled and profiles disabled"
@@ -558,7 +430,6 @@ if [ "${1:-"all"}" != "cluster" ]; then
     run_test test_storage_volume_recover "Recover storage volumes"
     run_test test_syslog_socket "Syslog socket"
     run_test test_lxd_user "lxd user"
-    run_test test_waitready "waitready"
 fi
 
 # shellcheck disable=SC2034
