@@ -1,4 +1,5 @@
-#!/bin/sh -eu
+#!/bin/bash
+set -eu
 #
 # Performance tests runner
 #
@@ -12,7 +13,6 @@ LXD_NETNS=""
 
 import_subdir_files() {
     test  "$1"
-    # shellcheck disable=SC2039,3043
     local file
     for file in "$1"/*.sh; do
         # shellcheck disable=SC1090
@@ -27,7 +27,6 @@ log_message() {
 }
 
 run_benchmark() {
-    # shellcheck disable=SC2039,3043
     local label description
     label="$1"
     description="$2"
@@ -54,7 +53,7 @@ trap cleanup EXIT HUP INT TERM
 TEST_DIR=$(mktemp -d -p "$(pwd)" tmp.XXX)
 
 if [ -n "${LXD_TMPFS:-}" ]; then
-  mount -t tmpfs tmpfs "${TEST_DIR}" -o mode=0751
+  mount -t tmpfs tmpfs "${TEST_DIR}" -o mode=0751 -o size=6G
 fi
 
 LXD_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
