@@ -1,5 +1,6 @@
 test_storage_volume_recover() {
   LXD_IMPORT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
+  chmod +x "${LXD_IMPORT_DIR}"
   spawn_lxd "${LXD_IMPORT_DIR}" true
 
   poolName=$(lxc profile device get default root pool)
@@ -49,13 +50,13 @@ yes
 EOF
 
   # Ensure custom storage volume has been recovered.
-  lxc storage volume show "${poolName}" vol1 | grep -xF 'content_type: block'
-  lxc storage volume show "${poolName}" vol2 | grep -xF 'content_type: iso'
+  lxc storage volume show "${poolName}" vol1 | grep -q 'content_type: block'
+  lxc storage volume show "${poolName}" vol2 | grep -q 'content_type: iso'
 
   if [ "$poolDriver" = "zfs" ]; then
     # Ensure custom storage volumes have been recovered.
-    lxc storage volume show "${poolName}" vol3 | grep -xF 'content_type: filesystem'
-    lxc storage volume show "${poolName}" vol4 | grep -xF 'content_type: filesystem'
+    lxc storage volume show "${poolName}" vol3 | grep -q 'content_type: filesystem'
+    lxc storage volume show "${poolName}" vol4 | grep -q 'content_type: filesystem'
 
     # Cleanup
     lxc storage volume delete "${poolName}" vol3
@@ -71,6 +72,7 @@ EOF
 
 test_container_recover() {
   LXD_IMPORT_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
+  chmod +x "${LXD_IMPORT_DIR}"
   spawn_lxd "${LXD_IMPORT_DIR}" true
 
   (
@@ -248,9 +250,6 @@ ceph.osd.pool_name=$(lxc storage get "${poolName}" ceph.osd.pool_name)
 ceph.user.name=$(lxc storage get "${poolName}" ceph.user.name)
 "
       ;;
-      *)
-        # nothing extra config needed
-      ;;
     esac
 
     lxd sql global "PRAGMA foreign_keys=ON; DELETE FROM instances WHERE name='c1'"
@@ -629,12 +628,12 @@ _backup_export_with_project() {
   rm -rf "${LXD_DIR}/optimized" "${LXD_DIR}/non-optimized"
 
   # Check if hyphens cause issues when creating backups
-  lxc init --empty c1-foo -d "${SMALL_ROOT_DISK}"
+  lxc launch testimage c1-foo -d "${SMALL_ROOT_DISK}"
   lxc snapshot c1-foo
 
   lxc export c1-foo "${LXD_DIR}/c1-foo.tar.gz"
 
-  lxc delete c1-foo
+  lxc delete --force c1-foo
 
   # Cleanup exported tarballs
   rm -f "${LXD_DIR}"/c*.tar.gz
@@ -650,16 +649,14 @@ test_backup_rename() {
   ensure_import_testimage
   ensure_has_localhost_remote "${LXD_ADDR}"
 
-  OUTPUT="$(! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 || false)"
-  if ! echo "${OUTPUT}" | grep -F "Error: Instance backup not found" ; then
+  if ! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 | grep -q "Error: Instance backup not found" ; then
     echo "invalid rename response for missing container"
     false
   fi
 
   lxc init --empty c1 -d "${SMALL_ROOT_DISK}"
 
-  OUTPUT="$(! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 || false)"
-  if ! echo "${OUTPUT}" | grep -F "Error: Instance backup not found" ; then
+  if ! lxc query -X POST /1.0/containers/c1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 | grep -q "Error: Instance backup not found" ; then
     echo "invalid rename response for missing backup"
     false
   fi
@@ -739,7 +736,6 @@ _backup_volume_export_with_project() {
 
   # Create file on the custom volume.
   echo foo | lxc file push - c1/mnt/test
-  LXC_LOCAL='' lxc_remote exec c1 -- sync /mnt/test
 
   # Snapshot the custom volume.
   lxc storage volume set "${custom_vol_pool}" testvol user.foo=test-snap0
@@ -747,7 +743,6 @@ _backup_volume_export_with_project() {
 
   # Change the content (the snapshot will contain the old value).
   echo bar | lxc file push - c1/mnt/test
-  LXC_LOCAL='' lxc_remote exec c1 -- sync /mnt/test
 
   lxc storage volume set "${custom_vol_pool}" testvol user.foo=test-snap1
   lxc storage volume snapshot "${custom_vol_pool}" testvol test-snap1
@@ -780,10 +775,10 @@ _backup_volume_export_with_project() {
   ls -l "${LXD_DIR}/non-optimized/backup/"
   [ -f "${LXD_DIR}/non-optimized/backup/index.yaml" ]
   [ -d "${LXD_DIR}/non-optimized/backup/volume" ]
-  [ "$(< "${LXD_DIR}/non-optimized/backup/volume/test")" = "bar" ]
+  [ "$(cat "${LXD_DIR}/non-optimized/backup/volume/test")" = "bar" ]
   [ ! -d "${LXD_DIR}/non-optimized/backup/volume-snapshots" ]
 
-  ! grep -F -- '- test-snap0' "${LXD_DIR}/non-optimized/backup/index.yaml" || false
+  ! grep -q -- '- test-snap0' "${LXD_DIR}/non-optimized/backup/index.yaml" || false
 
   rm -rf "${LXD_DIR}/non-optimized/"*
   rm "${LXD_DIR}/testvol.tar.gz"
@@ -815,11 +810,11 @@ _backup_volume_export_with_project() {
   ls -l "${LXD_DIR}/non-optimized/backup/"
   [ -f "${LXD_DIR}/non-optimized/backup/index.yaml" ]
   [ -d "${LXD_DIR}/non-optimized/backup/volume" ]
-  [ "$(< "${LXD_DIR}/non-optimized/backup/volume/test")" = "bar" ]
+  [ "$(cat "${LXD_DIR}/non-optimized/backup/volume/test")" = "bar" ]
   [ -d "${LXD_DIR}/non-optimized/backup/volume-snapshots/test-snap0" ]
-  [  "$(< "${LXD_DIR}/non-optimized/backup/volume-snapshots/test-snap0/test")" = "foo" ]
+  [  "$(cat "${LXD_DIR}/non-optimized/backup/volume-snapshots/test-snap0/test")" = "foo" ]
 
-  grep -F -- '- test-snap0' "${LXD_DIR}/non-optimized/backup/index.yaml"
+  grep -q -- '- test-snap0' "${LXD_DIR}/non-optimized/backup/index.yaml"
 
   rm -rf "${LXD_DIR}/non-optimized/"*
 
@@ -918,8 +913,7 @@ test_backup_volume_rename_delete() {
   # Create test volume.
   lxc storage volume create "${pool}" vol1 size=32MiB
 
-  OUTPUT="$(! lxc query -X POST /1.0/storage-pools/"${pool}"/volumes/custom/vol1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 || false)"
-  if ! echo "${OUTPUT}" | grep -F "Error: Storage volume backup not found" ; then
+  if ! lxc query -X POST /1.0/storage-pools/"${pool}"/volumes/custom/vol1/backups/backupmissing -d '{\"name\": \"backupnewname\"}' --wait 2>&1 | grep -q "Error: Storage volume backup not found" ; then
     echo "invalid rename response for missing storage volume"
     false
   fi
@@ -975,13 +969,13 @@ test_backup_instance_uuid() {
   ensure_has_localhost_remote "${LXD_ADDR}"
 
   echo "==> Checking instance UUIDs during backup operation"
-  lxc init --empty c1 -d "${SMALL_ROOT_DISK}"
+  lxc launch testimage c1 -d "${SMALL_ROOT_DISK}"
   initialUUID=$(lxc config get c1 volatile.uuid)
   initialGenerationID=$(lxc config get c1 volatile.uuid.generation)
 
   # export and import should preserve the UUID and generation UUID
   lxc export c1 "${LXD_DIR}/c1.tar.gz"
-  lxc delete c1
+  lxc delete -f c1
   lxc import "${LXD_DIR}/c1.tar.gz"
 
   newUUID=$(lxc config get c1 volatile.uuid)
@@ -992,7 +986,7 @@ test_backup_instance_uuid() {
     false
   fi
 
-  lxc delete c1
+  lxc delete -f c1
 
   # Cleanup exported tarballs
   rm -f "${LXD_DIR}"/c*.tar.gz
@@ -1090,84 +1084,3 @@ test_backup_export_import_instance_only() {
   rm "${LXD_DIR}/c1.tar.gz"
   lxc delete -f c1
 }
-
-test_backup_metadata() {
-  ensure_import_testimage
-  ensure_has_localhost_remote "${LXD_ADDR}"
-
-  # Fetch the least and most recent supported backup metadata version from the range.
-  lowest_version=$(lxc query /1.0 | jq -r .environment.backup_metadata_version_range[0])
-  highest_version=$(lxc query /1.0 | jq -r .environment.backup_metadata_version_range[1])
-
-  [ "$lowest_version" = "1" ]
-  [ "$highest_version" = "2" ]
-
-  tmpDir=$(mktemp -d -p "${TEST_DIR}" metadata-XXX)
-
-  # Create an instance with one snapshot.
-  lxc init --empty c1 -d "${SMALL_ROOT_DISK}"
-  lxc snapshot c1
-
-  # Export the instance without setting an export version.
-  # The server should implicitly pick its latest supported version.
-  lxc export c1 "${tmpDir}/c1.tar.gz"
-  tar -xzf "${tmpDir}/c1.tar.gz" -C "${tmpDir}" --occurrence=1 backup/index.yaml
-
-  cat "${tmpDir}/backup/index.yaml"
-  [ "$(yq '.snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
-  [ "$(yq .config.version < "${tmpDir}/backup/index.yaml")" = "null" ]
-  [ "$(yq '.config.volume_snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
-
-  rm -rf "${tmpDir}/backup" "${tmpDir}/c1.tar.gz"
-
-  # Export the instance using the specified lowest export version.
-  # The server should used the provided version instead of its default.
-  lxc export c1 "${tmpDir}/c1.tar.gz" --export-version "${lowest_version}"
-  tar -xzf "${tmpDir}/c1.tar.gz" -C "${tmpDir}" --occurrence=1 backup/index.yaml
-
-  cat "${tmpDir}/backup/index.yaml"
-  [ "$(yq .config.version < "${tmpDir}/backup/index.yaml")" = "null" ]
-  [ "$(yq .config.container < "${tmpDir}/backup/index.yaml")" != "null" ]
-  [ "$(yq .config.pool < "${tmpDir}/backup/index.yaml")" != "null" ]
-  [ "$(yq .config.volume < "${tmpDir}/backup/index.yaml")" != "null" ]
-  [ "$(yq '.config.snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
-  [ "$(yq '.config.volume_snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
-
-  rm -rf "${tmpDir}/backup" "${tmpDir}/c1.tar.gz"
-  lxc delete -f c1
-
-  # Create a custom storage volume with one snapshot.
-  poolName=$(lxc profile device get default root pool)
-  lxc storage volume create "${poolName}" vol1 size=32MiB
-  lxc storage volume snapshot "${poolName}" vol1
-
-  # Export the custom storage volume without setting an export version.
-  # The server should implicitly pick its latest supported version.
-  lxc storage volume export "${poolName}" vol1 "${tmpDir}/vol1.tar.gz"
-  tar -xzf "${tmpDir}/vol1.tar.gz" -C "${tmpDir}" --occurrence=1 backup/index.yaml
-
-  cat "${tmpDir}/backup/index.yaml"
-  [ "$(yq '.snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
-  [ "$(yq .config.version < "${tmpDir}/backup/index.yaml")" = "null" ]
-  [ "$(yq .config.instance < "${tmpDir}/backup/index.yaml")" = "null" ]
-  [ "$(yq '.config.volume_snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
-
-  rm -rf "${tmpDir}/backup" "${tmpDir}/vol1.tar.gz"
-
-  # Export the custom storage volume using the specified lowest export version.
-  # The server should used the provided version instead of its default.
-  lxc storage volume export "${poolName}" vol1 "${tmpDir}/vol1.tar.gz" --export-version "${lowest_version}"
-  tar -xzf "${tmpDir}/vol1.tar.gz" -C "${tmpDir}" --occurrence=1 backup/index.yaml
-
-  cat "${tmpDir}/backup/index.yaml"
-  [ "$(yq '.snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
-  [ "$(yq .config.version < "${tmpDir}/backup/index.yaml")" = "null" ]
-  [ "$(yq .config.container < "${tmpDir}/backup/index.yaml")" = "null" ]
-  [ "$(yq .config.volume < "${tmpDir}/backup/index.yaml")" != "null" ]
-  [ "$(yq '.config.volume_snapshots | length' < "${tmpDir}/backup/index.yaml")" = "1" ]
-
-  rm -rf "${tmpDir}/backup" "${tmpDir}/vol1.tar.gz"
-  lxc storage volume delete "${poolName}" vol1
-
-  rm -rf "${tmpDir}"
-}
