@@ -4,13 +4,18 @@ test_container_devices_tpm() {
     return
   fi
 
+  if ! modprobe tpm_vtpm_proxy; then
+    echo "==> SKIP: Required tpm_vtpm_proxy.ko is missing"
+    return
+  fi
+
   ensure_import_testimage
   ensure_has_localhost_remote "${LXD_ADDR}"
   ctName="ct$$"
   lxc launch testimage "${ctName}"
 
   # Check adding a device with no path
-  ! lxc config device add "${ctName}" test-dev-invalid
+  ! lxc config device add "${ctName}" test-dev-invalid tpm
 
   # Add device
   lxc config device add "${ctName}" test-dev1 tpm path=/dev/tpm0 pathrm=/dev/tpmrm0
@@ -23,5 +28,5 @@ test_container_devices_tpm() {
   ! lxc exec "${ctName}" -- stat /dev/tpmrm0
 
   # Clean up
-  lxc rm -f "${ctName}"
+  lxc delete -f "${ctName}"
 }
