@@ -21,14 +21,14 @@ INSERT INTO broken(n) VALUES(1);
 EOF
 
   # Starting LXD fails.
-  ! LXD_DIR="${LXD_RESTORE_DIR}" lxd --logfile "${LXD_RESTORE_DIR}/lxd.log" "${SERVER_DEBUG-}" 2>&1 || false
+  ! LXD_DIR="${LXD_RESTORE_DIR}" lxd --logfile "${LXD_RESTORE_DIR}/lxd.log" "${DEBUG-}" 2>&1 || false
 
   # Remove the broken patch
   rm -f "${LXD_RESTORE_DIR}/database/patch.global.sql"
 
   # Restore the backup
-  mv "${LXD_RESTORE_DIR}/database/global" "${LXD_RESTORE_DIR}/database/global.old"
-  mv "${LXD_RESTORE_DIR}/database/global.bak" "${LXD_RESTORE_DIR}/database/global"
+  rm -rf "${LXD_RESTORE_DIR}/database/global"
+  cp -a "${LXD_RESTORE_DIR}/database/global.bak" "${LXD_RESTORE_DIR}/database/global"
 
   # Restart the daemon and check that our previous settings are still there
   respawn_lxd "${LXD_RESTORE_DIR}" true
@@ -36,7 +36,7 @@ EOF
     set -e
     # shellcheck disable=SC2034
     LXD_DIR=${LXD_RESTORE_DIR}
-    [ "$(lxc config get "core.https_allowed_credentials")" = "true" ]
+    lxc config get "core.https_allowed_credentials" | grep -q "true"
   )
 
   kill_lxd "${LXD_RESTORE_DIR}"
@@ -53,8 +53,8 @@ test_database_no_disk_space() {
   GLOBAL_DB_DIR="${LXD_NOSPACE_DIR}/database/global"
   BIG_FILE="${GLOBAL_DB_DIR}/bigfile"
   mkdir -p "${GLOBAL_DB_DIR}"
-  mount -t tmpfs -o size=16M tmpfs "${GLOBAL_DB_DIR}"
-  fallocate -l 2M "${BIG_FILE}"
+  mount -t tmpfs -o size=67108864 tmpfs "${GLOBAL_DB_DIR}"
+  dd bs=1024 count=51200 if=/dev/zero of="${BIG_FILE}"
 
   spawn_lxd "${LXD_NOSPACE_DIR}" true
 
@@ -63,7 +63,8 @@ test_database_no_disk_space() {
     # shellcheck disable=SC2034,SC2030
     LXD_DIR="${LXD_NOSPACE_DIR}"
 
-    lxc init --empty c
+    ensure_import_testimage
+    lxc init testimage c
 
     # Set a custom user property with a big value, so we eventually eat up all
     # available disk space in the database directory.
