@@ -1,19 +1,22 @@
 # In case a cached image matching the desired alias is present, that
 # one is preferred, even if the its remote has a more recent one.
 test_image_prefer_cached() {
-  if lxc image alias list testimage | grep -wF "testimage"; then
+
+  if lxc image alias list | grep -q "^| testimage\\s*|.*$"; then
       lxc image delete testimage
   fi
 
   local LXD2_DIR LXD2_ADDR
   LXD2_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
+  chmod +x "${LXD2_DIR}"
   spawn_lxd "${LXD2_DIR}" true
   LXD2_ADDR=$(cat "${LXD2_DIR}/lxd.addr")
 
   (LXD_DIR=${LXD2_DIR} deps/import-busybox --alias testimage --public)
   fp1="$(LXD_DIR=${LXD2_DIR} lxc image info testimage | awk '/^Fingerprint/ {print $2}')"
 
-  lxc remote add l2 "${LXD2_ADDR}" --accept-certificate --password foo
+  token="$(LXD_DIR=${LXD2_DIR} lxc config trust add --name foo -q)"
+  lxc remote add l2 "${LXD2_ADDR}" --token "${token}"
   lxc init l2:testimage c1
 
   # Now the first image image is in the local store, since it was
