@@ -10,31 +10,39 @@ test_pki() {
   (
     set -e
     cd "${TEST_DIR}/pki"
+    export EASYRSA_KEY_SIZE=4096
 
-    cat << EOF > vars
-set_var EASYRSA_ALGO ec
-set_var EASYRSA_BATCH yes
-EOF
-
-    ./easyrsa init-pki
-    echo "lxd" | ./easyrsa build-ca nopass
-    ./easyrsa gen-crl
-    ./easyrsa build-client-full restricted nopass
-    ./easyrsa build-client-full unrestricted nopass
-    ./easyrsa build-client-full fine-grained nopass
-    ./easyrsa build-client-full ca-trusted nopass
-    ./easyrsa build-client-full prior-revoked nopass
-    mkdir keys
-    cp pki/private/* keys/
-    cp pki/issued/* keys/
-    cp pki/ca.crt keys/
-    echo "yes" | ./easyrsa revoke prior-revoked
-    ./easyrsa gen-crl
-    cp pki/crl.pem keys/
+    # shellcheck disable=SC1091
+    if [ -e pkitool ]; then
+        . ./vars
+        ./clean-all
+        ./pkitool --initca
+        ./pkitool lxd-client
+        ./pkitool lxd-client-revoked
+        # This will revoke the certificate but fail in the end as it tries to then verify the revoked certificate.
+        ./revoke-full lxd-client-revoked || true
+    else
+        ./easyrsa init-pki
+        echo "lxd" | ./easyrsa build-ca nopass
+        ./easyrsa gen-crl
+        ./easyrsa build-client-full restricted nopass
+        ./easyrsa build-client-full unrestricted nopass
+        ./easyrsa build-client-full fine-grained nopass
+        ./easyrsa build-client-full ca-trusted nopass
+        ./easyrsa build-client-full prior-revoked nopass
+        mkdir keys
+        cp pki/private/* keys/
+        cp pki/issued/* keys/
+        cp pki/ca.crt keys/
+        echo "yes" | ./easyrsa revoke prior-revoked
+        ./easyrsa gen-crl
+        cp pki/crl.pem keys/
+    fi
   )
 
   # Setup the daemon.
   LXD5_DIR=$(mktemp -d -p "${TEST_DIR}" XXX)
+  chmod +x "${LXD5_DIR}"
   spawn_lxd "${LXD5_DIR}" true
   LXD5_ADDR=$(cat "${LXD5_DIR}/lxd.addr")
 
@@ -45,9 +53,6 @@ EOF
   cert_common_name="$(openssl x509 -noout -subject -in "${LXD_CONF}/client.crt" -nameopt multiline | awk -F' = ' '/commonName/ {print $2}')"
   LXD_DIR="${LXD5_DIR}" lxc config trust list --format csv | grep -F "client,foo,${cert_common_name},$(cert_fingerprint "${LXD_CONF}/client.crt" | cut -c1-12)"
 
-  cert_common_name="$(openssl x509 -noout -subject -in "${LXD_CONF}/client.crt" -nameopt multiline | awk -F' = ' '/commonName/ {print $2}')"
-  LXD_DIR="${LXD5_DIR}" lxc config trust list --format csv | grep -F "client,foo,${cert_common_name},$(cert_fingerprint "${LXD_CONF}/client.crt" | cut -c1-12)"
-
   # Shutdown LXD. The CA certificate and revokation list must be present at start up to enable PKI.
   shutdown_lxd "${LXD5_DIR}"
   cp "${TEST_DIR}/pki/keys/ca.crt" "${LXD5_DIR}/server.ca"
@@ -73,26 +78,9 @@ EOF
     cat "${LXD_CONF}/client.crt" "${LXD_CONF}/client.key" > "${LXD_CONF}/client.pem"
     fingerprint="$(cert_fingerprint "${LXD_CONF}/client.crt")"
 
-    # Try adding remote using an incorrect token/password. This should fail even though the client certificate
+    # Try adding remote using an incorrect token. This should fail even though the client certificate
     # has been signed by the CA because `core.trust_ca_certificates` is not enabled.
-    ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --accept-certificate --password=bar || false
-    ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --accept-certificate --token=bar || false
-
-    # Add remote using the correct password.
-    # This should work because the client certificate is signed by the CA.
-    lxc_remote remote add pki-lxd "${LXD5_ADDR}" --accept-certificate --password=foo
-
-    # Should have trust store entry because `core.trust_ca_certificates` is disabled.
-    lxc_remote config trust ls pki-lxd: | grep -wF unrestricted
-
-    # Should be able to view server config
-    lxc_remote info pki-lxd: | grep -F 'core.https_address'
-    curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0" | jq -e '.metadata.config."core.https_address"'
-
-    # Remove cert from truststore.
-    fingerprint="$(cert_fingerprint "${LXD_CONF}/client.crt")"
-    LXD_DIR="${LXD5_DIR}" lxc config trust remove "${fingerprint}"
-    lxc_remote remote remove pki-lxd
+    ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --token=bar || false
 
     # Add remote using the correct token.
     # This should work because the client certificate is signed by the CA.
@@ -118,7 +106,7 @@ EOF
     # Revoked certificate no longer has access even though it is in the trust store.
     lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
     ! lxc_remote ls pki-lxd: || false
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
 
     # Remove cert from truststore.
     fingerprint="$(cert_fingerprint "${LXD_CONF}/client.crt")"
@@ -174,7 +162,7 @@ EOF
     # Revoked certificate no longer has access even though it is in the trust store.
     lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
     ! lxc_remote ls pki-lxd: || false
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
 
     # Remove cert from truststore.
     fingerprint="$(cert_fingerprint "${LXD_CONF}/client.crt")"
@@ -235,7 +223,7 @@ EOF
     # Revoked certificate no longer has access even though it is in the trust store.
     lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
     ! lxc_remote ls pki-lxd: || false
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
 
     # Remove cert from truststore.
     lxc auth identity delete "tls/${fingerprint}"
@@ -275,16 +263,6 @@ EOF
     # Client cert should not be present in trust store.
     [ "$(lxc config trust list --format csv | wc -l)" = 1 ]
 
-    # Remove remote
-    lxc_remote remote remove pki-lxd
-
-    # Add the remote again using an incorrect password.
-    # This should succeed as is the same as the test above but with an incorrect password rather than no password.
-    lxc_remote remote add pki-lxd "${LXD5_ADDR}" --accept-certificate --password=bar
-
-    # Client cert should not be present in trust store.
-    [ "$(lxc config trust list --format csv | wc -l)" = 1 ]
-
     # The certificate is trusted as root because `core.trust_ca_certificates` is enabled.
     lxc_remote info pki-lxd: | grep -F 'core.https_address'
     curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0" | jq -e '.metadata.config."core.https_address"'
@@ -295,7 +273,7 @@ EOF
     # Check that we no longer have access.
     lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
     ! lxc_remote ls pki-lxd: || false
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
 
     # Re-enable `core.trust_ca_certificates`.
     lxc config set core.trust_ca_certificates true
@@ -311,7 +289,7 @@ EOF
     # Check that we no longer have access (certificate was previously trusted, but is now revoked).
     lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
     ! lxc_remote ls pki-lxd: || false
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
 
     # Remove remote.
     lxc remote remove pki-lxd
@@ -322,13 +300,6 @@ EOF
     # Replace the client certificate with a revoked certificate in the CRL.
     cp "${TEST_DIR}/pki/keys/prior-revoked.crt" "${LXC5_DIR}/client.crt"
     cp "${TEST_DIR}/pki/keys/prior-revoked.key" "${LXC5_DIR}/client.key"
-    cat "${LXD_CONF}/client.crt" "${LXD_CONF}/client.key" > "${LXD_CONF}/client.pem"
-
-    # Try adding a remote using a revoked client certificate, and the correct password.
-    # This should fail, and the revoked certificate should not be added to the trust store.
-    token="$(LXD_DIR=${LXD5_DIR} lxc config trust add --name foo -q)"
-    ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --accept-certificate --password=foo || false
-    ! lxc config trust ls | grep -wF prior-revoked || false
 
     # Try adding a remote using a revoked client certificate, and the correct token.
     # This should fail, and the revoked certificate should not be added to the trust store.
@@ -336,11 +307,6 @@ EOF
     ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --token "${token}" || false
     [ "$(lxc config trust list --format csv | wc -l)" = 1 ]
 
-    # Try adding a remote using a revoked client certificate, and an incorrect password.
-    # This should fail, as if the certificate is revoked and token is wrong then no access should be allowed.
-    ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --accept-certificate --password=bar || false
-    [ "$(lxc config trust list --format csv | wc -l)" = 1 ]
-
     # Try adding a remote using a revoked client certificate, and an incorrect token.
     # This should fail, as if the certificate is revoked and token is wrong then no access should be allowed.
     ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --token=incorrect || false
@@ -348,23 +314,12 @@ EOF
     # Unset `core.trust_ca_certificates` and re-test, there should be no change in behaviour as the certificate is revoked.
     lxc config unset core.trust_ca_certificates
 
-    # Try adding a remote using a revoked client certificate, and the correct password.
-    # This should fail, and the revoked certificate should not be added to the trust store.
-    token="$(LXD_DIR=${LXD5_DIR} lxc config trust add --name foo -q)"
-    ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --accept-certificate --password=foo || false
-    ! lxc config trust ls | grep -wF prior-revoked || false
-
     # Try adding a remote using a revoked client certificate, and the correct token.
     # This should fail, and the revoked certificate should not be added to the trust store.
     token="$(lxc config trust add --name foo -q)"
     ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --token "${token}" || false
     [ "$(lxc config trust list --format csv | wc -l)" = 1 ]
 
-    # Try adding a remote using a revoked client certificate, and an incorrect password.
-    # This should fail, as if the certificate is revoked and token is wrong then no access should be allowed.
-    ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --accept-certificate --password=bar || false
-    [ "$(lxc config trust list --format csv | wc -l)" = 1 ]
-
     # The revoked certificate is not valid when an identity token is used either.
     token="$(lxc auth identity create tls/bar --quiet)"
     lxc auth identity list --format csv | grep -F 'tls,Client certificate (pending),bar'
@@ -372,12 +327,13 @@ EOF
     lxc auth identity list --format csv | grep -F 'tls,Client certificate (pending),bar'
     lxc auth identity delete tls/bar
 
+
     # Try adding a remote using a revoked client certificate, and an incorrect token.
     # This should fail, as if the certificate is revoked and token is wrong then no access should be allowed.
     ! lxc_remote remote add pki-lxd "${LXD5_ADDR}" --token=incorrect || false
 
     # Check we can't access anything with the revoked certificate.
-    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
+    [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
   )
 
   # Confirm that we cannot add a remote using a certificate that is not signed by the CA.
@@ -386,14 +342,14 @@ EOF
   # in the trust store, but not signed by the CA. So here we are checking that mTLS for a client does not work when CA
   # mode is enabled.
   token="$(LXD_DIR=${LXD5_DIR} lxc config trust add --name foo -q)"
-  ! lxc_remote remote add pki-lxd2 "${LXD5_ADDR}" --accept-certificate --password=foo || false
+  ! lxc_remote remote add pki-lxd2 "${LXD5_ADDR}" --token "${token}" || false
   [ "$(LXD_DIR="${LXD5_DIR}" lxc config trust list --format csv | wc -l)" = 1 ]
 
   # Confirm that the certificate we added earlier cannot authenticate with LXD.
   lxc_remote info pki-lxd: | grep -F 'auth: untrusted'
   ! lxc_remote ls pki-lxd: || false
   cat "${LXD_CONF}/client.crt" "${LXD_CONF}/client.key" > "${LXD_CONF}/client.pem"
-  [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "Forbidden" ]
+  [ "$(curl -s --cert "${LXD_CONF}/client.pem" --cacert "${LXD5_DIR}/server.crt" "https://${LXD5_ADDR}/1.0/instances" | jq -e -r '.error')" = "not authorized" ]
 
 
   ### Show that mTLS still works for server certificates:
