@@ -26,20 +26,6 @@ snapshots() {
 
   lxc init testimage foo
 
-  echo "Verify that / is not permitted in snapshots.pattern"
-  lxc config set foo snapshots.pattern="/"
-  SNAP_ERR="$(! lxc snapshot foo 2>&1)"
-  echo "${SNAP_ERR}" | grep -xF 'Error: Invalid snapshot name: Cannot contain "/"'
-  [ "$(lxc list -f csv -c S foo)" = "0" ]
-
-  echo "Test pongo2 template restrictions"
-  # XXX: using wordcount filter to avoid `\n` or other unexpected char.
-  lxc config set foo snapshots.pattern='{% filter wordcount %}{% include \"/etc/hosts\" %}{% endfilter %}'
-  SNAP_ERR="$(! lxc snapshot foo 2>&1)"
-  echo "${SNAP_ERR}" | grep -F "Usage of tag 'include' is not allowed (sandbox restriction active)"
-  [ "$(lxc list -f csv -c S foo)" = "0" ]
-  lxc config unset foo snapshots.pattern
-
   lxc snapshot foo
   # FIXME: make this backend agnostic
   if [ "$lxd_backend" = "dir" ]; then
@@ -77,25 +63,6 @@ EOF
   # Delete the snapshot
   lxc delete foo/tester_yaml
 
-  # Check if the instance with snapshots and non-default profile can be copied to a new project
-  lxc profile create source-profile
-  lxc profile add foo source-profile
-  lxc project create test-project
-
-  lxc copy foo foo --no-profiles -s "${pool}" --project default --target-project test-project
-  [ "$(lxc list -f csv -c S --project test-project foo)" = "3" ]
-  lxc delete --project=test-project foo
-
-  lxc profile create target-profile --project=test-project
-  lxc copy foo foo --profile=target-profile -s "${pool}" --target-project=test-project
-  [ "$(lxc list -f csv -c S --project test-project foo)" = "3" ]
-  lxc delete --project test-project foo
-
-  lxc profile delete target-profile --project=test-project
-  lxc profile remove foo source-profile
-  lxc profile delete source-profile
-  lxc project delete test-project
-
   lxc copy foo/tester foosnap1
   # FIXME: make this backend agnostic
   if [ "$lxd_backend" != "lvm" ] && [ "${lxd_backend}" != "zfs" ] && [ "$lxd_backend" != "ceph" ]; then
@@ -112,11 +79,11 @@ EOF
   lxc snapshot foo snap2
   lxc snapshot foo snap3
   lxc delete foo/snap2 foo/snap3
-  ! lxc info foo | grep -wF snap2 || false
-  ! lxc info foo | grep -wF snap3 || false
+  ! lxc info foo | grep -q snap2 || false
+  ! lxc info foo | grep -q snap3 || false
 
   # no CLI for this, so we use the API directly (rename a snapshot)
-  wait_for "${LXD_ADDR}" my_curl -X POST --fail-with-body -H 'Content-Type: application/json' "https://${LXD_ADDR}/1.0/containers/foo/snapshots/tester" -d "{\"name\":\"tester2\"}"
+  wait_for "${LXD_ADDR}" my_curl -X POST "https://${LXD_ADDR}/1.0/containers/foo/snapshots/tester" -d "{\"name\":\"tester2\"}"
   # FIXME: make this backend agnostic
   if [ "$lxd_backend" = "dir" ]; then
     [ ! -d "${LXD_DIR}/snapshots/foo/tester" ]
@@ -344,7 +311,7 @@ snap_restore() {
     restore_and_compare_fs snap0
 
     # check container is running after restore
-    lxc list --fast | grep -wF bar | grep -wF RUNNING
+    lxc list | grep bar | grep RUNNING
   fi
 
   lxc stop --force bar
@@ -362,25 +329,12 @@ snap_restore() {
   # Check snapshot creation dates.
   lxc init testimage c1
   lxc snapshot c1
-  ! lxc storage volume show "${pool}" container/c1 | grep '^created_at: 0001-01-01T00:00:00Z' || false
-  ! lxc storage volume show "${pool}" container/c1/snap0 | grep '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc storage volume show "${pool}" container/c1 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc storage volume show "${pool}" container/c1/snap0 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
   lxc copy c1 c2
-  ! lxc storage volume show "${pool}" container/c2 | grep '^created_at: 0001-01-01T00:00:00Z' || false
+  ! lxc storage volume show "${pool}" container/c2 | grep -q '^created_at: 0001-01-01T00:00:00Z' || false
   [ "$(lxc storage volume show "${pool}" container/c1/snap0 | awk /created_at:/)" = "$(lxc storage volume show "${pool}" container/c2/snap0 | awk /created_at:/)" ]
   lxc delete -f c1 c2
-
-  # Check the restore isn't blocked by not anymore existing custom volumes.
-  lxc init testimage c1
-  lxc storage volume create "${pool}" foo
-  lxc storage volume attach "${pool}" foo c1 path=/mnt
-  lxc snapshot c1
-  lxc storage volume detach "${pool}" foo c1
-  lxc storage volume delete "${pool}" foo
-  lxc restore c1 snap0
-  ! lxc start c1 || false # Fails because custom vol foo in "${pool}" doesn't exist anymore.
-  lxc config device remove c1 foo
-  lxc start c1
-  lxc delete -f c1
 }
 
 restore_and_compare_fs() {
@@ -405,29 +359,20 @@ test_snap_expiry() {
 
   lxc launch testimage c1
   lxc snapshot c1
-  lxc config show c1/snap0 | grep -F 'expires_at: 0001-01-01T00:00:00Z'
+  lxc config show c1/snap0 | grep -q 'expires_at: 0001-01-01T00:00:00Z'
   [ "$(lxc config get --property c1/snap0 expires_at)" = "0001-01-01 00:00:00 +0000 UTC" ]
 
-  # Check the API returns the zero time representation when listing all snapshots in recursive mode.
-  [ "$(lxc query "/1.0/instances/c1?recursion=2" | jq -r '.snapshots[] | select(.name == "snap0") | .expires_at')" = "0001-01-01T00:00:00Z" ]
-
   lxc config set c1 snapshots.expiry '1d'
   lxc snapshot c1
-
-  # Get snapshot created_at and expires_at properties.
-  # Remove the " +0000 UTC" from the end of the timestamp so we can add one day using `date`.
-  created_at="$(lxc config get c1/snap1 --property created_at | awk -F' +' '{print $1}')"
-  expires_at="$(lxc config get c1/snap1 --property expires_at | awk -F' +' '{print $1}')"
-
-  # Check if the expires_at propery is exactly 1d ahead.
-  [ "$(date -d "${created_at} today + 1days")" = "$(date -d "${expires_at}")" ]
+  ! lxc config show c1/snap1 | grep -q 'expires_at: 0001-01-01T00:00:00Z' || false
+  [ "$(lxc config get --property c1/snap1 expires_at)" != "0001-01-01 00:00:00 +0000 UTC" ]
 
   lxc copy c1 c2
-  ! lxc config show c2/snap1 | grep -F 'expires_at: 0001-01-01T00:00:00Z' || false
+  ! lxc config show c2/snap1 | grep -q 'expires_at: 0001-01-01T00:00:00Z' || false
   [ "$(lxc config get --property c2/snap1 expires_at)" != "0001-01-01 00:00:00 +0000 UTC" ]
 
   lxc snapshot c1 --no-expiry
-  lxc config show c1/snap2 | grep -F 'expires_at: 0001-01-01T00:00:00Z'
+  lxc config show c1/snap2 | grep -q 'expires_at: 0001-01-01T00:00:00Z'
   [ "$(lxc config get --property c1/snap2 expires_at)" = "0001-01-01 00:00:00 +0000 UTC" ]
 
   lxc rm -f c1
@@ -447,15 +392,15 @@ test_snap_schedule() {
   lxc launch testimage c3 -c snapshots.schedule='@startup, 10 5,6 * * *'
   lxc launch testimage c4 -c snapshots.schedule='@startup, 10 5-8 * * *'
   lxc launch testimage c5 -c snapshots.schedule='@startup, 10 2,5-8/2 * * *'
-  lxc info c1 | grep -wF snap0
-  lxc info c2 | grep -wF snap0
-  lxc info c3 | grep -wF snap0
-  lxc info c4 | grep -wF snap0
-  lxc info c5 | grep -wF snap0
+  lxc info c1 | grep -q snap0
+  lxc info c2 | grep -q snap0
+  lxc info c3 | grep -q snap0
+  lxc info c4 | grep -q snap0
+  lxc info c5 | grep -q snap0
 
   # Check we get a new snapshot on restart
   lxc restart c1 -f
-  lxc info c1 | grep -wF snap1
+  lxc info c1 | grep -q snap1
 
   lxc rm -f c1 c2 c3 c4 c5
 }
@@ -489,24 +434,18 @@ test_snap_fail() {
   local lxd_backend
   lxd_backend=$(storage_backend "$LXD_DIR")
 
-  if [ "${lxd_backend}" != "zfs" ]; then
-    echo "==> SKIP: test_snap_fail only supports 'zfs', not ${lxd_backend}"
-    return
-  fi
-
   ensure_import_testimage
 
-  # Containers should fail to snapshot when root is full (can't write to backup.yaml)
-  lxc launch testimage c1 --device root,size=2MiB
-  if lxc exec c1 -- dd if=/dev/urandom of=/root/big.bin count=100 bs=100K; then
-    echo "Writting more data than the root size should have failed"
-    false
+  if [ "${lxd_backend}" = "zfs" ]; then
+    # Containers should fail to snapshot when root is full (can't write to backup.yaml)
+    lxc launch testimage c1 --device root,size=2MiB
+    lxc exec c1 -- dd if=/dev/urandom of=/root/big.bin count=100 bs=100K || true
+
+    ! lxc snapshot c1 || false
+
+    # Make sure that the snapshot creation failed (c1 has 0 snapshots)
+    [ "$(lxc ls --columns nS --format csv | awk --field-separator , '/c1/{print $2}')" -eq 0 ]
+
+    lxc delete --force c1
   fi
-
-  ! lxc snapshot c1 || false
-
-  # Make sure that the snapshot creation failed (c1 has 0 snapshots)
-  [ "$(lxc list --columns nS --format csv c1)" = "c1,0" ]
-
-  lxc delete --force c1
 }
