@@ -1,7 +1,6 @@
 test_storage() {
   ensure_import_testimage
 
-  # shellcheck disable=2039,3043
   local LXD_STORAGE_DIR lxd_backend
 
   lxd_backend=$(storage_backend "$LXD_DIR")
@@ -10,19 +9,57 @@ test_storage() {
   spawn_lxd "${LXD_STORAGE_DIR}" false
 
   # edit storage and pool description
-  # shellcheck disable=2039,3043
   local storage_pool storage_volume
   storage_pool="lxdtest-$(basename "${LXD_DIR}")-pool"
   storage_volume="${storage_pool}-vol"
   lxc storage create "$storage_pool" "$lxd_backend"
   lxc storage show "$storage_pool" | sed 's/^description:.*/description: foo/' | lxc storage edit "$storage_pool"
-  lxc storage show "$storage_pool" | grep -q 'description: foo'
+  [ "$(lxc storage get "$storage_pool" -p description)" = "foo" ]
+  lxc storage set "$storage_pool" -p description="baz"
+  [ "$(lxc storage get "$storage_pool" -p description)" = "baz" ]
 
   lxc storage volume create "$storage_pool" "$storage_volume"
 
   # Test setting description on a storage volume
   lxc storage volume show "$storage_pool" "$storage_volume" | sed 's/^description:.*/description: bar/' | lxc storage volume edit "$storage_pool" "$storage_volume"
-  lxc storage volume show "$storage_pool" "$storage_volume" | grep -q 'description: bar'
+  [ "$(lxc storage volume get "$storage_pool" "$storage_volume" -p description)" = "bar" ]
+  lxc storage volume set "$storage_pool" "$storage_volume" -p description="baz"
+  [ "$(lxc storage volume get "$storage_pool" "$storage_volume" -p description)" = "baz" ]
+
+  # Test creating a storage pool from yaml
+  storage_pool_yaml="lxdtest-$(basename "${LXD_DIR}")-pool-yaml"
+  if [ "${lxd_backend}" = "btrfs" ] || [ "${lxd_backend}" = "zfs" ] || [ "${lxd_backend}" = "lvm" ]; then
+    lxc storage create "$storage_pool_yaml" "$lxd_backend" <<EOF
+description: foo
+config:
+  size: 2GiB
+EOF
+
+    [ "$(lxc storage get "$storage_pool_yaml" size)" = "2GiB" ]
+    [ "$(lxc storage get "$storage_pool_yaml" -p description)" = "foo" ]
+  elif [ "${lxd_backend}" = "dir" ]; then
+    tempdir=$(mktemp -d)
+    lxc storage create "$storage_pool_yaml" "$lxd_backend" <<EOF
+description: foo
+config:
+  source: ${tempdir}
+EOF
+
+    [ "$(lxc storage get "$storage_pool_yaml" source)" = "${tempdir}" ]
+    [ "$(lxc storage get "$storage_pool_yaml" -p description)" = "foo" ]
+  elif [ "${lxd_backend}" = "ceph" ]; then
+    lxc storage create "$storage_pool_yaml" "$lxd_backend" <<EOF
+description: foo
+config:
+  ceph.cluster_name: ceph
+EOF
+
+    [ "$(lxc storage get "$storage_pool_yaml" ceph.cluster_name)" = "ceph" ]
+    [ "$(lxc storage get "$storage_pool_yaml" -p description)" = "foo" ]
+  fi
+
+  # Delete storage pool
+  lxc storage delete "$storage_pool_yaml"
 
   # Validate get/set
   lxc storage set "$storage_pool" user.abc def
@@ -49,7 +86,6 @@ test_storage() {
 
   # Test btrfs resize
   if [ "$lxd_backend" = "lvm" ] || [ "$lxd_backend" = "ceph" ]; then
-      # shellcheck disable=2039,3043
       local btrfs_storage_pool btrfs_storage_volume
       btrfs_storage_pool="lxdtest-$(basename "${LXD_DIR}")-pool-btrfs"
       btrfs_storage_volume="${storage_pool}-vol"
@@ -126,6 +162,10 @@ test_storage() {
       configure_loop_device loop_file_1 loop_device_1
       # shellcheck disable=SC2154
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool2" zfs source="${loop_device_1}"
+      lxc storage delete "lxdtest-$(basename "${LXD_DIR}")-pool2"
+
+      # Ensure that source.wipe allows the device to be reused
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool2" zfs source="${loop_device_1}" source.wipe=true
 
       # Test that no invalid zfs storage pool configuration keys can be set.
       ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-invalid-zfs-pool-config" zfs lvm.thinpool_name=bla || false
@@ -157,6 +197,10 @@ test_storage() {
       configure_loop_device loop_file_2 loop_device_2
       # shellcheck disable=SC2154
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool4" btrfs source="${loop_device_2}"
+      lxc storage delete "lxdtest-$(basename "${LXD_DIR}")-pool4"
+
+      # Ensure that source.wipe allows the device to be reused
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool4" btrfs source="${loop_device_2}" source.wipe=true
 
       # Check that we cannot create storage pools inside of ${LXD_DIR} other than ${LXD_DIR}/storage-pools/{pool_name}.
       ! lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool5_under_lxd_dir" btrfs source="${LXD_DIR}" || false
@@ -211,6 +255,10 @@ test_storage() {
       configure_loop_device loop_file_3 loop_device_3
       # shellcheck disable=SC2154
       lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool6" lvm source="${loop_device_3}" volume.size=25MiB
+      lxc storage delete "lxdtest-$(basename "${LXD_DIR}")-pool6"
+
+      # Ensure that source.wipe allows the device to be reused
+      lxc storage create "lxdtest-$(basename "${LXD_DIR}")-pool6" lvm source="${loop_device_3}" source.wipe=true volume.size=25MiB
 
       configure_loop_device loop_file_5 loop_device_5
       # shellcheck disable=SC2154
@@ -289,10 +337,12 @@ test_storage() {
       lxc storage volume set "lxdtest-$(basename "${LXD_DIR}")-pool1" c1pool1 zfs.use_refquota true
       lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool1" c1pool1 c1pool1 testDevice /opt
       ! lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool1" c1pool1 c1pool1 testDevice2 /opt || false
+      lxc config show c1pool1 | grep -Pz "  testDevice:\n    path: /opt\n    pool: lxdtest-$(basename "${LXD_DIR}")-pool1\n    source: c1pool1\n    type: disk\n"
       lxc storage volume detach "lxdtest-$(basename "${LXD_DIR}")-pool1" c1pool1 c1pool1
+      ! lxc config show c1pool1 | grep "testDevice" || false
       lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool1" custom/c1pool1 c1pool1 testDevice /opt
       ! lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool1" custom/c1pool1 c1pool1 testDevice2 /opt || false
-      lxc storage volume detach "lxdtest-$(basename "${LXD_DIR}")-pool1" c1pool1 c1pool1
+      lxc storage volume detach "lxdtest-$(basename "${LXD_DIR}")-pool1" custom/c1pool1 c1pool1
 
       lxc storage volume create "lxdtest-$(basename "${LXD_DIR}")-pool1" c2pool2
       lxc storage volume attach "lxdtest-$(basename "${LXD_DIR}")-pool1" c2pool2 c2pool2 testDevice /opt
@@ -879,12 +929,19 @@ test_storage() {
 
     # Launch container.
     lxc launch -s "${pool_name}" testimage c1
+    lxc storage volume create "${pool_name}" fsvol
 
     # Disable quotas. The usage should be 0.
     # shellcheck disable=SC2031
     btrfs quota disable "${LXD_DIR}/storage-pools/${pool_name}"
-    usage=$(lxc query /1.0/instances/c1/state | jq '.disk.root')
-    [ "${usage}" = "null" ]
+    # Usage 0 indicates the driver does not support getting volume usage.
+    [ "$(lxc query /1.0/instances/c1/state | jq '.disk.root.usage')" = "0" ]
+    [ "$(lxc query "/1.0/storage-pools/${pool_name}/volumes/custom/fsvol/state" | jq '.usage.used')" = "0" ]
+    # Total 0 indicates the volume is unbound.
+    [ "$(lxc query /1.0/instances/c1/state | jq '.disk.root.total')" = "0" ]
+    [ "$(lxc query "/1.0/storage-pools/${pool_name}/volumes/custom/fsvol/state" | jq '.usage.total')" = "0" ]
+
+    lxc storage volume delete "${pool_name}" fsvol
 
     # Enable quotas. The usage should then be > 0.
     # shellcheck disable=SC2031
