@@ -1,12 +1,4 @@
 test_storage_driver_zfs() {
-  local lxd_backend
-
-  lxd_backend=$(storage_backend "$LXD_DIR")
-  if [ "$lxd_backend" != "zfs" ]; then
-    echo "==> SKIP: test_storage_driver_zfs only supports 'zfs', not ${lxd_backend}"
-    return
-  fi
-
   do_storage_driver_zfs ext4
   do_storage_driver_zfs xfs
   do_storage_driver_zfs btrfs
@@ -16,7 +8,14 @@ test_storage_driver_zfs() {
 }
 
 do_zfs_delegate() {
-  if ! zfs --help | grep -wF "zone" >/dev/null; then
+  local lxd_backend
+
+  lxd_backend=$(storage_backend "$LXD_DIR")
+  if [ "$lxd_backend" != "zfs" ]; then
+    return
+  fi
+
+  if ! zfs --help | grep -q '^\s\+zone\b'; then
     echo "==> SKIP: Skipping ZFS delegation tests due as installed version doesn't support it"
     return
   fi
@@ -31,24 +30,30 @@ do_zfs_delegate() {
   lxc storage volume set "${storage_pool}" container/c1 zfs.delegate=true
   lxc start c1
 
-  PID="$(lxc list -f csv -c p c1)"
-  nsenter -t "${PID}" -U -- zfs list | grep -wF containers/c1
+  PID=$(lxc info c1 | awk '/^PID:/ {print $2}')
+  nsenter -t "${PID}" -U -- zfs list | grep -q containers/c1
 
   # Confirm that ZFS dataset is empty when off.
   lxc stop -f c1
   lxc storage volume unset "${storage_pool}" container/c1 zfs.delegate
   lxc start c1
 
-  PID="$(lxc list -f csv -c p c1)"
-  ! nsenter -t "${PID}" -U -- zfs list | grep -wF containers/c1 || false
+  PID=$(lxc info c1 | awk '/^PID:/ {print $2}')
+  ! nsenter -t "${PID}" -U -- zfs list | grep -q containers/c1
 
   lxc delete -f c1
 }
 
 do_zfs_cross_pool_copy() {
-  local LXD_STORAGE_DIR
+  local LXD_STORAGE_DIR lxd_backend
+
+  lxd_backend=$(storage_backend "$LXD_DIR")
+  if [ "$lxd_backend" != "zfs" ]; then
+    return
+  fi
 
   LXD_STORAGE_DIR=$(mktemp -d -p "${TEST_DIR}" XXXXXXXXX)
+  chmod +x "${LXD_STORAGE_DIR}"
   spawn_lxd "${LXD_STORAGE_DIR}" false
 
   # Import image into default storage pool.
@@ -106,9 +111,20 @@ do_zfs_cross_pool_copy() {
 do_storage_driver_zfs() {
   filesystem="$1"
 
-  local LXD_STORAGE_DIR
+  if ! command -v "mkfs.${filesystem}" >/dev/null 2>&1; then
+    echo "==> SKIP: Skipping block mode test on ${filesystem} due to missing tools."
+    return
+  fi
+
+  local LXD_STORAGE_DIR lxd_backend
+
+  lxd_backend=$(storage_backend "$LXD_DIR")
+  if [ "$lxd_backend" != "zfs" ]; then
+    return
+  fi
 
   LXD_STORAGE_DIR=$(mktemp -d -p "${TEST_DIR}" XXXXXXXXX)
+  chmod +x "${LXD_STORAGE_DIR}"
   spawn_lxd "${LXD_STORAGE_DIR}" false
 
   # Import image into default storage pool.
