@@ -21,8 +21,8 @@ test_tls_restrictions() {
   lxc_remote project create localhost:blah
 
   # Validate normal view with no restrictions
-  lxc_remote project list localhost: | grep -wF default
-  lxc_remote project list localhost: | grep -wF blah
+  lxc_remote project list localhost: | grep -q default
+  lxc_remote project list localhost: | grep -q blah
 
   # Apply restrictions
   lxc config trust show "${FINGERPRINT}" | sed -e "s/restricted: false/restricted: true/" | lxc config trust edit "${FINGERPRINT}"
@@ -34,7 +34,7 @@ test_tls_restrictions() {
   lxc config unset user.foo
 
   # Confirm no project visible when none listed
-  [ "$(lxc_remote project list localhost: --format csv)" = "" ]
+  [ "$(lxc_remote project list localhost: --format csv | wc -l)" = 0 ]
 
   # Confirm we can still view storage pools
   [ "$(lxc_remote storage list localhost: --format csv | wc -l)" = 1 ]
@@ -47,8 +47,8 @@ test_tls_restrictions() {
   lxc config trust show "${FINGERPRINT}" | sed -e "s/projects: \[\]/projects: ['blah']/" -e "s/restricted: false/restricted: true/" | lxc config trust edit "${FINGERPRINT}"
 
   # Validate restricted view
-  ! lxc_remote project list localhost: | grep -wF default || false
-  lxc_remote project list localhost: | grep -wF blah
+  ! lxc_remote project list localhost: | grep -q default || false
+  lxc_remote project list localhost: | grep -q blah
 
   # Validate that the restricted caller cannot edit or delete the project.
   ! lxc_remote project set localhost:blah user.foo=bar || false
@@ -57,9 +57,6 @@ test_tls_restrictions() {
   # Validate restricted caller cannot create projects.
   ! lxc_remote project create localhost:blah1 || false
 
-  # Validate restricted caller cannot create instances in projects they don't have access to
-  ! lxc_remote init testimage localhost: --project default || false
-
   # Validate restricted caller cannot list resources in projects they do not have access to
   ! lxc_remote list localhost: --project default || false
   ! lxc_remote profile list localhost: --project default || false
@@ -152,9 +149,6 @@ test_tls_restrictions() {
   lxc_remote network show "localhost:${networkName}" --project blah
   lxc_remote network list localhost: --project blah | grep -F "${networkName}"
 
-  # The reported project is blah when listing networks with request project set to blah.
-  [ "$(lxc_remote query -X GET "/1.0/networks?project=blah&recursion=1" | jq -r '.[] | select(.name == "'"${networkName}"'") | .project' | grep -cF "blah")" = "1" ]
-
   # The restricted client can't view it via project default.
   ! lxc_remote network show "localhost:${networkName}" --project default || false
   ! lxc_remote network list localhost: --project default | grep -F "${networkName}" || false
@@ -181,42 +175,6 @@ test_tls_restrictions() {
   # The restricted client can delete the network.
   lxc_remote network delete localhost:blah-network --project blah
 
-  # Create a network ACL in the default project.
-  networkACLName="netacl$$"
-  lxc network acl create "${networkACLName}" --project default
-
-  # The network ACL we created in the default project is visible in project blah.
-  lxc_remote network acl show "localhost:${networkACLName}" --project blah
-  [ "$(lxc_remote network acl list localhost: --project blah | grep -cF "${networkACLName}")" = "1" ]
-
-  # The reported project is blah when listing network ACLs with request project set to blah.
-  [ "$(lxc_remote query -X GET "/1.0/network-acls?project=blah&recursion=1" | jq -r '.[] | select(.name == "'"${networkACLName}"'") | .project' | grep -cF "blah")" = "1" ]
-
-  # The restricted client can't view it via project default.
-  ! lxc_remote network acl show "localhost:${networkACLName}" --project default || false
-  ! lxc_remote network acl list localhost: --project default | grep -F "${networkACLName}" || false
-
-  # The restricted client can edit the network ACL.
-  lxc_remote network acl set "localhost:${networkACLName}" user.foo=bar --project blah
-
-  # The restricted client can delete the network ACL.
-  lxc_remote network acl delete "localhost:${networkACLName}" --project blah
-
-  # Create a network ACL in the blah project.
-  lxc_remote network acl create localhost:blah-network-acl --project blah
-
-  # Network ACL is visible to restricted client in project blah.
-  lxc_remote network acl show localhost:blah-network-acl --project blah
-  [ "$(lxc_remote network acl list localhost: --project blah | grep -cF blah-network-acl)" = "1" ]
-
-  # The network ACL is actually in the default project.
-  lxc network acl show blah-network-acl --project default
-
-  # The restricted client can't view it via the default project.
-  ! lxc_remote network acl show localhost:blah-network-acl --project default || false
-
-  # The restricted client can delete the network ACL.
-  lxc_remote network acl delete localhost:blah-network-acl --project blah
 
   ### NETWORK ZONES (initial value is false in new projects).
 
@@ -254,44 +212,6 @@ test_tls_restrictions() {
   # The restricted client can delete the network zone.
   lxc_remote network zone delete localhost:blah-zone --project blah
 
-  ### Network allocations
-
-  # Create a network in the default project.
-  networkName="net$$"
-  lxc network create "${networkName}" --project default
-
-  # Create instances in the default project and in the blah project that use the network
-  ensure_import_testimage
-  lxc image copy testimage local: --project default --target-project blah
-  lxc init testimage foo --network "${networkName}"
-  lxc_remote init testimage localhost:bar --network "${networkName}" --project blah
-
-  # The restricted client can't view allocations in the default project
-  ! lxc network list-allocations localhost: || false
-  ! lxc network list-allocations localhost: --project default || false
-
-  # The restricted client can't view allocations for all projects
-  ! lxc network list-allocations localhost: --all-projects || false
-
-  # The restricted client can view allocations for the blah project. Since blah doesn't have networks enabled, the client
-  # should see allocations for the default project, but they can't see the foo instance
-  [ "$(lxc network list-allocations localhost: --project blah --format csv | wc -l)" = 3 ]
-  ! lxc network list-allocations localhost: --project blah --format csv | grep 'instances/foo' || false
-
-  # Check restrictions when using blah as current project.
-  lxc project switch localhost:blah
-  [ "$(lxc network list-allocations localhost: --format csv | wc -l)" = 3 ]
-  ! lxc network list-allocations localhost: --format csv | grep 'instances/foo' || false
-
-  # Can't switch back to default while restricted to blah, so we need to modify the config file.
-  ! lxc project switch localhost:default || false
-  sed -i 's/project: blah/project: default/g' "${LXD_CONF}/config.yml"
-
-  # Clean up
-  lxc delete foo
-  lxc delete bar --project blah
-  lxc image delete testimage --project blah
-  lxc network delete "${networkName}"
 
   ### PROFILES (initial value is true for new projects)
 
@@ -430,7 +350,7 @@ test_certificate_edit() {
 
   # Try replacing the old certificate with a new one.
   # This should succeed as the user is listed as an admin.
-  my_curl -X PATCH --fail-with-body -H 'Content-Type: application/json' -d "{\"certificate\":\"$(sed ':a;N;$!ba;s/\n/\\n/g' "${LXD_CONF}/client-new.crt")\"}" "https://${LXD_ADDR}/1.0/certificates/${FINGERPRINT}"
+  my_curl -X PATCH -d "{\"certificate\":\"$(sed ':a;N;$!ba;s/\n/\\n/g' "${LXD_CONF}/client-new.crt")\"}" "https://${LXD_ADDR}/1.0/certificates/${FINGERPRINT}"
 
   # Record new fingerprint
   FINGERPRINT="$(lxc config trust list --format csv | cut -d, -f4)"
@@ -452,7 +372,7 @@ test_certificate_edit() {
 
   # Try replacing the new certificate with the old one.
   # This should succeed as well as the certificate may be changed.
-  my_curl -X PATCH --fail-with-body -H 'Content-Type: application/json' -d "{\"certificate\":\"$(sed ':a;N;$!ba;s/\n/\\n/g' "${LXD_CONF}/client.crt.bak")\"}" "https://${LXD_ADDR}/1.0/certificates/${FINGERPRINT}"
+  my_curl -X PATCH -d "{\"certificate\":\"$(sed ':a;N;$!ba;s/\n/\\n/g' "${LXD_CONF}/client.crt.bak")\"}" "https://${LXD_ADDR}/1.0/certificates/${FINGERPRINT}"
 
   # Move new certificate and key to LXD_CONF.
   mv "${LXD_CONF}/client.crt.bak" "${LXD_CONF}/client.crt"
@@ -464,15 +384,15 @@ test_certificate_edit() {
   # Trying to change other fields should fail as a non-admin.
   ! lxc_remote config trust show "${FINGERPRINT}" | sed -e "s/restricted: true/restricted: false/" | lxc_remote config trust edit localhost:"${FINGERPRINT}" || false
 
-  [ "$(my_curl -X PATCH -H 'Content-Type: application/json' -d "{\"restricted\": false}" "https://${LXD_ADDR}/1.0/certificates/${FINGERPRINT}" | jq -r '.error_code')" -eq 403 ]
+  my_curl -X PATCH -d "{\"restricted\": false}" "https://${LXD_ADDR}/1.0/certificates/${FINGERPRINT}" | grep -F '"error_code":403'
 
   ! lxc_remote config trust show "${FINGERPRINT}" | sed -e "s/name:.*/name: bar/" | lxc_remote config trust edit localhost:"${FINGERPRINT}" || false
 
-  [ "$(my_curl -X PATCH -H 'Content-Type: application/json' -d "{\"name\": \"bar\"}" "https://${LXD_ADDR}/1.0/certificates/${FINGERPRINT}" | jq -r '.error_code')" -eq 403 ]
+  my_curl -X PATCH -d "{\"name\": \"bar\"}" "https://${LXD_ADDR}/1.0/certificates/${FINGERPRINT}" | grep -F '"error_code":403'
 
   ! lxc_remote config trust show "${FINGERPRINT}" | sed -e ':a;N;$!ba;s/projects:\n- blah/projects: \[\]/' | lxc_remote config trust edit localhost:"${FINGERPRINT}" || false
 
-  [ "$(my_curl -X PATCH -H 'Content-Type: application/json' -d "{\"projects\": []}" "https://${LXD_ADDR}/1.0/certificates/${FINGERPRINT}" | jq -r '.error_code')" -eq 403 ]
+  my_curl -X PATCH -d "{\"projects\": []}" "https://${LXD_ADDR}/1.0/certificates/${FINGERPRINT}" | grep -F '"error_code":403'
 
   # Cleanup
   lxc config trust show "${FINGERPRINT}" | sed -e "s/restricted: true/restricted: false/" | lxc config trust edit "${FINGERPRINT}"
@@ -495,62 +415,5 @@ test_tls_version() {
 
   echo "TLS 1.2 is refused with a protocol version error"
   ! my_curl --tls-max 1.2 -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" || false
-  # rc=35: SSL connect error. The SSL handshaking failed.
-  CURL_ERR="$(my_curl --tls-max 1.2 -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" || [ "${?}" = 35 ])"
-  echo "${CURL_ERR}" | grep -F "alert protocol version"
-
-  echo "Enable TLS 1.2 with LXD_INSECURE_TLS=true"
-  shutdown_lxd "${LXD_DIR}"
-  export LXD_INSECURE_TLS=true
-  respawn_lxd "${LXD_DIR}" true
-
-  echo "TLS 1.3 is still working and used by default"
-  my_curl -X GET "https://${LXD_ADDR}"
-  my_curl --tlsv1.3 -X GET "https://${LXD_ADDR}"
-
-  echo "TLS 1.2 is now working"
-  my_curl --tls-max 1.2 -X GET "https://${LXD_ADDR}"
-
-  echo "TLS 1.2 with ciphers known to work"
-  for cipher in ECDHE-ECDSA-AES128-GCM-SHA256 ECDHE-ECDSA-AES256-GCM-SHA384 ECDHE-ECDSA-CHACHA20-POLY1305; do
-    echo "Testing TLS 1.2: ${cipher}"
-    my_curl --tls-max 1.2 --ciphers "${cipher}" -X GET "https://${LXD_ADDR}"
-  done
-
-  echo "TLS 1.2 does not work with RSA auth when the server uses ECDSA cert"
-  for cipher in ECDHE-RSA-AES128-GCM-SHA256 ECDHE-RSA-AES256-GCM-SHA384; do
-    echo "Testing TLS 1.2: ${cipher}"
-    ! my_curl --tls-max 1.2 --ciphers "${cipher}" -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" || false
-    CURL_ERR="$(my_curl --tls-max 1.2 --ciphers "${cipher}" -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" || [ "${?}" = 35 ])"
-    echo "${CURL_ERR}" | grep -F "alert handshake failure"
-  done
-
-  echo "TLS 1.2 with ciphers known to be refused with a handshake failure"
-  for cipher in ECDHE-ECDSA-AES128-SHA256 ECDHE-ECDSA-AES256-SHA384; do
-    echo "Testing TLS 1.2: ${cipher}"
-    ! my_curl --tls-max 1.2 --ciphers "${cipher}" -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" || false
-    CURL_ERR="$(my_curl --tls-max 1.2 --ciphers "${cipher}" -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" || [ "${?}" = 35 ])"
-    echo "${CURL_ERR}" | grep -F "alert handshake failure"
-  done
-
-  echo "TLS 1.2 with ciphers known to be cause broken pipe errors or empty replies or connection resets"
-  for cipher in ECDHE-ECDSA-AES128-SHA ECDHE-ECDSA-AES256-SHA; do
-    echo "Testing TLS 1.2: ${cipher}"
-    ! my_curl --tls-max 1.2 --ciphers "${cipher}" -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" || false
-  done
-
-  echo "TLS 1.1 is not working"
-  ! my_curl --tls-max 1.1 -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" || false
-  CURL_ERR="$(my_curl --tls-max 1.1 -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" || [ "${?}" = 35 ])"
-  echo "${CURL_ERR}" | grep -F "no protocols available"
-
-  echo "Disable TLS 1.2"
-  shutdown_lxd "${LXD_DIR}"
-  unset LXD_INSECURE_TLS
-  respawn_lxd "${LXD_DIR}" true
-
-  echo "TLS 1.2 is refused with a protocol version error"
-  ! my_curl --tls-max 1.2 -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" || false
-  CURL_ERR="$(my_curl --tls-max 1.2 -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" || [ "${?}" = 35 ])"
-  echo "${CURL_ERR}" | grep -F "alert protocol version"
+  my_curl --tls-max 1.2 -X GET "https://${LXD_ADDR}" -w "%{errormsg}\n" | grep -F "alert protocol version"
 }
